/* valaccodememberaccessmodule.vala
 *
 * Copyright (C) 2006-2009  Jürg Billeter
 * Copyright (C) 2006-2008  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))

typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;

#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))

typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;

#define VALA_TYPE_CCODE_STRUCT_MODULE (vala_ccode_struct_module_get_type ())
#define VALA_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModule))
#define VALA_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
#define VALA_IS_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_IS_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_CCODE_STRUCT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))

typedef struct _ValaCCodeStructModule ValaCCodeStructModule;
typedef struct _ValaCCodeStructModuleClass ValaCCodeStructModuleClass;
typedef struct _ValaCCodeStructModulePrivate ValaCCodeStructModulePrivate;

#define VALA_TYPE_CCODE_METHOD_MODULE (vala_ccode_method_module_get_type ())
#define VALA_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModule))
#define VALA_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))
#define VALA_IS_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_IS_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_CCODE_METHOD_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))

typedef struct _ValaCCodeMethodModule ValaCCodeMethodModule;
typedef struct _ValaCCodeMethodModuleClass ValaCCodeMethodModuleClass;
typedef struct _ValaCCodeMethodModulePrivate ValaCCodeMethodModulePrivate;

#define VALA_TYPE_CCODE_CONTROL_FLOW_MODULE (vala_ccode_control_flow_module_get_type ())
#define VALA_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModule))
#define VALA_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_CCODE_CONTROL_FLOW_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))

typedef struct _ValaCCodeControlFlowModule ValaCCodeControlFlowModule;
typedef struct _ValaCCodeControlFlowModuleClass ValaCCodeControlFlowModuleClass;
typedef struct _ValaCCodeControlFlowModulePrivate ValaCCodeControlFlowModulePrivate;

#define VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE (vala_ccode_member_access_module_get_type ())
#define VALA_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModule))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))

typedef struct _ValaCCodeMemberAccessModule ValaCCodeMemberAccessModule;
typedef struct _ValaCCodeMemberAccessModuleClass ValaCCodeMemberAccessModuleClass;
typedef struct _ValaCCodeMemberAccessModulePrivate ValaCCodeMemberAccessModulePrivate;

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

struct _ValaCCodeBaseModule {
	ValaCCodeModule parent_instance;
	ValaCCodeBaseModulePrivate * priv;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaTryStatement* current_try;
	ValaCCodeDeclarationSpace* header_declarations;
	ValaCCodeDeclarationSpace* internal_header_declarations;
	ValaCCodeDeclarationSpace* source_declarations;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* class_init_fragment;
	ValaCCodeFragment* base_init_fragment;
	ValaCCodeFragment* class_finalize_fragment;
	ValaCCodeFragment* base_finalize_fragment;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_finalize_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeFragment* module_init_fragment;
	ValaCCodeStruct* param_spec_struct;
	ValaCCodeStruct* closure_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeFunction* function;
	ValaCCodeFragment* pre_statement_fragment;
	GeeArrayList* temp_vars;
	GeeArrayList* temp_ref_vars;
	GeeSet* user_marshal_set;
	GeeSet* predefined_marshal_set;
	gint next_temp_var_id;
	gboolean in_constructor;
	gboolean in_static_or_class_context;
	gboolean current_method_inner_error;
	gint next_coroutine_state;
	ValaDataType* void_type;
	ValaDataType* bool_type;
	ValaDataType* char_type;
	ValaDataType* uchar_type;
	ValaDataType* unichar_type;
	ValaDataType* short_type;
	ValaDataType* ushort_type;
	ValaDataType* int_type;
	ValaDataType* uint_type;
	ValaDataType* long_type;
	ValaDataType* ulong_type;
	ValaDataType* int8_type;
	ValaDataType* uint8_type;
	ValaDataType* int16_type;
	ValaDataType* uint16_type;
	ValaDataType* int32_type;
	ValaDataType* uint32_type;
	ValaDataType* int64_type;
	ValaDataType* uint64_type;
	ValaDataType* string_type;
	ValaDataType* float_type;
	ValaDataType* double_type;
	ValaTypeSymbol* gtype_type;
	ValaTypeSymbol* gobject_type;
	ValaErrorType* gerror_type;
	ValaClass* glist_type;
	ValaClass* gslist_type;
	ValaClass* gvaluearray_type;
	ValaTypeSymbol* gstringbuilder_type;
	ValaTypeSymbol* garray_type;
	ValaTypeSymbol* gbytearray_type;
	ValaTypeSymbol* gptrarray_type;
	ValaDataType* gquark_type;
	ValaStruct* gvalue_type;
	ValaStruct* mutex_type;
	ValaTypeSymbol* type_module_type;
	ValaTypeSymbol* dbus_object_type;
	gboolean in_plugin;
	char* module_init_param_name;
	gboolean gvaluecollector_h_needed;
	gboolean requires_array_free;
	gboolean requires_array_move;
	gboolean requires_array_length;
	gboolean requires_strcmp0;
	gboolean dbus_glib_h_needed;
	gboolean dbus_glib_h_needed_in_header;
	GeeSet* wrappers;
	GeeMap* variable_name_map;
};

struct _ValaCCodeBaseModuleClass {
	ValaCCodeModuleClass parent_class;
	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
	char* (*append_struct_array_free) (ValaCCodeBaseModule* self, ValaStruct* st);
	ValaCCodeExpression* (*get_unref_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
	void (*append_local_free) (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
	gboolean (*variable_accessible_in_finally) (ValaCCodeBaseModule* self, ValaLocalVariable* local);
	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_ref_cexpression) (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
	gboolean (*is_gobject_property) (ValaCCodeBaseModule* self, ValaProperty* prop);
};

struct _ValaCCodeStructModule {
	ValaCCodeBaseModule parent_instance;
	ValaCCodeStructModulePrivate * priv;
};

struct _ValaCCodeStructModuleClass {
	ValaCCodeBaseModuleClass parent_class;
};

struct _ValaCCodeMethodModule {
	ValaCCodeStructModule parent_instance;
	ValaCCodeMethodModulePrivate * priv;
};

struct _ValaCCodeMethodModuleClass {
	ValaCCodeStructModuleClass parent_class;
	void (*generate_method_result_declaration) (ValaCCodeMethodModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeFunction* cfunc, GeeMap* cparam_map, GeeMap* carg_map);
	void (*generate_parameter) (ValaCCodeMethodModule* self, ValaFormalParameter* param, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, GeeMap* carg_map);
};

struct _ValaCCodeControlFlowModule {
	ValaCCodeMethodModule parent_instance;
	ValaCCodeControlFlowModulePrivate * priv;
};

struct _ValaCCodeControlFlowModuleClass {
	ValaCCodeMethodModuleClass parent_class;
};

struct _ValaCCodeMemberAccessModule {
	ValaCCodeControlFlowModule parent_instance;
	ValaCCodeMemberAccessModulePrivate * priv;
};

struct _ValaCCodeMemberAccessModuleClass {
	ValaCCodeControlFlowModuleClass parent_class;
};


static gpointer vala_ccode_member_access_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_base_module_get_type (void);
gpointer vala_ccode_declaration_space_ref (gpointer instance);
void vala_ccode_declaration_space_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_declaration_space (const GValue* value);
GType vala_ccode_declaration_space_get_type (void);
GType vala_ccode_struct_module_get_type (void);
GType vala_ccode_method_module_get_type (void);
GType vala_ccode_control_flow_module_get_type (void);
GType vala_ccode_member_access_module_get_type (void);
enum  {
	VALA_CCODE_MEMBER_ACCESS_MODULE_DUMMY_PROPERTY
};
GType vala_ccode_generator_get_type (void);
ValaCCodeControlFlowModule* vala_ccode_control_flow_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeControlFlowModule* vala_ccode_control_flow_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeMemberAccessModule* vala_ccode_member_access_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeMemberAccessModule* vala_ccode_member_access_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
void vala_ccode_base_module_generate_method_declaration (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
gboolean vala_ccode_base_module_add_generated_external_symbol (ValaCCodeBaseModule* self, ValaSymbol* external_symbol);
void vala_ccode_module_visit_method (ValaCCodeModule* self, ValaMethod* m);
ValaClass* vala_ccode_base_module_get_current_class (ValaCCodeBaseModule* self);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
gboolean vala_ccode_module_method_has_wrapper (ValaCCodeModule* self, ValaMethod* method);
ValaLocalVariable* vala_ccode_base_module_get_temp_variable (ValaCCodeBaseModule* self, ValaDataType* type, gboolean value_owned, ValaCodeNode* node_reference, gboolean init);
ValaCCodeExpression* vala_ccode_base_module_get_variable_cexpression (ValaCCodeBaseModule* self, const char* name);
ValaCCodeExpression* vala_ccode_module_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
ValaDataType* vala_ccode_base_module_get_data_type_for_symbol (ValaTypeSymbol* sym);
void vala_ccode_base_module_generate_class_struct_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_field_declaration (ValaCCodeBaseModule* self, ValaField* f, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_constant_declaration (ValaCCodeBaseModule* self, ValaConstant* c, ValaCCodeDeclarationSpace* decl_space);
ValaMethod* vala_ccode_base_module_get_current_method (ValaCCodeBaseModule* self);
void vala_ccode_base_module_generate_property_accessor_declaration (ValaCCodeBaseModule* self, ValaPropertyAccessor* acc, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_module_visit_property (ValaCCodeModule* self, ValaProperty* prop);
ValaTypeSymbol* vala_ccode_base_module_get_current_type_symbol (ValaCCodeBaseModule* self);
char* vala_ccode_module_get_dynamic_property_getter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
gboolean vala_ccode_base_module_requires_copy (ValaCCodeBaseModule* self, ValaDataType* type);
void vala_ccode_base_module_generate_enum_declaration (ValaCCodeBaseModule* self, ValaEnum* en, ValaCCodeDeclarationSpace* decl_space);
gint vala_ccode_base_module_get_block_id (ValaCCodeBaseModule* self, ValaBlock* b);
char* vala_ccode_base_module_get_variable_cname (ValaCCodeBaseModule* self, const char* name);
ValaPropertyAccessor* vala_ccode_base_module_get_current_property_accessor (ValaCCodeBaseModule* self);
static void vala_ccode_member_access_module_real_visit_member_access (ValaCCodeModule* base, ValaMemberAccess* expr);
static int _vala_strcmp0 (const char * str1, const char * str2);



ValaCCodeMemberAccessModule* vala_ccode_member_access_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaCCodeMemberAccessModule* self;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaCCodeMemberAccessModule*) vala_ccode_control_flow_module_construct (object_type, codegen, next);
	return self;
}


ValaCCodeMemberAccessModule* vala_ccode_member_access_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_ccode_member_access_module_construct (VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, codegen, next);
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


static void vala_ccode_member_access_module_real_visit_member_access (ValaCCodeModule* base, ValaMemberAccess* expr) {
	ValaCCodeMemberAccessModule * self;
	ValaCCodeExpression* pub_inst;
	ValaDataType* base_type;
	self = (ValaCCodeMemberAccessModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	pub_inst = NULL;
	base_type = NULL;
	if (vala_member_access_get_inner (expr) != NULL) {
		ValaCCodeExpression* _tmp0_;
		pub_inst = (_tmp0_ = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (expr)))), _vala_ccode_node_unref0 (pub_inst), _tmp0_);
		if (vala_expression_get_value_type (vala_member_access_get_inner (expr)) != NULL) {
			ValaDataType* _tmp1_;
			base_type = (_tmp1_ = _vala_code_node_ref0 (vala_expression_get_value_type (vala_member_access_get_inner (expr))), _vala_code_node_unref0 (base_type), _tmp1_);
		}
	}
	if (VALA_IS_METHOD (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
		ValaMethod* m;
		gboolean _tmp2_;
		gboolean _tmp3_;
		m = _vala_code_node_ref0 (VALA_METHOD (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
		_tmp2_ = FALSE;
		_tmp3_ = FALSE;
		if (VALA_IS_DYNAMIC_METHOD (m)) {
			_tmp3_ = TRUE;
		} else {
			_tmp3_ = VALA_IS_ARRAY_MOVE_METHOD (m);
		}
		if (_tmp3_) {
			_tmp2_ = TRUE;
		} else {
			_tmp2_ = VALA_IS_ARRAY_RESIZE_METHOD (m);
		}
		if (!_tmp2_) {
			gboolean _tmp4_;
			vala_ccode_base_module_generate_method_declaration ((ValaCCodeBaseModule*) self, m, ((ValaCCodeBaseModule*) self)->source_declarations);
			_tmp4_ = FALSE;
			if (!vala_symbol_get_external ((ValaSymbol*) m)) {
				_tmp4_ = vala_symbol_get_external_package ((ValaSymbol*) m);
			} else {
				_tmp4_ = FALSE;
			}
			if (_tmp4_) {
				if (vala_ccode_base_module_add_generated_external_symbol ((ValaCCodeBaseModule*) self, (ValaSymbol*) m)) {
					vala_ccode_module_visit_method ((ValaCCodeModule*) self, m);
				}
			}
		}
		if (VALA_IS_BASE_ACCESS (vala_member_access_get_inner (expr))) {
			if (vala_method_get_base_method (m) != NULL) {
				ValaClass* base_class;
				ValaCCodeIdentifier* _tmp7_;
				char* _tmp6_;
				char* _tmp5_;
				ValaCCodeFunctionCall* _tmp8_;
				ValaCCodeFunctionCall* vcast;
				ValaCCodeIdentifier* _tmp11_;
				char* _tmp10_;
				char* _tmp9_;
				ValaCCodeMemberAccess* _tmp12_;
				base_class = _vala_code_node_ref0 (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_method_get_base_method (m))));
				vcast = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = g_strdup_printf ("%s_CLASS", _tmp5_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) base_class, NULL))))), _vala_ccode_node_unref0 (_tmp7_), _g_free0 (_tmp6_), _g_free0 (_tmp5_), _tmp8_);
				vala_ccode_function_call_add_argument (vcast, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new (_tmp10_ = g_strdup_printf ("%s_parent_class", _tmp9_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self), NULL)))));
				_vala_ccode_node_unref0 (_tmp11_);
				_g_free0 (_tmp10_);
				_g_free0 (_tmp9_);
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp12_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) vcast, vala_method_get_vfunc_name (m))));
				_vala_ccode_node_unref0 (_tmp12_);
				_vala_code_node_unref0 (base_class);
				_vala_ccode_node_unref0 (vcast);
				_vala_code_node_unref0 (m);
				_vala_ccode_node_unref0 (pub_inst);
				_vala_code_node_unref0 (base_type);
				return;
			} else {
				if (vala_method_get_base_interface_method (m) != NULL) {
					ValaInterface* base_iface;
					char* _tmp14_;
					char* _tmp13_;
					char* _tmp15_;
					char* parent_iface_var;
					ValaCCodeMemberAccess* _tmp17_;
					ValaCCodeIdentifier* _tmp16_;
					base_iface = _vala_code_node_ref0 (VALA_INTERFACE (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_method_get_base_interface_method (m))));
					parent_iface_var = (_tmp15_ = g_strdup_printf ("%s_%s_parent_iface", _tmp13_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self), NULL), _tmp14_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) base_iface, NULL)), _g_free0 (_tmp14_), _g_free0 (_tmp13_), _tmp15_);
					vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp17_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new (parent_iface_var)), vala_method_get_vfunc_name (m))));
					_vala_ccode_node_unref0 (_tmp17_);
					_vala_ccode_node_unref0 (_tmp16_);
					_vala_code_node_unref0 (base_iface);
					_g_free0 (parent_iface_var);
					_vala_code_node_unref0 (m);
					_vala_ccode_node_unref0 (pub_inst);
					_vala_code_node_unref0 (base_type);
					return;
				}
			}
		}
		if (vala_method_get_base_method (m) != NULL) {
			if (!vala_ccode_module_method_has_wrapper (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_method_get_base_method (m))) {
				ValaCCodeExpression* inst;
				gboolean _tmp18_;
				ValaClass* base_class;
				ValaCCodeIdentifier* _tmp22_;
				char* _tmp21_;
				char* _tmp20_;
				ValaCCodeFunctionCall* _tmp23_;
				ValaCCodeFunctionCall* vclass;
				ValaCCodeMemberAccess* _tmp24_;
				inst = _vala_ccode_node_ref0 (pub_inst);
				_tmp18_ = FALSE;
				if (vala_member_access_get_inner (expr) != NULL) {
					_tmp18_ = !vala_expression_is_pure (vala_member_access_get_inner (expr));
				} else {
					_tmp18_ = FALSE;
				}
				if (_tmp18_) {
					ValaLocalVariable* temp_var;
					ValaCCodeExpression* ctemp;
					ValaCCodeExpression* _tmp19_;
					temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_member_access_get_inner (expr)), TRUE, NULL, FALSE);
					gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
					ctemp = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
					inst = (_tmp19_ = (ValaCCodeExpression*) vala_ccode_assignment_new (ctemp, pub_inst, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (inst), _tmp19_);
					vala_code_node_set_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (expr), (ValaCCodeNode*) ctemp);
					_vala_code_node_unref0 (temp_var);
					_vala_ccode_node_unref0 (ctemp);
				}
				base_class = _vala_code_node_ref0 (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_method_get_base_method (m))));
				vclass = (_tmp23_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new (_tmp21_ = g_strdup_printf ("%s_GET_CLASS", _tmp20_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) base_class, NULL))))), _vala_ccode_node_unref0 (_tmp22_), _g_free0 (_tmp21_), _g_free0 (_tmp20_), _tmp23_);
				vala_ccode_function_call_add_argument (vclass, inst);
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp24_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) vclass, vala_symbol_get_name ((ValaSymbol*) m))));
				_vala_ccode_node_unref0 (_tmp24_);
				_vala_ccode_node_unref0 (inst);
				_vala_code_node_unref0 (base_class);
				_vala_ccode_node_unref0 (vclass);
			} else {
				ValaCCodeIdentifier* _tmp26_;
				char* _tmp25_;
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp26_ = vala_ccode_identifier_new (_tmp25_ = vala_method_get_cname (vala_method_get_base_method (m)))));
				_vala_ccode_node_unref0 (_tmp26_);
				_g_free0 (_tmp25_);
			}
		} else {
			if (vala_method_get_base_interface_method (m) != NULL) {
				ValaCCodeIdentifier* _tmp28_;
				char* _tmp27_;
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp28_ = vala_ccode_identifier_new (_tmp27_ = vala_method_get_cname (vala_method_get_base_interface_method (m)))));
				_vala_ccode_node_unref0 (_tmp28_);
				_g_free0 (_tmp27_);
			} else {
				if (VALA_IS_CREATION_METHOD (m)) {
					ValaCCodeIdentifier* _tmp30_;
					char* _tmp29_;
					vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp30_ = vala_ccode_identifier_new (_tmp29_ = vala_method_get_real_cname (m))));
					_vala_ccode_node_unref0 (_tmp30_);
					_g_free0 (_tmp29_);
				} else {
					ValaCCodeIdentifier* _tmp32_;
					char* _tmp31_;
					vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp32_ = vala_ccode_identifier_new (_tmp31_ = vala_method_get_cname (m))));
					_vala_ccode_node_unref0 (_tmp32_);
					_g_free0 (_tmp31_);
				}
			}
		}
		_vala_code_node_unref0 (m);
	} else {
		if (VALA_IS_ARRAY_LENGTH_FIELD (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
			ValaCCodeExpression* _tmp33_;
			vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp33_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_member_access_get_inner (expr), 1)));
			_vala_ccode_node_unref0 (_tmp33_);
		} else {
			if (VALA_IS_FIELD (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
				ValaField* f;
				f = _vala_code_node_ref0 (VALA_FIELD (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
				if (vala_field_get_binding (f) == MEMBER_BINDING_INSTANCE) {
					ValaDataType* instance_target_type;
					ValaTypeSymbol* _tmp34_;
					ValaClass* cl;
					gboolean _tmp35_;
					gboolean is_gtypeinstance;
					ValaCCodeExpression* inst;
					gboolean _tmp37_;
					gboolean _tmp40_;
					instance_target_type = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) f)));
					cl = _vala_code_node_ref0 ((_tmp34_ = vala_data_type_get_data_type (instance_target_type), VALA_IS_CLASS (_tmp34_) ? ((ValaClass*) _tmp34_) : NULL));
					_tmp35_ = FALSE;
					if (vala_data_type_get_data_type (instance_target_type) == VALA_TYPESYMBOL (cl)) {
						gboolean _tmp36_;
						_tmp36_ = FALSE;
						if (cl == NULL) {
							_tmp36_ = TRUE;
						} else {
							_tmp36_ = !vala_class_get_is_compact (cl);
						}
						_tmp35_ = _tmp36_;
					} else {
						_tmp35_ = FALSE;
					}
					is_gtypeinstance = _tmp35_;
					inst = NULL;
					_tmp37_ = FALSE;
					if (is_gtypeinstance) {
						_tmp37_ = vala_symbol_get_access ((ValaSymbol*) f) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
					} else {
						_tmp37_ = FALSE;
					}
					if (_tmp37_) {
						ValaCCodeExpression* _tmp38_;
						inst = (_tmp38_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (pub_inst, "priv"), _vala_ccode_node_unref0 (inst), _tmp38_);
					} else {
						ValaCCodeExpression* _tmp39_;
						if (cl != NULL) {
							vala_ccode_base_module_generate_class_struct_declaration ((ValaCCodeBaseModule*) self, cl, ((ValaCCodeBaseModule*) self)->source_declarations);
						}
						inst = (_tmp39_ = _vala_ccode_node_ref0 (pub_inst), _vala_ccode_node_unref0 (inst), _tmp39_);
					}
					_tmp40_ = FALSE;
					if (vala_typesymbol_is_reference_type (vala_data_type_get_data_type (instance_target_type))) {
						_tmp40_ = TRUE;
					} else {
						gboolean _tmp41_;
						_tmp41_ = FALSE;
						if (vala_member_access_get_inner (expr) != NULL) {
							_tmp41_ = VALA_IS_POINTER_TYPE (vala_expression_get_value_type (vala_member_access_get_inner (expr)));
						} else {
							_tmp41_ = FALSE;
						}
						_tmp40_ = _tmp41_;
					}
					if (_tmp40_) {
						ValaCCodeMemberAccess* _tmp43_;
						char* _tmp42_;
						vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp43_ = vala_ccode_member_access_new_pointer (inst, _tmp42_ = vala_field_get_cname (f))));
						_vala_ccode_node_unref0 (_tmp43_);
						_g_free0 (_tmp42_);
					} else {
						ValaCCodeMemberAccess* _tmp45_;
						char* _tmp44_;
						vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp45_ = vala_ccode_member_access_new (inst, _tmp44_ = vala_field_get_cname (f), FALSE)));
						_vala_ccode_node_unref0 (_tmp45_);
						_g_free0 (_tmp44_);
					}
					_vala_code_node_unref0 (instance_target_type);
					_vala_code_node_unref0 (cl);
					_vala_ccode_node_unref0 (inst);
				} else {
					if (vala_field_get_binding (f) == MEMBER_BINDING_CLASS) {
						ValaClass* cl;
						ValaCCodeIdentifier* _tmp48_;
						char* _tmp47_;
						char* _tmp46_;
						ValaCCodeFunctionCall* _tmp49_;
						ValaCCodeFunctionCall* cast;
						ValaCCodeExpression* klass;
						cl = _vala_code_node_ref0 (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) f)));
						cast = (_tmp49_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp48_ = vala_ccode_identifier_new (_tmp47_ = g_strconcat (_tmp46_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) cl, NULL), "_CLASS", NULL)))), _vala_ccode_node_unref0 (_tmp48_), _g_free0 (_tmp47_), _g_free0 (_tmp46_), _tmp49_);
						klass = NULL;
						if (vala_member_access_get_inner (expr) == NULL) {
							if (((ValaCCodeBaseModule*) self)->in_static_or_class_context) {
								ValaCCodeExpression* _tmp50_;
								klass = (_tmp50_ = (ValaCCodeExpression*) vala_ccode_identifier_new ("klass"), _vala_ccode_node_unref0 (klass), _tmp50_);
							} else {
								ValaCCodeIdentifier* _tmp51_;
								ValaCCodeFunctionCall* _tmp52_;
								ValaCCodeFunctionCall* k;
								ValaCCodeIdentifier* _tmp53_;
								ValaCCodeExpression* _tmp54_;
								k = (_tmp52_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp51_ = vala_ccode_identifier_new ("G_OBJECT_GET_CLASS"))), _vala_ccode_node_unref0 (_tmp51_), _tmp52_);
								vala_ccode_function_call_add_argument (k, (ValaCCodeExpression*) (_tmp53_ = vala_ccode_identifier_new ("self")));
								_vala_ccode_node_unref0 (_tmp53_);
								klass = (_tmp54_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) k), _vala_ccode_node_unref0 (klass), _tmp54_);
								_vala_ccode_node_unref0 (k);
							}
						} else {
							ValaCCodeIdentifier* _tmp55_;
							ValaCCodeFunctionCall* _tmp56_;
							ValaCCodeFunctionCall* k;
							ValaCCodeExpression* _tmp57_;
							k = (_tmp56_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp55_ = vala_ccode_identifier_new ("G_OBJECT_GET_CLASS"))), _vala_ccode_node_unref0 (_tmp55_), _tmp56_);
							vala_ccode_function_call_add_argument (k, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (expr))));
							klass = (_tmp57_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) k), _vala_ccode_node_unref0 (klass), _tmp57_);
							_vala_ccode_node_unref0 (k);
						}
						vala_ccode_function_call_add_argument (cast, klass);
						if (vala_symbol_get_access ((ValaSymbol*) f) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
							ValaCCodeIdentifier* _tmp60_;
							char* _tmp59_;
							char* _tmp58_;
							ValaCCodeFunctionCall* _tmp61_;
							ValaCCodeFunctionCall* ccall;
							ValaCCodeIdentifier* _tmp62_;
							ValaCCodeFunctionCall* _tmp63_;
							ValaCCodeFunctionCall* ccall2;
							ValaCCodeMemberAccess* _tmp65_;
							char* _tmp64_;
							ccall = (_tmp61_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new (_tmp59_ = g_strdup_printf ("%s_GET_CLASS_PRIVATE", _tmp58_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) cl, NULL))))), _vala_ccode_node_unref0 (_tmp60_), _g_free0 (_tmp59_), _g_free0 (_tmp58_), _tmp61_);
							ccall2 = (_tmp63_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new ("G_TYPE_FROM_CLASS"))), _vala_ccode_node_unref0 (_tmp62_), _tmp63_);
							vala_ccode_function_call_add_argument (ccall2, (ValaCCodeExpression*) cast);
							vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) ccall2);
							vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp65_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) ccall, _tmp64_ = vala_field_get_cname (f))));
							_vala_ccode_node_unref0 (_tmp65_);
							_g_free0 (_tmp64_);
							_vala_ccode_node_unref0 (ccall);
							_vala_ccode_node_unref0 (ccall2);
						} else {
							ValaCCodeMemberAccess* _tmp67_;
							char* _tmp66_;
							vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp67_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) cast, _tmp66_ = vala_field_get_cname (f))));
							_vala_ccode_node_unref0 (_tmp67_);
							_g_free0 (_tmp66_);
						}
						_vala_code_node_unref0 (cl);
						_vala_ccode_node_unref0 (cast);
						_vala_ccode_node_unref0 (klass);
					} else {
						ValaCCodeIdentifier* _tmp69_;
						char* _tmp68_;
						vala_ccode_base_module_generate_field_declaration ((ValaCCodeBaseModule*) self, f, ((ValaCCodeBaseModule*) self)->source_declarations);
						vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp69_ = vala_ccode_identifier_new (_tmp68_ = vala_field_get_cname (f))));
						_vala_ccode_node_unref0 (_tmp69_);
						_g_free0 (_tmp68_);
					}
				}
				_vala_code_node_unref0 (f);
			} else {
				if (VALA_IS_CONSTANT (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
					ValaConstant* c;
					char* fn;
					c = _vala_code_node_ref0 (VALA_CONSTANT (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
					vala_ccode_base_module_generate_constant_declaration ((ValaCCodeBaseModule*) self, c, ((ValaCCodeBaseModule*) self)->source_declarations);
					fn = vala_symbol_get_full_name ((ValaSymbol*) c);
					if (_vala_strcmp0 (fn, "GLib.Log.FILE") == 0) {
						char* s;
						ValaCCodeConstant* _tmp71_;
						char* _tmp70_;
						s = g_path_get_basename (vala_source_file_get_filename (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) expr))));
						vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp71_ = vala_ccode_constant_new (_tmp70_ = g_strdup_printf ("\"%s\"", s))));
						_vala_ccode_node_unref0 (_tmp71_);
						_g_free0 (_tmp70_);
						_g_free0 (s);
					} else {
						if (_vala_strcmp0 (fn, "GLib.Log.LINE") == 0) {
							gint i;
							ValaCCodeConstant* _tmp73_;
							char* _tmp72_;
							i = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) expr));
							vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp73_ = vala_ccode_constant_new (_tmp72_ = g_strdup_printf ("%d", i))));
							_vala_ccode_node_unref0 (_tmp73_);
							_g_free0 (_tmp72_);
						} else {
							if (_vala_strcmp0 (fn, "GLib.Log.METHOD") == 0) {
								char* s;
								ValaCCodeConstant* _tmp76_;
								char* _tmp75_;
								s = g_strdup ("");
								if (vala_ccode_base_module_get_current_method ((ValaCCodeBaseModule*) self) != NULL) {
									char* _tmp74_;
									s = (_tmp74_ = vala_symbol_get_full_name ((ValaSymbol*) vala_ccode_base_module_get_current_method ((ValaCCodeBaseModule*) self)), _g_free0 (s), _tmp74_);
								}
								vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp76_ = vala_ccode_constant_new (_tmp75_ = g_strdup_printf ("\"%s\"", s))));
								_vala_ccode_node_unref0 (_tmp76_);
								_g_free0 (_tmp75_);
								_g_free0 (s);
							} else {
								ValaCCodeIdentifier* _tmp78_;
								char* _tmp77_;
								vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp78_ = vala_ccode_identifier_new (_tmp77_ = vala_constant_get_cname (c))));
								_vala_ccode_node_unref0 (_tmp78_);
								_g_free0 (_tmp77_);
							}
						}
					}
					_vala_code_node_unref0 (c);
					_g_free0 (fn);
				} else {
					if (VALA_IS_PROPERTY (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
						ValaProperty* prop;
						gboolean _tmp97_;
						gboolean _tmp98_;
						gboolean _tmp99_;
						gboolean _tmp100_;
						prop = _vala_code_node_ref0 (VALA_PROPERTY (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
						if (!VALA_IS_DYNAMIC_PROPERTY (prop)) {
							gboolean _tmp79_;
							vala_ccode_base_module_generate_property_accessor_declaration ((ValaCCodeBaseModule*) self, vala_property_get_get_accessor (prop), ((ValaCCodeBaseModule*) self)->source_declarations);
							_tmp79_ = FALSE;
							if (!vala_symbol_get_external ((ValaSymbol*) prop)) {
								_tmp79_ = vala_symbol_get_external_package ((ValaSymbol*) prop);
							} else {
								_tmp79_ = FALSE;
							}
							if (_tmp79_) {
								if (vala_ccode_base_module_add_generated_external_symbol ((ValaCCodeBaseModule*) self, (ValaSymbol*) prop)) {
									vala_ccode_module_visit_property ((ValaCCodeModule*) self, prop);
								}
							}
						}
						if (VALA_IS_BASE_ACCESS (vala_member_access_get_inner (expr))) {
							if (vala_property_get_base_property (prop) != NULL) {
								ValaClass* base_class;
								ValaCCodeIdentifier* _tmp82_;
								char* _tmp81_;
								char* _tmp80_;
								ValaCCodeFunctionCall* _tmp83_;
								ValaCCodeFunctionCall* vcast;
								ValaCCodeIdentifier* _tmp86_;
								char* _tmp85_;
								char* _tmp84_;
								ValaCCodeMemberAccess* _tmp88_;
								char* _tmp87_;
								ValaCCodeFunctionCall* _tmp89_;
								ValaCCodeFunctionCall* ccall;
								base_class = _vala_code_node_ref0 (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_property_get_base_property (prop))));
								vcast = (_tmp83_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp82_ = vala_ccode_identifier_new (_tmp81_ = g_strdup_printf ("%s_CLASS", _tmp80_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) base_class, NULL))))), _vala_ccode_node_unref0 (_tmp82_), _g_free0 (_tmp81_), _g_free0 (_tmp80_), _tmp83_);
								vala_ccode_function_call_add_argument (vcast, (ValaCCodeExpression*) (_tmp86_ = vala_ccode_identifier_new (_tmp85_ = g_strdup_printf ("%s_parent_class", _tmp84_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self), NULL)))));
								_vala_ccode_node_unref0 (_tmp86_);
								_g_free0 (_tmp85_);
								_g_free0 (_tmp84_);
								ccall = (_tmp89_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp88_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) vcast, _tmp87_ = g_strdup_printf ("get_%s", vala_symbol_get_name ((ValaSymbol*) prop))))), _vala_ccode_node_unref0 (_tmp88_), _g_free0 (_tmp87_), _tmp89_);
								vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (expr))));
								vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall);
								_vala_code_node_unref0 (base_class);
								_vala_ccode_node_unref0 (vcast);
								_vala_ccode_node_unref0 (ccall);
								_vala_code_node_unref0 (prop);
								_vala_ccode_node_unref0 (pub_inst);
								_vala_code_node_unref0 (base_type);
								return;
							} else {
								if (vala_property_get_base_interface_property (prop) != NULL) {
									ValaInterface* base_iface;
									char* _tmp91_;
									char* _tmp90_;
									char* _tmp92_;
									char* parent_iface_var;
									ValaCCodeMemberAccess* _tmp95_;
									char* _tmp94_;
									ValaCCodeIdentifier* _tmp93_;
									ValaCCodeFunctionCall* _tmp96_;
									ValaCCodeFunctionCall* ccall;
									base_iface = _vala_code_node_ref0 (VALA_INTERFACE (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_property_get_base_interface_property (prop))));
									parent_iface_var = (_tmp92_ = g_strdup_printf ("%s_%s_parent_iface", _tmp90_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self), NULL), _tmp91_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) base_iface, NULL)), _g_free0 (_tmp91_), _g_free0 (_tmp90_), _tmp92_);
									ccall = (_tmp96_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp95_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp93_ = vala_ccode_identifier_new (parent_iface_var)), _tmp94_ = g_strdup_printf ("get_%s", vala_symbol_get_name ((ValaSymbol*) prop))))), _vala_ccode_node_unref0 (_tmp95_), _g_free0 (_tmp94_), _vala_ccode_node_unref0 (_tmp93_), _tmp96_);
									vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (expr))));
									vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall);
									_vala_code_node_unref0 (base_iface);
									_g_free0 (parent_iface_var);
									_vala_ccode_node_unref0 (ccall);
									_vala_code_node_unref0 (prop);
									_vala_ccode_node_unref0 (pub_inst);
									_vala_code_node_unref0 (base_type);
									return;
								}
							}
						}
						_tmp97_ = FALSE;
						_tmp98_ = FALSE;
						_tmp99_ = FALSE;
						_tmp100_ = FALSE;
						if (vala_property_accessor_get_automatic_body (vala_property_get_get_accessor (prop))) {
							_tmp100_ = VALA_SYMBOL (vala_ccode_base_module_get_current_type_symbol ((ValaCCodeBaseModule*) self)) == vala_symbol_get_parent_symbol ((ValaSymbol*) prop);
						} else {
							_tmp100_ = FALSE;
						}
						if (_tmp100_) {
							_tmp99_ = vala_property_get_base_property (prop) == NULL;
						} else {
							_tmp99_ = FALSE;
						}
						if (_tmp99_) {
							_tmp98_ = vala_property_get_base_interface_property (prop) == NULL;
						} else {
							_tmp98_ = FALSE;
						}
						if (_tmp98_) {
							gboolean _tmp101_;
							_tmp101_ = FALSE;
							if (VALA_IS_ARRAY_TYPE (vala_property_get_property_type (prop))) {
								_tmp101_ = TRUE;
							} else {
								_tmp101_ = VALA_IS_DELEGATE_TYPE (vala_property_get_property_type (prop));
							}
							_tmp97_ = !_tmp101_;
						} else {
							_tmp97_ = FALSE;
						}
						if (_tmp97_) {
							ValaCCodeExpression* inst;
							ValaCCodeExpression* _tmp102_;
							ValaCCodeMemberAccess* _tmp104_;
							char* _tmp103_;
							inst = NULL;
							inst = (_tmp102_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (pub_inst, "priv"), _vala_ccode_node_unref0 (inst), _tmp102_);
							vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp104_ = vala_ccode_member_access_new_pointer (inst, _tmp103_ = vala_field_get_cname (vala_property_get_field (prop)))));
							_vala_ccode_node_unref0 (_tmp104_);
							_g_free0 (_tmp103_);
							_vala_ccode_node_unref0 (inst);
						} else {
							if (!vala_property_get_no_accessor_method (prop)) {
								ValaProperty* base_property;
								char* getter_cname;
								ValaCCodeIdentifier* _tmp109_;
								ValaCCodeFunctionCall* _tmp110_;
								ValaCCodeFunctionCall* ccall;
								base_property = _vala_code_node_ref0 (prop);
								if (vala_property_get_base_property (prop) != NULL) {
									ValaProperty* _tmp105_;
									base_property = (_tmp105_ = _vala_code_node_ref0 (vala_property_get_base_property (prop)), _vala_code_node_unref0 (base_property), _tmp105_);
								} else {
									if (vala_property_get_base_interface_property (prop) != NULL) {
										ValaProperty* _tmp106_;
										base_property = (_tmp106_ = _vala_code_node_ref0 (vala_property_get_base_interface_property (prop)), _vala_code_node_unref0 (base_property), _tmp106_);
									}
								}
								getter_cname = NULL;
								if (VALA_IS_DYNAMIC_PROPERTY (prop)) {
									char* _tmp107_;
									getter_cname = (_tmp107_ = vala_ccode_module_get_dynamic_property_getter_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), VALA_DYNAMIC_PROPERTY (prop)), _g_free0 (getter_cname), _tmp107_);
								} else {
									char* _tmp108_;
									getter_cname = (_tmp108_ = vala_property_accessor_get_cname (vala_property_get_get_accessor (base_property)), _g_free0 (getter_cname), _tmp108_);
								}
								ccall = (_tmp110_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp109_ = vala_ccode_identifier_new (getter_cname))), _vala_ccode_node_unref0 (_tmp109_), _tmp110_);
								if (vala_property_get_binding (prop) == MEMBER_BINDING_INSTANCE) {
									if (VALA_IS_STRUCT (vala_symbol_get_parent_symbol ((ValaSymbol*) prop))) {
										ValaCCodeExpression* _tmp111_;
										ValaCCodeUnaryExpression* unary;
										gboolean _tmp112_;
										unary = _vala_ccode_node_ref0 ((_tmp111_ = pub_inst, VALA_IS_CCODE_UNARY_EXPRESSION (_tmp111_) ? ((ValaCCodeUnaryExpression*) _tmp111_) : NULL));
										_tmp112_ = FALSE;
										if (unary != NULL) {
											_tmp112_ = vala_ccode_unary_expression_get_operator (unary) == VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION;
										} else {
											_tmp112_ = FALSE;
										}
										if (_tmp112_) {
											ValaCCodeExpression* _tmp113_;
											pub_inst = (_tmp113_ = _vala_ccode_node_ref0 (vala_ccode_unary_expression_get_inner (unary)), _vala_ccode_node_unref0 (pub_inst), _tmp113_);
										} else {
											gboolean _tmp114_;
											_tmp114_ = FALSE;
											if (VALA_IS_CCODE_IDENTIFIER (pub_inst)) {
												_tmp114_ = TRUE;
											} else {
												_tmp114_ = VALA_IS_CCODE_MEMBER_ACCESS (pub_inst);
											}
											if (_tmp114_) {
												ValaCCodeExpression* _tmp115_;
												pub_inst = (_tmp115_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, pub_inst), _vala_ccode_node_unref0 (pub_inst), _tmp115_);
											} else {
												ValaCCodeCommaExpression* ccomma;
												ValaLocalVariable* temp_var;
												ValaCCodeAssignment* _tmp117_;
												ValaCCodeExpression* _tmp116_;
												ValaCCodeUnaryExpression* _tmp119_;
												ValaCCodeExpression* _tmp118_;
												ValaCCodeExpression* _tmp120_;
												ccomma = vala_ccode_comma_expression_new ();
												temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_target_type (vala_member_access_get_inner (expr)), TRUE, NULL, FALSE);
												gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
												vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp117_ = vala_ccode_assignment_new (_tmp116_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)), pub_inst, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
												_vala_ccode_node_unref0 (_tmp117_);
												_vala_ccode_node_unref0 (_tmp116_);
												vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp119_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp118_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)))));
												_vala_ccode_node_unref0 (_tmp119_);
												_vala_ccode_node_unref0 (_tmp118_);
												pub_inst = (_tmp120_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (pub_inst), _tmp120_);
												_vala_ccode_node_unref0 (ccomma);
												_vala_code_node_unref0 (temp_var);
											}
										}
										_vala_ccode_node_unref0 (unary);
									}
									vala_ccode_function_call_add_argument (ccall, pub_inst);
								}
								if (vala_data_type_is_real_struct_type (vala_property_get_property_type (base_property))) {
									ValaCCodeCommaExpression* ccomma;
									ValaLocalVariable* temp_var;
									ValaCCodeExpression* ctemp;
									ValaCCodeUnaryExpression* _tmp121_;
									ccomma = vala_ccode_comma_expression_new ();
									temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_property_accessor_get_value_type (vala_property_get_get_accessor (base_property)), TRUE, NULL, TRUE);
									ctemp = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
									gee_collection_add ((GeeCollection*) ((ValaCCodeBaseModule*) self)->temp_vars, temp_var);
									vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp121_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ctemp)));
									_vala_ccode_node_unref0 (_tmp121_);
									vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
									vala_ccode_comma_expression_append_expression (ccomma, ctemp);
									vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
									_vala_ccode_node_unref0 (ccomma);
									_vala_code_node_unref0 (temp_var);
									_vala_ccode_node_unref0 (ctemp);
								} else {
									ValaDataType* _tmp122_;
									ValaArrayType* array_type;
									gboolean _tmp123_;
									array_type = _vala_code_node_ref0 ((_tmp122_ = vala_property_get_property_type (base_property), VALA_IS_ARRAY_TYPE (_tmp122_) ? ((ValaArrayType*) _tmp122_) : NULL));
									_tmp123_ = FALSE;
									if (array_type != NULL) {
										_tmp123_ = !vala_property_get_no_array_length (base_property);
									} else {
										_tmp123_ = FALSE;
									}
									if (_tmp123_) {
										{
											gint dim;
											dim = 1;
											{
												gboolean _tmp124_;
												_tmp124_ = TRUE;
												while (TRUE) {
													ValaLocalVariable* temp_var;
													ValaCCodeExpression* ctemp;
													ValaCCodeUnaryExpression* _tmp125_;
													if (!_tmp124_) {
														dim++;
													}
													_tmp124_ = FALSE;
													if (!(dim <= vala_array_type_get_rank (array_type))) {
														break;
													}
													temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL, TRUE);
													ctemp = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
													gee_collection_add ((GeeCollection*) ((ValaCCodeBaseModule*) self)->temp_vars, temp_var);
													vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp125_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ctemp)));
													_vala_ccode_node_unref0 (_tmp125_);
													vala_expression_append_array_size ((ValaExpression*) expr, ctemp);
													_vala_code_node_unref0 (temp_var);
													_vala_ccode_node_unref0 (ctemp);
												}
											}
										}
									} else {
										ValaDataType* _tmp126_;
										ValaDelegateType* delegate_type;
										gboolean _tmp127_;
										delegate_type = _vala_code_node_ref0 ((_tmp126_ = vala_property_get_property_type (base_property), VALA_IS_DELEGATE_TYPE (_tmp126_) ? ((ValaDelegateType*) _tmp126_) : NULL));
										_tmp127_ = FALSE;
										if (delegate_type != NULL) {
											_tmp127_ = vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type));
										} else {
											_tmp127_ = FALSE;
										}
										if (_tmp127_) {
											ValaPointerType* _tmp129_;
											ValaVoidType* _tmp128_;
											ValaLocalVariable* _tmp130_;
											ValaLocalVariable* temp_var;
											ValaCCodeExpression* ctemp;
											ValaCCodeUnaryExpression* _tmp131_;
											temp_var = (_tmp130_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, (ValaDataType*) (_tmp129_ = vala_pointer_type_new ((ValaDataType*) (_tmp128_ = vala_void_type_new (NULL)), NULL)), TRUE, NULL, TRUE), _vala_code_node_unref0 (_tmp129_), _vala_code_node_unref0 (_tmp128_), _tmp130_);
											ctemp = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
											gee_collection_add ((GeeCollection*) ((ValaCCodeBaseModule*) self)->temp_vars, temp_var);
											vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp131_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ctemp)));
											_vala_ccode_node_unref0 (_tmp131_);
											vala_expression_set_delegate_target ((ValaExpression*) expr, ctemp);
											_vala_code_node_unref0 (temp_var);
											_vala_ccode_node_unref0 (ctemp);
										}
										_vala_code_node_unref0 (delegate_type);
									}
									vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall);
									_vala_code_node_unref0 (array_type);
								}
								_vala_code_node_unref0 (base_property);
								_g_free0 (getter_cname);
								_vala_ccode_node_unref0 (ccall);
							} else {
								ValaCCodeIdentifier* _tmp132_;
								ValaCCodeFunctionCall* _tmp133_;
								ValaCCodeFunctionCall* ccall;
								ValaCCodeConstant* _tmp134_;
								ValaCCodeCommaExpression* ccomma;
								ValaLocalVariable* temp_var;
								ValaCCodeExpression* ctemp;
								ValaCCodeUnaryExpression* _tmp137_;
								ValaCCodeConstant* _tmp138_;
								ccall = (_tmp133_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp132_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (_tmp132_), _tmp133_);
								vala_ccode_function_call_add_argument (ccall, pub_inst);
								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp134_ = vala_property_get_canonical_cconstant (prop)));
								_vala_ccode_node_unref0 (_tmp134_);
								if (!vala_data_type_get_value_owned (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)))) {
									ValaDataType* owned_value_type;
									owned_value_type = vala_data_type_copy (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)));
									vala_data_type_set_value_owned (owned_value_type, TRUE);
									if (vala_ccode_base_module_requires_copy ((ValaCCodeBaseModule*) self, owned_value_type)) {
										char* _tmp136_;
										char* _tmp135_;
										vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_property_get_get_accessor (prop)), _tmp136_ = g_strdup_printf ("unowned return value for getter of property `%s' not supported without accessor", _tmp135_ = vala_symbol_get_full_name ((ValaSymbol*) prop)));
										_g_free0 (_tmp136_);
										_g_free0 (_tmp135_);
									}
									_vala_code_node_unref0 (owned_value_type);
								}
								ccomma = vala_ccode_comma_expression_new ();
								temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type ((ValaExpression*) expr), TRUE, NULL, TRUE);
								ctemp = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
								gee_collection_add ((GeeCollection*) ((ValaCCodeBaseModule*) self)->temp_vars, temp_var);
								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp137_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ctemp)));
								_vala_ccode_node_unref0 (_tmp137_);
								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp138_ = vala_ccode_constant_new ("NULL")));
								_vala_ccode_node_unref0 (_tmp138_);
								vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
								vala_ccode_comma_expression_append_expression (ccomma, ctemp);
								vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
								_vala_ccode_node_unref0 (ccall);
								_vala_ccode_node_unref0 (ccomma);
								_vala_code_node_unref0 (temp_var);
								_vala_ccode_node_unref0 (ctemp);
							}
						}
						_vala_code_node_unref0 (prop);
					} else {
						if (VALA_IS_ENUM_VALUE (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
							ValaEnumValue* ev;
							ValaCCodeConstant* _tmp140_;
							char* _tmp139_;
							ev = _vala_code_node_ref0 (VALA_ENUM_VALUE (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
							vala_ccode_base_module_generate_enum_declaration ((ValaCCodeBaseModule*) self, VALA_ENUM (vala_symbol_get_parent_symbol ((ValaSymbol*) ev)), ((ValaCCodeBaseModule*) self)->source_declarations);
							vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp140_ = vala_ccode_constant_new (_tmp139_ = vala_enum_value_get_cname (ev))));
							_vala_ccode_node_unref0 (_tmp140_);
							_g_free0 (_tmp139_);
							_vala_code_node_unref0 (ev);
						} else {
							if (VALA_IS_LOCAL_VARIABLE (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
								ValaLocalVariable* local;
								local = _vala_code_node_ref0 (VALA_LOCAL_VARIABLE (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
								if (vala_local_variable_get_is_result (local)) {
									ValaCCodeIdentifier* _tmp141_;
									vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp141_ = vala_ccode_identifier_new ("result")));
									_vala_ccode_node_unref0 (_tmp141_);
								} else {
									if (vala_local_variable_get_captured (local)) {
										ValaBlock* block;
										ValaCCodeMemberAccess* _tmp145_;
										char* _tmp144_;
										ValaCCodeExpression* _tmp143_;
										char* _tmp142_;
										block = _vala_code_node_ref0 (VALA_BLOCK (vala_symbol_get_parent_symbol ((ValaSymbol*) local)));
										vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp145_ = vala_ccode_member_access_new_pointer (_tmp143_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, _tmp142_ = g_strdup_printf ("_data%d_", vala_ccode_base_module_get_block_id ((ValaCCodeBaseModule*) self, block))), _tmp144_ = vala_ccode_base_module_get_variable_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) local)))));
										_vala_ccode_node_unref0 (_tmp145_);
										_g_free0 (_tmp144_);
										_vala_ccode_node_unref0 (_tmp143_);
										_g_free0 (_tmp142_);
										_vala_code_node_unref0 (block);
									} else {
										ValaCCodeExpression* _tmp146_;
										vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp146_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) local))));
										_vala_ccode_node_unref0 (_tmp146_);
									}
								}
								_vala_code_node_unref0 (local);
							} else {
								if (VALA_IS_FORMAL_PARAMETER (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
									ValaFormalParameter* p;
									p = _vala_code_node_ref0 (VALA_FORMAL_PARAMETER (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
									if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) p), "this") == 0) {
										gboolean _tmp147_;
										_tmp147_ = FALSE;
										if (vala_ccode_base_module_get_current_method ((ValaCCodeBaseModule*) self) != NULL) {
											_tmp147_ = vala_method_get_coroutine (vala_ccode_base_module_get_current_method ((ValaCCodeBaseModule*) self));
										} else {
											_tmp147_ = FALSE;
										}
										if (_tmp147_) {
											ValaCCodeMemberAccess* _tmp149_;
											ValaCCodeIdentifier* _tmp148_;
											vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp149_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp148_ = vala_ccode_identifier_new ("data")), "self")));
											_vala_ccode_node_unref0 (_tmp149_);
											_vala_ccode_node_unref0 (_tmp148_);
										} else {
											ValaTypeSymbol* _tmp150_;
											ValaStruct* st;
											gboolean _tmp151_;
											st = _vala_code_node_ref0 ((_tmp150_ = vala_ccode_base_module_get_current_type_symbol ((ValaCCodeBaseModule*) self), VALA_IS_STRUCT (_tmp150_) ? ((ValaStruct*) _tmp150_) : NULL));
											_tmp151_ = FALSE;
											if (st != NULL) {
												_tmp151_ = !vala_struct_is_simple_type (st);
											} else {
												_tmp151_ = FALSE;
											}
											if (_tmp151_) {
												ValaCCodeIdentifier* _tmp152_;
												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp152_ = vala_ccode_identifier_new ("(*self)")));
												_vala_ccode_node_unref0 (_tmp152_);
											} else {
												ValaCCodeIdentifier* _tmp153_;
												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp153_ = vala_ccode_identifier_new ("self")));
												_vala_ccode_node_unref0 (_tmp153_);
											}
											_vala_code_node_unref0 (st);
										}
									} else {
										if (vala_formal_parameter_get_captured (p)) {
											ValaBlock* block;
											ValaCCodeMemberAccess* _tmp157_;
											char* _tmp156_;
											ValaCCodeExpression* _tmp155_;
											char* _tmp154_;
											block = _vala_code_node_ref0 (vala_method_get_body (VALA_METHOD (vala_symbol_get_parent_symbol ((ValaSymbol*) p))));
											vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp157_ = vala_ccode_member_access_new_pointer (_tmp155_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, _tmp154_ = g_strdup_printf ("_data%d_", vala_ccode_base_module_get_block_id ((ValaCCodeBaseModule*) self, block))), _tmp156_ = vala_ccode_base_module_get_variable_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) p)))));
											_vala_ccode_node_unref0 (_tmp157_);
											_g_free0 (_tmp156_);
											_vala_ccode_node_unref0 (_tmp155_);
											_g_free0 (_tmp154_);
											_vala_code_node_unref0 (block);
										} else {
											gboolean _tmp158_;
											_tmp158_ = FALSE;
											if (vala_ccode_base_module_get_current_method ((ValaCCodeBaseModule*) self) != NULL) {
												_tmp158_ = vala_method_get_coroutine (vala_ccode_base_module_get_current_method ((ValaCCodeBaseModule*) self));
											} else {
												_tmp158_ = FALSE;
											}
											if (_tmp158_) {
												ValaCCodeExpression* _tmp159_;
												vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp159_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) p))));
												_vala_ccode_node_unref0 (_tmp159_);
											} else {
												ValaTypeSymbol* _tmp160_;
												ValaStruct* type_as_struct;
												gboolean _tmp161_;
												type_as_struct = _vala_code_node_ref0 ((_tmp160_ = vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (p)), VALA_IS_STRUCT (_tmp160_) ? ((ValaStruct*) _tmp160_) : NULL));
												_tmp161_ = FALSE;
												if (vala_formal_parameter_get_direction (p) != VALA_PARAMETER_DIRECTION_IN) {
													_tmp161_ = TRUE;
												} else {
													gboolean _tmp162_;
													gboolean _tmp163_;
													_tmp162_ = FALSE;
													_tmp163_ = FALSE;
													if (type_as_struct != NULL) {
														_tmp163_ = !vala_struct_is_simple_type (type_as_struct);
													} else {
														_tmp163_ = FALSE;
													}
													if (_tmp163_) {
														_tmp162_ = !vala_data_type_get_nullable (vala_formal_parameter_get_parameter_type (p));
													} else {
														_tmp162_ = FALSE;
													}
													_tmp161_ = _tmp162_;
												}
												if (_tmp161_) {
													ValaCCodeIdentifier* _tmp166_;
													char* _tmp165_;
													char* _tmp164_;
													vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp166_ = vala_ccode_identifier_new (_tmp165_ = g_strdup_printf ("(*%s)", _tmp164_ = vala_ccode_base_module_get_variable_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) p))))));
													_vala_ccode_node_unref0 (_tmp166_);
													_g_free0 (_tmp165_);
													_g_free0 (_tmp164_);
												} else {
													gboolean _tmp167_;
													gboolean _tmp168_;
													gboolean _tmp169_;
													_tmp167_ = FALSE;
													_tmp168_ = FALSE;
													_tmp169_ = FALSE;
													if (vala_ccode_base_module_get_current_property_accessor ((ValaCCodeBaseModule*) self) != NULL) {
														_tmp169_ = vala_property_accessor_get_writable (vala_ccode_base_module_get_current_property_accessor ((ValaCCodeBaseModule*) self));
													} else {
														_tmp169_ = FALSE;
													}
													if (_tmp169_) {
														_tmp168_ = vala_property_accessor_get_value_parameter (vala_ccode_base_module_get_current_property_accessor ((ValaCCodeBaseModule*) self)) == p;
													} else {
														_tmp168_ = FALSE;
													}
													if (_tmp168_) {
														_tmp167_ = vala_data_type_is_real_struct_type (vala_property_get_property_type (vala_property_accessor_get_prop (vala_ccode_base_module_get_current_property_accessor ((ValaCCodeBaseModule*) self))));
													} else {
														_tmp167_ = FALSE;
													}
													if (_tmp167_) {
														ValaCCodeIdentifier* _tmp170_;
														vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp170_ = vala_ccode_identifier_new ("(*value)")));
														_vala_ccode_node_unref0 (_tmp170_);
													} else {
														ValaCCodeExpression* _tmp171_;
														vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp171_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) p))));
														_vala_ccode_node_unref0 (_tmp171_);
													}
												}
												_vala_code_node_unref0 (type_as_struct);
											}
										}
									}
									_vala_code_node_unref0 (p);
								}
							}
						}
					}
				}
			}
		}
	}
	_vala_ccode_node_unref0 (pub_inst);
	_vala_code_node_unref0 (base_type);
}


static void vala_ccode_member_access_module_class_init (ValaCCodeMemberAccessModuleClass * klass) {
	vala_ccode_member_access_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->visit_member_access = vala_ccode_member_access_module_real_visit_member_access;
}


static void vala_ccode_member_access_module_instance_init (ValaCCodeMemberAccessModule * self) {
}


GType vala_ccode_member_access_module_get_type (void) {
	static GType vala_ccode_member_access_module_type_id = 0;
	if (vala_ccode_member_access_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeMemberAccessModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_member_access_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeMemberAccessModule), 0, (GInstanceInitFunc) vala_ccode_member_access_module_instance_init, NULL };
		vala_ccode_member_access_module_type_id = g_type_register_static (VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, "ValaCCodeMemberAccessModule", &g_define_type_info, 0);
	}
	return vala_ccode_member_access_module_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




