/* valaccodemethodcallmodule.vala
 *
 * Copyright (C) 2006-2009  Jürg Billeter
 * Copyright (C) 2006-2008  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>
#include <float.h>
#include <math.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))

typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;

#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))

typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;

#define VALA_TYPE_CCODE_STRUCT_MODULE (vala_ccode_struct_module_get_type ())
#define VALA_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModule))
#define VALA_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
#define VALA_IS_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_IS_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_CCODE_STRUCT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))

typedef struct _ValaCCodeStructModule ValaCCodeStructModule;
typedef struct _ValaCCodeStructModuleClass ValaCCodeStructModuleClass;
typedef struct _ValaCCodeStructModulePrivate ValaCCodeStructModulePrivate;

#define VALA_TYPE_CCODE_METHOD_MODULE (vala_ccode_method_module_get_type ())
#define VALA_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModule))
#define VALA_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))
#define VALA_IS_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_IS_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_CCODE_METHOD_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))

typedef struct _ValaCCodeMethodModule ValaCCodeMethodModule;
typedef struct _ValaCCodeMethodModuleClass ValaCCodeMethodModuleClass;
typedef struct _ValaCCodeMethodModulePrivate ValaCCodeMethodModulePrivate;

#define VALA_TYPE_CCODE_CONTROL_FLOW_MODULE (vala_ccode_control_flow_module_get_type ())
#define VALA_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModule))
#define VALA_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_CCODE_CONTROL_FLOW_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))

typedef struct _ValaCCodeControlFlowModule ValaCCodeControlFlowModule;
typedef struct _ValaCCodeControlFlowModuleClass ValaCCodeControlFlowModuleClass;
typedef struct _ValaCCodeControlFlowModulePrivate ValaCCodeControlFlowModulePrivate;

#define VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE (vala_ccode_member_access_module_get_type ())
#define VALA_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModule))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))

typedef struct _ValaCCodeMemberAccessModule ValaCCodeMemberAccessModule;
typedef struct _ValaCCodeMemberAccessModuleClass ValaCCodeMemberAccessModuleClass;
typedef struct _ValaCCodeMemberAccessModulePrivate ValaCCodeMemberAccessModulePrivate;

#define VALA_TYPE_CCODE_ASSIGNMENT_MODULE (vala_ccode_assignment_module_get_type ())
#define VALA_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModule))
#define VALA_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_CCODE_ASSIGNMENT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))

typedef struct _ValaCCodeAssignmentModule ValaCCodeAssignmentModule;
typedef struct _ValaCCodeAssignmentModuleClass ValaCCodeAssignmentModuleClass;
typedef struct _ValaCCodeAssignmentModulePrivate ValaCCodeAssignmentModulePrivate;

#define VALA_TYPE_CCODE_METHOD_CALL_MODULE (vala_ccode_method_call_module_get_type ())
#define VALA_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModule))
#define VALA_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))
#define VALA_IS_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_IS_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_CCODE_METHOD_CALL_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))

typedef struct _ValaCCodeMethodCallModule ValaCCodeMethodCallModule;
typedef struct _ValaCCodeMethodCallModuleClass ValaCCodeMethodCallModuleClass;
typedef struct _ValaCCodeMethodCallModulePrivate ValaCCodeMethodCallModulePrivate;

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

#define VALA_TYPE_CTYPE (vala_ctype_get_type ())
#define VALA_CTYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CTYPE, ValaCType))
#define VALA_CTYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CTYPE, ValaCTypeClass))
#define VALA_IS_CTYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CTYPE))
#define VALA_IS_CTYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CTYPE))
#define VALA_CTYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CTYPE, ValaCTypeClass))

typedef struct _ValaCType ValaCType;
typedef struct _ValaCTypeClass ValaCTypeClass;

struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

struct _ValaCCodeBaseModule {
	ValaCCodeModule parent_instance;
	ValaCCodeBaseModulePrivate * priv;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaTryStatement* current_try;
	ValaCCodeDeclarationSpace* header_declarations;
	ValaCCodeDeclarationSpace* internal_header_declarations;
	ValaCCodeDeclarationSpace* source_declarations;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* class_init_fragment;
	ValaCCodeFragment* base_init_fragment;
	ValaCCodeFragment* class_finalize_fragment;
	ValaCCodeFragment* base_finalize_fragment;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_finalize_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeFragment* module_init_fragment;
	ValaCCodeStruct* param_spec_struct;
	ValaCCodeStruct* closure_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeFunction* function;
	ValaCCodeFragment* pre_statement_fragment;
	GeeArrayList* temp_vars;
	GeeArrayList* temp_ref_vars;
	GeeSet* user_marshal_set;
	GeeSet* predefined_marshal_set;
	gint next_temp_var_id;
	gboolean in_constructor;
	gboolean in_static_or_class_context;
	gboolean current_method_inner_error;
	gint next_coroutine_state;
	ValaDataType* void_type;
	ValaDataType* bool_type;
	ValaDataType* char_type;
	ValaDataType* uchar_type;
	ValaDataType* unichar_type;
	ValaDataType* short_type;
	ValaDataType* ushort_type;
	ValaDataType* int_type;
	ValaDataType* uint_type;
	ValaDataType* long_type;
	ValaDataType* ulong_type;
	ValaDataType* int8_type;
	ValaDataType* uint8_type;
	ValaDataType* int16_type;
	ValaDataType* uint16_type;
	ValaDataType* int32_type;
	ValaDataType* uint32_type;
	ValaDataType* int64_type;
	ValaDataType* uint64_type;
	ValaDataType* string_type;
	ValaDataType* float_type;
	ValaDataType* double_type;
	ValaTypeSymbol* gtype_type;
	ValaTypeSymbol* gobject_type;
	ValaErrorType* gerror_type;
	ValaClass* glist_type;
	ValaClass* gslist_type;
	ValaClass* gvaluearray_type;
	ValaTypeSymbol* gstringbuilder_type;
	ValaTypeSymbol* garray_type;
	ValaTypeSymbol* gbytearray_type;
	ValaTypeSymbol* gptrarray_type;
	ValaDataType* gquark_type;
	ValaStruct* gvalue_type;
	ValaStruct* mutex_type;
	ValaTypeSymbol* type_module_type;
	ValaTypeSymbol* dbus_object_type;
	gboolean in_plugin;
	char* module_init_param_name;
	gboolean gvaluecollector_h_needed;
	gboolean requires_array_free;
	gboolean requires_array_move;
	gboolean requires_array_length;
	gboolean requires_strcmp0;
	gboolean dbus_glib_h_needed;
	gboolean dbus_glib_h_needed_in_header;
	GeeSet* wrappers;
	GeeMap* variable_name_map;
};

struct _ValaCCodeBaseModuleClass {
	ValaCCodeModuleClass parent_class;
	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
	char* (*append_struct_array_free) (ValaCCodeBaseModule* self, ValaStruct* st);
	ValaCCodeExpression* (*get_unref_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
	void (*append_local_free) (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
	gboolean (*variable_accessible_in_finally) (ValaCCodeBaseModule* self, ValaLocalVariable* local);
	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_ref_cexpression) (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
	gboolean (*is_gobject_property) (ValaCCodeBaseModule* self, ValaProperty* prop);
};

struct _ValaCCodeStructModule {
	ValaCCodeBaseModule parent_instance;
	ValaCCodeStructModulePrivate * priv;
};

struct _ValaCCodeStructModuleClass {
	ValaCCodeBaseModuleClass parent_class;
};

struct _ValaCCodeMethodModule {
	ValaCCodeStructModule parent_instance;
	ValaCCodeMethodModulePrivate * priv;
};

struct _ValaCCodeMethodModuleClass {
	ValaCCodeStructModuleClass parent_class;
	void (*generate_method_result_declaration) (ValaCCodeMethodModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeFunction* cfunc, GeeMap* cparam_map, GeeMap* carg_map);
	void (*generate_parameter) (ValaCCodeMethodModule* self, ValaFormalParameter* param, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, GeeMap* carg_map);
};

struct _ValaCCodeControlFlowModule {
	ValaCCodeMethodModule parent_instance;
	ValaCCodeControlFlowModulePrivate * priv;
};

struct _ValaCCodeControlFlowModuleClass {
	ValaCCodeMethodModuleClass parent_class;
};

struct _ValaCCodeMemberAccessModule {
	ValaCCodeControlFlowModule parent_instance;
	ValaCCodeMemberAccessModulePrivate * priv;
};

struct _ValaCCodeMemberAccessModuleClass {
	ValaCCodeControlFlowModuleClass parent_class;
};

struct _ValaCCodeAssignmentModule {
	ValaCCodeMemberAccessModule parent_instance;
	ValaCCodeAssignmentModulePrivate * priv;
};

struct _ValaCCodeAssignmentModuleClass {
	ValaCCodeMemberAccessModuleClass parent_class;
};

struct _ValaCCodeMethodCallModule {
	ValaCCodeAssignmentModule parent_instance;
	ValaCCodeMethodCallModulePrivate * priv;
};

struct _ValaCCodeMethodCallModuleClass {
	ValaCCodeAssignmentModuleClass parent_class;
};


static gpointer vala_ccode_method_call_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_base_module_get_type (void);
gpointer vala_ccode_declaration_space_ref (gpointer instance);
void vala_ccode_declaration_space_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_declaration_space (const GValue* value);
GType vala_ccode_declaration_space_get_type (void);
GType vala_ccode_struct_module_get_type (void);
GType vala_ccode_method_module_get_type (void);
GType vala_ccode_control_flow_module_get_type (void);
GType vala_ccode_member_access_module_get_type (void);
GType vala_ccode_assignment_module_get_type (void);
GType vala_ccode_method_call_module_get_type (void);
enum  {
	VALA_CCODE_METHOD_CALL_MODULE_DUMMY_PROPERTY
};
GType vala_ccode_generator_get_type (void);
ValaCCodeAssignmentModule* vala_ccode_assignment_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeAssignmentModule* vala_ccode_assignment_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeMethodCallModule* vala_ccode_method_call_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeMethodCallModule* vala_ccode_method_call_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
void vala_ccode_base_module_generate_method_declaration (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
gint vala_ccode_base_module_get_param_pos (ValaCCodeBaseModule* self, double param_pos, gboolean ellipsis);
ValaCodeContext* vala_ccode_base_module_get_context (ValaCCodeBaseModule* self);
ValaClass* vala_ccode_base_module_get_current_class (ValaCCodeBaseModule* self);
void vala_ccode_base_module_add_generic_type_arguments (ValaCCodeBaseModule* self, ValaCCodeFunctionCall* ccall, GeeList* type_args, ValaCodeNode* expr, gboolean is_chainup);
ValaLocalVariable* vala_ccode_base_module_get_temp_variable (ValaCCodeBaseModule* self, ValaDataType* type, gboolean value_owned, ValaCodeNode* node_reference, gboolean init);
ValaCCodeExpression* vala_ccode_base_module_get_variable_cexpression (ValaCCodeBaseModule* self, const char* name);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
void vala_ccode_module_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method);
ValaCCodeExpression* vala_ccode_module_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
ValaCCodeExpression* vala_ccode_base_module_get_delegate_target_cexpression (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
gint vala_ccode_base_module_get_block_id (ValaCCodeBaseModule* self, ValaBlock* b);
ValaDataType* vala_ccode_base_module_get_this_type (ValaCCodeBaseModule* self);
ValaCCodeExpression* vala_ccode_base_module_get_dup_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
ValaCCodeExpression* vala_ccode_base_module_get_destroy_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, gboolean is_chainup);
ValaCCodeExpression* vala_ccode_base_module_handle_struct_argument (ValaCCodeBaseModule* self, ValaFormalParameter* param, ValaExpression* arg, ValaCCodeExpression* cexpr);
gboolean vala_ccode_base_module_requires_destroy (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_transform_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
ValaCCodeExpression* vala_ccode_base_module_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
ValaCType* vala_ctype_new (const char* ctype_name);
ValaCType* vala_ctype_construct (GType object_type, const char* ctype_name);
GType vala_ctype_get_type (void);
ValaMethod* vala_ccode_base_module_get_current_method (ValaCCodeBaseModule* self);
void vala_ccode_declaration_space_add_include (ValaCCodeDeclarationSpace* self, const char* filename, gboolean local);
static void vala_ccode_method_call_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr);
static int _vala_strcmp0 (const char * str1, const char * str2);



ValaCCodeMethodCallModule* vala_ccode_method_call_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaCCodeMethodCallModule* self;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaCCodeMethodCallModule*) vala_ccode_assignment_module_construct (object_type, codegen, next);
	return self;
}


ValaCCodeMethodCallModule* vala_ccode_method_call_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_ccode_method_call_module_construct (VALA_TYPE_CCODE_METHOD_CALL_MODULE, codegen, next);
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


static gpointer _gee_collection_object_ref0 (gpointer self) {
	return self ? gee_collection_object_ref (self) : NULL;
}


static void vala_ccode_method_call_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr) {
	ValaCCodeMethodCallModule * self;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeFunctionCall* async_call;
	ValaMethod* m;
	ValaDelegate* deleg;
	GeeList* params;
	ValaExpression* _tmp0_;
	ValaMemberAccess* ma;
	ValaDataType* itype;
	GeeList* _tmp1_;
	GeeHashMap* in_arg_map;
	GeeHashMap* out_arg_map;
	gboolean _tmp12_;
	ValaCCodeExpression* ccall_expr;
	ValaCCodeExpression* instance;
	gboolean _tmp42_;
	gboolean _tmp43_;
	gboolean _tmp72_;
	gboolean ellipsis;
	gint i;
	gint arg_pos;
	GeeIterator* params_it;
	gboolean _tmp196_;
	gboolean _tmp197_;
	gboolean _tmp215_;
	gboolean _tmp229_;
	gboolean _tmp234_;
	ValaDataType* _tmp244_;
	gboolean _tmp245_;
	gboolean return_result_via_out_param;
	gboolean _tmp246_;
	gint last_pos;
	gint min_pos;
	gboolean _tmp264_;
	gboolean _tmp265_;
	self = (ValaCCodeMethodCallModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_method_call_get_call (expr))));
	async_call = NULL;
	m = NULL;
	deleg = NULL;
	params = NULL;
	ma = _vala_code_node_ref0 ((_tmp0_ = vala_method_call_get_call (expr), VALA_IS_MEMBER_ACCESS (_tmp0_) ? ((ValaMemberAccess*) _tmp0_) : NULL));
	itype = _vala_code_node_ref0 (vala_expression_get_value_type (vala_method_call_get_call (expr)));
	params = (_tmp1_ = vala_data_type_get_parameters (itype), _gee_collection_object_unref0 (params), _tmp1_);
	if (VALA_IS_METHOD_TYPE (itype)) {
		ValaMethod* _tmp2_;
		g_assert (ma != NULL);
		m = (_tmp2_ = _vala_code_node_ref0 (vala_method_type_get_method_symbol (VALA_METHOD_TYPE (itype))), _vala_code_node_unref0 (m), _tmp2_);
	} else {
		if (VALA_IS_SIGNAL_TYPE (itype)) {
			ValaSignalType* sig_type;
			gboolean _tmp3_;
			gboolean _tmp4_;
			sig_type = _vala_code_node_ref0 (VALA_SIGNAL_TYPE (itype));
			_tmp3_ = FALSE;
			_tmp4_ = FALSE;
			if (ma != NULL) {
				_tmp4_ = VALA_IS_BASE_ACCESS (vala_member_access_get_inner (ma));
			} else {
				_tmp4_ = FALSE;
			}
			if (_tmp4_) {
				_tmp3_ = vala_signal_get_is_virtual (vala_signal_type_get_signal_symbol (sig_type));
			} else {
				_tmp3_ = FALSE;
			}
			if (_tmp3_) {
				ValaMethod* _tmp5_;
				m = (_tmp5_ = _vala_code_node_ref0 (vala_signal_get_default_handler (vala_signal_type_get_signal_symbol (sig_type))), _vala_code_node_unref0 (m), _tmp5_);
			} else {
				ValaCCodeFunctionCall* _tmp6_;
				ccall = (_tmp6_ = _vala_ccode_node_ref0 (VALA_CCODE_FUNCTION_CALL (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_method_call_get_call (expr)))), _vala_ccode_node_unref0 (ccall), _tmp6_);
			}
			_vala_code_node_unref0 (sig_type);
		} else {
			if (VALA_IS_OBJECT_TYPE (itype)) {
				ValaClass* cl;
				ValaMethod* _tmp7_;
				ValaCCodeFunctionCall* _tmp10_;
				ValaCCodeIdentifier* _tmp9_;
				char* _tmp8_;
				cl = _vala_code_node_ref0 (VALA_CLASS (vala_object_type_get_type_symbol (VALA_OBJECT_TYPE (itype))));
				m = (_tmp7_ = _vala_code_node_ref0 (vala_class_get_default_construction_method (cl)), _vala_code_node_unref0 (m), _tmp7_);
				vala_ccode_base_module_generate_method_declaration ((ValaCCodeBaseModule*) self, m, ((ValaCCodeBaseModule*) self)->source_declarations);
				ccall = (_tmp10_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new (_tmp8_ = vala_method_get_real_cname (m)))), _vala_ccode_node_unref0 (ccall), _tmp10_);
				_vala_ccode_node_unref0 (_tmp9_);
				_g_free0 (_tmp8_);
				_vala_code_node_unref0 (cl);
			} else {
				if (VALA_IS_DELEGATE_TYPE (itype)) {
					ValaDelegate* _tmp11_;
					deleg = (_tmp11_ = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (VALA_DELEGATE_TYPE (itype))), _vala_code_node_unref0 (deleg), _tmp11_);
				}
			}
		}
	}
	in_arg_map = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_EXPRESSION, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal);
	out_arg_map = _gee_collection_object_ref0 (in_arg_map);
	_tmp12_ = FALSE;
	if (m != NULL) {
		_tmp12_ = vala_method_get_coroutine (m);
	} else {
		_tmp12_ = FALSE;
	}
	if (_tmp12_) {
		ValaCCodeFunctionCall* _tmp15_;
		ValaCCodeIdentifier* _tmp14_;
		char* _tmp13_;
		gboolean _tmp16_;
		async_call = (_tmp15_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new (_tmp13_ = vala_method_get_cname (m)))), _vala_ccode_node_unref0 (async_call), _tmp15_);
		_vala_ccode_node_unref0 (_tmp14_);
		_g_free0 (_tmp13_);
		_tmp16_ = FALSE;
		if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") == 0) {
			_tmp16_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
		} else {
			_tmp16_ = FALSE;
		}
		if (_tmp16_) {
			ValaCCodeFunctionCall* _tmp17_;
			GeeList* _tmp18_;
			ccall = (_tmp17_ = _vala_ccode_node_ref0 (async_call), _vala_ccode_node_unref0 (ccall), _tmp17_);
			params = (_tmp18_ = vala_method_get_async_begin_parameters (m), _gee_collection_object_unref0 (params), _tmp18_);
		} else {
			gboolean _tmp19_;
			_tmp19_ = FALSE;
			if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "end") == 0) {
				_tmp19_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
			} else {
				_tmp19_ = FALSE;
			}
			if (_tmp19_) {
				ValaCCodeFunctionCall* _tmp22_;
				ValaCCodeIdentifier* _tmp21_;
				char* _tmp20_;
				GeeList* _tmp23_;
				ccall = (_tmp22_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new (_tmp20_ = vala_method_get_finish_cname (m)))), _vala_ccode_node_unref0 (ccall), _tmp22_);
				_vala_ccode_node_unref0 (_tmp21_);
				_g_free0 (_tmp20_);
				params = (_tmp23_ = vala_method_get_async_end_parameters (m), _gee_collection_object_unref0 (params), _tmp23_);
			} else {
				if (!vala_method_call_get_is_yield_expression (expr)) {
					ValaCCodeFunctionCall* _tmp24_;
					GeeList* _tmp25_;
					ccall = (_tmp24_ = _vala_ccode_node_ref0 (async_call), _vala_ccode_node_unref0 (ccall), _tmp24_);
					params = (_tmp25_ = vala_method_get_async_begin_parameters (m), _gee_collection_object_unref0 (params), _tmp25_);
				} else {
					ValaCCodeFunctionCall* _tmp28_;
					ValaCCodeIdentifier* _tmp27_;
					char* _tmp26_;
					GeeHashMap* _tmp29_;
					ValaCCodeMemberAccess* _tmp31_;
					ValaCCodeIdentifier* _tmp30_;
					ccall = (_tmp28_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new (_tmp26_ = vala_method_get_finish_cname (m)))), _vala_ccode_node_unref0 (ccall), _tmp28_);
					_vala_ccode_node_unref0 (_tmp27_);
					_g_free0 (_tmp26_);
					out_arg_map = (_tmp29_ = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_EXPRESSION, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal), _gee_collection_object_unref0 (out_arg_map), _tmp29_);
					gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, 0.1, FALSE)), (ValaCCodeExpression*) (_tmp31_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("data")), "_res_")));
					_vala_ccode_node_unref0 (_tmp31_);
					_vala_ccode_node_unref0 (_tmp30_);
				}
			}
		}
	}
	if (VALA_IS_CREATION_METHOD (m)) {
		if (vala_code_context_get_profile (vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self)) == VALA_PROFILE_GOBJECT) {
			if (!vala_class_get_is_compact (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) m)))) {
				ValaCCodeIdentifier* _tmp32_;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp32_ = vala_ccode_identifier_new ("object_type")));
				_vala_ccode_node_unref0 (_tmp32_);
			}
		} else {
			ValaCCodeIdentifier* _tmp33_;
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp33_);
		}
		{
			GeeList* _tmp34_;
			GeeIterator* _tmp35_;
			GeeIterator* _base_type_it;
			_base_type_it = (_tmp35_ = gee_iterable_iterator ((GeeIterable*) (_tmp34_ = vala_class_get_base_types (vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self)))), _gee_collection_object_unref0 (_tmp34_), _tmp35_);
			while (TRUE) {
				ValaDataType* base_type;
				if (!gee_iterator_next (_base_type_it)) {
					break;
				}
				base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
				if (VALA_IS_CLASS (vala_data_type_get_data_type (base_type))) {
					GeeList* _tmp36_;
					vala_ccode_base_module_add_generic_type_arguments ((ValaCCodeBaseModule*) self, ccall, _tmp36_ = vala_data_type_get_type_arguments (base_type), (ValaCodeNode*) expr, TRUE);
					_gee_collection_object_unref0 (_tmp36_);
					_vala_code_node_unref0 (base_type);
					break;
				}
				_vala_code_node_unref0 (base_type);
			}
			_gee_collection_object_unref0 (_base_type_it);
		}
	} else {
		gboolean _tmp37_;
		_tmp37_ = FALSE;
		if (m != NULL) {
			GeeList* _tmp38_;
			_tmp37_ = gee_collection_get_size ((GeeCollection*) (_tmp38_ = vala_method_get_type_parameters (m))) > 0;
			_gee_collection_object_unref0 (_tmp38_);
		} else {
			_tmp37_ = FALSE;
		}
		if (_tmp37_) {
			GeeList* _tmp39_;
			vala_ccode_base_module_add_generic_type_arguments ((ValaCCodeBaseModule*) self, ccall, _tmp39_ = vala_member_access_get_type_arguments (ma), (ValaCodeNode*) expr, FALSE);
			_gee_collection_object_unref0 (_tmp39_);
		}
	}
	ccall_expr = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccall);
	if (VALA_IS_ARRAY_RESIZE_METHOD (m)) {
		ValaArrayType* array_type;
		ValaCCodeIdentifier* _tmp41_;
		char* _tmp40_;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_expression_get_value_type (vala_member_access_get_inner (ma))));
		gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) 0, FALSE)), (ValaCCodeExpression*) (_tmp41_ = vala_ccode_identifier_new (_tmp40_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
		_vala_ccode_node_unref0 (_tmp41_);
		_g_free0 (_tmp40_);
		_vala_code_node_unref0 (array_type);
	} else {
		if (VALA_IS_ARRAY_MOVE_METHOD (m)) {
			((ValaCCodeBaseModule*) self)->requires_array_move = TRUE;
		}
	}
	instance = NULL;
	_tmp42_ = FALSE;
	_tmp43_ = FALSE;
	if (m != NULL) {
		_tmp43_ = vala_method_get_binding (m) == MEMBER_BINDING_INSTANCE;
	} else {
		_tmp43_ = FALSE;
	}
	if (_tmp43_) {
		_tmp42_ = !VALA_IS_CREATION_METHOD (m);
	} else {
		_tmp42_ = FALSE;
	}
	if (_tmp42_) {
		ValaCCodeExpression* _tmp44_;
		gboolean _tmp45_;
		ValaSymbol* _tmp47_;
		ValaStruct* st;
		gboolean _tmp48_;
		instance = (_tmp44_ = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma)))), _vala_ccode_node_unref0 (instance), _tmp44_);
		_tmp45_ = FALSE;
		if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") == 0) {
			_tmp45_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
		} else {
			_tmp45_ = FALSE;
		}
		if (_tmp45_) {
			ValaMemberAccess* inner_ma;
			ValaCCodeExpression* _tmp46_;
			inner_ma = _vala_code_node_ref0 (VALA_MEMBER_ACCESS (vala_member_access_get_inner (ma)));
			instance = (_tmp46_ = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (inner_ma)))), _vala_ccode_node_unref0 (instance), _tmp46_);
			_vala_code_node_unref0 (inner_ma);
		}
		st = _vala_code_node_ref0 ((_tmp47_ = vala_symbol_get_parent_symbol ((ValaSymbol*) m), VALA_IS_STRUCT (_tmp47_) ? ((ValaStruct*) _tmp47_) : NULL));
		_tmp48_ = FALSE;
		if (st != NULL) {
			_tmp48_ = !vala_struct_is_simple_type (st);
		} else {
			_tmp48_ = FALSE;
		}
		if (_tmp48_) {
			ValaCCodeExpression* _tmp49_;
			ValaCCodeUnaryExpression* unary;
			gboolean _tmp50_;
			unary = _vala_ccode_node_ref0 ((_tmp49_ = instance, VALA_IS_CCODE_UNARY_EXPRESSION (_tmp49_) ? ((ValaCCodeUnaryExpression*) _tmp49_) : NULL));
			_tmp50_ = FALSE;
			if (unary != NULL) {
				_tmp50_ = vala_ccode_unary_expression_get_operator (unary) == VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION;
			} else {
				_tmp50_ = FALSE;
			}
			if (_tmp50_) {
				ValaCCodeExpression* _tmp51_;
				instance = (_tmp51_ = _vala_ccode_node_ref0 (vala_ccode_unary_expression_get_inner (unary)), _vala_ccode_node_unref0 (instance), _tmp51_);
			} else {
				gboolean _tmp52_;
				_tmp52_ = FALSE;
				if (VALA_IS_CCODE_IDENTIFIER (instance)) {
					_tmp52_ = TRUE;
				} else {
					_tmp52_ = VALA_IS_CCODE_MEMBER_ACCESS (instance);
				}
				if (_tmp52_) {
					ValaCCodeExpression* _tmp53_;
					instance = (_tmp53_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, instance), _vala_ccode_node_unref0 (instance), _tmp53_);
				} else {
					ValaCCodeCommaExpression* ccomma;
					ValaLocalVariable* temp_var;
					ValaCCodeAssignment* _tmp55_;
					ValaCCodeExpression* _tmp54_;
					ValaCCodeUnaryExpression* _tmp57_;
					ValaCCodeExpression* _tmp56_;
					ValaCCodeExpression* _tmp58_;
					ccomma = vala_ccode_comma_expression_new ();
					temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_target_type (vala_member_access_get_inner (ma)), TRUE, NULL, FALSE);
					gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp55_ = vala_ccode_assignment_new (_tmp54_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)), instance, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp55_);
					_vala_ccode_node_unref0 (_tmp54_);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp57_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp56_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)))));
					_vala_ccode_node_unref0 (_tmp57_);
					_vala_ccode_node_unref0 (_tmp56_);
					instance = (_tmp58_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (instance), _tmp58_);
					_vala_ccode_node_unref0 (ccomma);
					_vala_code_node_unref0 (temp_var);
				}
			}
			_vala_ccode_node_unref0 (unary);
		}
		gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), instance);
		gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), instance);
		_vala_code_node_unref0 (st);
	} else {
		gboolean _tmp59_;
		_tmp59_ = FALSE;
		if (m != NULL) {
			_tmp59_ = vala_method_get_binding (m) == MEMBER_BINDING_CLASS;
		} else {
			_tmp59_ = FALSE;
		}
		if (_tmp59_) {
			ValaClass* cl;
			ValaCCodeIdentifier* _tmp62_;
			char* _tmp61_;
			char* _tmp60_;
			ValaCCodeFunctionCall* _tmp63_;
			ValaCCodeFunctionCall* cast;
			ValaCCodeExpression* klass;
			cl = _vala_code_node_ref0 (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) m)));
			cast = (_tmp63_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new (_tmp61_ = g_strconcat (_tmp60_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) cl, NULL), "_CLASS", NULL)))), _vala_ccode_node_unref0 (_tmp62_), _g_free0 (_tmp61_), _g_free0 (_tmp60_), _tmp63_);
			klass = NULL;
			if (vala_member_access_get_inner (ma) == NULL) {
				if (((ValaCCodeBaseModule*) self)->in_static_or_class_context) {
					ValaCCodeExpression* _tmp64_;
					klass = (_tmp64_ = (ValaCCodeExpression*) vala_ccode_identifier_new ("klass"), _vala_ccode_node_unref0 (klass), _tmp64_);
				} else {
					ValaCCodeIdentifier* _tmp65_;
					ValaCCodeFunctionCall* _tmp66_;
					ValaCCodeFunctionCall* k;
					ValaCCodeIdentifier* _tmp67_;
					ValaCCodeExpression* _tmp68_;
					k = (_tmp66_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp65_ = vala_ccode_identifier_new ("G_OBJECT_GET_CLASS"))), _vala_ccode_node_unref0 (_tmp65_), _tmp66_);
					vala_ccode_function_call_add_argument (k, (ValaCCodeExpression*) (_tmp67_ = vala_ccode_identifier_new ("self")));
					_vala_ccode_node_unref0 (_tmp67_);
					klass = (_tmp68_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) k), _vala_ccode_node_unref0 (klass), _tmp68_);
					_vala_ccode_node_unref0 (k);
				}
			} else {
				ValaCCodeIdentifier* _tmp69_;
				ValaCCodeFunctionCall* _tmp70_;
				ValaCCodeFunctionCall* k;
				ValaCCodeExpression* _tmp71_;
				k = (_tmp70_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp69_ = vala_ccode_identifier_new ("G_OBJECT_GET_CLASS"))), _vala_ccode_node_unref0 (_tmp69_), _tmp70_);
				vala_ccode_function_call_add_argument (k, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma))));
				klass = (_tmp71_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) k), _vala_ccode_node_unref0 (klass), _tmp71_);
				_vala_ccode_node_unref0 (k);
			}
			vala_ccode_function_call_add_argument (cast, klass);
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), (ValaCCodeExpression*) cast);
			gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), (ValaCCodeExpression*) cast);
			_vala_code_node_unref0 (cl);
			_vala_ccode_node_unref0 (cast);
			_vala_ccode_node_unref0 (klass);
		}
	}
	_tmp72_ = FALSE;
	if (m != NULL) {
		_tmp72_ = vala_method_get_has_generic_type_parameter (m);
	} else {
		_tmp72_ = FALSE;
	}
	if (_tmp72_) {
		gint type_param_index;
		type_param_index = 0;
		{
			GeeList* _tmp73_;
			GeeIterator* _tmp74_;
			GeeIterator* _type_arg_it;
			_type_arg_it = (_tmp74_ = gee_iterable_iterator ((GeeIterable*) (_tmp73_ = vala_data_type_get_type_arguments (vala_expression_get_value_type (vala_member_access_get_inner (ma))))), _gee_collection_object_unref0 (_tmp73_), _tmp74_);
			while (TRUE) {
				ValaDataType* type_arg;
				ValaCCodeIdentifier* _tmp76_;
				char* _tmp75_;
				if (!gee_iterator_next (_type_arg_it)) {
					break;
				}
				type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
				gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_generic_type_parameter_position (m) + (0.01 * type_param_index), FALSE)), (ValaCCodeExpression*) (_tmp76_ = vala_ccode_identifier_new (_tmp75_ = vala_data_type_get_cname (type_arg))));
				_vala_ccode_node_unref0 (_tmp76_);
				_g_free0 (_tmp75_);
				type_param_index++;
				_vala_code_node_unref0 (type_arg);
			}
			_gee_collection_object_unref0 (_type_arg_it);
		}
	}
	if (VALA_IS_ARRAY_MOVE_METHOD (m)) {
		ValaArrayType* array_type;
		ValaCCodeIdentifier* _tmp77_;
		ValaCCodeFunctionCall* _tmp78_;
		ValaCCodeFunctionCall* csizeof;
		ValaCCodeIdentifier* _tmp80_;
		char* _tmp79_;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_expression_get_value_type (vala_member_access_get_inner (ma))));
		csizeof = (_tmp78_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp77_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp77_), _tmp78_);
		vala_ccode_function_call_add_argument (csizeof, (ValaCCodeExpression*) (_tmp80_ = vala_ccode_identifier_new (_tmp79_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
		_vala_ccode_node_unref0 (_tmp80_);
		_g_free0 (_tmp79_);
		gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, 0.1, FALSE)), (ValaCCodeExpression*) csizeof);
		_vala_code_node_unref0 (array_type);
		_vala_ccode_node_unref0 (csizeof);
	} else {
		if (VALA_IS_DYNAMIC_METHOD (m)) {
			gint param_nr;
			vala_method_clear_parameters (m);
			param_nr = 1;
			{
				GeeList* _tmp81_;
				GeeIterator* _tmp82_;
				GeeIterator* _arg_it;
				_arg_it = (_tmp82_ = gee_iterable_iterator ((GeeIterable*) (_tmp81_ = vala_method_call_get_argument_list (expr))), _gee_collection_object_unref0 (_tmp81_), _tmp82_);
				while (TRUE) {
					ValaExpression* arg;
					ValaExpression* _tmp83_;
					ValaUnaryExpression* unary;
					gboolean _tmp84_;
					if (!gee_iterator_next (_arg_it)) {
						break;
					}
					arg = (ValaExpression*) gee_iterator_get (_arg_it);
					unary = _vala_code_node_ref0 ((_tmp83_ = arg, VALA_IS_UNARY_EXPRESSION (_tmp83_) ? ((ValaUnaryExpression*) _tmp83_) : NULL));
					_tmp84_ = FALSE;
					if (unary != NULL) {
						_tmp84_ = vala_unary_expression_get_operator (unary) == VALA_UNARY_OPERATOR_OUT;
					} else {
						_tmp84_ = FALSE;
					}
					if (_tmp84_) {
						char* _tmp85_;
						ValaFormalParameter* _tmp86_;
						ValaFormalParameter* param;
						param = (_tmp86_ = vala_formal_parameter_new (_tmp85_ = g_strdup_printf ("param%d", param_nr), vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), NULL), _g_free0 (_tmp85_), _tmp86_);
						vala_formal_parameter_set_direction (param, VALA_PARAMETER_DIRECTION_OUT);
						vala_method_add_parameter (m, param);
						_vala_code_node_unref0 (param);
					} else {
						gboolean _tmp87_;
						_tmp87_ = FALSE;
						if (unary != NULL) {
							_tmp87_ = vala_unary_expression_get_operator (unary) == VALA_UNARY_OPERATOR_REF;
						} else {
							_tmp87_ = FALSE;
						}
						if (_tmp87_) {
							char* _tmp88_;
							ValaFormalParameter* _tmp89_;
							ValaFormalParameter* param;
							param = (_tmp89_ = vala_formal_parameter_new (_tmp88_ = g_strdup_printf ("param%d", param_nr), vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), NULL), _g_free0 (_tmp88_), _tmp89_);
							vala_formal_parameter_set_direction (param, VALA_PARAMETER_DIRECTION_REF);
							vala_method_add_parameter (m, param);
							_vala_code_node_unref0 (param);
						} else {
							ValaFormalParameter* _tmp91_;
							char* _tmp90_;
							vala_method_add_parameter (m, _tmp91_ = vala_formal_parameter_new (_tmp90_ = g_strdup_printf ("param%d", param_nr), vala_expression_get_value_type (arg), NULL));
							_vala_code_node_unref0 (_tmp91_);
							_g_free0 (_tmp90_);
						}
					}
					param_nr++;
					_vala_code_node_unref0 (arg);
					_vala_code_node_unref0 (unary);
				}
				_gee_collection_object_unref0 (_arg_it);
			}
			{
				GeeList* _tmp92_;
				GeeIterator* _tmp93_;
				GeeIterator* _param_it;
				_param_it = (_tmp93_ = gee_iterable_iterator ((GeeIterable*) (_tmp92_ = vala_method_get_parameters (m))), _gee_collection_object_unref0 (_tmp92_), _tmp93_);
				while (TRUE) {
					ValaFormalParameter* param;
					if (!gee_iterator_next (_param_it)) {
						break;
					}
					param = (ValaFormalParameter*) gee_iterator_get (_param_it);
					vala_code_node_accept ((ValaCodeNode*) param, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
					_vala_code_node_unref0 (param);
				}
				_gee_collection_object_unref0 (_param_it);
			}
			vala_ccode_module_generate_dynamic_method_wrapper (vala_ccode_module_get_head ((ValaCCodeModule*) self), VALA_DYNAMIC_METHOD (m));
		} else {
			gboolean _tmp94_;
			_tmp94_ = FALSE;
			if (VALA_IS_CREATION_METHOD (m)) {
				_tmp94_ = vala_code_context_get_profile (vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self)) == VALA_PROFILE_GOBJECT;
			} else {
				_tmp94_ = FALSE;
			}
			if (_tmp94_) {
				ValaCCodeExpression* _tmp99_;
				ValaCCodeCastExpression* _tmp98_;
				char* _tmp97_;
				char* _tmp96_;
				ValaCCodeIdentifier* _tmp95_;
				gboolean _tmp100_;
				ccall_expr = (_tmp99_ = (ValaCCodeExpression*) vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp95_ = vala_ccode_identifier_new ("self")), (ValaCCodeExpression*) (_tmp98_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) ccall, _tmp97_ = g_strconcat (_tmp96_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self), FALSE), "*", NULL))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (ccall_expr), _tmp99_);
				_vala_ccode_node_unref0 (_tmp98_);
				_g_free0 (_tmp97_);
				_g_free0 (_tmp96_);
				_vala_ccode_node_unref0 (_tmp95_);
				_tmp100_ = FALSE;
				if (!vala_class_get_is_compact (vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self))) {
					GeeList* _tmp101_;
					_tmp100_ = gee_collection_get_size ((GeeCollection*) (_tmp101_ = vala_object_type_symbol_get_type_parameters ((ValaObjectTypeSymbol*) vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self)))) > 0;
					_gee_collection_object_unref0 (_tmp101_);
				} else {
					_tmp100_ = FALSE;
				}
				if (_tmp100_) {
					ValaCCodeCommaExpression* ccomma;
					ValaCCodeExpression* _tmp121_;
					ccomma = vala_ccode_comma_expression_new ();
					vala_ccode_comma_expression_append_expression (ccomma, ccall_expr);
					{
						GeeList* _tmp102_;
						GeeIterator* _tmp103_;
						GeeIterator* _type_param_it;
						_type_param_it = (_tmp103_ = gee_iterable_iterator ((GeeIterable*) (_tmp102_ = vala_object_type_symbol_get_type_parameters ((ValaObjectTypeSymbol*) vala_ccode_base_module_get_current_class ((ValaCCodeBaseModule*) self)))), _gee_collection_object_unref0 (_tmp102_), _tmp103_);
						while (TRUE) {
							ValaTypeParameter* type_param;
							ValaCCodeIdentifier* param_name;
							ValaCCodeIdentifier* _tmp104_;
							ValaCCodeMemberAccess* _tmp105_;
							ValaCCodeMemberAccess* priv_access;
							ValaCCodeIdentifier* _tmp108_;
							char* _tmp107_;
							char* _tmp106_;
							ValaCCodeAssignment* _tmp110_;
							ValaCCodeMemberAccess* _tmp109_;
							ValaCCodeIdentifier* _tmp113_;
							char* _tmp112_;
							char* _tmp111_;
							ValaCCodeAssignment* _tmp115_;
							ValaCCodeMemberAccess* _tmp114_;
							ValaCCodeIdentifier* _tmp118_;
							char* _tmp117_;
							char* _tmp116_;
							ValaCCodeAssignment* _tmp120_;
							ValaCCodeMemberAccess* _tmp119_;
							if (!gee_iterator_next (_type_param_it)) {
								break;
							}
							type_param = (ValaTypeParameter*) gee_iterator_get (_type_param_it);
							param_name = NULL;
							priv_access = (_tmp105_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp104_ = vala_ccode_identifier_new ("self")), "priv"), _vala_ccode_node_unref0 (_tmp104_), _tmp105_);
							param_name = (_tmp108_ = vala_ccode_identifier_new (_tmp107_ = g_strdup_printf ("%s_type", _tmp106_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) type_param), -1))), _vala_ccode_node_unref0 (param_name), _tmp108_);
							_g_free0 (_tmp107_);
							_g_free0 (_tmp106_);
							vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp110_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp109_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) priv_access, vala_ccode_identifier_get_name (param_name))), (ValaCCodeExpression*) param_name, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							_vala_ccode_node_unref0 (_tmp110_);
							_vala_ccode_node_unref0 (_tmp109_);
							param_name = (_tmp113_ = vala_ccode_identifier_new (_tmp112_ = g_strdup_printf ("%s_dup_func", _tmp111_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) type_param), -1))), _vala_ccode_node_unref0 (param_name), _tmp113_);
							_g_free0 (_tmp112_);
							_g_free0 (_tmp111_);
							vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp115_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp114_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) priv_access, vala_ccode_identifier_get_name (param_name))), (ValaCCodeExpression*) param_name, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							_vala_ccode_node_unref0 (_tmp115_);
							_vala_ccode_node_unref0 (_tmp114_);
							param_name = (_tmp118_ = vala_ccode_identifier_new (_tmp117_ = g_strdup_printf ("%s_destroy_func", _tmp116_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) type_param), -1))), _vala_ccode_node_unref0 (param_name), _tmp118_);
							_g_free0 (_tmp117_);
							_g_free0 (_tmp116_);
							vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp120_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp119_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) priv_access, vala_ccode_identifier_get_name (param_name))), (ValaCCodeExpression*) param_name, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							_vala_ccode_node_unref0 (_tmp120_);
							_vala_ccode_node_unref0 (_tmp119_);
							_vala_code_node_unref0 (type_param);
							_vala_ccode_node_unref0 (param_name);
							_vala_ccode_node_unref0 (priv_access);
						}
						_gee_collection_object_unref0 (_type_param_it);
					}
					ccall_expr = (_tmp121_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (ccall_expr), _tmp121_);
					_vala_ccode_node_unref0 (ccomma);
				}
			}
		}
	}
	ellipsis = FALSE;
	i = 1;
	arg_pos = 0;
	params_it = gee_iterable_iterator ((GeeIterable*) params);
	{
		GeeList* _tmp122_;
		GeeIterator* _tmp123_;
		GeeIterator* _arg_it;
		_arg_it = (_tmp123_ = gee_iterable_iterator ((GeeIterable*) (_tmp122_ = vala_method_call_get_argument_list (expr))), _gee_collection_object_unref0 (_tmp122_), _tmp123_);
		while (TRUE) {
			ValaExpression* arg;
			ValaCCodeExpression* cexpr;
			GeeHashMap* carg_map;
			if (!gee_iterator_next (_arg_it)) {
				break;
			}
			arg = (ValaExpression*) gee_iterator_get (_arg_it);
			cexpr = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) arg)));
			carg_map = _gee_collection_object_ref0 (in_arg_map);
			if (gee_iterator_next (params_it)) {
				ValaFormalParameter* param;
				gboolean _tmp124_;
				param = (ValaFormalParameter*) gee_iterator_get (params_it);
				_tmp124_ = FALSE;
				if (vala_formal_parameter_get_params_array (param)) {
					_tmp124_ = TRUE;
				} else {
					_tmp124_ = vala_formal_parameter_get_ellipsis (param);
				}
				ellipsis = _tmp124_;
				if (!ellipsis) {
					gboolean multiple_cargs;
					gboolean _tmp126_;
					ValaCCodeExpression* _tmp162_;
					gboolean _tmp163_;
					gboolean _tmp168_;
					gboolean _tmp169_;
					gboolean _tmp170_;
					multiple_cargs = FALSE;
					if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_OUT) {
						GeeHashMap* _tmp125_;
						carg_map = (_tmp125_ = _gee_collection_object_ref0 (out_arg_map), _gee_collection_object_unref0 (carg_map), _tmp125_);
					}
					_tmp126_ = FALSE;
					if (!vala_formal_parameter_get_no_array_length (param)) {
						_tmp126_ = VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param));
					} else {
						_tmp126_ = FALSE;
					}
					if (_tmp126_) {
						ValaArrayType* array_type;
						array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)));
						{
							gint dim;
							dim = 1;
							{
								gboolean _tmp127_;
								_tmp127_ = TRUE;
								while (TRUE) {
									ValaCCodeExpression* _tmp128_;
									if (!_tmp127_) {
										dim++;
									}
									_tmp127_ = FALSE;
									if (!(dim <= vala_array_type_get_rank (array_type))) {
										break;
									}
									gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_carray_length_parameter_position (param) + (0.01 * dim), FALSE)), _tmp128_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), arg, dim));
									_vala_ccode_node_unref0 (_tmp128_);
								}
							}
						}
						multiple_cargs = TRUE;
						_vala_code_node_unref0 (array_type);
					} else {
						if (VALA_IS_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param))) {
							ValaDelegateType* deleg_type;
							ValaDelegate* d;
							deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param)));
							d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
							if (vala_delegate_get_has_target (d)) {
								ValaCCodeExpression* delegate_target;
								delegate_target = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, arg);
								if (vala_data_type_get_value_owned ((ValaDataType*) deleg_type)) {
									ValaCCodeExpression* delegate_target_destroy_notify;
									ValaSymbol* _tmp129_;
									ValaMethod* delegate_method;
									ValaExpression* _tmp130_;
									ValaLambdaExpression* lambda;
									ValaExpression* _tmp131_;
									ValaMemberAccess* arg_ma;
									delegate_target_destroy_notify = NULL;
									delegate_method = _vala_code_node_ref0 ((_tmp129_ = vala_expression_get_symbol_reference (arg), VALA_IS_METHOD (_tmp129_) ? ((ValaMethod*) _tmp129_) : NULL));
									lambda = _vala_code_node_ref0 ((_tmp130_ = arg, VALA_IS_LAMBDA_EXPRESSION (_tmp130_) ? ((ValaLambdaExpression*) _tmp130_) : NULL));
									arg_ma = _vala_code_node_ref0 ((_tmp131_ = arg, VALA_IS_MEMBER_ACCESS (_tmp131_) ? ((ValaMemberAccess*) _tmp131_) : NULL));
									if (lambda != NULL) {
										if (vala_method_get_closure (delegate_method)) {
											ValaSymbol* _tmp132_;
											ValaBlock* closure_block;
											gint block_id;
											ValaCCodeIdentifier* _tmp137_;
											char* _tmp136_;
											ValaCCodeFunctionCall* _tmp138_;
											ValaCCodeFunctionCall* ref_call;
											ValaCCodeExpression* _tmp139_;
											ValaCCodeExpression* _tmp141_;
											char* _tmp140_;
											closure_block = _vala_code_node_ref0 ((_tmp132_ = ((ValaCCodeBaseModule*) self)->current_symbol, VALA_IS_BLOCK (_tmp132_) ? ((ValaBlock*) _tmp132_) : NULL));
											while (TRUE) {
												gboolean _tmp133_;
												ValaBlock* _tmp135_;
												ValaSymbol* _tmp134_;
												_tmp133_ = FALSE;
												if (closure_block != NULL) {
													_tmp133_ = !vala_block_get_captured (closure_block);
												} else {
													_tmp133_ = FALSE;
												}
												if (!_tmp133_) {
													break;
												}
												closure_block = (_tmp135_ = _vala_code_node_ref0 ((_tmp134_ = vala_symbol_get_parent_symbol ((ValaSymbol*) closure_block), VALA_IS_BLOCK (_tmp134_) ? ((ValaBlock*) _tmp134_) : NULL)), _vala_code_node_unref0 (closure_block), _tmp135_);
											}
											block_id = vala_ccode_base_module_get_block_id ((ValaCCodeBaseModule*) self, closure_block);
											ref_call = (_tmp138_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp137_ = vala_ccode_identifier_new (_tmp136_ = g_strdup_printf ("block%d_data_ref", block_id)))), _vala_ccode_node_unref0 (_tmp137_), _g_free0 (_tmp136_), _tmp138_);
											vala_ccode_function_call_add_argument (ref_call, delegate_target);
											delegate_target = (_tmp139_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ref_call), _vala_ccode_node_unref0 (delegate_target), _tmp139_);
											delegate_target_destroy_notify = (_tmp141_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp140_ = g_strdup_printf ("block%d_data_unref", block_id)), _vala_ccode_node_unref0 (delegate_target_destroy_notify), _tmp141_);
											_g_free0 (_tmp140_);
											_vala_code_node_unref0 (closure_block);
											_vala_ccode_node_unref0 (ref_call);
										} else {
											ValaDataType* _tmp142_;
											gboolean _tmp143_;
											if ((_tmp143_ = (_tmp142_ = vala_ccode_base_module_get_this_type ((ValaCCodeBaseModule*) self)) != NULL, _vala_code_node_unref0 (_tmp142_), _tmp143_)) {
												ValaCCodeExpression* _tmp145_;
												ValaDataType* _tmp144_;
												ValaCCodeFunctionCall* _tmp146_;
												ValaCCodeFunctionCall* ref_call;
												ValaCCodeExpression* _tmp147_;
												ValaCCodeExpression* _tmp149_;
												ValaDataType* _tmp148_;
												ref_call = (_tmp146_ = vala_ccode_function_call_new (_tmp145_ = vala_ccode_base_module_get_dup_func_expression ((ValaCCodeBaseModule*) self, _tmp144_ = vala_ccode_base_module_get_this_type ((ValaCCodeBaseModule*) self), vala_code_node_get_source_reference ((ValaCodeNode*) arg), FALSE)), _vala_ccode_node_unref0 (_tmp145_), _vala_code_node_unref0 (_tmp144_), _tmp146_);
												vala_ccode_function_call_add_argument (ref_call, delegate_target);
												delegate_target = (_tmp147_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ref_call), _vala_ccode_node_unref0 (delegate_target), _tmp147_);
												delegate_target_destroy_notify = (_tmp149_ = vala_ccode_base_module_get_destroy_func_expression ((ValaCCodeBaseModule*) self, _tmp148_ = vala_ccode_base_module_get_this_type ((ValaCCodeBaseModule*) self), FALSE), _vala_ccode_node_unref0 (delegate_target_destroy_notify), _tmp149_);
												_vala_code_node_unref0 (_tmp148_);
												_vala_ccode_node_unref0 (ref_call);
											} else {
												ValaCCodeExpression* _tmp150_;
												delegate_target_destroy_notify = (_tmp150_ = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL"), _vala_ccode_node_unref0 (delegate_target_destroy_notify), _tmp150_);
											}
										}
									} else {
										gboolean _tmp151_;
										gboolean _tmp152_;
										gboolean _tmp153_;
										gboolean _tmp154_;
										gboolean _tmp155_;
										_tmp151_ = FALSE;
										_tmp152_ = FALSE;
										_tmp153_ = FALSE;
										_tmp154_ = FALSE;
										_tmp155_ = FALSE;
										if (delegate_method != NULL) {
											_tmp155_ = vala_method_get_binding (delegate_method) == MEMBER_BINDING_INSTANCE;
										} else {
											_tmp155_ = FALSE;
										}
										if (_tmp155_) {
											_tmp154_ = arg_ma != NULL;
										} else {
											_tmp154_ = FALSE;
										}
										if (_tmp154_) {
											_tmp153_ = vala_member_access_get_inner (arg_ma) != NULL;
										} else {
											_tmp153_ = FALSE;
										}
										if (_tmp153_) {
											_tmp152_ = vala_data_type_get_data_type (vala_expression_get_value_type (vala_member_access_get_inner (arg_ma))) != NULL;
										} else {
											_tmp152_ = FALSE;
										}
										if (_tmp152_) {
											_tmp151_ = vala_typesymbol_is_reference_counting (vala_data_type_get_data_type (vala_expression_get_value_type (vala_member_access_get_inner (arg_ma))));
										} else {
											_tmp151_ = FALSE;
										}
										if (_tmp151_) {
											ValaCCodeExpression* _tmp156_;
											ValaCCodeFunctionCall* _tmp157_;
											ValaCCodeFunctionCall* ref_call;
											ValaCCodeExpression* _tmp158_;
											ValaCCodeExpression* _tmp159_;
											ref_call = (_tmp157_ = vala_ccode_function_call_new (_tmp156_ = vala_ccode_base_module_get_dup_func_expression ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_member_access_get_inner (arg_ma)), vala_code_node_get_source_reference ((ValaCodeNode*) arg), FALSE)), _vala_ccode_node_unref0 (_tmp156_), _tmp157_);
											vala_ccode_function_call_add_argument (ref_call, delegate_target);
											delegate_target = (_tmp158_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ref_call), _vala_ccode_node_unref0 (delegate_target), _tmp158_);
											delegate_target_destroy_notify = (_tmp159_ = vala_ccode_base_module_get_destroy_func_expression ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_member_access_get_inner (arg_ma)), FALSE), _vala_ccode_node_unref0 (delegate_target_destroy_notify), _tmp159_);
											_vala_ccode_node_unref0 (ref_call);
										} else {
											ValaCCodeExpression* _tmp160_;
											delegate_target_destroy_notify = (_tmp160_ = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL"), _vala_ccode_node_unref0 (delegate_target_destroy_notify), _tmp160_);
										}
									}
									gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param) + 0.01, FALSE)), delegate_target_destroy_notify);
									_vala_ccode_node_unref0 (delegate_target_destroy_notify);
									_vala_code_node_unref0 (delegate_method);
									_vala_code_node_unref0 (lambda);
									_vala_code_node_unref0 (arg_ma);
								}
								gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), delegate_target);
								multiple_cargs = TRUE;
								_vala_ccode_node_unref0 (delegate_target);
							}
							_vala_code_node_unref0 (deleg_type);
							_vala_code_node_unref0 (d);
						} else {
							if (VALA_IS_METHOD_TYPE (vala_formal_parameter_get_parameter_type (param))) {
								ValaCCodeExpression* _tmp161_;
								gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), _tmp161_ = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, arg));
								_vala_ccode_node_unref0 (_tmp161_);
								multiple_cargs = TRUE;
							}
						}
					}
					cexpr = (_tmp162_ = vala_ccode_base_module_handle_struct_argument ((ValaCCodeBaseModule*) self, param, arg, cexpr), _vala_ccode_node_unref0 (cexpr), _tmp162_);
					_tmp163_ = FALSE;
					if (multiple_cargs) {
						_tmp163_ = VALA_IS_METHOD_CALL (arg);
					} else {
						_tmp163_ = FALSE;
					}
					if (_tmp163_) {
						ValaCCodeCommaExpression* ccomma;
						ValaLocalVariable* temp_decl;
						ValaCCodeAssignment* _tmp165_;
						ValaCCodeExpression* _tmp164_;
						ValaCCodeExpression* _tmp166_;
						ValaCCodeExpression* _tmp167_;
						ccomma = vala_ccode_comma_expression_new ();
						temp_decl = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (arg), TRUE, NULL, FALSE);
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_decl);
						vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp165_ = vala_ccode_assignment_new (_tmp164_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_decl)), cexpr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
						_vala_ccode_node_unref0 (_tmp165_);
						_vala_ccode_node_unref0 (_tmp164_);
						cexpr = (_tmp166_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_decl)), _vala_ccode_node_unref0 (cexpr), _tmp166_);
						vala_ccode_comma_expression_append_expression (ccomma, ccall_expr);
						ccall_expr = (_tmp167_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (ccall_expr), _tmp167_);
						_vala_ccode_node_unref0 (ccomma);
						_vala_code_node_unref0 (temp_decl);
					}
					_tmp168_ = FALSE;
					_tmp169_ = FALSE;
					_tmp170_ = FALSE;
					if (vala_formal_parameter_get_direction (param) != VALA_PARAMETER_DIRECTION_IN) {
						_tmp170_ = vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (arg));
					} else {
						_tmp170_ = FALSE;
					}
					if (_tmp170_) {
						gboolean _tmp171_;
						_tmp171_ = FALSE;
						if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_OUT) {
							_tmp171_ = TRUE;
						} else {
							_tmp171_ = !vala_data_type_get_value_owned (vala_formal_parameter_get_parameter_type (param));
						}
						_tmp169_ = _tmp171_;
					} else {
						_tmp169_ = FALSE;
					}
					if (_tmp169_) {
						_tmp168_ = !VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param));
					} else {
						_tmp168_ = FALSE;
					}
					if (_tmp168_) {
						ValaUnaryExpression* unary;
						ValaCCodeCommaExpression* ccomma;
						ValaLocalVariable* temp_var;
						ValaCCodeExpression* _tmp173_;
						ValaCCodeExpression* _tmp172_;
						ValaLocalVariable* ret_temp_var;
						ValaDataType* _tmp177_;
						gboolean _tmp178_;
						ValaCCodeCommaExpression* cassign_comma;
						ValaLocalVariable* assign_temp_var;
						ValaCCodeAssignment* _tmp186_;
						ValaCCodeExpression* _tmp185_;
						ValaCCodeExpression* _tmp184_;
						ValaCCodeExpression* _tmp183_;
						ValaCCodeExpression* _tmp187_;
						ValaCCodeExpression* _tmp188_;
						ValaCCodeAssignment* _tmp189_;
						ValaDataType* _tmp190_;
						gboolean _tmp191_;
						ValaCCodeExpression* _tmp193_;
						unary = _vala_code_node_ref0 (VALA_UNARY_EXPRESSION (arg));
						ccomma = vala_ccode_comma_expression_new ();
						temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_parameter_type (param), vala_data_type_get_value_owned (vala_formal_parameter_get_parameter_type (param)), NULL, TRUE);
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
						cexpr = (_tmp173_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp172_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var))), _vala_ccode_node_unref0 (cexpr), _tmp173_);
						_vala_ccode_node_unref0 (_tmp172_);
						if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_REF) {
							ValaCCodeCommaExpression* crefcomma;
							ValaCCodeAssignment* _tmp175_;
							ValaCCodeExpression* _tmp174_;
							ValaCCodeExpression* _tmp176_;
							crefcomma = vala_ccode_comma_expression_new ();
							vala_ccode_comma_expression_append_expression (crefcomma, (ValaCCodeExpression*) (_tmp175_ = vala_ccode_assignment_new (_tmp174_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_unary_expression_get_inner (unary))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							_vala_ccode_node_unref0 (_tmp175_);
							_vala_ccode_node_unref0 (_tmp174_);
							vala_ccode_comma_expression_append_expression (crefcomma, cexpr);
							cexpr = (_tmp176_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) crefcomma), _vala_ccode_node_unref0 (cexpr), _tmp176_);
							_vala_ccode_node_unref0 (crefcomma);
						}
						ret_temp_var = NULL;
						if ((_tmp178_ = VALA_IS_VOID_TYPE (_tmp177_ = vala_data_type_get_return_type (itype)), _vala_code_node_unref0 (_tmp177_), _tmp178_)) {
							vala_ccode_comma_expression_append_expression (ccomma, ccall_expr);
						} else {
							ValaLocalVariable* _tmp180_;
							ValaDataType* _tmp179_;
							ValaCCodeAssignment* _tmp182_;
							ValaCCodeExpression* _tmp181_;
							ret_temp_var = (_tmp180_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp179_ = vala_data_type_get_return_type (itype), TRUE, NULL, FALSE), _vala_code_node_unref0 (ret_temp_var), _tmp180_);
							_vala_code_node_unref0 (_tmp179_);
							gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, ret_temp_var);
							vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp182_ = vala_ccode_assignment_new (_tmp181_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) ret_temp_var)), ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							_vala_ccode_node_unref0 (_tmp182_);
							_vala_ccode_node_unref0 (_tmp181_);
						}
						cassign_comma = vala_ccode_comma_expression_new ();
						assign_temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), vala_data_type_get_value_owned (vala_expression_get_value_type (vala_unary_expression_get_inner (unary))), NULL, FALSE);
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, assign_temp_var);
						vala_ccode_comma_expression_append_expression (cassign_comma, (ValaCCodeExpression*) (_tmp186_ = vala_ccode_assignment_new (_tmp183_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) assign_temp_var)), _tmp185_ = vala_ccode_base_module_transform_expression ((ValaCCodeBaseModule*) self, _tmp184_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)), vala_formal_parameter_get_parameter_type (param), vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), arg), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
						_vala_ccode_node_unref0 (_tmp186_);
						_vala_ccode_node_unref0 (_tmp185_);
						_vala_ccode_node_unref0 (_tmp184_);
						_vala_ccode_node_unref0 (_tmp183_);
						vala_ccode_comma_expression_append_expression (cassign_comma, _tmp187_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_unary_expression_get_inner (unary))), vala_expression_get_value_type (arg), arg, FALSE));
						_vala_ccode_node_unref0 (_tmp187_);
						vala_ccode_comma_expression_append_expression (cassign_comma, _tmp188_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) assign_temp_var)));
						_vala_ccode_node_unref0 (_tmp188_);
						vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp189_ = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_unary_expression_get_inner (unary))), (ValaCCodeExpression*) cassign_comma, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
						_vala_ccode_node_unref0 (_tmp189_);
						if ((_tmp191_ = !VALA_IS_VOID_TYPE (_tmp190_ = vala_data_type_get_return_type (itype)), _vala_code_node_unref0 (_tmp190_), _tmp191_)) {
							ValaCCodeExpression* _tmp192_;
							vala_ccode_comma_expression_append_expression (ccomma, _tmp192_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) ret_temp_var)));
							_vala_ccode_node_unref0 (_tmp192_);
						}
						ccall_expr = (_tmp193_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (ccall_expr), _tmp193_);
						_vala_code_node_unref0 (unary);
						_vala_ccode_node_unref0 (ccomma);
						_vala_code_node_unref0 (temp_var);
						_vala_code_node_unref0 (ret_temp_var);
						_vala_ccode_node_unref0 (cassign_comma);
						_vala_code_node_unref0 (assign_temp_var);
					}
					if (vala_formal_parameter_get_ctype (param) != NULL) {
						ValaCCodeExpression* _tmp194_;
						cexpr = (_tmp194_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (cexpr, vala_formal_parameter_get_ctype (param)), _vala_ccode_node_unref0 (cexpr), _tmp194_);
					}
				}
				arg_pos = vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cparameter_position (param), ellipsis);
				_vala_code_node_unref0 (param);
			} else {
				arg_pos = vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) i, ellipsis);
			}
			gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (arg_pos), cexpr);
			i++;
			_vala_code_node_unref0 (arg);
			_vala_ccode_node_unref0 (cexpr);
			_gee_collection_object_unref0 (carg_map);
		}
		_gee_collection_object_unref0 (_arg_it);
	}
	if (gee_iterator_next (params_it)) {
		ValaFormalParameter* param;
		gboolean _tmp195_;
		param = (ValaFormalParameter*) gee_iterator_get (params_it);
		_tmp195_ = FALSE;
		if (vala_formal_parameter_get_params_array (param)) {
			_tmp195_ = TRUE;
		} else {
			_tmp195_ = vala_formal_parameter_get_ellipsis (param);
		}
		g_assert (_tmp195_);
		ellipsis = TRUE;
		_vala_code_node_unref0 (param);
	}
	_tmp196_ = FALSE;
	_tmp197_ = FALSE;
	if (m != NULL) {
		_tmp197_ = VALA_IS_ARRAY_TYPE (vala_method_get_return_type (m));
	} else {
		_tmp197_ = FALSE;
	}
	if (_tmp197_) {
		_tmp196_ = async_call != ccall;
	} else {
		_tmp196_ = FALSE;
	}
	if (_tmp196_) {
		ValaArrayType* array_type;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_method_get_return_type (m)));
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp198_;
				_tmp198_ = TRUE;
				while (TRUE) {
					if (!_tmp198_) {
						dim++;
					}
					_tmp198_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					if (vala_method_get_array_null_terminated (m)) {
						ValaDataType* _tmp199_;
						ValaLocalVariable* _tmp200_;
						ValaLocalVariable* temp_var;
						ValaCCodeExpression* temp_ref;
						ValaCCodeExpression* _tmp201_;
						ValaCCodeIdentifier* _tmp202_;
						ValaCCodeFunctionCall* _tmp203_;
						ValaCCodeFunctionCall* len_call;
						temp_var = (_tmp200_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp199_ = vala_data_type_get_return_type (itype), TRUE, NULL, FALSE), _vala_code_node_unref0 (_tmp199_), _tmp200_);
						temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
						ccall_expr = (_tmp201_ = (ValaCCodeExpression*) vala_ccode_assignment_new (temp_ref, ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (ccall_expr), _tmp201_);
						((ValaCCodeBaseModule*) self)->requires_array_length = TRUE;
						len_call = (_tmp203_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp202_ = vala_ccode_identifier_new ("_vala_array_length"))), _vala_ccode_node_unref0 (_tmp202_), _tmp203_);
						vala_ccode_function_call_add_argument (len_call, temp_ref);
						vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) len_call);
						_vala_code_node_unref0 (temp_var);
						_vala_ccode_node_unref0 (temp_ref);
						_vala_ccode_node_unref0 (len_call);
					} else {
						if (!vala_method_get_no_array_length (m)) {
							ValaLocalVariable* temp_var;
							ValaCCodeExpression* temp_ref;
							ValaCCodeUnaryExpression* _tmp207_;
							temp_var = NULL;
							if (vala_method_get_array_length_type (m) == NULL) {
								ValaLocalVariable* _tmp204_;
								temp_var = (_tmp204_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL, TRUE), _vala_code_node_unref0 (temp_var), _tmp204_);
							} else {
								ValaLocalVariable* _tmp206_;
								ValaCType* _tmp205_;
								temp_var = (_tmp206_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, (ValaDataType*) (_tmp205_ = vala_ctype_new (vala_method_get_array_length_type (m))), TRUE, NULL, TRUE), _vala_code_node_unref0 (temp_var), _tmp206_);
								_vala_code_node_unref0 (_tmp205_);
							}
							temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
							gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
							gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_carray_length_parameter_position (m) + (0.01 * dim), FALSE)), (ValaCCodeExpression*) (_tmp207_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
							_vala_ccode_node_unref0 (_tmp207_);
							vala_expression_append_array_size ((ValaExpression*) expr, temp_ref);
							_vala_code_node_unref0 (temp_var);
							_vala_ccode_node_unref0 (temp_ref);
						} else {
							ValaCCodeConstant* _tmp208_;
							vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) (_tmp208_ = vala_ccode_constant_new ("-1")));
							_vala_ccode_node_unref0 (_tmp208_);
						}
					}
				}
			}
		}
		_vala_code_node_unref0 (array_type);
	} else {
		gboolean _tmp209_;
		gboolean _tmp210_;
		_tmp209_ = FALSE;
		_tmp210_ = FALSE;
		if (m != NULL) {
			_tmp210_ = VALA_IS_DELEGATE_TYPE (vala_method_get_return_type (m));
		} else {
			_tmp210_ = FALSE;
		}
		if (_tmp210_) {
			_tmp209_ = async_call != ccall;
		} else {
			_tmp209_ = FALSE;
		}
		if (_tmp209_) {
			ValaDelegateType* deleg_type;
			ValaDelegate* d;
			deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_method_get_return_type (m)));
			d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
			if (vala_delegate_get_has_target (d)) {
				ValaPointerType* _tmp212_;
				ValaVoidType* _tmp211_;
				ValaLocalVariable* _tmp213_;
				ValaLocalVariable* temp_var;
				ValaCCodeExpression* temp_ref;
				ValaCCodeUnaryExpression* _tmp214_;
				temp_var = (_tmp213_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, (ValaDataType*) (_tmp212_ = vala_pointer_type_new ((ValaDataType*) (_tmp211_ = vala_void_type_new (NULL)), NULL)), TRUE, NULL, TRUE), _vala_code_node_unref0 (_tmp212_), _vala_code_node_unref0 (_tmp211_), _tmp213_);
				temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
				gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
				gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cdelegate_target_parameter_position (m), FALSE)), (ValaCCodeExpression*) (_tmp214_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
				_vala_ccode_node_unref0 (_tmp214_);
				vala_expression_set_delegate_target ((ValaExpression*) expr, temp_ref);
				_vala_code_node_unref0 (temp_var);
				_vala_ccode_node_unref0 (temp_ref);
			}
			_vala_code_node_unref0 (deleg_type);
			_vala_code_node_unref0 (d);
		}
	}
	_tmp215_ = FALSE;
	if (deleg != NULL) {
		_tmp215_ = VALA_IS_ARRAY_TYPE (vala_delegate_get_return_type (deleg));
	} else {
		_tmp215_ = FALSE;
	}
	if (_tmp215_) {
		ValaArrayType* array_type;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_delegate_get_return_type (deleg)));
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp216_;
				_tmp216_ = TRUE;
				while (TRUE) {
					if (!_tmp216_) {
						dim++;
					}
					_tmp216_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					if (vala_delegate_get_array_null_terminated (deleg)) {
						ValaDataType* _tmp217_;
						ValaLocalVariable* _tmp218_;
						ValaLocalVariable* temp_var;
						ValaCCodeExpression* temp_ref;
						ValaCCodeExpression* _tmp219_;
						ValaCCodeIdentifier* _tmp220_;
						ValaCCodeFunctionCall* _tmp221_;
						ValaCCodeFunctionCall* len_call;
						temp_var = (_tmp218_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp217_ = vala_data_type_get_return_type (itype), TRUE, NULL, FALSE), _vala_code_node_unref0 (_tmp217_), _tmp218_);
						temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
						ccall_expr = (_tmp219_ = (ValaCCodeExpression*) vala_ccode_assignment_new (temp_ref, ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (ccall_expr), _tmp219_);
						((ValaCCodeBaseModule*) self)->requires_array_length = TRUE;
						len_call = (_tmp221_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp220_ = vala_ccode_identifier_new ("_vala_array_length"))), _vala_ccode_node_unref0 (_tmp220_), _tmp221_);
						vala_ccode_function_call_add_argument (len_call, temp_ref);
						vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) len_call);
						_vala_code_node_unref0 (temp_var);
						_vala_ccode_node_unref0 (temp_ref);
						_vala_ccode_node_unref0 (len_call);
					} else {
						if (!vala_delegate_get_no_array_length (deleg)) {
							ValaLocalVariable* temp_var;
							ValaCCodeExpression* temp_ref;
							ValaCCodeUnaryExpression* _tmp222_;
							temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL, TRUE);
							temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
							gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
							gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_carray_length_parameter_position (deleg) + (0.01 * dim), FALSE)), (ValaCCodeExpression*) (_tmp222_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
							_vala_ccode_node_unref0 (_tmp222_);
							vala_expression_append_array_size ((ValaExpression*) expr, temp_ref);
							_vala_code_node_unref0 (temp_var);
							_vala_ccode_node_unref0 (temp_ref);
						} else {
							ValaCCodeConstant* _tmp223_;
							vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) (_tmp223_ = vala_ccode_constant_new ("-1")));
							_vala_ccode_node_unref0 (_tmp223_);
						}
					}
				}
			}
		}
		_vala_code_node_unref0 (array_type);
	} else {
		gboolean _tmp224_;
		_tmp224_ = FALSE;
		if (deleg != NULL) {
			_tmp224_ = VALA_IS_DELEGATE_TYPE (vala_delegate_get_return_type (deleg));
		} else {
			_tmp224_ = FALSE;
		}
		if (_tmp224_) {
			ValaDelegateType* deleg_type;
			ValaDelegate* d;
			deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_delegate_get_return_type (deleg)));
			d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
			if (vala_delegate_get_has_target (d)) {
				ValaPointerType* _tmp226_;
				ValaVoidType* _tmp225_;
				ValaLocalVariable* _tmp227_;
				ValaLocalVariable* temp_var;
				ValaCCodeExpression* temp_ref;
				ValaCCodeUnaryExpression* _tmp228_;
				temp_var = (_tmp227_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, (ValaDataType*) (_tmp226_ = vala_pointer_type_new ((ValaDataType*) (_tmp225_ = vala_void_type_new (NULL)), NULL)), TRUE, NULL, TRUE), _vala_code_node_unref0 (_tmp226_), _vala_code_node_unref0 (_tmp225_), _tmp227_);
				temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
				gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
				gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_cdelegate_target_parameter_position (deleg), FALSE)), (ValaCCodeExpression*) (_tmp228_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
				_vala_ccode_node_unref0 (_tmp228_);
				vala_expression_set_delegate_target ((ValaExpression*) expr, temp_ref);
				_vala_code_node_unref0 (temp_var);
				_vala_ccode_node_unref0 (temp_ref);
			}
			_vala_code_node_unref0 (deleg_type);
			_vala_code_node_unref0 (d);
		}
	}
	_tmp229_ = FALSE;
	if (m != NULL) {
		_tmp229_ = vala_method_get_coroutine (m);
	} else {
		_tmp229_ = FALSE;
	}
	if (_tmp229_) {
		if (vala_method_call_get_is_yield_expression (expr)) {
			ValaCCodeIdentifier* _tmp232_;
			char* _tmp231_;
			char* _tmp230_;
			ValaCCodeIdentifier* _tmp233_;
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), FALSE)), (ValaCCodeExpression*) (_tmp232_ = vala_ccode_identifier_new (_tmp231_ = g_strconcat (_tmp230_ = vala_method_get_cname (vala_ccode_base_module_get_current_method ((ValaCCodeBaseModule*) self)), "_ready", NULL))));
			_vala_ccode_node_unref0 (_tmp232_);
			_g_free0 (_tmp231_);
			_g_free0 (_tmp230_);
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, -0.9, FALSE)), (ValaCCodeExpression*) (_tmp233_ = vala_ccode_identifier_new ("data")));
			_vala_ccode_node_unref0 (_tmp233_);
		}
	}
	_tmp234_ = FALSE;
	if (VALA_IS_CREATION_METHOD (m)) {
		GeeList* _tmp235_;
		_tmp234_ = gee_collection_get_size ((GeeCollection*) (_tmp235_ = vala_code_node_get_error_types ((ValaCodeNode*) m))) > 0;
		_gee_collection_object_unref0 (_tmp235_);
	} else {
		_tmp234_ = FALSE;
	}
	if (_tmp234_) {
		ValaCCodeIdentifier* _tmp236_;
		gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), FALSE)), (ValaCCodeExpression*) (_tmp236_ = vala_ccode_identifier_new ("error")));
		_vala_ccode_node_unref0 (_tmp236_);
	} else {
		if (vala_code_node_get_tree_can_fail ((ValaCodeNode*) expr)) {
			ValaCCodeUnaryExpression* _tmp238_;
			ValaCCodeExpression* _tmp237_;
			((ValaCCodeBaseModule*) self)->current_method_inner_error = TRUE;
			gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), FALSE)), (ValaCCodeExpression*) (_tmp238_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp237_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "_inner_error_"))));
			_vala_ccode_node_unref0 (_tmp238_);
			_vala_ccode_node_unref0 (_tmp237_);
		}
	}
	if (ellipsis) {
		gboolean _tmp239_;
		gboolean _tmp240_;
		_tmp239_ = FALSE;
		_tmp240_ = FALSE;
		if (!vala_method_get_printf_format (m)) {
			_tmp240_ = !vala_method_get_scanf_format (m);
		} else {
			_tmp240_ = FALSE;
		}
		if (_tmp240_) {
			_tmp239_ = _vala_strcmp0 (vala_method_get_sentinel (m), "") != 0;
		} else {
			_tmp239_ = FALSE;
		}
		if (_tmp239_) {
			ValaCCodeConstant* _tmp241_;
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), TRUE)), (ValaCCodeExpression*) (_tmp241_ = vala_ccode_constant_new (vala_method_get_sentinel (m))));
			_vala_ccode_node_unref0 (_tmp241_);
		}
	} else {
		if (VALA_IS_DELEGATE_TYPE (itype)) {
			ValaDelegateType* deleg_type;
			ValaDelegate* d;
			deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (itype));
			d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
			if (vala_delegate_get_has_target (d)) {
				ValaCCodeExpression* _tmp242_;
				ValaCCodeExpression* _tmp243_;
				gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_cinstance_parameter_position (d), FALSE)), _tmp242_ = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, vala_method_call_get_call (expr)));
				_vala_ccode_node_unref0 (_tmp242_);
				gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_cinstance_parameter_position (d), FALSE)), _tmp243_ = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, vala_method_call_get_call (expr)));
				_vala_ccode_node_unref0 (_tmp243_);
			}
			_vala_code_node_unref0 (deleg_type);
			_vala_code_node_unref0 (d);
		}
	}
	return_result_via_out_param = (_tmp245_ = vala_data_type_is_real_non_null_struct_type (_tmp244_ = vala_data_type_get_return_type (itype)), _vala_code_node_unref0 (_tmp244_), _tmp245_);
	_tmp246_ = FALSE;
	if (VALA_IS_SIGNAL_TYPE (itype)) {
		ValaDataType* _tmp247_;
		_tmp246_ = !VALA_IS_VOID_TYPE (_tmp247_ = vala_data_type_get_return_type (itype));
		_vala_code_node_unref0 (_tmp247_);
	} else {
		_tmp246_ = FALSE;
	}
	if (_tmp246_) {
		ValaSignal* sig;
		gboolean _tmp248_;
		gboolean _tmp249_;
		sig = _vala_code_node_ref0 (vala_signal_type_get_signal_symbol (VALA_SIGNAL_TYPE (itype)));
		_tmp248_ = FALSE;
		_tmp249_ = FALSE;
		if (ma != NULL) {
			_tmp249_ = VALA_IS_BASE_ACCESS (vala_member_access_get_inner (ma));
		} else {
			_tmp249_ = FALSE;
		}
		if (_tmp249_) {
			_tmp248_ = vala_signal_get_is_virtual (sig);
		} else {
			_tmp248_ = FALSE;
		}
		if (_tmp248_) {
		} else {
			if (!vala_signal_get_has_emitter (sig)) {
				return_result_via_out_param = TRUE;
			}
		}
		_vala_code_node_unref0 (sig);
	}
	if (return_result_via_out_param) {
		ValaDataType* _tmp250_;
		ValaLocalVariable* _tmp251_;
		ValaLocalVariable* temp_var;
		ValaCCodeExpression* temp_ref;
		ValaCCodeUnaryExpression* _tmp252_;
		ValaCCodeCommaExpression* ccomma;
		ValaCCodeExpression* _tmp253_;
		temp_var = (_tmp251_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp250_ = vala_data_type_get_return_type (itype), TRUE, NULL, TRUE), _vala_code_node_unref0 (_tmp250_), _tmp251_);
		temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
		gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
		gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-3), TRUE)), (ValaCCodeExpression*) (_tmp252_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
		_vala_ccode_node_unref0 (_tmp252_);
		ccomma = vala_ccode_comma_expression_new ();
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ccall_expr));
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (temp_ref));
		ccall_expr = (_tmp253_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (ccall_expr), _tmp253_);
		_vala_code_node_unref0 (temp_var);
		_vala_ccode_node_unref0 (temp_ref);
		_vala_ccode_node_unref0 (ccomma);
	}
	last_pos = 0;
	min_pos = 0;
	if (async_call != ccall) {
		last_pos = -1;
		while (TRUE) {
			ValaCCodeExpression* _tmp258_;
			min_pos = -1;
			{
				GeeSet* _tmp254_;
				GeeIterator* _tmp255_;
				GeeIterator* _pos_it;
				_pos_it = (_tmp255_ = gee_iterable_iterator ((GeeIterable*) (_tmp254_ = gee_map_get_keys ((GeeMap*) out_arg_map))), _gee_collection_object_unref0 (_tmp254_), _tmp255_);
				while (TRUE) {
					gint pos;
					gboolean _tmp256_;
					if (!gee_iterator_next (_pos_it)) {
						break;
					}
					pos = GPOINTER_TO_INT (gee_iterator_get (_pos_it));
					_tmp256_ = FALSE;
					if (pos > last_pos) {
						gboolean _tmp257_;
						_tmp257_ = FALSE;
						if (min_pos == (-1)) {
							_tmp257_ = TRUE;
						} else {
							_tmp257_ = pos < min_pos;
						}
						_tmp256_ = _tmp257_;
					} else {
						_tmp256_ = FALSE;
					}
					if (_tmp256_) {
						min_pos = pos;
					}
				}
				_gee_collection_object_unref0 (_pos_it);
			}
			if (min_pos == (-1)) {
				break;
			}
			vala_ccode_function_call_add_argument (ccall, _tmp258_ = (ValaCCodeExpression*) gee_map_get ((GeeMap*) out_arg_map, GINT_TO_POINTER (min_pos)));
			_vala_ccode_node_unref0 (_tmp258_);
			last_pos = min_pos;
		}
	}
	if (async_call != NULL) {
		last_pos = -1;
		while (TRUE) {
			ValaCCodeExpression* _tmp263_;
			min_pos = -1;
			{
				GeeSet* _tmp259_;
				GeeIterator* _tmp260_;
				GeeIterator* _pos_it;
				_pos_it = (_tmp260_ = gee_iterable_iterator ((GeeIterable*) (_tmp259_ = gee_map_get_keys ((GeeMap*) in_arg_map))), _gee_collection_object_unref0 (_tmp259_), _tmp260_);
				while (TRUE) {
					gint pos;
					gboolean _tmp261_;
					if (!gee_iterator_next (_pos_it)) {
						break;
					}
					pos = GPOINTER_TO_INT (gee_iterator_get (_pos_it));
					_tmp261_ = FALSE;
					if (pos > last_pos) {
						gboolean _tmp262_;
						_tmp262_ = FALSE;
						if (min_pos == (-1)) {
							_tmp262_ = TRUE;
						} else {
							_tmp262_ = pos < min_pos;
						}
						_tmp261_ = _tmp262_;
					} else {
						_tmp261_ = FALSE;
					}
					if (_tmp261_) {
						min_pos = pos;
					}
				}
				_gee_collection_object_unref0 (_pos_it);
			}
			if (min_pos == (-1)) {
				break;
			}
			vala_ccode_function_call_add_argument (async_call, _tmp263_ = (ValaCCodeExpression*) gee_map_get ((GeeMap*) in_arg_map, GINT_TO_POINTER (min_pos)));
			_vala_ccode_node_unref0 (_tmp263_);
			last_pos = min_pos;
		}
	}
	_tmp264_ = FALSE;
	_tmp265_ = FALSE;
	if (m != NULL) {
		_tmp265_ = vala_method_get_binding (m) == MEMBER_BINDING_INSTANCE;
	} else {
		_tmp265_ = FALSE;
	}
	if (_tmp265_) {
		_tmp264_ = vala_method_get_returns_modified_pointer (m);
	} else {
		_tmp264_ = FALSE;
	}
	if (_tmp264_) {
		ValaCCodeAssignment* _tmp266_;
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp266_ = vala_ccode_assignment_new (instance, ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		_vala_ccode_node_unref0 (_tmp266_);
	} else {
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall_expr);
	}
	if (vala_method_call_get_is_yield_expression (expr)) {
		ValaCCodeExpressionStatement* _tmp268_;
		gint state;
		ValaCCodeExpressionStatement* _tmp274_;
		ValaCCodeAssignment* _tmp273_;
		ValaCCodeConstant* _tmp272_;
		char* _tmp271_;
		ValaCCodeMemberAccess* _tmp270_;
		ValaCCodeIdentifier* _tmp269_;
		ValaCCodeReturnStatement* _tmp276_;
		ValaCCodeConstant* _tmp275_;
		ValaCCodeCaseStatement* _tmp279_;
		ValaCCodeConstant* _tmp278_;
		char* _tmp277_;
		if (((ValaCCodeBaseModule*) self)->pre_statement_fragment == NULL) {
			ValaCCodeFragment* _tmp267_;
			((ValaCCodeBaseModule*) self)->pre_statement_fragment = (_tmp267_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (((ValaCCodeBaseModule*) self)->pre_statement_fragment), _tmp267_);
		}
		vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp268_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) async_call)));
		_vala_ccode_node_unref0 (_tmp268_);
		state = ((ValaCCodeBaseModule*) self)->next_coroutine_state++;
		vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp274_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp273_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp270_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp269_ = vala_ccode_identifier_new ("data")), "_state_")), (ValaCCodeExpression*) (_tmp272_ = vala_ccode_constant_new (_tmp271_ = g_strdup_printf ("%i", state))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp274_);
		_vala_ccode_node_unref0 (_tmp273_);
		_vala_ccode_node_unref0 (_tmp272_);
		_g_free0 (_tmp271_);
		_vala_ccode_node_unref0 (_tmp270_);
		_vala_ccode_node_unref0 (_tmp269_);
		vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp276_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp275_ = vala_ccode_constant_new ("FALSE")))));
		_vala_ccode_node_unref0 (_tmp276_);
		_vala_ccode_node_unref0 (_tmp275_);
		vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp279_ = vala_ccode_case_statement_new ((ValaCCodeExpression*) (_tmp278_ = vala_ccode_constant_new (_tmp277_ = g_strdup_printf ("%i", state))))));
		_vala_ccode_node_unref0 (_tmp279_);
		_vala_ccode_node_unref0 (_tmp278_);
		_g_free0 (_tmp277_);
	}
	if (VALA_IS_ARRAY_RESIZE_METHOD (m)) {
		GeeList* _tmp280_;
		GeeIterator* _tmp281_;
		GeeIterator* arg_it;
		ValaExpression* _tmp282_;
		ValaCCodeExpression* _tmp283_;
		ValaCCodeExpression* new_size;
		ValaLocalVariable* temp_decl;
		ValaCCodeExpression* temp_ref;
		ValaCCodeExpression* clen;
		ValaCCodeExpression* celems;
		ValaArrayType* array_type;
		char* _tmp285_;
		char* _tmp284_;
		ValaCCodeIdentifier* _tmp286_;
		ValaCCodeIdentifier* csizeof;
		ValaCCodeBinaryExpression* cdelta;
		ValaCCodeBinaryExpression* ccheck;
		ValaCCodeIdentifier* _tmp287_;
		ValaCCodeFunctionCall* _tmp288_;
		ValaCCodeFunctionCall* czero;
		ValaCCodeBinaryExpression* _tmp289_;
		ValaCCodeConstant* _tmp290_;
		ValaCCodeBinaryExpression* _tmp291_;
		ValaCCodeCommaExpression* ccomma;
		ValaCCodeAssignment* _tmp292_;
		ValaCCodeConditionalExpression* _tmp294_;
		ValaCCodeConstant* _tmp293_;
		ValaCCodeAssignment* _tmp296_;
		ValaCCodeExpression* _tmp295_;
		arg_it = (_tmp281_ = gee_iterable_iterator ((GeeIterable*) (_tmp280_ = vala_method_call_get_argument_list (expr))), _gee_collection_object_unref0 (_tmp280_), _tmp281_);
		gee_iterator_next (arg_it);
		new_size = (_tmp283_ = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) (_tmp282_ = (ValaExpression*) gee_iterator_get (arg_it))))), _vala_code_node_unref0 (_tmp282_), _tmp283_);
		temp_decl = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL, TRUE);
		temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_decl));
		gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_decl);
		vala_ccode_declaration_space_add_include (((ValaCCodeBaseModule*) self)->source_declarations, "string.h", FALSE);
		clen = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_member_access_get_inner (ma), 1);
		celems = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma))));
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_expression_get_value_type (vala_member_access_get_inner (ma))));
		csizeof = (_tmp286_ = vala_ccode_identifier_new (_tmp285_ = g_strdup_printf ("sizeof (%s)", _tmp284_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))), _g_free0 (_tmp285_), _g_free0 (_tmp284_), _tmp286_);
		cdelta = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MINUS, temp_ref, clen);
		ccheck = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_GREATER_THAN, temp_ref, clen);
		czero = (_tmp288_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp287_ = vala_ccode_identifier_new ("memset"))), _vala_ccode_node_unref0 (_tmp287_), _tmp288_);
		vala_ccode_function_call_add_argument (czero, (ValaCCodeExpression*) (_tmp289_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_PLUS, celems, clen)));
		_vala_ccode_node_unref0 (_tmp289_);
		vala_ccode_function_call_add_argument (czero, (ValaCCodeExpression*) (_tmp290_ = vala_ccode_constant_new ("0")));
		_vala_ccode_node_unref0 (_tmp290_);
		vala_ccode_function_call_add_argument (czero, (ValaCCodeExpression*) (_tmp291_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, (ValaCCodeExpression*) csizeof, (ValaCCodeExpression*) cdelta)));
		_vala_ccode_node_unref0 (_tmp291_);
		ccomma = vala_ccode_comma_expression_new ();
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp292_ = vala_ccode_assignment_new (temp_ref, new_size, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		_vala_ccode_node_unref0 (_tmp292_);
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)));
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp294_ = vala_ccode_conditional_expression_new ((ValaCCodeExpression*) ccheck, (ValaCCodeExpression*) czero, (ValaCCodeExpression*) (_tmp293_ = vala_ccode_constant_new ("NULL")))));
		_vala_ccode_node_unref0 (_tmp294_);
		_vala_ccode_node_unref0 (_tmp293_);
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp296_ = vala_ccode_assignment_new (_tmp295_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_member_access_get_inner (ma), 1), temp_ref, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		_vala_ccode_node_unref0 (_tmp296_);
		_vala_ccode_node_unref0 (_tmp295_);
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
		_gee_collection_object_unref0 (arg_it);
		_vala_ccode_node_unref0 (new_size);
		_vala_code_node_unref0 (temp_decl);
		_vala_ccode_node_unref0 (temp_ref);
		_vala_ccode_node_unref0 (clen);
		_vala_ccode_node_unref0 (celems);
		_vala_code_node_unref0 (array_type);
		_vala_ccode_node_unref0 (csizeof);
		_vala_ccode_node_unref0 (cdelta);
		_vala_ccode_node_unref0 (ccheck);
		_vala_ccode_node_unref0 (czero);
		_vala_ccode_node_unref0 (ccomma);
	}
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (async_call);
	_vala_code_node_unref0 (m);
	_vala_code_node_unref0 (deleg);
	_gee_collection_object_unref0 (params);
	_vala_code_node_unref0 (ma);
	_vala_code_node_unref0 (itype);
	_gee_collection_object_unref0 (in_arg_map);
	_gee_collection_object_unref0 (out_arg_map);
	_vala_ccode_node_unref0 (ccall_expr);
	_vala_ccode_node_unref0 (instance);
	_gee_collection_object_unref0 (params_it);
}


static void vala_ccode_method_call_module_class_init (ValaCCodeMethodCallModuleClass * klass) {
	vala_ccode_method_call_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->visit_method_call = vala_ccode_method_call_module_real_visit_method_call;
}


static void vala_ccode_method_call_module_instance_init (ValaCCodeMethodCallModule * self) {
}


GType vala_ccode_method_call_module_get_type (void) {
	static GType vala_ccode_method_call_module_type_id = 0;
	if (vala_ccode_method_call_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeMethodCallModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_method_call_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeMethodCallModule), 0, (GInstanceInitFunc) vala_ccode_method_call_module_instance_init, NULL };
		vala_ccode_method_call_module_type_id = g_type_register_static (VALA_TYPE_CCODE_ASSIGNMENT_MODULE, "ValaCCodeMethodCallModule", &g_define_type_info, 0);
	}
	return vala_ccode_method_call_module_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




