/* valaccodemodule.vala
 *
 * Copyright (C) 2008-2009  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
#define _vala_ccode_module_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_module_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _ValaParamSpecCCodeModule ValaParamSpecCCodeModule;

struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

struct _ValaCCodeModulePrivate {
	ValaCCodeGenerator* _codegen;
	ValaCCodeModule* _head;
	ValaCCodeModule* next;
};

struct _ValaParamSpecCCodeModule {
	GParamSpec parent_instance;
};


static gpointer vala_ccode_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_generator_get_type (void);
#define VALA_CCODE_MODULE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_MODULE, ValaCCodeModulePrivate))
enum  {
	VALA_CCODE_MODULE_DUMMY_PROPERTY
};
static void vala_ccode_module_set_codegen (ValaCCodeModule* self, ValaCCodeGenerator* value);
static void vala_ccode_module_set_head (ValaCCodeModule* self, ValaCCodeModule* value);
ValaCCodeModule* vala_ccode_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
void vala_ccode_module_emit (ValaCCodeModule* self, ValaCodeContext* context);
static void vala_ccode_module_real_emit (ValaCCodeModule* self, ValaCodeContext* context);
void vala_ccode_module_visit_source_file (ValaCCodeModule* self, ValaSourceFile* source_file);
static void vala_ccode_module_real_visit_source_file (ValaCCodeModule* self, ValaSourceFile* source_file);
void vala_ccode_module_visit_class (ValaCCodeModule* self, ValaClass* cl);
static void vala_ccode_module_real_visit_class (ValaCCodeModule* self, ValaClass* cl);
void vala_ccode_module_visit_interface (ValaCCodeModule* self, ValaInterface* iface);
static void vala_ccode_module_real_visit_interface (ValaCCodeModule* self, ValaInterface* iface);
void vala_ccode_module_visit_struct (ValaCCodeModule* self, ValaStruct* st);
static void vala_ccode_module_real_visit_struct (ValaCCodeModule* self, ValaStruct* st);
void vala_ccode_module_visit_enum (ValaCCodeModule* self, ValaEnum* en);
static void vala_ccode_module_real_visit_enum (ValaCCodeModule* self, ValaEnum* en);
void vala_ccode_module_visit_error_domain (ValaCCodeModule* self, ValaErrorDomain* edomain);
static void vala_ccode_module_real_visit_error_domain (ValaCCodeModule* self, ValaErrorDomain* edomain);
void vala_ccode_module_visit_delegate (ValaCCodeModule* self, ValaDelegate* d);
static void vala_ccode_module_real_visit_delegate (ValaCCodeModule* self, ValaDelegate* d);
void vala_ccode_module_visit_member (ValaCCodeModule* self, ValaMember* m);
static void vala_ccode_module_real_visit_member (ValaCCodeModule* self, ValaMember* m);
void vala_ccode_module_visit_constant (ValaCCodeModule* self, ValaConstant* c);
static void vala_ccode_module_real_visit_constant (ValaCCodeModule* self, ValaConstant* c);
void vala_ccode_module_visit_field (ValaCCodeModule* self, ValaField* f);
static void vala_ccode_module_real_visit_field (ValaCCodeModule* self, ValaField* f);
void vala_ccode_module_visit_method (ValaCCodeModule* self, ValaMethod* m);
static void vala_ccode_module_real_visit_method (ValaCCodeModule* self, ValaMethod* m);
void vala_ccode_module_visit_creation_method (ValaCCodeModule* self, ValaCreationMethod* m);
static void vala_ccode_module_real_visit_creation_method (ValaCCodeModule* self, ValaCreationMethod* m);
void vala_ccode_module_visit_formal_parameter (ValaCCodeModule* self, ValaFormalParameter* p);
static void vala_ccode_module_real_visit_formal_parameter (ValaCCodeModule* self, ValaFormalParameter* p);
void vala_ccode_module_visit_property (ValaCCodeModule* self, ValaProperty* prop);
static void vala_ccode_module_real_visit_property (ValaCCodeModule* self, ValaProperty* prop);
void vala_ccode_module_visit_property_accessor (ValaCCodeModule* self, ValaPropertyAccessor* acc);
static void vala_ccode_module_real_visit_property_accessor (ValaCCodeModule* self, ValaPropertyAccessor* acc);
void vala_ccode_module_visit_signal (ValaCCodeModule* self, ValaSignal* sig);
static void vala_ccode_module_real_visit_signal (ValaCCodeModule* self, ValaSignal* sig);
void vala_ccode_module_visit_constructor (ValaCCodeModule* self, ValaConstructor* c);
static void vala_ccode_module_real_visit_constructor (ValaCCodeModule* self, ValaConstructor* c);
void vala_ccode_module_visit_destructor (ValaCCodeModule* self, ValaDestructor* d);
static void vala_ccode_module_real_visit_destructor (ValaCCodeModule* self, ValaDestructor* d);
void vala_ccode_module_visit_block (ValaCCodeModule* self, ValaBlock* b);
static void vala_ccode_module_real_visit_block (ValaCCodeModule* self, ValaBlock* b);
void vala_ccode_module_visit_empty_statement (ValaCCodeModule* self, ValaEmptyStatement* stmt);
static void vala_ccode_module_real_visit_empty_statement (ValaCCodeModule* self, ValaEmptyStatement* stmt);
void vala_ccode_module_visit_declaration_statement (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
static void vala_ccode_module_real_visit_declaration_statement (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
void vala_ccode_module_visit_local_variable (ValaCCodeModule* self, ValaLocalVariable* local);
static void vala_ccode_module_real_visit_local_variable (ValaCCodeModule* self, ValaLocalVariable* local);
void vala_ccode_module_visit_initializer_list (ValaCCodeModule* self, ValaInitializerList* list);
static void vala_ccode_module_real_visit_initializer_list (ValaCCodeModule* self, ValaInitializerList* list);
void vala_ccode_module_visit_end_full_expression (ValaCCodeModule* self, ValaExpression* expr);
static void vala_ccode_module_real_visit_end_full_expression (ValaCCodeModule* self, ValaExpression* expr);
void vala_ccode_module_visit_expression_statement (ValaCCodeModule* self, ValaExpressionStatement* stmt);
static void vala_ccode_module_real_visit_expression_statement (ValaCCodeModule* self, ValaExpressionStatement* stmt);
void vala_ccode_module_visit_if_statement (ValaCCodeModule* self, ValaIfStatement* stmt);
static void vala_ccode_module_real_visit_if_statement (ValaCCodeModule* self, ValaIfStatement* stmt);
void vala_ccode_module_visit_switch_statement (ValaCCodeModule* self, ValaSwitchStatement* stmt);
static void vala_ccode_module_real_visit_switch_statement (ValaCCodeModule* self, ValaSwitchStatement* stmt);
void vala_ccode_module_visit_switch_section (ValaCCodeModule* self, ValaSwitchSection* section);
static void vala_ccode_module_real_visit_switch_section (ValaCCodeModule* self, ValaSwitchSection* section);
void vala_ccode_module_visit_switch_label (ValaCCodeModule* self, ValaSwitchLabel* label);
static void vala_ccode_module_real_visit_switch_label (ValaCCodeModule* self, ValaSwitchLabel* label);
void vala_ccode_module_visit_loop (ValaCCodeModule* self, ValaLoop* stmt);
static void vala_ccode_module_real_visit_loop (ValaCCodeModule* self, ValaLoop* stmt);
void vala_ccode_module_visit_foreach_statement (ValaCCodeModule* self, ValaForeachStatement* stmt);
static void vala_ccode_module_real_visit_foreach_statement (ValaCCodeModule* self, ValaForeachStatement* stmt);
void vala_ccode_module_visit_break_statement (ValaCCodeModule* self, ValaBreakStatement* stmt);
static void vala_ccode_module_real_visit_break_statement (ValaCCodeModule* self, ValaBreakStatement* stmt);
void vala_ccode_module_visit_continue_statement (ValaCCodeModule* self, ValaContinueStatement* stmt);
static void vala_ccode_module_real_visit_continue_statement (ValaCCodeModule* self, ValaContinueStatement* stmt);
void vala_ccode_module_visit_return_statement (ValaCCodeModule* self, ValaReturnStatement* stmt);
static void vala_ccode_module_real_visit_return_statement (ValaCCodeModule* self, ValaReturnStatement* stmt);
void vala_ccode_module_visit_yield_statement (ValaCCodeModule* self, ValaYieldStatement* stmt);
static void vala_ccode_module_real_visit_yield_statement (ValaCCodeModule* self, ValaYieldStatement* stmt);
void vala_ccode_module_visit_throw_statement (ValaCCodeModule* self, ValaThrowStatement* stmt);
static void vala_ccode_module_real_visit_throw_statement (ValaCCodeModule* self, ValaThrowStatement* stmt);
void vala_ccode_module_visit_try_statement (ValaCCodeModule* self, ValaTryStatement* stmt);
static void vala_ccode_module_real_visit_try_statement (ValaCCodeModule* self, ValaTryStatement* stmt);
void vala_ccode_module_visit_catch_clause (ValaCCodeModule* self, ValaCatchClause* clause);
static void vala_ccode_module_real_visit_catch_clause (ValaCCodeModule* self, ValaCatchClause* clause);
void vala_ccode_module_visit_lock_statement (ValaCCodeModule* self, ValaLockStatement* stmt);
static void vala_ccode_module_real_visit_lock_statement (ValaCCodeModule* self, ValaLockStatement* stmt);
void vala_ccode_module_visit_delete_statement (ValaCCodeModule* self, ValaDeleteStatement* stmt);
static void vala_ccode_module_real_visit_delete_statement (ValaCCodeModule* self, ValaDeleteStatement* stmt);
void vala_ccode_module_visit_expression (ValaCCodeModule* self, ValaExpression* expr);
static void vala_ccode_module_real_visit_expression (ValaCCodeModule* self, ValaExpression* expr);
void vala_ccode_module_visit_array_creation_expression (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
static void vala_ccode_module_real_visit_array_creation_expression (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
void vala_ccode_module_visit_boolean_literal (ValaCCodeModule* self, ValaBooleanLiteral* expr);
static void vala_ccode_module_real_visit_boolean_literal (ValaCCodeModule* self, ValaBooleanLiteral* expr);
void vala_ccode_module_visit_character_literal (ValaCCodeModule* self, ValaCharacterLiteral* expr);
static void vala_ccode_module_real_visit_character_literal (ValaCCodeModule* self, ValaCharacterLiteral* expr);
void vala_ccode_module_visit_integer_literal (ValaCCodeModule* self, ValaIntegerLiteral* expr);
static void vala_ccode_module_real_visit_integer_literal (ValaCCodeModule* self, ValaIntegerLiteral* expr);
void vala_ccode_module_visit_real_literal (ValaCCodeModule* self, ValaRealLiteral* expr);
static void vala_ccode_module_real_visit_real_literal (ValaCCodeModule* self, ValaRealLiteral* expr);
void vala_ccode_module_visit_string_literal (ValaCCodeModule* self, ValaStringLiteral* expr);
static void vala_ccode_module_real_visit_string_literal (ValaCCodeModule* self, ValaStringLiteral* expr);
void vala_ccode_module_visit_null_literal (ValaCCodeModule* self, ValaNullLiteral* expr);
static void vala_ccode_module_real_visit_null_literal (ValaCCodeModule* self, ValaNullLiteral* expr);
void vala_ccode_module_visit_member_access (ValaCCodeModule* self, ValaMemberAccess* expr);
static void vala_ccode_module_real_visit_member_access (ValaCCodeModule* self, ValaMemberAccess* expr);
void vala_ccode_module_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr);
static void vala_ccode_module_real_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr);
void vala_ccode_module_visit_element_access (ValaCCodeModule* self, ValaElementAccess* expr);
static void vala_ccode_module_real_visit_element_access (ValaCCodeModule* self, ValaElementAccess* expr);
void vala_ccode_module_visit_base_access (ValaCCodeModule* self, ValaBaseAccess* expr);
static void vala_ccode_module_real_visit_base_access (ValaCCodeModule* self, ValaBaseAccess* expr);
void vala_ccode_module_visit_postfix_expression (ValaCCodeModule* self, ValaPostfixExpression* expr);
static void vala_ccode_module_real_visit_postfix_expression (ValaCCodeModule* self, ValaPostfixExpression* expr);
void vala_ccode_module_visit_object_creation_expression (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
static void vala_ccode_module_real_visit_object_creation_expression (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
void vala_ccode_module_visit_sizeof_expression (ValaCCodeModule* self, ValaSizeofExpression* expr);
static void vala_ccode_module_real_visit_sizeof_expression (ValaCCodeModule* self, ValaSizeofExpression* expr);
void vala_ccode_module_visit_typeof_expression (ValaCCodeModule* self, ValaTypeofExpression* expr);
static void vala_ccode_module_real_visit_typeof_expression (ValaCCodeModule* self, ValaTypeofExpression* expr);
void vala_ccode_module_visit_unary_expression (ValaCCodeModule* self, ValaUnaryExpression* expr);
static void vala_ccode_module_real_visit_unary_expression (ValaCCodeModule* self, ValaUnaryExpression* expr);
void vala_ccode_module_visit_cast_expression (ValaCCodeModule* self, ValaCastExpression* expr);
static void vala_ccode_module_real_visit_cast_expression (ValaCCodeModule* self, ValaCastExpression* expr);
void vala_ccode_module_visit_pointer_indirection (ValaCCodeModule* self, ValaPointerIndirection* expr);
static void vala_ccode_module_real_visit_pointer_indirection (ValaCCodeModule* self, ValaPointerIndirection* expr);
void vala_ccode_module_visit_addressof_expression (ValaCCodeModule* self, ValaAddressofExpression* expr);
static void vala_ccode_module_real_visit_addressof_expression (ValaCCodeModule* self, ValaAddressofExpression* expr);
void vala_ccode_module_visit_reference_transfer_expression (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
static void vala_ccode_module_real_visit_reference_transfer_expression (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
void vala_ccode_module_visit_binary_expression (ValaCCodeModule* self, ValaBinaryExpression* expr);
static void vala_ccode_module_real_visit_binary_expression (ValaCCodeModule* self, ValaBinaryExpression* expr);
void vala_ccode_module_visit_type_check (ValaCCodeModule* self, ValaTypeCheck* expr);
static void vala_ccode_module_real_visit_type_check (ValaCCodeModule* self, ValaTypeCheck* expr);
void vala_ccode_module_visit_lambda_expression (ValaCCodeModule* self, ValaLambdaExpression* l);
static void vala_ccode_module_real_visit_lambda_expression (ValaCCodeModule* self, ValaLambdaExpression* l);
void vala_ccode_module_visit_assignment (ValaCCodeModule* self, ValaAssignment* a);
static void vala_ccode_module_real_visit_assignment (ValaCCodeModule* self, ValaAssignment* a);
char* vala_ccode_module_get_custom_creturn_type (ValaCCodeModule* self, ValaMethod* m);
static char* vala_ccode_module_real_get_custom_creturn_type (ValaCCodeModule* self, ValaMethod* m);
void vala_ccode_module_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method);
static void vala_ccode_module_real_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method);
gboolean vala_ccode_module_method_has_wrapper (ValaCCodeModule* self, ValaMethod* method);
static gboolean vala_ccode_module_real_method_has_wrapper (ValaCCodeModule* self, ValaMethod* method);
ValaCCodeIdentifier* vala_ccode_module_get_value_setter_function (ValaCCodeModule* self, ValaDataType* type_reference);
static ValaCCodeIdentifier* vala_ccode_module_real_get_value_setter_function (ValaCCodeModule* self, ValaDataType* type_reference);
ValaCCodeExpression* vala_ccode_module_get_construct_property_assignment (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
static ValaCCodeExpression* vala_ccode_module_real_get_construct_property_assignment (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
ValaCCodeFunctionCall* vala_ccode_module_get_param_spec (ValaCCodeModule* self, ValaProperty* prop);
static ValaCCodeFunctionCall* vala_ccode_module_real_get_param_spec (ValaCCodeModule* self, ValaProperty* prop);
ValaCCodeFunctionCall* vala_ccode_module_get_signal_creation (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
static ValaCCodeFunctionCall* vala_ccode_module_real_get_signal_creation (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
ValaCCodeFragment* vala_ccode_module_register_dbus_info (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
static ValaCCodeFragment* vala_ccode_module_real_register_dbus_info (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
char* vala_ccode_module_get_dynamic_property_getter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
static char* vala_ccode_module_real_get_dynamic_property_getter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
char* vala_ccode_module_get_dynamic_property_setter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
static char* vala_ccode_module_real_get_dynamic_property_setter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
char* vala_ccode_module_get_dynamic_signal_cname (ValaCCodeModule* self, ValaDynamicSignal* node);
static char* vala_ccode_module_real_get_dynamic_signal_cname (ValaCCodeModule* self, ValaDynamicSignal* node);
char* vala_ccode_module_get_dynamic_signal_connect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node);
static char* vala_ccode_module_real_get_dynamic_signal_connect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node);
char* vala_ccode_module_get_dynamic_signal_disconnect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node);
static char* vala_ccode_module_real_get_dynamic_signal_disconnect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node);
void vala_ccode_module_generate_marshaller (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
static void vala_ccode_module_real_generate_marshaller (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
char* vala_ccode_module_get_marshaller_function (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
static char* vala_ccode_module_real_get_marshaller_function (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
char* vala_ccode_module_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim);
static char* vala_ccode_module_real_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim);
ValaCCodeExpression* vala_ccode_module_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
static ValaCCodeExpression* vala_ccode_module_real_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
char* vala_ccode_module_get_array_size_cname (ValaCCodeModule* self, const char* array_cname);
static char* vala_ccode_module_real_get_array_size_cname (ValaCCodeModule* self, const char* array_cname);
ValaCCodeExpression* vala_ccode_module_get_array_size_cexpression (ValaCCodeModule* self, ValaExpression* array_expr);
static ValaCCodeExpression* vala_ccode_module_real_get_array_size_cexpression (ValaCCodeModule* self, ValaExpression* array_expr);
void vala_ccode_module_add_simple_check (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
static void vala_ccode_module_real_add_simple_check (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
static void vala_ccode_module_finalize (ValaCCodeModule* obj);



static gpointer _vala_ccode_module_ref0 (gpointer self) {
	return self ? vala_ccode_module_ref (self) : NULL;
}


ValaCCodeModule* vala_ccode_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaCCodeModule* self;
	ValaCCodeModule* _tmp0_;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaCCodeModule*) g_type_create_instance (object_type);
	vala_ccode_module_set_codegen (self, codegen);
	self->priv->next = (_tmp0_ = _vala_ccode_module_ref0 (next), _vala_ccode_module_unref0 (self->priv->next), _tmp0_);
	vala_ccode_module_set_head (self, self);
	return self;
}


static void vala_ccode_module_real_emit (ValaCCodeModule* self, ValaCodeContext* context) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (context != NULL);
	vala_ccode_module_emit (self->priv->next, context);
}


void vala_ccode_module_emit (ValaCCodeModule* self, ValaCodeContext* context) {
	VALA_CCODE_MODULE_GET_CLASS (self)->emit (self, context);
}


static void vala_ccode_module_real_visit_source_file (ValaCCodeModule* self, ValaSourceFile* source_file) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (source_file != NULL);
	vala_ccode_module_visit_source_file (self->priv->next, source_file);
}


void vala_ccode_module_visit_source_file (ValaCCodeModule* self, ValaSourceFile* source_file) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_source_file (self, source_file);
}


static void vala_ccode_module_real_visit_class (ValaCCodeModule* self, ValaClass* cl) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (cl != NULL);
	vala_ccode_module_visit_class (self->priv->next, cl);
}


void vala_ccode_module_visit_class (ValaCCodeModule* self, ValaClass* cl) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_class (self, cl);
}


static void vala_ccode_module_real_visit_interface (ValaCCodeModule* self, ValaInterface* iface) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (iface != NULL);
	vala_ccode_module_visit_interface (self->priv->next, iface);
}


void vala_ccode_module_visit_interface (ValaCCodeModule* self, ValaInterface* iface) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_interface (self, iface);
}


static void vala_ccode_module_real_visit_struct (ValaCCodeModule* self, ValaStruct* st) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	vala_ccode_module_visit_struct (self->priv->next, st);
}


void vala_ccode_module_visit_struct (ValaCCodeModule* self, ValaStruct* st) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_struct (self, st);
}


static void vala_ccode_module_real_visit_enum (ValaCCodeModule* self, ValaEnum* en) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (en != NULL);
	vala_ccode_module_visit_enum (self->priv->next, en);
}


void vala_ccode_module_visit_enum (ValaCCodeModule* self, ValaEnum* en) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_enum (self, en);
}


static void vala_ccode_module_real_visit_error_domain (ValaCCodeModule* self, ValaErrorDomain* edomain) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (edomain != NULL);
	vala_ccode_module_visit_error_domain (self->priv->next, edomain);
}


void vala_ccode_module_visit_error_domain (ValaCCodeModule* self, ValaErrorDomain* edomain) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_error_domain (self, edomain);
}


static void vala_ccode_module_real_visit_delegate (ValaCCodeModule* self, ValaDelegate* d) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (d != NULL);
	vala_ccode_module_visit_delegate (self->priv->next, d);
}


void vala_ccode_module_visit_delegate (ValaCCodeModule* self, ValaDelegate* d) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_delegate (self, d);
}


static void vala_ccode_module_real_visit_member (ValaCCodeModule* self, ValaMember* m) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (m != NULL);
	vala_ccode_module_visit_member (self->priv->next, m);
}


void vala_ccode_module_visit_member (ValaCCodeModule* self, ValaMember* m) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_member (self, m);
}


static void vala_ccode_module_real_visit_constant (ValaCCodeModule* self, ValaConstant* c) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (c != NULL);
	vala_ccode_module_visit_constant (self->priv->next, c);
}


void vala_ccode_module_visit_constant (ValaCCodeModule* self, ValaConstant* c) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_constant (self, c);
}


static void vala_ccode_module_real_visit_field (ValaCCodeModule* self, ValaField* f) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (f != NULL);
	vala_ccode_module_visit_field (self->priv->next, f);
}


void vala_ccode_module_visit_field (ValaCCodeModule* self, ValaField* f) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_field (self, f);
}


static void vala_ccode_module_real_visit_method (ValaCCodeModule* self, ValaMethod* m) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (m != NULL);
	vala_ccode_module_visit_method (self->priv->next, m);
}


void vala_ccode_module_visit_method (ValaCCodeModule* self, ValaMethod* m) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_method (self, m);
}


static void vala_ccode_module_real_visit_creation_method (ValaCCodeModule* self, ValaCreationMethod* m) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (m != NULL);
	vala_ccode_module_visit_creation_method (self->priv->next, m);
}


void vala_ccode_module_visit_creation_method (ValaCCodeModule* self, ValaCreationMethod* m) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_creation_method (self, m);
}


static void vala_ccode_module_real_visit_formal_parameter (ValaCCodeModule* self, ValaFormalParameter* p) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (p != NULL);
	vala_ccode_module_visit_formal_parameter (self->priv->next, p);
}


void vala_ccode_module_visit_formal_parameter (ValaCCodeModule* self, ValaFormalParameter* p) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_formal_parameter (self, p);
}


static void vala_ccode_module_real_visit_property (ValaCCodeModule* self, ValaProperty* prop) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (prop != NULL);
	vala_ccode_module_visit_property (self->priv->next, prop);
}


void vala_ccode_module_visit_property (ValaCCodeModule* self, ValaProperty* prop) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_property (self, prop);
}


static void vala_ccode_module_real_visit_property_accessor (ValaCCodeModule* self, ValaPropertyAccessor* acc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (acc != NULL);
	vala_ccode_module_visit_property_accessor (self->priv->next, acc);
}


void vala_ccode_module_visit_property_accessor (ValaCCodeModule* self, ValaPropertyAccessor* acc) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_property_accessor (self, acc);
}


static void vala_ccode_module_real_visit_signal (ValaCCodeModule* self, ValaSignal* sig) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sig != NULL);
	vala_ccode_module_visit_signal (self->priv->next, sig);
}


void vala_ccode_module_visit_signal (ValaCCodeModule* self, ValaSignal* sig) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_signal (self, sig);
}


static void vala_ccode_module_real_visit_constructor (ValaCCodeModule* self, ValaConstructor* c) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (c != NULL);
	vala_ccode_module_visit_constructor (self->priv->next, c);
}


void vala_ccode_module_visit_constructor (ValaCCodeModule* self, ValaConstructor* c) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_constructor (self, c);
}


static void vala_ccode_module_real_visit_destructor (ValaCCodeModule* self, ValaDestructor* d) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (d != NULL);
	vala_ccode_module_visit_destructor (self->priv->next, d);
}


void vala_ccode_module_visit_destructor (ValaCCodeModule* self, ValaDestructor* d) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_destructor (self, d);
}


static void vala_ccode_module_real_visit_block (ValaCCodeModule* self, ValaBlock* b) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (b != NULL);
	vala_ccode_module_visit_block (self->priv->next, b);
}


void vala_ccode_module_visit_block (ValaCCodeModule* self, ValaBlock* b) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_block (self, b);
}


static void vala_ccode_module_real_visit_empty_statement (ValaCCodeModule* self, ValaEmptyStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_empty_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_empty_statement (ValaCCodeModule* self, ValaEmptyStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_empty_statement (self, stmt);
}


static void vala_ccode_module_real_visit_declaration_statement (ValaCCodeModule* self, ValaDeclarationStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_declaration_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_declaration_statement (ValaCCodeModule* self, ValaDeclarationStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_declaration_statement (self, stmt);
}


static void vala_ccode_module_real_visit_local_variable (ValaCCodeModule* self, ValaLocalVariable* local) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (local != NULL);
	vala_ccode_module_visit_local_variable (self->priv->next, local);
}


void vala_ccode_module_visit_local_variable (ValaCCodeModule* self, ValaLocalVariable* local) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_local_variable (self, local);
}


static void vala_ccode_module_real_visit_initializer_list (ValaCCodeModule* self, ValaInitializerList* list) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (list != NULL);
	vala_ccode_module_visit_initializer_list (self->priv->next, list);
}


void vala_ccode_module_visit_initializer_list (ValaCCodeModule* self, ValaInitializerList* list) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_initializer_list (self, list);
}


static void vala_ccode_module_real_visit_end_full_expression (ValaCCodeModule* self, ValaExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_end_full_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_end_full_expression (ValaCCodeModule* self, ValaExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_end_full_expression (self, expr);
}


static void vala_ccode_module_real_visit_expression_statement (ValaCCodeModule* self, ValaExpressionStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_expression_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_expression_statement (ValaCCodeModule* self, ValaExpressionStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_expression_statement (self, stmt);
}


static void vala_ccode_module_real_visit_if_statement (ValaCCodeModule* self, ValaIfStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_if_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_if_statement (ValaCCodeModule* self, ValaIfStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_if_statement (self, stmt);
}


static void vala_ccode_module_real_visit_switch_statement (ValaCCodeModule* self, ValaSwitchStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_switch_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_switch_statement (ValaCCodeModule* self, ValaSwitchStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_switch_statement (self, stmt);
}


static void vala_ccode_module_real_visit_switch_section (ValaCCodeModule* self, ValaSwitchSection* section) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (section != NULL);
	vala_ccode_module_visit_switch_section (self->priv->next, section);
}


void vala_ccode_module_visit_switch_section (ValaCCodeModule* self, ValaSwitchSection* section) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_switch_section (self, section);
}


static void vala_ccode_module_real_visit_switch_label (ValaCCodeModule* self, ValaSwitchLabel* label) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (label != NULL);
	vala_ccode_module_visit_switch_label (self->priv->next, label);
}


void vala_ccode_module_visit_switch_label (ValaCCodeModule* self, ValaSwitchLabel* label) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_switch_label (self, label);
}


static void vala_ccode_module_real_visit_loop (ValaCCodeModule* self, ValaLoop* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_loop (self->priv->next, stmt);
}


void vala_ccode_module_visit_loop (ValaCCodeModule* self, ValaLoop* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_loop (self, stmt);
}


static void vala_ccode_module_real_visit_foreach_statement (ValaCCodeModule* self, ValaForeachStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_foreach_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_foreach_statement (ValaCCodeModule* self, ValaForeachStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_foreach_statement (self, stmt);
}


static void vala_ccode_module_real_visit_break_statement (ValaCCodeModule* self, ValaBreakStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_break_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_break_statement (ValaCCodeModule* self, ValaBreakStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_break_statement (self, stmt);
}


static void vala_ccode_module_real_visit_continue_statement (ValaCCodeModule* self, ValaContinueStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_continue_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_continue_statement (ValaCCodeModule* self, ValaContinueStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_continue_statement (self, stmt);
}


static void vala_ccode_module_real_visit_return_statement (ValaCCodeModule* self, ValaReturnStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_return_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_return_statement (ValaCCodeModule* self, ValaReturnStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_return_statement (self, stmt);
}


static void vala_ccode_module_real_visit_yield_statement (ValaCCodeModule* self, ValaYieldStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_yield_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_yield_statement (ValaCCodeModule* self, ValaYieldStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_yield_statement (self, stmt);
}


static void vala_ccode_module_real_visit_throw_statement (ValaCCodeModule* self, ValaThrowStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_throw_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_throw_statement (ValaCCodeModule* self, ValaThrowStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_throw_statement (self, stmt);
}


static void vala_ccode_module_real_visit_try_statement (ValaCCodeModule* self, ValaTryStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_try_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_try_statement (ValaCCodeModule* self, ValaTryStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_try_statement (self, stmt);
}


static void vala_ccode_module_real_visit_catch_clause (ValaCCodeModule* self, ValaCatchClause* clause) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (clause != NULL);
	vala_ccode_module_visit_catch_clause (self->priv->next, clause);
}


void vala_ccode_module_visit_catch_clause (ValaCCodeModule* self, ValaCatchClause* clause) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_catch_clause (self, clause);
}


static void vala_ccode_module_real_visit_lock_statement (ValaCCodeModule* self, ValaLockStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_lock_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_lock_statement (ValaCCodeModule* self, ValaLockStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_lock_statement (self, stmt);
}


static void vala_ccode_module_real_visit_delete_statement (ValaCCodeModule* self, ValaDeleteStatement* stmt) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	vala_ccode_module_visit_delete_statement (self->priv->next, stmt);
}


void vala_ccode_module_visit_delete_statement (ValaCCodeModule* self, ValaDeleteStatement* stmt) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_delete_statement (self, stmt);
}


static void vala_ccode_module_real_visit_expression (ValaCCodeModule* self, ValaExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_expression (ValaCCodeModule* self, ValaExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_expression (self, expr);
}


static void vala_ccode_module_real_visit_array_creation_expression (ValaCCodeModule* self, ValaArrayCreationExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_array_creation_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_array_creation_expression (ValaCCodeModule* self, ValaArrayCreationExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_array_creation_expression (self, expr);
}


static void vala_ccode_module_real_visit_boolean_literal (ValaCCodeModule* self, ValaBooleanLiteral* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_boolean_literal (self->priv->next, expr);
}


void vala_ccode_module_visit_boolean_literal (ValaCCodeModule* self, ValaBooleanLiteral* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_boolean_literal (self, expr);
}


static void vala_ccode_module_real_visit_character_literal (ValaCCodeModule* self, ValaCharacterLiteral* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_character_literal (self->priv->next, expr);
}


void vala_ccode_module_visit_character_literal (ValaCCodeModule* self, ValaCharacterLiteral* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_character_literal (self, expr);
}


static void vala_ccode_module_real_visit_integer_literal (ValaCCodeModule* self, ValaIntegerLiteral* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_integer_literal (self->priv->next, expr);
}


void vala_ccode_module_visit_integer_literal (ValaCCodeModule* self, ValaIntegerLiteral* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_integer_literal (self, expr);
}


static void vala_ccode_module_real_visit_real_literal (ValaCCodeModule* self, ValaRealLiteral* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_real_literal (self->priv->next, expr);
}


void vala_ccode_module_visit_real_literal (ValaCCodeModule* self, ValaRealLiteral* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_real_literal (self, expr);
}


static void vala_ccode_module_real_visit_string_literal (ValaCCodeModule* self, ValaStringLiteral* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_string_literal (self->priv->next, expr);
}


void vala_ccode_module_visit_string_literal (ValaCCodeModule* self, ValaStringLiteral* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_string_literal (self, expr);
}


static void vala_ccode_module_real_visit_null_literal (ValaCCodeModule* self, ValaNullLiteral* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_null_literal (self->priv->next, expr);
}


void vala_ccode_module_visit_null_literal (ValaCCodeModule* self, ValaNullLiteral* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_null_literal (self, expr);
}


static void vala_ccode_module_real_visit_member_access (ValaCCodeModule* self, ValaMemberAccess* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_member_access (self->priv->next, expr);
}


void vala_ccode_module_visit_member_access (ValaCCodeModule* self, ValaMemberAccess* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_member_access (self, expr);
}


static void vala_ccode_module_real_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_method_call (self->priv->next, expr);
}


void vala_ccode_module_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_method_call (self, expr);
}


static void vala_ccode_module_real_visit_element_access (ValaCCodeModule* self, ValaElementAccess* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_element_access (self->priv->next, expr);
}


void vala_ccode_module_visit_element_access (ValaCCodeModule* self, ValaElementAccess* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_element_access (self, expr);
}


static void vala_ccode_module_real_visit_base_access (ValaCCodeModule* self, ValaBaseAccess* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_base_access (self->priv->next, expr);
}


void vala_ccode_module_visit_base_access (ValaCCodeModule* self, ValaBaseAccess* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_base_access (self, expr);
}


static void vala_ccode_module_real_visit_postfix_expression (ValaCCodeModule* self, ValaPostfixExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_postfix_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_postfix_expression (ValaCCodeModule* self, ValaPostfixExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_postfix_expression (self, expr);
}


static void vala_ccode_module_real_visit_object_creation_expression (ValaCCodeModule* self, ValaObjectCreationExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_object_creation_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_object_creation_expression (ValaCCodeModule* self, ValaObjectCreationExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_object_creation_expression (self, expr);
}


static void vala_ccode_module_real_visit_sizeof_expression (ValaCCodeModule* self, ValaSizeofExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_sizeof_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_sizeof_expression (ValaCCodeModule* self, ValaSizeofExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_sizeof_expression (self, expr);
}


static void vala_ccode_module_real_visit_typeof_expression (ValaCCodeModule* self, ValaTypeofExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_typeof_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_typeof_expression (ValaCCodeModule* self, ValaTypeofExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_typeof_expression (self, expr);
}


static void vala_ccode_module_real_visit_unary_expression (ValaCCodeModule* self, ValaUnaryExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_unary_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_unary_expression (ValaCCodeModule* self, ValaUnaryExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_unary_expression (self, expr);
}


static void vala_ccode_module_real_visit_cast_expression (ValaCCodeModule* self, ValaCastExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_cast_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_cast_expression (ValaCCodeModule* self, ValaCastExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_cast_expression (self, expr);
}


static void vala_ccode_module_real_visit_pointer_indirection (ValaCCodeModule* self, ValaPointerIndirection* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_pointer_indirection (self->priv->next, expr);
}


void vala_ccode_module_visit_pointer_indirection (ValaCCodeModule* self, ValaPointerIndirection* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_pointer_indirection (self, expr);
}


static void vala_ccode_module_real_visit_addressof_expression (ValaCCodeModule* self, ValaAddressofExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_addressof_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_addressof_expression (ValaCCodeModule* self, ValaAddressofExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_addressof_expression (self, expr);
}


static void vala_ccode_module_real_visit_reference_transfer_expression (ValaCCodeModule* self, ValaReferenceTransferExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_reference_transfer_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_reference_transfer_expression (ValaCCodeModule* self, ValaReferenceTransferExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_reference_transfer_expression (self, expr);
}


static void vala_ccode_module_real_visit_binary_expression (ValaCCodeModule* self, ValaBinaryExpression* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_binary_expression (self->priv->next, expr);
}


void vala_ccode_module_visit_binary_expression (ValaCCodeModule* self, ValaBinaryExpression* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_binary_expression (self, expr);
}


static void vala_ccode_module_real_visit_type_check (ValaCCodeModule* self, ValaTypeCheck* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (expr != NULL);
	vala_ccode_module_visit_type_check (self->priv->next, expr);
}


void vala_ccode_module_visit_type_check (ValaCCodeModule* self, ValaTypeCheck* expr) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_type_check (self, expr);
}


static void vala_ccode_module_real_visit_lambda_expression (ValaCCodeModule* self, ValaLambdaExpression* l) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (l != NULL);
	vala_ccode_module_visit_lambda_expression (self->priv->next, l);
}


void vala_ccode_module_visit_lambda_expression (ValaCCodeModule* self, ValaLambdaExpression* l) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_lambda_expression (self, l);
}


static void vala_ccode_module_real_visit_assignment (ValaCCodeModule* self, ValaAssignment* a) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (a != NULL);
	vala_ccode_module_visit_assignment (self->priv->next, a);
}


void vala_ccode_module_visit_assignment (ValaCCodeModule* self, ValaAssignment* a) {
	VALA_CCODE_MODULE_GET_CLASS (self)->visit_assignment (self, a);
}


static char* vala_ccode_module_real_get_custom_creturn_type (ValaCCodeModule* self, ValaMethod* m) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (m != NULL, NULL);
	result = vala_ccode_module_get_custom_creturn_type (self->priv->next, m);
	return result;
}


char* vala_ccode_module_get_custom_creturn_type (ValaCCodeModule* self, ValaMethod* m) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_custom_creturn_type (self, m);
}


static void vala_ccode_module_real_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (method != NULL);
	vala_ccode_module_generate_dynamic_method_wrapper (self->priv->next, method);
}


void vala_ccode_module_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method) {
	VALA_CCODE_MODULE_GET_CLASS (self)->generate_dynamic_method_wrapper (self, method);
}


static gboolean vala_ccode_module_real_method_has_wrapper (ValaCCodeModule* self, ValaMethod* method) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (method != NULL, FALSE);
	result = vala_ccode_module_method_has_wrapper (self->priv->next, method);
	return result;
}


gboolean vala_ccode_module_method_has_wrapper (ValaCCodeModule* self, ValaMethod* method) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->method_has_wrapper (self, method);
}


static ValaCCodeIdentifier* vala_ccode_module_real_get_value_setter_function (ValaCCodeModule* self, ValaDataType* type_reference) {
	ValaCCodeIdentifier* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type_reference != NULL, NULL);
	result = vala_ccode_module_get_value_setter_function (self->priv->next, type_reference);
	return result;
}


ValaCCodeIdentifier* vala_ccode_module_get_value_setter_function (ValaCCodeModule* self, ValaDataType* type_reference) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_value_setter_function (self, type_reference);
}


static ValaCCodeExpression* vala_ccode_module_real_get_construct_property_assignment (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value) {
	ValaCCodeExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (canonical_cconstant != NULL, NULL);
	g_return_val_if_fail (property_type != NULL, NULL);
	g_return_val_if_fail (value != NULL, NULL);
	result = vala_ccode_module_get_construct_property_assignment (self->priv->next, canonical_cconstant, property_type, value);
	return result;
}


ValaCCodeExpression* vala_ccode_module_get_construct_property_assignment (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_construct_property_assignment (self, canonical_cconstant, property_type, value);
}


static ValaCCodeFunctionCall* vala_ccode_module_real_get_param_spec (ValaCCodeModule* self, ValaProperty* prop) {
	ValaCCodeFunctionCall* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (prop != NULL, NULL);
	result = vala_ccode_module_get_param_spec (self->priv->next, prop);
	return result;
}


ValaCCodeFunctionCall* vala_ccode_module_get_param_spec (ValaCCodeModule* self, ValaProperty* prop) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_param_spec (self, prop);
}


static ValaCCodeFunctionCall* vala_ccode_module_real_get_signal_creation (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type) {
	ValaCCodeFunctionCall* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sig != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	result = vala_ccode_module_get_signal_creation (self->priv->next, sig, type);
	return result;
}


ValaCCodeFunctionCall* vala_ccode_module_get_signal_creation (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_signal_creation (self, sig, type);
}


static ValaCCodeFragment* vala_ccode_module_real_register_dbus_info (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable) {
	ValaCCodeFragment* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (bindable != NULL, NULL);
	result = vala_ccode_module_register_dbus_info (self->priv->next, bindable);
	return result;
}


ValaCCodeFragment* vala_ccode_module_register_dbus_info (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->register_dbus_info (self, bindable);
}


static char* vala_ccode_module_real_get_dynamic_property_getter_cname (ValaCCodeModule* self, ValaDynamicProperty* node) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	if (self->priv->next == NULL) {
		char* _tmp1_;
		char* _tmp0_;
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) node), _tmp1_ = g_strdup_printf ("dynamic properties are not supported for %s", _tmp0_ = vala_code_node_to_string ((ValaCodeNode*) vala_dynamic_property_get_dynamic_type (node))));
		_g_free0 (_tmp1_);
		_g_free0 (_tmp0_);
		result = g_strdup ("");
		return result;
	}
	result = vala_ccode_module_get_dynamic_property_getter_cname (self->priv->next, node);
	return result;
}


char* vala_ccode_module_get_dynamic_property_getter_cname (ValaCCodeModule* self, ValaDynamicProperty* node) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_dynamic_property_getter_cname (self, node);
}


static char* vala_ccode_module_real_get_dynamic_property_setter_cname (ValaCCodeModule* self, ValaDynamicProperty* node) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	if (self->priv->next == NULL) {
		char* _tmp1_;
		char* _tmp0_;
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) node), _tmp1_ = g_strdup_printf ("dynamic properties are not supported for %s", _tmp0_ = vala_code_node_to_string ((ValaCodeNode*) vala_dynamic_property_get_dynamic_type (node))));
		_g_free0 (_tmp1_);
		_g_free0 (_tmp0_);
		result = g_strdup ("");
		return result;
	}
	result = vala_ccode_module_get_dynamic_property_setter_cname (self->priv->next, node);
	return result;
}


char* vala_ccode_module_get_dynamic_property_setter_cname (ValaCCodeModule* self, ValaDynamicProperty* node) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_dynamic_property_setter_cname (self, node);
}


static char* vala_ccode_module_real_get_dynamic_signal_cname (ValaCCodeModule* self, ValaDynamicSignal* node) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	result = vala_ccode_module_get_dynamic_signal_cname (self->priv->next, node);
	return result;
}


char* vala_ccode_module_get_dynamic_signal_cname (ValaCCodeModule* self, ValaDynamicSignal* node) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_dynamic_signal_cname (self, node);
}


static char* vala_ccode_module_real_get_dynamic_signal_connect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	result = vala_ccode_module_get_dynamic_signal_connect_wrapper_name (self->priv->next, node);
	return result;
}


char* vala_ccode_module_get_dynamic_signal_connect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_dynamic_signal_connect_wrapper_name (self, node);
}


static char* vala_ccode_module_real_get_dynamic_signal_disconnect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	result = vala_ccode_module_get_dynamic_signal_disconnect_wrapper_name (self->priv->next, node);
	return result;
}


char* vala_ccode_module_get_dynamic_signal_disconnect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_dynamic_signal_disconnect_wrapper_name (self, node);
}


static void vala_ccode_module_real_generate_marshaller (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (params != NULL);
	g_return_if_fail (return_type != NULL);
	vala_ccode_module_generate_marshaller (self->priv->next, params, return_type, dbus);
}


void vala_ccode_module_generate_marshaller (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus) {
	VALA_CCODE_MODULE_GET_CLASS (self)->generate_marshaller (self, params, return_type, dbus);
}


static char* vala_ccode_module_real_get_marshaller_function (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (params != NULL, NULL);
	g_return_val_if_fail (return_type != NULL, NULL);
	result = vala_ccode_module_get_marshaller_function (self->priv->next, params, return_type, prefix, dbus);
	return result;
}


char* vala_ccode_module_get_marshaller_function (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_marshaller_function (self, params, return_type, prefix, dbus);
}


static char* vala_ccode_module_real_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (array_cname != NULL, NULL);
	result = vala_ccode_module_get_array_length_cname (self->priv->next, array_cname, dim);
	return result;
}


char* vala_ccode_module_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_array_length_cname (self, array_cname, dim);
}


static ValaCCodeExpression* vala_ccode_module_real_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim) {
	ValaCCodeExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (array_expr != NULL, NULL);
	result = vala_ccode_module_get_array_length_cexpression (self->priv->next, array_expr, dim);
	return result;
}


ValaCCodeExpression* vala_ccode_module_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_array_length_cexpression (self, array_expr, dim);
}


static char* vala_ccode_module_real_get_array_size_cname (ValaCCodeModule* self, const char* array_cname) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (array_cname != NULL, NULL);
	result = vala_ccode_module_get_array_size_cname (self->priv->next, array_cname);
	return result;
}


char* vala_ccode_module_get_array_size_cname (ValaCCodeModule* self, const char* array_cname) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_array_size_cname (self, array_cname);
}


static ValaCCodeExpression* vala_ccode_module_real_get_array_size_cexpression (ValaCCodeModule* self, ValaExpression* array_expr) {
	ValaCCodeExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (array_expr != NULL, NULL);
	result = vala_ccode_module_get_array_size_cexpression (self->priv->next, array_expr);
	return result;
}


ValaCCodeExpression* vala_ccode_module_get_array_size_cexpression (ValaCCodeModule* self, ValaExpression* array_expr) {
	return VALA_CCODE_MODULE_GET_CLASS (self)->get_array_size_cexpression (self, array_expr);
}


static void vala_ccode_module_real_add_simple_check (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	g_return_if_fail (cfrag != NULL);
	vala_ccode_module_add_simple_check (self->priv->next, node, cfrag);
}


void vala_ccode_module_add_simple_check (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag) {
	VALA_CCODE_MODULE_GET_CLASS (self)->add_simple_check (self, node, cfrag);
}


ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self) {
	ValaCCodeGenerator* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_codegen;
	return result;
}


static void vala_ccode_module_set_codegen (ValaCCodeModule* self, ValaCCodeGenerator* value) {
	g_return_if_fail (self != NULL);
	self->priv->_codegen = value;
}


ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self) {
	ValaCCodeModule* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_head;
	return result;
}


static void vala_ccode_module_set_head (ValaCCodeModule* self, ValaCCodeModule* value) {
	g_return_if_fail (self != NULL);
	self->priv->_head = value;
	if (self->priv->next != NULL) {
		vala_ccode_module_set_head (self->priv->next, value);
	}
}


static void vala_value_ccode_module_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_ccode_module_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_ccode_module_unref (value->data[0].v_pointer);
	}
}


static void vala_value_ccode_module_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_ccode_module_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_ccode_module_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_ccode_module_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaCCodeModule* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_ccode_module_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_ccode_module_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaCCodeModule** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_ccode_module_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecCCodeModule* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_CCODE_MODULE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_ccode_module (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_MODULE), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_ccode_module (GValue* value, gpointer v_object) {
	ValaCCodeModule* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_MODULE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_CCODE_MODULE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_ccode_module_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_ccode_module_unref (old);
	}
}


static void vala_ccode_module_class_init (ValaCCodeModuleClass * klass) {
	vala_ccode_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->finalize = vala_ccode_module_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeModulePrivate));
	VALA_CCODE_MODULE_CLASS (klass)->emit = vala_ccode_module_real_emit;
	VALA_CCODE_MODULE_CLASS (klass)->visit_source_file = vala_ccode_module_real_visit_source_file;
	VALA_CCODE_MODULE_CLASS (klass)->visit_class = vala_ccode_module_real_visit_class;
	VALA_CCODE_MODULE_CLASS (klass)->visit_interface = vala_ccode_module_real_visit_interface;
	VALA_CCODE_MODULE_CLASS (klass)->visit_struct = vala_ccode_module_real_visit_struct;
	VALA_CCODE_MODULE_CLASS (klass)->visit_enum = vala_ccode_module_real_visit_enum;
	VALA_CCODE_MODULE_CLASS (klass)->visit_error_domain = vala_ccode_module_real_visit_error_domain;
	VALA_CCODE_MODULE_CLASS (klass)->visit_delegate = vala_ccode_module_real_visit_delegate;
	VALA_CCODE_MODULE_CLASS (klass)->visit_member = vala_ccode_module_real_visit_member;
	VALA_CCODE_MODULE_CLASS (klass)->visit_constant = vala_ccode_module_real_visit_constant;
	VALA_CCODE_MODULE_CLASS (klass)->visit_field = vala_ccode_module_real_visit_field;
	VALA_CCODE_MODULE_CLASS (klass)->visit_method = vala_ccode_module_real_visit_method;
	VALA_CCODE_MODULE_CLASS (klass)->visit_creation_method = vala_ccode_module_real_visit_creation_method;
	VALA_CCODE_MODULE_CLASS (klass)->visit_formal_parameter = vala_ccode_module_real_visit_formal_parameter;
	VALA_CCODE_MODULE_CLASS (klass)->visit_property = vala_ccode_module_real_visit_property;
	VALA_CCODE_MODULE_CLASS (klass)->visit_property_accessor = vala_ccode_module_real_visit_property_accessor;
	VALA_CCODE_MODULE_CLASS (klass)->visit_signal = vala_ccode_module_real_visit_signal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_constructor = vala_ccode_module_real_visit_constructor;
	VALA_CCODE_MODULE_CLASS (klass)->visit_destructor = vala_ccode_module_real_visit_destructor;
	VALA_CCODE_MODULE_CLASS (klass)->visit_block = vala_ccode_module_real_visit_block;
	VALA_CCODE_MODULE_CLASS (klass)->visit_empty_statement = vala_ccode_module_real_visit_empty_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_declaration_statement = vala_ccode_module_real_visit_declaration_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_local_variable = vala_ccode_module_real_visit_local_variable;
	VALA_CCODE_MODULE_CLASS (klass)->visit_initializer_list = vala_ccode_module_real_visit_initializer_list;
	VALA_CCODE_MODULE_CLASS (klass)->visit_end_full_expression = vala_ccode_module_real_visit_end_full_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_expression_statement = vala_ccode_module_real_visit_expression_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_if_statement = vala_ccode_module_real_visit_if_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_switch_statement = vala_ccode_module_real_visit_switch_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_switch_section = vala_ccode_module_real_visit_switch_section;
	VALA_CCODE_MODULE_CLASS (klass)->visit_switch_label = vala_ccode_module_real_visit_switch_label;
	VALA_CCODE_MODULE_CLASS (klass)->visit_loop = vala_ccode_module_real_visit_loop;
	VALA_CCODE_MODULE_CLASS (klass)->visit_foreach_statement = vala_ccode_module_real_visit_foreach_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_break_statement = vala_ccode_module_real_visit_break_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_continue_statement = vala_ccode_module_real_visit_continue_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_return_statement = vala_ccode_module_real_visit_return_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_yield_statement = vala_ccode_module_real_visit_yield_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_throw_statement = vala_ccode_module_real_visit_throw_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_try_statement = vala_ccode_module_real_visit_try_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_catch_clause = vala_ccode_module_real_visit_catch_clause;
	VALA_CCODE_MODULE_CLASS (klass)->visit_lock_statement = vala_ccode_module_real_visit_lock_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_delete_statement = vala_ccode_module_real_visit_delete_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_expression = vala_ccode_module_real_visit_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_array_creation_expression = vala_ccode_module_real_visit_array_creation_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_boolean_literal = vala_ccode_module_real_visit_boolean_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_character_literal = vala_ccode_module_real_visit_character_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_integer_literal = vala_ccode_module_real_visit_integer_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_real_literal = vala_ccode_module_real_visit_real_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_string_literal = vala_ccode_module_real_visit_string_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_null_literal = vala_ccode_module_real_visit_null_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_member_access = vala_ccode_module_real_visit_member_access;
	VALA_CCODE_MODULE_CLASS (klass)->visit_method_call = vala_ccode_module_real_visit_method_call;
	VALA_CCODE_MODULE_CLASS (klass)->visit_element_access = vala_ccode_module_real_visit_element_access;
	VALA_CCODE_MODULE_CLASS (klass)->visit_base_access = vala_ccode_module_real_visit_base_access;
	VALA_CCODE_MODULE_CLASS (klass)->visit_postfix_expression = vala_ccode_module_real_visit_postfix_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_object_creation_expression = vala_ccode_module_real_visit_object_creation_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_sizeof_expression = vala_ccode_module_real_visit_sizeof_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_typeof_expression = vala_ccode_module_real_visit_typeof_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_unary_expression = vala_ccode_module_real_visit_unary_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_cast_expression = vala_ccode_module_real_visit_cast_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_pointer_indirection = vala_ccode_module_real_visit_pointer_indirection;
	VALA_CCODE_MODULE_CLASS (klass)->visit_addressof_expression = vala_ccode_module_real_visit_addressof_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_reference_transfer_expression = vala_ccode_module_real_visit_reference_transfer_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_binary_expression = vala_ccode_module_real_visit_binary_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_type_check = vala_ccode_module_real_visit_type_check;
	VALA_CCODE_MODULE_CLASS (klass)->visit_lambda_expression = vala_ccode_module_real_visit_lambda_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_assignment = vala_ccode_module_real_visit_assignment;
	VALA_CCODE_MODULE_CLASS (klass)->get_custom_creturn_type = vala_ccode_module_real_get_custom_creturn_type;
	VALA_CCODE_MODULE_CLASS (klass)->generate_dynamic_method_wrapper = vala_ccode_module_real_generate_dynamic_method_wrapper;
	VALA_CCODE_MODULE_CLASS (klass)->method_has_wrapper = vala_ccode_module_real_method_has_wrapper;
	VALA_CCODE_MODULE_CLASS (klass)->get_value_setter_function = vala_ccode_module_real_get_value_setter_function;
	VALA_CCODE_MODULE_CLASS (klass)->get_construct_property_assignment = vala_ccode_module_real_get_construct_property_assignment;
	VALA_CCODE_MODULE_CLASS (klass)->get_param_spec = vala_ccode_module_real_get_param_spec;
	VALA_CCODE_MODULE_CLASS (klass)->get_signal_creation = vala_ccode_module_real_get_signal_creation;
	VALA_CCODE_MODULE_CLASS (klass)->register_dbus_info = vala_ccode_module_real_register_dbus_info;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_property_getter_cname = vala_ccode_module_real_get_dynamic_property_getter_cname;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_property_setter_cname = vala_ccode_module_real_get_dynamic_property_setter_cname;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_signal_cname = vala_ccode_module_real_get_dynamic_signal_cname;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_signal_connect_wrapper_name = vala_ccode_module_real_get_dynamic_signal_connect_wrapper_name;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_signal_disconnect_wrapper_name = vala_ccode_module_real_get_dynamic_signal_disconnect_wrapper_name;
	VALA_CCODE_MODULE_CLASS (klass)->generate_marshaller = vala_ccode_module_real_generate_marshaller;
	VALA_CCODE_MODULE_CLASS (klass)->get_marshaller_function = vala_ccode_module_real_get_marshaller_function;
	VALA_CCODE_MODULE_CLASS (klass)->get_array_length_cname = vala_ccode_module_real_get_array_length_cname;
	VALA_CCODE_MODULE_CLASS (klass)->get_array_length_cexpression = vala_ccode_module_real_get_array_length_cexpression;
	VALA_CCODE_MODULE_CLASS (klass)->get_array_size_cname = vala_ccode_module_real_get_array_size_cname;
	VALA_CCODE_MODULE_CLASS (klass)->get_array_size_cexpression = vala_ccode_module_real_get_array_size_cexpression;
	VALA_CCODE_MODULE_CLASS (klass)->add_simple_check = vala_ccode_module_real_add_simple_check;
}


static void vala_ccode_module_instance_init (ValaCCodeModule * self) {
	self->priv = VALA_CCODE_MODULE_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_ccode_module_finalize (ValaCCodeModule* obj) {
	ValaCCodeModule * self;
	self = VALA_CCODE_MODULE (obj);
	_vala_ccode_module_unref0 (self->priv->next);
}


GType vala_ccode_module_get_type (void) {
	static GType vala_ccode_module_type_id = 0;
	if (vala_ccode_module_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_ccode_module_init, vala_value_ccode_module_free_value, vala_value_ccode_module_copy_value, vala_value_ccode_module_peek_pointer, "p", vala_value_ccode_module_collect_value, "p", vala_value_ccode_module_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeModule), 0, (GInstanceInitFunc) vala_ccode_module_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_ccode_module_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaCCodeModule", &g_define_type_info, &g_define_type_fundamental_info, G_TYPE_FLAG_ABSTRACT);
	}
	return vala_ccode_module_type_id;
}


gpointer vala_ccode_module_ref (gpointer instance) {
	ValaCCodeModule* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_ccode_module_unref (gpointer instance) {
	ValaCCodeModule* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_CCODE_MODULE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




