/* valaccodestructmodule.vala
 *
 * Copyright (C) 2006-2009  Jürg Billeter
 * Copyright (C) 2006-2008  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))

typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;

#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))

typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;

#define VALA_TYPE_CCODE_STRUCT_MODULE (vala_ccode_struct_module_get_type ())
#define VALA_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModule))
#define VALA_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
#define VALA_IS_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_IS_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_CCODE_STRUCT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))

typedef struct _ValaCCodeStructModule ValaCCodeStructModule;
typedef struct _ValaCCodeStructModuleClass ValaCCodeStructModuleClass;
typedef struct _ValaCCodeStructModulePrivate ValaCCodeStructModulePrivate;

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))

#define VALA_TYPE_TYPEREGISTER_FUNCTION (vala_typeregister_function_get_type ())
#define VALA_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunction))
#define VALA_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
#define VALA_IS_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_IS_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_TYPEREGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))

typedef struct _ValaTypeRegisterFunction ValaTypeRegisterFunction;
typedef struct _ValaTypeRegisterFunctionClass ValaTypeRegisterFunctionClass;

#define VALA_TYPE_STRUCT_REGISTER_FUNCTION (vala_struct_register_function_get_type ())
#define VALA_STRUCT_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_STRUCT_REGISTER_FUNCTION, ValaStructRegisterFunction))
#define VALA_STRUCT_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_STRUCT_REGISTER_FUNCTION, ValaStructRegisterFunctionClass))
#define VALA_IS_STRUCT_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_STRUCT_REGISTER_FUNCTION))
#define VALA_IS_STRUCT_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_STRUCT_REGISTER_FUNCTION))
#define VALA_STRUCT_REGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_STRUCT_REGISTER_FUNCTION, ValaStructRegisterFunctionClass))

typedef struct _ValaStructRegisterFunction ValaStructRegisterFunction;
typedef struct _ValaStructRegisterFunctionClass ValaStructRegisterFunctionClass;
#define _vala_typeregister_function_unref0(var) ((var == NULL) ? NULL : (var = (vala_typeregister_function_unref (var), NULL)))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))

struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

struct _ValaCCodeBaseModule {
	ValaCCodeModule parent_instance;
	ValaCCodeBaseModulePrivate * priv;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaTryStatement* current_try;
	ValaCCodeDeclarationSpace* header_declarations;
	ValaCCodeDeclarationSpace* internal_header_declarations;
	ValaCCodeDeclarationSpace* source_declarations;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* class_init_fragment;
	ValaCCodeFragment* base_init_fragment;
	ValaCCodeFragment* class_finalize_fragment;
	ValaCCodeFragment* base_finalize_fragment;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_finalize_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeFragment* module_init_fragment;
	ValaCCodeStruct* param_spec_struct;
	ValaCCodeStruct* closure_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeFunction* function;
	ValaCCodeFragment* pre_statement_fragment;
	GeeArrayList* temp_vars;
	GeeArrayList* temp_ref_vars;
	GeeSet* user_marshal_set;
	GeeSet* predefined_marshal_set;
	gint next_temp_var_id;
	gboolean in_constructor;
	gboolean in_static_or_class_context;
	gboolean current_method_inner_error;
	gint next_coroutine_state;
	ValaDataType* void_type;
	ValaDataType* bool_type;
	ValaDataType* char_type;
	ValaDataType* uchar_type;
	ValaDataType* unichar_type;
	ValaDataType* short_type;
	ValaDataType* ushort_type;
	ValaDataType* int_type;
	ValaDataType* uint_type;
	ValaDataType* long_type;
	ValaDataType* ulong_type;
	ValaDataType* int8_type;
	ValaDataType* uint8_type;
	ValaDataType* int16_type;
	ValaDataType* uint16_type;
	ValaDataType* int32_type;
	ValaDataType* uint32_type;
	ValaDataType* int64_type;
	ValaDataType* uint64_type;
	ValaDataType* string_type;
	ValaDataType* float_type;
	ValaDataType* double_type;
	ValaTypeSymbol* gtype_type;
	ValaTypeSymbol* gobject_type;
	ValaErrorType* gerror_type;
	ValaClass* glist_type;
	ValaClass* gslist_type;
	ValaClass* gvaluearray_type;
	ValaTypeSymbol* gstringbuilder_type;
	ValaTypeSymbol* garray_type;
	ValaTypeSymbol* gbytearray_type;
	ValaTypeSymbol* gptrarray_type;
	ValaDataType* gquark_type;
	ValaStruct* gvalue_type;
	ValaStruct* mutex_type;
	ValaTypeSymbol* type_module_type;
	ValaTypeSymbol* dbus_object_type;
	gboolean in_plugin;
	char* module_init_param_name;
	gboolean gvaluecollector_h_needed;
	gboolean requires_array_free;
	gboolean requires_array_move;
	gboolean requires_array_length;
	gboolean requires_strcmp0;
	gboolean dbus_glib_h_needed;
	gboolean dbus_glib_h_needed_in_header;
	GeeSet* wrappers;
	GeeMap* variable_name_map;
};

struct _ValaCCodeBaseModuleClass {
	ValaCCodeModuleClass parent_class;
	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
	char* (*append_struct_array_free) (ValaCCodeBaseModule* self, ValaStruct* st);
	ValaCCodeExpression* (*get_unref_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
	void (*append_local_free) (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
	gboolean (*variable_accessible_in_finally) (ValaCCodeBaseModule* self, ValaLocalVariable* local);
	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_ref_cexpression) (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
	gboolean (*is_gobject_property) (ValaCCodeBaseModule* self, ValaProperty* prop);
};

struct _ValaCCodeStructModule {
	ValaCCodeBaseModule parent_instance;
	ValaCCodeStructModulePrivate * priv;
};

struct _ValaCCodeStructModuleClass {
	ValaCCodeBaseModuleClass parent_class;
};


static gpointer vala_ccode_struct_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_base_module_get_type (void);
gpointer vala_ccode_declaration_space_ref (gpointer instance);
void vala_ccode_declaration_space_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_declaration_space (const GValue* value);
GType vala_ccode_declaration_space_get_type (void);
GType vala_ccode_struct_module_get_type (void);
enum  {
	VALA_CCODE_STRUCT_MODULE_DUMMY_PROPERTY
};
GType vala_ccode_generator_get_type (void);
ValaCCodeBaseModule* vala_ccode_base_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeBaseModule* vala_ccode_base_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeStructModule* vala_ccode_struct_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeStructModule* vala_ccode_struct_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
gboolean vala_ccode_declaration_space_add_symbol_declaration (ValaCCodeDeclarationSpace* self, ValaSymbol* sym, const char* name);
void vala_ccode_declaration_space_add_include (ValaCCodeDeclarationSpace* self, const char* filename, gboolean local);
ValaCodeContext* vala_ccode_base_module_get_context (ValaCCodeBaseModule* self);
void vala_ccode_declaration_space_add_type_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
ValaStructRegisterFunction* vala_struct_register_function_new (ValaStruct* st, ValaCodeContext* context);
ValaStructRegisterFunction* vala_struct_register_function_construct (GType object_type, ValaStruct* st, ValaCodeContext* context);
gpointer vala_typeregister_function_ref (gpointer instance);
void vala_typeregister_function_unref (gpointer instance);
GParamSpec* vala_param_spec_typeregister_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_typeregister_function (GValue* value, gpointer v_object);
gpointer vala_value_get_typeregister_function (const GValue* value);
GType vala_typeregister_function_get_type (void);
GType vala_struct_register_function_get_type (void);
void vala_typeregister_function_init_from_type (ValaTypeRegisterFunction* self, gboolean plugin);
void vala_ccode_declaration_space_add_type_member_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
ValaCCodeFragment* vala_typeregister_function_get_declaration (ValaTypeRegisterFunction* self);
void vala_ccode_base_module_generate_type_declaration (ValaCCodeBaseModule* self, ValaDataType* type, ValaCCodeDeclarationSpace* decl_space);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
char* vala_ccode_module_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim);
char* vala_ccode_module_get_array_size_cname (ValaCCodeModule* self, const char* array_cname);
char* vala_ccode_base_module_get_delegate_target_cname (ValaCCodeBaseModule* self, const char* delegate_cname);
void vala_ccode_declaration_space_add_type_definition (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
static void vala_ccode_struct_module_real_generate_struct_declaration (ValaCCodeBaseModule* base, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_struct_declaration (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
static void vala_ccode_struct_module_add_struct_copy_function (ValaCCodeStructModule* self, ValaStruct* st);
static void vala_ccode_struct_module_add_struct_destroy_function (ValaCCodeStructModule* self, ValaStruct* st);
static void vala_ccode_struct_module_add_struct_dup_function (ValaCCodeStructModule* self, ValaStruct* st);
static void vala_ccode_struct_module_add_struct_free_function (ValaCCodeStructModule* self, ValaStruct* st);
static void vala_ccode_struct_module_real_visit_struct (ValaCCodeModule* base, ValaStruct* st);
gboolean vala_ccode_base_module_requires_copy (ValaCCodeBaseModule* self, ValaDataType* type);
ValaDataType* vala_ccode_base_module_get_data_type_for_symbol (ValaTypeSymbol* sym);
ValaCCodeExpression* vala_ccode_base_module_get_ref_cexpression (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
void vala_ccode_base_module_append_temp_decl (ValaCCodeBaseModule* self, ValaCCodeFragment* cfrag, GeeList* temp_vars);



ValaCCodeStructModule* vala_ccode_struct_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaCCodeStructModule* self;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaCCodeStructModule*) vala_ccode_base_module_construct (object_type, codegen, next);
	return self;
}


ValaCCodeStructModule* vala_ccode_struct_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_ccode_struct_module_construct (VALA_TYPE_CCODE_STRUCT_MODULE, codegen, next);
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


static void vala_ccode_struct_module_real_generate_struct_declaration (ValaCCodeBaseModule* base, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space) {
	ValaCCodeStructModule * self;
	char* _tmp0_;
	gboolean _tmp1_;
	char* _tmp12_;
	char* _tmp11_;
	ValaCCodeStruct* _tmp13_;
	ValaCCodeStruct* instance_struct;
	ValaCCodeTypeDefinition* _tmp32_;
	ValaCCodeVariableDeclarator* _tmp31_;
	char* _tmp30_;
	char* _tmp29_;
	char* _tmp28_;
	char* _tmp35_;
	char* _tmp34_;
	char* _tmp33_;
	ValaCCodeFunction* _tmp36_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp40_;
	char* _tmp39_;
	char* _tmp38_;
	char* _tmp37_;
	ValaCCodeFunction* _tmp42_;
	char* _tmp41_;
	ValaCCodeFormalParameter* _tmp45_;
	char* _tmp44_;
	char* _tmp43_;
	self = (ValaCCodeStructModule*) base;
	g_return_if_fail (st != NULL);
	g_return_if_fail (decl_space != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_symbol_declaration (decl_space, (ValaSymbol*) st, _tmp0_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
	if (vala_struct_is_boolean_type (st)) {
		vala_ccode_declaration_space_add_include (decl_space, "stdbool.h", FALSE);
		vala_struct_set_cname (st, "bool");
		return;
	} else {
		if (vala_struct_is_integer_type (st)) {
			const char* _tmp2_;
			char* _tmp3_;
			vala_ccode_declaration_space_add_include (decl_space, "stdint.h", FALSE);
			_tmp2_ = NULL;
			if (vala_struct_get_signed (st)) {
				_tmp2_ = "";
			} else {
				_tmp2_ = "u";
			}
			vala_struct_set_cname (st, _tmp3_ = g_strdup_printf ("%sint%d_t", _tmp2_, vala_struct_get_width (st)));
			_g_free0 (_tmp3_);
			return;
		} else {
			if (vala_struct_is_floating_type (st)) {
				const char* _tmp4_;
				_tmp4_ = NULL;
				if (vala_struct_get_width (st) == 64) {
					_tmp4_ = "double";
				} else {
					_tmp4_ = "float";
				}
				vala_struct_set_cname (st, _tmp4_);
				return;
			}
		}
	}
	if (vala_code_context_get_profile (vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self)) == VALA_PROFILE_GOBJECT) {
		if (vala_struct_get_has_type_id (st)) {
			ValaCCodeNewline* _tmp5_;
			char* _tmp6_;
			char* _tmp7_;
			char* macro;
			ValaCCodeMacroReplacement* _tmp9_;
			char* _tmp8_;
			ValaStructRegisterFunction* type_fun;
			ValaCCodeFragment* _tmp10_;
			vala_ccode_declaration_space_add_type_declaration (decl_space, (ValaCCodeNode*) (_tmp5_ = vala_ccode_newline_new ()));
			_vala_ccode_node_unref0 (_tmp5_);
			macro = (_tmp7_ = g_strdup_printf ("(%s_get_type ())", _tmp6_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) st, NULL)), _g_free0 (_tmp6_), _tmp7_);
			vala_ccode_declaration_space_add_type_declaration (decl_space, (ValaCCodeNode*) (_tmp9_ = vala_ccode_macro_replacement_new (_tmp8_ = vala_typesymbol_get_type_id ((ValaTypeSymbol*) st), macro)));
			_vala_ccode_node_unref0 (_tmp9_);
			_g_free0 (_tmp8_);
			type_fun = vala_struct_register_function_new (st, vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self));
			vala_typeregister_function_init_from_type ((ValaTypeRegisterFunction*) type_fun, FALSE);
			vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) (_tmp10_ = vala_typeregister_function_get_declaration ((ValaTypeRegisterFunction*) type_fun)));
			_vala_ccode_node_unref0 (_tmp10_);
			_g_free0 (macro);
			_vala_typeregister_function_unref0 (type_fun);
		}
	}
	instance_struct = (_tmp13_ = vala_ccode_struct_new (_tmp12_ = g_strdup_printf ("_%s", _tmp11_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE))), _g_free0 (_tmp12_), _g_free0 (_tmp11_), _tmp13_);
	{
		GeeList* _tmp14_;
		GeeIterator* _tmp15_;
		GeeIterator* _f_it;
		_f_it = (_tmp15_ = gee_iterable_iterator ((GeeIterable*) (_tmp14_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp14_), _tmp15_);
		while (TRUE) {
			ValaField* f;
			char* field_ctype;
			if (!gee_iterator_next (_f_it)) {
				break;
			}
			f = (ValaField*) gee_iterator_get (_f_it);
			field_ctype = vala_data_type_get_cname (vala_field_get_field_type (f));
			if (vala_field_get_is_volatile (f)) {
				char* _tmp16_;
				field_ctype = (_tmp16_ = g_strconcat ("volatile ", field_ctype, NULL), _g_free0 (field_ctype), _tmp16_);
			}
			if (vala_field_get_binding (f) == MEMBER_BINDING_INSTANCE) {
				char* _tmp19_;
				char* _tmp18_;
				char* _tmp17_;
				gboolean _tmp20_;
				vala_ccode_base_module_generate_type_declaration ((ValaCCodeBaseModule*) self, vala_field_get_field_type (f), decl_space);
				vala_ccode_struct_add_field (instance_struct, field_ctype, _tmp19_ = g_strconcat (_tmp17_ = vala_field_get_cname (f), _tmp18_ = vala_data_type_get_cdeclarator_suffix (vala_field_get_field_type (f)), NULL));
				_g_free0 (_tmp19_);
				_g_free0 (_tmp18_);
				_g_free0 (_tmp17_);
				_tmp20_ = FALSE;
				if (VALA_IS_ARRAY_TYPE (vala_field_get_field_type (f))) {
					_tmp20_ = !vala_field_get_no_array_length (f);
				} else {
					_tmp20_ = FALSE;
				}
				if (_tmp20_) {
					ValaArrayType* array_type;
					array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_field_get_field_type (f)));
					if (!vala_array_type_get_fixed_length (array_type)) {
						ValaDataType* len_type;
						gboolean _tmp24_;
						len_type = vala_data_type_copy (((ValaCCodeBaseModule*) self)->int_type);
						{
							gint dim;
							dim = 1;
							{
								gboolean _tmp21_;
								_tmp21_ = TRUE;
								while (TRUE) {
									char* _tmp23_;
									char* _tmp22_;
									if (!_tmp21_) {
										dim++;
									}
									_tmp21_ = FALSE;
									if (!(dim <= vala_array_type_get_rank (array_type))) {
										break;
									}
									vala_ccode_struct_add_field (instance_struct, _tmp22_ = vala_data_type_get_cname (len_type), _tmp23_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) f), dim));
									_g_free0 (_tmp23_);
									_g_free0 (_tmp22_);
								}
							}
						}
						_tmp24_ = FALSE;
						if (vala_array_type_get_rank (array_type) == 1) {
							_tmp24_ = vala_symbol_is_internal_symbol ((ValaSymbol*) f);
						} else {
							_tmp24_ = FALSE;
						}
						if (_tmp24_) {
							char* _tmp26_;
							char* _tmp25_;
							vala_ccode_struct_add_field (instance_struct, _tmp25_ = vala_data_type_get_cname (len_type), _tmp26_ = vala_ccode_module_get_array_size_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) f)));
							_g_free0 (_tmp26_);
							_g_free0 (_tmp25_);
						}
						_vala_code_node_unref0 (len_type);
					}
					_vala_code_node_unref0 (array_type);
				} else {
					if (VALA_IS_DELEGATE_TYPE (vala_field_get_field_type (f))) {
						ValaDelegateType* delegate_type;
						delegate_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_field_get_field_type (f)));
						if (vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type))) {
							char* _tmp27_;
							vala_ccode_struct_add_field (instance_struct, "gpointer", _tmp27_ = vala_ccode_base_module_get_delegate_target_cname ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) f)));
							_g_free0 (_tmp27_);
						}
						_vala_code_node_unref0 (delegate_type);
					}
				}
			}
			_vala_code_node_unref0 (f);
			_g_free0 (field_ctype);
		}
		_gee_collection_object_unref0 (_f_it);
	}
	vala_ccode_declaration_space_add_type_declaration (decl_space, (ValaCCodeNode*) (_tmp32_ = vala_ccode_type_definition_new (_tmp29_ = g_strdup_printf ("struct _%s", _tmp28_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE)), (ValaCCodeDeclarator*) (_tmp31_ = vala_ccode_variable_declarator_new (_tmp30_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), NULL, NULL)))));
	_vala_ccode_node_unref0 (_tmp32_);
	_vala_ccode_node_unref0 (_tmp31_);
	_g_free0 (_tmp30_);
	_g_free0 (_tmp29_);
	_g_free0 (_tmp28_);
	vala_ccode_declaration_space_add_type_definition (decl_space, (ValaCCodeNode*) instance_struct);
	function = (_tmp36_ = vala_ccode_function_new (_tmp33_ = vala_typesymbol_get_dup_function ((ValaTypeSymbol*) st), _tmp35_ = g_strconcat (_tmp34_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)), _g_free0 (_tmp35_), _g_free0 (_tmp34_), _g_free0 (_tmp33_), _tmp36_);
	if (vala_symbol_is_private_symbol ((ValaSymbol*) st)) {
		vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_function_add_parameter (function, _tmp40_ = vala_ccode_formal_parameter_new ("self", _tmp39_ = g_strconcat (_tmp38_ = g_strconcat ("const ", _tmp37_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), NULL), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp40_);
	_g_free0 (_tmp39_);
	_g_free0 (_tmp38_);
	_g_free0 (_tmp37_);
	vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) function);
	function = (_tmp42_ = vala_ccode_function_new (_tmp41_ = vala_typesymbol_get_free_function ((ValaTypeSymbol*) st), "void"), _vala_ccode_node_unref0 (function), _tmp42_);
	_g_free0 (_tmp41_);
	if (vala_symbol_is_private_symbol ((ValaSymbol*) st)) {
		vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_function_add_parameter (function, _tmp45_ = vala_ccode_formal_parameter_new ("self", _tmp44_ = g_strconcat (_tmp43_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp45_);
	_g_free0 (_tmp44_);
	_g_free0 (_tmp43_);
	vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) function);
	if (vala_struct_is_disposable (st)) {
		ValaCCodeFunction* _tmp47_;
		char* _tmp46_;
		ValaCCodeFormalParameter* _tmp51_;
		char* _tmp50_;
		char* _tmp49_;
		char* _tmp48_;
		ValaCCodeFormalParameter* _tmp54_;
		char* _tmp53_;
		char* _tmp52_;
		ValaCCodeFunction* _tmp56_;
		char* _tmp55_;
		ValaCCodeFormalParameter* _tmp59_;
		char* _tmp58_;
		char* _tmp57_;
		function = (_tmp47_ = vala_ccode_function_new (_tmp46_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st), "void"), _vala_ccode_node_unref0 (function), _tmp47_);
		_g_free0 (_tmp46_);
		if (vala_symbol_is_private_symbol ((ValaSymbol*) st)) {
			vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
		}
		vala_ccode_function_add_parameter (function, _tmp51_ = vala_ccode_formal_parameter_new ("self", _tmp50_ = g_strconcat (_tmp49_ = g_strconcat ("const ", _tmp48_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), NULL), "*", NULL)));
		_vala_ccode_node_unref0 (_tmp51_);
		_g_free0 (_tmp50_);
		_g_free0 (_tmp49_);
		_g_free0 (_tmp48_);
		vala_ccode_function_add_parameter (function, _tmp54_ = vala_ccode_formal_parameter_new ("dest", _tmp53_ = g_strconcat (_tmp52_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
		_vala_ccode_node_unref0 (_tmp54_);
		_g_free0 (_tmp53_);
		_g_free0 (_tmp52_);
		vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) function);
		function = (_tmp56_ = vala_ccode_function_new (_tmp55_ = vala_typesymbol_get_destroy_function ((ValaTypeSymbol*) st), "void"), _vala_ccode_node_unref0 (function), _tmp56_);
		_g_free0 (_tmp55_);
		if (vala_symbol_is_private_symbol ((ValaSymbol*) st)) {
			vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
		}
		vala_ccode_function_add_parameter (function, _tmp59_ = vala_ccode_formal_parameter_new ("self", _tmp58_ = g_strconcat (_tmp57_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
		_vala_ccode_node_unref0 (_tmp59_);
		_g_free0 (_tmp58_);
		_g_free0 (_tmp57_);
		vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) function);
	}
	_vala_ccode_node_unref0 (instance_struct);
	_vala_ccode_node_unref0 (function);
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


static void vala_ccode_struct_module_real_visit_struct (ValaCCodeModule* base, ValaStruct* st) {
	ValaCCodeStructModule * self;
	ValaSymbol* old_symbol;
	ValaCCodeFragment* old_instance_finalize_fragment;
	ValaSymbol* _tmp0_;
	ValaCCodeFragment* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	ValaSymbol* _tmp5_;
	ValaCCodeFragment* _tmp6_;
	self = (ValaCCodeStructModule*) base;
	g_return_if_fail (st != NULL);
	old_symbol = _vala_code_node_ref0 (((ValaCCodeBaseModule*) self)->current_symbol);
	old_instance_finalize_fragment = _vala_ccode_node_ref0 (((ValaCCodeBaseModule*) self)->instance_finalize_fragment);
	((ValaCCodeBaseModule*) self)->current_symbol = (_tmp0_ = _vala_code_node_ref0 ((ValaSymbol*) st), _vala_code_node_unref0 (((ValaCCodeBaseModule*) self)->current_symbol), _tmp0_);
	((ValaCCodeBaseModule*) self)->instance_finalize_fragment = (_tmp1_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (((ValaCCodeBaseModule*) self)->instance_finalize_fragment), _tmp1_);
	vala_ccode_base_module_generate_struct_declaration ((ValaCCodeBaseModule*) self, st, ((ValaCCodeBaseModule*) self)->source_declarations);
	if (!vala_symbol_is_internal_symbol ((ValaSymbol*) st)) {
		vala_ccode_base_module_generate_struct_declaration ((ValaCCodeBaseModule*) self, st, ((ValaCCodeBaseModule*) self)->header_declarations);
	}
	if (!vala_symbol_is_private_symbol ((ValaSymbol*) st)) {
		vala_ccode_base_module_generate_struct_declaration ((ValaCCodeBaseModule*) self, st, ((ValaCCodeBaseModule*) self)->internal_header_declarations);
	}
	vala_code_node_accept_children ((ValaCodeNode*) st, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	if (vala_code_context_get_profile (vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self)) == VALA_PROFILE_GOBJECT) {
		_tmp4_ = !vala_struct_is_boolean_type (st);
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		_tmp3_ = !vala_struct_is_integer_type (st);
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		_tmp2_ = !vala_struct_is_floating_type (st);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		if (vala_struct_is_disposable (st)) {
			vala_ccode_struct_module_add_struct_copy_function (self, st);
			vala_ccode_struct_module_add_struct_destroy_function (self, st);
		}
		vala_ccode_struct_module_add_struct_dup_function (self, st);
		vala_ccode_struct_module_add_struct_free_function (self, st);
	}
	((ValaCCodeBaseModule*) self)->current_symbol = (_tmp5_ = _vala_code_node_ref0 (old_symbol), _vala_code_node_unref0 (((ValaCCodeBaseModule*) self)->current_symbol), _tmp5_);
	((ValaCCodeBaseModule*) self)->instance_finalize_fragment = (_tmp6_ = _vala_ccode_node_ref0 (old_instance_finalize_fragment), _vala_ccode_node_unref0 (((ValaCCodeBaseModule*) self)->instance_finalize_fragment), _tmp6_);
	_vala_code_node_unref0 (old_symbol);
	_vala_ccode_node_unref0 (old_instance_finalize_fragment);
}


static void vala_ccode_struct_module_add_struct_dup_function (ValaCCodeStructModule* self, ValaStruct* st) {
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	ValaCCodeFunction* _tmp3_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp7_;
	char* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	ValaCCodeBlock* cblock;
	char* _tmp9_;
	char* _tmp8_;
	ValaCCodeDeclaration* _tmp10_;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp11_;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeFunctionCall* _tmp13_;
	ValaCCodeFunctionCall* creation_call;
	ValaCCodeConstant* _tmp15_;
	char* _tmp14_;
	ValaCCodeConstant* _tmp16_;
	ValaCCodeExpressionStatement* _tmp19_;
	ValaCCodeAssignment* _tmp18_;
	ValaCCodeIdentifier* _tmp17_;
	ValaCCodeReturnStatement* _tmp36_;
	ValaCCodeIdentifier* _tmp35_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	function = (_tmp3_ = vala_ccode_function_new (_tmp0_ = vala_typesymbol_get_dup_function ((ValaTypeSymbol*) st), _tmp2_ = g_strconcat (_tmp1_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)), _g_free0 (_tmp2_), _g_free0 (_tmp1_), _g_free0 (_tmp0_), _tmp3_);
	if (vala_symbol_get_access ((ValaSymbol*) st) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
		vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_function_add_parameter (function, _tmp7_ = vala_ccode_formal_parameter_new ("self", _tmp6_ = g_strconcat (_tmp5_ = g_strconcat ("const ", _tmp4_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), NULL), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp7_);
	_g_free0 (_tmp6_);
	_g_free0 (_tmp5_);
	_g_free0 (_tmp4_);
	cblock = vala_ccode_block_new ();
	_cdecl_ = (_tmp10_ = vala_ccode_declaration_new (_tmp9_ = g_strconcat (_tmp8_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)), _g_free0 (_tmp9_), _g_free0 (_tmp8_), _tmp10_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp11_ = vala_ccode_variable_declarator_new ("dup", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) _cdecl_);
	creation_call = (_tmp13_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("g_new0"))), _vala_ccode_node_unref0 (_tmp12_), _tmp13_);
	vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp15_ = vala_ccode_constant_new (_tmp14_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE))));
	_vala_ccode_node_unref0 (_tmp15_);
	_g_free0 (_tmp14_);
	vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_constant_new ("1")));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp19_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("dup")), (ValaCCodeExpression*) creation_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp19_);
	_vala_ccode_node_unref0 (_tmp18_);
	_vala_ccode_node_unref0 (_tmp17_);
	if (vala_struct_is_disposable (st)) {
		ValaCCodeIdentifier* _tmp21_;
		char* _tmp20_;
		ValaCCodeFunctionCall* _tmp22_;
		ValaCCodeFunctionCall* copy_call;
		ValaCCodeIdentifier* _tmp23_;
		ValaCCodeIdentifier* _tmp24_;
		ValaCCodeExpressionStatement* _tmp25_;
		copy_call = (_tmp22_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new (_tmp20_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st)))), _vala_ccode_node_unref0 (_tmp21_), _g_free0 (_tmp20_), _tmp22_);
		vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new ("self")));
		_vala_ccode_node_unref0 (_tmp23_);
		vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("dup")));
		_vala_ccode_node_unref0 (_tmp24_);
		vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp25_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) copy_call)));
		_vala_ccode_node_unref0 (_tmp25_);
		_vala_ccode_node_unref0 (copy_call);
	} else {
		ValaCCodeIdentifier* _tmp26_;
		ValaCCodeFunctionCall* _tmp27_;
		ValaCCodeFunctionCall* sizeof_call;
		ValaCCodeConstant* _tmp29_;
		char* _tmp28_;
		ValaCCodeIdentifier* _tmp30_;
		ValaCCodeFunctionCall* _tmp31_;
		ValaCCodeFunctionCall* copy_call;
		ValaCCodeIdentifier* _tmp32_;
		ValaCCodeIdentifier* _tmp33_;
		ValaCCodeExpressionStatement* _tmp34_;
		vala_ccode_declaration_space_add_include (((ValaCCodeBaseModule*) self)->source_declarations, "string.h", FALSE);
		sizeof_call = (_tmp27_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp26_), _tmp27_);
		vala_ccode_function_call_add_argument (sizeof_call, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_constant_new (_tmp28_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE))));
		_vala_ccode_node_unref0 (_tmp29_);
		_g_free0 (_tmp28_);
		copy_call = (_tmp31_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("memcpy"))), _vala_ccode_node_unref0 (_tmp30_), _tmp31_);
		vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp32_ = vala_ccode_identifier_new ("dup")));
		_vala_ccode_node_unref0 (_tmp32_);
		vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("self")));
		_vala_ccode_node_unref0 (_tmp33_);
		vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) sizeof_call);
		vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp34_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) copy_call)));
		_vala_ccode_node_unref0 (_tmp34_);
		_vala_ccode_node_unref0 (sizeof_call);
		_vala_ccode_node_unref0 (copy_call);
	}
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp36_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new ("dup")))));
	_vala_ccode_node_unref0 (_tmp36_);
	_vala_ccode_node_unref0 (_tmp35_);
	vala_ccode_function_set_block (function, cblock);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (cblock);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (creation_call);
}


static void vala_ccode_struct_module_add_struct_free_function (ValaCCodeStructModule* self, ValaStruct* st) {
	char* _tmp0_;
	ValaCCodeFunction* _tmp1_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	ValaCCodeBlock* cblock;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeFunctionCall* _tmp11_;
	ValaCCodeFunctionCall* free_call;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeExpressionStatement* _tmp13_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	function = (_tmp1_ = vala_ccode_function_new (_tmp0_ = vala_typesymbol_get_free_function ((ValaTypeSymbol*) st), "void"), _g_free0 (_tmp0_), _tmp1_);
	if (vala_symbol_get_access ((ValaSymbol*) st) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
		vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_function_add_parameter (function, _tmp4_ = vala_ccode_formal_parameter_new ("self", _tmp3_ = g_strconcat (_tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp2_);
	cblock = vala_ccode_block_new ();
	if (vala_struct_is_disposable (st)) {
		ValaCCodeIdentifier* _tmp6_;
		char* _tmp5_;
		ValaCCodeFunctionCall* _tmp7_;
		ValaCCodeFunctionCall* destroy_call;
		ValaCCodeIdentifier* _tmp8_;
		ValaCCodeExpressionStatement* _tmp9_;
		destroy_call = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new (_tmp5_ = vala_typesymbol_get_destroy_function ((ValaTypeSymbol*) st)))), _vala_ccode_node_unref0 (_tmp6_), _g_free0 (_tmp5_), _tmp7_);
		vala_ccode_function_call_add_argument (destroy_call, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("self")));
		_vala_ccode_node_unref0 (_tmp8_);
		vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp9_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) destroy_call)));
		_vala_ccode_node_unref0 (_tmp9_);
		_vala_ccode_node_unref0 (destroy_call);
	}
	free_call = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp10_), _tmp11_);
	vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp13_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
	_vala_ccode_node_unref0 (_tmp13_);
	vala_ccode_function_set_block (function, cblock);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (cblock);
	_vala_ccode_node_unref0 (free_call);
}


static void vala_ccode_struct_module_add_struct_copy_function (ValaCCodeStructModule* self, ValaStruct* st) {
	char* _tmp0_;
	ValaCCodeFunction* _tmp1_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	ValaCCodeFormalParameter* _tmp8_;
	char* _tmp7_;
	char* _tmp6_;
	ValaCCodeBlock* cblock;
	ValaCCodeFragment* cfrag;
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	function = (_tmp1_ = vala_ccode_function_new (_tmp0_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st), "void"), _g_free0 (_tmp0_), _tmp1_);
	if (vala_symbol_get_access ((ValaSymbol*) st) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
		vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("self", _tmp4_ = g_strconcat (_tmp3_ = g_strconcat ("const ", _tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), NULL), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp2_);
	vala_ccode_function_add_parameter (function, _tmp8_ = vala_ccode_formal_parameter_new ("dest", _tmp7_ = g_strconcat (_tmp6_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp8_);
	_g_free0 (_tmp7_);
	_g_free0 (_tmp6_);
	cblock = vala_ccode_block_new ();
	cfrag = vala_ccode_fragment_new ();
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) cfrag);
	{
		GeeList* _tmp9_;
		GeeIterator* _tmp10_;
		GeeIterator* _f_it;
		_f_it = (_tmp10_ = gee_iterable_iterator ((GeeIterable*) (_tmp9_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp9_), _tmp10_);
		while (TRUE) {
			ValaField* f;
			if (!gee_iterator_next (_f_it)) {
				break;
			}
			f = (ValaField*) gee_iterator_get (_f_it);
			if (vala_field_get_binding (f) == MEMBER_BINDING_INSTANCE) {
				ValaCCodeIdentifier* _tmp11_;
				ValaCCodeExpression* _tmp12_;
				ValaCCodeExpression* copy;
				ValaCCodeIdentifier* _tmp16_;
				ValaCCodeMemberAccess* _tmp17_;
				ValaCCodeMemberAccess* dest;
				ValaDataType* _tmp18_;
				ValaArrayType* array_type;
				gboolean _tmp19_;
				copy = (_tmp12_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new ("self")), vala_symbol_get_name ((ValaSymbol*) f)), _vala_ccode_node_unref0 (_tmp11_), _tmp12_);
				if (vala_ccode_base_module_requires_copy ((ValaCCodeBaseModule*) self, vala_field_get_field_type (f))) {
					ValaMemberAccess* this_access;
					ValaDataType* _tmp13_;
					ValaCCodeIdentifier* _tmp14_;
					ValaMemberAccess* ma;
					ValaCCodeExpression* _tmp15_;
					this_access = vala_member_access_new_simple ("this", NULL);
					vala_expression_set_value_type ((ValaExpression*) this_access, _tmp13_ = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) f))));
					_vala_code_node_unref0 (_tmp13_);
					vala_code_node_set_ccodenode ((ValaCodeNode*) this_access, (ValaCCodeNode*) (_tmp14_ = vala_ccode_identifier_new ("(*self)")));
					_vala_ccode_node_unref0 (_tmp14_);
					ma = vala_member_access_new ((ValaExpression*) this_access, vala_symbol_get_name ((ValaSymbol*) f), NULL);
					vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) f);
					copy = (_tmp15_ = vala_ccode_base_module_get_ref_cexpression ((ValaCCodeBaseModule*) self, vala_field_get_field_type (f), copy, (ValaExpression*) ma, (ValaCodeNode*) f), _vala_ccode_node_unref0 (copy), _tmp15_);
					_vala_code_node_unref0 (this_access);
					_vala_code_node_unref0 (ma);
				}
				dest = (_tmp17_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new ("dest")), vala_symbol_get_name ((ValaSymbol*) f)), _vala_ccode_node_unref0 (_tmp16_), _tmp17_);
				array_type = _vala_code_node_ref0 ((_tmp18_ = vala_field_get_field_type (f), VALA_IS_ARRAY_TYPE (_tmp18_) ? ((ValaArrayType*) _tmp18_) : NULL));
				_tmp19_ = FALSE;
				if (array_type != NULL) {
					_tmp19_ = vala_array_type_get_fixed_length (array_type);
				} else {
					_tmp19_ = FALSE;
				}
				if (_tmp19_) {
					ValaCCodeIdentifier* _tmp20_;
					ValaCCodeFunctionCall* _tmp21_;
					ValaCCodeFunctionCall* sizeof_call;
					ValaCCodeIdentifier* _tmp23_;
					char* _tmp22_;
					ValaCCodeConstant* _tmp25_;
					char* _tmp24_;
					ValaCCodeBinaryExpression* _tmp26_;
					ValaCCodeBinaryExpression* size;
					ValaCCodeIdentifier* _tmp27_;
					ValaCCodeFunctionCall* _tmp28_;
					ValaCCodeFunctionCall* array_copy_call;
					ValaCCodeExpressionStatement* _tmp29_;
					vala_ccode_declaration_space_add_include (((ValaCCodeBaseModule*) self)->source_declarations, "string.h", FALSE);
					sizeof_call = (_tmp21_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp20_), _tmp21_);
					vala_ccode_function_call_add_argument (sizeof_call, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new (_tmp22_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
					_vala_ccode_node_unref0 (_tmp23_);
					_g_free0 (_tmp22_);
					size = (_tmp26_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_constant_new (_tmp24_ = g_strdup_printf ("%d", vala_array_type_get_length (array_type)))), (ValaCCodeExpression*) sizeof_call), _vala_ccode_node_unref0 (_tmp25_), _g_free0 (_tmp24_), _tmp26_);
					array_copy_call = (_tmp28_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("memcpy"))), _vala_ccode_node_unref0 (_tmp27_), _tmp28_);
					vala_ccode_function_call_add_argument (array_copy_call, (ValaCCodeExpression*) dest);
					vala_ccode_function_call_add_argument (array_copy_call, copy);
					vala_ccode_function_call_add_argument (array_copy_call, (ValaCCodeExpression*) size);
					vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp29_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) array_copy_call)));
					_vala_ccode_node_unref0 (_tmp29_);
					_vala_ccode_node_unref0 (sizeof_call);
					_vala_ccode_node_unref0 (size);
					_vala_ccode_node_unref0 (array_copy_call);
				} else {
					ValaCCodeExpressionStatement* _tmp31_;
					ValaCCodeAssignment* _tmp30_;
					vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp31_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_assignment_new ((ValaCCodeExpression*) dest, copy, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
					_vala_ccode_node_unref0 (_tmp31_);
					_vala_ccode_node_unref0 (_tmp30_);
					if (array_type != NULL) {
						{
							gint dim;
							dim = 1;
							{
								gboolean _tmp32_;
								_tmp32_ = TRUE;
								while (TRUE) {
									char* _tmp34_;
									ValaCCodeIdentifier* _tmp33_;
									ValaCCodeMemberAccess* _tmp35_;
									ValaCCodeMemberAccess* len_src;
									char* _tmp37_;
									ValaCCodeIdentifier* _tmp36_;
									ValaCCodeMemberAccess* _tmp38_;
									ValaCCodeMemberAccess* len_dest;
									ValaCCodeExpressionStatement* _tmp40_;
									ValaCCodeAssignment* _tmp39_;
									if (!_tmp32_) {
										dim++;
									}
									_tmp32_ = FALSE;
									if (!(dim <= vala_array_type_get_rank (array_type))) {
										break;
									}
									len_src = (_tmp35_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("self")), _tmp34_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, vala_symbol_get_name ((ValaSymbol*) f), dim)), _g_free0 (_tmp34_), _vala_ccode_node_unref0 (_tmp33_), _tmp35_);
									len_dest = (_tmp38_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp36_ = vala_ccode_identifier_new ("dest")), _tmp37_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, vala_symbol_get_name ((ValaSymbol*) f), dim)), _g_free0 (_tmp37_), _vala_ccode_node_unref0 (_tmp36_), _tmp38_);
									vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp40_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_assignment_new ((ValaCCodeExpression*) len_dest, (ValaCCodeExpression*) len_src, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
									_vala_ccode_node_unref0 (_tmp40_);
									_vala_ccode_node_unref0 (_tmp39_);
									_vala_ccode_node_unref0 (len_src);
									_vala_ccode_node_unref0 (len_dest);
								}
							}
						}
					}
				}
				_vala_ccode_node_unref0 (copy);
				_vala_ccode_node_unref0 (dest);
				_vala_code_node_unref0 (array_type);
			}
			_vala_code_node_unref0 (f);
		}
		_gee_collection_object_unref0 (_f_it);
	}
	vala_ccode_base_module_append_temp_decl ((ValaCCodeBaseModule*) self, cfrag, (GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars);
	gee_collection_clear ((GeeCollection*) ((ValaCCodeBaseModule*) self)->temp_vars);
	vala_ccode_function_set_block (function, cblock);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (cblock);
	_vala_ccode_node_unref0 (cfrag);
}


static void vala_ccode_struct_module_add_struct_destroy_function (ValaCCodeStructModule* self, ValaStruct* st) {
	char* _tmp0_;
	ValaCCodeFunction* _tmp1_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	ValaCCodeBlock* cblock;
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	function = (_tmp1_ = vala_ccode_function_new (_tmp0_ = vala_typesymbol_get_destroy_function ((ValaTypeSymbol*) st), "void"), _g_free0 (_tmp0_), _tmp1_);
	if (vala_symbol_get_access ((ValaSymbol*) st) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
		vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_function_add_parameter (function, _tmp4_ = vala_ccode_formal_parameter_new ("self", _tmp3_ = g_strconcat (_tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp2_);
	cblock = vala_ccode_block_new ();
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) ((ValaCCodeBaseModule*) self)->instance_finalize_fragment);
	vala_ccode_function_set_block (function, cblock);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (cblock);
}


static void vala_ccode_struct_module_class_init (ValaCCodeStructModuleClass * klass) {
	vala_ccode_struct_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_struct_declaration = vala_ccode_struct_module_real_generate_struct_declaration;
	VALA_CCODE_MODULE_CLASS (klass)->visit_struct = vala_ccode_struct_module_real_visit_struct;
}


static void vala_ccode_struct_module_instance_init (ValaCCodeStructModule * self) {
}


GType vala_ccode_struct_module_get_type (void) {
	static GType vala_ccode_struct_module_type_id = 0;
	if (vala_ccode_struct_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeStructModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_struct_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeStructModule), 0, (GInstanceInitFunc) vala_ccode_struct_module_instance_init, NULL };
		vala_ccode_struct_module_type_id = g_type_register_static (VALA_TYPE_CCODE_BASE_MODULE, "ValaCCodeStructModule", &g_define_type_info, 0);
	}
	return vala_ccode_struct_module_type_id;
}




