/* valacompiler.vala
 *
 * Copyright (C) 2006-2009  Jürg Billeter
 * Copyright (C) 1996-2002, 2004, 2005, 2006 Free Software Foundation, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <glib/gstdio.h>
#include <valacodegen.h>
#include <config.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_COMPILER (vala_compiler_get_type ())
#define VALA_COMPILER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_COMPILER, ValaCompiler))
#define VALA_COMPILER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_COMPILER, ValaCompilerClass))
#define VALA_IS_COMPILER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_COMPILER))
#define VALA_IS_COMPILER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_COMPILER))
#define VALA_COMPILER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_COMPILER, ValaCompilerClass))

typedef struct _ValaCompiler ValaCompiler;
typedef struct _ValaCompilerClass ValaCompilerClass;
typedef struct _ValaCompilerPrivate ValaCompilerPrivate;
#define _vala_code_context_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_context_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_source_file_unref0(var) ((var == NULL) ? NULL : (var = (vala_source_file_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _vala_code_visitor_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_visitor_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _vala_ccode_compiler_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_compiler_unref (var), NULL)))
#define _g_option_context_free0(var) ((var == NULL) ? NULL : (var = (g_option_context_free (var), NULL)))
#define _vala_compiler_unref0(var) ((var == NULL) ? NULL : (var = (vala_compiler_unref (var), NULL)))
typedef struct _ValaParamSpecCompiler ValaParamSpecCompiler;

struct _ValaCompiler {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCompilerPrivate * priv;
};

struct _ValaCompilerClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCompiler *self);
};

struct _ValaCompilerPrivate {
	ValaCodeContext* context;
};

struct _ValaParamSpecCompiler {
	GParamSpec parent_instance;
};


static char* vala_compiler_basedir;
static char* vala_compiler_basedir = NULL;
static char* vala_compiler_directory;
static char* vala_compiler_directory = NULL;
static gboolean vala_compiler_version;
static gboolean vala_compiler_version = FALSE;
static char** vala_compiler_sources;
static char** vala_compiler_sources = NULL;
static char** vala_compiler_vapi_directories;
static char** vala_compiler_vapi_directories = NULL;
static char* vala_compiler_vapi_filename;
static char* vala_compiler_vapi_filename = NULL;
static char* vala_compiler_library;
static char* vala_compiler_library = NULL;
static char* vala_compiler_gir;
static char* vala_compiler_gir = NULL;
static char** vala_compiler_packages;
static char** vala_compiler_packages = NULL;
static char* vala_compiler_target_glib;
static char* vala_compiler_target_glib = NULL;
static gboolean vala_compiler_ccode_only;
static gboolean vala_compiler_ccode_only = FALSE;
static char* vala_compiler_header_filename;
static char* vala_compiler_header_filename = NULL;
static char* vala_compiler_internal_header_filename;
static char* vala_compiler_internal_header_filename = NULL;
static char* vala_compiler_internal_vapi_filename;
static char* vala_compiler_internal_vapi_filename = NULL;
static char* vala_compiler_includedir;
static char* vala_compiler_includedir = NULL;
static gboolean vala_compiler_compile_only;
static gboolean vala_compiler_compile_only = FALSE;
static char* vala_compiler_output;
static char* vala_compiler_output = NULL;
static gboolean vala_compiler_debug;
static gboolean vala_compiler_debug = FALSE;
static gboolean vala_compiler_thread;
static gboolean vala_compiler_thread = FALSE;
static gboolean vala_compiler_disable_assert;
static gboolean vala_compiler_disable_assert = FALSE;
static gboolean vala_compiler_enable_checking;
static gboolean vala_compiler_enable_checking = FALSE;
static gboolean vala_compiler_deprecated;
static gboolean vala_compiler_deprecated = FALSE;
static gboolean vala_compiler_experimental;
static gboolean vala_compiler_experimental = FALSE;
static gboolean vala_compiler_non_null_experimental;
static gboolean vala_compiler_non_null_experimental = FALSE;
static gboolean vala_compiler_disable_dbus_transformation;
static gboolean vala_compiler_disable_dbus_transformation = FALSE;
static char* vala_compiler_cc_command;
static char* vala_compiler_cc_command = NULL;
static char** vala_compiler_cc_options;
static char** vala_compiler_cc_options = NULL;
static char* vala_compiler_dump_tree;
static char* vala_compiler_dump_tree = NULL;
static gboolean vala_compiler_save_temps;
static gboolean vala_compiler_save_temps = FALSE;
static char** vala_compiler_defines;
static char** vala_compiler_defines = NULL;
static gboolean vala_compiler_quiet_mode;
static gboolean vala_compiler_quiet_mode = FALSE;
static gboolean vala_compiler_verbose_mode;
static gboolean vala_compiler_verbose_mode = FALSE;
static char* vala_compiler_profile;
static char* vala_compiler_profile = NULL;
static gpointer vala_compiler_parent_class = NULL;

gpointer vala_compiler_ref (gpointer instance);
void vala_compiler_unref (gpointer instance);
GParamSpec* vala_param_spec_compiler (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_compiler (GValue* value, gpointer v_object);
gpointer vala_value_get_compiler (const GValue* value);
GType vala_compiler_get_type (void);
#define VALA_COMPILER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_COMPILER, ValaCompilerPrivate))
enum  {
	VALA_COMPILER_DUMMY_PROPERTY
};
static gint vala_compiler_quit (ValaCompiler* self);
static gboolean vala_compiler_add_package (ValaCompiler* self, ValaCodeContext* context, const char* pkg);
static char* vala_compiler_realpath (const char* name);
static gint vala_compiler_run (ValaCompiler* self);
static gboolean vala_compiler_ends_with_dir_separator (const char* s);
ValaCompiler* vala_compiler_new (void);
ValaCompiler* vala_compiler_construct (GType object_type);
static gint vala_compiler_main (char** args, int args_length1);
static void vala_compiler_finalize (ValaCompiler* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GOptionEntry VALA_COMPILER_options[] = {{"vapidir", (gchar) 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_compiler_vapi_directories, "Look for package bindings in DIRECTORY", "DIRECTORY..."}, {"pkg", (gchar) 0, 0, G_OPTION_ARG_STRING_ARRAY, &vala_compiler_packages, "Include binding for PACKAGE", "PACKAGE..."}, {"vapi", (gchar) 0, 0, G_OPTION_ARG_FILENAME, &vala_compiler_vapi_filename, "Output VAPI file name", "FILE"}, {"library", (gchar) 0, 0, G_OPTION_ARG_STRING, &vala_compiler_library, "Library name", "NAME"}, {"gir", (gchar) 0, 0, G_OPTION_ARG_STRING, &vala_compiler_gir, "GObject-Introspection repository file name", "NAME-VERSION.gir"}, {"basedir", 'b', 0, G_OPTION_ARG_FILENAME, &vala_compiler_basedir, "Base source directory", "DIRECTORY"}, {"directory", 'd', 0, G_OPTION_ARG_FILENAME, &vala_compiler_directory, "Output directory", "DIRECTORY"}, {"version", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_version, "Display version number", NULL}, {"ccode", 'C', 0, G_OPTION_ARG_NONE, &vala_compiler_ccode_only, "Output C code", NULL}, {"header", 'H', 0, G_OPTION_ARG_FILENAME, &vala_compiler_header_filename, "Output C header file", "FILE"}, {"includedir", (gchar) 0, 0, G_OPTION_ARG_FILENAME, &vala_compiler_includedir, "Directory used to include the C header file", "DIRECTORY"}, {"internal-header", 'h', 0, G_OPTION_ARG_FILENAME, &vala_compiler_internal_header_filename, "Output internal C header file", "FILE"}, {"internal-vapi", (gchar) 0, 0, G_OPTION_ARG_FILENAME, &vala_compiler_internal_vapi_filename, "Output vapi with internal api", "FILE"}, {"compile", 'c', 0, G_OPTION_ARG_NONE, &vala_compiler_compile_only, "Compile but do not link", NULL}, {"output", 'o', 0, G_OPTION_ARG_FILENAME, &vala_compiler_output, "Place output in file FILE", "FILE"}, {"debug", 'g', 0, G_OPTION_ARG_NONE, &vala_compiler_debug, "Produce debug information", NULL}, {"thread", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_thread, "Enable multithreading support", NULL}, {"define", 'D', 0, G_OPTION_ARG_STRING_ARRAY, &vala_compiler_defines, "Define SYMBOL", "SYMBOL..."}, {"disable-assert", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_disable_assert, "Disable assertions", NULL}, {"enable-checking", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_enable_checking, "Enable additional run-time checks", NULL}, {"enable-deprecated", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_deprecated, "Enable deprecated features", NULL}, {"enable-experimental", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_experimental, "Enable experimental features", NULL}, {"enable-non-null-experimental", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_non_null_experimental, "Enable experimental enhancements for non-null types", NULL}, {"disable-dbus-transformation", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_disable_dbus_transformation, "Disable transformation of D-Bus member names", NULL}, {"cc", (gchar) 0, 0, G_OPTION_ARG_STRING, &vala_compiler_cc_command, "Use COMMAND as C compiler command", "COMMAND"}, {"Xcc", 'X', 0, G_OPTION_ARG_STRING_ARRAY, &vala_compiler_cc_options, "Pass OPTION to the C compiler", "OPTION..."}, {"dump-tree", (gchar) 0, 0, G_OPTION_ARG_FILENAME, &vala_compiler_dump_tree, "Write code tree to FILE", "FILE"}, {"save-temps", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_compiler_save_temps, "Keep temporary files", NULL}, {"profile", (gchar) 0, 0, G_OPTION_ARG_STRING, &vala_compiler_profile, "Use the given profile instead of the default", "PROFILE"}, {"quiet", 'q', 0, G_OPTION_ARG_NONE, &vala_compiler_quiet_mode, "Do not print messages to the console", NULL}, {"verbose", 'v', 0, G_OPTION_ARG_NONE, &vala_compiler_verbose_mode, "Print additional messages to the console", NULL}, {"target-glib", (gchar) 0, 0, G_OPTION_ARG_STRING, &vala_compiler_target_glib, "Target version of glib for code generation", "MAJOR.MINOR"}, {"", (gchar) 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_compiler_sources, NULL, "FILE..."}, {NULL}};


static gint vala_compiler_quit (ValaCompiler* self) {
	gint result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, 0);
	_tmp0_ = FALSE;
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) == 0) {
		_tmp0_ = vala_report_get_warnings (vala_code_context_get_report (self->priv->context)) == 0;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = 0;
		return result;
	}
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) == 0) {
		if (!vala_compiler_quiet_mode) {
			fprintf (stdout, "Compilation succeeded - %d warning(s)\n", vala_report_get_warnings (vala_code_context_get_report (self->priv->context)));
		}
		result = 0;
		return result;
	} else {
		if (!vala_compiler_quiet_mode) {
			fprintf (stdout, "Compilation failed: %d error(s), %d warning(s)\n", vala_report_get_errors (vala_code_context_get_report (self->priv->context)), vala_report_get_warnings (vala_code_context_get_report (self->priv->context)));
		}
		result = 1;
		return result;
	}
}


static char* string_strip (const char* self) {
	char* result;
	char* _result_;
	g_return_val_if_fail (self != NULL, NULL);
	_result_ = g_strdup (self);
	g_strstrip (_result_);
	result = _result_;
	return result;
}


static gboolean vala_compiler_add_package (ValaCompiler* self, ValaCodeContext* context, const char* pkg) {
	gboolean result;
	GError * _inner_error_;
	char* package_path;
	ValaSourceFile* _tmp0_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp3_;
	char* deps_filename;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (context != NULL, FALSE);
	g_return_val_if_fail (pkg != NULL, FALSE);
	_inner_error_ = NULL;
	if (vala_code_context_has_package (context, pkg)) {
		result = TRUE;
		return result;
	}
	package_path = vala_code_context_get_package_path (context, pkg, vala_compiler_vapi_directories, _vala_array_length (vala_compiler_vapi_directories));
	if (package_path == NULL) {
		result = FALSE;
		_g_free0 (package_path);
		return result;
	}
	vala_code_context_add_package (context, pkg);
	vala_code_context_add_source_file (context, _tmp0_ = vala_source_file_new (context, package_path, TRUE, NULL));
	_vala_source_file_unref0 (_tmp0_);
	deps_filename = (_tmp3_ = g_build_filename (_tmp1_ = g_path_get_dirname (package_path), _tmp2_ = g_strdup_printf ("%s.deps", pkg), NULL), _g_free0 (_tmp2_), _g_free0 (_tmp1_), _tmp3_);
	if (g_file_test (deps_filename, G_FILE_TEST_EXISTS)) {
		{
			char* deps_content;
			gulong deps_len;
			char* _tmp6_;
			gboolean _tmp5_;
			char* _tmp4_;
			deps_content = NULL;
			deps_len = 0UL;
			_tmp4_ = NULL;
			_tmp5_ = g_file_get_contents (deps_filename, &_tmp4_, &deps_len, &_inner_error_);
			deps_content = (_tmp6_ = _tmp4_, _g_free0 (deps_content), _tmp6_);
			_tmp5_;
			if (_inner_error_ != NULL) {
				_g_free0 (deps_content);
				if (_inner_error_->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
			{
				char** _tmp7_;
				char** dep_collection;
				int dep_collection_length1;
				int dep_it;
				dep_collection = _tmp7_ = g_strsplit (deps_content, "\n", 0);
				dep_collection_length1 = _vala_array_length (_tmp7_);
				for (dep_it = 0; dep_it < _vala_array_length (_tmp7_); dep_it = dep_it + 1) {
					char* dep;
					dep = g_strdup (dep_collection[dep_it]);
					{
						char* _tmp8_;
						dep = (_tmp8_ = string_strip (dep), _g_free0 (dep), _tmp8_);
						if (_vala_strcmp0 (dep, "") != 0) {
							if (!vala_compiler_add_package (self, context, dep)) {
								char* _tmp9_;
								vala_report_error (NULL, _tmp9_ = g_strdup_printf ("%s, dependency of %s, not found in specified Vala API directories", dep, pkg));
								_g_free0 (_tmp9_);
							}
						}
						_g_free0 (dep);
					}
				}
				dep_collection = (_vala_array_free (dep_collection, dep_collection_length1, (GDestroyNotify) g_free), NULL);
			}
			_g_free0 (deps_content);
		}
		goto __finally0;
		__catch0_g_file_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				char* _tmp10_;
				vala_report_error (NULL, _tmp10_ = g_strdup_printf ("Unable to read dependency file: %s", e->message));
				_g_free0 (_tmp10_);
				_g_error_free0 (e);
			}
		}
		__finally0:
		if (_inner_error_ != NULL) {
			_g_free0 (package_path);
			_g_free0 (deps_filename);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return FALSE;
		}
	}
	result = TRUE;
	_g_free0 (package_path);
	_g_free0 (deps_filename);
	return result;
}


static char* string_substring (const char* self, glong offset, glong len) {
	char* result;
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


static gboolean string_contains (const char* self, const char* needle) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (needle != NULL, FALSE);
	result = strstr (self, needle) != NULL;
	return result;
}


static gint vala_compiler_run (ValaCompiler* self) {
	gint result;
	ValaCodeContext* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	ValaCCodeGenerator* _tmp12_;
	char** _tmp23_;
	ValaParser* parser;
	ValaGenieParser* genie_parser;
	ValaSymbolResolver* resolver;
	ValaSemanticAnalyzer* analyzer;
	gboolean _tmp24_;
	gboolean _tmp25_;
	ValaFlowAnalyzer* flow_analyzer;
	gboolean _tmp26_;
	g_return_val_if_fail (self != NULL, 0);
	self->priv->context = (_tmp0_ = vala_code_context_new (), _vala_code_context_unref0 (self->priv->context), _tmp0_);
	vala_code_context_push (self->priv->context);
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	if (!vala_compiler_ccode_only) {
		_tmp2_ = !vala_compiler_compile_only;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		_tmp1_ = vala_compiler_output == NULL;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		if (g_utf8_strrchr (vala_compiler_sources[0], (gssize) (-1), (gunichar) '.') != NULL) {
			glong dot;
			char* _tmp4_;
			char* _tmp3_;
			dot = g_utf8_pointer_to_offset (vala_compiler_sources[0], g_utf8_strrchr (vala_compiler_sources[0], (gssize) (-1), (gunichar) '.'));
			vala_compiler_output = (_tmp4_ = g_path_get_basename (_tmp3_ = string_substring (vala_compiler_sources[0], (glong) 0, dot)), _g_free0 (vala_compiler_output), _tmp4_);
			_g_free0 (_tmp3_);
		}
	}
	vala_code_context_set_assert (self->priv->context, !vala_compiler_disable_assert);
	vala_code_context_set_checking (self->priv->context, vala_compiler_enable_checking);
	vala_code_context_set_deprecated (self->priv->context, vala_compiler_deprecated);
	vala_code_context_set_experimental (self->priv->context, vala_compiler_experimental);
	vala_code_context_set_non_null_experimental (self->priv->context, vala_compiler_non_null_experimental);
	vala_code_context_set_dbus_transformation (self->priv->context, !vala_compiler_disable_dbus_transformation);
	vala_report_set_verbose_errors (vala_code_context_get_report (self->priv->context), !vala_compiler_quiet_mode);
	vala_code_context_set_verbose_mode (self->priv->context, vala_compiler_verbose_mode);
	vala_code_context_set_ccode_only (self->priv->context, vala_compiler_ccode_only);
	vala_code_context_set_compile_only (self->priv->context, vala_compiler_compile_only);
	vala_code_context_set_header_filename (self->priv->context, vala_compiler_header_filename);
	vala_code_context_set_internal_header_filename (self->priv->context, vala_compiler_internal_header_filename);
	vala_code_context_set_includedir (self->priv->context, vala_compiler_includedir);
	vala_code_context_set_output (self->priv->context, vala_compiler_output);
	if (vala_compiler_basedir == NULL) {
		char* _tmp5_;
		vala_code_context_set_basedir (self->priv->context, _tmp5_ = vala_compiler_realpath ("."));
		_g_free0 (_tmp5_);
	} else {
		char* _tmp6_;
		vala_code_context_set_basedir (self->priv->context, _tmp6_ = vala_compiler_realpath (vala_compiler_basedir));
		_g_free0 (_tmp6_);
	}
	if (vala_compiler_directory != NULL) {
		char* _tmp7_;
		vala_code_context_set_directory (self->priv->context, _tmp7_ = vala_compiler_realpath (vala_compiler_directory));
		_g_free0 (_tmp7_);
	} else {
		vala_code_context_set_directory (self->priv->context, vala_code_context_get_basedir (self->priv->context));
	}
	vala_code_context_set_debug (self->priv->context, vala_compiler_debug);
	vala_code_context_set_thread (self->priv->context, vala_compiler_thread);
	vala_code_context_set_save_temps (self->priv->context, vala_compiler_save_temps);
	if (_vala_strcmp0 (vala_compiler_profile, "posix") == 0) {
		vala_code_context_set_profile (self->priv->context, VALA_PROFILE_POSIX);
		vala_code_context_add_define (self->priv->context, "POSIX");
	} else {
		gboolean _tmp8_;
		gboolean _tmp9_;
		_tmp8_ = FALSE;
		_tmp9_ = FALSE;
		if (_vala_strcmp0 (vala_compiler_profile, "gobject-2.0") == 0) {
			_tmp9_ = TRUE;
		} else {
			_tmp9_ = _vala_strcmp0 (vala_compiler_profile, "gobject") == 0;
		}
		if (_tmp9_) {
			_tmp8_ = TRUE;
		} else {
			_tmp8_ = vala_compiler_profile == NULL;
		}
		if (_tmp8_) {
			vala_code_context_set_profile (self->priv->context, VALA_PROFILE_GOBJECT);
			vala_code_context_add_define (self->priv->context, "GOBJECT");
			vala_code_context_add_define (self->priv->context, "VALA_0_7_6_NEW_METHODS");
		} else {
			char* _tmp10_;
			vala_report_error (NULL, _tmp10_ = g_strdup_printf ("Unknown profile %s", vala_compiler_profile));
			_g_free0 (_tmp10_);
		}
	}
	if (vala_compiler_defines != NULL) {
		{
			char** define_collection;
			int define_collection_length1;
			int define_it;
			define_collection = vala_compiler_defines;
			define_collection_length1 = _vala_array_length (vala_compiler_defines);
			for (define_it = 0; define_it < _vala_array_length (vala_compiler_defines); define_it = define_it + 1) {
				char* define;
				define = g_strdup (define_collection[define_it]);
				{
					vala_code_context_add_define (self->priv->context, define);
					_g_free0 (define);
				}
			}
		}
	}
	if (vala_code_context_get_profile (self->priv->context) == VALA_PROFILE_POSIX) {
		if (!vala_compiler_add_package (self, self->priv->context, "posix")) {
			vala_report_error (NULL, "posix not found in specified Vala API directories");
		}
	} else {
		if (vala_code_context_get_profile (self->priv->context) == VALA_PROFILE_GOBJECT) {
			gint glib_major;
			gint glib_minor;
			gboolean _tmp11_;
			glib_major = 2;
			glib_minor = 12;
			_tmp11_ = FALSE;
			if (vala_compiler_target_glib != NULL) {
				_tmp11_ = sscanf (vala_compiler_target_glib, "%d.%d", &glib_major, &glib_minor) != 2;
			} else {
				_tmp11_ = FALSE;
			}
			if (_tmp11_) {
				vala_report_error (NULL, "Invalid format for --target-glib");
			}
			vala_code_context_set_target_glib_major (self->priv->context, glib_major);
			vala_code_context_set_target_glib_minor (self->priv->context, glib_minor);
			if (vala_code_context_get_target_glib_major (self->priv->context) != 2) {
				vala_report_error (NULL, "This version of valac only supports GLib 2");
			}
			if (!vala_compiler_add_package (self, self->priv->context, "glib-2.0")) {
				vala_report_error (NULL, "glib-2.0 not found in specified Vala API directories");
			}
			if (!vala_compiler_add_package (self, self->priv->context, "gobject-2.0")) {
				vala_report_error (NULL, "gobject-2.0 not found in specified Vala API directories");
			}
		}
	}
	vala_code_context_set_codegen (self->priv->context, (ValaCodeGenerator*) (_tmp12_ = vala_ccode_generator_new ()));
	_vala_code_visitor_unref0 (_tmp12_);
	if (vala_compiler_packages != NULL) {
		char** _tmp14_;
		{
			char** package_collection;
			int package_collection_length1;
			int package_it;
			package_collection = vala_compiler_packages;
			package_collection_length1 = _vala_array_length (vala_compiler_packages);
			for (package_it = 0; package_it < _vala_array_length (vala_compiler_packages); package_it = package_it + 1) {
				char* package;
				package = g_strdup (package_collection[package_it]);
				{
					if (!vala_compiler_add_package (self, self->priv->context, package)) {
						char* _tmp13_;
						vala_report_error (NULL, _tmp13_ = g_strdup_printf ("%s not found in specified Vala API directories", package));
						_g_free0 (_tmp13_);
					}
					_g_free0 (package);
				}
			}
		}
		vala_compiler_packages = (_tmp14_ = NULL, vala_compiler_packages = (_vala_array_free (vala_compiler_packages, _vala_array_length (vala_compiler_packages), (GDestroyNotify) g_free), NULL), _tmp14_);
	}
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
		result = vala_compiler_quit (self);
		return result;
	}
	{
		char** source_collection;
		int source_collection_length1;
		int source_it;
		source_collection = vala_compiler_sources;
		source_collection_length1 = _vala_array_length (vala_compiler_sources);
		for (source_it = 0; source_it < _vala_array_length (vala_compiler_sources); source_it = source_it + 1) {
			char* source;
			source = g_strdup (source_collection[source_it]);
			{
				if (g_file_test (source, G_FILE_TEST_EXISTS)) {
					char* rpath;
					gboolean _tmp15_;
					rpath = vala_compiler_realpath (source);
					_tmp15_ = FALSE;
					if (g_str_has_suffix (source, ".vala")) {
						_tmp15_ = TRUE;
					} else {
						_tmp15_ = g_str_has_suffix (source, ".gs");
					}
					if (_tmp15_) {
						ValaSourceFile* source_file;
						source_file = vala_source_file_new (self->priv->context, rpath, FALSE, NULL);
						if (vala_code_context_get_profile (self->priv->context) == VALA_PROFILE_POSIX) {
							ValaUsingDirective* _tmp17_;
							ValaUnresolvedSymbol* _tmp16_;
							vala_source_file_add_using_directive (source_file, _tmp17_ = vala_using_directive_new ((ValaSymbol*) (_tmp16_ = vala_unresolved_symbol_new (NULL, "Posix", NULL)), NULL));
							_vala_code_node_unref0 (_tmp17_);
							_vala_code_node_unref0 (_tmp16_);
						} else {
							if (vala_code_context_get_profile (self->priv->context) == VALA_PROFILE_GOBJECT) {
								ValaUsingDirective* _tmp19_;
								ValaUnresolvedSymbol* _tmp18_;
								vala_source_file_add_using_directive (source_file, _tmp19_ = vala_using_directive_new ((ValaSymbol*) (_tmp18_ = vala_unresolved_symbol_new (NULL, "GLib", NULL)), NULL));
								_vala_code_node_unref0 (_tmp19_);
								_vala_code_node_unref0 (_tmp18_);
							}
						}
						vala_code_context_add_source_file (self->priv->context, source_file);
						_vala_source_file_unref0 (source_file);
					} else {
						if (g_str_has_suffix (source, ".vapi")) {
							ValaSourceFile* _tmp20_;
							vala_code_context_add_source_file (self->priv->context, _tmp20_ = vala_source_file_new (self->priv->context, rpath, TRUE, NULL));
							_vala_source_file_unref0 (_tmp20_);
						} else {
							if (g_str_has_suffix (source, ".c")) {
								vala_code_context_add_c_source_file (self->priv->context, rpath);
							} else {
								char* _tmp21_;
								vala_report_error (NULL, _tmp21_ = g_strdup_printf ("%s is not a supported source file type. Only .vala, .vapi, .gs, and .c files are supported.", source));
								_g_free0 (_tmp21_);
							}
						}
					}
					_g_free0 (rpath);
				} else {
					char* _tmp22_;
					vala_report_error (NULL, _tmp22_ = g_strdup_printf ("%s not found", source));
					_g_free0 (_tmp22_);
				}
				_g_free0 (source);
			}
		}
	}
	vala_compiler_sources = (_tmp23_ = NULL, vala_compiler_sources = (_vala_array_free (vala_compiler_sources, _vala_array_length (vala_compiler_sources), (GDestroyNotify) g_free), NULL), _tmp23_);
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
		result = vala_compiler_quit (self);
		return result;
	}
	parser = vala_parser_new ();
	vala_parser_parse (parser, self->priv->context);
	genie_parser = vala_genie_parser_new ();
	vala_genie_parser_parse (genie_parser, self->priv->context);
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
		result = vala_compiler_quit (self);
		_vala_code_visitor_unref0 (parser);
		_vala_code_visitor_unref0 (genie_parser);
		return result;
	}
	resolver = vala_symbol_resolver_new ();
	vala_symbol_resolver_resolve (resolver, self->priv->context);
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
		result = vala_compiler_quit (self);
		_vala_code_visitor_unref0 (parser);
		_vala_code_visitor_unref0 (genie_parser);
		_vala_code_visitor_unref0 (resolver);
		return result;
	}
	analyzer = vala_semantic_analyzer_new ();
	vala_semantic_analyzer_analyze (analyzer, self->priv->context);
	_tmp24_ = FALSE;
	_tmp25_ = FALSE;
	if (!vala_compiler_ccode_only) {
		_tmp25_ = !vala_compiler_compile_only;
	} else {
		_tmp25_ = FALSE;
	}
	if (_tmp25_) {
		_tmp24_ = vala_compiler_library == NULL;
	} else {
		_tmp24_ = FALSE;
	}
	if (_tmp24_) {
		if (vala_code_context_get_entry_point (self->priv->context) == NULL) {
			vala_report_error (NULL, "program does not contain a static `main' method");
		}
	}
	if (vala_compiler_dump_tree != NULL) {
		ValaCodeWriter* code_writer;
		code_writer = vala_code_writer_new (TRUE, FALSE);
		vala_code_writer_write_file (code_writer, self->priv->context, vala_compiler_dump_tree);
		_vala_code_visitor_unref0 (code_writer);
	}
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
		result = vala_compiler_quit (self);
		_vala_code_visitor_unref0 (parser);
		_vala_code_visitor_unref0 (genie_parser);
		_vala_code_visitor_unref0 (resolver);
		_vala_code_visitor_unref0 (analyzer);
		return result;
	}
	flow_analyzer = vala_flow_analyzer_new ();
	vala_flow_analyzer_analyze (flow_analyzer, self->priv->context);
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
		result = vala_compiler_quit (self);
		_vala_code_visitor_unref0 (parser);
		_vala_code_visitor_unref0 (genie_parser);
		_vala_code_visitor_unref0 (resolver);
		_vala_code_visitor_unref0 (analyzer);
		_vala_code_visitor_unref0 (flow_analyzer);
		return result;
	}
	if (vala_code_context_get_non_null_experimental (self->priv->context)) {
		ValaNullChecker* null_checker;
		null_checker = vala_null_checker_new ();
		vala_null_checker_check (null_checker, self->priv->context);
		if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
			result = vala_compiler_quit (self);
			_vala_code_visitor_unref0 (null_checker);
			_vala_code_visitor_unref0 (parser);
			_vala_code_visitor_unref0 (genie_parser);
			_vala_code_visitor_unref0 (resolver);
			_vala_code_visitor_unref0 (analyzer);
			_vala_code_visitor_unref0 (flow_analyzer);
			return result;
		}
		_vala_code_visitor_unref0 (null_checker);
	}
	vala_code_generator_emit (vala_code_context_get_codegen (self->priv->context), self->priv->context);
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->context)) > 0) {
		result = vala_compiler_quit (self);
		_vala_code_visitor_unref0 (parser);
		_vala_code_visitor_unref0 (genie_parser);
		_vala_code_visitor_unref0 (resolver);
		_vala_code_visitor_unref0 (analyzer);
		_vala_code_visitor_unref0 (flow_analyzer);
		return result;
	}
	_tmp26_ = FALSE;
	if (vala_compiler_vapi_filename == NULL) {
		_tmp26_ = vala_compiler_library != NULL;
	} else {
		_tmp26_ = FALSE;
	}
	if (_tmp26_) {
		char* _tmp27_;
		vala_compiler_vapi_filename = (_tmp27_ = g_strdup_printf ("%s.vapi", vala_compiler_library), _g_free0 (vala_compiler_vapi_filename), _tmp27_);
	}
	if (vala_compiler_vapi_filename != NULL) {
		ValaCodeWriter* interface_writer;
		gboolean _tmp28_;
		interface_writer = vala_code_writer_new (FALSE, FALSE);
		_tmp28_ = FALSE;
		if (vala_compiler_directory != NULL) {
			_tmp28_ = !g_path_is_absolute (vala_compiler_vapi_filename);
		} else {
			_tmp28_ = FALSE;
		}
		if (_tmp28_) {
			char* _tmp29_;
			vala_compiler_vapi_filename = (_tmp29_ = g_strdup_printf ("%s%c%s", vala_code_context_get_directory (self->priv->context), (gint) G_DIR_SEPARATOR, vala_compiler_vapi_filename), _g_free0 (vala_compiler_vapi_filename), _tmp29_);
		}
		vala_code_writer_write_file (interface_writer, self->priv->context, vala_compiler_vapi_filename);
		_vala_code_visitor_unref0 (interface_writer);
	}
	if (vala_compiler_library != NULL) {
		char* _tmp38_;
		if (vala_compiler_gir != NULL) {
			char* _tmp37_;
			if (vala_code_context_get_profile (self->priv->context) == VALA_PROFILE_GOBJECT) {
				glong gir_len;
				const char* last_hyphen;
				gboolean _tmp30_;
				gir_len = g_utf8_strlen (vala_compiler_gir, -1);
				last_hyphen = g_utf8_strrchr (vala_compiler_gir, (gssize) gir_len, (gunichar) '-');
				_tmp30_ = FALSE;
				if (last_hyphen == NULL) {
					_tmp30_ = TRUE;
				} else {
					_tmp30_ = !g_str_has_suffix (vala_compiler_gir, ".gir");
				}
				if (_tmp30_) {
					char* _tmp31_;
					vala_report_error (NULL, _tmp31_ = g_strdup_printf ("GIR file name `%s' is not well-formed, expected NAME-VERSION.gir", vala_compiler_gir));
					_g_free0 (_tmp31_);
				} else {
					glong offset;
					char* gir_namespace;
					char* gir_version;
					gboolean _tmp32_;
					gboolean _tmp33_;
					gboolean _tmp34_;
					offset = g_utf8_pointer_to_offset (vala_compiler_gir, last_hyphen);
					gir_namespace = string_substring (vala_compiler_gir, (glong) 0, offset);
					gir_version = string_substring (vala_compiler_gir, offset + 1, (gir_len - offset) - 5);
					g_strcanon (gir_version, "0123456789.", '?');
					_tmp32_ = FALSE;
					_tmp33_ = FALSE;
					_tmp34_ = FALSE;
					if (_vala_strcmp0 (gir_namespace, "") == 0) {
						_tmp34_ = TRUE;
					} else {
						_tmp34_ = _vala_strcmp0 (gir_version, "") == 0;
					}
					if (_tmp34_) {
						_tmp33_ = TRUE;
					} else {
						_tmp33_ = !g_unichar_isdigit (g_utf8_get_char (g_utf8_offset_to_pointer (gir_version, 0)));
					}
					if (_tmp33_) {
						_tmp32_ = TRUE;
					} else {
						_tmp32_ = string_contains (gir_version, "?");
					}
					if (_tmp32_) {
						char* _tmp35_;
						vala_report_error (NULL, _tmp35_ = g_strdup_printf ("GIR file name `%s' is not well-formed, expected NAME-VERSION.gir", vala_compiler_gir));
						_g_free0 (_tmp35_);
					} else {
						ValaGIRWriter* gir_writer;
						char* gir_directory;
						gir_writer = vala_gir_writer_new ();
						gir_directory = g_strdup (".");
						if (vala_compiler_directory != NULL) {
							char* _tmp36_;
							gir_directory = (_tmp36_ = g_strdup (vala_code_context_get_directory (self->priv->context)), _g_free0 (gir_directory), _tmp36_);
						}
						vala_gir_writer_write_file (gir_writer, self->priv->context, gir_directory, gir_namespace, gir_version, vala_compiler_library);
						_vala_code_visitor_unref0 (gir_writer);
						_g_free0 (gir_directory);
					}
					_g_free0 (gir_namespace);
					_g_free0 (gir_version);
				}
			}
			vala_compiler_gir = (_tmp37_ = NULL, _g_free0 (vala_compiler_gir), _tmp37_);
		}
		vala_compiler_library = (_tmp38_ = NULL, _g_free0 (vala_compiler_library), _tmp38_);
	}
	if (vala_compiler_internal_vapi_filename != NULL) {
		gboolean _tmp39_;
		ValaCodeWriter* interface_writer;
		char* vapi_filename;
		gboolean _tmp40_;
		char* _tmp42_;
		_tmp39_ = FALSE;
		if (vala_compiler_internal_header_filename == NULL) {
			_tmp39_ = TRUE;
		} else {
			_tmp39_ = vala_compiler_header_filename == NULL;
		}
		if (_tmp39_) {
			vala_report_error (NULL, "--internal-vapi may only be used in combination with --header and --internal-header");
			result = vala_compiler_quit (self);
			_vala_code_visitor_unref0 (parser);
			_vala_code_visitor_unref0 (genie_parser);
			_vala_code_visitor_unref0 (resolver);
			_vala_code_visitor_unref0 (analyzer);
			_vala_code_visitor_unref0 (flow_analyzer);
			return result;
		}
		interface_writer = vala_code_writer_new (FALSE, TRUE);
		vala_code_writer_set_cheader_override (interface_writer, vala_compiler_header_filename, vala_compiler_internal_header_filename);
		vapi_filename = g_strdup (vala_compiler_internal_vapi_filename);
		_tmp40_ = FALSE;
		if (vala_compiler_directory != NULL) {
			_tmp40_ = !g_path_is_absolute (vapi_filename);
		} else {
			_tmp40_ = FALSE;
		}
		if (_tmp40_) {
			char* _tmp41_;
			vapi_filename = (_tmp41_ = g_strdup_printf ("%s%c%s", vala_code_context_get_directory (self->priv->context), (gint) G_DIR_SEPARATOR, vapi_filename), _g_free0 (vapi_filename), _tmp41_);
		}
		vala_code_writer_write_file (interface_writer, self->priv->context, vapi_filename);
		vala_compiler_internal_vapi_filename = (_tmp42_ = NULL, _g_free0 (vala_compiler_internal_vapi_filename), _tmp42_);
		_vala_code_visitor_unref0 (interface_writer);
		_g_free0 (vapi_filename);
	}
	if (!vala_compiler_ccode_only) {
		ValaCCodeCompiler* ccompiler;
		gboolean _tmp43_;
		ccompiler = vala_ccode_compiler_new ();
		_tmp43_ = FALSE;
		if (vala_compiler_cc_command == NULL) {
			_tmp43_ = g_getenv ("CC") != NULL;
		} else {
			_tmp43_ = FALSE;
		}
		if (_tmp43_) {
			char* _tmp44_;
			vala_compiler_cc_command = (_tmp44_ = g_strdup (g_getenv ("CC")), _g_free0 (vala_compiler_cc_command), _tmp44_);
		}
		if (vala_compiler_cc_options == NULL) {
			gint _tmp46__length1;
			char** _tmp46_;
			char** _tmp45_;
			_tmp45_ = NULL;
			vala_ccode_compiler_compile (ccompiler, self->priv->context, vala_compiler_cc_command, (_tmp46_ = (_tmp45_ = g_new0 (char*, 0 + 1), _tmp45_), _tmp46__length1 = 0, _tmp46_), 0);
			_tmp46_ = (_vala_array_free (_tmp46_, _tmp46__length1, (GDestroyNotify) g_free), NULL);
		} else {
			vala_ccode_compiler_compile (ccompiler, self->priv->context, vala_compiler_cc_command, vala_compiler_cc_options, _vala_array_length (vala_compiler_cc_options));
		}
		_vala_ccode_compiler_unref0 (ccompiler);
	}
	result = vala_compiler_quit (self);
	_vala_code_visitor_unref0 (parser);
	_vala_code_visitor_unref0 (genie_parser);
	_vala_code_visitor_unref0 (resolver);
	_vala_code_visitor_unref0 (analyzer);
	_vala_code_visitor_unref0 (flow_analyzer);
	return result;
}


static gboolean vala_compiler_ends_with_dir_separator (const char* s) {
	gboolean result;
	g_return_val_if_fail (s != NULL, FALSE);
	result = G_IS_DIR_SEPARATOR (g_utf8_get_char (g_utf8_offset_to_pointer (s, g_utf8_strlen (s, -1) - 1)));
	return result;
}


static char* vala_compiler_realpath (const char* name) {
	char* result;
	char* rpath;
	const char* start;
	const char* end;
	glong root_len;
	gboolean _tmp12_;
	g_return_val_if_fail (name != NULL, NULL);
	rpath = NULL;
	start = NULL;
	end = NULL;
	if (!g_path_is_absolute (name)) {
		char* _tmp0_;
		rpath = (_tmp0_ = g_get_current_dir (), _g_free0 (rpath), _tmp0_);
		start = end = name;
	} else {
		char* _tmp1_;
		start = end = g_path_skip_root (name);
		rpath = (_tmp1_ = string_substring (name, (glong) 0, g_utf8_pointer_to_offset (name, start)), _g_free0 (rpath), _tmp1_);
	}
	root_len = g_utf8_pointer_to_offset (rpath, g_path_skip_root (rpath));
	{
		gboolean _tmp2_;
		_tmp2_ = TRUE;
		while (TRUE) {
			glong len;
			if (!_tmp2_) {
				start = end;
			}
			_tmp2_ = FALSE;
			if (!(g_utf8_get_char (start) != 0)) {
				break;
			}
			while (TRUE) {
				if (!G_IS_DIR_SEPARATOR (g_utf8_get_char (start))) {
					break;
				}
				start = g_utf8_next_char (start);
			}
			len = (glong) 0;
			{
				gboolean _tmp3_;
				end = start;
				_tmp3_ = TRUE;
				while (TRUE) {
					gboolean _tmp4_;
					if (!_tmp3_) {
						end = g_utf8_next_char (end);
					}
					_tmp3_ = FALSE;
					_tmp4_ = FALSE;
					if (g_utf8_get_char (end) != 0) {
						_tmp4_ = !G_IS_DIR_SEPARATOR (g_utf8_get_char (end));
					} else {
						_tmp4_ = FALSE;
					}
					if (!_tmp4_) {
						break;
					}
					len++;
				}
			}
			if (len == 0) {
				break;
			} else {
				gboolean _tmp5_;
				_tmp5_ = FALSE;
				if (len == 1) {
					_tmp5_ = g_utf8_get_char (start) == '.';
				} else {
					_tmp5_ = FALSE;
				}
				if (_tmp5_) {
				} else {
					gboolean _tmp6_;
					_tmp6_ = FALSE;
					if (len == 2) {
						_tmp6_ = g_str_has_prefix (start, "..");
					} else {
						_tmp6_ = FALSE;
					}
					if (_tmp6_) {
						if (g_utf8_strlen (rpath, -1) > root_len) {
							{
								gboolean _tmp7_;
								_tmp7_ = TRUE;
								while (TRUE) {
									char* _tmp8_;
									if (!_tmp7_) {
										if (!(!vala_compiler_ends_with_dir_separator (rpath))) {
											break;
										}
									}
									_tmp7_ = FALSE;
									rpath = (_tmp8_ = string_substring (rpath, (glong) 0, g_utf8_strlen (rpath, -1) - 1), _g_free0 (rpath), _tmp8_);
								}
							}
						}
					} else {
						char* _tmp11_;
						char* _tmp10_;
						if (!vala_compiler_ends_with_dir_separator (rpath)) {
							char* _tmp9_;
							rpath = (_tmp9_ = g_strconcat (rpath, G_DIR_SEPARATOR_S, NULL), _g_free0 (rpath), _tmp9_);
						}
						rpath = (_tmp11_ = g_strconcat (rpath, _tmp10_ = string_substring (start, (glong) 0, len), NULL), _g_free0 (rpath), _tmp11_);
						_g_free0 (_tmp10_);
					}
				}
			}
		}
	}
	_tmp12_ = FALSE;
	if (g_utf8_strlen (rpath, -1) > root_len) {
		_tmp12_ = vala_compiler_ends_with_dir_separator (rpath);
	} else {
		_tmp12_ = FALSE;
	}
	if (_tmp12_) {
		char* _tmp13_;
		rpath = (_tmp13_ = string_substring (rpath, (glong) 0, g_utf8_strlen (rpath, -1) - 1), _g_free0 (rpath), _tmp13_);
	}
	if (G_DIR_SEPARATOR != '/') {
		char** _tmp15_;
		gint components_size;
		gint components_length1;
		char** _tmp14_;
		char** components;
		char* _tmp16_;
		components = (_tmp15_ = _tmp14_ = g_strsplit (rpath, "\\", 0), components_length1 = _vala_array_length (_tmp14_), components_size = components_length1, _tmp15_);
		rpath = (_tmp16_ = g_strjoinv ("/", components), _g_free0 (rpath), _tmp16_);
		components = (_vala_array_free (components, components_length1, (GDestroyNotify) g_free), NULL);
	}
	result = rpath;
	return result;
}


static gint vala_compiler_main (char** args, int args_length1) {
	gint result;
	GError * _inner_error_;
	ValaCompiler* compiler;
	_inner_error_ = NULL;
	{
		GOptionContext* opt_context;
		opt_context = g_option_context_new ("- Vala Compiler");
		g_option_context_set_help_enabled (opt_context, TRUE);
		g_option_context_add_main_entries (opt_context, VALA_COMPILER_options, NULL);
		g_option_context_parse (opt_context, &args_length1, &args, &_inner_error_);
		if (_inner_error_ != NULL) {
			_g_option_context_free0 (opt_context);
			if (_inner_error_->domain == G_OPTION_ERROR) {
				goto __catch1_g_option_error;
			}
			goto __finally1;
		}
		_g_option_context_free0 (opt_context);
	}
	goto __finally1;
	__catch1_g_option_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			fprintf (stdout, "%s\n", e->message);
			fprintf (stdout, "Run '%s --help' to see a full list of available command line options.\n", args[0]);
			result = 1;
			_g_error_free0 (e);
			return result;
		}
	}
	__finally1:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	if (vala_compiler_version) {
		fprintf (stdout, "Vala %s\n", PACKAGE_VERSION);
		result = 0;
		return result;
	}
	if (vala_compiler_sources == NULL) {
		fprintf (stderr, "No source file specified.\n");
		result = 1;
		return result;
	}
	compiler = vala_compiler_new ();
	result = vala_compiler_run (compiler);
	_vala_compiler_unref0 (compiler);
	return result;
}


int main (int argc, char ** argv) {
	g_type_init ();
	return vala_compiler_main (argv, argc);
}


ValaCompiler* vala_compiler_construct (GType object_type) {
	ValaCompiler* self;
	self = (ValaCompiler*) g_type_create_instance (object_type);
	return self;
}


ValaCompiler* vala_compiler_new (void) {
	return vala_compiler_construct (VALA_TYPE_COMPILER);
}


static void vala_value_compiler_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_compiler_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_compiler_unref (value->data[0].v_pointer);
	}
}


static void vala_value_compiler_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_compiler_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_compiler_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_compiler_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaCompiler* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_compiler_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_compiler_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaCompiler** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_compiler_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_compiler (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecCompiler* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_COMPILER), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_compiler (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_COMPILER), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_compiler (GValue* value, gpointer v_object) {
	ValaCompiler* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_COMPILER));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_COMPILER));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_compiler_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_compiler_unref (old);
	}
}


static void vala_compiler_class_init (ValaCompilerClass * klass) {
	vala_compiler_parent_class = g_type_class_peek_parent (klass);
	VALA_COMPILER_CLASS (klass)->finalize = vala_compiler_finalize;
	g_type_class_add_private (klass, sizeof (ValaCompilerPrivate));
}


static void vala_compiler_instance_init (ValaCompiler * self) {
	self->priv = VALA_COMPILER_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_compiler_finalize (ValaCompiler* obj) {
	ValaCompiler * self;
	self = VALA_COMPILER (obj);
	_vala_code_context_unref0 (self->priv->context);
}


GType vala_compiler_get_type (void) {
	static GType vala_compiler_type_id = 0;
	if (vala_compiler_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_compiler_init, vala_value_compiler_free_value, vala_value_compiler_copy_value, vala_value_compiler_peek_pointer, "p", vala_value_compiler_collect_value, "p", vala_value_compiler_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaCompilerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_compiler_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCompiler), 0, (GInstanceInitFunc) vala_compiler_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_compiler_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaCompiler", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vala_compiler_type_id;
}


gpointer vala_compiler_ref (gpointer instance) {
	ValaCompiler* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_compiler_unref (gpointer instance) {
	ValaCompiler* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_COMPILER_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




