/* readonlyset.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_COLLECTION_OBJECT (gee_collection_object_get_type ())
#define GEE_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObject))
#define GEE_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
#define GEE_IS_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION_OBJECT))
#define GEE_IS_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_COLLECTION_OBJECT))
#define GEE_COLLECTION_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))

typedef struct _GeeCollectionObject GeeCollectionObject;
typedef struct _GeeCollectionObjectClass GeeCollectionObjectClass;
typedef struct _GeeCollectionObjectPrivate GeeCollectionObjectPrivate;

#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_SET (gee_set_get_type ())
#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))

typedef struct _GeeSet GeeSet;
typedef struct _GeeSetIface GeeSetIface;

#define GEE_TYPE_READ_ONLY_SET (gee_read_only_set_get_type ())
#define GEE_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet))
#define GEE_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
#define GEE_IS_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SET))
#define GEE_IS_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SET))
#define GEE_READ_ONLY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))

typedef struct _GeeReadOnlySet GeeReadOnlySet;
typedef struct _GeeReadOnlySetClass GeeReadOnlySetClass;
typedef struct _GeeReadOnlySetPrivate GeeReadOnlySetPrivate;
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))

#define GEE_READ_ONLY_SET_TYPE_ITERATOR (gee_read_only_set_iterator_get_type ())
#define GEE_READ_ONLY_SET_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIterator))
#define GEE_READ_ONLY_SET_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIteratorClass))
#define GEE_READ_ONLY_SET_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_SET_TYPE_ITERATOR))
#define GEE_READ_ONLY_SET_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_SET_TYPE_ITERATOR))
#define GEE_READ_ONLY_SET_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIteratorClass))

typedef struct _GeeReadOnlySetIterator GeeReadOnlySetIterator;
typedef struct _GeeReadOnlySetIteratorClass GeeReadOnlySetIteratorClass;
typedef struct _GeeReadOnlySetIteratorPrivate GeeReadOnlySetIteratorPrivate;

struct _GeeCollectionObject {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GeeCollectionObjectPrivate * priv;
};

struct _GeeCollectionObjectClass {
	GTypeClass parent_class;
	void (*finalize) (GeeCollectionObject *self);
};

struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
};

struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GType (*get_element_type) (GeeIterable* self);
	GeeIterator* (*iterator) (GeeIterable* self);
};

struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gint (*get_size) (GeeCollection* self);
};

struct _GeeSetIface {
	GTypeInterface parent_iface;
};

struct _GeeReadOnlySet {
	GeeCollectionObject parent_instance;
	GeeReadOnlySetPrivate * priv;
};

struct _GeeReadOnlySetClass {
	GeeCollectionObjectClass parent_class;
};

struct _GeeReadOnlySetPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeSet* _set;
};

struct _GeeReadOnlySetIterator {
	GeeCollectionObject parent_instance;
	GeeReadOnlySetIteratorPrivate * priv;
};

struct _GeeReadOnlySetIteratorClass {
	GeeCollectionObjectClass parent_class;
};

struct _GeeReadOnlySetIteratorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};


static gpointer gee_read_only_set_iterator_parent_class = NULL;
static GeeIteratorIface* gee_read_only_set_iterator_gee_iterator_parent_iface = NULL;
static gpointer gee_read_only_set_parent_class = NULL;
static GeeIterableIface* gee_read_only_set_gee_iterable_parent_iface = NULL;
static GeeCollectionIface* gee_read_only_set_gee_collection_parent_iface = NULL;
static GeeSetIface* gee_read_only_set_gee_set_parent_iface = NULL;

gpointer gee_collection_object_ref (gpointer instance);
void gee_collection_object_unref (gpointer instance);
GParamSpec* gee_param_spec_collection_object (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gee_value_set_collection_object (GValue* value, gpointer v_object);
gpointer gee_value_get_collection_object (const GValue* value);
GType gee_collection_object_get_type (void);
GType gee_iterator_get_type (void);
GType gee_iterable_get_type (void);
GType gee_collection_get_type (void);
GType gee_set_get_type (void);
GType gee_read_only_set_get_type (void);
#define GEE_READ_ONLY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetPrivate))
enum  {
	GEE_READ_ONLY_SET_DUMMY_PROPERTY
};
GeeCollectionObject* gee_collection_object_new (void);
GeeCollectionObject* gee_collection_object_construct (GType object_type);
void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value);
GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
static GType gee_read_only_set_real_get_element_type (GeeIterable* base);
static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
static GType gee_read_only_set_iterator_get_type (void);
GeeIterator* gee_iterable_iterator (GeeIterable* self);
static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base);
gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item);
static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer item);
static gboolean gee_read_only_set_real_remove (GeeCollection* base, gconstpointer item);
static void gee_read_only_set_real_clear (GeeCollection* base);
gint gee_collection_get_size (GeeCollection* self);
#define GEE_READ_ONLY_SET_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIteratorPrivate))
enum  {
	GEE_READ_ONLY_SET_ITERATOR_DUMMY_PROPERTY
};
static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base);
static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base);
static void gee_read_only_set_finalize (GeeCollectionObject* obj);



GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set) {
	GeeReadOnlySet* self;
	self = (GeeReadOnlySet*) gee_collection_object_construct (object_type);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	gee_read_only_set_set_set (self, set);
	return self;
}


GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set) {
	return gee_read_only_set_construct (GEE_TYPE_READ_ONLY_SET, g_type, g_dup_func, g_destroy_func, set);
}


static GType gee_read_only_set_real_get_element_type (GeeIterable* base) {
	GeeReadOnlySet * self;
	GType result;
	self = (GeeReadOnlySet*) base;
	result = self->priv->g_type;
	return result;
}


static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base) {
	GeeReadOnlySet * self;
	GeeIterator* result;
	self = (GeeReadOnlySet*) base;
	if (self->priv->_set == NULL) {
		result = (GeeIterator*) gee_read_only_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
		return result;
	}
	result = gee_iterable_iterator ((GeeIterable*) self->priv->_set);
	return result;
}


static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item) {
	GeeReadOnlySet * self;
	gboolean result;
	self = (GeeReadOnlySet*) base;
	if (self->priv->_set == NULL) {
		result = FALSE;
		return result;
	}
	result = gee_collection_contains ((GeeCollection*) self->priv->_set, item);
	return result;
}


static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer item) {
	GeeReadOnlySet * self;
	self = (GeeReadOnlySet*) base;
	g_assert_not_reached ();
}


static gboolean gee_read_only_set_real_remove (GeeCollection* base, gconstpointer item) {
	GeeReadOnlySet * self;
	self = (GeeReadOnlySet*) base;
	g_assert_not_reached ();
}


static void gee_read_only_set_real_clear (GeeCollection* base) {
	GeeReadOnlySet * self;
	self = (GeeReadOnlySet*) base;
	g_assert_not_reached ();
}


static gint gee_read_only_set_real_get_size (GeeCollection* base) {
	gint result;
	GeeReadOnlySet* self;
	self = (GeeReadOnlySet*) base;
	result = gee_collection_get_size ((GeeCollection*) self->priv->_set);
	return result;
}


static gpointer _gee_collection_object_ref0 (gpointer self) {
	return self ? gee_collection_object_ref (self) : NULL;
}


void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value) {
	GeeSet* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_set = (_tmp0_ = _gee_collection_object_ref0 (value), _gee_collection_object_unref0 (self->priv->_set), _tmp0_);
}


static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base) {
	GeeReadOnlySetIterator * self;
	gboolean result;
	self = (GeeReadOnlySetIterator*) base;
	result = FALSE;
	return result;
}


static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base) {
	GeeReadOnlySetIterator * self;
	gpointer result;
	self = (GeeReadOnlySetIterator*) base;
	result = NULL;
	return result;
}


static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func) {
	GeeReadOnlySetIterator* self;
	self = (GeeReadOnlySetIterator*) gee_collection_object_construct (object_type);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}


static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func) {
	return gee_read_only_set_iterator_construct (GEE_READ_ONLY_SET_TYPE_ITERATOR, g_type, g_dup_func, g_destroy_func);
}


static void gee_read_only_set_iterator_class_init (GeeReadOnlySetIteratorClass * klass) {
	gee_read_only_set_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeReadOnlySetIteratorPrivate));
}


static void gee_read_only_set_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_read_only_set_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = gee_read_only_set_iterator_real_next;
	iface->get = gee_read_only_set_iterator_real_get;
}


static void gee_read_only_set_iterator_instance_init (GeeReadOnlySetIterator * self) {
	self->priv = GEE_READ_ONLY_SET_ITERATOR_GET_PRIVATE (self);
}


static GType gee_read_only_set_iterator_get_type (void) {
	static GType gee_read_only_set_iterator_type_id = 0;
	if (gee_read_only_set_iterator_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlySetIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_set_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlySetIterator), 0, (GInstanceInitFunc) gee_read_only_set_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_read_only_set_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_read_only_set_iterator_type_id = g_type_register_static (GEE_TYPE_COLLECTION_OBJECT, "GeeReadOnlySetIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_read_only_set_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
	}
	return gee_read_only_set_iterator_type_id;
}


static void gee_read_only_set_class_init (GeeReadOnlySetClass * klass) {
	gee_read_only_set_parent_class = g_type_class_peek_parent (klass);
	GEE_COLLECTION_OBJECT_CLASS (klass)->finalize = gee_read_only_set_finalize;
	g_type_class_add_private (klass, sizeof (GeeReadOnlySetPrivate));
}


static void gee_read_only_set_gee_iterable_interface_init (GeeIterableIface * iface) {
	gee_read_only_set_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_element_type = gee_read_only_set_real_get_element_type;
	iface->iterator = gee_read_only_set_real_iterator;
}


static void gee_read_only_set_gee_collection_interface_init (GeeCollectionIface * iface) {
	gee_read_only_set_gee_collection_parent_iface = g_type_interface_peek_parent (iface);
	iface->contains = gee_read_only_set_real_contains;
	iface->add = gee_read_only_set_real_add;
	iface->remove = gee_read_only_set_real_remove;
	iface->clear = gee_read_only_set_real_clear;
	iface->get_size = gee_read_only_set_real_get_size;
}


static void gee_read_only_set_gee_set_interface_init (GeeSetIface * iface) {
	gee_read_only_set_gee_set_parent_iface = g_type_interface_peek_parent (iface);
}


static void gee_read_only_set_instance_init (GeeReadOnlySet * self) {
	self->priv = GEE_READ_ONLY_SET_GET_PRIVATE (self);
}


static void gee_read_only_set_finalize (GeeCollectionObject* obj) {
	GeeReadOnlySet * self;
	self = GEE_READ_ONLY_SET (obj);
	_gee_collection_object_unref0 (self->priv->_set);
	GEE_COLLECTION_OBJECT_CLASS (gee_read_only_set_parent_class)->finalize (obj);
}


GType gee_read_only_set_get_type (void) {
	static GType gee_read_only_set_type_id = 0;
	if (gee_read_only_set_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlySetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlySet), 0, (GInstanceInitFunc) gee_read_only_set_instance_init, NULL };
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) gee_read_only_set_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_collection_info = { (GInterfaceInitFunc) gee_read_only_set_gee_collection_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_set_info = { (GInterfaceInitFunc) gee_read_only_set_gee_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_read_only_set_type_id = g_type_register_static (GEE_TYPE_COLLECTION_OBJECT, "GeeReadOnlySet", &g_define_type_info, 0);
		g_type_add_interface_static (gee_read_only_set_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
		g_type_add_interface_static (gee_read_only_set_type_id, GEE_TYPE_COLLECTION, &gee_collection_info);
		g_type_add_interface_static (gee_read_only_set_type_id, GEE_TYPE_SET, &gee_set_info);
	}
	return gee_read_only_set_type_id;
}




