/* valaccodebasemodule.vala
 *
 * Copyright (C) 2006-2009  Jürg Billeter
 * Copyright (C) 2006-2008  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 * 	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>
#include <float.h>
#include <math.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))

typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;

#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))

typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;
#define _vala_code_context_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_context_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _vala_ccode_declaration_space_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_declaration_space_unref (var), NULL)))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
#define _vala_source_file_unref0(var) ((var == NULL) ? NULL : (var = (vala_source_file_unref (var), NULL)))
#define _vala_ccode_writer_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_writer_unref (var), NULL)))
typedef struct _ValaCCodeDeclarationSpacePrivate ValaCCodeDeclarationSpacePrivate;
#define _vala_comment_unref0(var) ((var == NULL) ? NULL : (var = (vala_comment_unref (var), NULL)))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))

struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

struct _ValaCCodeBaseModule {
	ValaCCodeModule parent_instance;
	ValaCCodeBaseModulePrivate * priv;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaTryStatement* current_try;
	ValaCCodeDeclarationSpace* header_declarations;
	ValaCCodeDeclarationSpace* internal_header_declarations;
	ValaCCodeDeclarationSpace* source_declarations;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* class_init_fragment;
	ValaCCodeFragment* base_init_fragment;
	ValaCCodeFragment* class_finalize_fragment;
	ValaCCodeFragment* base_finalize_fragment;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_finalize_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeFragment* module_init_fragment;
	ValaCCodeStruct* param_spec_struct;
	ValaCCodeStruct* closure_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeFunction* function;
	ValaCCodeFragment* pre_statement_fragment;
	GeeArrayList* temp_vars;
	GeeArrayList* temp_ref_vars;
	GeeSet* user_marshal_set;
	GeeSet* predefined_marshal_set;
	gint next_temp_var_id;
	gboolean in_constructor;
	gboolean in_static_or_class_context;
	gboolean current_method_inner_error;
	gint next_coroutine_state;
	ValaDataType* void_type;
	ValaDataType* bool_type;
	ValaDataType* char_type;
	ValaDataType* uchar_type;
	ValaDataType* unichar_type;
	ValaDataType* short_type;
	ValaDataType* ushort_type;
	ValaDataType* int_type;
	ValaDataType* uint_type;
	ValaDataType* long_type;
	ValaDataType* ulong_type;
	ValaDataType* int8_type;
	ValaDataType* uint8_type;
	ValaDataType* int16_type;
	ValaDataType* uint16_type;
	ValaDataType* int32_type;
	ValaDataType* uint32_type;
	ValaDataType* int64_type;
	ValaDataType* uint64_type;
	ValaDataType* string_type;
	ValaDataType* float_type;
	ValaDataType* double_type;
	ValaTypeSymbol* gtype_type;
	ValaTypeSymbol* gobject_type;
	ValaErrorType* gerror_type;
	ValaClass* glist_type;
	ValaClass* gslist_type;
	ValaClass* gvaluearray_type;
	ValaTypeSymbol* gstringbuilder_type;
	ValaTypeSymbol* garray_type;
	ValaTypeSymbol* gbytearray_type;
	ValaTypeSymbol* gptrarray_type;
	ValaDataType* gquark_type;
	ValaStruct* gvalue_type;
	ValaStruct* mutex_type;
	ValaTypeSymbol* type_module_type;
	ValaTypeSymbol* dbus_object_type;
	gboolean in_plugin;
	char* module_init_param_name;
	gboolean gvaluecollector_h_needed;
	gboolean requires_array_free;
	gboolean requires_array_move;
	gboolean requires_array_length;
	gboolean requires_strcmp0;
	gboolean dbus_glib_h_needed;
	gboolean dbus_glib_h_needed_in_header;
	GeeSet* wrappers;
	GeeMap* variable_name_map;
};

struct _ValaCCodeBaseModuleClass {
	ValaCCodeModuleClass parent_class;
	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
	char* (*append_struct_array_free) (ValaCCodeBaseModule* self, ValaStruct* st);
	ValaCCodeExpression* (*get_unref_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
	void (*append_local_free) (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
	gboolean (*variable_accessible_in_finally) (ValaCCodeBaseModule* self, ValaLocalVariable* local);
	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_ref_cexpression) (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
	gboolean (*is_gobject_property) (ValaCCodeBaseModule* self, ValaProperty* prop);
};

struct _ValaCCodeBaseModulePrivate {
	ValaCodeContext* _context;
	GeeSet* reserved_identifiers;
	gint next_block_id;
	GeeMap* block_map;
	GeeSet* generated_external_symbols;
};

struct _ValaCCodeDeclarationSpace {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeDeclarationSpacePrivate * priv;
	ValaCCodeFragment* include_directives;
	ValaCCodeFragment* type_declaration;
	ValaCCodeFragment* type_definition;
	ValaCCodeFragment* type_member_declaration;
	ValaCCodeFragment* constant_declaration;
};

struct _ValaCCodeDeclarationSpaceClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeDeclarationSpace *self);
};


static gpointer vala_ccode_base_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_base_module_get_type (void);
gpointer vala_ccode_declaration_space_ref (gpointer instance);
void vala_ccode_declaration_space_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_declaration_space (const GValue* value);
GType vala_ccode_declaration_space_get_type (void);
#define VALA_CCODE_BASE_MODULE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModulePrivate))
enum  {
	VALA_CCODE_BASE_MODULE_DUMMY_PROPERTY
};
GType vala_ccode_generator_get_type (void);
ValaCCodeModule* vala_ccode_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeBaseModule* vala_ccode_base_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeBaseModule* vala_ccode_base_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
void vala_ccode_base_module_set_context (ValaCCodeBaseModule* self, ValaCodeContext* value);
ValaCCodeDeclarationSpace* vala_ccode_declaration_space_new (void);
ValaCCodeDeclarationSpace* vala_ccode_declaration_space_construct (GType object_type);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
static char* vala_ccode_base_module_get_define_for_filename (const char* filename);
static void vala_ccode_base_module_real_emit (ValaCCodeModule* base, ValaCodeContext* context);
static ValaCCodeIdentifier* vala_ccode_base_module_real_get_value_setter_function (ValaCCodeModule* base, ValaDataType* type_reference);
static ValaCCodeIdentifier* vala_ccode_base_module_get_value_getter_function (ValaCCodeBaseModule* self, ValaDataType* type_reference);
void vala_ccode_base_module_append_vala_array_free (ValaCCodeBaseModule* self);
static void vala_ccode_base_module_real_append_vala_array_free (ValaCCodeBaseModule* self);
void vala_ccode_base_module_append_vala_array_move (ValaCCodeBaseModule* self);
static void vala_ccode_base_module_real_append_vala_array_move (ValaCCodeBaseModule* self);
void vala_ccode_base_module_append_vala_array_length (ValaCCodeBaseModule* self);
static void vala_ccode_base_module_real_append_vala_array_length (ValaCCodeBaseModule* self);
void vala_ccode_declaration_space_add_type_member_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
static void vala_ccode_base_module_append_vala_strcmp0 (ValaCCodeBaseModule* self);
ValaCodeContext* vala_ccode_base_module_get_context (ValaCCodeBaseModule* self);
void vala_ccode_declaration_space_add_include (ValaCCodeDeclarationSpace* self, const char* filename, gboolean local);
void vala_ccode_declaration_space_add_type_definition (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
void vala_ccode_declaration_space_add_type_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
static void vala_ccode_base_module_real_visit_source_file (ValaCCodeModule* base, ValaSourceFile* source_file);
gboolean vala_ccode_declaration_space_add_symbol_declaration (ValaCCodeDeclarationSpace* self, ValaSymbol* sym, const char* name);
void vala_ccode_base_module_generate_enum_declaration (ValaCCodeBaseModule* self, ValaEnum* en, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_visit_enum (ValaCCodeModule* base, ValaEnum* en);
char* vala_ccode_base_module_get_symbol_lock_name (ValaCCodeBaseModule* self, const char* symname);
static void vala_ccode_base_module_real_visit_member (ValaCCodeModule* base, ValaMember* m);
void vala_ccode_declaration_space_add_constant_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
void vala_ccode_base_module_generate_constant_declaration (ValaCCodeBaseModule* self, ValaConstant* c, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_visit_constant (ValaCCodeModule* base, ValaConstant* c);
void vala_ccode_base_module_generate_type_declaration (ValaCCodeBaseModule* self, ValaDataType* type, ValaCCodeDeclarationSpace* decl_space);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
char* vala_ccode_module_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim);
char* vala_ccode_base_module_get_delegate_target_cname (ValaCCodeBaseModule* self, const char* delegate_cname);
void vala_ccode_base_module_generate_field_declaration (ValaCCodeBaseModule* self, ValaField* f, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_check_type (ValaCCodeBaseModule* self, ValaDataType* type);
ValaDataType* vala_ccode_base_module_get_data_type_for_symbol (ValaTypeSymbol* sym);
ValaCCodeExpression* vala_ccode_module_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
void vala_ccode_base_module_append_temp_decl (ValaCCodeBaseModule* self, ValaCCodeFragment* cfrag, GeeList* temp_vars);
gboolean vala_ccode_base_module_requires_destroy (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
ValaCCodeExpression* vala_ccode_base_module_default_value_for_type (ValaCCodeBaseModule* self, ValaDataType* type, gboolean initializer_expression);
gboolean vala_ccode_base_module_is_constant_ccode_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr);
char* vala_ccode_module_get_array_size_cname (ValaCCodeModule* self, const char* array_cname);
static void vala_ccode_base_module_real_visit_field (ValaCCodeModule* base, ValaField* f);
gboolean vala_ccode_base_module_is_pure_ccode_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr);
static void vala_ccode_base_module_real_visit_formal_parameter (ValaCCodeModule* base, ValaFormalParameter* p);
static void vala_ccode_base_module_real_visit_property (ValaCCodeModule* base, ValaProperty* prop);
void vala_ccode_base_module_generate_class_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_interface_declaration (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_delegate_declaration (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_struct_declaration (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_error_domain_declaration (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_class_struct_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_generate_class_struct_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_generate_struct_declaration (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_generate_delegate_declaration (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_cparameters (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
static void vala_ccode_base_module_real_generate_cparameters (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
void vala_ccode_base_module_generate_property_accessor_declaration (ValaCCodeBaseModule* self, ValaPropertyAccessor* acc, ValaCCodeDeclarationSpace* decl_space);
ValaDataType* vala_ccode_base_module_get_current_return_type (ValaCCodeBaseModule* self);
ValaCCodeExpression* vala_ccode_base_module_transform_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
static ValaCCodeStatement* vala_ccode_base_module_create_property_type_check_statement (ValaCCodeBaseModule* self, ValaProperty* prop, gboolean check_return_type, ValaTypeSymbol* t, gboolean non_null, const char* var_name);
gboolean vala_ccode_base_module_is_gobject_property (ValaCCodeBaseModule* self, ValaProperty* prop);
static void vala_ccode_base_module_real_visit_property_accessor (ValaCCodeModule* base, ValaPropertyAccessor* acc);
static void vala_ccode_base_module_real_visit_destructor (ValaCCodeModule* base, ValaDestructor* d);
gint vala_ccode_base_module_get_block_id (ValaCCodeBaseModule* self, ValaBlock* b);
ValaMethod* vala_ccode_base_module_get_current_method (ValaCCodeBaseModule* self);
ValaClass* vala_ccode_base_module_get_current_class (ValaCCodeBaseModule* self);
char* vala_ccode_base_module_get_variable_cname (ValaCCodeBaseModule* self, const char* name);
ValaCCodeExpression* vala_ccode_base_module_get_variable_cexpression (ValaCCodeBaseModule* self, const char* name);
ValaCCodeExpression* vala_ccode_base_module_get_dup_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
gboolean vala_ccode_base_module_requires_copy (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_get_ref_cexpression (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
static void vala_ccode_base_module_real_visit_block (ValaCCodeModule* base, ValaBlock* b);
static void vala_ccode_base_module_real_visit_empty_statement (ValaCCodeModule* base, ValaEmptyStatement* stmt);
void vala_ccode_base_module_create_temp_decl (ValaCCodeBaseModule* self, ValaStatement* stmt, GeeList* temp_vars);
static void vala_ccode_base_module_real_visit_declaration_statement (ValaCCodeModule* base, ValaDeclarationStatement* stmt);
ValaCCodeExpression* vala_ccode_base_module_get_result_cexpression (ValaCCodeBaseModule* self, const char* cname);
ValaLocalVariable* vala_ccode_base_module_get_temp_variable (ValaCCodeBaseModule* self, ValaDataType* type, gboolean value_owned, ValaCodeNode* node_reference, gboolean init);
ValaCCodeExpression* vala_ccode_base_module_get_delegate_target_cexpression (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
void vala_ccode_module_add_simple_check (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
static void vala_ccode_base_module_real_visit_local_variable (ValaCCodeModule* base, ValaLocalVariable* local);
static void vala_ccode_base_module_real_visit_initializer_list (ValaCCodeModule* base, ValaInitializerList* list);
static gboolean vala_ccode_base_module_is_in_generic_type (ValaCCodeBaseModule* self, ValaDataType* type);
gboolean vala_ccode_base_module_get_in_creation_method (ValaCCodeBaseModule* self);
static ValaCCodeExpression* vala_ccode_base_module_get_type_id_expression (ValaCCodeBaseModule* self, ValaDataType* type, gboolean is_chainup);
static char* vala_ccode_base_module_generate_struct_dup_wrapper (ValaCCodeBaseModule* self, ValaValueType* value_type);
static ValaCCodeExpression* vala_ccode_base_module_real_get_dup_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
gboolean vala_ccode_base_module_add_wrapper (ValaCCodeBaseModule* self, const char* wrapper_name);
static void vala_ccode_base_module_generate_struct_copy_function (ValaCCodeBaseModule* self, ValaStruct* st);
char* vala_ccode_base_module_generate_destroy_func_wrapper (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_get_destroy_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, gboolean is_chainup);
static char* vala_ccode_base_module_generate_glist_free_wrapper (ValaCCodeBaseModule* self, ValaDataType* list_type, ValaCCodeIdentifier* element_destroy_func_expression);
static void vala_ccode_base_module_generate_struct_destroy_function (ValaCCodeBaseModule* self, ValaStruct* st);
ValaTypeSymbol* vala_ccode_base_module_get_current_type_symbol (ValaCCodeBaseModule* self);
char* vala_ccode_base_module_append_struct_array_free (ValaCCodeBaseModule* self, ValaStruct* st);
static char* vala_ccode_base_module_real_append_struct_array_free (ValaCCodeBaseModule* self, ValaStruct* st);
static ValaCCodeExpression* vala_ccode_base_module_real_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
static void vala_ccode_base_module_real_visit_end_full_expression (ValaCCodeModule* base, ValaExpression* expr);
static void vala_ccode_base_module_real_visit_expression_statement (ValaCCodeModule* base, ValaExpressionStatement* stmt);
void vala_ccode_base_module_append_local_free (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
static void vala_ccode_base_module_append_param_free (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeFragment* cfrag);
static void vala_ccode_base_module_real_append_local_free (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
void vala_ccode_base_module_append_error_free (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, ValaTryStatement* current_try);
void vala_ccode_base_module_create_local_free (ValaCCodeBaseModule* self, ValaCodeNode* stmt, gboolean stop_at_loop);
gboolean vala_ccode_base_module_variable_accessible_in_finally (ValaCCodeBaseModule* self, ValaLocalVariable* local);
static gboolean vala_ccode_base_module_real_variable_accessible_in_finally (ValaCCodeBaseModule* self, ValaLocalVariable* local);
ValaCCodeStatement* vala_ccode_base_module_create_postcondition_statement (ValaCCodeBaseModule* self, ValaExpression* postcondition);
ValaPropertyAccessor* vala_ccode_base_module_get_current_property_accessor (ValaCCodeBaseModule* self);
static void vala_ccode_base_module_real_visit_return_statement (ValaCCodeModule* base, ValaReturnStatement* stmt);
ValaCCodeFunctionCall* vala_ccode_base_module_generate_instance_cast (ValaCCodeBaseModule* self, ValaCCodeExpression* expr, ValaTypeSymbol* type);
static void vala_ccode_base_module_real_visit_lock_statement (ValaCCodeModule* base, ValaLockStatement* stmt);
static void vala_ccode_base_module_real_visit_delete_statement (ValaCCodeModule* base, ValaDeleteStatement* stmt);
ValaCCodeExpression* vala_ccode_base_module_convert_from_generic_pointer (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr, ValaDataType* actual_type);
ValaCCodeExpression* vala_ccode_base_module_convert_to_generic_pointer (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr, ValaDataType* actual_type);
static void vala_ccode_base_module_real_visit_expression (ValaCCodeModule* base, ValaExpression* expr);
static void vala_ccode_base_module_real_visit_boolean_literal (ValaCCodeModule* base, ValaBooleanLiteral* expr);
static void vala_ccode_base_module_real_visit_character_literal (ValaCCodeModule* base, ValaCharacterLiteral* expr);
static void vala_ccode_base_module_real_visit_integer_literal (ValaCCodeModule* base, ValaIntegerLiteral* expr);
static void vala_ccode_base_module_real_visit_real_literal (ValaCCodeModule* base, ValaRealLiteral* expr);
static void vala_ccode_base_module_real_visit_string_literal (ValaCCodeModule* base, ValaStringLiteral* expr);
static void vala_ccode_base_module_real_visit_null_literal (ValaCCodeModule* base, ValaNullLiteral* expr);
static char* vala_ccode_base_module_real_get_delegate_target_cname (ValaCCodeBaseModule* self, const char* delegate_cname);
static ValaCCodeExpression* vala_ccode_base_module_real_get_delegate_target_cexpression (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
char* vala_ccode_base_module_get_delegate_target_destroy_notify_cname (ValaCCodeBaseModule* self, const char* delegate_cname);
static char* vala_ccode_base_module_real_get_delegate_target_destroy_notify_cname (ValaCCodeBaseModule* self, const char* delegate_cname);
static void vala_ccode_base_module_real_visit_base_access (ValaCCodeModule* base, ValaBaseAccess* expr);
static ValaMemberAccess* vala_ccode_base_module_find_property_access (ValaCCodeBaseModule* self, ValaExpression* expr);
ValaCCodeFunctionCall* vala_ccode_base_module_get_property_set_call (ValaCCodeBaseModule* self, ValaProperty* prop, ValaMemberAccess* ma, ValaCCodeExpression* cexpr, ValaExpression* rhs);
static void vala_ccode_base_module_real_visit_postfix_expression (ValaCCodeModule* base, ValaPostfixExpression* expr);
static gboolean vala_ccode_base_module_is_limited_generic_type (ValaCCodeBaseModule* self, ValaDataType* type);
static gboolean vala_ccode_base_module_is_ref_function_void (ValaCCodeBaseModule* self, ValaDataType* type);
static ValaCCodeExpression* vala_ccode_base_module_real_get_ref_cexpression (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
static gboolean vala_ccode_base_module_is_reference_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg);
static gboolean vala_ccode_base_module_is_nullable_value_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg);
static gboolean vala_ccode_base_module_is_signed_integer_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg);
static gboolean vala_ccode_base_module_is_unsigned_integer_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg);
static void vala_ccode_base_module_check_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg);
static void vala_ccode_base_module_real_generate_class_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_generate_interface_declaration (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_generate_method_declaration (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_generate_method_declaration (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_base_module_real_generate_error_domain_declaration (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
void vala_ccode_base_module_add_generic_type_arguments (ValaCCodeBaseModule* self, ValaCCodeFunctionCall* ccall, GeeList* type_args, ValaCodeNode* expr, gboolean is_chainup);
gint vala_ccode_base_module_get_param_pos (ValaCCodeBaseModule* self, double param_pos, gboolean ellipsis);
ValaCCodeExpression* vala_ccode_base_module_handle_struct_argument (ValaCCodeBaseModule* self, ValaFormalParameter* param, ValaExpression* arg, ValaCCodeExpression* cexpr);
char* vala_ccode_module_get_custom_creturn_type (ValaCCodeModule* self, ValaMethod* m);
static void vala_ccode_base_module_real_visit_object_creation_expression (ValaCCodeModule* base, ValaObjectCreationExpression* expr);
static void vala_ccode_base_module_real_visit_sizeof_expression (ValaCCodeModule* base, ValaSizeofExpression* expr);
static void vala_ccode_base_module_real_visit_typeof_expression (ValaCCodeModule* base, ValaTypeofExpression* expr);
static void vala_ccode_base_module_real_visit_unary_expression (ValaCCodeModule* base, ValaUnaryExpression* expr);
static ValaCCodeExpression* vala_ccode_base_module_create_type_check (ValaCCodeBaseModule* self, ValaCCodeNode* ccodenode, ValaDataType* type);
static void vala_ccode_base_module_real_visit_cast_expression (ValaCCodeModule* base, ValaCastExpression* expr);
static void vala_ccode_base_module_real_visit_pointer_indirection (ValaCCodeModule* base, ValaPointerIndirection* expr);
static void vala_ccode_base_module_real_visit_addressof_expression (ValaCCodeModule* base, ValaAddressofExpression* expr);
static void vala_ccode_base_module_real_visit_reference_transfer_expression (ValaCCodeModule* base, ValaReferenceTransferExpression* expr);
static void vala_ccode_base_module_real_visit_binary_expression (ValaCCodeModule* base, ValaBinaryExpression* expr);
char* vala_ccode_base_module_get_type_check_function (ValaCCodeBaseModule* self, ValaTypeSymbol* type);
static void vala_ccode_base_module_real_visit_type_check (ValaCCodeModule* base, ValaTypeCheck* expr);
static void vala_ccode_base_module_real_visit_lambda_expression (ValaCCodeModule* base, ValaLambdaExpression* l);
ValaCCodeIdentifier* vala_ccode_module_get_value_setter_function (ValaCCodeModule* self, ValaDataType* type_reference);
ValaCCodeExpression* vala_ccode_base_module_get_implicit_cast_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
static ValaCCodeExpression* vala_ccode_base_module_real_get_implicit_cast_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
ValaCCodeNode* vala_ccode_base_module_get_ccodenode (ValaCCodeBaseModule* self, ValaCodeNode* node);
char* vala_ccode_module_get_dynamic_property_setter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
gboolean vala_ccode_base_module_is_address_of_possible (ValaCCodeBaseModule* self, ValaExpression* e);
ValaCCodeExpression* vala_ccode_base_module_get_address_of_expression (ValaCCodeBaseModule* self, ValaExpression* e, ValaCCodeExpression* ce);
gboolean vala_ccode_base_module_add_generated_external_symbol (ValaCCodeBaseModule* self, ValaSymbol* external_symbol);
ValaCCodeStatement* vala_ccode_base_module_create_type_check_statement (ValaCCodeBaseModule* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaTypeSymbol* t, gboolean non_null, const char* var_name);
static void vala_ccode_base_module_real_visit_class (ValaCCodeModule* base, ValaClass* cl);
static gboolean vala_ccode_base_module_real_is_gobject_property (ValaCCodeBaseModule* self, ValaProperty* prop);
ValaDataType* vala_ccode_base_module_get_this_type (ValaCCodeBaseModule* self);
gboolean vala_ccode_declaration_space_add_declaration (ValaCCodeDeclarationSpace* self, const char* name);
static void vala_ccode_base_module_finalize (ValaCCodeModule* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



ValaCCodeBaseModule* vala_ccode_base_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaCCodeBaseModule* self;
	GeeSet* _tmp0_;
	GeeSet* _tmp1_;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaCCodeBaseModule*) vala_ccode_module_construct (object_type, codegen, next);
	self->predefined_marshal_set = (_tmp0_ = (GeeSet*) gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal), _gee_collection_object_unref0 (self->predefined_marshal_set), _tmp0_);
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:VOID");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:BOOLEAN");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:CHAR");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:UCHAR");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:INT");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:UINT");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:LONG");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:ULONG");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:ENUM");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:FLAGS");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:FLOAT");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:DOUBLE");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:STRING");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:POINTER");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:OBJECT");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "STRING:OBJECT,POINTER");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "VOID:UINT,POINTER");
	gee_collection_add ((GeeCollection*) self->predefined_marshal_set, "BOOLEAN:FLAGS");
	self->priv->reserved_identifiers = (_tmp1_ = (GeeSet*) gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal), _gee_collection_object_unref0 (self->priv->reserved_identifiers), _tmp1_);
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "_Bool");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "_Complex");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "_Imaginary");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "auto");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "break");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "case");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "char");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "const");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "continue");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "default");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "do");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "double");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "else");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "enum");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "extern");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "float");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "for");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "goto");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "if");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "inline");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "int");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "long");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "register");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "restrict");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "return");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "short");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "signed");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "sizeof");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "static");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "struct");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "switch");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "typedef");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "union");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "unsigned");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "void");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "volatile");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "while");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "cdecl");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "error");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "result");
	gee_collection_add ((GeeCollection*) self->priv->reserved_identifiers, "self");
	return self;
}


ValaCCodeBaseModule* vala_ccode_base_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_ccode_base_module_construct (VALA_TYPE_CCODE_BASE_MODULE, codegen, next);
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


static void vala_ccode_base_module_real_emit (ValaCCodeModule* base, ValaCodeContext* context) {
	ValaCCodeBaseModule * self;
	ValaSymbol* _tmp0_;
	ValaDataType* _tmp2_;
	ValaStruct* _tmp1_;
	ValaDataType* _tmp4_;
	ValaStruct* _tmp3_;
	ValaDataType* _tmp6_;
	ValaStruct* _tmp5_;
	ValaDataType* _tmp8_;
	ValaStruct* _tmp7_;
	ValaDataType* _tmp10_;
	ValaStruct* _tmp9_;
	ValaDataType* _tmp12_;
	ValaStruct* _tmp11_;
	ValaDataType* _tmp14_;
	ValaStruct* _tmp13_;
	ValaDataType* _tmp16_;
	ValaStruct* _tmp15_;
	ValaDataType* _tmp18_;
	ValaStruct* _tmp17_;
	ValaDataType* _tmp20_;
	ValaStruct* _tmp19_;
	ValaDataType* _tmp22_;
	ValaStruct* _tmp21_;
	ValaDataType* _tmp24_;
	ValaStruct* _tmp23_;
	ValaDataType* _tmp26_;
	ValaStruct* _tmp25_;
	ValaDataType* _tmp28_;
	ValaStruct* _tmp27_;
	ValaDataType* _tmp30_;
	ValaStruct* _tmp29_;
	ValaDataType* _tmp32_;
	ValaStruct* _tmp31_;
	ValaDataType* _tmp34_;
	ValaStruct* _tmp33_;
	ValaDataType* _tmp36_;
	ValaStruct* _tmp35_;
	ValaDataType* _tmp38_;
	ValaStruct* _tmp37_;
	ValaDataType* _tmp40_;
	ValaClass* _tmp39_;
	ValaStruct* unichar_struct;
	ValaCCodeDeclarationSpace* _tmp62_;
	ValaCCodeDeclarationSpace* _tmp63_;
	GeeList* source_files;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (context != NULL);
	vala_ccode_base_module_set_context (self, context);
	self->root_symbol = (_tmp0_ = _vala_code_node_ref0 ((ValaSymbol*) vala_code_context_get_root (context)), _vala_code_node_unref0 (self->root_symbol), _tmp0_);
	self->bool_type = (_tmp2_ = (ValaDataType*) vala_boolean_type_new (_tmp1_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "bool"))), _vala_code_node_unref0 (self->bool_type), _tmp2_);
	_vala_code_node_unref0 (_tmp1_);
	self->char_type = (_tmp4_ = (ValaDataType*) vala_integer_type_new (_tmp3_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "char")), NULL, NULL), _vala_code_node_unref0 (self->char_type), _tmp4_);
	_vala_code_node_unref0 (_tmp3_);
	self->uchar_type = (_tmp6_ = (ValaDataType*) vala_integer_type_new (_tmp5_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "uchar")), NULL, NULL), _vala_code_node_unref0 (self->uchar_type), _tmp6_);
	_vala_code_node_unref0 (_tmp5_);
	self->short_type = (_tmp8_ = (ValaDataType*) vala_integer_type_new (_tmp7_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "short")), NULL, NULL), _vala_code_node_unref0 (self->short_type), _tmp8_);
	_vala_code_node_unref0 (_tmp7_);
	self->ushort_type = (_tmp10_ = (ValaDataType*) vala_integer_type_new (_tmp9_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "ushort")), NULL, NULL), _vala_code_node_unref0 (self->ushort_type), _tmp10_);
	_vala_code_node_unref0 (_tmp9_);
	self->int_type = (_tmp12_ = (ValaDataType*) vala_integer_type_new (_tmp11_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "int")), NULL, NULL), _vala_code_node_unref0 (self->int_type), _tmp12_);
	_vala_code_node_unref0 (_tmp11_);
	self->uint_type = (_tmp14_ = (ValaDataType*) vala_integer_type_new (_tmp13_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "uint")), NULL, NULL), _vala_code_node_unref0 (self->uint_type), _tmp14_);
	_vala_code_node_unref0 (_tmp13_);
	self->long_type = (_tmp16_ = (ValaDataType*) vala_integer_type_new (_tmp15_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "long")), NULL, NULL), _vala_code_node_unref0 (self->long_type), _tmp16_);
	_vala_code_node_unref0 (_tmp15_);
	self->ulong_type = (_tmp18_ = (ValaDataType*) vala_integer_type_new (_tmp17_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "ulong")), NULL, NULL), _vala_code_node_unref0 (self->ulong_type), _tmp18_);
	_vala_code_node_unref0 (_tmp17_);
	self->int8_type = (_tmp20_ = (ValaDataType*) vala_integer_type_new (_tmp19_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "int8")), NULL, NULL), _vala_code_node_unref0 (self->int8_type), _tmp20_);
	_vala_code_node_unref0 (_tmp19_);
	self->uint8_type = (_tmp22_ = (ValaDataType*) vala_integer_type_new (_tmp21_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "uint8")), NULL, NULL), _vala_code_node_unref0 (self->uint8_type), _tmp22_);
	_vala_code_node_unref0 (_tmp21_);
	self->int16_type = (_tmp24_ = (ValaDataType*) vala_integer_type_new (_tmp23_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "int16")), NULL, NULL), _vala_code_node_unref0 (self->int16_type), _tmp24_);
	_vala_code_node_unref0 (_tmp23_);
	self->uint16_type = (_tmp26_ = (ValaDataType*) vala_integer_type_new (_tmp25_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "uint16")), NULL, NULL), _vala_code_node_unref0 (self->uint16_type), _tmp26_);
	_vala_code_node_unref0 (_tmp25_);
	self->int32_type = (_tmp28_ = (ValaDataType*) vala_integer_type_new (_tmp27_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "int32")), NULL, NULL), _vala_code_node_unref0 (self->int32_type), _tmp28_);
	_vala_code_node_unref0 (_tmp27_);
	self->uint32_type = (_tmp30_ = (ValaDataType*) vala_integer_type_new (_tmp29_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "uint32")), NULL, NULL), _vala_code_node_unref0 (self->uint32_type), _tmp30_);
	_vala_code_node_unref0 (_tmp29_);
	self->int64_type = (_tmp32_ = (ValaDataType*) vala_integer_type_new (_tmp31_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "int64")), NULL, NULL), _vala_code_node_unref0 (self->int64_type), _tmp32_);
	_vala_code_node_unref0 (_tmp31_);
	self->uint64_type = (_tmp34_ = (ValaDataType*) vala_integer_type_new (_tmp33_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "uint64")), NULL, NULL), _vala_code_node_unref0 (self->uint64_type), _tmp34_);
	_vala_code_node_unref0 (_tmp33_);
	self->float_type = (_tmp36_ = (ValaDataType*) vala_floating_type_new (_tmp35_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "float"))), _vala_code_node_unref0 (self->float_type), _tmp36_);
	_vala_code_node_unref0 (_tmp35_);
	self->double_type = (_tmp38_ = (ValaDataType*) vala_floating_type_new (_tmp37_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "double"))), _vala_code_node_unref0 (self->double_type), _tmp38_);
	_vala_code_node_unref0 (_tmp37_);
	self->string_type = (_tmp40_ = (ValaDataType*) vala_object_type_new ((ValaObjectTypeSymbol*) (_tmp39_ = VALA_CLASS (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "string")))), _vala_code_node_unref0 (self->string_type), _tmp40_);
	_vala_code_node_unref0 (_tmp39_);
	unichar_struct = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "unichar"));
	if (unichar_struct != NULL) {
		ValaDataType* _tmp41_;
		self->unichar_type = (_tmp41_ = (ValaDataType*) vala_integer_type_new (unichar_struct, NULL, NULL), _vala_code_node_unref0 (self->unichar_type), _tmp41_);
	}
	if (vala_code_context_get_profile (context) == VALA_PROFILE_GOBJECT) {
		ValaSymbol* glib_ns;
		ValaTypeSymbol* _tmp42_;
		ValaTypeSymbol* _tmp43_;
		ValaErrorType* _tmp44_;
		ValaClass* _tmp45_;
		ValaClass* _tmp46_;
		ValaClass* _tmp47_;
		ValaTypeSymbol* _tmp48_;
		ValaTypeSymbol* _tmp49_;
		ValaTypeSymbol* _tmp50_;
		ValaTypeSymbol* _tmp51_;
		ValaDataType* _tmp53_;
		ValaStruct* _tmp52_;
		ValaStruct* _tmp54_;
		ValaStruct* _tmp55_;
		ValaTypeSymbol* _tmp56_;
		ValaSymbol* dbus_ns;
		glib_ns = vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "GLib");
		self->gtype_type = (_tmp42_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "Type")), _vala_code_node_unref0 (self->gtype_type), _tmp42_);
		self->gobject_type = (_tmp43_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "Object")), _vala_code_node_unref0 (self->gobject_type), _tmp43_);
		self->gerror_type = (_tmp44_ = vala_error_type_new (NULL, NULL, NULL), _vala_code_node_unref0 (self->gerror_type), _tmp44_);
		self->glist_type = (_tmp45_ = VALA_CLASS (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "List")), _vala_code_node_unref0 (self->glist_type), _tmp45_);
		self->gslist_type = (_tmp46_ = VALA_CLASS (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "SList")), _vala_code_node_unref0 (self->gslist_type), _tmp46_);
		self->gvaluearray_type = (_tmp47_ = VALA_CLASS (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "ValueArray")), _vala_code_node_unref0 (self->gvaluearray_type), _tmp47_);
		self->gstringbuilder_type = (_tmp48_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "StringBuilder")), _vala_code_node_unref0 (self->gstringbuilder_type), _tmp48_);
		self->garray_type = (_tmp49_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "Array")), _vala_code_node_unref0 (self->garray_type), _tmp49_);
		self->gbytearray_type = (_tmp50_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "ByteArray")), _vala_code_node_unref0 (self->gbytearray_type), _tmp50_);
		self->gptrarray_type = (_tmp51_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "PtrArray")), _vala_code_node_unref0 (self->gptrarray_type), _tmp51_);
		self->gquark_type = (_tmp53_ = (ValaDataType*) vala_integer_type_new (_tmp52_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "Quark")), NULL, NULL), _vala_code_node_unref0 (self->gquark_type), _tmp53_);
		_vala_code_node_unref0 (_tmp52_);
		self->gvalue_type = (_tmp54_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "Value")), _vala_code_node_unref0 (self->gvalue_type), _tmp54_);
		self->mutex_type = (_tmp55_ = VALA_STRUCT (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "StaticRecMutex")), _vala_code_node_unref0 (self->mutex_type), _tmp55_);
		self->type_module_type = (_tmp56_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (glib_ns), "TypeModule")), _vala_code_node_unref0 (self->type_module_type), _tmp56_);
		if (vala_code_context_get_module_init_method (context) != NULL) {
			ValaCCodeFragment* _tmp57_;
			self->module_init_fragment = (_tmp57_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (self->module_init_fragment), _tmp57_);
			{
				GeeList* _tmp58_;
				GeeIterator* _tmp59_;
				GeeIterator* _parameter_it;
				_parameter_it = (_tmp59_ = gee_iterable_iterator ((GeeIterable*) (_tmp58_ = vala_method_get_parameters (vala_code_context_get_module_init_method (context)))), _gee_collection_object_unref0 (_tmp58_), _tmp59_);
				while (TRUE) {
					ValaFormalParameter* parameter;
					if (!gee_iterator_next (_parameter_it)) {
						break;
					}
					parameter = (ValaFormalParameter*) gee_iterator_get (_parameter_it);
					if (vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (parameter)) == self->type_module_type) {
						char* _tmp60_;
						self->in_plugin = TRUE;
						self->module_init_param_name = (_tmp60_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) parameter)), _g_free0 (self->module_init_param_name), _tmp60_);
						_vala_code_node_unref0 (parameter);
						break;
					}
					_vala_code_node_unref0 (parameter);
				}
				_gee_collection_object_unref0 (_parameter_it);
			}
		}
		dbus_ns = vala_scope_lookup (vala_symbol_get_scope (self->root_symbol), "DBus");
		if (dbus_ns != NULL) {
			ValaTypeSymbol* _tmp61_;
			self->dbus_object_type = (_tmp61_ = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (dbus_ns), "Object")), _vala_code_node_unref0 (self->dbus_object_type), _tmp61_);
		}
		_vala_code_node_unref0 (glib_ns);
		_vala_code_node_unref0 (dbus_ns);
	}
	self->header_declarations = (_tmp62_ = vala_ccode_declaration_space_new (), _vala_ccode_declaration_space_unref0 (self->header_declarations), _tmp62_);
	self->internal_header_declarations = (_tmp63_ = vala_ccode_declaration_space_new (), _vala_ccode_declaration_space_unref0 (self->internal_header_declarations), _tmp63_);
	source_files = vala_code_context_get_source_files (context);
	{
		GeeIterator* _file_it;
		_file_it = gee_iterable_iterator ((GeeIterable*) source_files);
		while (TRUE) {
			ValaSourceFile* file;
			if (!gee_iterator_next (_file_it)) {
				break;
			}
			file = (ValaSourceFile*) gee_iterator_get (_file_it);
			if (!vala_source_file_get_external_package (file)) {
				vala_source_file_accept (file, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
			}
			_vala_source_file_unref0 (file);
		}
		_gee_collection_object_unref0 (_file_it);
	}
	if (vala_code_context_get_header_filename (context) != NULL) {
		ValaCCodeWriter* writer;
		char* _tmp65_;
		ValaCCodeOnceSection* _tmp66_;
		ValaCCodeOnceSection* once;
		ValaCCodeNewline* _tmp67_;
		ValaCCodeNewline* _tmp68_;
		ValaCCodeNewline* _tmp71_;
		ValaCCodeNewline* _tmp72_;
		ValaCCodeNewline* _tmp73_;
		ValaCCodeNewline* _tmp74_;
		ValaCCodeNewline* _tmp75_;
		ValaCCodeNewline* _tmp78_;
		writer = vala_ccode_writer_new (vala_code_context_get_header_filename (context));
		if (!vala_ccode_writer_open (writer)) {
			char* _tmp64_;
			vala_report_error (NULL, _tmp64_ = g_strdup_printf ("unable to open `%s' for writing", vala_ccode_writer_get_filename (writer)));
			_g_free0 (_tmp64_);
			_vala_ccode_writer_unref0 (writer);
			_vala_code_node_unref0 (unichar_struct);
			_gee_collection_object_unref0 (source_files);
			return;
		}
		vala_ccode_writer_write_newline (writer);
		once = (_tmp66_ = vala_ccode_once_section_new (_tmp65_ = vala_ccode_base_module_get_define_for_filename (vala_ccode_writer_get_filename (writer))), _g_free0 (_tmp65_), _tmp66_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp67_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp67_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->header_declarations->include_directives);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp68_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp68_);
		if (vala_code_context_get_profile (context) == VALA_PROFILE_GOBJECT) {
			ValaCCodeIdentifier* _tmp69_;
			ValaCCodeNewline* _tmp70_;
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp69_ = vala_ccode_identifier_new ("G_BEGIN_DECLS")));
			_vala_ccode_node_unref0 (_tmp69_);
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp70_ = vala_ccode_newline_new ()));
			_vala_ccode_node_unref0 (_tmp70_);
		}
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp71_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp71_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->header_declarations->type_declaration);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp72_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp72_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->header_declarations->type_definition);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp73_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp73_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->header_declarations->type_member_declaration);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp74_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp74_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->header_declarations->constant_declaration);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp75_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp75_);
		if (vala_code_context_get_profile (context) == VALA_PROFILE_GOBJECT) {
			ValaCCodeIdentifier* _tmp76_;
			ValaCCodeNewline* _tmp77_;
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp76_ = vala_ccode_identifier_new ("G_END_DECLS")));
			_vala_ccode_node_unref0 (_tmp76_);
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp77_ = vala_ccode_newline_new ()));
			_vala_ccode_node_unref0 (_tmp77_);
		}
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp78_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp78_);
		vala_ccode_node_write ((ValaCCodeNode*) once, writer);
		vala_ccode_writer_close (writer);
		_vala_ccode_writer_unref0 (writer);
		_vala_ccode_node_unref0 (once);
	}
	if (vala_code_context_get_internal_header_filename (context) != NULL) {
		ValaCCodeWriter* writer;
		char* _tmp80_;
		ValaCCodeOnceSection* _tmp81_;
		ValaCCodeOnceSection* once;
		ValaCCodeNewline* _tmp82_;
		ValaCCodeNewline* _tmp83_;
		ValaCCodeNewline* _tmp86_;
		ValaCCodeNewline* _tmp87_;
		ValaCCodeNewline* _tmp88_;
		ValaCCodeNewline* _tmp89_;
		ValaCCodeNewline* _tmp90_;
		ValaCCodeNewline* _tmp93_;
		writer = vala_ccode_writer_new (vala_code_context_get_internal_header_filename (context));
		if (!vala_ccode_writer_open (writer)) {
			char* _tmp79_;
			vala_report_error (NULL, _tmp79_ = g_strdup_printf ("unable to open `%s' for writing", vala_ccode_writer_get_filename (writer)));
			_g_free0 (_tmp79_);
			_vala_ccode_writer_unref0 (writer);
			_vala_code_node_unref0 (unichar_struct);
			_gee_collection_object_unref0 (source_files);
			return;
		}
		vala_ccode_writer_write_newline (writer);
		once = (_tmp81_ = vala_ccode_once_section_new (_tmp80_ = vala_ccode_base_module_get_define_for_filename (vala_ccode_writer_get_filename (writer))), _g_free0 (_tmp80_), _tmp81_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp82_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp82_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->internal_header_declarations->include_directives);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp83_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp83_);
		if (vala_code_context_get_profile (context) == VALA_PROFILE_GOBJECT) {
			ValaCCodeIdentifier* _tmp84_;
			ValaCCodeNewline* _tmp85_;
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp84_ = vala_ccode_identifier_new ("G_BEGIN_DECLS")));
			_vala_ccode_node_unref0 (_tmp84_);
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp85_ = vala_ccode_newline_new ()));
			_vala_ccode_node_unref0 (_tmp85_);
		}
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp86_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp86_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->internal_header_declarations->type_declaration);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp87_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp87_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->internal_header_declarations->type_definition);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp88_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp88_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->internal_header_declarations->type_member_declaration);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp89_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp89_);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) self->internal_header_declarations->constant_declaration);
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp90_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp90_);
		if (vala_code_context_get_profile (context) == VALA_PROFILE_GOBJECT) {
			ValaCCodeIdentifier* _tmp91_;
			ValaCCodeNewline* _tmp92_;
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp91_ = vala_ccode_identifier_new ("G_END_DECLS")));
			_vala_ccode_node_unref0 (_tmp91_);
			vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp92_ = vala_ccode_newline_new ()));
			_vala_ccode_node_unref0 (_tmp92_);
		}
		vala_ccode_fragment_append ((ValaCCodeFragment*) once, (ValaCCodeNode*) (_tmp93_ = vala_ccode_newline_new ()));
		_vala_ccode_node_unref0 (_tmp93_);
		vala_ccode_node_write ((ValaCCodeNode*) once, writer);
		vala_ccode_writer_close (writer);
		_vala_ccode_writer_unref0 (writer);
		_vala_ccode_node_unref0 (once);
	}
	_vala_code_node_unref0 (unichar_struct);
	_gee_collection_object_unref0 (source_files);
}


static ValaCCodeIdentifier* vala_ccode_base_module_real_get_value_setter_function (ValaCCodeModule* base, ValaDataType* type_reference) {
	ValaCCodeBaseModule * self;
	ValaCCodeIdentifier* result;
	ValaDataType* _tmp0_;
	ValaArrayType* array_type;
	self = (ValaCCodeBaseModule*) base;
	g_return_val_if_fail (type_reference != NULL, NULL);
	array_type = _vala_code_node_ref0 ((_tmp0_ = type_reference, VALA_IS_ARRAY_TYPE (_tmp0_) ? ((ValaArrayType*) _tmp0_) : NULL));
	if (vala_data_type_get_data_type (type_reference) != NULL) {
		char* _tmp1_;
		ValaCCodeIdentifier* _tmp2_;
		result = (_tmp2_ = vala_ccode_identifier_new (_tmp1_ = vala_typesymbol_get_set_value_function (vala_data_type_get_data_type (type_reference))), _g_free0 (_tmp1_), _tmp2_);
		_vala_code_node_unref0 (array_type);
		return result;
	} else {
		gboolean _tmp3_;
		_tmp3_ = FALSE;
		if (array_type != NULL) {
			_tmp3_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (self->string_type);
		} else {
			_tmp3_ = FALSE;
		}
		if (_tmp3_) {
			result = vala_ccode_identifier_new ("g_value_set_boxed");
			_vala_code_node_unref0 (array_type);
			return result;
		} else {
			result = vala_ccode_identifier_new ("g_value_set_pointer");
			_vala_code_node_unref0 (array_type);
			return result;
		}
	}
	_vala_code_node_unref0 (array_type);
}


static ValaCCodeIdentifier* vala_ccode_base_module_get_value_getter_function (ValaCCodeBaseModule* self, ValaDataType* type_reference) {
	ValaCCodeIdentifier* result;
	ValaDataType* _tmp0_;
	ValaArrayType* array_type;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type_reference != NULL, NULL);
	array_type = _vala_code_node_ref0 ((_tmp0_ = type_reference, VALA_IS_ARRAY_TYPE (_tmp0_) ? ((ValaArrayType*) _tmp0_) : NULL));
	if (vala_data_type_get_data_type (type_reference) != NULL) {
		char* _tmp1_;
		ValaCCodeIdentifier* _tmp2_;
		result = (_tmp2_ = vala_ccode_identifier_new (_tmp1_ = vala_typesymbol_get_get_value_function (vala_data_type_get_data_type (type_reference))), _g_free0 (_tmp1_), _tmp2_);
		_vala_code_node_unref0 (array_type);
		return result;
	} else {
		gboolean _tmp3_;
		_tmp3_ = FALSE;
		if (array_type != NULL) {
			_tmp3_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (self->string_type);
		} else {
			_tmp3_ = FALSE;
		}
		if (_tmp3_) {
			result = vala_ccode_identifier_new ("g_value_get_boxed");
			_vala_code_node_unref0 (array_type);
			return result;
		} else {
			result = vala_ccode_identifier_new ("g_value_get_pointer");
			_vala_code_node_unref0 (array_type);
			return result;
		}
	}
	_vala_code_node_unref0 (array_type);
}


static void vala_ccode_base_module_real_append_vala_array_free (ValaCCodeBaseModule* self) {
	g_return_if_fail (self != NULL);
}


void vala_ccode_base_module_append_vala_array_free (ValaCCodeBaseModule* self) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->append_vala_array_free (self);
}


static void vala_ccode_base_module_real_append_vala_array_move (ValaCCodeBaseModule* self) {
	g_return_if_fail (self != NULL);
}


void vala_ccode_base_module_append_vala_array_move (ValaCCodeBaseModule* self) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->append_vala_array_move (self);
}


static void vala_ccode_base_module_real_append_vala_array_length (ValaCCodeBaseModule* self) {
	g_return_if_fail (self != NULL);
}


void vala_ccode_base_module_append_vala_array_length (ValaCCodeBaseModule* self) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->append_vala_array_length (self);
}


static void vala_ccode_base_module_append_vala_strcmp0 (ValaCCodeBaseModule* self) {
	ValaCCodeFunction* fun;
	ValaCCodeFormalParameter* _tmp0_;
	ValaCCodeFormalParameter* _tmp1_;
	ValaCCodeFunction* _tmp2_;
	ValaCCodeIdentifier* _tmp4_;
	ValaCCodeIdentifier* _tmp3_;
	ValaCCodeBinaryExpression* _tmp5_;
	ValaCCodeBinaryExpression* cineq;
	ValaCCodeBlock* _tmp6_;
	ValaCCodeBlock* cblock;
	ValaCCodeBinaryExpression* _tmp9_;
	ValaCCodeConstant* _tmp8_;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeIfStatement* _tmp10_;
	ValaCCodeIfStatement* cif;
	ValaCCodeReturnStatement* _tmp12_;
	ValaCCodeUnaryExpression* _tmp11_;
	ValaCCodeBlock* _tmp13_;
	ValaCCodeIfStatement* _tmp17_;
	ValaCCodeBinaryExpression* _tmp16_;
	ValaCCodeConstant* _tmp15_;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeReturnStatement* _tmp18_;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeFunctionCall* _tmp20_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp21_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeReturnStatement* _tmp23_;
	g_return_if_fail (self != NULL);
	fun = vala_ccode_function_new ("_vala_strcmp0", "int");
	vala_ccode_function_set_modifiers (fun, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (fun, _tmp0_ = vala_ccode_formal_parameter_new ("str1", "const char *"));
	_vala_ccode_node_unref0 (_tmp0_);
	vala_ccode_function_add_parameter (fun, _tmp1_ = vala_ccode_formal_parameter_new ("str2", "const char *"));
	_vala_ccode_node_unref0 (_tmp1_);
	vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp2_ = vala_ccode_function_copy (fun)));
	_vala_ccode_node_unref0 (_tmp2_);
	cineq = (_tmp5_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("str1")), (ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("str2"))), _vala_ccode_node_unref0 (_tmp4_), _vala_ccode_node_unref0 (_tmp3_), _tmp5_);
	vala_ccode_function_set_block (fun, _tmp6_ = vala_ccode_block_new ());
	_vala_ccode_node_unref0 (_tmp6_);
	cblock = vala_ccode_block_new ();
	cif = (_tmp10_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp9_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("str1")), (ValaCCodeExpression*) (_tmp8_ = vala_ccode_constant_new ("NULL")))), (ValaCCodeStatement*) cblock, NULL), _vala_ccode_node_unref0 (_tmp9_), _vala_ccode_node_unref0 (_tmp8_), _vala_ccode_node_unref0 (_tmp7_), _tmp10_);
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp12_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_MINUS, (ValaCCodeExpression*) cineq)))));
	_vala_ccode_node_unref0 (_tmp12_);
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (fun), (ValaCCodeNode*) cif);
	cblock = (_tmp13_ = vala_ccode_block_new (), _vala_ccode_node_unref0 (cblock), _tmp13_);
	cif = (_tmp17_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("str2")), (ValaCCodeExpression*) (_tmp15_ = vala_ccode_constant_new ("NULL")))), (ValaCCodeStatement*) cblock, NULL), _vala_ccode_node_unref0 (cif), _tmp17_);
	_vala_ccode_node_unref0 (_tmp16_);
	_vala_ccode_node_unref0 (_tmp15_);
	_vala_ccode_node_unref0 (_tmp14_);
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp18_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) cineq)));
	_vala_ccode_node_unref0 (_tmp18_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (fun), (ValaCCodeNode*) cif);
	ccall = (_tmp20_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp19_), _tmp20_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("str1")));
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("str2")));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (fun), (ValaCCodeNode*) (_tmp23_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) fun);
	_vala_ccode_node_unref0 (fun);
	_vala_ccode_node_unref0 (cineq);
	_vala_ccode_node_unref0 (cblock);
	_vala_ccode_node_unref0 (cif);
	_vala_ccode_node_unref0 (ccall);
}


static void vala_ccode_base_module_real_visit_source_file (ValaCCodeModule* base, ValaSourceFile* source_file) {
	ValaCCodeBaseModule * self;
	ValaCCodeDeclarationSpace* _tmp0_;
	ValaCCodeFragment* _tmp1_;
	ValaCCodeFragment* _tmp2_;
	ValaCCodeFragment* _tmp3_;
	GeeSet* _tmp4_;
	GeeSet* _tmp5_;
	GeeSet* _tmp6_;
	gboolean _tmp7_;
	char* _tmp61_;
	ValaCCodeWriter* _tmp62_;
	ValaCCodeWriter* writer;
	GeeList* comments;
	ValaCCodeDeclarationSpace* _tmp64_;
	ValaCCodeFragment* _tmp65_;
	ValaCCodeFragment* _tmp66_;
	ValaCCodeFragment* _tmp67_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (source_file != NULL);
	self->source_declarations = (_tmp0_ = vala_ccode_declaration_space_new (), _vala_ccode_declaration_space_unref0 (self->source_declarations), _tmp0_);
	self->source_type_member_definition = (_tmp1_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (self->source_type_member_definition), _tmp1_);
	self->source_signal_marshaller_definition = (_tmp2_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (self->source_signal_marshaller_definition), _tmp2_);
	self->source_signal_marshaller_declaration = (_tmp3_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (self->source_signal_marshaller_declaration), _tmp3_);
	self->user_marshal_set = (_tmp4_ = (GeeSet*) gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal), _gee_collection_object_unref0 (self->user_marshal_set), _tmp4_);
	self->next_temp_var_id = 0;
	gee_map_clear (self->variable_name_map);
	self->gvaluecollector_h_needed = FALSE;
	self->dbus_glib_h_needed = FALSE;
	self->dbus_glib_h_needed_in_header = FALSE;
	self->requires_array_free = FALSE;
	self->requires_array_move = FALSE;
	self->requires_array_length = FALSE;
	self->requires_strcmp0 = FALSE;
	self->wrappers = (_tmp5_ = (GeeSet*) gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal), _gee_collection_object_unref0 (self->wrappers), _tmp5_);
	self->priv->generated_external_symbols = (_tmp6_ = (GeeSet*) gee_hash_set_new (VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_hash, g_direct_equal), _gee_collection_object_unref0 (self->priv->generated_external_symbols), _tmp6_);
	if (vala_code_context_get_profile (self->priv->_context) == VALA_PROFILE_GOBJECT) {
		vala_ccode_declaration_space_add_include (self->header_declarations, "glib.h", FALSE);
		vala_ccode_declaration_space_add_include (self->internal_header_declarations, "glib.h", FALSE);
		vala_ccode_declaration_space_add_include (self->source_declarations, "glib.h", FALSE);
		vala_ccode_declaration_space_add_include (self->source_declarations, "glib-object.h", FALSE);
	}
	vala_source_file_accept_children (source_file, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	if (vala_report_get_errors (vala_code_context_get_report (self->priv->_context)) > 0) {
		return;
	}
	if (self->requires_array_free) {
		vala_ccode_base_module_append_vala_array_free (self);
	}
	if (self->requires_array_move) {
		vala_ccode_base_module_append_vala_array_move (self);
	}
	if (self->requires_array_length) {
		vala_ccode_base_module_append_vala_array_length (self);
	}
	if (self->requires_strcmp0) {
		vala_ccode_base_module_append_vala_strcmp0 (self);
	}
	if (self->gvaluecollector_h_needed) {
		vala_ccode_declaration_space_add_include (self->source_declarations, "gobject/gvaluecollector.h", FALSE);
	}
	if (self->dbus_glib_h_needed) {
		vala_ccode_declaration_space_add_include (self->source_declarations, "dbus/dbus.h", FALSE);
		vala_ccode_declaration_space_add_include (self->source_declarations, "dbus/dbus-glib.h", FALSE);
		vala_ccode_declaration_space_add_include (self->source_declarations, "dbus/dbus-glib-lowlevel.h", FALSE);
	}
	_tmp7_ = FALSE;
	if (self->dbus_glib_h_needed_in_header) {
		_tmp7_ = TRUE;
	} else {
		_tmp7_ = self->dbus_glib_h_needed;
	}
	if (_tmp7_) {
		ValaCCodeStruct* dbusvtable;
		ValaCCodeTypeDefinition* _tmp9_;
		ValaCCodeVariableDeclarator* _tmp8_;
		ValaCCodeFunction* cfunc;
		ValaCCodeFormalParameter* _tmp10_;
		ValaCCodeFormalParameter* _tmp11_;
		ValaCCodeFormalParameter* _tmp12_;
		ValaCCodeFunction* _tmp13_;
		ValaCCodeBlock* block;
		ValaCCodeDeclaration* _cdecl_;
		ValaCCodeVariableDeclarator* _tmp14_;
		ValaCCodeIdentifier* _tmp15_;
		ValaCCodeFunctionCall* _tmp16_;
		ValaCCodeFunctionCall* quark;
		ValaCCodeConstant* _tmp17_;
		ValaCCodeIdentifier* _tmp18_;
		ValaCCodeFunctionCall* _tmp19_;
		ValaCCodeFunctionCall* get_qdata;
		ValaCCodeIdentifier* _tmp20_;
		ValaCCodeExpressionStatement* _tmp23_;
		ValaCCodeAssignment* _tmp22_;
		ValaCCodeIdentifier* _tmp21_;
		ValaCCodeMemberAccess* _tmp25_;
		ValaCCodeIdentifier* _tmp24_;
		ValaCCodeFunctionCall* _tmp26_;
		ValaCCodeFunctionCall* cregister;
		ValaCCodeIdentifier* _tmp27_;
		ValaCCodeIdentifier* _tmp28_;
		ValaCCodeIdentifier* _tmp29_;
		ValaCCodeBlock* ifblock;
		ValaCCodeExpressionStatement* _tmp30_;
		ValaCCodeBlock* elseblock;
		ValaCCodeIdentifier* _tmp31_;
		ValaCCodeFunctionCall* _tmp32_;
		ValaCCodeFunctionCall* warn;
		ValaCCodeConstant* _tmp33_;
		ValaCCodeExpressionStatement* _tmp34_;
		ValaCCodeIfStatement* _tmp36_;
		ValaCCodeIdentifier* _tmp35_;
		ValaCCodeFunction* _tmp37_;
		ValaCCodeFormalParameter* _tmp38_;
		ValaCCodeFormalParameter* _tmp39_;
		ValaCCodeFunction* _tmp40_;
		ValaCCodeBlock* _tmp41_;
		ValaCCodeDeclaration* _tmp42_;
		ValaCCodeVariableDeclarator* _tmp43_;
		ValaCCodeIdentifier* _tmp44_;
		ValaCCodeFunctionCall* _tmp45_;
		ValaCCodeFunctionCall* path;
		ValaCCodeCastExpression* _tmp47_;
		ValaCCodeIdentifier* _tmp46_;
		ValaCCodeConstant* _tmp48_;
		ValaCCodeExpressionStatement* _tmp51_;
		ValaCCodeAssignment* _tmp50_;
		ValaCCodeIdentifier* _tmp49_;
		ValaCCodeIdentifier* _tmp52_;
		ValaCCodeFunctionCall* _tmp53_;
		ValaCCodeFunctionCall* unregister_call;
		ValaCCodeIdentifier* _tmp54_;
		ValaCCodeIdentifier* _tmp55_;
		ValaCCodeExpressionStatement* _tmp56_;
		ValaCCodeIdentifier* _tmp57_;
		ValaCCodeFunctionCall* _tmp58_;
		ValaCCodeFunctionCall* path_free;
		ValaCCodeIdentifier* _tmp59_;
		ValaCCodeExpressionStatement* _tmp60_;
		dbusvtable = vala_ccode_struct_new ("_DBusObjectVTable");
		vala_ccode_struct_add_field (dbusvtable, "void", "(*register_object) (DBusConnection*, const char*, void*)");
		vala_ccode_declaration_space_add_type_definition (self->source_declarations, (ValaCCodeNode*) dbusvtable);
		vala_ccode_declaration_space_add_type_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp9_ = vala_ccode_type_definition_new ("struct _DBusObjectVTable", (ValaCCodeDeclarator*) (_tmp8_ = vala_ccode_variable_declarator_new ("_DBusObjectVTable", NULL, NULL)))));
		_vala_ccode_node_unref0 (_tmp9_);
		_vala_ccode_node_unref0 (_tmp8_);
		cfunc = vala_ccode_function_new ("_vala_dbus_register_object", "void");
		vala_ccode_function_add_parameter (cfunc, _tmp10_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
		_vala_ccode_node_unref0 (_tmp10_);
		vala_ccode_function_add_parameter (cfunc, _tmp11_ = vala_ccode_formal_parameter_new ("path", "const char*"));
		_vala_ccode_node_unref0 (_tmp11_);
		vala_ccode_function_add_parameter (cfunc, _tmp12_ = vala_ccode_formal_parameter_new ("object", "void*"));
		_vala_ccode_node_unref0 (_tmp12_);
		vala_ccode_function_set_modifiers (cfunc, vala_ccode_function_get_modifiers (cfunc) | VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp13_ = vala_ccode_function_copy (cfunc)));
		_vala_ccode_node_unref0 (_tmp13_);
		block = vala_ccode_block_new ();
		vala_ccode_function_set_block (cfunc, block);
		_cdecl_ = vala_ccode_declaration_new ("const _DBusObjectVTable *");
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp14_ = vala_ccode_variable_declarator_new ("vtable", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp14_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
		quark = (_tmp16_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("g_quark_from_static_string"))), _vala_ccode_node_unref0 (_tmp15_), _tmp16_);
		vala_ccode_function_call_add_argument (quark, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_constant_new ("\"DBusObjectVTable\"")));
		_vala_ccode_node_unref0 (_tmp17_);
		get_qdata = (_tmp19_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_identifier_new ("g_type_get_qdata"))), _vala_ccode_node_unref0 (_tmp18_), _tmp19_);
		vala_ccode_function_call_add_argument (get_qdata, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("G_TYPE_FROM_INSTANCE (object)")));
		_vala_ccode_node_unref0 (_tmp20_);
		vala_ccode_function_call_add_argument (get_qdata, (ValaCCodeExpression*) quark);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp23_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("vtable")), (ValaCCodeExpression*) get_qdata, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp23_);
		_vala_ccode_node_unref0 (_tmp22_);
		_vala_ccode_node_unref0 (_tmp21_);
		cregister = (_tmp26_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp25_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("vtable")), "register_object"))), _vala_ccode_node_unref0 (_tmp25_), _vala_ccode_node_unref0 (_tmp24_), _tmp26_);
		vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("connection")));
		_vala_ccode_node_unref0 (_tmp27_);
		vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("path")));
		_vala_ccode_node_unref0 (_tmp28_);
		vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("object")));
		_vala_ccode_node_unref0 (_tmp29_);
		ifblock = vala_ccode_block_new ();
		vala_ccode_block_add_statement (ifblock, (ValaCCodeNode*) (_tmp30_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cregister)));
		_vala_ccode_node_unref0 (_tmp30_);
		elseblock = vala_ccode_block_new ();
		warn = (_tmp32_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("g_warning"))), _vala_ccode_node_unref0 (_tmp31_), _tmp32_);
		vala_ccode_function_call_add_argument (warn, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_constant_new ("\"Object does not implement any D-Bus interface\"")));
		_vala_ccode_node_unref0 (_tmp33_);
		vala_ccode_block_add_statement (elseblock, (ValaCCodeNode*) (_tmp34_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) warn)));
		_vala_ccode_node_unref0 (_tmp34_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp36_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new ("vtable")), (ValaCCodeStatement*) ifblock, (ValaCCodeStatement*) elseblock)));
		_vala_ccode_node_unref0 (_tmp36_);
		_vala_ccode_node_unref0 (_tmp35_);
		vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) cfunc);
		cfunc = (_tmp37_ = vala_ccode_function_new ("_vala_dbus_unregister_object", "void"), _vala_ccode_node_unref0 (cfunc), _tmp37_);
		vala_ccode_function_add_parameter (cfunc, _tmp38_ = vala_ccode_formal_parameter_new ("connection", "gpointer"));
		_vala_ccode_node_unref0 (_tmp38_);
		vala_ccode_function_add_parameter (cfunc, _tmp39_ = vala_ccode_formal_parameter_new ("object", "GObject*"));
		_vala_ccode_node_unref0 (_tmp39_);
		vala_ccode_function_set_modifiers (cfunc, vala_ccode_function_get_modifiers (cfunc) | VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp40_ = vala_ccode_function_copy (cfunc)));
		_vala_ccode_node_unref0 (_tmp40_);
		block = (_tmp41_ = vala_ccode_block_new (), _vala_ccode_node_unref0 (block), _tmp41_);
		vala_ccode_function_set_block (cfunc, block);
		_cdecl_ = (_tmp42_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp42_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp43_ = vala_ccode_variable_declarator_new ("path", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp43_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
		path = (_tmp45_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("g_object_steal_data"))), _vala_ccode_node_unref0 (_tmp44_), _tmp45_);
		vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp46_ = vala_ccode_identifier_new ("object")), "GObject*")));
		_vala_ccode_node_unref0 (_tmp47_);
		_vala_ccode_node_unref0 (_tmp46_);
		vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp48_ = vala_ccode_constant_new ("\"dbus_object_path\"")));
		_vala_ccode_node_unref0 (_tmp48_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp51_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp50_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("path")), (ValaCCodeExpression*) path, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp51_);
		_vala_ccode_node_unref0 (_tmp50_);
		_vala_ccode_node_unref0 (_tmp49_);
		unregister_call = (_tmp53_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp52_ = vala_ccode_identifier_new ("dbus_connection_unregister_object_path"))), _vala_ccode_node_unref0 (_tmp52_), _tmp53_);
		vala_ccode_function_call_add_argument (unregister_call, (ValaCCodeExpression*) (_tmp54_ = vala_ccode_identifier_new ("connection")));
		_vala_ccode_node_unref0 (_tmp54_);
		vala_ccode_function_call_add_argument (unregister_call, (ValaCCodeExpression*) (_tmp55_ = vala_ccode_identifier_new ("path")));
		_vala_ccode_node_unref0 (_tmp55_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp56_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) unregister_call)));
		_vala_ccode_node_unref0 (_tmp56_);
		path_free = (_tmp58_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp57_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp57_), _tmp58_);
		vala_ccode_function_call_add_argument (path_free, (ValaCCodeExpression*) (_tmp59_ = vala_ccode_identifier_new ("path")));
		_vala_ccode_node_unref0 (_tmp59_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp60_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) path_free)));
		_vala_ccode_node_unref0 (_tmp60_);
		vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) cfunc);
		_vala_ccode_node_unref0 (dbusvtable);
		_vala_ccode_node_unref0 (cfunc);
		_vala_ccode_node_unref0 (block);
		_vala_ccode_node_unref0 (_cdecl_);
		_vala_ccode_node_unref0 (quark);
		_vala_ccode_node_unref0 (get_qdata);
		_vala_ccode_node_unref0 (cregister);
		_vala_ccode_node_unref0 (ifblock);
		_vala_ccode_node_unref0 (elseblock);
		_vala_ccode_node_unref0 (warn);
		_vala_ccode_node_unref0 (path);
		_vala_ccode_node_unref0 (unregister_call);
		_vala_ccode_node_unref0 (path_free);
	}
	writer = (_tmp62_ = vala_ccode_writer_new (_tmp61_ = vala_source_file_get_csource_filename (source_file)), _g_free0 (_tmp61_), _tmp62_);
	if (!vala_ccode_writer_open (writer)) {
		char* _tmp63_;
		vala_report_error (NULL, _tmp63_ = g_strdup_printf ("unable to open `%s' for writing", vala_ccode_writer_get_filename (writer)));
		_g_free0 (_tmp63_);
		_vala_ccode_writer_unref0 (writer);
		return;
	}
	vala_ccode_writer_set_line_directives (writer, vala_code_context_get_debug (self->priv->_context));
	comments = vala_source_file_get_comments (source_file);
	if (comments != NULL) {
		{
			GeeIterator* _comment_it;
			_comment_it = gee_iterable_iterator ((GeeIterable*) comments);
			while (TRUE) {
				ValaComment* comment;
				ValaCCodeComment* ccomment;
				if (!gee_iterator_next (_comment_it)) {
					break;
				}
				comment = (ValaComment*) gee_iterator_get (_comment_it);
				ccomment = vala_ccode_comment_new (vala_comment_get_content (comment));
				vala_ccode_node_write ((ValaCCodeNode*) ccomment, writer);
				_vala_comment_unref0 (comment);
				_vala_ccode_node_unref0 (ccomment);
			}
			_gee_collection_object_unref0 (_comment_it);
		}
	}
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write ((ValaCCodeNode*) self->source_declarations->include_directives, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write_combined ((ValaCCodeNode*) self->source_declarations->type_declaration, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write_combined ((ValaCCodeNode*) self->source_declarations->type_definition, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write_declaration ((ValaCCodeNode*) self->source_declarations->type_member_declaration, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write ((ValaCCodeNode*) self->source_declarations->type_member_declaration, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write_combined ((ValaCCodeNode*) self->source_declarations->constant_declaration, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write_declaration ((ValaCCodeNode*) self->source_signal_marshaller_declaration, writer);
	vala_ccode_node_write ((ValaCCodeNode*) self->source_signal_marshaller_declaration, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write ((ValaCCodeNode*) self->source_type_member_definition, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write ((ValaCCodeNode*) self->source_signal_marshaller_definition, writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_writer_close (writer);
	self->source_declarations = (_tmp64_ = NULL, _vala_ccode_declaration_space_unref0 (self->source_declarations), _tmp64_);
	self->source_type_member_definition = (_tmp65_ = NULL, _vala_ccode_node_unref0 (self->source_type_member_definition), _tmp65_);
	self->source_signal_marshaller_definition = (_tmp66_ = NULL, _vala_ccode_node_unref0 (self->source_signal_marshaller_definition), _tmp66_);
	self->source_signal_marshaller_declaration = (_tmp67_ = NULL, _vala_ccode_node_unref0 (self->source_signal_marshaller_declaration), _tmp67_);
	_vala_ccode_writer_unref0 (writer);
	_gee_collection_object_unref0 (comments);
}


static char* vala_ccode_base_module_get_define_for_filename (const char* filename) {
	char* result;
	GString* define;
	char* i;
	g_return_val_if_fail (filename != NULL, NULL);
	define = g_string_new ("__");
	i = g_strdup (filename);
	while (TRUE) {
		gunichar c;
		gboolean _tmp0_;
		char* _tmp1_;
		if (!(g_utf8_strlen (i, -1) > 0)) {
			break;
		}
		c = g_utf8_get_char (i);
		_tmp0_ = FALSE;
		if (g_unichar_isalnum (c)) {
			_tmp0_ = c < 0x80;
		} else {
			_tmp0_ = FALSE;
		}
		if (_tmp0_) {
			g_string_append_unichar (define, g_unichar_toupper (c));
		} else {
			g_string_append_c (define, '_');
		}
		i = (_tmp1_ = g_strdup (g_utf8_next_char (i)), _g_free0 (i), _tmp1_);
	}
	g_string_append (define, "__");
	result = g_strdup (define->str);
	_g_string_free0 (define);
	_g_free0 (i);
	return result;
}


void vala_ccode_base_module_generate_enum_declaration (ValaCCodeBaseModule* self, ValaEnum* en, ValaCCodeDeclarationSpace* decl_space) {
	char* _tmp0_;
	gboolean _tmp1_;
	char* _tmp2_;
	ValaCCodeEnum* _tmp3_;
	ValaCCodeEnum* cenum;
	ValaCCodeNewline* _tmp10_;
	ValaCCodeNewline* _tmp11_;
	char* _tmp12_;
	char* _tmp13_;
	char* macro;
	ValaCCodeMacroReplacement* _tmp15_;
	char* _tmp14_;
	char* _tmp16_;
	char* _tmp17_;
	char* fun_name;
	ValaCCodeFunction* regfun;
	g_return_if_fail (self != NULL);
	g_return_if_fail (en != NULL);
	g_return_if_fail (decl_space != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_symbol_declaration (decl_space, (ValaSymbol*) en, _tmp0_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) en, FALSE)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
	cenum = (_tmp3_ = vala_ccode_enum_new (_tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) en, FALSE)), _g_free0 (_tmp2_), _tmp3_);
	{
		GeeList* _tmp4_;
		GeeIterator* _tmp5_;
		GeeIterator* _ev_it;
		_ev_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vala_enum_get_values (en))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
		while (TRUE) {
			ValaEnumValue* ev;
			if (!gee_iterator_next (_ev_it)) {
				break;
			}
			ev = (ValaEnumValue*) gee_iterator_get (_ev_it);
			if (vala_enum_value_get_value (ev) == NULL) {
				ValaCCodeEnumValue* _tmp7_;
				char* _tmp6_;
				vala_ccode_enum_add_value (cenum, _tmp7_ = vala_ccode_enum_value_new (_tmp6_ = vala_enum_value_get_cname (ev), NULL));
				_vala_ccode_node_unref0 (_tmp7_);
				_g_free0 (_tmp6_);
			} else {
				ValaCCodeEnumValue* _tmp9_;
				char* _tmp8_;
				vala_code_node_accept ((ValaCodeNode*) vala_enum_value_get_value (ev), (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
				vala_ccode_enum_add_value (cenum, _tmp9_ = vala_ccode_enum_value_new (_tmp8_ = vala_enum_value_get_cname (ev), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_enum_value_get_value (ev)))));
				_vala_ccode_node_unref0 (_tmp9_);
				_g_free0 (_tmp8_);
			}
			_vala_code_node_unref0 (ev);
		}
		_gee_collection_object_unref0 (_ev_it);
	}
	vala_ccode_declaration_space_add_type_definition (decl_space, (ValaCCodeNode*) cenum);
	vala_ccode_declaration_space_add_type_definition (decl_space, (ValaCCodeNode*) (_tmp10_ = vala_ccode_newline_new ()));
	_vala_ccode_node_unref0 (_tmp10_);
	if (!vala_enum_get_has_type_id (en)) {
		_vala_ccode_node_unref0 (cenum);
		return;
	}
	vala_ccode_declaration_space_add_type_declaration (decl_space, (ValaCCodeNode*) (_tmp11_ = vala_ccode_newline_new ()));
	_vala_ccode_node_unref0 (_tmp11_);
	macro = (_tmp13_ = g_strdup_printf ("(%s_get_type ())", _tmp12_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) en, NULL)), _g_free0 (_tmp12_), _tmp13_);
	vala_ccode_declaration_space_add_type_declaration (decl_space, (ValaCCodeNode*) (_tmp15_ = vala_ccode_macro_replacement_new (_tmp14_ = vala_typesymbol_get_type_id ((ValaTypeSymbol*) en), macro)));
	_vala_ccode_node_unref0 (_tmp15_);
	_g_free0 (_tmp14_);
	fun_name = (_tmp17_ = g_strdup_printf ("%s_get_type", _tmp16_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) en, NULL)), _g_free0 (_tmp16_), _tmp17_);
	regfun = vala_ccode_function_new (fun_name, "GType");
	if (vala_symbol_get_access ((ValaSymbol*) en) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
		vala_ccode_function_set_modifiers (regfun, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) regfun);
	_vala_ccode_node_unref0 (cenum);
	_g_free0 (macro);
	_g_free0 (fun_name);
	_vala_ccode_node_unref0 (regfun);
}


static void vala_ccode_base_module_real_visit_enum (ValaCCodeModule* base, ValaEnum* en) {
	ValaCCodeBaseModule * self;
	ValaCCodeInitializerList* clist;
	ValaCCodeInitializerList* clist_ev;
	ValaCCodeInitializerList* _tmp9_;
	ValaCCodeConstant* _tmp10_;
	ValaCCodeConstant* _tmp11_;
	ValaCCodeConstant* _tmp12_;
	ValaCCodeVariableDeclarator* enum_decl;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeBlock* type_init;
	char* _tmp15_;
	char* _tmp16_;
	char* fun_name;
	ValaCCodeFunction* regfun;
	ValaCCodeBlock* regblock;
	ValaCCodeDeclaration* _tmp17_;
	char* _tmp18_;
	char* _tmp19_;
	char* type_id_name;
	ValaCCodeVariableDeclarator* _tmp21_;
	ValaCCodeConstant* _tmp20_;
	ValaCCodeFunctionCall* reg_call;
	ValaCCodeConstant* _tmp28_;
	char* _tmp27_;
	char* _tmp26_;
	ValaCCodeIdentifier* _tmp29_;
	ValaCCodeExpressionStatement* _tmp32_;
	ValaCCodeAssignment* _tmp31_;
	ValaCCodeIdentifier* _tmp30_;
	ValaCCodeIdentifier* _tmp33_;
	ValaCCodeFunctionCall* _tmp34_;
	ValaCCodeFunctionCall* cond;
	ValaCCodeBinaryExpression* _tmp37_;
	ValaCCodeConstant* _tmp36_;
	ValaCCodeIdentifier* _tmp35_;
	ValaCCodeIfStatement* cif;
	ValaCCodeReturnStatement* _tmp39_;
	ValaCCodeConstant* _tmp38_;
	ValaCCodeNewline* _tmp40_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (en != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) en, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_base_module_generate_enum_declaration (self, en, self->source_declarations);
	if (!vala_symbol_is_internal_symbol ((ValaSymbol*) en)) {
		vala_ccode_base_module_generate_enum_declaration (self, en, self->header_declarations);
	}
	if (!vala_symbol_is_private_symbol ((ValaSymbol*) en)) {
		vala_ccode_base_module_generate_enum_declaration (self, en, self->internal_header_declarations);
	}
	if (!vala_enum_get_has_type_id (en)) {
		return;
	}
	clist = vala_ccode_initializer_list_new ();
	clist_ev = NULL;
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _ev_it;
		_ev_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_enum_get_values (en))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaEnumValue* ev;
			ValaCCodeInitializerList* _tmp2_;
			ValaCCodeConstant* _tmp4_;
			char* _tmp3_;
			ValaCCodeIdentifier* _tmp7_;
			char* _tmp6_;
			char* _tmp5_;
			ValaCCodeConstant* _tmp8_;
			if (!gee_iterator_next (_ev_it)) {
				break;
			}
			ev = (ValaEnumValue*) gee_iterator_get (_ev_it);
			clist_ev = (_tmp2_ = vala_ccode_initializer_list_new (), _vala_ccode_node_unref0 (clist_ev), _tmp2_);
			vala_ccode_initializer_list_append (clist_ev, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new (_tmp3_ = vala_enum_value_get_cname (ev))));
			_vala_ccode_node_unref0 (_tmp4_);
			_g_free0 (_tmp3_);
			vala_ccode_initializer_list_append (clist_ev, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = g_strdup_printf ("\"%s\"", _tmp5_ = vala_enum_value_get_cname (ev)))));
			_vala_ccode_node_unref0 (_tmp7_);
			_g_free0 (_tmp6_);
			_g_free0 (_tmp5_);
			vala_ccode_initializer_list_append (clist_ev, (ValaCCodeExpression*) (_tmp8_ = vala_enum_value_get_canonical_cconstant (ev)));
			_vala_ccode_node_unref0 (_tmp8_);
			vala_ccode_initializer_list_append (clist, (ValaCCodeExpression*) clist_ev);
			_vala_code_node_unref0 (ev);
		}
		_gee_collection_object_unref0 (_ev_it);
	}
	clist_ev = (_tmp9_ = vala_ccode_initializer_list_new (), _vala_ccode_node_unref0 (clist_ev), _tmp9_);
	vala_ccode_initializer_list_append (clist_ev, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_constant_new ("0")));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_initializer_list_append (clist_ev, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_initializer_list_append (clist_ev, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_initializer_list_append (clist, (ValaCCodeExpression*) clist_ev);
	enum_decl = vala_ccode_variable_declarator_new ("values[]", (ValaCCodeExpression*) clist, NULL);
	_cdecl_ = NULL;
	if (vala_enum_get_is_flags (en)) {
		ValaCCodeDeclaration* _tmp13_;
		_cdecl_ = (_tmp13_ = vala_ccode_declaration_new ("const GFlagsValue"), _vala_ccode_node_unref0 (_cdecl_), _tmp13_);
	} else {
		ValaCCodeDeclaration* _tmp14_;
		_cdecl_ = (_tmp14_ = vala_ccode_declaration_new ("const GEnumValue"), _vala_ccode_node_unref0 (_cdecl_), _tmp14_);
	}
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) enum_decl);
	vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
	type_init = vala_ccode_block_new ();
	vala_ccode_block_add_statement (type_init, (ValaCCodeNode*) _cdecl_);
	fun_name = (_tmp16_ = g_strdup_printf ("%s_get_type", _tmp15_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) en, NULL)), _g_free0 (_tmp15_), _tmp16_);
	regfun = vala_ccode_function_new (fun_name, "GType");
	regblock = vala_ccode_block_new ();
	_cdecl_ = (_tmp17_ = vala_ccode_declaration_new ("GType"), _vala_ccode_node_unref0 (_cdecl_), _tmp17_);
	type_id_name = (_tmp19_ = g_strdup_printf ("%s_type_id", _tmp18_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) en, NULL)), _g_free0 (_tmp18_), _tmp19_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp21_ = vala_ccode_variable_declarator_new (type_id_name, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_constant_new ("0")), NULL)));
	_vala_ccode_node_unref0 (_tmp21_);
	_vala_ccode_node_unref0 (_tmp20_);
	vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_block_add_statement (regblock, (ValaCCodeNode*) _cdecl_);
	reg_call = NULL;
	if (vala_enum_get_is_flags (en)) {
		ValaCCodeFunctionCall* _tmp23_;
		ValaCCodeIdentifier* _tmp22_;
		reg_call = (_tmp23_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("g_flags_register_static"))), _vala_ccode_node_unref0 (reg_call), _tmp23_);
		_vala_ccode_node_unref0 (_tmp22_);
	} else {
		ValaCCodeFunctionCall* _tmp25_;
		ValaCCodeIdentifier* _tmp24_;
		reg_call = (_tmp25_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("g_enum_register_static"))), _vala_ccode_node_unref0 (reg_call), _tmp25_);
		_vala_ccode_node_unref0 (_tmp24_);
	}
	vala_ccode_function_call_add_argument (reg_call, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_constant_new (_tmp27_ = g_strdup_printf ("\"%s\"", _tmp26_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) en, FALSE)))));
	_vala_ccode_node_unref0 (_tmp28_);
	_g_free0 (_tmp27_);
	_g_free0 (_tmp26_);
	vala_ccode_function_call_add_argument (reg_call, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("values")));
	_vala_ccode_node_unref0 (_tmp29_);
	vala_ccode_block_add_statement (type_init, (ValaCCodeNode*) (_tmp32_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new (type_id_name)), (ValaCCodeExpression*) reg_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp32_);
	_vala_ccode_node_unref0 (_tmp31_);
	_vala_ccode_node_unref0 (_tmp30_);
	cond = (_tmp34_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("G_UNLIKELY"))), _vala_ccode_node_unref0 (_tmp33_), _tmp34_);
	vala_ccode_function_call_add_argument (cond, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new (type_id_name)), (ValaCCodeExpression*) (_tmp36_ = vala_ccode_constant_new ("0")))));
	_vala_ccode_node_unref0 (_tmp37_);
	_vala_ccode_node_unref0 (_tmp36_);
	_vala_ccode_node_unref0 (_tmp35_);
	cif = vala_ccode_if_statement_new ((ValaCCodeExpression*) cond, (ValaCCodeStatement*) type_init, NULL);
	vala_ccode_block_add_statement (regblock, (ValaCCodeNode*) cif);
	vala_ccode_block_add_statement (regblock, (ValaCCodeNode*) (_tmp39_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp38_ = vala_ccode_constant_new (type_id_name)))));
	_vala_ccode_node_unref0 (_tmp39_);
	_vala_ccode_node_unref0 (_tmp38_);
	if (vala_symbol_get_access ((ValaSymbol*) en) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
		vala_ccode_function_set_modifiers (regfun, VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_function_set_block (regfun, regblock);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) (_tmp40_ = vala_ccode_newline_new ()));
	_vala_ccode_node_unref0 (_tmp40_);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) regfun);
	_vala_ccode_node_unref0 (clist);
	_vala_ccode_node_unref0 (clist_ev);
	_vala_ccode_node_unref0 (enum_decl);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (type_init);
	_g_free0 (fun_name);
	_vala_ccode_node_unref0 (regfun);
	_vala_ccode_node_unref0 (regblock);
	_g_free0 (type_id_name);
	_vala_ccode_node_unref0 (reg_call);
	_vala_ccode_node_unref0 (cond);
	_vala_ccode_node_unref0 (cif);
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


static void vala_ccode_base_module_real_visit_member (ValaCCodeModule* base, ValaMember* m) {
	ValaCCodeBaseModule * self;
	gboolean _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (m != NULL);
	_tmp0_ = FALSE;
	if (VALA_IS_LOCKABLE (m)) {
		_tmp0_ = vala_lockable_get_lock_used (VALA_LOCKABLE (m));
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaCCodeExpression* l;
		ValaCCodeFragment* init_fragment;
		ValaCCodeFragment* finalize_fragment;
		ValaCCodeIdentifier* _tmp17_;
		char* _tmp16_;
		ValaCCodeFunctionCall* _tmp18_;
		ValaCCodeFunctionCall* initf;
		ValaCCodeUnaryExpression* _tmp19_;
		ValaCCodeExpressionStatement* _tmp20_;
		l = (ValaCCodeExpression*) vala_ccode_identifier_new ("self");
		init_fragment = _vala_ccode_node_ref0 (self->class_init_fragment);
		finalize_fragment = _vala_ccode_node_ref0 (self->class_finalize_fragment);
		if (vala_symbol_is_instance_member ((ValaSymbol*) m)) {
			ValaCCodeExpression* _tmp3_;
			char* _tmp2_;
			ValaCCodeMemberAccess* _tmp1_;
			ValaCCodeFragment* _tmp4_;
			ValaCCodeFragment* _tmp5_;
			l = (_tmp3_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp1_ = vala_ccode_member_access_new_pointer (l, "priv")), _tmp2_ = vala_ccode_base_module_get_symbol_lock_name (self, vala_symbol_get_name ((ValaSymbol*) m))), _vala_ccode_node_unref0 (l), _tmp3_);
			_g_free0 (_tmp2_);
			_vala_ccode_node_unref0 (_tmp1_);
			init_fragment = (_tmp4_ = _vala_ccode_node_ref0 (self->instance_init_fragment), _vala_ccode_node_unref0 (init_fragment), _tmp4_);
			finalize_fragment = (_tmp5_ = _vala_ccode_node_ref0 (self->instance_finalize_fragment), _vala_ccode_node_unref0 (finalize_fragment), _tmp5_);
		} else {
			if (vala_symbol_is_class_member ((ValaSymbol*) m)) {
				ValaTypeSymbol* parent;
				ValaCCodeExpression* _tmp9_;
				char* _tmp8_;
				char* _tmp7_;
				char* _tmp6_;
				ValaCCodeExpression* _tmp11_;
				char* _tmp10_;
				parent = _vala_code_node_ref0 (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) m)));
				l = (_tmp9_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp8_ = g_strdup_printf ("%s_GET_CLASS_PRIVATE(%s)", _tmp6_ = vala_typesymbol_get_upper_case_cname (parent, NULL), _tmp7_ = vala_typesymbol_get_type_id (parent))), _vala_ccode_node_unref0 (l), _tmp9_);
				_g_free0 (_tmp8_);
				_g_free0 (_tmp7_);
				_g_free0 (_tmp6_);
				l = (_tmp11_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (l, _tmp10_ = vala_ccode_base_module_get_symbol_lock_name (self, vala_symbol_get_name ((ValaSymbol*) m))), _vala_ccode_node_unref0 (l), _tmp11_);
				_g_free0 (_tmp10_);
				_vala_code_node_unref0 (parent);
			} else {
				ValaCCodeExpression* _tmp15_;
				char* _tmp14_;
				char* _tmp13_;
				char* _tmp12_;
				l = (_tmp15_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp14_ = vala_ccode_base_module_get_symbol_lock_name (self, _tmp13_ = g_strdup_printf ("%s_%s", _tmp12_ = vala_symbol_get_lower_case_cname (vala_symbol_get_parent_symbol ((ValaSymbol*) m), NULL), vala_symbol_get_name ((ValaSymbol*) m)))), _vala_ccode_node_unref0 (l), _tmp15_);
				_g_free0 (_tmp14_);
				_g_free0 (_tmp13_);
				_g_free0 (_tmp12_);
			}
		}
		initf = (_tmp18_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new (_tmp16_ = vala_method_get_cname (vala_struct_get_default_construction_method (self->mutex_type))))), _vala_ccode_node_unref0 (_tmp17_), _g_free0 (_tmp16_), _tmp18_);
		vala_ccode_function_call_add_argument (initf, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, l)));
		_vala_ccode_node_unref0 (_tmp19_);
		vala_ccode_fragment_append (init_fragment, (ValaCCodeNode*) (_tmp20_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) initf)));
		_vala_ccode_node_unref0 (_tmp20_);
		if (finalize_fragment != NULL) {
			ValaCCodeIdentifier* _tmp21_;
			ValaCCodeFunctionCall* _tmp22_;
			ValaCCodeFunctionCall* fc;
			ValaCCodeUnaryExpression* _tmp23_;
			ValaCCodeExpressionStatement* _tmp24_;
			fc = (_tmp22_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("g_static_rec_mutex_free"))), _vala_ccode_node_unref0 (_tmp21_), _tmp22_);
			vala_ccode_function_call_add_argument (fc, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, l)));
			_vala_ccode_node_unref0 (_tmp23_);
			vala_ccode_fragment_append (finalize_fragment, (ValaCCodeNode*) (_tmp24_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) fc)));
			_vala_ccode_node_unref0 (_tmp24_);
			_vala_ccode_node_unref0 (fc);
		}
		_vala_ccode_node_unref0 (l);
		_vala_ccode_node_unref0 (init_fragment);
		_vala_ccode_node_unref0 (finalize_fragment);
		_vala_ccode_node_unref0 (initf);
	}
}


void vala_ccode_base_module_generate_constant_declaration (ValaCCodeBaseModule* self, ValaConstant* c, ValaCCodeDeclarationSpace* decl_space) {
	char* _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (c != NULL);
	g_return_if_fail (decl_space != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_symbol_declaration (decl_space, (ValaSymbol*) c, _tmp0_ = vala_constant_get_cname (c)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
	vala_code_node_accept_children ((ValaCodeNode*) c, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	if (!vala_symbol_get_external ((ValaSymbol*) c)) {
		if (VALA_IS_INITIALIZER_LIST (vala_constant_get_initializer (c))) {
			char* _tmp2_;
			ValaCCodeDeclaration* _tmp3_;
			ValaCCodeDeclaration* _cdecl_;
			char* arr;
			ValaCCodeVariableDeclarator* _tmp7_;
			char* _tmp6_;
			char* _tmp5_;
			_cdecl_ = (_tmp3_ = vala_ccode_declaration_new (_tmp2_ = vala_data_type_get_const_cname (vala_constant_get_type_reference (c))), _g_free0 (_tmp2_), _tmp3_);
			arr = g_strdup ("");
			if (VALA_IS_ARRAY_TYPE (vala_constant_get_type_reference (c))) {
				char* _tmp4_;
				arr = (_tmp4_ = g_strdup ("[]"), _g_free0 (arr), _tmp4_);
			}
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp7_ = vala_ccode_variable_declarator_new (_tmp6_ = g_strdup_printf ("%s%s", _tmp5_ = vala_constant_get_cname (c), arr), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_constant_get_initializer (c))), NULL)));
			_vala_ccode_node_unref0 (_tmp7_);
			_g_free0 (_tmp6_);
			_g_free0 (_tmp5_);
			vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
			vala_ccode_declaration_space_add_constant_declaration (decl_space, (ValaCCodeNode*) _cdecl_);
			_vala_ccode_node_unref0 (_cdecl_);
			_g_free0 (arr);
		} else {
			char* _tmp8_;
			ValaCCodeMacroReplacement* _tmp9_;
			ValaCCodeMacroReplacement* cdefine;
			cdefine = (_tmp9_ = vala_ccode_macro_replacement_new_with_expression (_tmp8_ = vala_constant_get_cname (c), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_constant_get_initializer (c)))), _g_free0 (_tmp8_), _tmp9_);
			vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) cdefine);
			_vala_ccode_node_unref0 (cdefine);
		}
	}
}


static void vala_ccode_base_module_real_visit_constant (ValaCCodeModule* base, ValaConstant* c) {
	ValaCCodeBaseModule * self;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (c != NULL);
	vala_ccode_base_module_generate_constant_declaration (self, c, self->source_declarations);
	if (!vala_symbol_is_internal_symbol ((ValaSymbol*) c)) {
		vala_ccode_base_module_generate_constant_declaration (self, c, self->header_declarations);
	}
	if (!vala_symbol_is_private_symbol ((ValaSymbol*) c)) {
		vala_ccode_base_module_generate_constant_declaration (self, c, self->internal_header_declarations);
	}
}


void vala_ccode_base_module_generate_field_declaration (ValaCCodeBaseModule* self, ValaField* f, ValaCCodeDeclarationSpace* decl_space) {
	char* _tmp0_;
	gboolean _tmp1_;
	char* field_ctype;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp4_;
	char* _tmp3_;
	gboolean _tmp11_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (f != NULL);
	g_return_if_fail (decl_space != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_symbol_declaration (decl_space, (ValaSymbol*) f, _tmp0_ = vala_field_get_cname (f)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
	vala_ccode_base_module_generate_type_declaration (self, vala_field_get_field_type (f), decl_space);
	field_ctype = vala_data_type_get_cname (vala_field_get_field_type (f));
	if (vala_field_get_is_volatile (f)) {
		char* _tmp2_;
		field_ctype = (_tmp2_ = g_strconcat ("volatile ", field_ctype, NULL), _g_free0 (field_ctype), _tmp2_);
	}
	_cdecl_ = vala_ccode_declaration_new (field_ctype);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp4_ = vala_ccode_variable_declarator_new (_tmp3_ = vala_field_get_cname (f), NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	if (vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
		vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
	} else {
		vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_EXTERN);
	}
	vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) _cdecl_);
	if (vala_lockable_get_lock_used ((ValaLockable*) f)) {
		char* _tmp5_;
		ValaCCodeDeclaration* _tmp6_;
		ValaCCodeDeclaration* flock;
		ValaCCodeConstant* _tmp9_;
		char* _tmp8_;
		char* _tmp7_;
		ValaCCodeVariableDeclarator* _tmp10_;
		ValaCCodeVariableDeclarator* flock_decl;
		flock = (_tmp6_ = vala_ccode_declaration_new (_tmp5_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) self->mutex_type, FALSE)), _g_free0 (_tmp5_), _tmp6_);
		flock_decl = (_tmp10_ = vala_ccode_variable_declarator_new (_tmp8_ = vala_ccode_base_module_get_symbol_lock_name (self, _tmp7_ = vala_field_get_cname (f)), (ValaCCodeExpression*) (_tmp9_ = vala_ccode_constant_new ("{0}")), NULL), _vala_ccode_node_unref0 (_tmp9_), _g_free0 (_tmp8_), _g_free0 (_tmp7_), _tmp10_);
		vala_ccode_declaration_add_declarator (flock, (ValaCCodeDeclarator*) flock_decl);
		if (vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
			vala_ccode_declaration_set_modifiers (flock, VALA_CCODE_MODIFIERS_STATIC);
		} else {
			vala_ccode_declaration_set_modifiers (flock, VALA_CCODE_MODIFIERS_EXTERN);
		}
		vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) flock);
		_vala_ccode_node_unref0 (flock);
		_vala_ccode_node_unref0 (flock_decl);
	}
	_tmp11_ = FALSE;
	if (VALA_IS_ARRAY_TYPE (vala_field_get_field_type (f))) {
		_tmp11_ = !vala_field_get_no_array_length (f);
	} else {
		_tmp11_ = FALSE;
	}
	if (_tmp11_) {
		ValaArrayType* array_type;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_field_get_field_type (f)));
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp12_;
				_tmp12_ = TRUE;
				while (TRUE) {
					ValaDataType* len_type;
					ValaCCodeDeclaration* _tmp14_;
					char* _tmp13_;
					ValaCCodeVariableDeclarator* _tmp17_;
					char* _tmp16_;
					char* _tmp15_;
					if (!_tmp12_) {
						dim++;
					}
					_tmp12_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					len_type = vala_data_type_copy (self->int_type);
					_cdecl_ = (_tmp14_ = vala_ccode_declaration_new (_tmp13_ = vala_data_type_get_cname (len_type)), _vala_ccode_node_unref0 (_cdecl_), _tmp14_);
					_g_free0 (_tmp13_);
					vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp17_ = vala_ccode_variable_declarator_new (_tmp16_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp15_ = vala_field_get_cname (f), dim), NULL, NULL)));
					_vala_ccode_node_unref0 (_tmp17_);
					_g_free0 (_tmp16_);
					_g_free0 (_tmp15_);
					if (vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
						vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
					} else {
						vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_EXTERN);
					}
					vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) _cdecl_);
					_vala_code_node_unref0 (len_type);
				}
			}
		}
		_vala_code_node_unref0 (array_type);
	} else {
		if (VALA_IS_DELEGATE_TYPE (vala_field_get_field_type (f))) {
			ValaDelegateType* delegate_type;
			delegate_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_field_get_field_type (f)));
			if (vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type))) {
				ValaCCodeDeclaration* _tmp18_;
				ValaCCodeVariableDeclarator* _tmp21_;
				char* _tmp20_;
				char* _tmp19_;
				_cdecl_ = (_tmp18_ = vala_ccode_declaration_new ("gpointer"), _vala_ccode_node_unref0 (_cdecl_), _tmp18_);
				vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp21_ = vala_ccode_variable_declarator_new (_tmp20_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp19_ = vala_field_get_cname (f)), NULL, NULL)));
				_vala_ccode_node_unref0 (_tmp21_);
				_g_free0 (_tmp20_);
				_g_free0 (_tmp19_);
				if (vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
					vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
				} else {
					vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_EXTERN);
				}
				vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) _cdecl_);
			}
			_vala_code_node_unref0 (delegate_type);
		}
	}
	_g_free0 (field_ctype);
	_vala_ccode_node_unref0 (_cdecl_);
}


static void vala_ccode_base_module_real_visit_field (ValaCCodeModule* base, ValaField* f) {
	ValaCCodeBaseModule * self;
	ValaSymbol* _tmp0_;
	ValaClass* cl;
	gboolean _tmp1_;
	gboolean is_gtypeinstance;
	ValaCCodeExpression* lhs;
	char* field_ctype;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (f != NULL);
	vala_ccode_base_module_check_type (self, vala_field_get_field_type (f));
	vala_code_node_accept_children ((ValaCodeNode*) f, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	cl = _vala_code_node_ref0 ((_tmp0_ = vala_symbol_get_parent_symbol ((ValaSymbol*) f), VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (cl != NULL) {
		_tmp1_ = !vala_class_get_is_compact (cl);
	} else {
		_tmp1_ = FALSE;
	}
	is_gtypeinstance = _tmp1_;
	lhs = NULL;
	field_ctype = vala_data_type_get_cname (vala_field_get_field_type (f));
	if (vala_field_get_is_volatile (f)) {
		char* _tmp2_;
		field_ctype = (_tmp2_ = g_strconcat ("volatile ", field_ctype, NULL), _g_free0 (field_ctype), _tmp2_);
	}
	if (vala_field_get_binding (f) == MEMBER_BINDING_INSTANCE) {
		gboolean _tmp3_;
		gboolean _tmp20_;
		_tmp3_ = FALSE;
		if (is_gtypeinstance) {
			_tmp3_ = vala_symbol_get_access ((ValaSymbol*) f) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
		} else {
			_tmp3_ = FALSE;
		}
		if (_tmp3_) {
			ValaCCodeExpression* _tmp7_;
			char* _tmp6_;
			ValaCCodeMemberAccess* _tmp5_;
			ValaCCodeIdentifier* _tmp4_;
			lhs = (_tmp7_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp5_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("self")), "priv")), _tmp6_ = vala_field_get_cname (f)), _vala_ccode_node_unref0 (lhs), _tmp7_);
			_g_free0 (_tmp6_);
			_vala_ccode_node_unref0 (_tmp5_);
			_vala_ccode_node_unref0 (_tmp4_);
		} else {
			ValaCCodeExpression* _tmp10_;
			char* _tmp9_;
			ValaCCodeIdentifier* _tmp8_;
			lhs = (_tmp10_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("self")), _tmp9_ = vala_field_get_cname (f)), _vala_ccode_node_unref0 (lhs), _tmp10_);
			_g_free0 (_tmp9_);
			_vala_ccode_node_unref0 (_tmp8_);
		}
		if (vala_field_get_initializer (f) != NULL) {
			ValaCCodeExpression* rhs;
			ValaCCodeExpressionStatement* _tmp12_;
			ValaCCodeAssignment* _tmp11_;
			gboolean _tmp13_;
			gboolean _tmp14_;
			rhs = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_field_get_initializer (f))));
			vala_ccode_fragment_append (self->instance_init_fragment, (ValaCCodeNode*) (_tmp12_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_assignment_new (lhs, rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp12_);
			_vala_ccode_node_unref0 (_tmp11_);
			_tmp13_ = FALSE;
			_tmp14_ = FALSE;
			if (VALA_IS_ARRAY_TYPE (vala_field_get_field_type (f))) {
				_tmp14_ = !vala_field_get_no_array_length (f);
			} else {
				_tmp14_ = FALSE;
			}
			if (_tmp14_) {
				_tmp13_ = VALA_IS_ARRAY_CREATION_EXPRESSION (vala_field_get_initializer (f));
			} else {
				_tmp13_ = FALSE;
			}
			if (_tmp13_) {
				ValaArrayType* array_type;
				ValaMemberAccess* this_access;
				ValaDataType* _tmp15_;
				ValaCCodeIdentifier* _tmp16_;
				ValaMemberAccess* ma;
				GeeList* sizes;
				array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_field_get_field_type (f)));
				this_access = vala_member_access_new_simple ("this", NULL);
				vala_expression_set_value_type ((ValaExpression*) this_access, _tmp15_ = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) f))));
				_vala_code_node_unref0 (_tmp15_);
				vala_code_node_set_ccodenode ((ValaCodeNode*) this_access, (ValaCCodeNode*) (_tmp16_ = vala_ccode_identifier_new ("self")));
				_vala_ccode_node_unref0 (_tmp16_);
				ma = vala_member_access_new ((ValaExpression*) this_access, vala_symbol_get_name ((ValaSymbol*) f), NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) f);
				sizes = vala_array_creation_expression_get_sizes (VALA_ARRAY_CREATION_EXPRESSION (vala_field_get_initializer (f)));
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp17_;
						_tmp17_ = TRUE;
						while (TRUE) {
							ValaCCodeExpression* array_len_lhs;
							ValaExpression* size;
							ValaCCodeExpressionStatement* _tmp19_;
							ValaCCodeAssignment* _tmp18_;
							if (!_tmp17_) {
								dim++;
							}
							_tmp17_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							array_len_lhs = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), (ValaExpression*) ma, dim);
							size = (ValaExpression*) gee_list_get (sizes, dim - 1);
							vala_ccode_fragment_append (self->instance_init_fragment, (ValaCCodeNode*) (_tmp19_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_assignment_new (array_len_lhs, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) size)), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
							_vala_ccode_node_unref0 (_tmp19_);
							_vala_ccode_node_unref0 (_tmp18_);
							_vala_ccode_node_unref0 (array_len_lhs);
							_vala_code_node_unref0 (size);
						}
					}
				}
				_vala_code_node_unref0 (array_type);
				_vala_code_node_unref0 (this_access);
				_vala_code_node_unref0 (ma);
				_gee_collection_object_unref0 (sizes);
			}
			vala_ccode_base_module_append_temp_decl (self, self->instance_init_fragment, (GeeList*) self->temp_vars);
			gee_collection_clear ((GeeCollection*) self->temp_vars);
			_vala_ccode_node_unref0 (rhs);
		}
		_tmp20_ = FALSE;
		if (vala_ccode_base_module_requires_destroy (self, vala_field_get_field_type (f))) {
			_tmp20_ = self->instance_finalize_fragment != NULL;
		} else {
			_tmp20_ = FALSE;
		}
		if (_tmp20_) {
			ValaMemberAccess* this_access;
			ValaDataType* _tmp21_;
			ValaSymbol* _tmp22_;
			ValaStruct* field_st;
			gboolean _tmp23_;
			ValaMemberAccess* ma;
			ValaCCodeExpressionStatement* _tmp27_;
			ValaCCodeExpression* _tmp26_;
			this_access = vala_member_access_new_simple ("this", NULL);
			vala_expression_set_value_type ((ValaExpression*) this_access, _tmp21_ = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) f))));
			_vala_code_node_unref0 (_tmp21_);
			field_st = _vala_code_node_ref0 ((_tmp22_ = vala_symbol_get_parent_symbol ((ValaSymbol*) f), VALA_IS_STRUCT (_tmp22_) ? ((ValaStruct*) _tmp22_) : NULL));
			_tmp23_ = FALSE;
			if (field_st != NULL) {
				_tmp23_ = !vala_struct_is_simple_type (field_st);
			} else {
				_tmp23_ = FALSE;
			}
			if (_tmp23_) {
				ValaCCodeIdentifier* _tmp24_;
				vala_code_node_set_ccodenode ((ValaCodeNode*) this_access, (ValaCCodeNode*) (_tmp24_ = vala_ccode_identifier_new ("(*self)")));
				_vala_ccode_node_unref0 (_tmp24_);
			} else {
				ValaCCodeIdentifier* _tmp25_;
				vala_code_node_set_ccodenode ((ValaCodeNode*) this_access, (ValaCCodeNode*) (_tmp25_ = vala_ccode_identifier_new ("self")));
				_vala_ccode_node_unref0 (_tmp25_);
			}
			ma = vala_member_access_new ((ValaExpression*) this_access, vala_symbol_get_name ((ValaSymbol*) f), NULL);
			vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) f);
			vala_ccode_fragment_append (self->instance_finalize_fragment, (ValaCCodeNode*) (_tmp27_ = vala_ccode_expression_statement_new (_tmp26_ = vala_ccode_base_module_get_unref_expression (self, lhs, vala_field_get_field_type (f), (ValaExpression*) ma, FALSE))));
			_vala_ccode_node_unref0 (_tmp27_);
			_vala_ccode_node_unref0 (_tmp26_);
			_vala_code_node_unref0 (this_access);
			_vala_code_node_unref0 (field_st);
			_vala_code_node_unref0 (ma);
		}
	} else {
		if (vala_field_get_binding (f) == MEMBER_BINDING_CLASS) {
			if (!is_gtypeinstance) {
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) f), "class fields are not supported in compact classes");
				vala_code_node_set_error ((ValaCodeNode*) f, TRUE);
				_vala_code_node_unref0 (cl);
				_vala_ccode_node_unref0 (lhs);
				_g_free0 (field_ctype);
				return;
			}
		} else {
			ValaCCodeExpression* _tmp29_;
			char* _tmp28_;
			char* _tmp30_;
			ValaCCodeVariableDeclarator* _tmp31_;
			ValaCCodeVariableDeclarator* var_decl;
			ValaCCodeExpression* _tmp32_;
			ValaCCodeDeclaration* var_def;
			gboolean _tmp33_;
			vala_ccode_base_module_generate_field_declaration (self, f, self->source_declarations);
			if (!vala_symbol_is_internal_symbol ((ValaSymbol*) f)) {
				vala_ccode_base_module_generate_field_declaration (self, f, self->header_declarations);
			}
			if (!vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
				vala_ccode_base_module_generate_field_declaration (self, f, self->internal_header_declarations);
			}
			lhs = (_tmp29_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp28_ = vala_field_get_cname (f)), _vala_ccode_node_unref0 (lhs), _tmp29_);
			_g_free0 (_tmp28_);
			var_decl = (_tmp31_ = vala_ccode_variable_declarator_new (_tmp30_ = vala_field_get_cname (f), NULL, NULL), _g_free0 (_tmp30_), _tmp31_);
			vala_ccode_variable_declarator_set_initializer (var_decl, _tmp32_ = vala_ccode_base_module_default_value_for_type (self, vala_field_get_field_type (f), TRUE));
			_vala_ccode_node_unref0 (_tmp32_);
			if (vala_field_get_initializer (f) != NULL) {
				ValaCCodeExpression* init;
				init = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_field_get_initializer (f))));
				if (vala_ccode_base_module_is_constant_ccode_expression (self, init)) {
					vala_ccode_variable_declarator_set_initializer (var_decl, init);
				}
				_vala_ccode_node_unref0 (init);
			}
			var_def = vala_ccode_declaration_new (field_ctype);
			vala_ccode_declaration_add_declarator (var_def, (ValaCCodeDeclarator*) var_decl);
			if (!vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
				vala_ccode_declaration_set_modifiers (var_def, VALA_CCODE_MODIFIERS_EXTERN);
			} else {
				vala_ccode_declaration_set_modifiers (var_def, VALA_CCODE_MODIFIERS_STATIC);
			}
			vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) var_def);
			_tmp33_ = FALSE;
			if (VALA_IS_ARRAY_TYPE (vala_field_get_field_type (f))) {
				_tmp33_ = !vala_field_get_no_array_length (f);
			} else {
				_tmp33_ = FALSE;
			}
			if (_tmp33_) {
				ValaArrayType* array_type;
				gboolean _tmp41_;
				array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_field_get_field_type (f)));
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp34_;
						_tmp34_ = TRUE;
						while (TRUE) {
							ValaDataType* len_type;
							char* _tmp35_;
							ValaCCodeDeclaration* _tmp36_;
							ValaCCodeDeclaration* len_def;
							ValaCCodeVariableDeclarator* _tmp40_;
							ValaCCodeConstant* _tmp39_;
							char* _tmp38_;
							char* _tmp37_;
							if (!_tmp34_) {
								dim++;
							}
							_tmp34_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							len_type = vala_data_type_copy (self->int_type);
							len_def = (_tmp36_ = vala_ccode_declaration_new (_tmp35_ = vala_data_type_get_cname (len_type)), _g_free0 (_tmp35_), _tmp36_);
							vala_ccode_declaration_add_declarator (len_def, (ValaCCodeDeclarator*) (_tmp40_ = vala_ccode_variable_declarator_new (_tmp38_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp37_ = vala_field_get_cname (f), dim), (ValaCCodeExpression*) (_tmp39_ = vala_ccode_constant_new ("0")), NULL)));
							_vala_ccode_node_unref0 (_tmp40_);
							_vala_ccode_node_unref0 (_tmp39_);
							_g_free0 (_tmp38_);
							_g_free0 (_tmp37_);
							if (!vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
								vala_ccode_declaration_set_modifiers (len_def, VALA_CCODE_MODIFIERS_EXTERN);
							} else {
								vala_ccode_declaration_set_modifiers (len_def, VALA_CCODE_MODIFIERS_STATIC);
							}
							vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) len_def);
							_vala_code_node_unref0 (len_type);
							_vala_ccode_node_unref0 (len_def);
						}
					}
				}
				_tmp41_ = FALSE;
				if (vala_array_type_get_rank (array_type) == 1) {
					_tmp41_ = vala_symbol_is_internal_symbol ((ValaSymbol*) f);
				} else {
					_tmp41_ = FALSE;
				}
				if (_tmp41_) {
					ValaDataType* len_type;
					char* _tmp42_;
					ValaCCodeDeclaration* _tmp43_;
					ValaCCodeDeclaration* _cdecl_;
					ValaCCodeVariableDeclarator* _tmp47_;
					ValaCCodeConstant* _tmp46_;
					char* _tmp45_;
					char* _tmp44_;
					len_type = vala_data_type_copy (self->int_type);
					_cdecl_ = (_tmp43_ = vala_ccode_declaration_new (_tmp42_ = vala_data_type_get_cname (len_type)), _g_free0 (_tmp42_), _tmp43_);
					vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp47_ = vala_ccode_variable_declarator_new (_tmp45_ = vala_ccode_module_get_array_size_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp44_ = vala_field_get_cname (f)), (ValaCCodeExpression*) (_tmp46_ = vala_ccode_constant_new ("0")), NULL)));
					_vala_ccode_node_unref0 (_tmp47_);
					_vala_ccode_node_unref0 (_tmp46_);
					_g_free0 (_tmp45_);
					_g_free0 (_tmp44_);
					vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
					vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) _cdecl_);
					_vala_code_node_unref0 (len_type);
					_vala_ccode_node_unref0 (_cdecl_);
				}
				_vala_code_node_unref0 (array_type);
			} else {
				if (VALA_IS_DELEGATE_TYPE (vala_field_get_field_type (f))) {
					ValaDelegateType* delegate_type;
					delegate_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_field_get_field_type (f)));
					if (vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type))) {
						ValaCCodeDeclaration* target_def;
						ValaCCodeVariableDeclarator* _tmp51_;
						ValaCCodeConstant* _tmp50_;
						char* _tmp49_;
						char* _tmp48_;
						target_def = vala_ccode_declaration_new ("gpointer");
						vala_ccode_declaration_add_declarator (target_def, (ValaCCodeDeclarator*) (_tmp51_ = vala_ccode_variable_declarator_new (_tmp49_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp48_ = vala_field_get_cname (f)), (ValaCCodeExpression*) (_tmp50_ = vala_ccode_constant_new ("NULL")), NULL)));
						_vala_ccode_node_unref0 (_tmp51_);
						_vala_ccode_node_unref0 (_tmp50_);
						_g_free0 (_tmp49_);
						_g_free0 (_tmp48_);
						if (!vala_symbol_is_private_symbol ((ValaSymbol*) f)) {
							vala_ccode_declaration_set_modifiers (target_def, VALA_CCODE_MODIFIERS_EXTERN);
						} else {
							vala_ccode_declaration_set_modifiers (target_def, VALA_CCODE_MODIFIERS_STATIC);
						}
						vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) target_def);
						_vala_ccode_node_unref0 (target_def);
					}
					_vala_code_node_unref0 (delegate_type);
				}
			}
			if (vala_field_get_initializer (f) != NULL) {
				ValaCCodeExpression* rhs;
				rhs = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_field_get_initializer (f))));
				if (!vala_ccode_base_module_is_constant_ccode_expression (self, rhs)) {
					if (VALA_IS_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) f))) {
						ValaCCodeExpressionStatement* _tmp53_;
						ValaCCodeAssignment* _tmp52_;
						gboolean _tmp54_;
						gboolean _tmp55_;
						vala_ccode_fragment_append (self->class_init_fragment, (ValaCCodeNode*) (_tmp53_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp52_ = vala_ccode_assignment_new (lhs, rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
						_vala_ccode_node_unref0 (_tmp53_);
						_vala_ccode_node_unref0 (_tmp52_);
						_tmp54_ = FALSE;
						_tmp55_ = FALSE;
						if (VALA_IS_ARRAY_TYPE (vala_field_get_field_type (f))) {
							_tmp55_ = !vala_field_get_no_array_length (f);
						} else {
							_tmp55_ = FALSE;
						}
						if (_tmp55_) {
							_tmp54_ = VALA_IS_ARRAY_CREATION_EXPRESSION (vala_field_get_initializer (f));
						} else {
							_tmp54_ = FALSE;
						}
						if (_tmp54_) {
							ValaArrayType* array_type;
							ValaMemberAccess* ma;
							GeeList* sizes;
							array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_field_get_field_type (f)));
							ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) f), NULL);
							vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) f);
							sizes = vala_array_creation_expression_get_sizes (VALA_ARRAY_CREATION_EXPRESSION (vala_field_get_initializer (f)));
							{
								gint dim;
								dim = 1;
								{
									gboolean _tmp56_;
									_tmp56_ = TRUE;
									while (TRUE) {
										ValaCCodeExpression* array_len_lhs;
										ValaExpression* size;
										ValaCCodeExpressionStatement* _tmp58_;
										ValaCCodeAssignment* _tmp57_;
										if (!_tmp56_) {
											dim++;
										}
										_tmp56_ = FALSE;
										if (!(dim <= vala_array_type_get_rank (array_type))) {
											break;
										}
										array_len_lhs = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), (ValaExpression*) ma, dim);
										size = (ValaExpression*) gee_list_get (sizes, dim - 1);
										vala_ccode_fragment_append (self->class_init_fragment, (ValaCCodeNode*) (_tmp58_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp57_ = vala_ccode_assignment_new (array_len_lhs, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) size)), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
										_vala_ccode_node_unref0 (_tmp58_);
										_vala_ccode_node_unref0 (_tmp57_);
										_vala_ccode_node_unref0 (array_len_lhs);
										_vala_code_node_unref0 (size);
									}
								}
							}
							_vala_code_node_unref0 (array_type);
							_vala_code_node_unref0 (ma);
							_gee_collection_object_unref0 (sizes);
						}
						vala_ccode_base_module_append_temp_decl (self, self->class_init_fragment, (GeeList*) self->temp_vars);
						gee_collection_clear ((GeeCollection*) self->temp_vars);
					} else {
						vala_code_node_set_error ((ValaCodeNode*) f, TRUE);
						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) f), "Non-constant field initializers not supported in this context");
						_vala_ccode_node_unref0 (rhs);
						_vala_ccode_node_unref0 (var_decl);
						_vala_ccode_node_unref0 (var_def);
						_vala_code_node_unref0 (cl);
						_vala_ccode_node_unref0 (lhs);
						_g_free0 (field_ctype);
						return;
					}
				}
				_vala_ccode_node_unref0 (rhs);
			}
			_vala_ccode_node_unref0 (var_decl);
			_vala_ccode_node_unref0 (var_def);
		}
	}
	_vala_code_node_unref0 (cl);
	_vala_ccode_node_unref0 (lhs);
	_g_free0 (field_ctype);
}


gboolean vala_ccode_base_module_is_constant_ccode_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr) {
	gboolean result;
	ValaCCodeExpression* _tmp1_;
	ValaCCodeParenthesizedExpression* cparenthesized;
	gboolean _tmp2_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (cexpr != NULL, FALSE);
	if (VALA_IS_CCODE_CONSTANT (cexpr)) {
		result = TRUE;
		return result;
	} else {
		if (VALA_IS_CCODE_CAST_EXPRESSION (cexpr)) {
			ValaCCodeCastExpression* ccast;
			ccast = _vala_ccode_node_ref0 (VALA_CCODE_CAST_EXPRESSION (cexpr));
			result = vala_ccode_base_module_is_constant_ccode_expression (self, vala_ccode_cast_expression_get_inner (ccast));
			_vala_ccode_node_unref0 (ccast);
			return result;
		} else {
			if (VALA_IS_CCODE_BINARY_EXPRESSION (cexpr)) {
				ValaCCodeBinaryExpression* cbinary;
				gboolean _tmp0_;
				cbinary = _vala_ccode_node_ref0 (VALA_CCODE_BINARY_EXPRESSION (cexpr));
				_tmp0_ = FALSE;
				if (vala_ccode_base_module_is_constant_ccode_expression (self, vala_ccode_binary_expression_get_left (cbinary))) {
					_tmp0_ = vala_ccode_base_module_is_constant_ccode_expression (self, vala_ccode_binary_expression_get_right (cbinary));
				} else {
					_tmp0_ = FALSE;
				}
				result = _tmp0_;
				_vala_ccode_node_unref0 (cbinary);
				return result;
			}
		}
	}
	cparenthesized = _vala_ccode_node_ref0 ((_tmp1_ = cexpr, VALA_IS_CCODE_PARENTHESIZED_EXPRESSION (_tmp1_) ? ((ValaCCodeParenthesizedExpression*) _tmp1_) : NULL));
	_tmp2_ = FALSE;
	if (NULL != cparenthesized) {
		_tmp2_ = vala_ccode_base_module_is_constant_ccode_expression (self, vala_ccode_parenthesized_expression_get_inner (cparenthesized));
	} else {
		_tmp2_ = FALSE;
	}
	result = _tmp2_;
	_vala_ccode_node_unref0 (cparenthesized);
	return result;
}


gboolean vala_ccode_base_module_is_pure_ccode_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (cexpr != NULL, FALSE);
	_tmp0_ = FALSE;
	if (VALA_IS_CCODE_CONSTANT (cexpr)) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = VALA_IS_CCODE_IDENTIFIER (cexpr);
	}
	if (_tmp0_) {
		result = TRUE;
		return result;
	} else {
		if (VALA_IS_CCODE_BINARY_EXPRESSION (cexpr)) {
			ValaCCodeBinaryExpression* cbinary;
			gboolean _tmp1_;
			cbinary = _vala_ccode_node_ref0 (VALA_CCODE_BINARY_EXPRESSION (cexpr));
			_tmp1_ = FALSE;
			if (vala_ccode_base_module_is_pure_ccode_expression (self, vala_ccode_binary_expression_get_left (cbinary))) {
				_tmp1_ = vala_ccode_base_module_is_constant_ccode_expression (self, vala_ccode_binary_expression_get_right (cbinary));
			} else {
				_tmp1_ = FALSE;
			}
			result = _tmp1_;
			_vala_ccode_node_unref0 (cbinary);
			return result;
		} else {
			if (VALA_IS_CCODE_UNARY_EXPRESSION (cexpr)) {
				ValaCCodeUnaryExpression* cunary;
				cunary = _vala_ccode_node_ref0 (VALA_CCODE_UNARY_EXPRESSION (cexpr));
				switch (vala_ccode_unary_expression_get_operator (cunary)) {
					case VALA_CCODE_UNARY_OPERATOR_PREFIX_INCREMENT:
					case VALA_CCODE_UNARY_OPERATOR_PREFIX_DECREMENT:
					case VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT:
					case VALA_CCODE_UNARY_OPERATOR_POSTFIX_DECREMENT:
					{
						result = FALSE;
						_vala_ccode_node_unref0 (cunary);
						return result;
					}
					default:
					{
						result = vala_ccode_base_module_is_pure_ccode_expression (self, vala_ccode_unary_expression_get_inner (cunary));
						_vala_ccode_node_unref0 (cunary);
						return result;
					}
				}
				_vala_ccode_node_unref0 (cunary);
			} else {
				if (VALA_IS_CCODE_MEMBER_ACCESS (cexpr)) {
					ValaCCodeMemberAccess* cma;
					cma = _vala_ccode_node_ref0 (VALA_CCODE_MEMBER_ACCESS (cexpr));
					result = vala_ccode_base_module_is_pure_ccode_expression (self, vala_ccode_member_access_get_inner (cma));
					_vala_ccode_node_unref0 (cma);
					return result;
				} else {
					if (VALA_IS_CCODE_ELEMENT_ACCESS (cexpr)) {
						ValaCCodeElementAccess* cea;
						gboolean _tmp2_;
						cea = _vala_ccode_node_ref0 (VALA_CCODE_ELEMENT_ACCESS (cexpr));
						_tmp2_ = FALSE;
						if (vala_ccode_base_module_is_pure_ccode_expression (self, vala_ccode_element_access_get_container (cea))) {
							_tmp2_ = vala_ccode_base_module_is_pure_ccode_expression (self, vala_ccode_element_access_get_index (cea));
						} else {
							_tmp2_ = FALSE;
						}
						result = _tmp2_;
						_vala_ccode_node_unref0 (cea);
						return result;
					} else {
						if (VALA_IS_CCODE_CAST_EXPRESSION (cexpr)) {
							ValaCCodeCastExpression* ccast;
							ccast = _vala_ccode_node_ref0 (VALA_CCODE_CAST_EXPRESSION (cexpr));
							result = vala_ccode_base_module_is_pure_ccode_expression (self, vala_ccode_cast_expression_get_inner (ccast));
							_vala_ccode_node_unref0 (ccast);
							return result;
						} else {
							if (VALA_IS_CCODE_PARENTHESIZED_EXPRESSION (cexpr)) {
								ValaCCodeParenthesizedExpression* cparenthesized;
								cparenthesized = _vala_ccode_node_ref0 (VALA_CCODE_PARENTHESIZED_EXPRESSION (cexpr));
								result = vala_ccode_base_module_is_pure_ccode_expression (self, vala_ccode_parenthesized_expression_get_inner (cparenthesized));
								_vala_ccode_node_unref0 (cparenthesized);
								return result;
							}
						}
					}
				}
			}
		}
	}
	result = FALSE;
	return result;
}


static void vala_ccode_base_module_real_visit_formal_parameter (ValaCCodeModule* base, ValaFormalParameter* p) {
	ValaCCodeBaseModule * self;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (p != NULL);
	vala_ccode_base_module_check_type (self, vala_formal_parameter_get_parameter_type (p));
	vala_code_node_accept_children ((ValaCodeNode*) p, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
}


static gpointer _gee_collection_object_ref0 (gpointer self) {
	return self ? gee_collection_object_ref (self) : NULL;
}


static void vala_ccode_base_module_real_visit_property (ValaCCodeModule* base, ValaProperty* prop) {
	ValaCCodeBaseModule * self;
	gint old_next_temp_var_id;
	GeeArrayList* old_temp_vars;
	GeeArrayList* old_temp_ref_vars;
	GeeMap* old_variable_name_map;
	GeeArrayList* _tmp0_;
	GeeArrayList* _tmp1_;
	GeeMap* _tmp2_;
	GeeArrayList* _tmp3_;
	GeeArrayList* _tmp4_;
	GeeMap* _tmp5_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (prop != NULL);
	vala_ccode_base_module_check_type (self, vala_property_get_property_type (prop));
	old_next_temp_var_id = self->next_temp_var_id;
	old_temp_vars = _gee_collection_object_ref0 (self->temp_vars);
	old_temp_ref_vars = _gee_collection_object_ref0 (self->temp_ref_vars);
	old_variable_name_map = _gee_collection_object_ref0 (self->variable_name_map);
	self->next_temp_var_id = 0;
	self->temp_vars = (_tmp0_ = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), _gee_collection_object_unref0 (self->temp_vars), _tmp0_);
	self->temp_ref_vars = (_tmp1_ = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), _gee_collection_object_unref0 (self->temp_ref_vars), _tmp1_);
	self->variable_name_map = (_tmp2_ = (GeeMap*) gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal, g_direct_equal), _gee_collection_object_unref0 (self->variable_name_map), _tmp2_);
	vala_code_node_accept_children ((ValaCodeNode*) prop, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	self->next_temp_var_id = old_next_temp_var_id;
	self->temp_vars = (_tmp3_ = _gee_collection_object_ref0 (old_temp_vars), _gee_collection_object_unref0 (self->temp_vars), _tmp3_);
	self->temp_ref_vars = (_tmp4_ = _gee_collection_object_ref0 (old_temp_ref_vars), _gee_collection_object_unref0 (self->temp_ref_vars), _tmp4_);
	self->variable_name_map = (_tmp5_ = _gee_collection_object_ref0 (old_variable_name_map), _gee_collection_object_unref0 (self->variable_name_map), _tmp5_);
	_gee_collection_object_unref0 (old_temp_vars);
	_gee_collection_object_unref0 (old_temp_ref_vars);
	_gee_collection_object_unref0 (old_variable_name_map);
}


void vala_ccode_base_module_generate_type_declaration (ValaCCodeBaseModule* self, ValaDataType* type, ValaCCodeDeclarationSpace* decl_space) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (type != NULL);
	g_return_if_fail (decl_space != NULL);
	if (VALA_IS_OBJECT_TYPE (type)) {
		ValaObjectType* object_type;
		object_type = _vala_code_node_ref0 (VALA_OBJECT_TYPE (type));
		if (VALA_IS_CLASS (vala_object_type_get_type_symbol (object_type))) {
			vala_ccode_base_module_generate_class_declaration (self, VALA_CLASS (vala_object_type_get_type_symbol (object_type)), decl_space);
		} else {
			if (VALA_IS_INTERFACE (vala_object_type_get_type_symbol (object_type))) {
				vala_ccode_base_module_generate_interface_declaration (self, VALA_INTERFACE (vala_object_type_get_type_symbol (object_type)), decl_space);
			}
		}
		_vala_code_node_unref0 (object_type);
	} else {
		if (VALA_IS_DELEGATE_TYPE (type)) {
			ValaDelegateType* deleg_type;
			ValaDelegate* d;
			deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (type));
			d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
			vala_ccode_base_module_generate_delegate_declaration (self, d, decl_space);
			_vala_code_node_unref0 (deleg_type);
			_vala_code_node_unref0 (d);
		} else {
			if (VALA_IS_ENUM (vala_data_type_get_data_type (type))) {
				ValaEnum* en;
				en = _vala_code_node_ref0 (VALA_ENUM (vala_data_type_get_data_type (type)));
				vala_ccode_base_module_generate_enum_declaration (self, en, decl_space);
				_vala_code_node_unref0 (en);
			} else {
				if (VALA_IS_VALUE_TYPE (type)) {
					ValaValueType* value_type;
					value_type = _vala_code_node_ref0 (VALA_VALUE_TYPE (type));
					vala_ccode_base_module_generate_struct_declaration (self, VALA_STRUCT (vala_value_type_get_type_symbol (value_type)), decl_space);
					_vala_code_node_unref0 (value_type);
				} else {
					if (VALA_IS_ARRAY_TYPE (type)) {
						ValaArrayType* array_type;
						array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (type));
						vala_ccode_base_module_generate_type_declaration (self, vala_array_type_get_element_type (array_type), decl_space);
						_vala_code_node_unref0 (array_type);
					} else {
						if (VALA_IS_ERROR_TYPE (type)) {
							ValaErrorType* error_type;
							error_type = _vala_code_node_ref0 (VALA_ERROR_TYPE (type));
							if (vala_error_type_get_error_domain (error_type) != NULL) {
								vala_ccode_base_module_generate_error_domain_declaration (self, vala_error_type_get_error_domain (error_type), decl_space);
							}
							_vala_code_node_unref0 (error_type);
						} else {
							if (VALA_IS_POINTER_TYPE (type)) {
								ValaPointerType* pointer_type;
								pointer_type = _vala_code_node_ref0 (VALA_POINTER_TYPE (type));
								vala_ccode_base_module_generate_type_declaration (self, vala_pointer_type_get_base_type (pointer_type), decl_space);
								_vala_code_node_unref0 (pointer_type);
							}
						}
					}
				}
			}
		}
	}
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _type_arg_it;
		_type_arg_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_data_type_get_type_arguments (type))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaDataType* type_arg;
			if (!gee_iterator_next (_type_arg_it)) {
				break;
			}
			type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
			vala_ccode_base_module_generate_type_declaration (self, type_arg, decl_space);
			_vala_code_node_unref0 (type_arg);
		}
		_gee_collection_object_unref0 (_type_arg_it);
	}
}


static void vala_ccode_base_module_real_generate_class_struct_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (cl != NULL);
	g_return_if_fail (decl_space != NULL);
}


void vala_ccode_base_module_generate_class_struct_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_class_struct_declaration (self, cl, decl_space);
}


static void vala_ccode_base_module_real_generate_struct_declaration (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	g_return_if_fail (decl_space != NULL);
}


void vala_ccode_base_module_generate_struct_declaration (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_struct_declaration (self, st, decl_space);
}


static void vala_ccode_base_module_real_generate_delegate_declaration (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (d != NULL);
	g_return_if_fail (decl_space != NULL);
}


void vala_ccode_base_module_generate_delegate_declaration (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_delegate_declaration (self, d, decl_space);
}


static void vala_ccode_base_module_real_generate_cparameters (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (m != NULL);
	g_return_if_fail (decl_space != NULL);
	g_return_if_fail (cparam_map != NULL);
	g_return_if_fail (func != NULL);
}


void vala_ccode_base_module_generate_cparameters (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_cparameters (self, m, decl_space, cparam_map, func, vdeclarator, carg_map, vcall, direction);
}


void vala_ccode_base_module_generate_property_accessor_declaration (ValaCCodeBaseModule* self, ValaPropertyAccessor* acc, ValaCCodeDeclarationSpace* decl_space) {
	char* _tmp0_;
	gboolean _tmp1_;
	ValaProperty* prop;
	gboolean _tmp2_;
	gboolean returns_real_struct;
	ValaCCodeFormalParameter* cvalueparam;
	gboolean _tmp12_;
	gboolean _tmp21_;
	gboolean _tmp22_;
	gboolean _tmp33_;
	gboolean _tmp34_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (acc != NULL);
	g_return_if_fail (decl_space != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_symbol_declaration (decl_space, (ValaSymbol*) vala_property_accessor_get_prop (acc), _tmp0_ = vala_property_accessor_get_cname (acc)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
	prop = _vala_code_node_ref0 (VALA_PROPERTY (vala_property_accessor_get_prop (acc)));
	_tmp2_ = FALSE;
	if (vala_property_accessor_get_readable (acc)) {
		_tmp2_ = vala_data_type_is_real_non_null_struct_type (vala_property_get_property_type (prop));
	} else {
		_tmp2_ = FALSE;
	}
	returns_real_struct = _tmp2_;
	cvalueparam = NULL;
	if (returns_real_struct) {
		ValaCCodeFormalParameter* _tmp5_;
		char* _tmp4_;
		char* _tmp3_;
		cvalueparam = (_tmp5_ = vala_ccode_formal_parameter_new ("result", _tmp4_ = g_strconcat (_tmp3_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc)), "*", NULL)), _vala_ccode_node_unref0 (cvalueparam), _tmp5_);
		_g_free0 (_tmp4_);
		_g_free0 (_tmp3_);
	} else {
		gboolean _tmp6_;
		_tmp6_ = FALSE;
		if (!vala_property_accessor_get_readable (acc)) {
			_tmp6_ = vala_data_type_is_real_non_null_struct_type (vala_property_get_property_type (prop));
		} else {
			_tmp6_ = FALSE;
		}
		if (_tmp6_) {
			ValaCCodeFormalParameter* _tmp9_;
			char* _tmp8_;
			char* _tmp7_;
			cvalueparam = (_tmp9_ = vala_ccode_formal_parameter_new ("value", _tmp8_ = g_strconcat (_tmp7_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc)), "*", NULL)), _vala_ccode_node_unref0 (cvalueparam), _tmp9_);
			_g_free0 (_tmp8_);
			_g_free0 (_tmp7_);
		} else {
			ValaCCodeFormalParameter* _tmp11_;
			char* _tmp10_;
			cvalueparam = (_tmp11_ = vala_ccode_formal_parameter_new ("value", _tmp10_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc))), _vala_ccode_node_unref0 (cvalueparam), _tmp11_);
			_g_free0 (_tmp10_);
		}
	}
	vala_ccode_base_module_generate_type_declaration (self, vala_property_accessor_get_value_type (acc), decl_space);
	_tmp12_ = FALSE;
	if (vala_property_accessor_get_readable (acc)) {
		_tmp12_ = !returns_real_struct;
	} else {
		_tmp12_ = FALSE;
	}
	if (_tmp12_) {
		ValaCCodeFunction* _tmp15_;
		char* _tmp14_;
		char* _tmp13_;
		self->function = (_tmp15_ = vala_ccode_function_new (_tmp13_ = vala_property_accessor_get_cname (acc), _tmp14_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc))), _vala_ccode_node_unref0 (self->function), _tmp15_);
		_g_free0 (_tmp14_);
		_g_free0 (_tmp13_);
	} else {
		ValaCCodeFunction* _tmp17_;
		char* _tmp16_;
		self->function = (_tmp17_ = vala_ccode_function_new (_tmp16_ = vala_property_accessor_get_cname (acc), "void"), _vala_ccode_node_unref0 (self->function), _tmp17_);
		_g_free0 (_tmp16_);
	}
	if (vala_property_get_binding (prop) == MEMBER_BINDING_INSTANCE) {
		ValaTypeSymbol* t;
		ValaDataType* this_type;
		char* _tmp18_;
		ValaCCodeFormalParameter* _tmp19_;
		ValaCCodeFormalParameter* cselfparam;
		t = _vala_code_node_ref0 (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) prop)));
		this_type = vala_ccode_base_module_get_data_type_for_symbol (t);
		vala_ccode_base_module_generate_type_declaration (self, this_type, decl_space);
		cselfparam = (_tmp19_ = vala_ccode_formal_parameter_new ("self", _tmp18_ = vala_data_type_get_cname (this_type)), _g_free0 (_tmp18_), _tmp19_);
		if (VALA_IS_STRUCT (t)) {
			char* _tmp20_;
			vala_ccode_formal_parameter_set_type_name (cselfparam, _tmp20_ = g_strconcat (vala_ccode_formal_parameter_get_type_name (cselfparam), "*", NULL));
			_g_free0 (_tmp20_);
		}
		vala_ccode_function_add_parameter (self->function, cselfparam);
		_vala_code_node_unref0 (t);
		_vala_code_node_unref0 (this_type);
		_vala_ccode_node_unref0 (cselfparam);
	}
	_tmp21_ = FALSE;
	_tmp22_ = FALSE;
	if (vala_property_accessor_get_writable (acc)) {
		_tmp22_ = TRUE;
	} else {
		_tmp22_ = vala_property_accessor_get_construction (acc);
	}
	if (_tmp22_) {
		_tmp21_ = TRUE;
	} else {
		_tmp21_ = returns_real_struct;
	}
	if (_tmp21_) {
		vala_ccode_function_add_parameter (self->function, cvalueparam);
	}
	if (VALA_IS_ARRAY_TYPE (vala_property_accessor_get_value_type (acc))) {
		ValaArrayType* array_type;
		char* length_ctype;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_property_accessor_get_value_type (acc)));
		length_ctype = g_strdup ("int");
		if (vala_property_accessor_get_readable (acc)) {
			char* _tmp23_;
			length_ctype = (_tmp23_ = g_strdup ("int*"), _g_free0 (length_ctype), _tmp23_);
		}
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp24_;
				_tmp24_ = TRUE;
				while (TRUE) {
					const char* _tmp25_;
					ValaCCodeFormalParameter* _tmp27_;
					char* _tmp26_;
					if (!_tmp24_) {
						dim++;
					}
					_tmp24_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					_tmp25_ = NULL;
					if (vala_property_accessor_get_readable (acc)) {
						_tmp25_ = "result";
					} else {
						_tmp25_ = "value";
					}
					vala_ccode_function_add_parameter (self->function, _tmp27_ = vala_ccode_formal_parameter_new (_tmp26_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp25_, dim), length_ctype));
					_vala_ccode_node_unref0 (_tmp27_);
					_g_free0 (_tmp26_);
				}
			}
		}
		_vala_code_node_unref0 (array_type);
		_g_free0 (length_ctype);
	} else {
		gboolean _tmp28_;
		_tmp28_ = FALSE;
		if (VALA_IS_DELEGATE_TYPE (vala_property_accessor_get_value_type (acc))) {
			_tmp28_ = vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (VALA_DELEGATE_TYPE (vala_property_accessor_get_value_type (acc))));
		} else {
			_tmp28_ = FALSE;
		}
		if (_tmp28_) {
			const char* _tmp29_;
			const char* _tmp30_;
			ValaCCodeFormalParameter* _tmp32_;
			char* _tmp31_;
			_tmp29_ = NULL;
			if (vala_property_accessor_get_readable (acc)) {
				_tmp29_ = "result";
			} else {
				_tmp29_ = "value";
			}
			_tmp30_ = NULL;
			if (vala_property_accessor_get_readable (acc)) {
				_tmp30_ = "gpointer*";
			} else {
				_tmp30_ = "gpointer";
			}
			vala_ccode_function_add_parameter (self->function, _tmp32_ = vala_ccode_formal_parameter_new (_tmp31_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp29_), _tmp30_));
			_vala_ccode_node_unref0 (_tmp32_);
			_g_free0 (_tmp31_);
		}
	}
	_tmp33_ = FALSE;
	_tmp34_ = FALSE;
	if (vala_symbol_is_private_symbol ((ValaSymbol*) prop)) {
		_tmp34_ = TRUE;
	} else {
		gboolean _tmp35_;
		_tmp35_ = FALSE;
		if (!vala_property_accessor_get_readable (acc)) {
			_tmp35_ = !vala_property_accessor_get_writable (acc);
		} else {
			_tmp35_ = FALSE;
		}
		_tmp34_ = _tmp35_;
	}
	if (_tmp34_) {
		_tmp33_ = TRUE;
	} else {
		_tmp33_ = vala_symbol_get_access ((ValaSymbol*) acc) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
	}
	if (_tmp33_) {
		vala_ccode_function_set_modifiers (self->function, vala_ccode_function_get_modifiers (self->function) | VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_declaration_space_add_type_member_declaration (decl_space, (ValaCCodeNode*) self->function);
	_vala_code_node_unref0 (prop);
	_vala_ccode_node_unref0 (cvalueparam);
}


static void vala_ccode_base_module_real_visit_property_accessor (ValaCCodeModule* base, ValaPropertyAccessor* acc) {
	ValaCCodeBaseModule * self;
	ValaSymbol* old_symbol;
	gboolean old_method_inner_error;
	ValaSymbol* _tmp0_;
	ValaProperty* prop;
	gboolean _tmp1_;
	gboolean returns_real_struct;
	ValaTypeSymbol* t;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	ValaDataType* this_type;
	char* _tmp10_;
	ValaCCodeFormalParameter* _tmp11_;
	ValaCCodeFormalParameter* cselfparam;
	ValaCCodeFormalParameter* cvalueparam;
	gboolean _tmp22_;
	ValaSymbol* _tmp128_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (acc != NULL);
	old_symbol = _vala_code_node_ref0 (self->current_symbol);
	old_method_inner_error = self->current_method_inner_error;
	self->current_symbol = (_tmp0_ = _vala_code_node_ref0 ((ValaSymbol*) acc), _vala_code_node_unref0 (self->current_symbol), _tmp0_);
	self->current_method_inner_error = FALSE;
	prop = _vala_code_node_ref0 (VALA_PROPERTY (vala_property_accessor_get_prop (acc)));
	_tmp1_ = FALSE;
	if (vala_property_accessor_get_readable (acc)) {
		_tmp1_ = vala_data_type_is_real_non_null_struct_type (vala_property_get_property_type (prop));
	} else {
		_tmp1_ = FALSE;
	}
	returns_real_struct = _tmp1_;
	vala_code_node_accept_children ((ValaCodeNode*) acc, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	t = _vala_code_node_ref0 (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) prop)));
	_tmp2_ = FALSE;
	if (vala_property_accessor_get_construction (acc)) {
		_tmp2_ = !vala_typesymbol_is_subtype_of (t, self->gobject_type);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) acc), "construct properties require GLib.Object");
		vala_code_node_set_error ((ValaCodeNode*) acc, TRUE);
		_vala_code_node_unref0 (old_symbol);
		_vala_code_node_unref0 (prop);
		_vala_code_node_unref0 (t);
		return;
	}
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	if (vala_property_get_is_abstract (prop)) {
		_tmp4_ = TRUE;
	} else {
		_tmp4_ = vala_property_get_is_virtual (prop);
	}
	if (_tmp4_) {
		_tmp3_ = TRUE;
	} else {
		gboolean _tmp5_;
		_tmp5_ = FALSE;
		if (vala_property_get_base_property (prop) == NULL) {
			_tmp5_ = vala_property_get_base_interface_property (prop) == NULL;
		} else {
			_tmp5_ = FALSE;
		}
		_tmp3_ = _tmp5_;
	}
	if (_tmp3_) {
		gboolean _tmp6_;
		vala_ccode_base_module_generate_property_accessor_declaration (self, acc, self->source_declarations);
		_tmp6_ = FALSE;
		if (vala_property_accessor_get_readable (acc)) {
			_tmp6_ = TRUE;
		} else {
			_tmp6_ = vala_property_accessor_get_writable (acc);
		}
		if (_tmp6_) {
			gboolean _tmp7_;
			gboolean _tmp9_;
			_tmp7_ = FALSE;
			if (!vala_symbol_is_internal_symbol ((ValaSymbol*) prop)) {
				gboolean _tmp8_;
				_tmp8_ = FALSE;
				if (vala_symbol_get_access ((ValaSymbol*) acc) == VALA_SYMBOL_ACCESSIBILITY_PUBLIC) {
					_tmp8_ = TRUE;
				} else {
					_tmp8_ = vala_symbol_get_access ((ValaSymbol*) acc) == VALA_SYMBOL_ACCESSIBILITY_PROTECTED;
				}
				_tmp7_ = _tmp8_;
			} else {
				_tmp7_ = FALSE;
			}
			if (_tmp7_) {
				vala_ccode_base_module_generate_property_accessor_declaration (self, acc, self->header_declarations);
			}
			_tmp9_ = FALSE;
			if (!vala_symbol_is_private_symbol ((ValaSymbol*) prop)) {
				_tmp9_ = vala_symbol_get_access ((ValaSymbol*) acc) != VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
			} else {
				_tmp9_ = FALSE;
			}
			if (_tmp9_) {
				vala_ccode_base_module_generate_property_accessor_declaration (self, acc, self->internal_header_declarations);
			}
		}
	}
	this_type = vala_ccode_base_module_get_data_type_for_symbol (t);
	cselfparam = (_tmp11_ = vala_ccode_formal_parameter_new ("self", _tmp10_ = vala_data_type_get_cname (this_type)), _g_free0 (_tmp10_), _tmp11_);
	if (VALA_IS_STRUCT (t)) {
		char* _tmp12_;
		vala_ccode_formal_parameter_set_type_name (cselfparam, _tmp12_ = g_strconcat (vala_ccode_formal_parameter_get_type_name (cselfparam), "*", NULL));
		_g_free0 (_tmp12_);
	}
	cvalueparam = NULL;
	if (returns_real_struct) {
		ValaCCodeFormalParameter* _tmp15_;
		char* _tmp14_;
		char* _tmp13_;
		cvalueparam = (_tmp15_ = vala_ccode_formal_parameter_new ("result", _tmp14_ = g_strconcat (_tmp13_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc)), "*", NULL)), _vala_ccode_node_unref0 (cvalueparam), _tmp15_);
		_g_free0 (_tmp14_);
		_g_free0 (_tmp13_);
	} else {
		gboolean _tmp16_;
		_tmp16_ = FALSE;
		if (!vala_property_accessor_get_readable (acc)) {
			_tmp16_ = vala_data_type_is_real_non_null_struct_type (vala_property_get_property_type (prop));
		} else {
			_tmp16_ = FALSE;
		}
		if (_tmp16_) {
			ValaCCodeFormalParameter* _tmp19_;
			char* _tmp18_;
			char* _tmp17_;
			cvalueparam = (_tmp19_ = vala_ccode_formal_parameter_new ("value", _tmp18_ = g_strconcat (_tmp17_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc)), "*", NULL)), _vala_ccode_node_unref0 (cvalueparam), _tmp19_);
			_g_free0 (_tmp18_);
			_g_free0 (_tmp17_);
		} else {
			ValaCCodeFormalParameter* _tmp21_;
			char* _tmp20_;
			cvalueparam = (_tmp21_ = vala_ccode_formal_parameter_new ("value", _tmp20_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc))), _vala_ccode_node_unref0 (cvalueparam), _tmp21_);
			_g_free0 (_tmp20_);
		}
	}
	_tmp22_ = FALSE;
	if (vala_property_get_is_abstract (prop)) {
		_tmp22_ = TRUE;
	} else {
		_tmp22_ = vala_property_get_is_virtual (prop);
	}
	if (_tmp22_) {
		gboolean _tmp23_;
		gboolean _tmp29_;
		gboolean _tmp30_;
		gboolean _tmp41_;
		gboolean _tmp42_;
		ValaCCodeBlock* block;
		ValaCCodeFunctionCall* vcast;
		ValaCCodeIdentifier* _tmp52_;
		_tmp23_ = FALSE;
		if (vala_property_accessor_get_readable (acc)) {
			_tmp23_ = !returns_real_struct;
		} else {
			_tmp23_ = FALSE;
		}
		if (_tmp23_) {
			ValaCCodeFunction* _tmp26_;
			char* _tmp25_;
			char* _tmp24_;
			self->function = (_tmp26_ = vala_ccode_function_new (_tmp24_ = vala_property_accessor_get_cname (acc), _tmp25_ = vala_data_type_get_cname (vala_ccode_base_module_get_current_return_type (self))), _vala_ccode_node_unref0 (self->function), _tmp26_);
			_g_free0 (_tmp25_);
			_g_free0 (_tmp24_);
		} else {
			ValaCCodeFunction* _tmp28_;
			char* _tmp27_;
			self->function = (_tmp28_ = vala_ccode_function_new (_tmp27_ = vala_property_accessor_get_cname (acc), "void"), _vala_ccode_node_unref0 (self->function), _tmp28_);
			_g_free0 (_tmp27_);
		}
		vala_ccode_function_add_parameter (self->function, cselfparam);
		_tmp29_ = FALSE;
		_tmp30_ = FALSE;
		if (vala_property_accessor_get_writable (acc)) {
			_tmp30_ = TRUE;
		} else {
			_tmp30_ = vala_property_accessor_get_construction (acc);
		}
		if (_tmp30_) {
			_tmp29_ = TRUE;
		} else {
			_tmp29_ = returns_real_struct;
		}
		if (_tmp29_) {
			vala_ccode_function_add_parameter (self->function, cvalueparam);
		}
		if (VALA_IS_ARRAY_TYPE (vala_property_accessor_get_value_type (acc))) {
			ValaArrayType* array_type;
			char* length_ctype;
			array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_property_accessor_get_value_type (acc)));
			length_ctype = g_strdup ("int");
			if (vala_property_accessor_get_readable (acc)) {
				char* _tmp31_;
				length_ctype = (_tmp31_ = g_strdup ("int*"), _g_free0 (length_ctype), _tmp31_);
			}
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp32_;
					_tmp32_ = TRUE;
					while (TRUE) {
						const char* _tmp33_;
						ValaCCodeFormalParameter* _tmp35_;
						char* _tmp34_;
						if (!_tmp32_) {
							dim++;
						}
						_tmp32_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						_tmp33_ = NULL;
						if (vala_property_accessor_get_readable (acc)) {
							_tmp33_ = "result";
						} else {
							_tmp33_ = "value";
						}
						vala_ccode_function_add_parameter (self->function, _tmp35_ = vala_ccode_formal_parameter_new (_tmp34_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp33_, dim), length_ctype));
						_vala_ccode_node_unref0 (_tmp35_);
						_g_free0 (_tmp34_);
					}
				}
			}
			_vala_code_node_unref0 (array_type);
			_g_free0 (length_ctype);
		} else {
			gboolean _tmp36_;
			_tmp36_ = FALSE;
			if (VALA_IS_DELEGATE_TYPE (vala_property_accessor_get_value_type (acc))) {
				_tmp36_ = vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (VALA_DELEGATE_TYPE (vala_property_accessor_get_value_type (acc))));
			} else {
				_tmp36_ = FALSE;
			}
			if (_tmp36_) {
				const char* _tmp37_;
				const char* _tmp38_;
				ValaCCodeFormalParameter* _tmp40_;
				char* _tmp39_;
				_tmp37_ = NULL;
				if (vala_property_accessor_get_readable (acc)) {
					_tmp37_ = "result";
				} else {
					_tmp37_ = "value";
				}
				_tmp38_ = NULL;
				if (vala_property_accessor_get_readable (acc)) {
					_tmp38_ = "gpointer*";
				} else {
					_tmp38_ = "gpointer";
				}
				vala_ccode_function_add_parameter (self->function, _tmp40_ = vala_ccode_formal_parameter_new (_tmp39_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp37_), _tmp38_));
				_vala_ccode_node_unref0 (_tmp40_);
				_g_free0 (_tmp39_);
			}
		}
		_tmp41_ = FALSE;
		_tmp42_ = FALSE;
		if (vala_symbol_is_private_symbol ((ValaSymbol*) prop)) {
			_tmp42_ = TRUE;
		} else {
			gboolean _tmp43_;
			_tmp43_ = FALSE;
			if (vala_property_accessor_get_readable (acc)) {
				_tmp43_ = TRUE;
			} else {
				_tmp43_ = vala_property_accessor_get_writable (acc);
			}
			_tmp42_ = !_tmp43_;
		}
		if (_tmp42_) {
			_tmp41_ = TRUE;
		} else {
			_tmp41_ = vala_symbol_get_access ((ValaSymbol*) acc) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
		}
		if (_tmp41_) {
			vala_ccode_function_set_modifiers (self->function, vala_ccode_function_get_modifiers (self->function) | VALA_CCODE_MODIFIERS_STATIC);
		}
		block = vala_ccode_block_new ();
		vala_ccode_function_set_block (self->function, block);
		vcast = NULL;
		if (VALA_IS_INTERFACE (vala_symbol_get_parent_symbol ((ValaSymbol*) prop))) {
			ValaInterface* iface;
			ValaCCodeFunctionCall* _tmp47_;
			ValaCCodeIdentifier* _tmp46_;
			char* _tmp45_;
			char* _tmp44_;
			iface = _vala_code_node_ref0 (VALA_INTERFACE (vala_symbol_get_parent_symbol ((ValaSymbol*) prop)));
			vcast = (_tmp47_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp46_ = vala_ccode_identifier_new (_tmp45_ = g_strdup_printf ("%s_GET_INTERFACE", _tmp44_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) iface, NULL))))), _vala_ccode_node_unref0 (vcast), _tmp47_);
			_vala_ccode_node_unref0 (_tmp46_);
			_g_free0 (_tmp45_);
			_g_free0 (_tmp44_);
			_vala_code_node_unref0 (iface);
		} else {
			ValaClass* cl;
			ValaCCodeFunctionCall* _tmp51_;
			ValaCCodeIdentifier* _tmp50_;
			char* _tmp49_;
			char* _tmp48_;
			cl = _vala_code_node_ref0 (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) prop)));
			vcast = (_tmp51_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp50_ = vala_ccode_identifier_new (_tmp49_ = g_strdup_printf ("%s_GET_CLASS", _tmp48_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) cl, NULL))))), _vala_ccode_node_unref0 (vcast), _tmp51_);
			_vala_ccode_node_unref0 (_tmp50_);
			_g_free0 (_tmp49_);
			_g_free0 (_tmp48_);
			_vala_code_node_unref0 (cl);
		}
		vala_ccode_function_call_add_argument (vcast, (ValaCCodeExpression*) (_tmp52_ = vala_ccode_identifier_new ("self")));
		_vala_ccode_node_unref0 (_tmp52_);
		if (vala_property_accessor_get_readable (acc)) {
			ValaCCodeMemberAccess* _tmp54_;
			char* _tmp53_;
			ValaCCodeFunctionCall* _tmp55_;
			ValaCCodeFunctionCall* vcall;
			ValaCCodeIdentifier* _tmp56_;
			vcall = (_tmp55_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp54_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) vcast, _tmp53_ = g_strdup_printf ("get_%s", vala_symbol_get_name ((ValaSymbol*) prop))))), _vala_ccode_node_unref0 (_tmp54_), _g_free0 (_tmp53_), _tmp55_);
			vala_ccode_function_call_add_argument (vcall, (ValaCCodeExpression*) (_tmp56_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp56_);
			if (returns_real_struct) {
				ValaCCodeIdentifier* _tmp57_;
				ValaCCodeExpressionStatement* _tmp58_;
				vala_ccode_function_call_add_argument (vcall, (ValaCCodeExpression*) (_tmp57_ = vala_ccode_identifier_new ("result")));
				_vala_ccode_node_unref0 (_tmp57_);
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp58_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) vcall)));
				_vala_ccode_node_unref0 (_tmp58_);
			} else {
				ValaCCodeReturnStatement* _tmp62_;
				if (VALA_IS_ARRAY_TYPE (vala_property_accessor_get_value_type (acc))) {
					ValaArrayType* array_type;
					array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_property_accessor_get_value_type (acc)));
					{
						gint dim;
						dim = 1;
						{
							gboolean _tmp59_;
							_tmp59_ = TRUE;
							while (TRUE) {
								char* _tmp60_;
								ValaCCodeIdentifier* _tmp61_;
								ValaCCodeIdentifier* len_expr;
								if (!_tmp59_) {
									dim++;
								}
								_tmp59_ = FALSE;
								if (!(dim <= vala_array_type_get_rank (array_type))) {
									break;
								}
								len_expr = (_tmp61_ = vala_ccode_identifier_new (_tmp60_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), "result", dim)), _g_free0 (_tmp60_), _tmp61_);
								vala_ccode_function_call_add_argument (vcall, (ValaCCodeExpression*) len_expr);
								_vala_ccode_node_unref0 (len_expr);
							}
						}
					}
					_vala_code_node_unref0 (array_type);
				}
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp62_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) vcall)));
				_vala_ccode_node_unref0 (_tmp62_);
			}
			_vala_ccode_node_unref0 (vcall);
		} else {
			ValaCCodeMemberAccess* _tmp64_;
			char* _tmp63_;
			ValaCCodeFunctionCall* _tmp65_;
			ValaCCodeFunctionCall* vcall;
			ValaCCodeIdentifier* _tmp66_;
			ValaCCodeIdentifier* _tmp67_;
			ValaCCodeExpressionStatement* _tmp71_;
			vcall = (_tmp65_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp64_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) vcast, _tmp63_ = g_strdup_printf ("set_%s", vala_symbol_get_name ((ValaSymbol*) prop))))), _vala_ccode_node_unref0 (_tmp64_), _g_free0 (_tmp63_), _tmp65_);
			vala_ccode_function_call_add_argument (vcall, (ValaCCodeExpression*) (_tmp66_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp66_);
			vala_ccode_function_call_add_argument (vcall, (ValaCCodeExpression*) (_tmp67_ = vala_ccode_identifier_new ("value")));
			_vala_ccode_node_unref0 (_tmp67_);
			if (VALA_IS_ARRAY_TYPE (vala_property_accessor_get_value_type (acc))) {
				ValaArrayType* array_type;
				array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_property_accessor_get_value_type (acc)));
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp68_;
						_tmp68_ = TRUE;
						while (TRUE) {
							char* _tmp69_;
							ValaCCodeIdentifier* _tmp70_;
							ValaCCodeIdentifier* len_expr;
							if (!_tmp68_) {
								dim++;
							}
							_tmp68_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							len_expr = (_tmp70_ = vala_ccode_identifier_new (_tmp69_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), "value", dim)), _g_free0 (_tmp69_), _tmp70_);
							vala_ccode_function_call_add_argument (vcall, (ValaCCodeExpression*) len_expr);
							_vala_ccode_node_unref0 (len_expr);
						}
					}
				}
				_vala_code_node_unref0 (array_type);
			}
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp71_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) vcall)));
			_vala_ccode_node_unref0 (_tmp71_);
			_vala_ccode_node_unref0 (vcall);
		}
		vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) self->function);
		_vala_ccode_node_unref0 (block);
		_vala_ccode_node_unref0 (vcast);
	}
	if (!vala_property_get_is_abstract (prop)) {
		gboolean _tmp72_;
		gboolean is_virtual;
		char* cname;
		gboolean _tmp78_;
		gboolean _tmp79_;
		ValaObjectType* base_type;
		gboolean _tmp87_;
		gboolean _tmp88_;
		gboolean _tmp107_;
		gboolean _tmp115_;
		gboolean _tmp119_;
		gboolean _tmp120_;
		_tmp72_ = FALSE;
		if (vala_property_get_base_property (prop) != NULL) {
			_tmp72_ = TRUE;
		} else {
			_tmp72_ = vala_property_get_base_interface_property (prop) != NULL;
		}
		is_virtual = _tmp72_;
		cname = NULL;
		if (is_virtual) {
			if (vala_property_accessor_get_readable (acc)) {
				char* _tmp74_;
				char* _tmp73_;
				cname = (_tmp74_ = g_strdup_printf ("%s_real_get_%s", _tmp73_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) t, NULL), vala_symbol_get_name ((ValaSymbol*) prop)), _g_free0 (cname), _tmp74_);
				_g_free0 (_tmp73_);
			} else {
				char* _tmp76_;
				char* _tmp75_;
				cname = (_tmp76_ = g_strdup_printf ("%s_real_set_%s", _tmp75_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) t, NULL), vala_symbol_get_name ((ValaSymbol*) prop)), _g_free0 (cname), _tmp76_);
				_g_free0 (_tmp75_);
			}
		} else {
			char* _tmp77_;
			cname = (_tmp77_ = vala_property_accessor_get_cname (acc), _g_free0 (cname), _tmp77_);
		}
		_tmp78_ = FALSE;
		_tmp79_ = FALSE;
		if (vala_property_accessor_get_writable (acc)) {
			_tmp79_ = TRUE;
		} else {
			_tmp79_ = vala_property_accessor_get_construction (acc);
		}
		if (_tmp79_) {
			_tmp78_ = TRUE;
		} else {
			_tmp78_ = returns_real_struct;
		}
		if (_tmp78_) {
			ValaCCodeFunction* _tmp80_;
			self->function = (_tmp80_ = vala_ccode_function_new (cname, "void"), _vala_ccode_node_unref0 (self->function), _tmp80_);
		} else {
			ValaCCodeFunction* _tmp82_;
			char* _tmp81_;
			self->function = (_tmp82_ = vala_ccode_function_new (cname, _tmp81_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc))), _vala_ccode_node_unref0 (self->function), _tmp82_);
			_g_free0 (_tmp81_);
		}
		base_type = NULL;
		if (vala_property_get_binding (prop) == MEMBER_BINDING_INSTANCE) {
			if (is_virtual) {
				ValaCCodeFormalParameter* _tmp86_;
				char* _tmp85_;
				if (vala_property_get_base_property (prop) != NULL) {
					ValaObjectType* _tmp83_;
					base_type = (_tmp83_ = vala_object_type_new (VALA_OBJECT_TYPE_SYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_property_get_base_property (prop)))), _vala_code_node_unref0 (base_type), _tmp83_);
				} else {
					if (vala_property_get_base_interface_property (prop) != NULL) {
						ValaObjectType* _tmp84_;
						base_type = (_tmp84_ = vala_object_type_new (VALA_OBJECT_TYPE_SYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_property_get_base_interface_property (prop)))), _vala_code_node_unref0 (base_type), _tmp84_);
					}
				}
				vala_ccode_function_set_modifiers (self->function, vala_ccode_function_get_modifiers (self->function) | VALA_CCODE_MODIFIERS_STATIC);
				vala_ccode_function_add_parameter (self->function, _tmp86_ = vala_ccode_formal_parameter_new ("base", _tmp85_ = vala_data_type_get_cname ((ValaDataType*) base_type)));
				_vala_ccode_node_unref0 (_tmp86_);
				_g_free0 (_tmp85_);
			} else {
				vala_ccode_function_add_parameter (self->function, cselfparam);
			}
		}
		_tmp87_ = FALSE;
		_tmp88_ = FALSE;
		if (vala_property_accessor_get_writable (acc)) {
			_tmp88_ = TRUE;
		} else {
			_tmp88_ = vala_property_accessor_get_construction (acc);
		}
		if (_tmp88_) {
			_tmp87_ = TRUE;
		} else {
			_tmp87_ = returns_real_struct;
		}
		if (_tmp87_) {
			vala_ccode_function_add_parameter (self->function, cvalueparam);
		}
		if (VALA_IS_ARRAY_TYPE (vala_property_accessor_get_value_type (acc))) {
			ValaArrayType* array_type;
			char* length_ctype;
			array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_property_accessor_get_value_type (acc)));
			length_ctype = g_strdup ("int");
			if (vala_property_accessor_get_readable (acc)) {
				char* _tmp89_;
				length_ctype = (_tmp89_ = g_strdup ("int*"), _g_free0 (length_ctype), _tmp89_);
			}
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp90_;
					_tmp90_ = TRUE;
					while (TRUE) {
						const char* _tmp91_;
						ValaCCodeFormalParameter* _tmp93_;
						char* _tmp92_;
						if (!_tmp90_) {
							dim++;
						}
						_tmp90_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						_tmp91_ = NULL;
						if (vala_property_accessor_get_readable (acc)) {
							_tmp91_ = "result";
						} else {
							_tmp91_ = "value";
						}
						vala_ccode_function_add_parameter (self->function, _tmp93_ = vala_ccode_formal_parameter_new (_tmp92_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp91_, dim), length_ctype));
						_vala_ccode_node_unref0 (_tmp93_);
						_g_free0 (_tmp92_);
					}
				}
			}
			_vala_code_node_unref0 (array_type);
			_g_free0 (length_ctype);
		} else {
			gboolean _tmp94_;
			_tmp94_ = FALSE;
			if (VALA_IS_DELEGATE_TYPE (vala_property_accessor_get_value_type (acc))) {
				_tmp94_ = vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (VALA_DELEGATE_TYPE (vala_property_accessor_get_value_type (acc))));
			} else {
				_tmp94_ = FALSE;
			}
			if (_tmp94_) {
				const char* _tmp95_;
				const char* _tmp96_;
				ValaCCodeFormalParameter* _tmp98_;
				char* _tmp97_;
				_tmp95_ = NULL;
				if (vala_property_accessor_get_readable (acc)) {
					_tmp95_ = "result";
				} else {
					_tmp95_ = "value";
				}
				_tmp96_ = NULL;
				if (vala_property_accessor_get_readable (acc)) {
					_tmp96_ = "gpointer*";
				} else {
					_tmp96_ = "gpointer";
				}
				vala_ccode_function_add_parameter (self->function, _tmp98_ = vala_ccode_formal_parameter_new (_tmp97_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp95_), _tmp96_));
				_vala_ccode_node_unref0 (_tmp98_);
				_g_free0 (_tmp97_);
			}
		}
		if (!is_virtual) {
			gboolean _tmp99_;
			gboolean _tmp100_;
			_tmp99_ = FALSE;
			_tmp100_ = FALSE;
			if (vala_symbol_is_private_symbol ((ValaSymbol*) prop)) {
				_tmp100_ = TRUE;
			} else {
				gboolean _tmp101_;
				_tmp101_ = FALSE;
				if (vala_property_accessor_get_readable (acc)) {
					_tmp101_ = TRUE;
				} else {
					_tmp101_ = vala_property_accessor_get_writable (acc);
				}
				_tmp100_ = !_tmp101_;
			}
			if (_tmp100_) {
				_tmp99_ = TRUE;
			} else {
				_tmp99_ = vala_symbol_get_access ((ValaSymbol*) acc) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
			}
			if (_tmp99_) {
				vala_ccode_function_set_modifiers (self->function, vala_ccode_function_get_modifiers (self->function) | VALA_CCODE_MODIFIERS_STATIC);
			}
		}
		vala_ccode_function_set_block (self->function, VALA_CCODE_BLOCK (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_property_accessor_get_body (acc))));
		if (is_virtual) {
			char* _tmp102_;
			ValaCCodeDeclaration* _tmp103_;
			ValaCCodeDeclaration* _cdecl_;
			ValaCCodeVariableDeclarator* _tmp106_;
			ValaCCodeExpression* _tmp105_;
			ValaCCodeIdentifier* _tmp104_;
			_cdecl_ = (_tmp103_ = vala_ccode_declaration_new (_tmp102_ = vala_data_type_get_cname (this_type)), _g_free0 (_tmp102_), _tmp103_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp106_ = vala_ccode_variable_declarator_new ("self", _tmp105_ = vala_ccode_base_module_transform_expression (self, (ValaCCodeExpression*) (_tmp104_ = vala_ccode_identifier_new ("base")), (ValaDataType*) base_type, this_type, NULL), NULL)));
			_vala_ccode_node_unref0 (_tmp106_);
			_vala_ccode_node_unref0 (_tmp105_);
			_vala_ccode_node_unref0 (_tmp104_);
			vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->function), (ValaCCodeNode*) _cdecl_);
			_vala_ccode_node_unref0 (_cdecl_);
		}
		_tmp107_ = FALSE;
		if (vala_property_accessor_get_readable (acc)) {
			_tmp107_ = !returns_real_struct;
		} else {
			_tmp107_ = FALSE;
		}
		if (_tmp107_) {
			gboolean _tmp108_;
			_tmp108_ = FALSE;
			if (vala_property_accessor_get_exit_block (acc) == NULL) {
				_tmp108_ = TRUE;
			} else {
				GeeList* _tmp109_;
				_tmp108_ = gee_collection_get_size ((GeeCollection*) (_tmp109_ = vala_basic_block_get_predecessors (vala_property_accessor_get_exit_block (acc)))) > 0;
				_gee_collection_object_unref0 (_tmp109_);
			}
			if (_tmp108_) {
				char* _tmp110_;
				ValaCCodeDeclaration* _tmp111_;
				ValaCCodeDeclaration* _cdecl_;
				ValaCCodeVariableDeclarator* _tmp112_;
				_cdecl_ = (_tmp111_ = vala_ccode_declaration_new (_tmp110_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (acc))), _g_free0 (_tmp110_), _tmp111_);
				vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp112_ = vala_ccode_variable_declarator_new ("result", NULL, NULL)));
				_vala_ccode_node_unref0 (_tmp112_);
				vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->function), (ValaCCodeNode*) _cdecl_);
				_vala_ccode_node_unref0 (_cdecl_);
			}
		}
		if (self->current_method_inner_error) {
			ValaCCodeDeclaration* _cdecl_;
			ValaCCodeVariableDeclarator* _tmp114_;
			ValaCCodeConstant* _tmp113_;
			_cdecl_ = vala_ccode_declaration_new ("GError *");
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp114_ = vala_ccode_variable_declarator_new ("_inner_error_", (ValaCCodeExpression*) (_tmp113_ = vala_ccode_constant_new ("NULL")), NULL)));
			_vala_ccode_node_unref0 (_tmp114_);
			_vala_ccode_node_unref0 (_tmp113_);
			vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->function), (ValaCCodeNode*) _cdecl_);
			_vala_ccode_node_unref0 (_cdecl_);
		}
		_tmp115_ = FALSE;
		if (vala_property_get_binding (prop) == MEMBER_BINDING_INSTANCE) {
			_tmp115_ = !is_virtual;
		} else {
			_tmp115_ = FALSE;
		}
		if (_tmp115_) {
			ValaCCodeStatement* check_stmt;
			gboolean _tmp116_;
			check_stmt = NULL;
			_tmp116_ = FALSE;
			if (!vala_property_accessor_get_readable (acc)) {
				_tmp116_ = TRUE;
			} else {
				_tmp116_ = returns_real_struct;
			}
			if (_tmp116_) {
				ValaCCodeStatement* _tmp117_;
				check_stmt = (_tmp117_ = vala_ccode_base_module_create_property_type_check_statement (self, prop, FALSE, t, TRUE, "self"), _vala_ccode_node_unref0 (check_stmt), _tmp117_);
			} else {
				ValaCCodeStatement* _tmp118_;
				check_stmt = (_tmp118_ = vala_ccode_base_module_create_property_type_check_statement (self, prop, TRUE, t, TRUE, "self"), _vala_ccode_node_unref0 (check_stmt), _tmp118_);
			}
			if (check_stmt != NULL) {
				vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->function), (ValaCCodeNode*) check_stmt);
			}
			_vala_ccode_node_unref0 (check_stmt);
		}
		_tmp119_ = FALSE;
		_tmp120_ = FALSE;
		if (vala_ccode_base_module_is_gobject_property (self, prop)) {
			_tmp120_ = vala_property_get_notify (prop);
		} else {
			_tmp120_ = FALSE;
		}
		if (_tmp120_) {
			gboolean _tmp121_;
			_tmp121_ = FALSE;
			if (vala_property_accessor_get_writable (acc)) {
				_tmp121_ = TRUE;
			} else {
				_tmp121_ = vala_property_accessor_get_construction (acc);
			}
			_tmp119_ = _tmp121_;
		} else {
			_tmp119_ = FALSE;
		}
		if (_tmp119_) {
			ValaCCodeIdentifier* _tmp122_;
			ValaCCodeFunctionCall* _tmp123_;
			ValaCCodeFunctionCall* notify_call;
			ValaCCodeCastExpression* _tmp125_;
			ValaCCodeIdentifier* _tmp124_;
			ValaCCodeConstant* _tmp126_;
			ValaCCodeExpressionStatement* _tmp127_;
			notify_call = (_tmp123_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp122_ = vala_ccode_identifier_new ("g_object_notify"))), _vala_ccode_node_unref0 (_tmp122_), _tmp123_);
			vala_ccode_function_call_add_argument (notify_call, (ValaCCodeExpression*) (_tmp125_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp124_ = vala_ccode_identifier_new ("self")), "GObject *")));
			_vala_ccode_node_unref0 (_tmp125_);
			_vala_ccode_node_unref0 (_tmp124_);
			vala_ccode_function_call_add_argument (notify_call, (ValaCCodeExpression*) (_tmp126_ = vala_property_get_canonical_cconstant (prop)));
			_vala_ccode_node_unref0 (_tmp126_);
			vala_ccode_block_add_statement (vala_ccode_function_get_block (self->function), (ValaCCodeNode*) (_tmp127_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) notify_call)));
			_vala_ccode_node_unref0 (_tmp127_);
			_vala_ccode_node_unref0 (notify_call);
		}
		vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) self->function);
		_g_free0 (cname);
		_vala_code_node_unref0 (base_type);
	}
	self->current_symbol = (_tmp128_ = _vala_code_node_ref0 (old_symbol), _vala_code_node_unref0 (self->current_symbol), _tmp128_);
	self->current_method_inner_error = old_method_inner_error;
	_vala_code_node_unref0 (old_symbol);
	_vala_code_node_unref0 (prop);
	_vala_code_node_unref0 (t);
	_vala_code_node_unref0 (this_type);
	_vala_ccode_node_unref0 (cselfparam);
	_vala_ccode_node_unref0 (cvalueparam);
}


static void vala_ccode_base_module_real_visit_destructor (ValaCCodeModule* base, ValaDestructor* d) {
	ValaCCodeBaseModule * self;
	gboolean old_method_inner_error;
	gboolean _tmp0_;
	ValaCCodeFragment* cfrag;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (d != NULL);
	old_method_inner_error = self->current_method_inner_error;
	self->current_method_inner_error = FALSE;
	vala_code_node_accept_children ((ValaCodeNode*) d, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	_tmp0_ = FALSE;
	if (vala_destructor_get_binding (d) == MEMBER_BINDING_STATIC) {
		_tmp0_ = !self->in_plugin;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) d), "static destructors are only supported for dynamic types");
		vala_code_node_set_error ((ValaCodeNode*) d, TRUE);
		return;
	}
	cfrag = vala_ccode_fragment_new ();
	if (self->current_method_inner_error) {
		ValaCCodeDeclaration* _cdecl_;
		ValaCCodeVariableDeclarator* _tmp2_;
		ValaCCodeConstant* _tmp1_;
		_cdecl_ = vala_ccode_declaration_new ("GError *");
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp2_ = vala_ccode_variable_declarator_new ("_inner_error_", (ValaCCodeExpression*) (_tmp1_ = vala_ccode_constant_new ("NULL")), NULL)));
		_vala_ccode_node_unref0 (_tmp2_);
		_vala_ccode_node_unref0 (_tmp1_);
		vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) _cdecl_);
		_vala_ccode_node_unref0 (_cdecl_);
	}
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode ((ValaCodeNode*) vala_destructor_get_body (d)));
	vala_code_node_set_ccodenode ((ValaCodeNode*) d, (ValaCCodeNode*) cfrag);
	self->current_method_inner_error = old_method_inner_error;
	_vala_ccode_node_unref0 (cfrag);
}


gint vala_ccode_base_module_get_block_id (ValaCCodeBaseModule* self, ValaBlock* b) {
	gint result;
	gint _result_;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (b != NULL, 0);
	_result_ = GPOINTER_TO_INT (gee_map_get (self->priv->block_map, b));
	if (_result_ == 0) {
		_result_ = self->priv->next_block_id = self->priv->next_block_id + 1;
		gee_map_set (self->priv->block_map, b, GINT_TO_POINTER (_result_));
	}
	result = _result_;
	return result;
}


static void vala_ccode_base_module_real_visit_block (ValaCCodeModule* base, ValaBlock* b) {
	ValaCCodeBaseModule * self;
	ValaSymbol* old_symbol;
	ValaSymbol* _tmp0_;
	GeeList* local_vars;
	ValaCCodeBlock* cblock;
	ValaSymbol* _tmp189_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (b != NULL);
	old_symbol = _vala_code_node_ref0 (self->current_symbol);
	self->current_symbol = (_tmp0_ = _vala_code_node_ref0 ((ValaSymbol*) b), _vala_code_node_unref0 (self->current_symbol), _tmp0_);
	vala_code_node_accept_children ((ValaCodeNode*) b, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	local_vars = vala_block_get_local_variables (b);
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) local_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			vala_symbol_set_active ((ValaSymbol*) local, FALSE);
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	cblock = vala_ccode_block_new ();
	if (vala_block_get_captured (b)) {
		ValaSymbol* _tmp1_;
		ValaBlock* parent_block;
		gint block_id;
		char* struct_name;
		ValaCCodeBlock* free_block;
		char* _tmp5_;
		ValaCCodeStruct* _tmp6_;
		ValaCCodeStruct* data;
		ValaCCodeVariableDeclarator* _tmp43_;
		char* _tmp42_;
		ValaCCodeTypeDefinition* _tmp44_;
		ValaCCodeTypeDefinition* _typedef_;
		ValaCCodeIdentifier* _tmp45_;
		ValaCCodeFunctionCall* _tmp46_;
		ValaCCodeFunctionCall* data_alloc;
		ValaCCodeIdentifier* _tmp47_;
		gboolean _tmp48_;
		ValaCCodeExpressionStatement* _tmp64_;
		ValaCCodeAssignment* _tmp63_;
		ValaCCodeIdentifier* _tmp62_;
		ValaCCodeMemberAccess* _tmp61_;
		ValaCCodeExpression* _tmp60_;
		char* _tmp59_;
		ValaCCodeIdentifier* _tmp130_;
		ValaCCodeFunctionCall* _tmp131_;
		ValaCCodeFunctionCall* data_free;
		ValaCCodeIdentifier* _tmp132_;
		ValaCCodeIdentifier* _tmp134_;
		char* _tmp133_;
		ValaCCodeExpressionStatement* _tmp135_;
		char* _tmp137_;
		char* _tmp136_;
		ValaCCodeFunction* _tmp138_;
		ValaCCodeFunction* ref_fun;
		ValaCCodeFormalParameter* _tmp141_;
		char* _tmp140_;
		char* _tmp139_;
		ValaCCodeFunction* _tmp142_;
		ValaCCodeBlock* _tmp143_;
		ValaCCodeExpressionStatement* _tmp148_;
		ValaCCodeUnaryExpression* _tmp147_;
		ValaCCodeMemberAccess* _tmp146_;
		ValaCCodeIdentifier* _tmp145_;
		char* _tmp144_;
		ValaCCodeReturnStatement* _tmp151_;
		ValaCCodeIdentifier* _tmp150_;
		char* _tmp149_;
		char* _tmp153_;
		char* _tmp152_;
		ValaCCodeFunction* _tmp154_;
		ValaCCodeFunction* unref_fun;
		ValaCCodeFormalParameter* _tmp157_;
		char* _tmp156_;
		char* _tmp155_;
		ValaCCodeFunction* _tmp158_;
		ValaCCodeBlock* _tmp159_;
		ValaCCodeConstant* _tmp164_;
		ValaCCodeUnaryExpression* _tmp163_;
		ValaCCodeMemberAccess* _tmp162_;
		ValaCCodeIdentifier* _tmp161_;
		char* _tmp160_;
		ValaCCodeBinaryExpression* _tmp165_;
		ValaCCodeBinaryExpression* dec;
		ValaCCodeIfStatement* _tmp166_;
		parent_block = _vala_code_node_ref0 ((_tmp1_ = vala_symbol_get_parent_symbol ((ValaSymbol*) b), VALA_IS_BLOCK (_tmp1_) ? ((ValaBlock*) _tmp1_) : NULL));
		while (TRUE) {
			gboolean _tmp2_;
			ValaBlock* _tmp4_;
			ValaSymbol* _tmp3_;
			_tmp2_ = FALSE;
			if (parent_block != NULL) {
				_tmp2_ = !vala_block_get_captured (parent_block);
			} else {
				_tmp2_ = FALSE;
			}
			if (!_tmp2_) {
				break;
			}
			parent_block = (_tmp4_ = _vala_code_node_ref0 ((_tmp3_ = vala_symbol_get_parent_symbol ((ValaSymbol*) parent_block), VALA_IS_BLOCK (_tmp3_) ? ((ValaBlock*) _tmp3_) : NULL)), _vala_code_node_unref0 (parent_block), _tmp4_);
		}
		block_id = vala_ccode_base_module_get_block_id (self, b);
		struct_name = g_strdup_printf ("Block%dData", block_id);
		free_block = vala_ccode_block_new ();
		data = (_tmp6_ = vala_ccode_struct_new (_tmp5_ = g_strconcat ("_", struct_name, NULL)), _g_free0 (_tmp5_), _tmp6_);
		vala_ccode_struct_add_field (data, "int", "_ref_count_");
		if (parent_block != NULL) {
			gint parent_block_id;
			char* _tmp8_;
			char* _tmp7_;
			ValaCCodeIdentifier* _tmp10_;
			char* _tmp9_;
			ValaCCodeFunctionCall* _tmp11_;
			ValaCCodeFunctionCall* unref_call;
			ValaCCodeMemberAccess* _tmp15_;
			char* _tmp14_;
			ValaCCodeIdentifier* _tmp13_;
			char* _tmp12_;
			ValaCCodeExpressionStatement* _tmp16_;
			parent_block_id = vala_ccode_base_module_get_block_id (self, parent_block);
			vala_ccode_struct_add_field (data, _tmp7_ = g_strdup_printf ("Block%dData *", parent_block_id), _tmp8_ = g_strdup_printf ("_data%d_", parent_block_id));
			_g_free0 (_tmp8_);
			_g_free0 (_tmp7_);
			unref_call = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new (_tmp9_ = g_strdup_printf ("block%d_data_unref", parent_block_id)))), _vala_ccode_node_unref0 (_tmp10_), _g_free0 (_tmp9_), _tmp11_);
			vala_ccode_function_call_add_argument (unref_call, (ValaCCodeExpression*) (_tmp15_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new (_tmp12_ = g_strdup_printf ("_data%d_", block_id))), _tmp14_ = g_strdup_printf ("_data%d_", parent_block_id))));
			_vala_ccode_node_unref0 (_tmp15_);
			_g_free0 (_tmp14_);
			_vala_ccode_node_unref0 (_tmp13_);
			_g_free0 (_tmp12_);
			vala_ccode_block_add_statement (free_block, (ValaCCodeNode*) (_tmp16_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) unref_call)));
			_vala_ccode_node_unref0 (_tmp16_);
			_vala_ccode_node_unref0 (unref_call);
		} else {
			gboolean _tmp17_;
			_tmp17_ = FALSE;
			if (self->in_constructor) {
				_tmp17_ = TRUE;
			} else {
				gboolean _tmp18_;
				_tmp18_ = FALSE;
				if (vala_ccode_base_module_get_current_method (self) != NULL) {
					_tmp18_ = vala_method_get_binding (vala_ccode_base_module_get_current_method (self)) == MEMBER_BINDING_INSTANCE;
				} else {
					_tmp18_ = FALSE;
				}
				_tmp17_ = _tmp18_;
			}
			if (_tmp17_) {
				char* _tmp20_;
				char* _tmp19_;
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp26_;
				ValaCCodeExpression* _tmp25_;
				ValaObjectType* _tmp24_;
				ValaCCodeMemberAccess* _tmp23_;
				ValaCCodeIdentifier* _tmp22_;
				char* _tmp21_;
				vala_ccode_struct_add_field (data, _tmp20_ = g_strdup_printf ("%s *", _tmp19_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) vala_ccode_base_module_get_current_class (self), FALSE)), "self");
				_g_free0 (_tmp20_);
				_g_free0 (_tmp19_);
				ma = vala_member_access_new_simple ("this", NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) vala_ccode_base_module_get_current_class (self));
				vala_ccode_block_add_statement (free_block, (ValaCCodeNode*) (_tmp26_ = vala_ccode_expression_statement_new (_tmp25_ = vala_ccode_base_module_get_unref_expression (self, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new (_tmp21_ = g_strdup_printf ("_data%d_", block_id))), "self")), (ValaDataType*) (_tmp24_ = vala_object_type_new ((ValaObjectTypeSymbol*) vala_ccode_base_module_get_current_class (self))), (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp26_);
				_vala_ccode_node_unref0 (_tmp25_);
				_vala_code_node_unref0 (_tmp24_);
				_vala_ccode_node_unref0 (_tmp23_);
				_vala_ccode_node_unref0 (_tmp22_);
				_g_free0 (_tmp21_);
				_vala_code_node_unref0 (ma);
			}
		}
		{
			GeeIterator* _local_it;
			_local_it = gee_iterable_iterator ((GeeIterable*) local_vars);
			while (TRUE) {
				ValaLocalVariable* local;
				if (!gee_iterator_next (_local_it)) {
					break;
				}
				local = (ValaLocalVariable*) gee_iterator_get (_local_it);
				if (vala_local_variable_get_captured (local)) {
					char* _tmp30_;
					char* _tmp29_;
					char* _tmp28_;
					char* _tmp27_;
					vala_ccode_struct_add_field (data, _tmp27_ = vala_data_type_get_cname (vala_local_variable_get_variable_type (local)), _tmp30_ = g_strconcat (_tmp28_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), _tmp29_ = vala_data_type_get_cdeclarator_suffix (vala_local_variable_get_variable_type (local)), NULL));
					_g_free0 (_tmp30_);
					_g_free0 (_tmp29_);
					_g_free0 (_tmp28_);
					_g_free0 (_tmp27_);
					if (VALA_IS_ARRAY_TYPE (vala_local_variable_get_variable_type (local))) {
						ValaArrayType* array_type;
						array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_local_variable_get_variable_type (local)));
						{
							gint dim;
							dim = 1;
							{
								gboolean _tmp31_;
								_tmp31_ = TRUE;
								while (TRUE) {
									char* _tmp33_;
									char* _tmp32_;
									if (!_tmp31_) {
										dim++;
									}
									_tmp31_ = FALSE;
									if (!(dim <= vala_array_type_get_rank (array_type))) {
										break;
									}
									vala_ccode_struct_add_field (data, "gint", _tmp33_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, _tmp32_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), dim));
									_g_free0 (_tmp33_);
									_g_free0 (_tmp32_);
								}
							}
						}
						_vala_code_node_unref0 (array_type);
					} else {
						if (VALA_IS_DELEGATE_TYPE (vala_local_variable_get_variable_type (local))) {
							char* _tmp35_;
							char* _tmp34_;
							vala_ccode_struct_add_field (data, "gpointer", _tmp35_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp34_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local))));
							_g_free0 (_tmp35_);
							_g_free0 (_tmp34_);
						}
					}
					if (vala_ccode_base_module_requires_destroy (self, vala_local_variable_get_variable_type (local))) {
						ValaMemberAccess* ma;
						ValaCCodeExpressionStatement* _tmp41_;
						ValaCCodeExpression* _tmp40_;
						ValaCCodeMemberAccess* _tmp39_;
						char* _tmp38_;
						ValaCCodeIdentifier* _tmp37_;
						char* _tmp36_;
						ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) local), NULL);
						vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
						vala_ccode_block_add_statement (free_block, (ValaCCodeNode*) (_tmp41_ = vala_ccode_expression_statement_new (_tmp40_ = vala_ccode_base_module_get_unref_expression (self, (ValaCCodeExpression*) (_tmp39_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp37_ = vala_ccode_identifier_new (_tmp36_ = g_strdup_printf ("_data%d_", block_id))), _tmp38_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)))), vala_local_variable_get_variable_type (local), (ValaExpression*) ma, FALSE))));
						_vala_ccode_node_unref0 (_tmp41_);
						_vala_ccode_node_unref0 (_tmp40_);
						_vala_ccode_node_unref0 (_tmp39_);
						_g_free0 (_tmp38_);
						_vala_ccode_node_unref0 (_tmp37_);
						_g_free0 (_tmp36_);
						_vala_code_node_unref0 (ma);
					}
				}
				_vala_code_node_unref0 (local);
			}
			_gee_collection_object_unref0 (_local_it);
		}
		_typedef_ = (_tmp44_ = vala_ccode_type_definition_new (_tmp42_ = g_strconcat ("struct _", struct_name, NULL), (ValaCCodeDeclarator*) (_tmp43_ = vala_ccode_variable_declarator_new (struct_name, NULL, NULL))), _vala_ccode_node_unref0 (_tmp43_), _g_free0 (_tmp42_), _tmp44_);
		vala_ccode_declaration_space_add_type_declaration (self->source_declarations, (ValaCCodeNode*) _typedef_);
		vala_ccode_declaration_space_add_type_definition (self->source_declarations, (ValaCCodeNode*) data);
		data_alloc = (_tmp46_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("g_slice_new0"))), _vala_ccode_node_unref0 (_tmp45_), _tmp46_);
		vala_ccode_function_call_add_argument (data_alloc, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_identifier_new (struct_name)));
		_vala_ccode_node_unref0 (_tmp47_);
		_tmp48_ = FALSE;
		if (vala_ccode_base_module_get_current_method (self) != NULL) {
			_tmp48_ = vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
		} else {
			_tmp48_ = FALSE;
		}
		if (_tmp48_) {
			char* _tmp50_;
			char* _tmp49_;
			ValaCCodeExpressionStatement* _tmp54_;
			ValaCCodeAssignment* _tmp53_;
			ValaCCodeExpression* _tmp52_;
			char* _tmp51_;
			vala_ccode_struct_add_field (self->closure_struct, _tmp49_ = g_strconcat (struct_name, "*", NULL), _tmp50_ = g_strdup_printf ("_data%d_", block_id));
			_g_free0 (_tmp50_);
			_g_free0 (_tmp49_);
			vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp54_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp53_ = vala_ccode_assignment_new (_tmp52_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp51_ = g_strdup_printf ("_data%d_", block_id)), (ValaCCodeExpression*) data_alloc, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp54_);
			_vala_ccode_node_unref0 (_tmp53_);
			_vala_ccode_node_unref0 (_tmp52_);
			_g_free0 (_tmp51_);
		} else {
			char* _tmp55_;
			ValaCCodeDeclaration* _tmp56_;
			ValaCCodeDeclaration* data_decl;
			ValaCCodeVariableDeclarator* _tmp58_;
			char* _tmp57_;
			data_decl = (_tmp56_ = vala_ccode_declaration_new (_tmp55_ = g_strconcat (struct_name, "*", NULL)), _g_free0 (_tmp55_), _tmp56_);
			vala_ccode_declaration_add_declarator (data_decl, (ValaCCodeDeclarator*) (_tmp58_ = vala_ccode_variable_declarator_new (_tmp57_ = g_strdup_printf ("_data%d_", block_id), (ValaCCodeExpression*) data_alloc, NULL)));
			_vala_ccode_node_unref0 (_tmp58_);
			_g_free0 (_tmp57_);
			vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) data_decl);
			_vala_ccode_node_unref0 (data_decl);
		}
		vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp64_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp63_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp61_ = vala_ccode_member_access_new_pointer (_tmp60_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp59_ = g_strdup_printf ("_data%d_", block_id)), "_ref_count_")), (ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new ("1")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp64_);
		_vala_ccode_node_unref0 (_tmp63_);
		_vala_ccode_node_unref0 (_tmp62_);
		_vala_ccode_node_unref0 (_tmp61_);
		_vala_ccode_node_unref0 (_tmp60_);
		_g_free0 (_tmp59_);
		if (parent_block != NULL) {
			gint parent_block_id;
			ValaCCodeIdentifier* _tmp66_;
			char* _tmp65_;
			ValaCCodeFunctionCall* _tmp67_;
			ValaCCodeFunctionCall* ref_call;
			ValaCCodeExpression* _tmp69_;
			char* _tmp68_;
			ValaCCodeExpressionStatement* _tmp75_;
			ValaCCodeAssignment* _tmp74_;
			ValaCCodeMemberAccess* _tmp73_;
			char* _tmp72_;
			ValaCCodeExpression* _tmp71_;
			char* _tmp70_;
			parent_block_id = vala_ccode_base_module_get_block_id (self, parent_block);
			ref_call = (_tmp67_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp66_ = vala_ccode_identifier_new (_tmp65_ = g_strdup_printf ("block%d_data_ref", parent_block_id)))), _vala_ccode_node_unref0 (_tmp66_), _g_free0 (_tmp65_), _tmp67_);
			vala_ccode_function_call_add_argument (ref_call, _tmp69_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp68_ = g_strdup_printf ("_data%d_", parent_block_id)));
			_vala_ccode_node_unref0 (_tmp69_);
			_g_free0 (_tmp68_);
			vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp75_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp74_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp73_ = vala_ccode_member_access_new_pointer (_tmp71_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp70_ = g_strdup_printf ("_data%d_", block_id)), _tmp72_ = g_strdup_printf ("_data%d_", parent_block_id))), (ValaCCodeExpression*) ref_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp75_);
			_vala_ccode_node_unref0 (_tmp74_);
			_vala_ccode_node_unref0 (_tmp73_);
			_g_free0 (_tmp72_);
			_vala_ccode_node_unref0 (_tmp71_);
			_g_free0 (_tmp70_);
			_vala_ccode_node_unref0 (ref_call);
		} else {
			gboolean _tmp76_;
			_tmp76_ = FALSE;
			if (self->in_constructor) {
				_tmp76_ = TRUE;
			} else {
				gboolean _tmp77_;
				_tmp77_ = FALSE;
				if (vala_ccode_base_module_get_current_method (self) != NULL) {
					_tmp77_ = vala_method_get_binding (vala_ccode_base_module_get_current_method (self)) == MEMBER_BINDING_INSTANCE;
				} else {
					_tmp77_ = FALSE;
				}
				_tmp76_ = _tmp77_;
			}
			if (_tmp76_) {
				ValaCCodeExpression* _tmp79_;
				ValaObjectType* _tmp78_;
				ValaCCodeFunctionCall* _tmp80_;
				ValaCCodeFunctionCall* ref_call;
				ValaCCodeIdentifier* _tmp81_;
				ValaCCodeExpressionStatement* _tmp86_;
				ValaCCodeAssignment* _tmp85_;
				ValaCCodeMemberAccess* _tmp84_;
				ValaCCodeExpression* _tmp83_;
				char* _tmp82_;
				ref_call = (_tmp80_ = vala_ccode_function_call_new (_tmp79_ = vala_ccode_base_module_get_dup_func_expression (self, (ValaDataType*) (_tmp78_ = vala_object_type_new ((ValaObjectTypeSymbol*) vala_ccode_base_module_get_current_class (self))), vala_code_node_get_source_reference ((ValaCodeNode*) b), FALSE)), _vala_ccode_node_unref0 (_tmp79_), _vala_code_node_unref0 (_tmp78_), _tmp80_);
				vala_ccode_function_call_add_argument (ref_call, (ValaCCodeExpression*) (_tmp81_ = vala_ccode_identifier_new ("self")));
				_vala_ccode_node_unref0 (_tmp81_);
				vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp86_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp85_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp84_ = vala_ccode_member_access_new_pointer (_tmp83_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp82_ = g_strdup_printf ("_data%d_", block_id)), "self")), (ValaCCodeExpression*) ref_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp86_);
				_vala_ccode_node_unref0 (_tmp85_);
				_vala_ccode_node_unref0 (_tmp84_);
				_vala_ccode_node_unref0 (_tmp83_);
				_g_free0 (_tmp82_);
				_vala_ccode_node_unref0 (ref_call);
			}
		}
		if (VALA_IS_METHOD (vala_symbol_get_parent_symbol ((ValaSymbol*) b))) {
			ValaCCodeFragment* cfrag;
			{
				GeeList* _tmp87_;
				GeeIterator* _tmp88_;
				GeeIterator* _param_it;
				_param_it = (_tmp88_ = gee_iterable_iterator ((GeeIterable*) (_tmp87_ = vala_method_get_parameters (VALA_METHOD (vala_symbol_get_parent_symbol ((ValaSymbol*) b))))), _gee_collection_object_unref0 (_tmp87_), _tmp88_);
				while (TRUE) {
					ValaFormalParameter* param;
					if (!gee_iterator_next (_param_it)) {
						break;
					}
					param = (ValaFormalParameter*) gee_iterator_get (_param_it);
					if (vala_formal_parameter_get_captured (param)) {
						ValaDataType* param_type;
						char* _tmp90_;
						char* _tmp89_;
						ValaCCodeExpression* cparam;
						gboolean _tmp91_;
						ValaCCodeExpressionStatement* _tmp98_;
						ValaCCodeAssignment* _tmp97_;
						ValaCCodeMemberAccess* _tmp96_;
						char* _tmp95_;
						ValaCCodeExpression* _tmp94_;
						char* _tmp93_;
						param_type = vala_data_type_copy (vala_formal_parameter_get_parameter_type (param));
						vala_data_type_set_value_owned (param_type, TRUE);
						vala_ccode_struct_add_field (data, _tmp89_ = vala_data_type_get_cname (param_type), _tmp90_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param)));
						_g_free0 (_tmp90_);
						_g_free0 (_tmp89_);
						cparam = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) param));
						_tmp91_ = FALSE;
						if (vala_ccode_base_module_requires_copy (self, param_type)) {
							_tmp91_ = !vala_data_type_get_value_owned (vala_formal_parameter_get_parameter_type (param));
						} else {
							_tmp91_ = FALSE;
						}
						if (_tmp91_) {
							ValaMemberAccess* ma;
							ValaCCodeExpression* _tmp92_;
							ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) param), NULL);
							vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) param);
							vala_formal_parameter_set_captured (param, FALSE);
							cparam = (_tmp92_ = vala_ccode_base_module_get_ref_cexpression (self, vala_formal_parameter_get_parameter_type (param), cparam, (ValaExpression*) ma, (ValaCodeNode*) param), _vala_ccode_node_unref0 (cparam), _tmp92_);
							vala_formal_parameter_set_captured (param, TRUE);
							_vala_code_node_unref0 (ma);
						}
						vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp98_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp97_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp96_ = vala_ccode_member_access_new_pointer (_tmp94_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp93_ = g_strdup_printf ("_data%d_", block_id)), _tmp95_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param)))), cparam, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
						_vala_ccode_node_unref0 (_tmp98_);
						_vala_ccode_node_unref0 (_tmp97_);
						_vala_ccode_node_unref0 (_tmp96_);
						_g_free0 (_tmp95_);
						_vala_ccode_node_unref0 (_tmp94_);
						_g_free0 (_tmp93_);
						if (VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param))) {
							ValaArrayType* array_type;
							array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)));
							{
								gint dim;
								dim = 1;
								{
									gboolean _tmp99_;
									_tmp99_ = TRUE;
									while (TRUE) {
										char* _tmp101_;
										char* _tmp100_;
										ValaCCodeExpressionStatement* _tmp111_;
										ValaCCodeAssignment* _tmp110_;
										ValaCCodeIdentifier* _tmp109_;
										char* _tmp108_;
										char* _tmp107_;
										ValaCCodeMemberAccess* _tmp106_;
										char* _tmp105_;
										char* _tmp104_;
										ValaCCodeExpression* _tmp103_;
										char* _tmp102_;
										if (!_tmp99_) {
											dim++;
										}
										_tmp99_ = FALSE;
										if (!(dim <= vala_array_type_get_rank (array_type))) {
											break;
										}
										vala_ccode_struct_add_field (data, "gint", _tmp101_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, _tmp100_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param)), dim));
										_g_free0 (_tmp101_);
										_g_free0 (_tmp100_);
										vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp111_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp110_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp106_ = vala_ccode_member_access_new_pointer (_tmp103_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp102_ = g_strdup_printf ("_data%d_", block_id)), _tmp105_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, _tmp104_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param)), dim))), (ValaCCodeExpression*) (_tmp109_ = vala_ccode_identifier_new (_tmp108_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, _tmp107_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param)), dim))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
										_vala_ccode_node_unref0 (_tmp111_);
										_vala_ccode_node_unref0 (_tmp110_);
										_vala_ccode_node_unref0 (_tmp109_);
										_g_free0 (_tmp108_);
										_g_free0 (_tmp107_);
										_vala_ccode_node_unref0 (_tmp106_);
										_g_free0 (_tmp105_);
										_g_free0 (_tmp104_);
										_vala_ccode_node_unref0 (_tmp103_);
										_g_free0 (_tmp102_);
									}
								}
							}
							_vala_code_node_unref0 (array_type);
						} else {
							if (VALA_IS_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param))) {
								char* _tmp113_;
								char* _tmp112_;
								ValaCCodeExpressionStatement* _tmp123_;
								ValaCCodeAssignment* _tmp122_;
								ValaCCodeIdentifier* _tmp121_;
								char* _tmp120_;
								char* _tmp119_;
								ValaCCodeMemberAccess* _tmp118_;
								char* _tmp117_;
								char* _tmp116_;
								ValaCCodeExpression* _tmp115_;
								char* _tmp114_;
								vala_ccode_struct_add_field (data, "gpointer", _tmp113_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp112_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param))));
								_g_free0 (_tmp113_);
								_g_free0 (_tmp112_);
								vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp123_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp122_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp118_ = vala_ccode_member_access_new_pointer (_tmp115_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp114_ = g_strdup_printf ("_data%d_", block_id)), _tmp117_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp116_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param))))), (ValaCCodeExpression*) (_tmp121_ = vala_ccode_identifier_new (_tmp120_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp119_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
								_vala_ccode_node_unref0 (_tmp123_);
								_vala_ccode_node_unref0 (_tmp122_);
								_vala_ccode_node_unref0 (_tmp121_);
								_g_free0 (_tmp120_);
								_g_free0 (_tmp119_);
								_vala_ccode_node_unref0 (_tmp118_);
								_g_free0 (_tmp117_);
								_g_free0 (_tmp116_);
								_vala_ccode_node_unref0 (_tmp115_);
								_g_free0 (_tmp114_);
							}
						}
						if (vala_ccode_base_module_requires_destroy (self, param_type)) {
							ValaMemberAccess* ma;
							ValaCCodeExpressionStatement* _tmp129_;
							ValaCCodeExpression* _tmp128_;
							ValaCCodeMemberAccess* _tmp127_;
							char* _tmp126_;
							ValaCCodeIdentifier* _tmp125_;
							char* _tmp124_;
							ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) param), NULL);
							vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) param);
							vala_ccode_block_add_statement (free_block, (ValaCCodeNode*) (_tmp129_ = vala_ccode_expression_statement_new (_tmp128_ = vala_ccode_base_module_get_unref_expression (self, (ValaCCodeExpression*) (_tmp127_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp125_ = vala_ccode_identifier_new (_tmp124_ = g_strdup_printf ("_data%d_", block_id))), _tmp126_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) param)))), vala_formal_parameter_get_parameter_type (param), (ValaExpression*) ma, FALSE))));
							_vala_ccode_node_unref0 (_tmp129_);
							_vala_ccode_node_unref0 (_tmp128_);
							_vala_ccode_node_unref0 (_tmp127_);
							_g_free0 (_tmp126_);
							_vala_ccode_node_unref0 (_tmp125_);
							_g_free0 (_tmp124_);
							_vala_code_node_unref0 (ma);
						}
						_vala_code_node_unref0 (param_type);
						_vala_ccode_node_unref0 (cparam);
					}
					_vala_code_node_unref0 (param);
				}
				_gee_collection_object_unref0 (_param_it);
			}
			cfrag = vala_ccode_fragment_new ();
			vala_ccode_base_module_append_temp_decl (self, cfrag, (GeeList*) self->temp_vars);
			gee_collection_clear ((GeeCollection*) self->temp_vars);
			vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) cfrag);
			_vala_ccode_node_unref0 (cfrag);
		}
		data_free = (_tmp131_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp130_ = vala_ccode_identifier_new ("g_slice_free"))), _vala_ccode_node_unref0 (_tmp130_), _tmp131_);
		vala_ccode_function_call_add_argument (data_free, (ValaCCodeExpression*) (_tmp132_ = vala_ccode_identifier_new (struct_name)));
		_vala_ccode_node_unref0 (_tmp132_);
		vala_ccode_function_call_add_argument (data_free, (ValaCCodeExpression*) (_tmp134_ = vala_ccode_identifier_new (_tmp133_ = g_strdup_printf ("_data%d_", block_id))));
		_vala_ccode_node_unref0 (_tmp134_);
		_g_free0 (_tmp133_);
		vala_ccode_block_add_statement (free_block, (ValaCCodeNode*) (_tmp135_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) data_free)));
		_vala_ccode_node_unref0 (_tmp135_);
		ref_fun = (_tmp138_ = vala_ccode_function_new (_tmp136_ = g_strdup_printf ("block%d_data_ref", block_id), _tmp137_ = g_strconcat (struct_name, "*", NULL)), _g_free0 (_tmp137_), _g_free0 (_tmp136_), _tmp138_);
		vala_ccode_function_add_parameter (ref_fun, _tmp141_ = vala_ccode_formal_parameter_new (_tmp139_ = g_strdup_printf ("_data%d_", block_id), _tmp140_ = g_strconcat (struct_name, "*", NULL)));
		_vala_ccode_node_unref0 (_tmp141_);
		_g_free0 (_tmp140_);
		_g_free0 (_tmp139_);
		vala_ccode_function_set_modifiers (ref_fun, VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp142_ = vala_ccode_function_copy (ref_fun)));
		_vala_ccode_node_unref0 (_tmp142_);
		vala_ccode_function_set_block (ref_fun, _tmp143_ = vala_ccode_block_new ());
		_vala_ccode_node_unref0 (_tmp143_);
		vala_ccode_block_add_statement (vala_ccode_function_get_block (ref_fun), (ValaCCodeNode*) (_tmp148_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp147_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_PREFIX_INCREMENT, (ValaCCodeExpression*) (_tmp146_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp145_ = vala_ccode_identifier_new (_tmp144_ = g_strdup_printf ("_data%d_", block_id))), "_ref_count_")))))));
		_vala_ccode_node_unref0 (_tmp148_);
		_vala_ccode_node_unref0 (_tmp147_);
		_vala_ccode_node_unref0 (_tmp146_);
		_vala_ccode_node_unref0 (_tmp145_);
		_g_free0 (_tmp144_);
		vala_ccode_block_add_statement (vala_ccode_function_get_block (ref_fun), (ValaCCodeNode*) (_tmp151_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp150_ = vala_ccode_identifier_new (_tmp149_ = g_strdup_printf ("_data%d_", block_id))))));
		_vala_ccode_node_unref0 (_tmp151_);
		_vala_ccode_node_unref0 (_tmp150_);
		_g_free0 (_tmp149_);
		vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) ref_fun);
		unref_fun = (_tmp154_ = vala_ccode_function_new (_tmp152_ = g_strdup_printf ("block%d_data_unref", block_id), _tmp153_ = g_strconcat (struct_name, "*", NULL)), _g_free0 (_tmp153_), _g_free0 (_tmp152_), _tmp154_);
		vala_ccode_function_add_parameter (unref_fun, _tmp157_ = vala_ccode_formal_parameter_new (_tmp155_ = g_strdup_printf ("_data%d_", block_id), _tmp156_ = g_strconcat (struct_name, "*", NULL)));
		_vala_ccode_node_unref0 (_tmp157_);
		_g_free0 (_tmp156_);
		_g_free0 (_tmp155_);
		vala_ccode_function_set_modifiers (unref_fun, VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp158_ = vala_ccode_function_copy (unref_fun)));
		_vala_ccode_node_unref0 (_tmp158_);
		vala_ccode_function_set_block (unref_fun, _tmp159_ = vala_ccode_block_new ());
		_vala_ccode_node_unref0 (_tmp159_);
		dec = (_tmp165_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp163_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_PREFIX_DECREMENT, (ValaCCodeExpression*) (_tmp162_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp161_ = vala_ccode_identifier_new (_tmp160_ = g_strdup_printf ("_data%d_", block_id))), "_ref_count_")))), (ValaCCodeExpression*) (_tmp164_ = vala_ccode_constant_new ("0"))), _vala_ccode_node_unref0 (_tmp164_), _vala_ccode_node_unref0 (_tmp163_), _vala_ccode_node_unref0 (_tmp162_), _vala_ccode_node_unref0 (_tmp161_), _g_free0 (_tmp160_), _tmp165_);
		vala_ccode_block_add_statement (vala_ccode_function_get_block (unref_fun), (ValaCCodeNode*) (_tmp166_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) dec, (ValaCCodeStatement*) free_block, NULL)));
		_vala_ccode_node_unref0 (_tmp166_);
		vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) unref_fun);
		_vala_code_node_unref0 (parent_block);
		_g_free0 (struct_name);
		_vala_ccode_node_unref0 (free_block);
		_vala_ccode_node_unref0 (data);
		_vala_ccode_node_unref0 (_typedef_);
		_vala_ccode_node_unref0 (data_alloc);
		_vala_ccode_node_unref0 (data_free);
		_vala_ccode_node_unref0 (ref_fun);
		_vala_ccode_node_unref0 (unref_fun);
		_vala_ccode_node_unref0 (dec);
	}
	{
		GeeList* _tmp167_;
		GeeIterator* _tmp168_;
		GeeIterator* _stmt_it;
		_stmt_it = (_tmp168_ = gee_iterable_iterator ((GeeIterable*) (_tmp167_ = vala_block_get_statements (b))), _gee_collection_object_unref0 (_tmp167_), _tmp168_);
		while (TRUE) {
			ValaCodeNode* stmt;
			if (!gee_iterator_next (_stmt_it)) {
				break;
			}
			stmt = (ValaCodeNode*) ((ValaStatement*) gee_iterator_get (_stmt_it));
			if (vala_code_node_get_error (stmt)) {
				_vala_code_node_unref0 (stmt);
				continue;
			}
			if (VALA_IS_CCODE_FRAGMENT (vala_code_node_get_ccodenode (stmt))) {
				{
					GeeList* _tmp169_;
					GeeIterator* _tmp170_;
					GeeIterator* _cstmt_it;
					_cstmt_it = (_tmp170_ = gee_iterable_iterator ((GeeIterable*) (_tmp169_ = vala_ccode_fragment_get_children (VALA_CCODE_FRAGMENT (vala_code_node_get_ccodenode (stmt))))), _gee_collection_object_unref0 (_tmp169_), _tmp170_);
					while (TRUE) {
						ValaCCodeNode* cstmt;
						if (!gee_iterator_next (_cstmt_it)) {
							break;
						}
						cstmt = (ValaCCodeNode*) gee_iterator_get (_cstmt_it);
						vala_ccode_block_add_statement (cblock, cstmt);
						_vala_ccode_node_unref0 (cstmt);
					}
					_gee_collection_object_unref0 (_cstmt_it);
				}
			} else {
				vala_ccode_block_add_statement (cblock, vala_code_node_get_ccodenode (stmt));
			}
			_vala_code_node_unref0 (stmt);
		}
		_gee_collection_object_unref0 (_stmt_it);
	}
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) local_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			gboolean _tmp171_;
			gboolean _tmp172_;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			_tmp171_ = FALSE;
			_tmp172_ = FALSE;
			if (!vala_local_variable_get_floating (local)) {
				_tmp172_ = !vala_local_variable_get_captured (local);
			} else {
				_tmp172_ = FALSE;
			}
			if (_tmp172_) {
				_tmp171_ = vala_ccode_base_module_requires_destroy (self, vala_local_variable_get_variable_type (local));
			} else {
				_tmp171_ = FALSE;
			}
			if (_tmp171_) {
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp175_;
				ValaCCodeExpression* _tmp174_;
				ValaCCodeExpression* _tmp173_;
				ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) local), NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
				vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp175_ = vala_ccode_expression_statement_new (_tmp174_ = vala_ccode_base_module_get_unref_expression (self, _tmp173_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) local)), vala_local_variable_get_variable_type (local), (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp175_);
				_vala_ccode_node_unref0 (_tmp174_);
				_vala_ccode_node_unref0 (_tmp173_);
				_vala_code_node_unref0 (ma);
			}
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	if (VALA_IS_METHOD (vala_symbol_get_parent_symbol ((ValaSymbol*) b))) {
		ValaMethod* m;
		m = _vala_code_node_ref0 (VALA_METHOD (vala_symbol_get_parent_symbol ((ValaSymbol*) b)));
		{
			GeeList* _tmp176_;
			GeeIterator* _tmp177_;
			GeeIterator* _param_it;
			_param_it = (_tmp177_ = gee_iterable_iterator ((GeeIterable*) (_tmp176_ = vala_method_get_parameters (m))), _gee_collection_object_unref0 (_tmp176_), _tmp177_);
			while (TRUE) {
				ValaFormalParameter* param;
				gboolean _tmp178_;
				gboolean _tmp179_;
				if (!gee_iterator_next (_param_it)) {
					break;
				}
				param = (ValaFormalParameter*) gee_iterator_get (_param_it);
				_tmp178_ = FALSE;
				_tmp179_ = FALSE;
				if (!vala_formal_parameter_get_captured (param)) {
					_tmp179_ = vala_ccode_base_module_requires_destroy (self, vala_formal_parameter_get_parameter_type (param));
				} else {
					_tmp179_ = FALSE;
				}
				if (_tmp179_) {
					_tmp178_ = vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN;
				} else {
					_tmp178_ = FALSE;
				}
				if (_tmp178_) {
					ValaMemberAccess* ma;
					ValaCCodeExpressionStatement* _tmp182_;
					ValaCCodeExpression* _tmp181_;
					ValaCCodeExpression* _tmp180_;
					ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) param), NULL);
					vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) param);
					vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp182_ = vala_ccode_expression_statement_new (_tmp181_ = vala_ccode_base_module_get_unref_expression (self, _tmp180_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) param)), vala_formal_parameter_get_parameter_type (param), (ValaExpression*) ma, FALSE))));
					_vala_ccode_node_unref0 (_tmp182_);
					_vala_ccode_node_unref0 (_tmp181_);
					_vala_ccode_node_unref0 (_tmp180_);
					_vala_code_node_unref0 (ma);
				}
				_vala_code_node_unref0 (param);
			}
			_gee_collection_object_unref0 (_param_it);
		}
		_vala_code_node_unref0 (m);
	}
	if (vala_block_get_captured (b)) {
		gint block_id;
		ValaCCodeIdentifier* _tmp184_;
		char* _tmp183_;
		ValaCCodeFunctionCall* _tmp185_;
		ValaCCodeFunctionCall* data_unref;
		ValaCCodeExpression* _tmp187_;
		char* _tmp186_;
		ValaCCodeExpressionStatement* _tmp188_;
		block_id = vala_ccode_base_module_get_block_id (self, b);
		data_unref = (_tmp185_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp184_ = vala_ccode_identifier_new (_tmp183_ = g_strdup_printf ("block%d_data_unref", block_id)))), _vala_ccode_node_unref0 (_tmp184_), _g_free0 (_tmp183_), _tmp185_);
		vala_ccode_function_call_add_argument (data_unref, _tmp187_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp186_ = g_strdup_printf ("_data%d_", block_id)));
		_vala_ccode_node_unref0 (_tmp187_);
		_g_free0 (_tmp186_);
		vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp188_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) data_unref)));
		_vala_ccode_node_unref0 (_tmp188_);
		_vala_ccode_node_unref0 (data_unref);
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) b, (ValaCCodeNode*) cblock);
	self->current_symbol = (_tmp189_ = _vala_code_node_ref0 (old_symbol), _vala_code_node_unref0 (self->current_symbol), _tmp189_);
	_vala_code_node_unref0 (old_symbol);
	_gee_collection_object_unref0 (local_vars);
	_vala_ccode_node_unref0 (cblock);
}


static void vala_ccode_base_module_real_visit_empty_statement (ValaCCodeModule* base, ValaEmptyStatement* stmt) {
	ValaCCodeBaseModule * self;
	ValaCCodeEmptyStatement* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (stmt != NULL);
	vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) (_tmp0_ = vala_ccode_empty_statement_new ()));
	_vala_ccode_node_unref0 (_tmp0_);
}


static void vala_ccode_base_module_real_visit_declaration_statement (ValaCCodeModule* base, ValaDeclarationStatement* stmt) {
	ValaCCodeBaseModule * self;
	ValaSymbol* _tmp0_;
	ValaLocalVariable* local;
	gboolean _tmp1_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (stmt != NULL);
	vala_code_node_accept ((ValaCodeNode*) vala_declaration_statement_get_declaration (stmt), (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, vala_code_node_get_ccodenode ((ValaCodeNode*) vala_declaration_statement_get_declaration (stmt)));
	local = _vala_code_node_ref0 ((_tmp0_ = vala_declaration_statement_get_declaration (stmt), VALA_IS_LOCAL_VARIABLE (_tmp0_) ? ((ValaLocalVariable*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (local != NULL) {
		_tmp1_ = vala_local_variable_get_initializer (local) != NULL;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		vala_ccode_base_module_create_temp_decl (self, (ValaStatement*) stmt, (GeeList*) vala_local_variable_get_initializer (local)->temp_vars);
	}
	vala_ccode_base_module_create_temp_decl (self, (ValaStatement*) stmt, (GeeList*) self->temp_vars);
	gee_collection_clear ((GeeCollection*) self->temp_vars);
	_vala_code_node_unref0 (local);
}


ValaCCodeExpression* vala_ccode_base_module_get_variable_cexpression (ValaCCodeBaseModule* self, const char* name) {
	ValaCCodeExpression* result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	_tmp0_ = FALSE;
	if (vala_ccode_base_module_get_current_method (self) != NULL) {
		_tmp0_ = vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		char* _tmp2_;
		ValaCCodeIdentifier* _tmp1_;
		ValaCCodeExpression* _tmp3_;
		result = (_tmp3_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp1_ = vala_ccode_identifier_new ("data")), _tmp2_ = vala_ccode_base_module_get_variable_cname (self, name)), _g_free0 (_tmp2_), _vala_ccode_node_unref0 (_tmp1_), _tmp3_);
		return result;
	} else {
		char* _tmp4_;
		ValaCCodeExpression* _tmp5_;
		result = (_tmp5_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp4_ = vala_ccode_base_module_get_variable_cname (self, name)), _g_free0 (_tmp4_), _tmp5_);
		return result;
	}
}


char* vala_ccode_base_module_get_variable_cname (ValaCCodeBaseModule* self, const char* name) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	if (g_utf8_get_char (g_utf8_offset_to_pointer (name, 0)) == '.') {
		if (_vala_strcmp0 (name, ".result") == 0) {
			result = g_strdup ("result");
			return result;
		}
		if (!gee_map_contains (self->variable_name_map, name)) {
			char* _tmp0_;
			gee_map_set (self->variable_name_map, name, _tmp0_ = g_strdup_printf ("_tmp%d_", self->next_temp_var_id));
			_g_free0 (_tmp0_);
			self->next_temp_var_id++;
		}
		result = (char*) gee_map_get (self->variable_name_map, name);
		return result;
	} else {
		if (gee_collection_contains ((GeeCollection*) self->priv->reserved_identifiers, name)) {
			result = g_strdup_printf ("_%s_", name);
			return result;
		} else {
			result = g_strdup (name);
			return result;
		}
	}
}


ValaCCodeExpression* vala_ccode_base_module_get_result_cexpression (ValaCCodeBaseModule* self, const char* cname) {
	ValaCCodeExpression* result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (cname != NULL, NULL);
	_tmp0_ = FALSE;
	if (vala_ccode_base_module_get_current_method (self) != NULL) {
		_tmp0_ = vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaCCodeIdentifier* _tmp1_;
		ValaCCodeExpression* _tmp2_;
		result = (_tmp2_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp1_ = vala_ccode_identifier_new ("data")), cname), _vala_ccode_node_unref0 (_tmp1_), _tmp2_);
		return result;
	} else {
		result = (ValaCCodeExpression*) vala_ccode_identifier_new (cname);
		return result;
	}
}


static void vala_ccode_base_module_real_visit_local_variable (ValaCCodeModule* base, ValaLocalVariable* local) {
	ValaCCodeBaseModule * self;
	ValaCCodeExpression* rhs;
	gboolean _tmp14_;
	ValaCCodeFragment* cfrag;
	gboolean _tmp77_;
	gboolean _tmp89_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (local != NULL);
	vala_ccode_base_module_check_type (self, vala_local_variable_get_variable_type (local));
	vala_code_node_accept_children ((ValaCodeNode*) local, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_base_module_generate_type_declaration (self, vala_local_variable_get_variable_type (local), self->source_declarations);
	if (VALA_IS_ARRAY_TYPE (vala_local_variable_get_variable_type (local))) {
		ValaArrayType* array_type;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_local_variable_get_variable_type (local)));
		if (!vala_array_type_get_fixed_length (array_type)) {
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp0_;
					_tmp0_ = TRUE;
					while (TRUE) {
						char* _tmp3_;
						char* _tmp2_;
						ValaDataType* _tmp1_;
						ValaLocalVariable* _tmp4_;
						ValaLocalVariable* len_var;
						if (!_tmp0_) {
							dim++;
						}
						_tmp0_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						len_var = (_tmp4_ = vala_local_variable_new (_tmp1_ = vala_data_type_copy (self->int_type), _tmp3_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp2_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), dim), NULL, NULL), _g_free0 (_tmp3_), _g_free0 (_tmp2_), _vala_code_node_unref0 (_tmp1_), _tmp4_);
						gee_list_insert ((GeeList*) self->temp_vars, 0, len_var);
						_vala_code_node_unref0 (len_var);
					}
				}
			}
			if (vala_array_type_get_rank (array_type) == 1) {
				char* _tmp7_;
				char* _tmp6_;
				ValaDataType* _tmp5_;
				ValaLocalVariable* _tmp8_;
				ValaLocalVariable* size_var;
				size_var = (_tmp8_ = vala_local_variable_new (_tmp5_ = vala_data_type_copy (self->int_type), _tmp7_ = vala_ccode_module_get_array_size_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp6_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local))), NULL, NULL), _g_free0 (_tmp7_), _g_free0 (_tmp6_), _vala_code_node_unref0 (_tmp5_), _tmp8_);
				gee_list_insert ((GeeList*) self->temp_vars, 0, size_var);
				_vala_code_node_unref0 (size_var);
			}
		}
		_vala_code_node_unref0 (array_type);
	} else {
		if (VALA_IS_DELEGATE_TYPE (vala_local_variable_get_variable_type (local))) {
			ValaDelegateType* deleg_type;
			ValaDelegate* d;
			deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_local_variable_get_variable_type (local)));
			d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
			if (vala_delegate_get_has_target (d)) {
				char* _tmp12_;
				char* _tmp11_;
				ValaPointerType* _tmp10_;
				ValaVoidType* _tmp9_;
				ValaLocalVariable* _tmp13_;
				ValaLocalVariable* target_var;
				target_var = (_tmp13_ = vala_local_variable_new ((ValaDataType*) (_tmp10_ = vala_pointer_type_new ((ValaDataType*) (_tmp9_ = vala_void_type_new (NULL)), NULL)), _tmp12_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp11_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local))), NULL, NULL), _g_free0 (_tmp12_), _g_free0 (_tmp11_), _vala_code_node_unref0 (_tmp10_), _vala_code_node_unref0 (_tmp9_), _tmp13_);
				gee_list_insert ((GeeList*) self->temp_vars, 0, target_var);
				_vala_code_node_unref0 (target_var);
			}
			_vala_code_node_unref0 (deleg_type);
			_vala_code_node_unref0 (d);
		}
	}
	rhs = NULL;
	_tmp14_ = FALSE;
	if (vala_local_variable_get_initializer (local) != NULL) {
		_tmp14_ = vala_code_node_get_ccodenode ((ValaCodeNode*) vala_local_variable_get_initializer (local)) != NULL;
	} else {
		_tmp14_ = FALSE;
	}
	if (_tmp14_) {
		ValaCCodeExpression* _tmp15_;
		rhs = (_tmp15_ = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_local_variable_get_initializer (local)))), _vala_ccode_node_unref0 (rhs), _tmp15_);
		if (VALA_IS_ARRAY_TYPE (vala_local_variable_get_variable_type (local))) {
			ValaArrayType* array_type;
			array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_local_variable_get_variable_type (local)));
			if (vala_array_type_get_fixed_length (array_type)) {
				ValaCCodeExpression* _tmp16_;
				rhs = (_tmp16_ = NULL, _vala_ccode_node_unref0 (rhs), _tmp16_);
			} else {
				ValaCCodeCommaExpression* ccomma;
				ValaLocalVariable* temp_var;
				ValaCCodeAssignment* _tmp18_;
				ValaCCodeExpression* _tmp17_;
				ValaCCodeExpression* _tmp31_;
				ValaCCodeExpression* _tmp32_;
				ccomma = vala_ccode_comma_expression_new ();
				temp_var = vala_ccode_base_module_get_temp_variable (self, vala_local_variable_get_variable_type (local), TRUE, (ValaCodeNode*) local, FALSE);
				gee_list_insert ((GeeList*) self->temp_vars, 0, temp_var);
				vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp18_ = vala_ccode_assignment_new (_tmp17_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
				_vala_ccode_node_unref0 (_tmp18_);
				_vala_ccode_node_unref0 (_tmp17_);
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp19_;
						_tmp19_ = TRUE;
						while (TRUE) {
							char* _tmp21_;
							char* _tmp20_;
							ValaCCodeExpression* _tmp22_;
							ValaCCodeExpression* lhs_array_len;
							ValaCCodeExpression* rhs_array_len;
							ValaCCodeAssignment* _tmp23_;
							if (!_tmp19_) {
								dim++;
							}
							_tmp19_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							lhs_array_len = (_tmp22_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp21_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp20_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), dim)), _g_free0 (_tmp21_), _g_free0 (_tmp20_), _tmp22_);
							rhs_array_len = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_local_variable_get_initializer (local), dim);
							vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_assignment_new (lhs_array_len, rhs_array_len, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							_vala_ccode_node_unref0 (_tmp23_);
							_vala_ccode_node_unref0 (lhs_array_len);
							_vala_ccode_node_unref0 (rhs_array_len);
						}
					}
				}
				if (vala_array_type_get_rank (array_type) == 1) {
					char* _tmp25_;
					char* _tmp24_;
					ValaCCodeExpression* _tmp26_;
					ValaCCodeExpression* lhs_array_size;
					char* _tmp28_;
					char* _tmp27_;
					ValaCCodeExpression* _tmp29_;
					ValaCCodeExpression* rhs_array_len;
					ValaCCodeAssignment* _tmp30_;
					lhs_array_size = (_tmp26_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp25_ = vala_ccode_module_get_array_size_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp24_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)))), _g_free0 (_tmp25_), _g_free0 (_tmp24_), _tmp26_);
					rhs_array_len = (_tmp29_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp28_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp27_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), 1)), _g_free0 (_tmp28_), _g_free0 (_tmp27_), _tmp29_);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_assignment_new (lhs_array_size, rhs_array_len, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp30_);
					_vala_ccode_node_unref0 (lhs_array_size);
					_vala_ccode_node_unref0 (rhs_array_len);
				}
				vala_ccode_comma_expression_append_expression (ccomma, _tmp31_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)));
				_vala_ccode_node_unref0 (_tmp31_);
				rhs = (_tmp32_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (rhs), _tmp32_);
				_vala_ccode_node_unref0 (ccomma);
				_vala_code_node_unref0 (temp_var);
			}
			_vala_code_node_unref0 (array_type);
		} else {
			if (VALA_IS_DELEGATE_TYPE (vala_local_variable_get_variable_type (local))) {
				ValaDelegateType* deleg_type;
				ValaDelegate* d;
				deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_local_variable_get_variable_type (local)));
				d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
				if (vala_delegate_get_has_target (d)) {
					ValaCCodeCommaExpression* ccomma;
					ValaLocalVariable* temp_var;
					ValaCCodeAssignment* _tmp34_;
					ValaCCodeExpression* _tmp33_;
					char* _tmp36_;
					char* _tmp35_;
					ValaCCodeExpression* _tmp37_;
					ValaCCodeExpression* lhs_delegate_target;
					ValaCCodeExpression* rhs_delegate_target;
					ValaCCodeAssignment* _tmp42_;
					ValaCCodeExpression* _tmp43_;
					ValaCCodeExpression* _tmp44_;
					ccomma = vala_ccode_comma_expression_new ();
					temp_var = vala_ccode_base_module_get_temp_variable (self, vala_local_variable_get_variable_type (local), TRUE, (ValaCodeNode*) local, FALSE);
					gee_list_insert ((GeeList*) self->temp_vars, 0, temp_var);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp34_ = vala_ccode_assignment_new (_tmp33_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp34_);
					_vala_ccode_node_unref0 (_tmp33_);
					lhs_delegate_target = (_tmp37_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp36_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp35_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)))), _g_free0 (_tmp36_), _g_free0 (_tmp35_), _tmp37_);
					if (vala_local_variable_get_captured (local)) {
						ValaBlock* block;
						ValaCCodeExpression* _tmp41_;
						char* _tmp40_;
						ValaCCodeExpression* _tmp39_;
						char* _tmp38_;
						block = _vala_code_node_ref0 (VALA_BLOCK (vala_symbol_get_parent_symbol ((ValaSymbol*) local)));
						lhs_delegate_target = (_tmp41_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (_tmp39_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp38_ = g_strdup_printf ("_data%d_", vala_ccode_base_module_get_block_id (self, block))), _tmp40_ = vala_ccode_base_module_get_delegate_target_cname (self, vala_symbol_get_name ((ValaSymbol*) local))), _vala_ccode_node_unref0 (lhs_delegate_target), _tmp41_);
						_g_free0 (_tmp40_);
						_vala_ccode_node_unref0 (_tmp39_);
						_g_free0 (_tmp38_);
						_vala_code_node_unref0 (block);
					}
					rhs_delegate_target = vala_ccode_base_module_get_delegate_target_cexpression (self, vala_local_variable_get_initializer (local));
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_assignment_new (lhs_delegate_target, rhs_delegate_target, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp42_);
					vala_ccode_comma_expression_append_expression (ccomma, _tmp43_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)));
					_vala_ccode_node_unref0 (_tmp43_);
					rhs = (_tmp44_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (rhs), _tmp44_);
					_vala_ccode_node_unref0 (ccomma);
					_vala_code_node_unref0 (temp_var);
					_vala_ccode_node_unref0 (lhs_delegate_target);
					_vala_ccode_node_unref0 (rhs_delegate_target);
				}
				_vala_code_node_unref0 (deleg_type);
				_vala_code_node_unref0 (d);
			}
		}
	} else {
		if (vala_data_type_is_reference_type_or_type_parameter (vala_local_variable_get_variable_type (local))) {
			ValaCCodeExpression* _tmp45_;
			rhs = (_tmp45_ = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL"), _vala_ccode_node_unref0 (rhs), _tmp45_);
			if (VALA_IS_ARRAY_TYPE (vala_local_variable_get_variable_type (local))) {
				ValaArrayType* array_type;
				array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_local_variable_get_variable_type (local)));
				if (vala_array_type_get_fixed_length (array_type)) {
					ValaCCodeExpression* _tmp46_;
					rhs = (_tmp46_ = NULL, _vala_ccode_node_unref0 (rhs), _tmp46_);
				} else {
					ValaCCodeCommaExpression* ccomma;
					ValaCCodeExpression* _tmp53_;
					ccomma = vala_ccode_comma_expression_new ();
					{
						gint dim;
						dim = 1;
						{
							gboolean _tmp47_;
							_tmp47_ = TRUE;
							while (TRUE) {
								ValaCCodeAssignment* _tmp52_;
								ValaCCodeConstant* _tmp51_;
								ValaCCodeExpression* _tmp50_;
								char* _tmp49_;
								char* _tmp48_;
								if (!_tmp47_) {
									dim++;
								}
								_tmp47_ = FALSE;
								if (!(dim <= vala_array_type_get_rank (array_type))) {
									break;
								}
								vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp52_ = vala_ccode_assignment_new (_tmp50_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp49_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp48_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), dim)), (ValaCCodeExpression*) (_tmp51_ = vala_ccode_constant_new ("0")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
								_vala_ccode_node_unref0 (_tmp52_);
								_vala_ccode_node_unref0 (_tmp51_);
								_vala_ccode_node_unref0 (_tmp50_);
								_g_free0 (_tmp49_);
								_g_free0 (_tmp48_);
							}
						}
					}
					vala_ccode_comma_expression_append_expression (ccomma, rhs);
					rhs = (_tmp53_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (rhs), _tmp53_);
					_vala_ccode_node_unref0 (ccomma);
				}
				_vala_code_node_unref0 (array_type);
			}
		}
	}
	cfrag = vala_ccode_fragment_new ();
	if (self->pre_statement_fragment != NULL) {
		ValaCCodeFragment* _tmp54_;
		vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) self->pre_statement_fragment);
		self->pre_statement_fragment = (_tmp54_ = NULL, _vala_ccode_node_unref0 (self->pre_statement_fragment), _tmp54_);
	}
	if (vala_local_variable_get_captured (local)) {
		if (vala_local_variable_get_initializer (local) != NULL) {
			ValaCCodeExpressionStatement* _tmp60_;
			ValaCCodeAssignment* _tmp59_;
			ValaCCodeMemberAccess* _tmp58_;
			char* _tmp57_;
			ValaCCodeExpression* _tmp56_;
			char* _tmp55_;
			vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp60_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp59_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp58_ = vala_ccode_member_access_new_pointer (_tmp56_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp55_ = g_strdup_printf ("_data%d_", vala_ccode_base_module_get_block_id (self, VALA_BLOCK (vala_symbol_get_parent_symbol ((ValaSymbol*) local))))), _tmp57_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)))), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp60_);
			_vala_ccode_node_unref0 (_tmp59_);
			_vala_ccode_node_unref0 (_tmp58_);
			_g_free0 (_tmp57_);
			_vala_ccode_node_unref0 (_tmp56_);
			_g_free0 (_tmp55_);
		}
	} else {
		gboolean _tmp61_;
		_tmp61_ = FALSE;
		if (vala_ccode_base_module_get_current_method (self) != NULL) {
			_tmp61_ = vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
		} else {
			_tmp61_ = FALSE;
		}
		if (_tmp61_) {
			char* _tmp65_;
			char* _tmp64_;
			char* _tmp63_;
			char* _tmp62_;
			vala_ccode_struct_add_field (self->closure_struct, _tmp62_ = vala_data_type_get_cname (vala_local_variable_get_variable_type (local)), _tmp65_ = g_strconcat (_tmp63_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), _tmp64_ = vala_data_type_get_cdeclarator_suffix (vala_local_variable_get_variable_type (local)), NULL));
			_g_free0 (_tmp65_);
			_g_free0 (_tmp64_);
			_g_free0 (_tmp63_);
			_g_free0 (_tmp62_);
			if (vala_local_variable_get_initializer (local) != NULL) {
				ValaCCodeExpressionStatement* _tmp70_;
				ValaCCodeAssignment* _tmp69_;
				ValaCCodeMemberAccess* _tmp68_;
				char* _tmp67_;
				ValaCCodeIdentifier* _tmp66_;
				vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp70_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp69_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp68_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp66_ = vala_ccode_identifier_new ("data")), _tmp67_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)))), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp70_);
				_vala_ccode_node_unref0 (_tmp69_);
				_vala_ccode_node_unref0 (_tmp68_);
				_g_free0 (_tmp67_);
				_vala_ccode_node_unref0 (_tmp66_);
			}
		} else {
			char* _tmp72_;
			char* _tmp71_;
			ValaCCodeVariableDeclarator* _tmp73_;
			ValaCCodeVariableDeclarator* cvar;
			char* _tmp74_;
			ValaCCodeDeclaration* _tmp75_;
			ValaCCodeDeclaration* _cdecl_;
			cvar = (_tmp73_ = vala_ccode_variable_declarator_new (_tmp71_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) local)), rhs, _tmp72_ = vala_data_type_get_cdeclarator_suffix (vala_local_variable_get_variable_type (local))), _g_free0 (_tmp72_), _g_free0 (_tmp71_), _tmp73_);
			_cdecl_ = (_tmp75_ = vala_ccode_declaration_new (_tmp74_ = vala_data_type_get_cname (vala_local_variable_get_variable_type (local))), _g_free0 (_tmp74_), _tmp75_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) cvar);
			vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) _cdecl_);
			if (vala_ccode_variable_declarator_get_initializer (cvar) == NULL) {
				ValaCCodeExpression* _tmp76_;
				vala_ccode_variable_declarator_set_initializer (cvar, _tmp76_ = vala_ccode_base_module_default_value_for_type (self, vala_local_variable_get_variable_type (local), TRUE));
				_vala_ccode_node_unref0 (_tmp76_);
			}
			_vala_ccode_node_unref0 (cvar);
			_vala_ccode_node_unref0 (_cdecl_);
		}
	}
	_tmp77_ = FALSE;
	if (vala_local_variable_get_initializer (local) != NULL) {
		_tmp77_ = VALA_IS_ARRAY_TYPE (vala_local_variable_get_variable_type (local));
	} else {
		_tmp77_ = FALSE;
	}
	if (_tmp77_) {
		ValaArrayType* array_type;
		array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_local_variable_get_variable_type (local)));
		if (vala_array_type_get_fixed_length (array_type)) {
			ValaCCodeIdentifier* _tmp78_;
			ValaCCodeFunctionCall* _tmp79_;
			ValaCCodeFunctionCall* sizeof_call;
			ValaCCodeIdentifier* _tmp81_;
			char* _tmp80_;
			ValaCCodeConstant* _tmp83_;
			char* _tmp82_;
			ValaCCodeBinaryExpression* _tmp84_;
			ValaCCodeBinaryExpression* size;
			ValaCCodeIdentifier* _tmp85_;
			ValaCCodeFunctionCall* _tmp86_;
			ValaCCodeFunctionCall* ccopy;
			ValaCCodeExpression* _tmp87_;
			ValaCCodeExpressionStatement* _tmp88_;
			vala_ccode_declaration_space_add_include (self->source_declarations, "string.h", FALSE);
			sizeof_call = (_tmp79_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp78_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp78_), _tmp79_);
			vala_ccode_function_call_add_argument (sizeof_call, (ValaCCodeExpression*) (_tmp81_ = vala_ccode_identifier_new (_tmp80_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
			_vala_ccode_node_unref0 (_tmp81_);
			_g_free0 (_tmp80_);
			size = (_tmp84_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, (ValaCCodeExpression*) (_tmp83_ = vala_ccode_constant_new (_tmp82_ = g_strdup_printf ("%d", vala_array_type_get_length (array_type)))), (ValaCCodeExpression*) sizeof_call), _vala_ccode_node_unref0 (_tmp83_), _g_free0 (_tmp82_), _tmp84_);
			ccopy = (_tmp86_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp85_ = vala_ccode_identifier_new ("memcpy"))), _vala_ccode_node_unref0 (_tmp85_), _tmp86_);
			vala_ccode_function_call_add_argument (ccopy, _tmp87_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) local)));
			_vala_ccode_node_unref0 (_tmp87_);
			vala_ccode_function_call_add_argument (ccopy, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_local_variable_get_initializer (local))));
			vala_ccode_function_call_add_argument (ccopy, (ValaCCodeExpression*) size);
			vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp88_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccopy)));
			_vala_ccode_node_unref0 (_tmp88_);
			_vala_ccode_node_unref0 (sizeof_call);
			_vala_ccode_node_unref0 (size);
			_vala_ccode_node_unref0 (ccopy);
		}
		_vala_code_node_unref0 (array_type);
	}
	_tmp89_ = FALSE;
	if (vala_local_variable_get_initializer (local) != NULL) {
		_tmp89_ = vala_code_node_get_tree_can_fail ((ValaCodeNode*) vala_local_variable_get_initializer (local));
	} else {
		_tmp89_ = FALSE;
	}
	if (_tmp89_) {
		vala_ccode_module_add_simple_check (vala_ccode_module_get_head ((ValaCCodeModule*) self), (ValaCodeNode*) vala_local_variable_get_initializer (local), cfrag);
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) local, (ValaCCodeNode*) cfrag);
	vala_symbol_set_active ((ValaSymbol*) local, TRUE);
	_vala_ccode_node_unref0 (rhs);
	_vala_ccode_node_unref0 (cfrag);
}


static void vala_ccode_base_module_real_visit_initializer_list (ValaCCodeModule* base, ValaInitializerList* list) {
	ValaCCodeBaseModule * self;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (list != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) list, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	if (VALA_IS_STRUCT (vala_data_type_get_data_type (vala_expression_get_target_type ((ValaExpression*) list)))) {
		ValaStruct* st;
		ValaCCodeInitializerList* clist;
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* field_it;
		st = _vala_code_node_ref0 (VALA_STRUCT (vala_data_type_get_data_type (vala_expression_get_target_type ((ValaExpression*) list))));
		clist = vala_ccode_initializer_list_new ();
		field_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		{
			GeeList* _tmp2_;
			GeeIterator* _tmp3_;
			GeeIterator* _expr_it;
			_expr_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_initializer_list_get_initializers (list))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
			while (TRUE) {
				ValaExpression* expr;
				ValaField* field;
				ValaCCodeExpression* cexpr;
				char* ctype;
				if (!gee_iterator_next (_expr_it)) {
					break;
				}
				expr = (ValaExpression*) gee_iterator_get (_expr_it);
				field = NULL;
				while (TRUE) {
					ValaField* _tmp4_;
					if (!(field == NULL)) {
						break;
					}
					gee_iterator_next (field_it);
					field = (_tmp4_ = (ValaField*) gee_iterator_get (field_it), _vala_code_node_unref0 (field), _tmp4_);
					if (vala_field_get_binding (field) != MEMBER_BINDING_INSTANCE) {
						ValaField* _tmp5_;
						field = (_tmp5_ = NULL, _vala_code_node_unref0 (field), _tmp5_);
					}
				}
				cexpr = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)));
				ctype = vala_field_get_ctype (field);
				if (ctype != NULL) {
					ValaCCodeExpression* _tmp6_;
					cexpr = (_tmp6_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (cexpr, ctype), _vala_ccode_node_unref0 (cexpr), _tmp6_);
				}
				vala_ccode_initializer_list_append (clist, cexpr);
				_vala_code_node_unref0 (expr);
				_vala_code_node_unref0 (field);
				_vala_ccode_node_unref0 (cexpr);
				_g_free0 (ctype);
			}
			_gee_collection_object_unref0 (_expr_it);
		}
		vala_code_node_set_ccodenode ((ValaCodeNode*) list, (ValaCCodeNode*) clist);
		_vala_code_node_unref0 (st);
		_vala_ccode_node_unref0 (clist);
		_gee_collection_object_unref0 (field_it);
	} else {
		ValaCCodeInitializerList* clist;
		clist = vala_ccode_initializer_list_new ();
		{
			GeeList* _tmp7_;
			GeeIterator* _tmp8_;
			GeeIterator* _expr_it;
			_expr_it = (_tmp8_ = gee_iterable_iterator ((GeeIterable*) (_tmp7_ = vala_initializer_list_get_initializers (list))), _gee_collection_object_unref0 (_tmp7_), _tmp8_);
			while (TRUE) {
				ValaExpression* expr;
				if (!gee_iterator_next (_expr_it)) {
					break;
				}
				expr = (ValaExpression*) gee_iterator_get (_expr_it);
				vala_ccode_initializer_list_append (clist, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)));
				_vala_code_node_unref0 (expr);
			}
			_gee_collection_object_unref0 (_expr_it);
		}
		vala_code_node_set_ccodenode ((ValaCodeNode*) list, (ValaCCodeNode*) clist);
		_vala_ccode_node_unref0 (clist);
	}
}


ValaLocalVariable* vala_ccode_base_module_get_temp_variable (ValaCCodeBaseModule* self, ValaDataType* type, gboolean value_owned, ValaCodeNode* node_reference, gboolean init) {
	ValaLocalVariable* result;
	ValaDataType* var_type;
	char* _tmp0_;
	ValaLocalVariable* _tmp1_;
	ValaLocalVariable* local;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	var_type = vala_data_type_copy (type);
	vala_data_type_set_value_owned (var_type, value_owned);
	local = (_tmp1_ = vala_local_variable_new (var_type, _tmp0_ = g_strdup_printf ("_tmp%d_", self->next_temp_var_id), NULL, NULL), _g_free0 (_tmp0_), _tmp1_);
	vala_local_variable_set_no_init (local, !init);
	if (node_reference != NULL) {
		vala_code_node_set_source_reference ((ValaCodeNode*) local, vala_code_node_get_source_reference (node_reference));
	}
	self->next_temp_var_id++;
	result = local;
	_vala_code_node_unref0 (var_type);
	return result;
}


static gboolean vala_ccode_base_module_is_in_generic_type (ValaCCodeBaseModule* self, ValaDataType* type) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type != NULL, FALSE);
	_tmp0_ = FALSE;
	if (VALA_IS_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_data_type_get_type_parameter (type)))) {
		gboolean _tmp1_;
		_tmp1_ = FALSE;
		if (vala_ccode_base_module_get_current_method (self) == NULL) {
			_tmp1_ = TRUE;
		} else {
			_tmp1_ = vala_method_get_binding (vala_ccode_base_module_get_current_method (self)) == MEMBER_BINDING_INSTANCE;
		}
		_tmp0_ = _tmp1_;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = TRUE;
		return result;
	} else {
		result = FALSE;
		return result;
	}
}


static ValaCCodeExpression* vala_ccode_base_module_get_type_id_expression (ValaCCodeBaseModule* self, ValaDataType* type, gboolean is_chainup) {
	ValaCCodeExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	if (VALA_IS_GENERIC_TYPE (type)) {
		char* _tmp0_;
		char* _tmp1_;
		char* var_name;
		gboolean _tmp2_;
		gboolean _tmp3_;
		var_name = (_tmp1_ = g_strdup_printf ("%s_type", _tmp0_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) vala_data_type_get_type_parameter (type)), -1)), _g_free0 (_tmp0_), _tmp1_);
		_tmp2_ = FALSE;
		_tmp3_ = FALSE;
		if (vala_ccode_base_module_is_in_generic_type (self, type)) {
			_tmp3_ = !is_chainup;
		} else {
			_tmp3_ = FALSE;
		}
		if (_tmp3_) {
			_tmp2_ = !vala_ccode_base_module_get_in_creation_method (self);
		} else {
			_tmp2_ = FALSE;
		}
		if (_tmp2_) {
			ValaCCodeMemberAccess* _tmp5_;
			ValaCCodeIdentifier* _tmp4_;
			ValaCCodeExpression* _tmp6_;
			result = (_tmp6_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp5_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("self")), "priv")), var_name), _vala_ccode_node_unref0 (_tmp5_), _vala_ccode_node_unref0 (_tmp4_), _tmp6_);
			_g_free0 (var_name);
			return result;
		} else {
			result = (ValaCCodeExpression*) vala_ccode_identifier_new (var_name);
			_g_free0 (var_name);
			return result;
		}
		_g_free0 (var_name);
	} else {
		char* type_id;
		type_id = vala_data_type_get_type_id (type);
		if (type_id == NULL) {
			char* _tmp7_;
			type_id = (_tmp7_ = g_strdup ("G_TYPE_INVALID"), _g_free0 (type_id), _tmp7_);
		} else {
			vala_ccode_base_module_generate_type_declaration (self, type, self->source_declarations);
		}
		result = (ValaCCodeExpression*) vala_ccode_identifier_new (type_id);
		_g_free0 (type_id);
		return result;
	}
}


static ValaCCodeExpression* vala_ccode_base_module_real_get_dup_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup) {
	ValaCCodeExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	if (VALA_IS_ERROR_TYPE (type)) {
		result = (ValaCCodeExpression*) vala_ccode_identifier_new ("g_error_copy");
		return result;
	} else {
		if (vala_data_type_get_data_type (type) != NULL) {
			char* dup_function;
			ValaTypeSymbol* _tmp0_;
			ValaClass* cl;
			dup_function = NULL;
			cl = _vala_code_node_ref0 ((_tmp0_ = vala_data_type_get_data_type (type), VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
			if (vala_typesymbol_is_reference_counting (vala_data_type_get_data_type (type))) {
				char* _tmp1_;
				gboolean _tmp2_;
				dup_function = (_tmp1_ = vala_typesymbol_get_ref_function (vala_data_type_get_data_type (type)), _g_free0 (dup_function), _tmp1_);
				_tmp2_ = FALSE;
				if (VALA_IS_INTERFACE (vala_data_type_get_data_type (type))) {
					_tmp2_ = dup_function == NULL;
				} else {
					_tmp2_ = FALSE;
				}
				if (_tmp2_) {
					char* _tmp4_;
					char* _tmp3_;
					vala_report_error (source_reference, _tmp4_ = g_strdup_printf ("missing class prerequisite for interface `%s', add GLib.Object to interface declaration if unsure", _tmp3_ = vala_symbol_get_full_name ((ValaSymbol*) vala_data_type_get_data_type (type))));
					_g_free0 (_tmp4_);
					_g_free0 (_tmp3_);
					result = NULL;
					_g_free0 (dup_function);
					_vala_code_node_unref0 (cl);
					return result;
				}
			} else {
				gboolean _tmp5_;
				_tmp5_ = FALSE;
				if (cl != NULL) {
					_tmp5_ = vala_class_get_is_immutable (cl);
				} else {
					_tmp5_ = FALSE;
				}
				if (_tmp5_) {
					char* _tmp6_;
					dup_function = (_tmp6_ = vala_typesymbol_get_dup_function (vala_data_type_get_data_type (type)), _g_free0 (dup_function), _tmp6_);
					if (dup_function == NULL) {
						char* _tmp7_;
						dup_function = (_tmp7_ = g_strdup (""), _g_free0 (dup_function), _tmp7_);
					}
				} else {
					if (VALA_IS_VALUE_TYPE (type)) {
						char* _tmp8_;
						gboolean _tmp9_;
						dup_function = (_tmp8_ = vala_typesymbol_get_dup_function (vala_data_type_get_data_type (type)), _g_free0 (dup_function), _tmp8_);
						_tmp9_ = FALSE;
						if (dup_function == NULL) {
							_tmp9_ = vala_data_type_get_nullable (type);
						} else {
							_tmp9_ = FALSE;
						}
						if (_tmp9_) {
							char* _tmp10_;
							dup_function = (_tmp10_ = vala_ccode_base_module_generate_struct_dup_wrapper (self, VALA_VALUE_TYPE (type)), _g_free0 (dup_function), _tmp10_);
						} else {
							if (dup_function == NULL) {
								char* _tmp11_;
								dup_function = (_tmp11_ = g_strdup (""), _g_free0 (dup_function), _tmp11_);
							}
						}
					} else {
						char* _tmp12_;
						vala_report_error (source_reference, _tmp12_ = g_strdup_printf ("duplicating %s instance, use unowned variable or explicitly invoke copy method", vala_symbol_get_name ((ValaSymbol*) vala_data_type_get_data_type (type))));
						_g_free0 (_tmp12_);
						result = NULL;
						_g_free0 (dup_function);
						_vala_code_node_unref0 (cl);
						return result;
					}
				}
			}
			result = (ValaCCodeExpression*) vala_ccode_identifier_new (dup_function);
			_g_free0 (dup_function);
			_vala_code_node_unref0 (cl);
			return result;
		} else {
			if (vala_data_type_get_type_parameter (type) != NULL) {
				char* _tmp13_;
				char* _tmp14_;
				char* func_name;
				gboolean _tmp15_;
				gboolean _tmp16_;
				func_name = (_tmp14_ = g_strdup_printf ("%s_dup_func", _tmp13_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) vala_data_type_get_type_parameter (type)), -1)), _g_free0 (_tmp13_), _tmp14_);
				_tmp15_ = FALSE;
				_tmp16_ = FALSE;
				if (vala_ccode_base_module_is_in_generic_type (self, type)) {
					_tmp16_ = !is_chainup;
				} else {
					_tmp16_ = FALSE;
				}
				if (_tmp16_) {
					_tmp15_ = !vala_ccode_base_module_get_in_creation_method (self);
				} else {
					_tmp15_ = FALSE;
				}
				if (_tmp15_) {
					ValaCCodeMemberAccess* _tmp18_;
					ValaCCodeIdentifier* _tmp17_;
					ValaCCodeExpression* _tmp19_;
					result = (_tmp19_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("self")), "priv")), func_name), _vala_ccode_node_unref0 (_tmp18_), _vala_ccode_node_unref0 (_tmp17_), _tmp19_);
					_g_free0 (func_name);
					return result;
				} else {
					result = (ValaCCodeExpression*) vala_ccode_identifier_new (func_name);
					_g_free0 (func_name);
					return result;
				}
				_g_free0 (func_name);
			} else {
				if (VALA_IS_POINTER_TYPE (type)) {
					ValaPointerType* pointer_type;
					pointer_type = _vala_code_node_ref0 (VALA_POINTER_TYPE (type));
					result = vala_ccode_base_module_get_dup_func_expression (self, vala_pointer_type_get_base_type (pointer_type), source_reference, FALSE);
					_vala_code_node_unref0 (pointer_type);
					return result;
				} else {
					result = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL");
					return result;
				}
			}
		}
	}
}


ValaCCodeExpression* vala_ccode_base_module_get_dup_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->get_dup_func_expression (self, type, source_reference, is_chainup);
}


static char* vala_ccode_base_module_generate_struct_dup_wrapper (ValaCCodeBaseModule* self, ValaValueType* value_type) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* dup_func;
	char* _tmp2_;
	ValaCCodeFunction* _tmp3_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp5_;
	char* _tmp4_;
	ValaCCodeBlock* block;
	ValaCCodeFunction* _tmp41_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (value_type != NULL, NULL);
	dup_func = (_tmp1_ = g_strdup_printf ("_%sdup", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) vala_value_type_get_type_symbol (value_type))), _g_free0 (_tmp0_), _tmp1_);
	if (!vala_ccode_base_module_add_wrapper (self, dup_func)) {
		result = dup_func;
		return result;
	}
	function = (_tmp3_ = vala_ccode_function_new (dup_func, _tmp2_ = vala_data_type_get_cname ((ValaDataType*) value_type)), _g_free0 (_tmp2_), _tmp3_);
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("self", _tmp4_ = vala_data_type_get_cname ((ValaDataType*) value_type)));
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	block = vala_ccode_block_new ();
	if (vala_value_type_get_type_symbol (value_type) == VALA_TYPESYMBOL (self->gvalue_type)) {
		ValaCCodeIdentifier* _tmp6_;
		ValaCCodeFunctionCall* _tmp7_;
		ValaCCodeFunctionCall* dup_call;
		ValaCCodeIdentifier* _tmp8_;
		ValaCCodeIdentifier* _tmp9_;
		ValaCCodeReturnStatement* _tmp10_;
		dup_call = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("g_boxed_copy"))), _vala_ccode_node_unref0 (_tmp6_), _tmp7_);
		vala_ccode_function_call_add_argument (dup_call, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("G_TYPE_VALUE")));
		_vala_ccode_node_unref0 (_tmp8_);
		vala_ccode_function_call_add_argument (dup_call, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("self")));
		_vala_ccode_node_unref0 (_tmp9_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp10_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) dup_call)));
		_vala_ccode_node_unref0 (_tmp10_);
		_vala_ccode_node_unref0 (dup_call);
	} else {
		char* _tmp11_;
		ValaCCodeDeclaration* _tmp12_;
		ValaCCodeDeclaration* _cdecl_;
		ValaCCodeVariableDeclarator* _tmp13_;
		ValaCCodeIdentifier* _tmp14_;
		ValaCCodeFunctionCall* _tmp15_;
		ValaCCodeFunctionCall* creation_call;
		ValaCCodeConstant* _tmp17_;
		char* _tmp16_;
		ValaCCodeConstant* _tmp18_;
		ValaCCodeExpressionStatement* _tmp21_;
		ValaCCodeAssignment* _tmp20_;
		ValaCCodeIdentifier* _tmp19_;
		ValaTypeSymbol* _tmp22_;
		ValaStruct* st;
		gboolean _tmp23_;
		ValaCCodeReturnStatement* _tmp40_;
		ValaCCodeIdentifier* _tmp39_;
		_cdecl_ = (_tmp12_ = vala_ccode_declaration_new (_tmp11_ = vala_data_type_get_cname ((ValaDataType*) value_type)), _g_free0 (_tmp11_), _tmp12_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp13_ = vala_ccode_variable_declarator_new ("dup", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp13_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
		creation_call = (_tmp15_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("g_new0"))), _vala_ccode_node_unref0 (_tmp14_), _tmp15_);
		vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_constant_new (_tmp16_ = vala_typesymbol_get_cname (vala_data_type_get_data_type ((ValaDataType*) value_type), FALSE))));
		_vala_ccode_node_unref0 (_tmp17_);
		_g_free0 (_tmp16_);
		vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp18_ = vala_ccode_constant_new ("1")));
		_vala_ccode_node_unref0 (_tmp18_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp21_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("dup")), (ValaCCodeExpression*) creation_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp21_);
		_vala_ccode_node_unref0 (_tmp20_);
		_vala_ccode_node_unref0 (_tmp19_);
		st = _vala_code_node_ref0 ((_tmp22_ = vala_data_type_get_data_type ((ValaDataType*) value_type), VALA_IS_STRUCT (_tmp22_) ? ((ValaStruct*) _tmp22_) : NULL));
		_tmp23_ = FALSE;
		if (st != NULL) {
			_tmp23_ = vala_struct_is_disposable (st);
		} else {
			_tmp23_ = FALSE;
		}
		if (_tmp23_) {
			ValaCCodeIdentifier* _tmp25_;
			char* _tmp24_;
			ValaCCodeFunctionCall* _tmp26_;
			ValaCCodeFunctionCall* copy_call;
			ValaCCodeIdentifier* _tmp27_;
			ValaCCodeIdentifier* _tmp28_;
			ValaCCodeExpressionStatement* _tmp29_;
			if (!vala_struct_get_has_copy_function (st)) {
				vala_ccode_base_module_generate_struct_copy_function (self, st);
			}
			copy_call = (_tmp26_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp25_ = vala_ccode_identifier_new (_tmp24_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st)))), _vala_ccode_node_unref0 (_tmp25_), _g_free0 (_tmp24_), _tmp26_);
			vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp27_);
			vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("dup")));
			_vala_ccode_node_unref0 (_tmp28_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp29_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) copy_call)));
			_vala_ccode_node_unref0 (_tmp29_);
			_vala_ccode_node_unref0 (copy_call);
		} else {
			ValaCCodeIdentifier* _tmp30_;
			ValaCCodeFunctionCall* _tmp31_;
			ValaCCodeFunctionCall* sizeof_call;
			ValaCCodeConstant* _tmp33_;
			char* _tmp32_;
			ValaCCodeIdentifier* _tmp34_;
			ValaCCodeFunctionCall* _tmp35_;
			ValaCCodeFunctionCall* copy_call;
			ValaCCodeIdentifier* _tmp36_;
			ValaCCodeIdentifier* _tmp37_;
			ValaCCodeExpressionStatement* _tmp38_;
			vala_ccode_declaration_space_add_include (self->source_declarations, "string.h", FALSE);
			sizeof_call = (_tmp31_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp30_), _tmp31_);
			vala_ccode_function_call_add_argument (sizeof_call, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_constant_new (_tmp32_ = vala_typesymbol_get_cname (vala_data_type_get_data_type ((ValaDataType*) value_type), FALSE))));
			_vala_ccode_node_unref0 (_tmp33_);
			_g_free0 (_tmp32_);
			copy_call = (_tmp35_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("memcpy"))), _vala_ccode_node_unref0 (_tmp34_), _tmp35_);
			vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_identifier_new ("dup")));
			_vala_ccode_node_unref0 (_tmp36_);
			vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp37_);
			vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) sizeof_call);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp38_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) copy_call)));
			_vala_ccode_node_unref0 (_tmp38_);
			_vala_ccode_node_unref0 (sizeof_call);
			_vala_ccode_node_unref0 (copy_call);
		}
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp40_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new ("dup")))));
		_vala_ccode_node_unref0 (_tmp40_);
		_vala_ccode_node_unref0 (_tmp39_);
		_vala_ccode_node_unref0 (_cdecl_);
		_vala_ccode_node_unref0 (creation_call);
		_vala_code_node_unref0 (st);
	}
	vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp41_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp41_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) function);
	result = dup_func;
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	return result;
}


char* vala_ccode_base_module_generate_destroy_func_wrapper (ValaCCodeBaseModule* self, ValaDataType* type) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* destroy_func;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp3_;
	char* _tmp2_;
	ValaCCodeBlock* block;
	ValaCCodeIdentifier* _tmp5_;
	char* _tmp4_;
	ValaCCodeFunctionCall* _tmp6_;
	ValaCCodeFunctionCall* free_call;
	ValaCCodeUnaryExpression* _tmp8_;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeExpressionStatement* _tmp9_;
	ValaCCodeFunction* _tmp10_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	destroy_func = (_tmp1_ = g_strdup_printf ("_vala_%s_free", _tmp0_ = vala_typesymbol_get_cname (vala_data_type_get_data_type (type), FALSE)), _g_free0 (_tmp0_), _tmp1_);
	if (!vala_ccode_base_module_add_wrapper (self, destroy_func)) {
		result = destroy_func;
		return result;
	}
	function = vala_ccode_function_new (destroy_func, "void");
	vala_ccode_function_add_parameter (function, _tmp3_ = vala_ccode_formal_parameter_new ("self", _tmp2_ = vala_data_type_get_cname (type)));
	_vala_ccode_node_unref0 (_tmp3_);
	_g_free0 (_tmp2_);
	block = vala_ccode_block_new ();
	free_call = (_tmp6_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new (_tmp4_ = vala_typesymbol_get_free_function (vala_data_type_get_data_type (type))))), _vala_ccode_node_unref0 (_tmp5_), _g_free0 (_tmp4_), _tmp6_);
	vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("self")))));
	_vala_ccode_node_unref0 (_tmp8_);
	_vala_ccode_node_unref0 (_tmp7_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp9_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp10_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) function);
	result = destroy_func;
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (free_call);
	return result;
}


ValaCCodeExpression* vala_ccode_base_module_get_destroy_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, gboolean is_chainup) {
	ValaCCodeExpression* result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	_tmp0_ = FALSE;
	if (vala_code_context_get_profile (self->priv->_context) == VALA_PROFILE_GOBJECT) {
		gboolean _tmp1_;
		_tmp1_ = FALSE;
		if (vala_data_type_get_data_type (type) == VALA_TYPESYMBOL (self->glist_type)) {
			_tmp1_ = TRUE;
		} else {
			_tmp1_ = vala_data_type_get_data_type (type) == VALA_TYPESYMBOL (self->gslist_type);
		}
		_tmp0_ = _tmp1_;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gboolean elements_require_free;
		ValaCCodeExpression* element_destroy_func_expression;
		gboolean _tmp5_;
		elements_require_free = FALSE;
		element_destroy_func_expression = NULL;
		{
			GeeList* _tmp2_;
			GeeIterator* _tmp3_;
			GeeIterator* _type_arg_it;
			_type_arg_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_data_type_get_type_arguments (type))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
			while (TRUE) {
				ValaDataType* type_arg;
				if (!gee_iterator_next (_type_arg_it)) {
					break;
				}
				type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
				elements_require_free = vala_ccode_base_module_requires_destroy (self, type_arg);
				if (elements_require_free) {
					ValaCCodeExpression* _tmp4_;
					element_destroy_func_expression = (_tmp4_ = vala_ccode_base_module_get_destroy_func_expression (self, type_arg, FALSE), _vala_ccode_node_unref0 (element_destroy_func_expression), _tmp4_);
				}
				_vala_code_node_unref0 (type_arg);
			}
			_gee_collection_object_unref0 (_type_arg_it);
		}
		_tmp5_ = FALSE;
		if (elements_require_free) {
			_tmp5_ = VALA_IS_CCODE_IDENTIFIER (element_destroy_func_expression);
		} else {
			_tmp5_ = FALSE;
		}
		if (_tmp5_) {
			char* _tmp6_;
			ValaCCodeExpression* _tmp7_;
			result = (_tmp7_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp6_ = vala_ccode_base_module_generate_glist_free_wrapper (self, type, VALA_CCODE_IDENTIFIER (element_destroy_func_expression))), _g_free0 (_tmp6_), _tmp7_);
			_vala_ccode_node_unref0 (element_destroy_func_expression);
			return result;
		} else {
			char* _tmp8_;
			ValaCCodeExpression* _tmp9_;
			result = (_tmp9_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp8_ = vala_typesymbol_get_free_function (vala_data_type_get_data_type (type))), _g_free0 (_tmp8_), _tmp9_);
			_vala_ccode_node_unref0 (element_destroy_func_expression);
			return result;
		}
		_vala_ccode_node_unref0 (element_destroy_func_expression);
	} else {
		if (VALA_IS_ERROR_TYPE (type)) {
			result = (ValaCCodeExpression*) vala_ccode_identifier_new ("g_error_free");
			return result;
		} else {
			if (vala_data_type_get_data_type (type) != NULL) {
				char* unref_function;
				unref_function = NULL;
				if (VALA_IS_REFERENCE_TYPE (type)) {
					if (vala_typesymbol_is_reference_counting (vala_data_type_get_data_type (type))) {
						char* _tmp10_;
						gboolean _tmp11_;
						unref_function = (_tmp10_ = vala_typesymbol_get_unref_function (vala_data_type_get_data_type (type)), _g_free0 (unref_function), _tmp10_);
						_tmp11_ = FALSE;
						if (VALA_IS_INTERFACE (vala_data_type_get_data_type (type))) {
							_tmp11_ = unref_function == NULL;
						} else {
							_tmp11_ = FALSE;
						}
						if (_tmp11_) {
							char* _tmp13_;
							char* _tmp12_;
							vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) type), _tmp13_ = g_strdup_printf ("missing class prerequisite for interface `%s', add GLib.Object to interface declaration if unsure", _tmp12_ = vala_symbol_get_full_name ((ValaSymbol*) vala_data_type_get_data_type (type))));
							_g_free0 (_tmp13_);
							_g_free0 (_tmp12_);
							result = NULL;
							_g_free0 (unref_function);
							return result;
						}
					} else {
						ValaTypeSymbol* _tmp14_;
						ValaClass* cl;
						gboolean _tmp15_;
						cl = _vala_code_node_ref0 ((_tmp14_ = vala_data_type_get_data_type (type), VALA_IS_CLASS (_tmp14_) ? ((ValaClass*) _tmp14_) : NULL));
						_tmp15_ = FALSE;
						if (cl != NULL) {
							_tmp15_ = vala_class_get_free_function_address_of (cl);
						} else {
							_tmp15_ = FALSE;
						}
						if (_tmp15_) {
							char* _tmp16_;
							unref_function = (_tmp16_ = vala_ccode_base_module_generate_destroy_func_wrapper (self, type), _g_free0 (unref_function), _tmp16_);
						} else {
							char* _tmp17_;
							unref_function = (_tmp17_ = vala_typesymbol_get_free_function (vala_data_type_get_data_type (type)), _g_free0 (unref_function), _tmp17_);
						}
						_vala_code_node_unref0 (cl);
					}
				} else {
					if (vala_data_type_get_nullable (type)) {
						char* _tmp18_;
						unref_function = (_tmp18_ = vala_typesymbol_get_free_function (vala_data_type_get_data_type (type)), _g_free0 (unref_function), _tmp18_);
						if (unref_function == NULL) {
							char* _tmp19_;
							unref_function = (_tmp19_ = g_strdup ("g_free"), _g_free0 (unref_function), _tmp19_);
						}
					} else {
						ValaStruct* st;
						char* _tmp20_;
						st = _vala_code_node_ref0 (VALA_STRUCT (vala_data_type_get_data_type (type)));
						if (!vala_struct_get_has_destroy_function (st)) {
							vala_ccode_base_module_generate_struct_destroy_function (self, st);
						}
						unref_function = (_tmp20_ = vala_typesymbol_get_destroy_function ((ValaTypeSymbol*) st), _g_free0 (unref_function), _tmp20_);
						_vala_code_node_unref0 (st);
					}
				}
				if (unref_function == NULL) {
					result = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL");
					_g_free0 (unref_function);
					return result;
				}
				result = (ValaCCodeExpression*) vala_ccode_identifier_new (unref_function);
				_g_free0 (unref_function);
				return result;
			} else {
				gboolean _tmp21_;
				_tmp21_ = FALSE;
				if (vala_data_type_get_type_parameter (type) != NULL) {
					_tmp21_ = VALA_IS_CLASS (vala_ccode_base_module_get_current_type_symbol (self));
				} else {
					_tmp21_ = FALSE;
				}
				if (_tmp21_) {
					char* _tmp22_;
					char* _tmp23_;
					char* func_name;
					gboolean _tmp24_;
					gboolean _tmp25_;
					func_name = (_tmp23_ = g_strdup_printf ("%s_destroy_func", _tmp22_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) vala_data_type_get_type_parameter (type)), -1)), _g_free0 (_tmp22_), _tmp23_);
					_tmp24_ = FALSE;
					_tmp25_ = FALSE;
					if (vala_ccode_base_module_is_in_generic_type (self, type)) {
						_tmp25_ = !is_chainup;
					} else {
						_tmp25_ = FALSE;
					}
					if (_tmp25_) {
						_tmp24_ = !vala_ccode_base_module_get_in_creation_method (self);
					} else {
						_tmp24_ = FALSE;
					}
					if (_tmp24_) {
						ValaCCodeMemberAccess* _tmp27_;
						ValaCCodeIdentifier* _tmp26_;
						ValaCCodeExpression* _tmp28_;
						result = (_tmp28_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp27_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("self")), "priv")), func_name), _vala_ccode_node_unref0 (_tmp27_), _vala_ccode_node_unref0 (_tmp26_), _tmp28_);
						_g_free0 (func_name);
						return result;
					} else {
						result = (ValaCCodeExpression*) vala_ccode_identifier_new (func_name);
						_g_free0 (func_name);
						return result;
					}
					_g_free0 (func_name);
				} else {
					if (VALA_IS_ARRAY_TYPE (type)) {
						result = (ValaCCodeExpression*) vala_ccode_identifier_new ("g_free");
						return result;
					} else {
						if (VALA_IS_POINTER_TYPE (type)) {
							result = (ValaCCodeExpression*) vala_ccode_identifier_new ("g_free");
							return result;
						} else {
							result = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL");
							return result;
						}
					}
				}
			}
		}
	}
}


static char* vala_ccode_base_module_generate_glist_free_wrapper (ValaCCodeBaseModule* self, ValaDataType* list_type, ValaCCodeIdentifier* element_destroy_func_expression) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* destroy_func;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp3_;
	char* _tmp2_;
	ValaCCodeBlock* block;
	ValaCCodeFunctionCall* element_free_call;
	ValaCCodeIdentifier* _tmp8_;
	ValaCCodeCastExpression* _tmp9_;
	ValaCCodeConstant* _tmp10_;
	ValaCCodeExpressionStatement* _tmp11_;
	ValaCCodeIdentifier* _tmp13_;
	char* _tmp12_;
	ValaCCodeFunctionCall* _tmp14_;
	ValaCCodeFunctionCall* cfreecall;
	ValaCCodeIdentifier* _tmp15_;
	ValaCCodeExpressionStatement* _tmp16_;
	ValaCCodeFunction* _tmp17_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (list_type != NULL, NULL);
	g_return_val_if_fail (element_destroy_func_expression != NULL, NULL);
	destroy_func = (_tmp1_ = g_strdup_printf ("_%s_%s", _tmp0_ = vala_typesymbol_get_free_function (vala_data_type_get_data_type (list_type)), vala_ccode_identifier_get_name (element_destroy_func_expression)), _g_free0 (_tmp0_), _tmp1_);
	if (!vala_ccode_base_module_add_wrapper (self, destroy_func)) {
		result = destroy_func;
		return result;
	}
	function = vala_ccode_function_new (destroy_func, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp3_ = vala_ccode_formal_parameter_new ("self", _tmp2_ = vala_data_type_get_cname (list_type)));
	_vala_ccode_node_unref0 (_tmp3_);
	_g_free0 (_tmp2_);
	block = vala_ccode_block_new ();
	element_free_call = NULL;
	if (vala_data_type_get_data_type (list_type) == VALA_TYPESYMBOL (self->glist_type)) {
		ValaCCodeFunctionCall* _tmp5_;
		ValaCCodeIdentifier* _tmp4_;
		element_free_call = (_tmp5_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("g_list_foreach"))), _vala_ccode_node_unref0 (element_free_call), _tmp5_);
		_vala_ccode_node_unref0 (_tmp4_);
	} else {
		ValaCCodeFunctionCall* _tmp7_;
		ValaCCodeIdentifier* _tmp6_;
		element_free_call = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("g_slist_foreach"))), _vala_ccode_node_unref0 (element_free_call), _tmp7_);
		_vala_ccode_node_unref0 (_tmp6_);
	}
	vala_ccode_function_call_add_argument (element_free_call, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp8_);
	vala_ccode_function_call_add_argument (element_free_call, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) element_destroy_func_expression, "GFunc")));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_function_call_add_argument (element_free_call, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp11_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) element_free_call)));
	_vala_ccode_node_unref0 (_tmp11_);
	cfreecall = (_tmp14_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new (_tmp12_ = vala_typesymbol_get_free_function (vala_data_type_get_data_type (list_type))))), _vala_ccode_node_unref0 (_tmp13_), _g_free0 (_tmp12_), _tmp14_);
	vala_ccode_function_call_add_argument (cfreecall, (ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp15_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp16_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cfreecall)));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp17_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp17_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) function);
	result = destroy_func;
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (element_free_call);
	_vala_ccode_node_unref0 (cfreecall);
	return result;
}


static char* vala_ccode_base_module_real_append_struct_array_free (ValaCCodeBaseModule* self, ValaStruct* st) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (st != NULL, NULL);
	result = NULL;
	return result;
}


char* vala_ccode_base_module_append_struct_array_free (ValaCCodeBaseModule* self, ValaStruct* st) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->append_struct_array_free (self, st);
}


static ValaCCodeExpression* vala_ccode_base_module_real_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition) {
	ValaCCodeExpression* result;
	ValaCCodeExpression* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* ccall;
	gboolean _tmp2_;
	gboolean _tmp11_;
	gboolean _tmp12_;
	ValaCCodeConstant* _tmp19_;
	ValaCCodeBinaryExpression* _tmp20_;
	ValaCCodeBinaryExpression* cisnull;
	ValaCCodeCommaExpression* ccomma;
	gboolean _tmp26_;
	ValaCCodeConstant* _tmp42_;
	ValaCCodeAssignment* cassign;
	gboolean _tmp43_;
	gboolean _tmp44_;
	gboolean uses_gfree;
	gboolean _tmp46_;
	ValaCCodeConstant* _tmp47_;
	ValaCCodeExpression* _tmp48_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (cvar != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	g_return_val_if_fail (expr != NULL, NULL);
	ccall = (_tmp1_ = vala_ccode_function_call_new (_tmp0_ = vala_ccode_base_module_get_destroy_func_expression (self, type, FALSE)), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	_tmp2_ = FALSE;
	if (VALA_IS_VALUE_TYPE (type)) {
		_tmp2_ = !vala_data_type_get_nullable (type);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		ValaCCodeUnaryExpression* _tmp3_;
		gboolean _tmp4_;
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, cvar)));
		_vala_ccode_node_unref0 (_tmp3_);
		_tmp4_ = FALSE;
		if (self->gvalue_type != NULL) {
			_tmp4_ = vala_data_type_get_data_type (type) == VALA_TYPESYMBOL (self->gvalue_type);
		} else {
			_tmp4_ = FALSE;
		}
		if (_tmp4_) {
			ValaCCodeIdentifier* _tmp5_;
			ValaCCodeFunctionCall* _tmp6_;
			ValaCCodeFunctionCall* cisvalid;
			ValaCCodeUnaryExpression* _tmp7_;
			ValaCCodeCommaExpression* ccomma;
			ValaCCodeConstant* _tmp8_;
			ValaCCodeConstant* _tmp9_;
			ValaCCodeExpression* _tmp10_;
			cisvalid = (_tmp6_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new ("G_IS_VALUE"))), _vala_ccode_node_unref0 (_tmp5_), _tmp6_);
			vala_ccode_function_call_add_argument (cisvalid, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, cvar)));
			_vala_ccode_node_unref0 (_tmp7_);
			ccomma = vala_ccode_comma_expression_new ();
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_constant_new ("NULL")));
			_vala_ccode_node_unref0 (_tmp8_);
			result = (_tmp10_ = (ValaCCodeExpression*) vala_ccode_conditional_expression_new ((ValaCCodeExpression*) cisvalid, (ValaCCodeExpression*) ccomma, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_constant_new ("NULL"))), _vala_ccode_node_unref0 (_tmp9_), _tmp10_);
			_vala_ccode_node_unref0 (cisvalid);
			_vala_ccode_node_unref0 (ccomma);
			_vala_ccode_node_unref0 (ccall);
			return result;
			_vala_ccode_node_unref0 (cisvalid);
			_vala_ccode_node_unref0 (ccomma);
		} else {
			result = (ValaCCodeExpression*) ccall;
			return result;
		}
	}
	_tmp11_ = FALSE;
	_tmp12_ = FALSE;
	if (VALA_IS_CCODE_IDENTIFIER (vala_ccode_function_call_get_call (ccall))) {
		_tmp12_ = !VALA_IS_ARRAY_TYPE (type);
	} else {
		_tmp12_ = FALSE;
	}
	if (_tmp12_) {
		_tmp11_ = !is_macro_definition;
	} else {
		_tmp11_ = FALSE;
	}
	if (_tmp11_) {
		ValaCCodeIdentifier* freeid;
		char* free0_func;
		ValaCCodeFunctionCall* _tmp18_;
		ValaCCodeIdentifier* _tmp17_;
		freeid = _vala_ccode_node_ref0 (VALA_CCODE_IDENTIFIER (vala_ccode_function_call_get_call (ccall)));
		free0_func = g_strdup_printf ("_%s0", vala_ccode_identifier_get_name (freeid));
		if (vala_ccode_base_module_add_wrapper (self, free0_func)) {
			ValaCCodeIdentifier* _tmp13_;
			ValaCCodeExpression* _tmp14_;
			ValaCCodeExpression* macro;
			ValaCCodeMacroReplacement* _tmp16_;
			char* _tmp15_;
			macro = (_tmp14_ = vala_ccode_base_module_get_unref_expression (self, (ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("var")), type, expr, TRUE), _vala_ccode_node_unref0 (_tmp13_), _tmp14_);
			vala_ccode_declaration_space_add_type_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp16_ = vala_ccode_macro_replacement_new_with_expression (_tmp15_ = g_strdup_printf ("%s(var)", free0_func), macro)));
			_vala_ccode_node_unref0 (_tmp16_);
			_g_free0 (_tmp15_);
			_vala_ccode_node_unref0 (macro);
		}
		ccall = (_tmp18_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new (free0_func))), _vala_ccode_node_unref0 (ccall), _tmp18_);
		_vala_ccode_node_unref0 (_tmp17_);
		vala_ccode_function_call_add_argument (ccall, cvar);
		result = (ValaCCodeExpression*) ccall;
		_vala_ccode_node_unref0 (freeid);
		_g_free0 (free0_func);
		return result;
	}
	cisnull = (_tmp20_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, cvar, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_constant_new ("NULL"))), _vala_ccode_node_unref0 (_tmp19_), _tmp20_);
	if (vala_data_type_get_type_parameter (type) != NULL) {
		gboolean _tmp21_;
		ValaCCodeConstant* _tmp23_;
		ValaCCodeExpression* _tmp22_;
		ValaCCodeBinaryExpression* _tmp24_;
		ValaCCodeBinaryExpression* cunrefisnull;
		ValaCCodeBinaryExpression* _tmp25_;
		_tmp21_ = FALSE;
		if (!VALA_IS_CLASS (vala_ccode_base_module_get_current_type_symbol (self))) {
			_tmp21_ = TRUE;
		} else {
			_tmp21_ = vala_class_get_is_compact (vala_ccode_base_module_get_current_class (self));
		}
		if (_tmp21_) {
			result = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL");
			_vala_ccode_node_unref0 (ccall);
			_vala_ccode_node_unref0 (cisnull);
			return result;
		}
		cunrefisnull = (_tmp24_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, _tmp22_ = vala_ccode_base_module_get_destroy_func_expression (self, type, FALSE), (ValaCCodeExpression*) (_tmp23_ = vala_ccode_constant_new ("NULL"))), _vala_ccode_node_unref0 (_tmp23_), _vala_ccode_node_unref0 (_tmp22_), _tmp24_);
		cisnull = (_tmp25_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_OR, (ValaCCodeExpression*) cisnull, (ValaCCodeExpression*) cunrefisnull), _vala_ccode_node_unref0 (cisnull), _tmp25_);
		_vala_ccode_node_unref0 (cunrefisnull);
	}
	vala_ccode_function_call_add_argument (ccall, cvar);
	ccomma = vala_ccode_comma_expression_new ();
	_tmp26_ = FALSE;
	if (vala_code_context_get_profile (self->priv->_context) == VALA_PROFILE_GOBJECT) {
		gboolean _tmp27_;
		gboolean _tmp28_;
		gboolean _tmp29_;
		_tmp27_ = FALSE;
		_tmp28_ = FALSE;
		_tmp29_ = FALSE;
		if (vala_data_type_get_data_type (type) == self->gstringbuilder_type) {
			_tmp29_ = TRUE;
		} else {
			_tmp29_ = vala_data_type_get_data_type (type) == self->garray_type;
		}
		if (_tmp29_) {
			_tmp28_ = TRUE;
		} else {
			_tmp28_ = vala_data_type_get_data_type (type) == self->gbytearray_type;
		}
		if (_tmp28_) {
			_tmp27_ = TRUE;
		} else {
			_tmp27_ = vala_data_type_get_data_type (type) == self->gptrarray_type;
		}
		_tmp26_ = _tmp27_;
	} else {
		_tmp26_ = FALSE;
	}
	if (_tmp26_) {
		ValaCCodeConstant* _tmp30_;
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_constant_new ("TRUE")));
		_vala_ccode_node_unref0 (_tmp30_);
	} else {
		if (VALA_IS_ARRAY_TYPE (type)) {
			ValaArrayType* array_type;
			array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (type));
			if (vala_ccode_base_module_requires_destroy (self, vala_array_type_get_element_type (array_type))) {
				ValaCCodeExpression* csizeexpr;
				gboolean first;
				ValaTypeSymbol* _tmp35_;
				ValaStruct* st;
				gboolean _tmp36_;
				csizeexpr = NULL;
				first = TRUE;
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp31_;
						_tmp31_ = TRUE;
						while (TRUE) {
							if (!_tmp31_) {
								dim++;
							}
							_tmp31_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							if (first) {
								ValaCCodeExpression* _tmp32_;
								csizeexpr = (_tmp32_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), expr, dim), _vala_ccode_node_unref0 (csizeexpr), _tmp32_);
								first = FALSE;
							} else {
								ValaCCodeExpression* _tmp34_;
								ValaCCodeExpression* _tmp33_;
								csizeexpr = (_tmp34_ = (ValaCCodeExpression*) vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, csizeexpr, _tmp33_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), expr, dim)), _vala_ccode_node_unref0 (csizeexpr), _tmp34_);
								_vala_ccode_node_unref0 (_tmp33_);
							}
						}
					}
				}
				st = _vala_code_node_ref0 ((_tmp35_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)), VALA_IS_STRUCT (_tmp35_) ? ((ValaStruct*) _tmp35_) : NULL));
				_tmp36_ = FALSE;
				if (st != NULL) {
					_tmp36_ = !vala_data_type_get_nullable (vala_array_type_get_element_type (array_type));
				} else {
					_tmp36_ = FALSE;
				}
				if (_tmp36_) {
					ValaCCodeIdentifier* _tmp38_;
					char* _tmp37_;
					vala_ccode_function_call_set_call (ccall, (ValaCCodeExpression*) (_tmp38_ = vala_ccode_identifier_new (_tmp37_ = vala_ccode_base_module_append_struct_array_free (self, st))));
					_vala_ccode_node_unref0 (_tmp38_);
					_g_free0 (_tmp37_);
					vala_ccode_function_call_add_argument (ccall, csizeexpr);
				} else {
					ValaCCodeIdentifier* _tmp39_;
					ValaCCodeCastExpression* _tmp41_;
					ValaCCodeExpression* _tmp40_;
					self->requires_array_free = TRUE;
					vala_ccode_function_call_set_call (ccall, (ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new ("_vala_array_free")));
					_vala_ccode_node_unref0 (_tmp39_);
					vala_ccode_function_call_add_argument (ccall, csizeexpr);
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp41_ = vala_ccode_cast_expression_new (_tmp40_ = vala_ccode_base_module_get_destroy_func_expression (self, vala_array_type_get_element_type (array_type), FALSE), "GDestroyNotify")));
					_vala_ccode_node_unref0 (_tmp41_);
					_vala_ccode_node_unref0 (_tmp40_);
				}
				_vala_ccode_node_unref0 (csizeexpr);
				_vala_code_node_unref0 (st);
			}
			_vala_code_node_unref0 (array_type);
		}
	}
	vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
	vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp42_);
	cassign = vala_ccode_assignment_new (cvar, (ValaCCodeExpression*) ccomma, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
	_tmp43_ = FALSE;
	_tmp44_ = FALSE;
	if (vala_data_type_get_data_type (type) != NULL) {
		_tmp44_ = !vala_typesymbol_is_reference_counting (vala_data_type_get_data_type (type));
	} else {
		_tmp44_ = FALSE;
	}
	if (_tmp44_) {
		char* _tmp45_;
		_tmp43_ = _vala_strcmp0 (_tmp45_ = vala_typesymbol_get_free_function (vala_data_type_get_data_type (type)), "g_free") == 0;
		_g_free0 (_tmp45_);
	} else {
		_tmp43_ = FALSE;
	}
	uses_gfree = _tmp43_;
	_tmp46_ = FALSE;
	if (uses_gfree) {
		_tmp46_ = TRUE;
	} else {
		_tmp46_ = VALA_IS_ARRAY_TYPE (type);
	}
	uses_gfree = _tmp46_;
	if (uses_gfree) {
		result = (ValaCCodeExpression*) cassign;
		_vala_ccode_node_unref0 (ccall);
		_vala_ccode_node_unref0 (cisnull);
		_vala_ccode_node_unref0 (ccomma);
		return result;
	}
	result = (_tmp48_ = (ValaCCodeExpression*) vala_ccode_conditional_expression_new ((ValaCCodeExpression*) cisnull, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_constant_new ("NULL")), (ValaCCodeExpression*) cassign), _vala_ccode_node_unref0 (_tmp47_), _tmp48_);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (cisnull);
	_vala_ccode_node_unref0 (ccomma);
	_vala_ccode_node_unref0 (cassign);
	return result;
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (cisnull);
	_vala_ccode_node_unref0 (ccomma);
	_vala_ccode_node_unref0 (cassign);
}


ValaCCodeExpression* vala_ccode_base_module_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->get_unref_expression (self, cvar, type, expr, is_macro_definition);
}


static void vala_ccode_base_module_real_visit_end_full_expression (ValaCCodeModule* base, ValaExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaDataType* expr_type;
	ValaLocalVariable* full_expr_var;
	ValaCCodeCommaExpression* expr_list;
	ValaCCodeAssignment* _tmp2_;
	ValaCCodeExpression* _tmp1_;
	ValaCCodeExpression* _tmp5_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	gee_collection_clear ((GeeCollection*) expr->temp_vars);
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) self->temp_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			gee_collection_add ((GeeCollection*) expr->temp_vars, local);
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	gee_collection_clear ((GeeCollection*) self->temp_vars);
	if (gee_collection_get_size ((GeeCollection*) GEE_LIST (self->temp_ref_vars)) == 0) {
		return;
	}
	expr_type = _vala_code_node_ref0 (vala_expression_get_value_type (expr));
	if (vala_expression_get_target_type (expr) != NULL) {
		ValaDataType* _tmp0_;
		expr_type = (_tmp0_ = _vala_code_node_ref0 (vala_expression_get_target_type (expr)), _vala_code_node_unref0 (expr_type), _tmp0_);
	}
	full_expr_var = vala_ccode_base_module_get_temp_variable (self, expr_type, TRUE, (ValaCodeNode*) expr, FALSE);
	gee_collection_add ((GeeCollection*) expr->temp_vars, full_expr_var);
	expr_list = vala_ccode_comma_expression_new ();
	vala_ccode_comma_expression_append_expression (expr_list, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_assignment_new (_tmp1_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) full_expr_var)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
	_vala_ccode_node_unref0 (_tmp2_);
	_vala_ccode_node_unref0 (_tmp1_);
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) self->temp_ref_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			ValaMemberAccess* ma;
			ValaCCodeExpression* _tmp4_;
			ValaCCodeExpression* _tmp3_;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) local), NULL);
			vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
			vala_ccode_comma_expression_append_expression (expr_list, _tmp4_ = vala_ccode_base_module_get_unref_expression (self, _tmp3_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) local)), vala_local_variable_get_variable_type (local), (ValaExpression*) ma, FALSE));
			_vala_ccode_node_unref0 (_tmp4_);
			_vala_ccode_node_unref0 (_tmp3_);
			_vala_code_node_unref0 (local);
			_vala_code_node_unref0 (ma);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	vala_ccode_comma_expression_append_expression (expr_list, _tmp5_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) full_expr_var)));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) expr_list);
	gee_collection_clear ((GeeCollection*) self->temp_ref_vars);
	_vala_code_node_unref0 (expr_type);
	_vala_code_node_unref0 (full_expr_var);
	_vala_ccode_node_unref0 (expr_list);
}


void vala_ccode_base_module_append_temp_decl (ValaCCodeBaseModule* self, ValaCCodeFragment* cfrag, GeeList* temp_vars) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (cfrag != NULL);
	g_return_if_fail (temp_vars != NULL);
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) temp_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			gboolean _tmp0_;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			_tmp0_ = FALSE;
			if (vala_ccode_base_module_get_current_method (self) != NULL) {
				_tmp0_ = vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
			} else {
				_tmp0_ = FALSE;
			}
			if (_tmp0_) {
				char* _tmp1_;
				vala_ccode_struct_add_field (self->closure_struct, _tmp1_ = vala_data_type_get_cname (vala_local_variable_get_variable_type (local)), vala_symbol_get_name ((ValaSymbol*) local));
				_g_free0 (_tmp1_);
			} else {
				char* _tmp2_;
				ValaCCodeDeclaration* _tmp3_;
				ValaCCodeDeclaration* _cdecl_;
				char* _tmp4_;
				ValaCCodeVariableDeclarator* _tmp5_;
				ValaCCodeVariableDeclarator* vardecl;
				ValaTypeSymbol* _tmp6_;
				ValaStruct* st;
				ValaDataType* _tmp7_;
				ValaArrayType* array_type;
				_cdecl_ = (_tmp3_ = vala_ccode_declaration_new (_tmp2_ = vala_data_type_get_cname (vala_local_variable_get_variable_type (local))), _g_free0 (_tmp2_), _tmp3_);
				vardecl = (_tmp5_ = vala_ccode_variable_declarator_new (vala_symbol_get_name ((ValaSymbol*) local), NULL, _tmp4_ = vala_data_type_get_cdeclarator_suffix (vala_local_variable_get_variable_type (local))), _g_free0 (_tmp4_), _tmp5_);
				vala_code_node_set_ccodenode ((ValaCodeNode*) local, (ValaCCodeNode*) vardecl);
				vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) vardecl);
				st = _vala_code_node_ref0 ((_tmp6_ = vala_data_type_get_data_type (vala_local_variable_get_variable_type (local)), VALA_IS_STRUCT (_tmp6_) ? ((ValaStruct*) _tmp6_) : NULL));
				array_type = _vala_code_node_ref0 ((_tmp7_ = vala_local_variable_get_variable_type (local), VALA_IS_ARRAY_TYPE (_tmp7_) ? ((ValaArrayType*) _tmp7_) : NULL));
				if (g_str_has_prefix (vala_symbol_get_name ((ValaSymbol*) local), "*")) {
				} else {
					if (vala_local_variable_get_no_init (local)) {
					} else {
						gboolean _tmp8_;
						gboolean _tmp9_;
						_tmp8_ = FALSE;
						_tmp9_ = FALSE;
						if (!vala_data_type_get_nullable (vala_local_variable_get_variable_type (local))) {
							gboolean _tmp10_;
							_tmp10_ = FALSE;
							if (st != NULL) {
								_tmp10_ = !vala_struct_is_simple_type (st);
							} else {
								_tmp10_ = FALSE;
							}
							_tmp9_ = _tmp10_;
						} else {
							_tmp9_ = FALSE;
						}
						if (_tmp9_) {
							_tmp8_ = TRUE;
						} else {
							gboolean _tmp11_;
							_tmp11_ = FALSE;
							if (array_type != NULL) {
								_tmp11_ = vala_array_type_get_fixed_length (array_type);
							} else {
								_tmp11_ = FALSE;
							}
							_tmp8_ = _tmp11_;
						}
						if (_tmp8_) {
							ValaCCodeInitializerList* clist;
							ValaCCodeConstant* _tmp12_;
							clist = vala_ccode_initializer_list_new ();
							vala_ccode_initializer_list_append (clist, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_constant_new ("0")));
							_vala_ccode_node_unref0 (_tmp12_);
							vala_ccode_variable_declarator_set_initializer (vardecl, (ValaCCodeExpression*) clist);
							_vala_ccode_node_unref0 (clist);
						} else {
							gboolean _tmp13_;
							_tmp13_ = FALSE;
							if (vala_data_type_is_reference_type_or_type_parameter (vala_local_variable_get_variable_type (local))) {
								_tmp13_ = TRUE;
							} else {
								_tmp13_ = vala_data_type_get_nullable (vala_local_variable_get_variable_type (local));
							}
							if (_tmp13_) {
								ValaCCodeConstant* _tmp14_;
								vala_ccode_variable_declarator_set_initializer (vardecl, (ValaCCodeExpression*) (_tmp14_ = vala_ccode_constant_new ("NULL")));
								_vala_ccode_node_unref0 (_tmp14_);
							}
						}
					}
				}
				vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) _cdecl_);
				_vala_ccode_node_unref0 (_cdecl_);
				_vala_ccode_node_unref0 (vardecl);
				_vala_code_node_unref0 (st);
				_vala_code_node_unref0 (array_type);
			}
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
}


static void vala_ccode_base_module_real_visit_expression_statement (ValaCCodeModule* base, ValaExpressionStatement* stmt) {
	ValaCCodeBaseModule * self;
	ValaCCodeExpressionStatement* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	ValaCCodeFragment* cfrag;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (stmt != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) stmt, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	if (vala_code_node_get_error ((ValaCodeNode*) vala_expression_statement_get_expression (stmt))) {
		vala_code_node_set_error ((ValaCodeNode*) stmt, TRUE);
		return;
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) (_tmp0_ = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_expression_statement_get_expression (stmt))))));
	_vala_ccode_node_unref0 (_tmp0_);
	_tmp1_ = FALSE;
	if (vala_code_node_get_tree_can_fail ((ValaCodeNode*) stmt)) {
		_tmp1_ = vala_code_node_get_tree_can_fail ((ValaCodeNode*) vala_expression_statement_get_expression (stmt));
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		ValaCCodeFragment* cfrag;
		cfrag = vala_ccode_fragment_new ();
		vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode ((ValaCodeNode*) stmt));
		vala_ccode_module_add_simple_check (vala_ccode_module_get_head ((ValaCCodeModule*) self), (ValaCodeNode*) vala_expression_statement_get_expression (stmt), cfrag);
		vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) cfrag);
		_vala_ccode_node_unref0 (cfrag);
	}
	_tmp2_ = FALSE;
	if (gee_collection_get_size ((GeeCollection*) GEE_LIST (self->temp_vars)) == 0) {
		_tmp2_ = self->pre_statement_fragment == NULL;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		return;
	}
	cfrag = vala_ccode_fragment_new ();
	vala_ccode_base_module_append_temp_decl (self, cfrag, (GeeList*) self->temp_vars);
	if (self->pre_statement_fragment != NULL) {
		ValaCCodeFragment* _tmp3_;
		vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) self->pre_statement_fragment);
		self->pre_statement_fragment = (_tmp3_ = NULL, _vala_ccode_node_unref0 (self->pre_statement_fragment), _tmp3_);
	}
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode ((ValaCodeNode*) stmt));
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) self->temp_ref_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			ValaMemberAccess* ma;
			ValaCCodeExpressionStatement* _tmp6_;
			ValaCCodeExpression* _tmp5_;
			ValaCCodeExpression* _tmp4_;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) local), NULL);
			vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
			vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp6_ = vala_ccode_expression_statement_new (_tmp5_ = vala_ccode_base_module_get_unref_expression (self, _tmp4_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) local)), vala_local_variable_get_variable_type (local), (ValaExpression*) ma, FALSE))));
			_vala_ccode_node_unref0 (_tmp6_);
			_vala_ccode_node_unref0 (_tmp5_);
			_vala_ccode_node_unref0 (_tmp4_);
			_vala_code_node_unref0 (local);
			_vala_code_node_unref0 (ma);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) cfrag);
	gee_collection_clear ((GeeCollection*) self->temp_vars);
	gee_collection_clear ((GeeCollection*) self->temp_ref_vars);
	_vala_ccode_node_unref0 (cfrag);
}


void vala_ccode_base_module_create_temp_decl (ValaCCodeBaseModule* self, ValaStatement* stmt, GeeList* temp_vars) {
	ValaCCodeFragment* cfrag;
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	g_return_if_fail (temp_vars != NULL);
	if (gee_collection_get_size ((GeeCollection*) temp_vars) == 0) {
		return;
	}
	cfrag = vala_ccode_fragment_new ();
	vala_ccode_base_module_append_temp_decl (self, cfrag, temp_vars);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode ((ValaCodeNode*) stmt));
	vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) cfrag);
	_vala_ccode_node_unref0 (cfrag);
}


static void vala_ccode_base_module_real_append_local_free (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop) {
	ValaBlock* b;
	GeeList* local_vars;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sym != NULL);
	g_return_if_fail (cfrag != NULL);
	b = _vala_code_node_ref0 (VALA_BLOCK (sym));
	local_vars = vala_block_get_local_variables (b);
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) local_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			gboolean _tmp0_;
			gboolean _tmp1_;
			gboolean _tmp2_;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			_tmp0_ = FALSE;
			_tmp1_ = FALSE;
			_tmp2_ = FALSE;
			if (vala_symbol_get_active ((ValaSymbol*) local)) {
				_tmp2_ = !vala_local_variable_get_floating (local);
			} else {
				_tmp2_ = FALSE;
			}
			if (_tmp2_) {
				_tmp1_ = !vala_local_variable_get_captured (local);
			} else {
				_tmp1_ = FALSE;
			}
			if (_tmp1_) {
				_tmp0_ = vala_ccode_base_module_requires_destroy (self, vala_local_variable_get_variable_type (local));
			} else {
				_tmp0_ = FALSE;
			}
			if (_tmp0_) {
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp5_;
				ValaCCodeExpression* _tmp4_;
				ValaCCodeExpression* _tmp3_;
				ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) local), NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
				vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp5_ = vala_ccode_expression_statement_new (_tmp4_ = vala_ccode_base_module_get_unref_expression (self, _tmp3_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) local)), vala_local_variable_get_variable_type (local), (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp5_);
				_vala_ccode_node_unref0 (_tmp4_);
				_vala_ccode_node_unref0 (_tmp3_);
				_vala_code_node_unref0 (ma);
			}
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	if (vala_block_get_captured (b)) {
		gint block_id;
		ValaCCodeIdentifier* _tmp7_;
		char* _tmp6_;
		ValaCCodeFunctionCall* _tmp8_;
		ValaCCodeFunctionCall* data_unref;
		ValaCCodeExpression* _tmp10_;
		char* _tmp9_;
		ValaCCodeExpressionStatement* _tmp11_;
		block_id = vala_ccode_base_module_get_block_id (self, b);
		data_unref = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = g_strdup_printf ("block%d_data_unref", block_id)))), _vala_ccode_node_unref0 (_tmp7_), _g_free0 (_tmp6_), _tmp8_);
		vala_ccode_function_call_add_argument (data_unref, _tmp10_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp9_ = g_strdup_printf ("_data%d_", block_id)));
		_vala_ccode_node_unref0 (_tmp10_);
		_g_free0 (_tmp9_);
		vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp11_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) data_unref)));
		_vala_ccode_node_unref0 (_tmp11_);
		_vala_ccode_node_unref0 (data_unref);
	}
	if (stop_at_loop) {
		gboolean _tmp12_;
		gboolean _tmp13_;
		_tmp12_ = FALSE;
		_tmp13_ = FALSE;
		if (VALA_IS_LOOP (vala_code_node_get_parent_node ((ValaCodeNode*) b))) {
			_tmp13_ = TRUE;
		} else {
			_tmp13_ = VALA_IS_FOREACH_STATEMENT (vala_code_node_get_parent_node ((ValaCodeNode*) b));
		}
		if (_tmp13_) {
			_tmp12_ = TRUE;
		} else {
			_tmp12_ = VALA_IS_SWITCH_STATEMENT (vala_code_node_get_parent_node ((ValaCodeNode*) b));
		}
		if (_tmp12_) {
			_vala_code_node_unref0 (b);
			_gee_collection_object_unref0 (local_vars);
			return;
		}
	}
	if (VALA_IS_BLOCK (vala_symbol_get_parent_symbol (sym))) {
		vala_ccode_base_module_append_local_free (self, vala_symbol_get_parent_symbol (sym), cfrag, stop_at_loop);
	} else {
		if (VALA_IS_METHOD (vala_symbol_get_parent_symbol (sym))) {
			vala_ccode_base_module_append_param_free (self, VALA_METHOD (vala_symbol_get_parent_symbol (sym)), cfrag);
		}
	}
	_vala_code_node_unref0 (b);
	_gee_collection_object_unref0 (local_vars);
}


void vala_ccode_base_module_append_local_free (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->append_local_free (self, sym, cfrag, stop_at_loop);
}


void vala_ccode_base_module_append_error_free (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, ValaTryStatement* current_try) {
	ValaBlock* b;
	GeeList* local_vars;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sym != NULL);
	g_return_if_fail (cfrag != NULL);
	g_return_if_fail (current_try != NULL);
	b = _vala_code_node_ref0 (VALA_BLOCK (sym));
	local_vars = vala_block_get_local_variables (b);
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) local_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			gboolean _tmp0_;
			gboolean _tmp1_;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			_tmp0_ = FALSE;
			_tmp1_ = FALSE;
			if (vala_symbol_get_active ((ValaSymbol*) local)) {
				_tmp1_ = !vala_local_variable_get_floating (local);
			} else {
				_tmp1_ = FALSE;
			}
			if (_tmp1_) {
				_tmp0_ = vala_ccode_base_module_requires_destroy (self, vala_local_variable_get_variable_type (local));
			} else {
				_tmp0_ = FALSE;
			}
			if (_tmp0_) {
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp4_;
				ValaCCodeExpression* _tmp3_;
				ValaCCodeExpression* _tmp2_;
				ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) local), NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
				vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp4_ = vala_ccode_expression_statement_new (_tmp3_ = vala_ccode_base_module_get_unref_expression (self, _tmp2_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) local)), vala_local_variable_get_variable_type (local), (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp4_);
				_vala_ccode_node_unref0 (_tmp3_);
				_vala_ccode_node_unref0 (_tmp2_);
				_vala_code_node_unref0 (ma);
			}
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	if (sym == VALA_SYMBOL (vala_try_statement_get_body (current_try))) {
		_vala_code_node_unref0 (b);
		_gee_collection_object_unref0 (local_vars);
		return;
	}
	if (VALA_IS_BLOCK (vala_symbol_get_parent_symbol (sym))) {
		vala_ccode_base_module_append_error_free (self, vala_symbol_get_parent_symbol (sym), cfrag, current_try);
	} else {
		if (VALA_IS_METHOD (vala_symbol_get_parent_symbol (sym))) {
			vala_ccode_base_module_append_param_free (self, VALA_METHOD (vala_symbol_get_parent_symbol (sym)), cfrag);
		}
	}
	_vala_code_node_unref0 (b);
	_gee_collection_object_unref0 (local_vars);
}


static void vala_ccode_base_module_append_param_free (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeFragment* cfrag) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (m != NULL);
	g_return_if_fail (cfrag != NULL);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _param_it;
		_param_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_method_get_parameters (m))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaFormalParameter* param;
			gboolean _tmp2_;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			_tmp2_ = FALSE;
			if (vala_ccode_base_module_requires_destroy (self, vala_formal_parameter_get_parameter_type (param))) {
				_tmp2_ = vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN;
			} else {
				_tmp2_ = FALSE;
			}
			if (_tmp2_) {
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp5_;
				ValaCCodeExpression* _tmp4_;
				ValaCCodeExpression* _tmp3_;
				ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) param), NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) param);
				vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp5_ = vala_ccode_expression_statement_new (_tmp4_ = vala_ccode_base_module_get_unref_expression (self, _tmp3_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) param)), vala_formal_parameter_get_parameter_type (param), (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp5_);
				_vala_ccode_node_unref0 (_tmp4_);
				_vala_ccode_node_unref0 (_tmp3_);
				_vala_code_node_unref0 (ma);
			}
			_vala_code_node_unref0 (param);
		}
		_gee_collection_object_unref0 (_param_it);
	}
}


void vala_ccode_base_module_create_local_free (ValaCCodeBaseModule* self, ValaCodeNode* stmt, gboolean stop_at_loop) {
	ValaCCodeFragment* cfrag;
	g_return_if_fail (self != NULL);
	g_return_if_fail (stmt != NULL);
	cfrag = vala_ccode_fragment_new ();
	vala_ccode_base_module_append_local_free (self, self->current_symbol, cfrag, stop_at_loop);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (stmt));
	vala_code_node_set_ccodenode (stmt, (ValaCCodeNode*) cfrag);
	_vala_ccode_node_unref0 (cfrag);
}


static gboolean vala_ccode_base_module_real_variable_accessible_in_finally (ValaCCodeBaseModule* self, ValaLocalVariable* local) {
	gboolean result;
	ValaSymbol* sym;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (local != NULL, FALSE);
	if (self->current_try == NULL) {
		result = FALSE;
		return result;
	}
	sym = _vala_code_node_ref0 (self->current_symbol);
	while (TRUE) {
		gboolean _tmp0_;
		gboolean _tmp2_;
		gboolean _tmp3_;
		ValaSymbol* _tmp5_;
		_tmp0_ = FALSE;
		if (!VALA_IS_METHOD (sym)) {
			ValaSymbol* _tmp1_;
			_tmp0_ = (_tmp1_ = vala_scope_lookup (vala_symbol_get_scope (sym), vala_symbol_get_name ((ValaSymbol*) local))) == NULL;
			_vala_code_node_unref0 (_tmp1_);
		} else {
			_tmp0_ = FALSE;
		}
		if (!_tmp0_) {
			break;
		}
		_tmp2_ = FALSE;
		_tmp3_ = FALSE;
		if (VALA_IS_TRY_STATEMENT (vala_code_node_get_parent_node ((ValaCodeNode*) sym))) {
			_tmp3_ = vala_try_statement_get_finally_body (VALA_TRY_STATEMENT (vala_code_node_get_parent_node ((ValaCodeNode*) sym))) != NULL;
		} else {
			_tmp3_ = FALSE;
		}
		if (_tmp3_) {
			_tmp2_ = TRUE;
		} else {
			gboolean _tmp4_;
			_tmp4_ = FALSE;
			if (VALA_IS_CATCH_CLAUSE (vala_code_node_get_parent_node ((ValaCodeNode*) sym))) {
				_tmp4_ = vala_try_statement_get_finally_body (VALA_TRY_STATEMENT (vala_code_node_get_parent_node (vala_code_node_get_parent_node ((ValaCodeNode*) sym)))) != NULL;
			} else {
				_tmp4_ = FALSE;
			}
			_tmp2_ = _tmp4_;
		}
		if (_tmp2_) {
			result = TRUE;
			_vala_code_node_unref0 (sym);
			return result;
		}
		sym = (_tmp5_ = _vala_code_node_ref0 (vala_symbol_get_parent_symbol (sym)), _vala_code_node_unref0 (sym), _tmp5_);
	}
	result = FALSE;
	_vala_code_node_unref0 (sym);
	return result;
}


gboolean vala_ccode_base_module_variable_accessible_in_finally (ValaCCodeBaseModule* self, ValaLocalVariable* local) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->variable_accessible_in_finally (self, local);
}


static void vala_ccode_base_module_real_visit_return_statement (ValaCCodeModule* base, ValaReturnStatement* stmt) {
	ValaCCodeBaseModule * self;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_return_statement_get_return_expression (stmt) != NULL) {
		ValaSymbol* _tmp0_;
		ValaLocalVariable* local;
		gboolean _tmp1_;
		gboolean _tmp2_;
		gboolean _tmp3_;
		local = _vala_code_node_ref0 ((_tmp0_ = vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)), VALA_IS_LOCAL_VARIABLE (_tmp0_) ? ((ValaLocalVariable*) _tmp0_) : NULL));
		_tmp1_ = FALSE;
		_tmp2_ = FALSE;
		_tmp3_ = FALSE;
		if (vala_data_type_get_value_owned (vala_ccode_base_module_get_current_return_type (self))) {
			_tmp3_ = local != NULL;
		} else {
			_tmp3_ = FALSE;
		}
		if (_tmp3_) {
			_tmp2_ = vala_data_type_get_value_owned (vala_local_variable_get_variable_type (local));
		} else {
			_tmp2_ = FALSE;
		}
		if (_tmp2_) {
			_tmp1_ = !vala_ccode_base_module_variable_accessible_in_finally (self, local);
		} else {
			_tmp1_ = FALSE;
		}
		if (_tmp1_) {
			vala_data_type_set_value_owned (vala_expression_get_value_type (vala_return_statement_get_return_expression (stmt)), TRUE);
		}
		_vala_code_node_unref0 (local);
	}
	vala_code_node_accept_children ((ValaCodeNode*) stmt, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	if (vala_return_statement_get_return_expression (stmt) == NULL) {
		ValaCCodeFragment* cfrag;
		ValaCCodeReturnStatement* _tmp7_;
		cfrag = vala_ccode_fragment_new ();
		vala_ccode_base_module_append_local_free (self, self->current_symbol, cfrag, FALSE);
		if (vala_ccode_base_module_get_current_method (self) != NULL) {
			{
				GeeList* _tmp4_;
				GeeIterator* _tmp5_;
				GeeIterator* _postcondition_it;
				_postcondition_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vala_method_get_postconditions (vala_ccode_base_module_get_current_method (self)))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
				while (TRUE) {
					ValaExpression* postcondition;
					ValaCCodeStatement* _tmp6_;
					if (!gee_iterator_next (_postcondition_it)) {
						break;
					}
					postcondition = (ValaExpression*) gee_iterator_get (_postcondition_it);
					vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp6_ = vala_ccode_base_module_create_postcondition_statement (self, postcondition)));
					_vala_ccode_node_unref0 (_tmp6_);
					_vala_code_node_unref0 (postcondition);
				}
				_gee_collection_object_unref0 (_postcondition_it);
			}
		}
		vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp7_ = vala_ccode_return_statement_new (NULL)));
		_vala_ccode_node_unref0 (_tmp7_);
		vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) cfrag);
		_vala_ccode_node_unref0 (cfrag);
	} else {
		ValaSymbol* return_expression_symbol;
		ValaSymbol* _tmp8_;
		ValaLocalVariable* local;
		gboolean _tmp9_;
		gboolean _tmp10_;
		gboolean _tmp11_;
		gboolean _tmp13_;
		gboolean _tmp14_;
		gboolean _tmp15_;
		ValaCCodeFragment* cfrag;
		ValaCCodeExpression* result_lhs;
		gboolean _tmp35_;
		ValaCCodeExpressionStatement* _tmp39_;
		ValaCCodeAssignment* _tmp38_;
		gboolean _tmp43_;
		return_expression_symbol = NULL;
		local = _vala_code_node_ref0 ((_tmp8_ = vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)), VALA_IS_LOCAL_VARIABLE (_tmp8_) ? ((ValaLocalVariable*) _tmp8_) : NULL));
		_tmp9_ = FALSE;
		_tmp10_ = FALSE;
		_tmp11_ = FALSE;
		if (vala_data_type_get_value_owned (vala_ccode_base_module_get_current_return_type (self))) {
			_tmp11_ = local != NULL;
		} else {
			_tmp11_ = FALSE;
		}
		if (_tmp11_) {
			_tmp10_ = vala_data_type_get_value_owned (vala_local_variable_get_variable_type (local));
		} else {
			_tmp10_ = FALSE;
		}
		if (_tmp10_) {
			_tmp9_ = !vala_ccode_base_module_variable_accessible_in_finally (self, local);
		} else {
			_tmp9_ = FALSE;
		}
		if (_tmp9_) {
			ValaSymbol* _tmp12_;
			return_expression_symbol = (_tmp12_ = _vala_code_node_ref0 ((ValaSymbol*) local), _vala_code_node_unref0 (return_expression_symbol), _tmp12_);
			vala_symbol_set_active (return_expression_symbol, FALSE);
		}
		_tmp13_ = FALSE;
		_tmp14_ = FALSE;
		_tmp15_ = FALSE;
		if (vala_ccode_base_module_get_current_method (self) != NULL) {
			_tmp15_ = !vala_method_get_no_array_length (vala_ccode_base_module_get_current_method (self));
		} else {
			_tmp15_ = FALSE;
		}
		if (_tmp15_) {
			_tmp14_ = TRUE;
		} else {
			_tmp14_ = vala_ccode_base_module_get_current_property_accessor (self) != NULL;
		}
		if (_tmp14_) {
			_tmp13_ = VALA_IS_ARRAY_TYPE (vala_ccode_base_module_get_current_return_type (self));
		} else {
			_tmp13_ = FALSE;
		}
		if (_tmp13_) {
			ValaLocalVariable* return_expr_decl;
			ValaCCodeCommaExpression* ccomma;
			ValaCCodeAssignment* _tmp17_;
			ValaCCodeExpression* _tmp16_;
			ValaArrayType* array_type;
			ValaCCodeExpression* _tmp24_;
			return_expr_decl = vala_ccode_base_module_get_temp_variable (self, vala_expression_get_value_type (vala_return_statement_get_return_expression (stmt)), TRUE, (ValaCodeNode*) stmt, FALSE);
			ccomma = vala_ccode_comma_expression_new ();
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_assignment_new (_tmp16_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) return_expr_decl)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_return_statement_get_return_expression (stmt))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
			_vala_ccode_node_unref0 (_tmp17_);
			_vala_ccode_node_unref0 (_tmp16_);
			array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_ccode_base_module_get_current_return_type (self)));
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp18_;
					_tmp18_ = TRUE;
					while (TRUE) {
						char* _tmp19_;
						ValaCCodeExpression* _tmp20_;
						ValaCCodeExpression* len_l;
						gboolean _tmp21_;
						ValaCCodeExpression* len_r;
						ValaCCodeAssignment* _tmp23_;
						if (!_tmp18_) {
							dim++;
						}
						_tmp18_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						len_l = (_tmp20_ = vala_ccode_base_module_get_result_cexpression (self, _tmp19_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), "result", dim)), _g_free0 (_tmp19_), _tmp20_);
						_tmp21_ = FALSE;
						if (vala_ccode_base_module_get_current_method (self) == NULL) {
							_tmp21_ = TRUE;
						} else {
							_tmp21_ = !vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
						}
						if (_tmp21_) {
							ValaCCodeExpression* _tmp22_;
							len_l = (_tmp22_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, len_l), _vala_ccode_node_unref0 (len_l), _tmp22_);
						}
						len_r = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_return_statement_get_return_expression (stmt), dim);
						vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_assignment_new (len_l, len_r, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
						_vala_ccode_node_unref0 (_tmp23_);
						_vala_ccode_node_unref0 (len_l);
						_vala_ccode_node_unref0 (len_r);
					}
				}
			}
			vala_ccode_comma_expression_append_expression (ccomma, _tmp24_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) return_expr_decl)));
			_vala_ccode_node_unref0 (_tmp24_);
			vala_code_node_set_ccodenode ((ValaCodeNode*) vala_return_statement_get_return_expression (stmt), (ValaCCodeNode*) ccomma);
			gee_collection_add ((GeeCollection*) vala_return_statement_get_return_expression (stmt)->temp_vars, return_expr_decl);
			_vala_code_node_unref0 (return_expr_decl);
			_vala_ccode_node_unref0 (ccomma);
			_vala_code_node_unref0 (array_type);
		} else {
			gboolean _tmp25_;
			gboolean _tmp26_;
			_tmp25_ = FALSE;
			_tmp26_ = FALSE;
			if (vala_ccode_base_module_get_current_method (self) != NULL) {
				_tmp26_ = TRUE;
			} else {
				_tmp26_ = vala_ccode_base_module_get_current_property_accessor (self) != NULL;
			}
			if (_tmp26_) {
				_tmp25_ = VALA_IS_DELEGATE_TYPE (vala_ccode_base_module_get_current_return_type (self));
			} else {
				_tmp25_ = FALSE;
			}
			if (_tmp25_) {
				ValaDelegateType* delegate_type;
				delegate_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_ccode_base_module_get_current_return_type (self)));
				if (vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type))) {
					ValaLocalVariable* return_expr_decl;
					ValaCCodeCommaExpression* ccomma;
					ValaCCodeAssignment* _tmp28_;
					ValaCCodeExpression* _tmp27_;
					char* _tmp29_;
					ValaCCodeExpression* _tmp30_;
					ValaCCodeExpression* target_l;
					gboolean _tmp31_;
					ValaCCodeExpression* target_r;
					ValaCCodeAssignment* _tmp33_;
					ValaCCodeExpression* _tmp34_;
					return_expr_decl = vala_ccode_base_module_get_temp_variable (self, vala_expression_get_value_type (vala_return_statement_get_return_expression (stmt)), TRUE, (ValaCodeNode*) stmt, FALSE);
					ccomma = vala_ccode_comma_expression_new ();
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_assignment_new (_tmp27_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) return_expr_decl)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_return_statement_get_return_expression (stmt))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp28_);
					_vala_ccode_node_unref0 (_tmp27_);
					target_l = (_tmp30_ = vala_ccode_base_module_get_result_cexpression (self, _tmp29_ = vala_ccode_base_module_get_delegate_target_cname (self, "result")), _g_free0 (_tmp29_), _tmp30_);
					_tmp31_ = FALSE;
					if (vala_ccode_base_module_get_current_method (self) == NULL) {
						_tmp31_ = TRUE;
					} else {
						_tmp31_ = !vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
					}
					if (_tmp31_) {
						ValaCCodeExpression* _tmp32_;
						target_l = (_tmp32_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, target_l), _vala_ccode_node_unref0 (target_l), _tmp32_);
					}
					target_r = vala_ccode_base_module_get_delegate_target_cexpression (self, vala_return_statement_get_return_expression (stmt));
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_assignment_new (target_l, target_r, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp33_);
					vala_ccode_comma_expression_append_expression (ccomma, _tmp34_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) return_expr_decl)));
					_vala_ccode_node_unref0 (_tmp34_);
					vala_code_node_set_ccodenode ((ValaCodeNode*) vala_return_statement_get_return_expression (stmt), (ValaCCodeNode*) ccomma);
					gee_collection_add ((GeeCollection*) vala_return_statement_get_return_expression (stmt)->temp_vars, return_expr_decl);
					_vala_code_node_unref0 (return_expr_decl);
					_vala_ccode_node_unref0 (ccomma);
					_vala_ccode_node_unref0 (target_l);
					_vala_ccode_node_unref0 (target_r);
				}
				_vala_code_node_unref0 (delegate_type);
			}
		}
		cfrag = vala_ccode_fragment_new ();
		result_lhs = vala_ccode_base_module_get_result_cexpression (self, "result");
		_tmp35_ = FALSE;
		if (vala_data_type_is_real_non_null_struct_type (vala_ccode_base_module_get_current_return_type (self))) {
			gboolean _tmp36_;
			_tmp36_ = FALSE;
			if (vala_ccode_base_module_get_current_method (self) == NULL) {
				_tmp36_ = TRUE;
			} else {
				_tmp36_ = !vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
			}
			_tmp35_ = _tmp36_;
		} else {
			_tmp35_ = FALSE;
		}
		if (_tmp35_) {
			ValaCCodeExpression* _tmp37_;
			result_lhs = (_tmp37_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, result_lhs), _vala_ccode_node_unref0 (result_lhs), _tmp37_);
		}
		vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp39_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp38_ = vala_ccode_assignment_new (result_lhs, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_return_statement_get_return_expression (stmt))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp39_);
		_vala_ccode_node_unref0 (_tmp38_);
		vala_ccode_base_module_append_local_free (self, self->current_symbol, cfrag, FALSE);
		if (vala_ccode_base_module_get_current_method (self) != NULL) {
			{
				GeeList* _tmp40_;
				GeeIterator* _tmp41_;
				GeeIterator* _postcondition_it;
				_postcondition_it = (_tmp41_ = gee_iterable_iterator ((GeeIterable*) (_tmp40_ = vala_method_get_postconditions (vala_ccode_base_module_get_current_method (self)))), _gee_collection_object_unref0 (_tmp40_), _tmp41_);
				while (TRUE) {
					ValaExpression* postcondition;
					ValaCCodeStatement* _tmp42_;
					if (!gee_iterator_next (_postcondition_it)) {
						break;
					}
					postcondition = (ValaExpression*) gee_iterator_get (_postcondition_it);
					vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp42_ = vala_ccode_base_module_create_postcondition_statement (self, postcondition)));
					_vala_ccode_node_unref0 (_tmp42_);
					_vala_code_node_unref0 (postcondition);
				}
				_gee_collection_object_unref0 (_postcondition_it);
			}
		}
		_tmp43_ = FALSE;
		if (vala_ccode_base_module_get_current_method (self) == NULL) {
			_tmp43_ = TRUE;
		} else {
			_tmp43_ = !vala_method_get_coroutine (vala_ccode_base_module_get_current_method (self));
		}
		if (_tmp43_) {
			if (vala_data_type_is_real_non_null_struct_type (vala_ccode_base_module_get_current_return_type (self))) {
				ValaCCodeReturnStatement* _tmp44_;
				vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp44_ = vala_ccode_return_statement_new (NULL)));
				_vala_ccode_node_unref0 (_tmp44_);
			} else {
				ValaCCodeReturnStatement* _tmp46_;
				ValaCCodeIdentifier* _tmp45_;
				vala_ccode_fragment_append (cfrag, (ValaCCodeNode*) (_tmp46_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("result")))));
				_vala_ccode_node_unref0 (_tmp46_);
				_vala_ccode_node_unref0 (_tmp45_);
			}
		}
		vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) cfrag);
		vala_ccode_base_module_create_temp_decl (self, (ValaStatement*) stmt, (GeeList*) vala_return_statement_get_return_expression (stmt)->temp_vars);
		if (return_expression_symbol != NULL) {
			vala_symbol_set_active (return_expression_symbol, TRUE);
		}
		_vala_code_node_unref0 (return_expression_symbol);
		_vala_code_node_unref0 (local);
		_vala_ccode_node_unref0 (cfrag);
		_vala_ccode_node_unref0 (result_lhs);
	}
}


char* vala_ccode_base_module_get_symbol_lock_name (ValaCCodeBaseModule* self, const char* symname) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (symname != NULL, NULL);
	result = g_strdup_printf ("__lock_%s", symname);
	return result;
}


static void vala_ccode_base_module_real_visit_lock_statement (ValaCCodeModule* base, ValaLockStatement* stmt) {
	ValaCCodeBaseModule * self;
	ValaCCodeFragment* cn;
	ValaCCodeExpression* l;
	ValaCCodeFunctionCall* fc;
	ValaExpression* inner_node;
	ValaMember* member;
	ValaTypeSymbol* parent;
	ValaCCodeFunctionCall* _tmp19_;
	ValaCCodeIdentifier* _tmp18_;
	char* _tmp17_;
	ValaMethod* _tmp16_;
	ValaCCodeUnaryExpression* _tmp20_;
	ValaCCodeExpressionStatement* _tmp21_;
	ValaCCodeFunctionCall* _tmp25_;
	ValaCCodeIdentifier* _tmp24_;
	char* _tmp23_;
	ValaMethod* _tmp22_;
	ValaCCodeUnaryExpression* _tmp26_;
	ValaCCodeExpressionStatement* _tmp27_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (stmt != NULL);
	cn = vala_ccode_fragment_new ();
	l = NULL;
	fc = NULL;
	inner_node = _vala_code_node_ref0 (vala_member_access_get_inner (VALA_MEMBER_ACCESS (vala_lock_statement_get_resource (stmt))));
	member = _vala_code_node_ref0 (VALA_MEMBER (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt))));
	parent = _vala_code_node_ref0 (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))));
	if (vala_symbol_is_instance_member ((ValaSymbol*) member)) {
		ValaCCodeExpression* _tmp5_;
		char* _tmp4_;
		ValaCCodeMemberAccess* _tmp3_;
		if (inner_node == NULL) {
			ValaCCodeExpression* _tmp0_;
			l = (_tmp0_ = (ValaCCodeExpression*) vala_ccode_identifier_new ("self"), _vala_ccode_node_unref0 (l), _tmp0_);
		} else {
			if (vala_symbol_get_parent_symbol (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt))) != VALA_SYMBOL (vala_ccode_base_module_get_current_type_symbol (self))) {
				ValaCCodeExpression* _tmp1_;
				l = (_tmp1_ = (ValaCCodeExpression*) vala_ccode_base_module_generate_instance_cast (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) inner_node)), parent), _vala_ccode_node_unref0 (l), _tmp1_);
			} else {
				ValaCCodeExpression* _tmp2_;
				l = (_tmp2_ = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) inner_node))), _vala_ccode_node_unref0 (l), _tmp2_);
			}
		}
		l = (_tmp5_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp3_ = vala_ccode_member_access_new_pointer (l, "priv")), _tmp4_ = vala_ccode_base_module_get_symbol_lock_name (self, vala_symbol_get_name (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt))))), _vala_ccode_node_unref0 (l), _tmp5_);
		_g_free0 (_tmp4_);
		_vala_ccode_node_unref0 (_tmp3_);
	} else {
		if (vala_symbol_is_class_member ((ValaSymbol*) member)) {
			ValaCCodeExpression* _tmp9_;
			char* _tmp8_;
			char* _tmp7_;
			char* _tmp6_;
			ValaCCodeExpression* _tmp11_;
			char* _tmp10_;
			l = (_tmp9_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp8_ = g_strdup_printf ("%s_GET_CLASS_PRIVATE(%s)", _tmp6_ = vala_typesymbol_get_upper_case_cname (parent, NULL), _tmp7_ = vala_typesymbol_get_type_id (parent))), _vala_ccode_node_unref0 (l), _tmp9_);
			_g_free0 (_tmp8_);
			_g_free0 (_tmp7_);
			_g_free0 (_tmp6_);
			l = (_tmp11_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (l, _tmp10_ = vala_ccode_base_module_get_symbol_lock_name (self, vala_symbol_get_name (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt))))), _vala_ccode_node_unref0 (l), _tmp11_);
			_g_free0 (_tmp10_);
		} else {
			char* _tmp12_;
			char* _tmp13_;
			char* lock_name;
			ValaCCodeExpression* _tmp15_;
			char* _tmp14_;
			lock_name = (_tmp13_ = g_strdup_printf ("%s_%s", _tmp12_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) parent, NULL), vala_symbol_get_name (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))), _g_free0 (_tmp12_), _tmp13_);
			l = (_tmp15_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp14_ = vala_ccode_base_module_get_symbol_lock_name (self, lock_name)), _vala_ccode_node_unref0 (l), _tmp15_);
			_g_free0 (_tmp14_);
			_g_free0 (lock_name);
		}
	}
	fc = (_tmp19_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_identifier_new (_tmp17_ = vala_method_get_cname (_tmp16_ = VALA_METHOD (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) self->mutex_type), "lock")))))), _vala_ccode_node_unref0 (fc), _tmp19_);
	_vala_ccode_node_unref0 (_tmp18_);
	_g_free0 (_tmp17_);
	_vala_code_node_unref0 (_tmp16_);
	vala_ccode_function_call_add_argument (fc, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, l)));
	_vala_ccode_node_unref0 (_tmp20_);
	vala_ccode_fragment_append (cn, (ValaCCodeNode*) (_tmp21_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) fc)));
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_fragment_append (cn, vala_code_node_get_ccodenode ((ValaCodeNode*) vala_lock_statement_get_body (stmt)));
	fc = (_tmp25_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new (_tmp23_ = vala_method_get_cname (_tmp22_ = VALA_METHOD (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) self->mutex_type), "unlock")))))), _vala_ccode_node_unref0 (fc), _tmp25_);
	_vala_ccode_node_unref0 (_tmp24_);
	_g_free0 (_tmp23_);
	_vala_code_node_unref0 (_tmp22_);
	vala_ccode_function_call_add_argument (fc, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, l)));
	_vala_ccode_node_unref0 (_tmp26_);
	vala_ccode_fragment_append (cn, (ValaCCodeNode*) (_tmp27_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) fc)));
	_vala_ccode_node_unref0 (_tmp27_);
	vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) cn);
	_vala_ccode_node_unref0 (cn);
	_vala_ccode_node_unref0 (l);
	_vala_ccode_node_unref0 (fc);
	_vala_code_node_unref0 (inner_node);
	_vala_code_node_unref0 (member);
	_vala_code_node_unref0 (parent);
}


static void vala_ccode_base_module_real_visit_delete_statement (ValaCCodeModule* base, ValaDeleteStatement* stmt) {
	ValaCCodeBaseModule * self;
	ValaPointerType* pointer_type;
	ValaDataType* type;
	gboolean _tmp0_;
	ValaCCodeExpression* _tmp2_;
	ValaCCodeFunctionCall* _tmp3_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeExpressionStatement* _tmp4_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (stmt != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) stmt, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	pointer_type = _vala_code_node_ref0 (VALA_POINTER_TYPE (vala_expression_get_value_type (vala_delete_statement_get_expression (stmt))));
	type = _vala_code_node_ref0 ((ValaDataType*) pointer_type);
	_tmp0_ = FALSE;
	if (vala_data_type_get_data_type (vala_pointer_type_get_base_type (pointer_type)) != NULL) {
		_tmp0_ = vala_typesymbol_is_reference_type (vala_data_type_get_data_type (vala_pointer_type_get_base_type (pointer_type)));
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaDataType* _tmp1_;
		type = (_tmp1_ = _vala_code_node_ref0 (vala_pointer_type_get_base_type (pointer_type)), _vala_code_node_unref0 (type), _tmp1_);
	}
	ccall = (_tmp3_ = vala_ccode_function_call_new (_tmp2_ = vala_ccode_base_module_get_destroy_func_expression (self, type, FALSE)), _vala_ccode_node_unref0 (_tmp2_), _tmp3_);
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_delete_statement_get_expression (stmt))));
	vala_code_node_set_ccodenode ((ValaCodeNode*) stmt, (ValaCCodeNode*) (_tmp4_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp4_);
	_vala_code_node_unref0 (pointer_type);
	_vala_code_node_unref0 (type);
	_vala_ccode_node_unref0 (ccall);
}


static void vala_ccode_base_module_real_visit_expression (ValaCCodeModule* base, ValaExpression* expr) {
	ValaCCodeBaseModule * self;
	gboolean _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	_tmp0_ = FALSE;
	if (vala_code_node_get_ccodenode ((ValaCodeNode*) expr) != NULL) {
		_tmp0_ = !vala_expression_get_lvalue (expr);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gboolean _tmp1_;
		ValaCCodeExpression* _tmp3_;
		gboolean _tmp4_;
		_tmp1_ = FALSE;
		if (VALA_IS_GENERIC_TYPE (vala_expression_get_formal_value_type (expr))) {
			_tmp1_ = !VALA_IS_GENERIC_TYPE (vala_expression_get_value_type (expr));
		} else {
			_tmp1_ = FALSE;
		}
		if (_tmp1_) {
			if (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_data_type_get_type_parameter (vala_expression_get_formal_value_type (expr))) != VALA_SYMBOL (self->garray_type)) {
				ValaCCodeExpression* _tmp2_;
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp2_ = vala_ccode_base_module_convert_from_generic_pointer (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)), vala_expression_get_value_type (expr))));
				_vala_ccode_node_unref0 (_tmp2_);
			}
		}
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp3_ = vala_ccode_base_module_transform_expression (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)), vala_expression_get_value_type (expr), vala_expression_get_target_type (expr), expr)));
		_vala_ccode_node_unref0 (_tmp3_);
		_tmp4_ = FALSE;
		if (VALA_IS_GENERIC_TYPE (vala_expression_get_formal_target_type (expr))) {
			_tmp4_ = !VALA_IS_GENERIC_TYPE (vala_expression_get_target_type (expr));
		} else {
			_tmp4_ = FALSE;
		}
		if (_tmp4_) {
			if (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_data_type_get_type_parameter (vala_expression_get_formal_target_type (expr))) != VALA_SYMBOL (self->garray_type)) {
				ValaCCodeExpression* _tmp5_;
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp5_ = vala_ccode_base_module_convert_to_generic_pointer (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)), vala_expression_get_target_type (expr))));
				_vala_ccode_node_unref0 (_tmp5_);
			}
		}
	}
}


static void vala_ccode_base_module_real_visit_boolean_literal (ValaCCodeModule* base, ValaBooleanLiteral* expr) {
	ValaCCodeBaseModule * self;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	if (vala_code_context_get_profile (self->priv->_context) == VALA_PROFILE_GOBJECT) {
		const char* _tmp0_;
		ValaCCodeConstant* _tmp1_;
		_tmp0_ = NULL;
		if (vala_boolean_literal_get_value (expr)) {
			_tmp0_ = "TRUE";
		} else {
			_tmp0_ = "FALSE";
		}
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp1_ = vala_ccode_constant_new (_tmp0_)));
		_vala_ccode_node_unref0 (_tmp1_);
	} else {
		const char* _tmp2_;
		ValaCCodeConstant* _tmp3_;
		vala_ccode_declaration_space_add_include (self->source_declarations, "stdbool.h", FALSE);
		_tmp2_ = NULL;
		if (vala_boolean_literal_get_value (expr)) {
			_tmp2_ = "true";
		} else {
			_tmp2_ = "false";
		}
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp3_ = vala_ccode_constant_new (_tmp2_)));
		_vala_ccode_node_unref0 (_tmp3_);
	}
}


static void vala_ccode_base_module_real_visit_character_literal (ValaCCodeModule* base, ValaCharacterLiteral* expr) {
	ValaCCodeBaseModule * self;
	gboolean _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	_tmp0_ = FALSE;
	if (vala_character_literal_get_char (expr) >= 0x20) {
		_tmp0_ = vala_character_literal_get_char (expr) < 0x80;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaCCodeConstant* _tmp1_;
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp1_ = vala_ccode_constant_new (vala_character_literal_get_value (expr))));
		_vala_ccode_node_unref0 (_tmp1_);
	} else {
		ValaCCodeConstant* _tmp3_;
		char* _tmp2_;
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp3_ = vala_ccode_constant_new (_tmp2_ = g_strdup_printf ("%uU", (guint) vala_character_literal_get_char (expr)))));
		_vala_ccode_node_unref0 (_tmp3_);
		_g_free0 (_tmp2_);
	}
}


static void vala_ccode_base_module_real_visit_integer_literal (ValaCCodeModule* base, ValaIntegerLiteral* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeConstant* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_constant_new (vala_integer_literal_get_value (expr))));
	_vala_ccode_node_unref0 (_tmp0_);
}


static char* string_substring (const char* self, glong offset, glong len) {
	char* result;
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static gboolean string_contains (const char* self, const char* needle) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (needle != NULL, FALSE);
	result = strstr (self, needle) != NULL;
	return result;
}


static void vala_ccode_base_module_real_visit_real_literal (ValaCCodeModule* base, ValaRealLiteral* expr) {
	ValaCCodeBaseModule * self;
	char* c_literal;
	gboolean _tmp0_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	ValaCCodeConstant* _tmp8_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	c_literal = g_strdup (vala_real_literal_get_value (expr));
	_tmp0_ = FALSE;
	if (g_str_has_suffix (c_literal, "d")) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = g_str_has_suffix (c_literal, "D");
	}
	if (_tmp0_) {
		char* _tmp1_;
		c_literal = (_tmp1_ = string_substring (c_literal, (glong) 0, string_get_length (c_literal) - 1), _g_free0 (c_literal), _tmp1_);
	}
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	if (string_contains (c_literal, ".")) {
		_tmp3_ = TRUE;
	} else {
		_tmp3_ = string_contains (c_literal, "e");
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = string_contains (c_literal, "E");
	}
	if (!_tmp2_) {
		gboolean _tmp4_;
		_tmp4_ = FALSE;
		if (string_contains (c_literal, "f")) {
			_tmp4_ = TRUE;
		} else {
			_tmp4_ = string_contains (c_literal, "F");
		}
		if (_tmp4_) {
			char* _tmp6_;
			char* _tmp5_;
			c_literal = (_tmp6_ = g_strconcat (_tmp5_ = string_substring (c_literal, (glong) 0, string_get_length (c_literal) - 1), ".f", NULL), _g_free0 (c_literal), _tmp6_);
			_g_free0 (_tmp5_);
		} else {
			char* _tmp7_;
			c_literal = (_tmp7_ = g_strconcat (c_literal, ".", NULL), _g_free0 (c_literal), _tmp7_);
		}
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp8_ = vala_ccode_constant_new (c_literal)));
	_vala_ccode_node_unref0 (_tmp8_);
	_g_free0 (c_literal);
}


static void vala_ccode_base_module_real_visit_string_literal (ValaCCodeModule* base, ValaStringLiteral* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeConstant* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_constant_new (vala_string_literal_get_value (expr))));
	_vala_ccode_node_unref0 (_tmp0_);
}


static void vala_ccode_base_module_real_visit_null_literal (ValaCCodeModule* base, ValaNullLiteral* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeConstant* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	if (vala_code_context_get_profile (self->priv->_context) != VALA_PROFILE_GOBJECT) {
		vala_ccode_declaration_space_add_include (self->source_declarations, "stddef.h", FALSE);
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp0_);
}


static char* vala_ccode_base_module_real_get_delegate_target_cname (ValaCCodeBaseModule* self, const char* delegate_cname) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (delegate_cname != NULL, NULL);
	g_assert_not_reached ();
}


char* vala_ccode_base_module_get_delegate_target_cname (ValaCCodeBaseModule* self, const char* delegate_cname) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->get_delegate_target_cname (self, delegate_cname);
}


static ValaCCodeExpression* vala_ccode_base_module_real_get_delegate_target_cexpression (ValaCCodeBaseModule* self, ValaExpression* delegate_expr) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (delegate_expr != NULL, NULL);
	g_assert_not_reached ();
}


ValaCCodeExpression* vala_ccode_base_module_get_delegate_target_cexpression (ValaCCodeBaseModule* self, ValaExpression* delegate_expr) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->get_delegate_target_cexpression (self, delegate_expr);
}


static char* vala_ccode_base_module_real_get_delegate_target_destroy_notify_cname (ValaCCodeBaseModule* self, const char* delegate_cname) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (delegate_cname != NULL, NULL);
	g_assert_not_reached ();
}


char* vala_ccode_base_module_get_delegate_target_destroy_notify_cname (ValaCCodeBaseModule* self, const char* delegate_cname) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->get_delegate_target_destroy_notify_cname (self, delegate_cname);
}


static void vala_ccode_base_module_real_visit_base_access (ValaCCodeModule* base, ValaBaseAccess* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeIdentifier* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp1_ = vala_ccode_base_module_generate_instance_cast (self, (ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("self")), vala_data_type_get_data_type (vala_expression_get_value_type ((ValaExpression*) expr)))));
	_vala_ccode_node_unref0 (_tmp1_);
	_vala_ccode_node_unref0 (_tmp0_);
}


static void vala_ccode_base_module_real_visit_postfix_expression (ValaCCodeModule* base, ValaPostfixExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaMemberAccess* ma;
	ValaCCodeUnaryOperator _tmp7_;
	ValaCCodeUnaryOperator op;
	ValaCCodeUnaryExpression* _tmp8_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	ma = vala_ccode_base_module_find_property_access (self, vala_postfix_expression_get_inner (expr));
	if (ma != NULL) {
		ValaProperty* prop;
		ValaCCodeCommaExpression* ccomma;
		ValaLocalVariable* temp_decl;
		ValaCCodeAssignment* _tmp1_;
		ValaCCodeExpression* _tmp0_;
		ValaCCodeBinaryOperator _tmp2_;
		ValaCCodeBinaryOperator op;
		ValaCCodeConstant* _tmp4_;
		ValaCCodeExpression* _tmp3_;
		ValaCCodeBinaryExpression* _tmp5_;
		ValaCCodeBinaryExpression* cexpr;
		ValaCCodeFunctionCall* ccall;
		ValaCCodeExpression* _tmp6_;
		prop = _vala_code_node_ref0 (VALA_PROPERTY (vala_expression_get_symbol_reference ((ValaExpression*) ma)));
		ccomma = vala_ccode_comma_expression_new ();
		temp_decl = vala_ccode_base_module_get_temp_variable (self, vala_property_get_property_type (prop), TRUE, (ValaCodeNode*) expr, FALSE);
		gee_list_insert ((GeeList*) self->temp_vars, 0, temp_decl);
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp1_ = vala_ccode_assignment_new (_tmp0_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_decl)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_postfix_expression_get_inner (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		_vala_ccode_node_unref0 (_tmp1_);
		_vala_ccode_node_unref0 (_tmp0_);
		_tmp2_ = 0;
		if (vala_postfix_expression_get_increment (expr)) {
			_tmp2_ = VALA_CCODE_BINARY_OPERATOR_PLUS;
		} else {
			_tmp2_ = VALA_CCODE_BINARY_OPERATOR_MINUS;
		}
		op = _tmp2_;
		cexpr = (_tmp5_ = vala_ccode_binary_expression_new (op, _tmp3_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_decl)), (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new ("1"))), _vala_ccode_node_unref0 (_tmp4_), _vala_ccode_node_unref0 (_tmp3_), _tmp5_);
		ccall = vala_ccode_base_module_get_property_set_call (self, prop, ma, (ValaCCodeExpression*) cexpr, NULL);
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
		vala_ccode_comma_expression_append_expression (ccomma, _tmp6_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_decl)));
		_vala_ccode_node_unref0 (_tmp6_);
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
		_vala_code_node_unref0 (prop);
		_vala_ccode_node_unref0 (ccomma);
		_vala_code_node_unref0 (temp_decl);
		_vala_ccode_node_unref0 (cexpr);
		_vala_ccode_node_unref0 (ccall);
		_vala_code_node_unref0 (ma);
		return;
	}
	_tmp7_ = 0;
	if (vala_postfix_expression_get_increment (expr)) {
		_tmp7_ = VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT;
	} else {
		_tmp7_ = VALA_CCODE_UNARY_OPERATOR_POSTFIX_DECREMENT;
	}
	op = _tmp7_;
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp8_ = vala_ccode_unary_expression_new (op, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_postfix_expression_get_inner (expr))))));
	_vala_ccode_node_unref0 (_tmp8_);
	_vala_code_node_unref0 (ma);
}


static ValaMemberAccess* vala_ccode_base_module_find_property_access (ValaCCodeBaseModule* self, ValaExpression* expr) {
	ValaMemberAccess* result;
	ValaMemberAccess* ma;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (expr != NULL, NULL);
	if (!VALA_IS_MEMBER_ACCESS (expr)) {
		result = NULL;
		return result;
	}
	ma = _vala_code_node_ref0 (VALA_MEMBER_ACCESS (expr));
	if (VALA_IS_PROPERTY (vala_expression_get_symbol_reference ((ValaExpression*) ma))) {
		result = ma;
		return result;
	}
	result = NULL;
	_vala_code_node_unref0 (ma);
	return result;
}


static gboolean vala_ccode_base_module_is_limited_generic_type (ValaCCodeBaseModule* self, ValaDataType* type) {
	gboolean result;
	ValaSymbol* _tmp0_;
	ValaClass* cl;
	ValaSymbol* _tmp1_;
	ValaStruct* st;
	gboolean _tmp2_;
	gboolean _tmp3_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type != NULL, FALSE);
	cl = _vala_code_node_ref0 ((_tmp0_ = vala_symbol_get_parent_symbol ((ValaSymbol*) vala_data_type_get_type_parameter (type)), VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
	st = _vala_code_node_ref0 ((_tmp1_ = vala_symbol_get_parent_symbol ((ValaSymbol*) vala_data_type_get_type_parameter (type)), VALA_IS_STRUCT (_tmp1_) ? ((ValaStruct*) _tmp1_) : NULL));
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	if (cl != NULL) {
		_tmp3_ = vala_class_get_is_compact (cl);
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = st != NULL;
	}
	if (_tmp2_) {
		result = TRUE;
		_vala_code_node_unref0 (cl);
		_vala_code_node_unref0 (st);
		return result;
	}
	result = FALSE;
	_vala_code_node_unref0 (cl);
	_vala_code_node_unref0 (st);
	return result;
}


gboolean vala_ccode_base_module_requires_copy (ValaCCodeBaseModule* self, ValaDataType* type) {
	gboolean result;
	ValaTypeSymbol* _tmp0_;
	ValaClass* cl;
	gboolean _tmp1_;
	gboolean _tmp2_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type != NULL, FALSE);
	if (!vala_data_type_is_disposable (type)) {
		result = FALSE;
		return result;
	}
	cl = _vala_code_node_ref0 ((_tmp0_ = vala_data_type_get_data_type (type), VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	if (cl != NULL) {
		_tmp2_ = vala_typesymbol_is_reference_counting ((ValaTypeSymbol*) cl);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		char* _tmp3_;
		_tmp1_ = _vala_strcmp0 (_tmp3_ = vala_typesymbol_get_ref_function ((ValaTypeSymbol*) cl), "") == 0;
		_g_free0 (_tmp3_);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		result = FALSE;
		_vala_code_node_unref0 (cl);
		return result;
	}
	if (vala_data_type_get_type_parameter (type) != NULL) {
		if (vala_ccode_base_module_is_limited_generic_type (self, type)) {
			result = FALSE;
			_vala_code_node_unref0 (cl);
			return result;
		}
	}
	result = TRUE;
	_vala_code_node_unref0 (cl);
	return result;
}


gboolean vala_ccode_base_module_requires_destroy (ValaCCodeBaseModule* self, ValaDataType* type) {
	gboolean result;
	ValaDataType* _tmp0_;
	ValaArrayType* array_type;
	gboolean _tmp1_;
	ValaTypeSymbol* _tmp2_;
	ValaClass* cl;
	gboolean _tmp3_;
	gboolean _tmp4_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type != NULL, FALSE);
	if (!vala_data_type_is_disposable (type)) {
		result = FALSE;
		return result;
	}
	array_type = _vala_code_node_ref0 ((_tmp0_ = type, VALA_IS_ARRAY_TYPE (_tmp0_) ? ((ValaArrayType*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (array_type != NULL) {
		_tmp1_ = vala_array_type_get_fixed_length (array_type);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		result = vala_ccode_base_module_requires_destroy (self, vala_array_type_get_element_type (array_type));
		_vala_code_node_unref0 (array_type);
		return result;
	}
	cl = _vala_code_node_ref0 ((_tmp2_ = vala_data_type_get_data_type (type), VALA_IS_CLASS (_tmp2_) ? ((ValaClass*) _tmp2_) : NULL));
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	if (cl != NULL) {
		_tmp4_ = vala_typesymbol_is_reference_counting ((ValaTypeSymbol*) cl);
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		char* _tmp5_;
		_tmp3_ = _vala_strcmp0 (_tmp5_ = vala_typesymbol_get_unref_function ((ValaTypeSymbol*) cl), "") == 0;
		_g_free0 (_tmp5_);
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		result = FALSE;
		_vala_code_node_unref0 (array_type);
		_vala_code_node_unref0 (cl);
		return result;
	}
	if (vala_data_type_get_type_parameter (type) != NULL) {
		if (vala_ccode_base_module_is_limited_generic_type (self, type)) {
			result = FALSE;
			_vala_code_node_unref0 (array_type);
			_vala_code_node_unref0 (cl);
			return result;
		}
	}
	result = TRUE;
	_vala_code_node_unref0 (array_type);
	_vala_code_node_unref0 (cl);
	return result;
}


static gboolean vala_ccode_base_module_is_ref_function_void (ValaCCodeBaseModule* self, ValaDataType* type) {
	gboolean result;
	ValaTypeSymbol* _tmp0_;
	ValaClass* cl;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type != NULL, FALSE);
	cl = _vala_code_node_ref0 ((_tmp0_ = vala_data_type_get_data_type (type), VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (cl != NULL) {
		_tmp1_ = vala_class_get_ref_function_void (cl);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		result = TRUE;
		_vala_code_node_unref0 (cl);
		return result;
	} else {
		result = FALSE;
		_vala_code_node_unref0 (cl);
		return result;
	}
	_vala_code_node_unref0 (cl);
}


static ValaCCodeExpression* vala_ccode_base_module_real_get_ref_cexpression (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node) {
	ValaCCodeExpression* result;
	gboolean _tmp0_;
	ValaCCodeExpression* dupexpr;
	gboolean _tmp18_;
	gboolean _tmp19_;
	ValaCCodeFunctionCall* ccall;
	gboolean _tmp30_;
	gboolean _tmp31_;
	gboolean _tmp32_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (expression_type != NULL, NULL);
	g_return_val_if_fail (cexpr != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	_tmp0_ = FALSE;
	if (VALA_IS_VALUE_TYPE (expression_type)) {
		_tmp0_ = !vala_data_type_get_nullable (expression_type);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaLocalVariable* decl;
		ValaCCodeExpression* ctemp;
		ValaValueType* vt;
		ValaStruct* st;
		ValaCCodeIdentifier* _tmp2_;
		char* _tmp1_;
		ValaCCodeFunctionCall* _tmp3_;
		ValaCCodeFunctionCall* copy_call;
		ValaCCodeUnaryExpression* _tmp4_;
		ValaCCodeUnaryExpression* _tmp5_;
		ValaCCodeCommaExpression* ccomma;
		char* _tmp6_;
		gboolean _tmp7_;
		gboolean _tmp14_;
		decl = vala_ccode_base_module_get_temp_variable (self, expression_type, FALSE, node, TRUE);
		gee_list_insert ((GeeList*) self->temp_vars, 0, decl);
		ctemp = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl));
		vt = _vala_code_node_ref0 (VALA_VALUE_TYPE (expression_type));
		st = _vala_code_node_ref0 (VALA_STRUCT (vala_value_type_get_type_symbol (vt)));
		copy_call = (_tmp3_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new (_tmp1_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st)))), _vala_ccode_node_unref0 (_tmp2_), _g_free0 (_tmp1_), _tmp3_);
		vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, cexpr)));
		_vala_ccode_node_unref0 (_tmp4_);
		vala_ccode_function_call_add_argument (copy_call, (ValaCCodeExpression*) (_tmp5_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ctemp)));
		_vala_ccode_node_unref0 (_tmp5_);
		if (!vala_struct_get_has_copy_function (st)) {
			vala_ccode_base_module_generate_struct_copy_function (self, st);
		}
		ccomma = vala_ccode_comma_expression_new ();
		if ((_tmp7_ = _vala_strcmp0 (_tmp6_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st), "g_value_copy") == 0, _g_free0 (_tmp6_), _tmp7_)) {
			ValaCCodeIdentifier* _tmp8_;
			ValaCCodeFunctionCall* _tmp9_;
			ValaCCodeFunctionCall* value_type_call;
			ValaCCodeUnaryExpression* _tmp10_;
			ValaCCodeIdentifier* _tmp11_;
			ValaCCodeFunctionCall* _tmp12_;
			ValaCCodeFunctionCall* init_call;
			ValaCCodeUnaryExpression* _tmp13_;
			value_type_call = (_tmp9_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("G_VALUE_TYPE"))), _vala_ccode_node_unref0 (_tmp8_), _tmp9_);
			vala_ccode_function_call_add_argument (value_type_call, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, cexpr)));
			_vala_ccode_node_unref0 (_tmp10_);
			init_call = (_tmp12_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new ("g_value_init"))), _vala_ccode_node_unref0 (_tmp11_), _tmp12_);
			vala_ccode_function_call_add_argument (init_call, (ValaCCodeExpression*) (_tmp13_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ctemp)));
			_vala_ccode_node_unref0 (_tmp13_);
			vala_ccode_function_call_add_argument (init_call, (ValaCCodeExpression*) value_type_call);
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) init_call);
			_vala_ccode_node_unref0 (value_type_call);
			_vala_ccode_node_unref0 (init_call);
		}
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) copy_call);
		vala_ccode_comma_expression_append_expression (ccomma, ctemp);
		_tmp14_ = FALSE;
		if (self->gvalue_type != NULL) {
			_tmp14_ = vala_data_type_get_data_type (expression_type) == VALA_TYPESYMBOL (self->gvalue_type);
		} else {
			_tmp14_ = FALSE;
		}
		if (_tmp14_) {
			ValaCCodeIdentifier* _tmp15_;
			ValaCCodeFunctionCall* _tmp16_;
			ValaCCodeFunctionCall* cisvalid;
			ValaCCodeUnaryExpression* _tmp17_;
			cisvalid = (_tmp16_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("G_IS_VALUE"))), _vala_ccode_node_unref0 (_tmp15_), _tmp16_);
			vala_ccode_function_call_add_argument (cisvalid, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, cexpr)));
			_vala_ccode_node_unref0 (_tmp17_);
			result = (ValaCCodeExpression*) vala_ccode_conditional_expression_new ((ValaCCodeExpression*) cisvalid, (ValaCCodeExpression*) ccomma, cexpr);
			_vala_ccode_node_unref0 (cisvalid);
			_vala_code_node_unref0 (decl);
			_vala_ccode_node_unref0 (ctemp);
			_vala_code_node_unref0 (vt);
			_vala_code_node_unref0 (st);
			_vala_ccode_node_unref0 (copy_call);
			_vala_ccode_node_unref0 (ccomma);
			return result;
		} else {
			result = (ValaCCodeExpression*) ccomma;
			_vala_code_node_unref0 (decl);
			_vala_ccode_node_unref0 (ctemp);
			_vala_code_node_unref0 (vt);
			_vala_code_node_unref0 (st);
			_vala_ccode_node_unref0 (copy_call);
			return result;
		}
		_vala_code_node_unref0 (decl);
		_vala_ccode_node_unref0 (ctemp);
		_vala_code_node_unref0 (vt);
		_vala_code_node_unref0 (st);
		_vala_ccode_node_unref0 (copy_call);
		_vala_ccode_node_unref0 (ccomma);
	}
	dupexpr = vala_ccode_base_module_get_dup_func_expression (self, expression_type, vala_code_node_get_source_reference (node), FALSE);
	if (dupexpr == NULL) {
		vala_code_node_set_error (node, TRUE);
		result = NULL;
		_vala_ccode_node_unref0 (dupexpr);
		return result;
	}
	_tmp18_ = FALSE;
	_tmp19_ = FALSE;
	if (VALA_IS_CCODE_IDENTIFIER (dupexpr)) {
		_tmp19_ = !VALA_IS_ARRAY_TYPE (expression_type);
	} else {
		_tmp19_ = FALSE;
	}
	if (_tmp19_) {
		_tmp18_ = !vala_ccode_base_module_is_ref_function_void (self, expression_type);
	} else {
		_tmp18_ = FALSE;
	}
	if (_tmp18_) {
		ValaCCodeIdentifier* dupid;
		char* dup0_func;
		ValaCCodeIdentifier* _tmp28_;
		ValaCCodeFunctionCall* _tmp29_;
		ValaCCodeFunctionCall* ccall;
		dupid = _vala_ccode_node_ref0 (VALA_CCODE_IDENTIFIER (dupexpr));
		dup0_func = g_strdup_printf ("_%s0", vala_ccode_identifier_get_name (dupid));
		if (_vala_strcmp0 (vala_ccode_identifier_get_name (dupid), "g_strdup") == 0) {
			char* _tmp20_;
			dup0_func = (_tmp20_ = g_strdup (vala_ccode_identifier_get_name (dupid)), _g_free0 (dup0_func), _tmp20_);
		} else {
			if (vala_ccode_base_module_add_wrapper (self, dup0_func)) {
				ValaCCodeFunction* dup0_fun;
				ValaCCodeFormalParameter* _tmp21_;
				ValaCCodeBlock* _tmp22_;
				ValaCCodeFunctionCall* dup_call;
				ValaCCodeIdentifier* _tmp23_;
				ValaCCodeReturnStatement* _tmp27_;
				ValaCCodeConditionalExpression* _tmp26_;
				ValaCCodeConstant* _tmp25_;
				ValaCCodeIdentifier* _tmp24_;
				dup0_fun = vala_ccode_function_new (dup0_func, "gpointer");
				vala_ccode_function_add_parameter (dup0_fun, _tmp21_ = vala_ccode_formal_parameter_new ("self", "gpointer"));
				_vala_ccode_node_unref0 (_tmp21_);
				vala_ccode_function_set_modifiers (dup0_fun, VALA_CCODE_MODIFIERS_STATIC);
				vala_ccode_function_set_block (dup0_fun, _tmp22_ = vala_ccode_block_new ());
				_vala_ccode_node_unref0 (_tmp22_);
				dup_call = vala_ccode_function_call_new (dupexpr);
				vala_ccode_function_call_add_argument (dup_call, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new ("self")));
				_vala_ccode_node_unref0 (_tmp23_);
				vala_ccode_block_add_statement (vala_ccode_function_get_block (dup0_fun), (ValaCCodeNode*) (_tmp27_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp26_ = vala_ccode_conditional_expression_new ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("self")), (ValaCCodeExpression*) dup_call, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_constant_new ("NULL")))))));
				_vala_ccode_node_unref0 (_tmp27_);
				_vala_ccode_node_unref0 (_tmp26_);
				_vala_ccode_node_unref0 (_tmp25_);
				_vala_ccode_node_unref0 (_tmp24_);
				vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) dup0_fun);
				_vala_ccode_node_unref0 (dup0_fun);
				_vala_ccode_node_unref0 (dup_call);
			}
		}
		ccall = (_tmp29_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new (dup0_func))), _vala_ccode_node_unref0 (_tmp28_), _tmp29_);
		vala_ccode_function_call_add_argument (ccall, cexpr);
		result = (ValaCCodeExpression*) ccall;
		_vala_ccode_node_unref0 (dupid);
		_g_free0 (dup0_func);
		_vala_ccode_node_unref0 (dupexpr);
		return result;
	}
	ccall = vala_ccode_function_call_new (dupexpr);
	_tmp30_ = FALSE;
	_tmp31_ = FALSE;
	_tmp32_ = FALSE;
	if (!VALA_IS_ARRAY_TYPE (expression_type)) {
		_tmp32_ = expr != NULL;
	} else {
		_tmp32_ = FALSE;
	}
	if (_tmp32_) {
		_tmp31_ = vala_expression_is_non_null (expr);
	} else {
		_tmp31_ = FALSE;
	}
	if (_tmp31_) {
		_tmp30_ = !vala_ccode_base_module_is_ref_function_void (self, expression_type);
	} else {
		_tmp30_ = FALSE;
	}
	if (_tmp30_) {
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)));
		result = (ValaCCodeExpression*) ccall;
		_vala_ccode_node_unref0 (dupexpr);
		return result;
	} else {
		ValaLocalVariable* decl;
		ValaCCodeExpression* ctemp;
		ValaCCodeConstant* _tmp33_;
		ValaCCodeBinaryExpression* _tmp34_;
		ValaCCodeBinaryExpression* cisnull;
		ValaCCodeCommaExpression* ccomma;
		ValaCCodeAssignment* _tmp44_;
		ValaCCodeExpression* cifnull;
		ValaCCodeConditionalExpression* _tmp47_;
		decl = vala_ccode_base_module_get_temp_variable (self, expression_type, FALSE, node, FALSE);
		gee_list_insert ((GeeList*) self->temp_vars, 0, decl);
		ctemp = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl));
		cisnull = (_tmp34_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, ctemp, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_constant_new ("NULL"))), _vala_ccode_node_unref0 (_tmp33_), _tmp34_);
		if (vala_data_type_get_type_parameter (expression_type) != NULL) {
			ValaCCodeConstant* _tmp36_;
			ValaCCodeExpression* _tmp35_;
			ValaCCodeBinaryExpression* _tmp37_;
			ValaCCodeBinaryExpression* cdupisnull;
			ValaCCodeBinaryExpression* _tmp38_;
			cdupisnull = (_tmp37_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, _tmp35_ = vala_ccode_base_module_get_dup_func_expression (self, expression_type, vala_code_node_get_source_reference (node), FALSE), (ValaCCodeExpression*) (_tmp36_ = vala_ccode_constant_new ("NULL"))), _vala_ccode_node_unref0 (_tmp36_), _vala_ccode_node_unref0 (_tmp35_), _tmp37_);
			cisnull = (_tmp38_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_OR, (ValaCCodeExpression*) cisnull, (ValaCCodeExpression*) cdupisnull), _vala_ccode_node_unref0 (cisnull), _tmp38_);
			_vala_ccode_node_unref0 (cdupisnull);
		}
		if (vala_data_type_get_type_parameter (expression_type) != NULL) {
			ValaCCodeCastExpression* _tmp39_;
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp39_ = vala_ccode_cast_expression_new (ctemp, "gpointer")));
			_vala_ccode_node_unref0 (_tmp39_);
		} else {
			vala_ccode_function_call_add_argument (ccall, ctemp);
		}
		if (VALA_IS_ARRAY_TYPE (expression_type)) {
			ValaArrayType* array_type;
			gboolean first;
			ValaCCodeExpression* csizeexpr;
			array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (expression_type));
			first = TRUE;
			csizeexpr = NULL;
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp40_;
					_tmp40_ = TRUE;
					while (TRUE) {
						if (!_tmp40_) {
							dim++;
						}
						_tmp40_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						if (first) {
							ValaCCodeExpression* _tmp41_;
							csizeexpr = (_tmp41_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), expr, dim), _vala_ccode_node_unref0 (csizeexpr), _tmp41_);
							first = FALSE;
						} else {
							ValaCCodeExpression* _tmp43_;
							ValaCCodeExpression* _tmp42_;
							csizeexpr = (_tmp43_ = (ValaCCodeExpression*) vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, csizeexpr, _tmp42_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), expr, dim)), _vala_ccode_node_unref0 (csizeexpr), _tmp43_);
							_vala_ccode_node_unref0 (_tmp42_);
						}
					}
				}
			}
			vala_ccode_function_call_add_argument (ccall, csizeexpr);
			_vala_code_node_unref0 (array_type);
			_vala_ccode_node_unref0 (csizeexpr);
		}
		ccomma = vala_ccode_comma_expression_new ();
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_assignment_new (ctemp, cexpr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		_vala_ccode_node_unref0 (_tmp44_);
		cifnull = NULL;
		if (vala_data_type_get_data_type (expression_type) != NULL) {
			ValaCCodeExpression* _tmp45_;
			cifnull = (_tmp45_ = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL"), _vala_ccode_node_unref0 (cifnull), _tmp45_);
		} else {
			ValaCCodeExpression* _tmp46_;
			cifnull = (_tmp46_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (ctemp, "gpointer"), _vala_ccode_node_unref0 (cifnull), _tmp46_);
		}
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_conditional_expression_new ((ValaCCodeExpression*) cisnull, cifnull, (ValaCCodeExpression*) ccall)));
		_vala_ccode_node_unref0 (_tmp47_);
		if (vala_ccode_base_module_is_ref_function_void (self, expression_type)) {
			vala_ccode_comma_expression_append_expression (ccomma, ctemp);
		}
		result = (ValaCCodeExpression*) ccomma;
		_vala_code_node_unref0 (decl);
		_vala_ccode_node_unref0 (ctemp);
		_vala_ccode_node_unref0 (cisnull);
		_vala_ccode_node_unref0 (cifnull);
		_vala_ccode_node_unref0 (dupexpr);
		_vala_ccode_node_unref0 (ccall);
		return result;
	}
	_vala_ccode_node_unref0 (dupexpr);
	_vala_ccode_node_unref0 (ccall);
}


ValaCCodeExpression* vala_ccode_base_module_get_ref_cexpression (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->get_ref_cexpression (self, expression_type, cexpr, expr, node);
}


static gboolean vala_ccode_base_module_is_reference_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type_arg != NULL, FALSE);
	_tmp0_ = FALSE;
	if (vala_data_type_get_data_type (type_arg) != NULL) {
		_tmp0_ = vala_typesymbol_is_reference_type (vala_data_type_get_data_type (type_arg));
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = TRUE;
		return result;
	} else {
		result = FALSE;
		return result;
	}
}


static gboolean vala_ccode_base_module_is_nullable_value_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type_arg != NULL, FALSE);
	_tmp0_ = FALSE;
	if (VALA_IS_VALUE_TYPE (type_arg)) {
		_tmp0_ = vala_data_type_get_nullable (type_arg);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = TRUE;
		return result;
	} else {
		result = FALSE;
		return result;
	}
}


static gboolean vala_ccode_base_module_is_signed_integer_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg) {
	gboolean result;
	ValaTypeSymbol* _tmp0_;
	ValaStruct* st;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type_arg != NULL, FALSE);
	st = _vala_code_node_ref0 ((_tmp0_ = vala_data_type_get_data_type (type_arg), VALA_IS_STRUCT (_tmp0_) ? ((ValaStruct*) _tmp0_) : NULL));
	if (vala_data_type_get_nullable (type_arg)) {
		result = FALSE;
		_vala_code_node_unref0 (st);
		return result;
	} else {
		if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->bool_type)) {
			result = TRUE;
			_vala_code_node_unref0 (st);
			return result;
		} else {
			if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->char_type)) {
				result = TRUE;
				_vala_code_node_unref0 (st);
				return result;
			} else {
				gboolean _tmp1_;
				_tmp1_ = FALSE;
				if (self->unichar_type != NULL) {
					_tmp1_ = VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->unichar_type);
				} else {
					_tmp1_ = FALSE;
				}
				if (_tmp1_) {
					result = TRUE;
					_vala_code_node_unref0 (st);
					return result;
				} else {
					if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->short_type)) {
						result = TRUE;
						_vala_code_node_unref0 (st);
						return result;
					} else {
						if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->int_type)) {
							result = TRUE;
							_vala_code_node_unref0 (st);
							return result;
						} else {
							if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->long_type)) {
								result = TRUE;
								_vala_code_node_unref0 (st);
								return result;
							} else {
								if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->int8_type)) {
									result = TRUE;
									_vala_code_node_unref0 (st);
									return result;
								} else {
									if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->int16_type)) {
										result = TRUE;
										_vala_code_node_unref0 (st);
										return result;
									} else {
										if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->int32_type)) {
											result = TRUE;
											_vala_code_node_unref0 (st);
											return result;
										} else {
											if (VALA_TYPESYMBOL (st) == self->gtype_type) {
												result = TRUE;
												_vala_code_node_unref0 (st);
												return result;
											} else {
												result = FALSE;
												_vala_code_node_unref0 (st);
												return result;
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	_vala_code_node_unref0 (st);
}


static gboolean vala_ccode_base_module_is_unsigned_integer_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg) {
	gboolean result;
	ValaTypeSymbol* _tmp0_;
	ValaStruct* st;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (type_arg != NULL, FALSE);
	st = _vala_code_node_ref0 ((_tmp0_ = vala_data_type_get_data_type (type_arg), VALA_IS_STRUCT (_tmp0_) ? ((ValaStruct*) _tmp0_) : NULL));
	if (vala_data_type_get_nullable (type_arg)) {
		result = FALSE;
		_vala_code_node_unref0 (st);
		return result;
	} else {
		if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->uchar_type)) {
			result = TRUE;
			_vala_code_node_unref0 (st);
			return result;
		} else {
			if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->ushort_type)) {
				result = TRUE;
				_vala_code_node_unref0 (st);
				return result;
			} else {
				if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->uint_type)) {
					result = TRUE;
					_vala_code_node_unref0 (st);
					return result;
				} else {
					if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->ulong_type)) {
						result = TRUE;
						_vala_code_node_unref0 (st);
						return result;
					} else {
						if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->uint8_type)) {
							result = TRUE;
							_vala_code_node_unref0 (st);
							return result;
						} else {
							if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->uint16_type)) {
								result = TRUE;
								_vala_code_node_unref0 (st);
								return result;
							} else {
								if (VALA_TYPESYMBOL (st) == vala_data_type_get_data_type (self->uint32_type)) {
									result = TRUE;
									_vala_code_node_unref0 (st);
									return result;
								} else {
									result = FALSE;
									_vala_code_node_unref0 (st);
									return result;
								}
							}
						}
					}
				}
			}
		}
	}
	_vala_code_node_unref0 (st);
}


void vala_ccode_base_module_check_type (ValaCCodeBaseModule* self, ValaDataType* type) {
	ValaDataType* _tmp0_;
	ValaArrayType* array_type;
	g_return_if_fail (self != NULL);
	g_return_if_fail (type != NULL);
	array_type = _vala_code_node_ref0 ((_tmp0_ = type, VALA_IS_ARRAY_TYPE (_tmp0_) ? ((ValaArrayType*) _tmp0_) : NULL));
	if (array_type != NULL) {
		vala_ccode_base_module_check_type (self, vala_array_type_get_element_type (array_type));
	}
	{
		GeeList* _tmp1_;
		GeeIterator* _tmp2_;
		GeeIterator* _type_arg_it;
		_type_arg_it = (_tmp2_ = gee_iterable_iterator ((GeeIterable*) (_tmp1_ = vala_data_type_get_type_arguments (type))), _gee_collection_object_unref0 (_tmp1_), _tmp2_);
		while (TRUE) {
			ValaDataType* type_arg;
			if (!gee_iterator_next (_type_arg_it)) {
				break;
			}
			type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
			vala_ccode_base_module_check_type (self, type_arg);
			vala_ccode_base_module_check_type_argument (self, type_arg);
			_vala_code_node_unref0 (type_arg);
		}
		_gee_collection_object_unref0 (_type_arg_it);
	}
	_vala_code_node_unref0 (array_type);
}


static void vala_ccode_base_module_check_type_argument (ValaCCodeBaseModule* self, ValaDataType* type_arg) {
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (type_arg != NULL);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	if (VALA_IS_GENERIC_TYPE (type_arg)) {
		_tmp4_ = TRUE;
	} else {
		_tmp4_ = VALA_IS_POINTER_TYPE (type_arg);
	}
	if (_tmp4_) {
		_tmp3_ = TRUE;
	} else {
		_tmp3_ = vala_ccode_base_module_is_reference_type_argument (self, type_arg);
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = vala_ccode_base_module_is_nullable_value_type_argument (self, type_arg);
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = vala_ccode_base_module_is_signed_integer_type_argument (self, type_arg);
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = vala_ccode_base_module_is_unsigned_integer_type_argument (self, type_arg);
	}
	if (_tmp0_) {
	} else {
		if (VALA_IS_DELEGATE_TYPE (type_arg)) {
			ValaDelegateType* delegate_type;
			delegate_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (type_arg));
			if (vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type))) {
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) type_arg), "Delegates with target are not supported as generic type arguments");
			}
			_vala_code_node_unref0 (delegate_type);
		} else {
			char* _tmp6_;
			char* _tmp5_;
			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) type_arg), _tmp6_ = g_strdup_printf ("`%s' is not a supported generic type argument, use `?' to box value types", _tmp5_ = vala_code_node_to_string ((ValaCodeNode*) type_arg)));
			_g_free0 (_tmp6_);
			_g_free0 (_tmp5_);
		}
	}
}


static void vala_ccode_base_module_real_generate_class_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space) {
	char* _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (cl != NULL);
	g_return_if_fail (decl_space != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_symbol_declaration (decl_space, (ValaSymbol*) cl, _tmp0_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) cl, FALSE)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
}


void vala_ccode_base_module_generate_class_declaration (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_class_declaration (self, cl, decl_space);
}


static void vala_ccode_base_module_real_generate_interface_declaration (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (iface != NULL);
	g_return_if_fail (decl_space != NULL);
}


void vala_ccode_base_module_generate_interface_declaration (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_interface_declaration (self, iface, decl_space);
}


static void vala_ccode_base_module_real_generate_method_declaration (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (m != NULL);
	g_return_if_fail (decl_space != NULL);
}


void vala_ccode_base_module_generate_method_declaration (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_method_declaration (self, m, decl_space);
}


static void vala_ccode_base_module_real_generate_error_domain_declaration (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (edomain != NULL);
	g_return_if_fail (decl_space != NULL);
}


void vala_ccode_base_module_generate_error_domain_declaration (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space) {
	VALA_CCODE_BASE_MODULE_GET_CLASS (self)->generate_error_domain_declaration (self, edomain, decl_space);
}


void vala_ccode_base_module_add_generic_type_arguments (ValaCCodeBaseModule* self, ValaCCodeFunctionCall* ccall, GeeList* type_args, ValaCodeNode* expr, gboolean is_chainup) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ccall != NULL);
	g_return_if_fail (type_args != NULL);
	g_return_if_fail (expr != NULL);
	{
		GeeIterator* _type_arg_it;
		_type_arg_it = gee_iterable_iterator ((GeeIterable*) type_args);
		while (TRUE) {
			ValaDataType* type_arg;
			ValaCCodeExpression* _tmp0_;
			if (!gee_iterator_next (_type_arg_it)) {
				break;
			}
			type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
			vala_ccode_function_call_add_argument (ccall, _tmp0_ = vala_ccode_base_module_get_type_id_expression (self, type_arg, is_chainup));
			_vala_ccode_node_unref0 (_tmp0_);
			if (vala_ccode_base_module_requires_copy (self, type_arg)) {
				ValaCCodeExpression* dup_func;
				ValaCCodeCastExpression* _tmp1_;
				ValaCCodeExpression* _tmp2_;
				dup_func = vala_ccode_base_module_get_dup_func_expression (self, type_arg, vala_code_node_get_source_reference ((ValaCodeNode*) type_arg), is_chainup);
				if (dup_func == NULL) {
					vala_code_node_set_error (expr, TRUE);
					_vala_ccode_node_unref0 (dup_func);
					_vala_code_node_unref0 (type_arg);
					_gee_collection_object_unref0 (_type_arg_it);
					return;
				}
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp1_ = vala_ccode_cast_expression_new (dup_func, "GBoxedCopyFunc")));
				_vala_ccode_node_unref0 (_tmp1_);
				vala_ccode_function_call_add_argument (ccall, _tmp2_ = vala_ccode_base_module_get_destroy_func_expression (self, type_arg, is_chainup));
				_vala_ccode_node_unref0 (_tmp2_);
				_vala_ccode_node_unref0 (dup_func);
			} else {
				ValaCCodeConstant* _tmp3_;
				ValaCCodeConstant* _tmp4_;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_constant_new ("NULL")));
				_vala_ccode_node_unref0 (_tmp3_);
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new ("NULL")));
				_vala_ccode_node_unref0 (_tmp4_);
			}
			_vala_code_node_unref0 (type_arg);
		}
		_gee_collection_object_unref0 (_type_arg_it);
	}
}


static void vala_ccode_base_module_real_visit_object_creation_expression (ValaCCodeModule* base, ValaObjectCreationExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeExpression* instance;
	ValaCCodeExpression* creation_expr;
	ValaTypeSymbol* _tmp0_;
	ValaStruct* st;
	gboolean _tmp1_;
	gboolean _tmp2_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	instance = NULL;
	creation_expr = NULL;
	vala_ccode_base_module_check_type (self, vala_object_creation_expression_get_type_reference (expr));
	st = _vala_code_node_ref0 ((_tmp0_ = vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)), VALA_IS_STRUCT (_tmp0_) ? ((ValaStruct*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	if (st != NULL) {
		_tmp2_ = !vala_struct_is_simple_type (st);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		GeeList* _tmp3_;
		_tmp1_ = gee_collection_get_size ((GeeCollection*) (_tmp3_ = vala_object_creation_expression_get_object_initializer (expr))) > 0;
		_gee_collection_object_unref0 (_tmp3_);
	}
	if (_tmp1_) {
		ValaLocalVariable* temp_decl;
		ValaCCodeExpression* _tmp5_;
		char* _tmp4_;
		temp_decl = vala_ccode_base_module_get_temp_variable (self, vala_object_creation_expression_get_type_reference (expr), FALSE, (ValaCodeNode*) expr, TRUE);
		gee_collection_add ((GeeCollection*) self->temp_vars, temp_decl);
		instance = (_tmp5_ = vala_ccode_base_module_get_variable_cexpression (self, _tmp4_ = vala_ccode_base_module_get_variable_cname (self, vala_symbol_get_name ((ValaSymbol*) temp_decl))), _vala_ccode_node_unref0 (instance), _tmp5_);
		_g_free0 (_tmp4_);
		_vala_code_node_unref0 (temp_decl);
	}
	if (vala_expression_get_symbol_reference ((ValaExpression*) expr) == NULL) {
		if (VALA_IS_STRUCT (vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)))) {
			ValaCCodeIdentifier* _tmp6_;
			ValaCCodeFunctionCall* _tmp7_;
			ValaCCodeFunctionCall* creation_call;
			ValaCCodeUnaryExpression* _tmp8_;
			ValaCCodeConstant* _tmp9_;
			ValaCCodeIdentifier* _tmp12_;
			char* _tmp11_;
			char* _tmp10_;
			ValaCCodeExpression* _tmp13_;
			vala_ccode_declaration_space_add_include (self->source_declarations, "string.h", FALSE);
			creation_call = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("memset"))), _vala_ccode_node_unref0 (_tmp6_), _tmp7_);
			vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, instance)));
			_vala_ccode_node_unref0 (_tmp8_);
			vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_constant_new ("0")));
			_vala_ccode_node_unref0 (_tmp9_);
			vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new (_tmp11_ = g_strdup_printf ("sizeof (%s)", _tmp10_ = vala_data_type_get_cname (vala_object_creation_expression_get_type_reference (expr))))));
			_vala_ccode_node_unref0 (_tmp12_);
			_g_free0 (_tmp11_);
			_g_free0 (_tmp10_);
			creation_expr = (_tmp13_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) creation_call), _vala_ccode_node_unref0 (creation_expr), _tmp13_);
			_vala_ccode_node_unref0 (creation_call);
		}
	} else {
		gboolean _tmp14_;
		_tmp14_ = FALSE;
		if (vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)) == VALA_TYPESYMBOL (self->glist_type)) {
			_tmp14_ = TRUE;
		} else {
			_tmp14_ = vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)) == VALA_TYPESYMBOL (self->gslist_type);
		}
		if (_tmp14_) {
			ValaCCodeConstant* _tmp15_;
			vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp15_ = vala_ccode_constant_new ("NULL")));
			_vala_ccode_node_unref0 (_tmp15_);
		} else {
			if (VALA_IS_METHOD (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
				ValaMethod* m;
				GeeList* params;
				ValaCCodeFunctionCall* creation_call;
				ValaTypeSymbol* _tmp16_;
				ValaClass* cl;
				gboolean _tmp25_;
				gboolean _tmp26_;
				gboolean _tmp28_;
				GeeHashMap* carg_map;
				gboolean ellipsis;
				gint i;
				gint arg_pos;
				GeeIterator* params_it;
				gint last_pos;
				gint min_pos;
				gboolean _tmp43_;
				gboolean _tmp44_;
				ValaCCodeExpression* _tmp51_;
				char* _tmp52_;
				gboolean _tmp53_;
				m = _vala_code_node_ref0 (VALA_METHOD (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
				params = vala_method_get_parameters (m);
				creation_call = NULL;
				vala_ccode_base_module_generate_method_declaration (self, m, self->source_declarations);
				cl = _vala_code_node_ref0 ((_tmp16_ = vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)), VALA_IS_CLASS (_tmp16_) ? ((ValaClass*) _tmp16_) : NULL));
				if (!vala_method_get_has_new_function (m)) {
					ValaCCodeFunctionCall* _tmp19_;
					ValaCCodeIdentifier* _tmp18_;
					char* _tmp17_;
					ValaCCodeIdentifier* _tmp21_;
					char* _tmp20_;
					creation_call = (_tmp19_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_identifier_new (_tmp17_ = vala_method_get_real_cname (m)))), _vala_ccode_node_unref0 (creation_call), _tmp19_);
					_vala_ccode_node_unref0 (_tmp18_);
					_g_free0 (_tmp17_);
					vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new (_tmp20_ = vala_typesymbol_get_type_id ((ValaTypeSymbol*) cl))));
					_vala_ccode_node_unref0 (_tmp21_);
					_g_free0 (_tmp20_);
				} else {
					ValaCCodeFunctionCall* _tmp24_;
					ValaCCodeIdentifier* _tmp23_;
					char* _tmp22_;
					creation_call = (_tmp24_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new (_tmp22_ = vala_method_get_cname (m)))), _vala_ccode_node_unref0 (creation_call), _tmp24_);
					_vala_ccode_node_unref0 (_tmp23_);
					_g_free0 (_tmp22_);
				}
				_tmp25_ = FALSE;
				_tmp26_ = FALSE;
				if (st != NULL) {
					_tmp26_ = !vala_struct_is_simple_type (st);
				} else {
					_tmp26_ = FALSE;
				}
				if (_tmp26_) {
					_tmp25_ = !(vala_method_get_cinstance_parameter_position (m) < 0);
				} else {
					_tmp25_ = FALSE;
				}
				if (_tmp25_) {
					ValaCCodeUnaryExpression* _tmp27_;
					vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp27_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, instance)));
					_vala_ccode_node_unref0 (_tmp27_);
				}
				vala_ccode_base_module_generate_type_declaration (self, vala_object_creation_expression_get_type_reference (expr), self->source_declarations);
				_tmp28_ = FALSE;
				if (cl != NULL) {
					_tmp28_ = !vala_class_get_is_compact (cl);
				} else {
					_tmp28_ = FALSE;
				}
				if (_tmp28_) {
					GeeList* _tmp29_;
					vala_ccode_base_module_add_generic_type_arguments (self, creation_call, _tmp29_ = vala_data_type_get_type_arguments (vala_object_creation_expression_get_type_reference (expr)), (ValaCodeNode*) expr, FALSE);
					_gee_collection_object_unref0 (_tmp29_);
				}
				carg_map = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_EXPRESSION, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal);
				ellipsis = FALSE;
				i = 1;
				arg_pos = 0;
				params_it = gee_iterable_iterator ((GeeIterable*) params);
				{
					GeeList* _tmp30_;
					GeeIterator* _tmp31_;
					GeeIterator* _arg_it;
					_arg_it = (_tmp31_ = gee_iterable_iterator ((GeeIterable*) (_tmp30_ = vala_object_creation_expression_get_argument_list (expr))), _gee_collection_object_unref0 (_tmp30_), _tmp31_);
					while (TRUE) {
						ValaExpression* arg;
						ValaCCodeExpression* cexpr;
						ValaFormalParameter* param;
						if (!gee_iterator_next (_arg_it)) {
							break;
						}
						arg = (ValaExpression*) gee_iterator_get (_arg_it);
						cexpr = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) arg)));
						param = NULL;
						if (gee_iterator_next (params_it)) {
							ValaFormalParameter* _tmp32_;
							param = (_tmp32_ = (ValaFormalParameter*) gee_iterator_get (params_it), _vala_code_node_unref0 (param), _tmp32_);
							ellipsis = vala_formal_parameter_get_ellipsis (param);
							if (!ellipsis) {
								gboolean _tmp33_;
								ValaCCodeExpression* _tmp36_;
								_tmp33_ = FALSE;
								if (!vala_formal_parameter_get_no_array_length (param)) {
									_tmp33_ = VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param));
								} else {
									_tmp33_ = FALSE;
								}
								if (_tmp33_) {
									ValaArrayType* array_type;
									array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)));
									{
										gint dim;
										dim = 1;
										{
											gboolean _tmp34_;
											_tmp34_ = TRUE;
											while (TRUE) {
												ValaCCodeExpression* _tmp35_;
												if (!_tmp34_) {
													dim++;
												}
												_tmp34_ = FALSE;
												if (!(dim <= vala_array_type_get_rank (array_type))) {
													break;
												}
												gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos (self, vala_formal_parameter_get_carray_length_parameter_position (param) + (0.01 * dim), FALSE)), _tmp35_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), arg, dim));
												_vala_ccode_node_unref0 (_tmp35_);
											}
										}
									}
									_vala_code_node_unref0 (array_type);
								} else {
									if (VALA_IS_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param))) {
										ValaDelegateType* deleg_type;
										ValaDelegate* d;
										deleg_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param)));
										d = _vala_code_node_ref0 (vala_delegate_type_get_delegate_symbol (deleg_type));
										if (vala_delegate_get_has_target (d)) {
											ValaCCodeExpression* delegate_target;
											delegate_target = vala_ccode_base_module_get_delegate_target_cexpression (self, arg);
											gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos (self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), delegate_target);
											_vala_ccode_node_unref0 (delegate_target);
										}
										_vala_code_node_unref0 (deleg_type);
										_vala_code_node_unref0 (d);
									}
								}
								cexpr = (_tmp36_ = vala_ccode_base_module_handle_struct_argument (self, param, arg, cexpr), _vala_ccode_node_unref0 (cexpr), _tmp36_);
							}
							arg_pos = vala_ccode_base_module_get_param_pos (self, vala_formal_parameter_get_cparameter_position (param), ellipsis);
						} else {
							arg_pos = vala_ccode_base_module_get_param_pos (self, (double) i, ellipsis);
						}
						gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (arg_pos), cexpr);
						i++;
						_vala_code_node_unref0 (arg);
						_vala_ccode_node_unref0 (cexpr);
						_vala_code_node_unref0 (param);
					}
					_gee_collection_object_unref0 (_arg_it);
				}
				while (TRUE) {
					ValaFormalParameter* param;
					if (!gee_iterator_next (params_it)) {
						break;
					}
					param = (ValaFormalParameter*) gee_iterator_get (params_it);
					if (vala_formal_parameter_get_ellipsis (param)) {
						ellipsis = TRUE;
						_vala_code_node_unref0 (param);
						break;
					}
					if (vala_formal_parameter_get_default_expression (param) == NULL) {
						char* _tmp37_;
						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), _tmp37_ = g_strdup_printf ("no default expression for argument %d", i));
						_g_free0 (_tmp37_);
						_vala_code_node_unref0 (param);
						_vala_code_node_unref0 (m);
						_gee_collection_object_unref0 (params);
						_vala_ccode_node_unref0 (creation_call);
						_vala_code_node_unref0 (cl);
						_gee_collection_object_unref0 (carg_map);
						_gee_collection_object_unref0 (params_it);
						_vala_ccode_node_unref0 (instance);
						_vala_ccode_node_unref0 (creation_expr);
						_vala_code_node_unref0 (st);
						return;
					}
					vala_code_node_accept ((ValaCodeNode*) vala_formal_parameter_get_default_expression (param), (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
					gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos (self, vala_formal_parameter_get_cparameter_position (param), FALSE)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_formal_parameter_get_default_expression (param))));
					i++;
					_vala_code_node_unref0 (param);
				}
				last_pos = -1;
				min_pos = 0;
				while (TRUE) {
					ValaCCodeExpression* _tmp42_;
					min_pos = -1;
					{
						GeeSet* _tmp38_;
						GeeIterator* _tmp39_;
						GeeIterator* _pos_it;
						_pos_it = (_tmp39_ = gee_iterable_iterator ((GeeIterable*) (_tmp38_ = gee_map_get_keys ((GeeMap*) carg_map))), _gee_collection_object_unref0 (_tmp38_), _tmp39_);
						while (TRUE) {
							gint pos;
							gboolean _tmp40_;
							if (!gee_iterator_next (_pos_it)) {
								break;
							}
							pos = GPOINTER_TO_INT (gee_iterator_get (_pos_it));
							_tmp40_ = FALSE;
							if (pos > last_pos) {
								gboolean _tmp41_;
								_tmp41_ = FALSE;
								if (min_pos == (-1)) {
									_tmp41_ = TRUE;
								} else {
									_tmp41_ = pos < min_pos;
								}
								_tmp40_ = _tmp41_;
							} else {
								_tmp40_ = FALSE;
							}
							if (_tmp40_) {
								min_pos = pos;
							}
						}
						_gee_collection_object_unref0 (_pos_it);
					}
					if (min_pos == (-1)) {
						break;
					}
					vala_ccode_function_call_add_argument (creation_call, _tmp42_ = (ValaCCodeExpression*) gee_map_get ((GeeMap*) carg_map, GINT_TO_POINTER (min_pos)));
					_vala_ccode_node_unref0 (_tmp42_);
					last_pos = min_pos;
				}
				_tmp43_ = FALSE;
				_tmp44_ = FALSE;
				if (st != NULL) {
					_tmp44_ = !vala_struct_is_simple_type (st);
				} else {
					_tmp44_ = FALSE;
				}
				if (_tmp44_) {
					_tmp43_ = vala_method_get_cinstance_parameter_position (m) < 0;
				} else {
					_tmp43_ = FALSE;
				}
				if (_tmp43_) {
					ValaCCodeUnaryExpression* _tmp45_;
					vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp45_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, instance)));
					_vala_ccode_node_unref0 (_tmp45_);
				}
				if (vala_code_node_get_tree_can_fail ((ValaCodeNode*) expr)) {
					ValaCCodeUnaryExpression* _tmp47_;
					ValaCCodeIdentifier* _tmp46_;
					self->current_method_inner_error = TRUE;
					vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp46_ = vala_ccode_identifier_new ("_inner_error_")))));
					_vala_ccode_node_unref0 (_tmp47_);
					_vala_ccode_node_unref0 (_tmp46_);
				}
				if (ellipsis) {
					gboolean _tmp48_;
					gboolean _tmp49_;
					_tmp48_ = FALSE;
					_tmp49_ = FALSE;
					if (!vala_method_get_printf_format (m)) {
						_tmp49_ = !vala_method_get_scanf_format (m);
					} else {
						_tmp49_ = FALSE;
					}
					if (_tmp49_) {
						_tmp48_ = _vala_strcmp0 (vala_method_get_sentinel (m), "") != 0;
					} else {
						_tmp48_ = FALSE;
					}
					if (_tmp48_) {
						ValaCCodeConstant* _tmp50_;
						vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp50_ = vala_ccode_constant_new (vala_method_get_sentinel (m))));
						_vala_ccode_node_unref0 (_tmp50_);
					}
				}
				creation_expr = (_tmp51_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) creation_call), _vala_ccode_node_unref0 (creation_expr), _tmp51_);
				if ((_tmp53_ = (_tmp52_ = vala_ccode_module_get_custom_creturn_type (vala_ccode_module_get_head ((ValaCCodeModule*) self), m)) != NULL, _g_free0 (_tmp52_), _tmp53_)) {
					ValaCCodeExpression* _tmp55_;
					char* _tmp54_;
					creation_expr = (_tmp55_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (creation_expr, _tmp54_ = vala_data_type_get_cname (vala_object_creation_expression_get_type_reference (expr))), _vala_ccode_node_unref0 (creation_expr), _tmp55_);
					_g_free0 (_tmp54_);
				}
				_vala_code_node_unref0 (m);
				_gee_collection_object_unref0 (params);
				_vala_ccode_node_unref0 (creation_call);
				_vala_code_node_unref0 (cl);
				_gee_collection_object_unref0 (carg_map);
				_gee_collection_object_unref0 (params_it);
			} else {
				if (VALA_IS_ERROR_CODE (vala_expression_get_symbol_reference ((ValaExpression*) expr))) {
					ValaErrorCode* ecode;
					ValaErrorDomain* edomain;
					ValaCCodeFunctionCall* creation_call;
					GeeList* _tmp56_;
					gboolean _tmp57_;
					ValaCCodeIdentifier* _tmp63_;
					char* _tmp62_;
					ValaCCodeIdentifier* _tmp65_;
					char* _tmp64_;
					ValaCCodeExpression* _tmp68_;
					ecode = _vala_code_node_ref0 (VALA_ERROR_CODE (vala_expression_get_symbol_reference ((ValaExpression*) expr)));
					edomain = _vala_code_node_ref0 (VALA_ERROR_DOMAIN (vala_symbol_get_parent_symbol ((ValaSymbol*) ecode)));
					creation_call = NULL;
					vala_ccode_base_module_generate_error_domain_declaration (self, edomain, self->source_declarations);
					if ((_tmp57_ = gee_collection_get_size ((GeeCollection*) (_tmp56_ = vala_object_creation_expression_get_argument_list (expr))) == 1, _gee_collection_object_unref0 (_tmp56_), _tmp57_)) {
						ValaCCodeFunctionCall* _tmp59_;
						ValaCCodeIdentifier* _tmp58_;
						creation_call = (_tmp59_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp58_ = vala_ccode_identifier_new ("g_error_new_literal"))), _vala_ccode_node_unref0 (creation_call), _tmp59_);
						_vala_ccode_node_unref0 (_tmp58_);
					} else {
						ValaCCodeFunctionCall* _tmp61_;
						ValaCCodeIdentifier* _tmp60_;
						creation_call = (_tmp61_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new ("g_error_new"))), _vala_ccode_node_unref0 (creation_call), _tmp61_);
						_vala_ccode_node_unref0 (_tmp60_);
					}
					vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp63_ = vala_ccode_identifier_new (_tmp62_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) edomain, NULL))));
					_vala_ccode_node_unref0 (_tmp63_);
					_g_free0 (_tmp62_);
					vala_ccode_function_call_add_argument (creation_call, (ValaCCodeExpression*) (_tmp65_ = vala_ccode_identifier_new (_tmp64_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) ecode, FALSE))));
					_vala_ccode_node_unref0 (_tmp65_);
					_g_free0 (_tmp64_);
					{
						GeeList* _tmp66_;
						GeeIterator* _tmp67_;
						GeeIterator* _arg_it;
						_arg_it = (_tmp67_ = gee_iterable_iterator ((GeeIterable*) (_tmp66_ = vala_object_creation_expression_get_argument_list (expr))), _gee_collection_object_unref0 (_tmp66_), _tmp67_);
						while (TRUE) {
							ValaExpression* arg;
							if (!gee_iterator_next (_arg_it)) {
								break;
							}
							arg = (ValaExpression*) gee_iterator_get (_arg_it);
							vala_ccode_function_call_add_argument (creation_call, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) arg)));
							_vala_code_node_unref0 (arg);
						}
						_gee_collection_object_unref0 (_arg_it);
					}
					creation_expr = (_tmp68_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) creation_call), _vala_ccode_node_unref0 (creation_expr), _tmp68_);
					_vala_code_node_unref0 (ecode);
					_vala_code_node_unref0 (edomain);
					_vala_ccode_node_unref0 (creation_call);
				} else {
					g_assert (FALSE);
				}
			}
		}
	}
	if (instance != NULL) {
		ValaCCodeCommaExpression* ccomma;
		ccomma = vala_ccode_comma_expression_new ();
		if (VALA_IS_STRUCT (vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)))) {
			vala_ccode_comma_expression_append_expression (ccomma, creation_expr);
		} else {
			ValaCCodeAssignment* _tmp69_;
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp69_ = vala_ccode_assignment_new (instance, creation_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
			_vala_ccode_node_unref0 (_tmp69_);
		}
		{
			GeeList* _tmp70_;
			GeeIterator* _tmp71_;
			GeeIterator* _init_it;
			_init_it = (_tmp71_ = gee_iterable_iterator ((GeeIterable*) (_tmp70_ = vala_object_creation_expression_get_object_initializer (expr))), _gee_collection_object_unref0 (_tmp70_), _tmp71_);
			while (TRUE) {
				ValaMemberInitializer* init;
				if (!gee_iterator_next (_init_it)) {
					break;
				}
				init = (ValaMemberInitializer*) gee_iterator_get (_init_it);
				if (VALA_IS_FIELD (vala_member_initializer_get_symbol_reference (init))) {
					ValaField* f;
					ValaDataType* instance_target_type;
					ValaCCodeExpression* typed_inst;
					ValaCCodeExpression* lhs;
					ValaCCodeAssignment* _tmp76_;
					gboolean _tmp77_;
					f = _vala_code_node_ref0 (VALA_FIELD (vala_member_initializer_get_symbol_reference (init)));
					instance_target_type = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) f)));
					typed_inst = vala_ccode_base_module_transform_expression (self, instance, vala_object_creation_expression_get_type_reference (expr), instance_target_type, NULL);
					lhs = NULL;
					if (VALA_IS_STRUCT (vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)))) {
						ValaCCodeExpression* _tmp73_;
						char* _tmp72_;
						lhs = (_tmp73_ = (ValaCCodeExpression*) vala_ccode_member_access_new (typed_inst, _tmp72_ = vala_field_get_cname (f), FALSE), _vala_ccode_node_unref0 (lhs), _tmp73_);
						_g_free0 (_tmp72_);
					} else {
						ValaCCodeExpression* _tmp75_;
						char* _tmp74_;
						lhs = (_tmp75_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (typed_inst, _tmp74_ = vala_field_get_cname (f)), _vala_ccode_node_unref0 (lhs), _tmp75_);
						_g_free0 (_tmp74_);
					}
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp76_ = vala_ccode_assignment_new (lhs, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_initializer_get_initializer (init))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp76_);
					_tmp77_ = FALSE;
					if (VALA_IS_ARRAY_TYPE (vala_field_get_field_type (f))) {
						_tmp77_ = !vala_field_get_no_array_length (f);
					} else {
						_tmp77_ = FALSE;
					}
					if (_tmp77_) {
						ValaArrayType* array_type;
						array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_field_get_field_type (f)));
						{
							gint dim;
							dim = 1;
							{
								gboolean _tmp78_;
								_tmp78_ = TRUE;
								while (TRUE) {
									ValaCCodeExpression* rhs_array_len;
									ValaCCodeAssignment* _tmp85_;
									if (!_tmp78_) {
										dim++;
									}
									_tmp78_ = FALSE;
									if (!(dim <= vala_array_type_get_rank (array_type))) {
										break;
									}
									if (VALA_IS_STRUCT (vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)))) {
										ValaCCodeExpression* _tmp81_;
										char* _tmp80_;
										char* _tmp79_;
										lhs = (_tmp81_ = (ValaCCodeExpression*) vala_ccode_member_access_new (typed_inst, _tmp80_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp79_ = vala_field_get_cname (f), dim), FALSE), _vala_ccode_node_unref0 (lhs), _tmp81_);
										_g_free0 (_tmp80_);
										_g_free0 (_tmp79_);
									} else {
										ValaCCodeExpression* _tmp84_;
										char* _tmp83_;
										char* _tmp82_;
										lhs = (_tmp84_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (typed_inst, _tmp83_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp82_ = vala_field_get_cname (f), dim)), _vala_ccode_node_unref0 (lhs), _tmp84_);
										_g_free0 (_tmp83_);
										_g_free0 (_tmp82_);
									}
									rhs_array_len = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_member_initializer_get_initializer (init), dim);
									vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp85_ = vala_ccode_assignment_new (lhs, rhs_array_len, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
									_vala_ccode_node_unref0 (_tmp85_);
									_vala_ccode_node_unref0 (rhs_array_len);
								}
							}
						}
						_vala_code_node_unref0 (array_type);
					} else {
						gboolean _tmp86_;
						_tmp86_ = FALSE;
						if (VALA_IS_DELEGATE_TYPE (vala_field_get_field_type (f))) {
							_tmp86_ = !vala_field_get_no_delegate_target (f);
						} else {
							_tmp86_ = FALSE;
						}
						if (_tmp86_) {
							ValaCCodeExpression* rhs_delegate_target;
							ValaCCodeAssignment* _tmp93_;
							if (VALA_IS_STRUCT (vala_data_type_get_data_type (vala_object_creation_expression_get_type_reference (expr)))) {
								ValaCCodeExpression* _tmp89_;
								char* _tmp88_;
								char* _tmp87_;
								lhs = (_tmp89_ = (ValaCCodeExpression*) vala_ccode_member_access_new (typed_inst, _tmp88_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp87_ = vala_field_get_cname (f)), FALSE), _vala_ccode_node_unref0 (lhs), _tmp89_);
								_g_free0 (_tmp88_);
								_g_free0 (_tmp87_);
							} else {
								ValaCCodeExpression* _tmp92_;
								char* _tmp91_;
								char* _tmp90_;
								lhs = (_tmp92_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer (typed_inst, _tmp91_ = vala_ccode_base_module_get_delegate_target_cname (self, _tmp90_ = vala_field_get_cname (f))), _vala_ccode_node_unref0 (lhs), _tmp92_);
								_g_free0 (_tmp91_);
								_g_free0 (_tmp90_);
							}
							rhs_delegate_target = vala_ccode_base_module_get_delegate_target_cexpression (self, vala_member_initializer_get_initializer (init));
							vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_assignment_new (lhs, rhs_delegate_target, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							_vala_ccode_node_unref0 (_tmp93_);
							_vala_ccode_node_unref0 (rhs_delegate_target);
						}
					}
					_vala_code_node_unref0 (f);
					_vala_code_node_unref0 (instance_target_type);
					_vala_ccode_node_unref0 (typed_inst);
					_vala_ccode_node_unref0 (lhs);
				} else {
					if (VALA_IS_PROPERTY (vala_member_initializer_get_symbol_reference (init))) {
						ValaMemberAccess* inst_ma;
						ValaMemberAccess* ma;
						ValaCCodeFunctionCall* _tmp94_;
						inst_ma = vala_member_access_new_simple ("new", NULL);
						vala_expression_set_value_type ((ValaExpression*) inst_ma, vala_object_creation_expression_get_type_reference (expr));
						vala_code_node_set_ccodenode ((ValaCodeNode*) inst_ma, (ValaCCodeNode*) instance);
						ma = vala_member_access_new ((ValaExpression*) inst_ma, vala_member_initializer_get_name (init), NULL);
						vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp94_ = vala_ccode_base_module_get_property_set_call (self, VALA_PROPERTY (vala_member_initializer_get_symbol_reference (init)), ma, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_initializer_get_initializer (init))), NULL)));
						_vala_ccode_node_unref0 (_tmp94_);
						_vala_code_node_unref0 (inst_ma);
						_vala_code_node_unref0 (ma);
					}
				}
				_vala_code_node_unref0 (init);
			}
			_gee_collection_object_unref0 (_init_it);
		}
		vala_ccode_comma_expression_append_expression (ccomma, instance);
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
		_vala_ccode_node_unref0 (ccomma);
	} else {
		if (creation_expr != NULL) {
			vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) creation_expr);
		}
	}
	_vala_ccode_node_unref0 (instance);
	_vala_ccode_node_unref0 (creation_expr);
	_vala_code_node_unref0 (st);
}


ValaCCodeExpression* vala_ccode_base_module_handle_struct_argument (ValaCCodeBaseModule* self, ValaFormalParameter* param, ValaExpression* arg, ValaCCodeExpression* cexpr) {
	ValaCCodeExpression* result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (param != NULL, NULL);
	g_return_val_if_fail (arg != NULL, NULL);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	if (!VALA_IS_NULL_TYPE (vala_expression_get_value_type (arg))) {
		_tmp1_ = VALA_IS_STRUCT (vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param)));
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = !vala_struct_is_simple_type (VALA_STRUCT (vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param))));
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gboolean _tmp2_;
		_tmp2_ = FALSE;
		if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN) {
			_tmp2_ = !vala_data_type_get_nullable (vala_formal_parameter_get_parameter_type (param));
		} else {
			_tmp2_ = FALSE;
		}
		if (_tmp2_) {
			ValaCCodeExpression* _tmp3_;
			ValaCCodeUnaryExpression* unary;
			gboolean _tmp4_;
			unary = _vala_ccode_node_ref0 ((_tmp3_ = cexpr, VALA_IS_CCODE_UNARY_EXPRESSION (_tmp3_) ? ((ValaCCodeUnaryExpression*) _tmp3_) : NULL));
			_tmp4_ = FALSE;
			if (unary != NULL) {
				_tmp4_ = vala_ccode_unary_expression_get_operator (unary) == VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION;
			} else {
				_tmp4_ = FALSE;
			}
			if (_tmp4_) {
				result = _vala_ccode_node_ref0 (vala_ccode_unary_expression_get_inner (unary));
				_vala_ccode_node_unref0 (unary);
				return result;
			} else {
				gboolean _tmp5_;
				_tmp5_ = FALSE;
				if (VALA_IS_CCODE_IDENTIFIER (cexpr)) {
					_tmp5_ = TRUE;
				} else {
					_tmp5_ = VALA_IS_CCODE_MEMBER_ACCESS (cexpr);
				}
				if (_tmp5_) {
					result = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, cexpr);
					_vala_ccode_node_unref0 (unary);
					return result;
				} else {
					ValaCCodeCommaExpression* ccomma;
					ValaLocalVariable* temp_var;
					ValaCCodeAssignment* _tmp7_;
					ValaCCodeExpression* _tmp6_;
					ValaCCodeUnaryExpression* _tmp9_;
					ValaCCodeExpression* _tmp8_;
					ccomma = vala_ccode_comma_expression_new ();
					temp_var = vala_ccode_base_module_get_temp_variable (self, vala_formal_parameter_get_parameter_type (param), TRUE, NULL, FALSE);
					gee_list_insert ((GeeList*) self->temp_vars, 0, temp_var);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_assignment_new (_tmp6_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)), cexpr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp7_);
					_vala_ccode_node_unref0 (_tmp6_);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp8_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)))));
					_vala_ccode_node_unref0 (_tmp9_);
					_vala_ccode_node_unref0 (_tmp8_);
					result = (ValaCCodeExpression*) ccomma;
					_vala_code_node_unref0 (temp_var);
					_vala_ccode_node_unref0 (unary);
					return result;
				}
			}
			_vala_ccode_node_unref0 (unary);
		}
	}
	result = _vala_ccode_node_ref0 (cexpr);
	return result;
}


static void vala_ccode_base_module_real_visit_sizeof_expression (ValaCCodeModule* base, ValaSizeofExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* csizeof;
	ValaCCodeIdentifier* _tmp3_;
	char* _tmp2_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	csizeof = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (csizeof, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new (_tmp2_ = vala_data_type_get_cname (vala_sizeof_expression_get_type_reference (expr)))));
	_vala_ccode_node_unref0 (_tmp3_);
	_g_free0 (_tmp2_);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) csizeof);
	_vala_ccode_node_unref0 (csizeof);
}


static void vala_ccode_base_module_real_visit_typeof_expression (ValaCCodeModule* base, ValaTypeofExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeExpression* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_base_module_get_type_id_expression (self, vala_typeof_expression_get_type_reference (expr), FALSE)));
	_vala_ccode_node_unref0 (_tmp0_);
}


static void vala_ccode_base_module_real_visit_unary_expression (ValaCCodeModule* base, ValaUnaryExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeUnaryOperator op;
	ValaCCodeUnaryExpression* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	op = 0;
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS) {
		op = VALA_CCODE_UNARY_OPERATOR_PLUS;
	} else {
		if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
			op = VALA_CCODE_UNARY_OPERATOR_MINUS;
		} else {
			if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
				op = VALA_CCODE_UNARY_OPERATOR_LOGICAL_NEGATION;
			} else {
				if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
					op = VALA_CCODE_UNARY_OPERATOR_BITWISE_COMPLEMENT;
				} else {
					if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT) {
						op = VALA_CCODE_UNARY_OPERATOR_PREFIX_INCREMENT;
					} else {
						if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_DECREMENT) {
							op = VALA_CCODE_UNARY_OPERATOR_PREFIX_DECREMENT;
						} else {
							if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
								op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
							} else {
								if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
									op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
								} else {
									g_assert_not_reached ();
								}
							}
						}
					}
				}
			}
		}
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_unary_expression_new (op, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_unary_expression_get_inner (expr))))));
	_vala_ccode_node_unref0 (_tmp0_);
}


static void vala_ccode_base_module_real_visit_cast_expression (ValaCCodeModule* base, ValaCastExpression* expr) {
	ValaCCodeBaseModule * self;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	ValaTypeSymbol* _tmp9_;
	ValaClass* cl;
	ValaTypeSymbol* _tmp10_;
	ValaInterface* iface;
	gboolean _tmp11_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	if (vala_expression_get_value_type (vala_cast_expression_get_inner (expr)) != NULL) {
		_tmp2_ = self->gvalue_type != NULL;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		_tmp1_ = vala_data_type_get_data_type (vala_expression_get_value_type (vala_cast_expression_get_inner (expr))) == VALA_TYPESYMBOL (self->gvalue_type);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		char* _tmp3_;
		_tmp0_ = (_tmp3_ = vala_data_type_get_type_id (vala_cast_expression_get_type_reference (expr))) != NULL;
		_g_free0 (_tmp3_);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaCCodeIdentifier* _tmp4_;
		ValaCCodeFunctionCall* _tmp5_;
		ValaCCodeFunctionCall* ccall;
		ValaCCodeUnaryExpression* _tmp6_;
		ccall = (_tmp5_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_base_module_get_value_getter_function (self, vala_cast_expression_get_type_reference (expr)))), _vala_ccode_node_unref0 (_tmp4_), _tmp5_);
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_cast_expression_get_inner (expr))))));
		_vala_ccode_node_unref0 (_tmp6_);
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall);
		if (VALA_IS_ARRAY_TYPE (vala_cast_expression_get_type_reference (expr))) {
			ValaCCodeIdentifier* _tmp7_;
			ValaCCodeFunctionCall* _tmp8_;
			ValaCCodeFunctionCall* len_call;
			len_call = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("g_strv_length"))), _vala_ccode_node_unref0 (_tmp7_), _tmp8_);
			vala_ccode_function_call_add_argument (len_call, (ValaCCodeExpression*) ccall);
			vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) len_call);
			_vala_ccode_node_unref0 (len_call);
		}
		_vala_ccode_node_unref0 (ccall);
		return;
	}
	cl = _vala_code_node_ref0 ((_tmp9_ = vala_data_type_get_data_type (vala_cast_expression_get_type_reference (expr)), VALA_IS_CLASS (_tmp9_) ? ((ValaClass*) _tmp9_) : NULL));
	iface = _vala_code_node_ref0 ((_tmp10_ = vala_data_type_get_data_type (vala_cast_expression_get_type_reference (expr)), VALA_IS_INTERFACE (_tmp10_) ? ((ValaInterface*) _tmp10_) : NULL));
	_tmp11_ = FALSE;
	if (vala_code_context_get_profile (self->priv->_context) == VALA_PROFILE_GOBJECT) {
		gboolean _tmp12_;
		_tmp12_ = FALSE;
		if (iface != NULL) {
			_tmp12_ = TRUE;
		} else {
			gboolean _tmp13_;
			_tmp13_ = FALSE;
			if (cl != NULL) {
				_tmp13_ = !vala_class_get_is_compact (cl);
			} else {
				_tmp13_ = FALSE;
			}
			_tmp12_ = _tmp13_;
		}
		_tmp11_ = _tmp12_;
	} else {
		_tmp11_ = FALSE;
	}
	if (_tmp11_) {
		if (vala_cast_expression_get_is_silent_cast (expr)) {
			ValaCCodeCommaExpression* ccomma;
			ValaLocalVariable* temp_decl;
			ValaCCodeExpression* ctemp;
			ValaCCodeAssignment* cinit;
			ValaCCodeExpression* ccheck;
			char* _tmp14_;
			ValaCCodeCastExpression* _tmp15_;
			ValaCCodeCastExpression* ccast;
			ValaCCodeConstant* cnull;
			ValaCCodeConditionalExpression* _tmp16_;
			ccomma = vala_ccode_comma_expression_new ();
			temp_decl = vala_ccode_base_module_get_temp_variable (self, vala_expression_get_value_type (vala_cast_expression_get_inner (expr)), TRUE, (ValaCodeNode*) expr, FALSE);
			gee_collection_add ((GeeCollection*) self->temp_vars, temp_decl);
			ctemp = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_decl));
			cinit = vala_ccode_assignment_new (ctemp, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_cast_expression_get_inner (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
			ccheck = vala_ccode_base_module_create_type_check (self, (ValaCCodeNode*) ctemp, vala_cast_expression_get_type_reference (expr));
			ccast = (_tmp15_ = vala_ccode_cast_expression_new (ctemp, _tmp14_ = vala_data_type_get_cname (vala_cast_expression_get_type_reference (expr))), _g_free0 (_tmp14_), _tmp15_);
			cnull = vala_ccode_constant_new ("NULL");
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) cinit);
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_conditional_expression_new (ccheck, (ValaCCodeExpression*) ccast, (ValaCCodeExpression*) cnull)));
			_vala_ccode_node_unref0 (_tmp16_);
			vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
			_vala_ccode_node_unref0 (ccomma);
			_vala_code_node_unref0 (temp_decl);
			_vala_ccode_node_unref0 (ctemp);
			_vala_ccode_node_unref0 (cinit);
			_vala_ccode_node_unref0 (ccheck);
			_vala_ccode_node_unref0 (ccast);
			_vala_ccode_node_unref0 (cnull);
		} else {
			ValaCCodeFunctionCall* _tmp17_;
			vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp17_ = vala_ccode_base_module_generate_instance_cast (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_cast_expression_get_inner (expr))), vala_data_type_get_data_type (vala_cast_expression_get_type_reference (expr)))));
			_vala_ccode_node_unref0 (_tmp17_);
		}
	} else {
		ValaDataType* _tmp18_;
		ValaArrayType* array_type;
		gboolean _tmp19_;
		ValaCCodeCastExpression* _tmp23_;
		char* _tmp22_;
		if (vala_cast_expression_get_is_silent_cast (expr)) {
			vala_code_node_set_error ((ValaCodeNode*) expr, TRUE);
			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), "Operation not supported for this type");
			_vala_code_node_unref0 (cl);
			_vala_code_node_unref0 (iface);
			return;
		}
		array_type = _vala_code_node_ref0 ((_tmp18_ = vala_cast_expression_get_type_reference (expr), VALA_IS_ARRAY_TYPE (_tmp18_) ? ((ValaArrayType*) _tmp18_) : NULL));
		_tmp19_ = FALSE;
		if (array_type != NULL) {
			_tmp19_ = VALA_IS_ARRAY_TYPE (vala_expression_get_value_type (vala_cast_expression_get_inner (expr)));
		} else {
			_tmp19_ = FALSE;
		}
		if (_tmp19_) {
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp20_;
					_tmp20_ = TRUE;
					while (TRUE) {
						ValaCCodeExpression* _tmp21_;
						if (!_tmp20_) {
							dim++;
						}
						_tmp20_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						vala_expression_append_array_size ((ValaExpression*) expr, _tmp21_ = vala_ccode_module_get_array_length_cexpression ((ValaCCodeModule*) self, vala_cast_expression_get_inner (expr), dim));
						_vala_ccode_node_unref0 (_tmp21_);
					}
				}
			}
		}
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp23_ = vala_ccode_cast_expression_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_cast_expression_get_inner (expr))), _tmp22_ = vala_data_type_get_cname (vala_cast_expression_get_type_reference (expr)))));
		_vala_ccode_node_unref0 (_tmp23_);
		_g_free0 (_tmp22_);
		_vala_code_node_unref0 (array_type);
	}
	_vala_code_node_unref0 (cl);
	_vala_code_node_unref0 (iface);
}


static void vala_ccode_base_module_real_visit_pointer_indirection (ValaCCodeModule* base, ValaPointerIndirection* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeUnaryExpression* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_pointer_indirection_get_inner (expr))))));
	_vala_ccode_node_unref0 (_tmp0_);
}


static void vala_ccode_base_module_real_visit_addressof_expression (ValaCCodeModule* base, ValaAddressofExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeUnaryExpression* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_addressof_expression_get_inner (expr))))));
	_vala_ccode_node_unref0 (_tmp0_);
}


static void vala_ccode_base_module_real_visit_reference_transfer_expression (ValaCCodeModule* base, ValaReferenceTransferExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeCommaExpression* ccomma;
	ValaLocalVariable* temp_decl;
	ValaCCodeExpression* cvar;
	ValaCCodeAssignment* _tmp0_;
	ValaCCodeAssignment* _tmp2_;
	ValaCCodeConstant* _tmp1_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	ccomma = vala_ccode_comma_expression_new ();
	temp_decl = vala_ccode_base_module_get_temp_variable (self, vala_expression_get_value_type ((ValaExpression*) expr), TRUE, (ValaCodeNode*) expr, FALSE);
	gee_list_insert ((GeeList*) self->temp_vars, 0, temp_decl);
	cvar = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_decl));
	vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp0_ = vala_ccode_assignment_new (cvar, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_reference_transfer_expression_get_inner (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
	_vala_ccode_node_unref0 (_tmp0_);
	vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_reference_transfer_expression_get_inner (expr))), (ValaCCodeExpression*) (_tmp1_ = vala_ccode_constant_new ("NULL")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
	_vala_ccode_node_unref0 (_tmp2_);
	_vala_ccode_node_unref0 (_tmp1_);
	vala_ccode_comma_expression_append_expression (ccomma, cvar);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
	_vala_ccode_node_unref0 (ccomma);
	_vala_code_node_unref0 (temp_decl);
	_vala_ccode_node_unref0 (cvar);
}


static void vala_ccode_base_module_real_visit_binary_expression (ValaCCodeModule* base, ValaBinaryExpression* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeExpression* cleft;
	ValaCCodeExpression* cright;
	ValaCCodeBinaryOperator op;
	gboolean _tmp2_;
	gboolean _tmp14_;
	gboolean _tmp15_;
	gboolean _tmp16_;
	ValaCCodeBinaryExpression* _tmp36_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	cleft = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_binary_expression_get_left (expr))));
	cright = _vala_ccode_node_ref0 (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_binary_expression_get_right (expr))));
	op = 0;
	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
		op = VALA_CCODE_BINARY_OPERATOR_PLUS;
	} else {
		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MINUS) {
			op = VALA_CCODE_BINARY_OPERATOR_MINUS;
		} else {
			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MUL) {
				op = VALA_CCODE_BINARY_OPERATOR_MUL;
			} else {
				if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_DIV) {
					op = VALA_CCODE_BINARY_OPERATOR_DIV;
				} else {
					if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MOD) {
						op = VALA_CCODE_BINARY_OPERATOR_MOD;
					} else {
						if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_LEFT) {
							op = VALA_CCODE_BINARY_OPERATOR_SHIFT_LEFT;
						} else {
							if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_RIGHT) {
								op = VALA_CCODE_BINARY_OPERATOR_SHIFT_RIGHT;
							} else {
								if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN) {
									op = VALA_CCODE_BINARY_OPERATOR_LESS_THAN;
								} else {
									if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN) {
										op = VALA_CCODE_BINARY_OPERATOR_GREATER_THAN;
									} else {
										if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL) {
											op = VALA_CCODE_BINARY_OPERATOR_LESS_THAN_OR_EQUAL;
										} else {
											if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL) {
												op = VALA_CCODE_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL;
											} else {
												if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY) {
													op = VALA_CCODE_BINARY_OPERATOR_EQUALITY;
												} else {
													if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
														op = VALA_CCODE_BINARY_OPERATOR_INEQUALITY;
													} else {
														if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_AND) {
															op = VALA_CCODE_BINARY_OPERATOR_BITWISE_AND;
														} else {
															if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_OR) {
																op = VALA_CCODE_BINARY_OPERATOR_BITWISE_OR;
															} else {
																if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_XOR) {
																	op = VALA_CCODE_BINARY_OPERATOR_BITWISE_XOR;
																} else {
																	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_AND) {
																		op = VALA_CCODE_BINARY_OPERATOR_AND;
																	} else {
																		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_OR) {
																			op = VALA_CCODE_BINARY_OPERATOR_OR;
																		} else {
																			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_IN) {
																				ValaCCodeBinaryExpression* _tmp1_;
																				ValaCCodeBinaryExpression* _tmp0_;
																				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp1_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp0_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_BITWISE_AND, cright, cleft)), cleft)));
																				_vala_ccode_node_unref0 (_tmp1_);
																				_vala_ccode_node_unref0 (_tmp0_);
																				_vala_ccode_node_unref0 (cleft);
																				_vala_ccode_node_unref0 (cright);
																				return;
																			} else {
																				g_assert_not_reached ();
																			}
																		}
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	_tmp2_ = FALSE;
	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY;
	}
	if (_tmp2_) {
		ValaTypeSymbol* _tmp3_;
		ValaStruct* left_type_as_struct;
		ValaTypeSymbol* _tmp4_;
		ValaStruct* right_type_as_struct;
		gboolean _tmp5_;
		gboolean _tmp6_;
		gboolean _tmp7_;
		left_type_as_struct = _vala_code_node_ref0 ((_tmp3_ = vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_left (expr))), VALA_IS_STRUCT (_tmp3_) ? ((ValaStruct*) _tmp3_) : NULL));
		right_type_as_struct = _vala_code_node_ref0 ((_tmp4_ = vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_right (expr))), VALA_IS_STRUCT (_tmp4_) ? ((ValaStruct*) _tmp4_) : NULL));
		_tmp5_ = FALSE;
		_tmp6_ = FALSE;
		_tmp7_ = FALSE;
		if (VALA_IS_CLASS (vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_left (expr))))) {
			_tmp7_ = !vala_class_get_is_compact (VALA_CLASS (vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_left (expr)))));
		} else {
			_tmp7_ = FALSE;
		}
		if (_tmp7_) {
			_tmp6_ = VALA_IS_CLASS (vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_right (expr))));
		} else {
			_tmp6_ = FALSE;
		}
		if (_tmp6_) {
			_tmp5_ = !vala_class_get_is_compact (VALA_CLASS (vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_right (expr)))));
		} else {
			_tmp5_ = FALSE;
		}
		if (_tmp5_) {
			ValaClass* left_cl;
			ValaClass* right_cl;
			left_cl = _vala_code_node_ref0 (VALA_CLASS (vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_left (expr)))));
			right_cl = _vala_code_node_ref0 (VALA_CLASS (vala_data_type_get_data_type (vala_expression_get_value_type (vala_binary_expression_get_right (expr)))));
			if (left_cl != right_cl) {
				if (vala_typesymbol_is_subtype_of ((ValaTypeSymbol*) left_cl, (ValaTypeSymbol*) right_cl)) {
					ValaCCodeExpression* _tmp8_;
					cleft = (_tmp8_ = (ValaCCodeExpression*) vala_ccode_base_module_generate_instance_cast (self, cleft, (ValaTypeSymbol*) right_cl), _vala_ccode_node_unref0 (cleft), _tmp8_);
				} else {
					if (vala_typesymbol_is_subtype_of ((ValaTypeSymbol*) right_cl, (ValaTypeSymbol*) left_cl)) {
						ValaCCodeExpression* _tmp9_;
						cright = (_tmp9_ = (ValaCCodeExpression*) vala_ccode_base_module_generate_instance_cast (self, cright, (ValaTypeSymbol*) left_cl), _vala_ccode_node_unref0 (cright), _tmp9_);
					}
				}
			}
			_vala_code_node_unref0 (left_cl);
			_vala_code_node_unref0 (right_cl);
		} else {
			gboolean _tmp10_;
			_tmp10_ = FALSE;
			if (left_type_as_struct != NULL) {
				_tmp10_ = right_type_as_struct != NULL;
			} else {
				_tmp10_ = FALSE;
			}
			if (_tmp10_) {
				gboolean _tmp11_;
				_tmp11_ = FALSE;
				if (vala_data_type_get_nullable (vala_expression_get_value_type (vala_binary_expression_get_left (expr)))) {
					_tmp11_ = vala_data_type_get_nullable (vala_expression_get_value_type (vala_binary_expression_get_right (expr)));
				} else {
					_tmp11_ = FALSE;
				}
				if (_tmp11_) {
				} else {
					if (vala_data_type_get_nullable (vala_expression_get_value_type (vala_binary_expression_get_left (expr)))) {
						ValaCCodeExpression* _tmp12_;
						cleft = (_tmp12_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, cleft), _vala_ccode_node_unref0 (cleft), _tmp12_);
					} else {
						if (vala_data_type_get_nullable (vala_expression_get_value_type (vala_binary_expression_get_right (expr)))) {
							ValaCCodeExpression* _tmp13_;
							cright = (_tmp13_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, cright), _vala_ccode_node_unref0 (cright), _tmp13_);
						}
					}
				}
			}
		}
		_vala_code_node_unref0 (left_type_as_struct);
		_vala_code_node_unref0 (right_type_as_struct);
	}
	_tmp14_ = FALSE;
	_tmp15_ = FALSE;
	_tmp16_ = FALSE;
	if (!VALA_IS_NULL_TYPE (vala_expression_get_value_type (vala_binary_expression_get_left (expr)))) {
		_tmp16_ = vala_data_type_compatible (vala_expression_get_value_type (vala_binary_expression_get_left (expr)), self->string_type);
	} else {
		_tmp16_ = FALSE;
	}
	if (_tmp16_) {
		_tmp15_ = !VALA_IS_NULL_TYPE (vala_expression_get_value_type (vala_binary_expression_get_right (expr)));
	} else {
		_tmp15_ = FALSE;
	}
	if (_tmp15_) {
		_tmp14_ = vala_data_type_compatible (vala_expression_get_value_type (vala_binary_expression_get_right (expr)), self->string_type);
	} else {
		_tmp14_ = FALSE;
	}
	if (_tmp14_) {
		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
			gboolean _tmp17_;
			_tmp17_ = FALSE;
			if (vala_expression_is_constant (vala_binary_expression_get_left (expr))) {
				_tmp17_ = vala_expression_is_constant (vala_binary_expression_get_right (expr));
			} else {
				_tmp17_ = FALSE;
			}
			if (_tmp17_) {
				char* left;
				char* right;
				ValaCCodeConstant* _tmp23_;
				char* _tmp22_;
				left = NULL;
				right = NULL;
				if (VALA_IS_CCODE_IDENTIFIER (cleft)) {
					char* _tmp18_;
					left = (_tmp18_ = g_strdup (vala_ccode_identifier_get_name (VALA_CCODE_IDENTIFIER (cleft))), _g_free0 (left), _tmp18_);
				} else {
					if (VALA_IS_CCODE_CONSTANT (cleft)) {
						char* _tmp19_;
						left = (_tmp19_ = g_strdup (vala_ccode_constant_get_name (VALA_CCODE_CONSTANT (cleft))), _g_free0 (left), _tmp19_);
					} else {
						g_assert_not_reached ();
					}
				}
				if (VALA_IS_CCODE_IDENTIFIER (cright)) {
					char* _tmp20_;
					right = (_tmp20_ = g_strdup (vala_ccode_identifier_get_name (VALA_CCODE_IDENTIFIER (cright))), _g_free0 (right), _tmp20_);
				} else {
					if (VALA_IS_CCODE_CONSTANT (cright)) {
						char* _tmp21_;
						right = (_tmp21_ = g_strdup (vala_ccode_constant_get_name (VALA_CCODE_CONSTANT (cright))), _g_free0 (right), _tmp21_);
					} else {
						g_assert_not_reached ();
					}
				}
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp23_ = vala_ccode_constant_new (_tmp22_ = g_strdup_printf ("%s %s", left, right))));
				_vala_ccode_node_unref0 (_tmp23_);
				_g_free0 (_tmp22_);
				_g_free0 (left);
				_g_free0 (right);
				_vala_ccode_node_unref0 (cleft);
				_vala_ccode_node_unref0 (cright);
				return;
			} else {
				ValaCCodeIdentifier* _tmp24_;
				ValaCCodeFunctionCall* _tmp25_;
				ValaCCodeFunctionCall* ccall;
				ValaCCodeConstant* _tmp26_;
				ccall = (_tmp25_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("g_strconcat"))), _vala_ccode_node_unref0 (_tmp24_), _tmp25_);
				vala_ccode_function_call_add_argument (ccall, cleft);
				vala_ccode_function_call_add_argument (ccall, cright);
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_constant_new ("NULL")));
				_vala_ccode_node_unref0 (_tmp26_);
				vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall);
				_vala_ccode_node_unref0 (ccall);
				_vala_ccode_node_unref0 (cleft);
				_vala_ccode_node_unref0 (cright);
				return;
			}
		} else {
			gboolean _tmp27_;
			gboolean _tmp28_;
			gboolean _tmp29_;
			gboolean _tmp30_;
			gboolean _tmp31_;
			_tmp27_ = FALSE;
			_tmp28_ = FALSE;
			_tmp29_ = FALSE;
			_tmp30_ = FALSE;
			_tmp31_ = FALSE;
			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY) {
				_tmp31_ = TRUE;
			} else {
				_tmp31_ = vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY;
			}
			if (_tmp31_) {
				_tmp30_ = TRUE;
			} else {
				_tmp30_ = vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN;
			}
			if (_tmp30_) {
				_tmp29_ = TRUE;
			} else {
				_tmp29_ = vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN;
			}
			if (_tmp29_) {
				_tmp28_ = TRUE;
			} else {
				_tmp28_ = vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL;
			}
			if (_tmp28_) {
				_tmp27_ = TRUE;
			} else {
				_tmp27_ = vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL;
			}
			if (_tmp27_) {
				ValaCCodeIdentifier* _tmp32_;
				ValaCCodeFunctionCall* _tmp33_;
				ValaCCodeFunctionCall* ccall;
				ValaCCodeExpression* _tmp34_;
				ValaCCodeExpression* _tmp35_;
				self->requires_strcmp0 = TRUE;
				ccall = (_tmp33_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp32_ = vala_ccode_identifier_new ("_vala_strcmp0"))), _vala_ccode_node_unref0 (_tmp32_), _tmp33_);
				vala_ccode_function_call_add_argument (ccall, cleft);
				vala_ccode_function_call_add_argument (ccall, cright);
				cleft = (_tmp34_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccall), _vala_ccode_node_unref0 (cleft), _tmp34_);
				cright = (_tmp35_ = (ValaCCodeExpression*) vala_ccode_constant_new ("0"), _vala_ccode_node_unref0 (cright), _tmp35_);
				_vala_ccode_node_unref0 (ccall);
			}
		}
	}
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp36_ = vala_ccode_binary_expression_new (op, cleft, cright)));
	_vala_ccode_node_unref0 (_tmp36_);
	_vala_ccode_node_unref0 (cleft);
	_vala_ccode_node_unref0 (cright);
}


char* vala_ccode_base_module_get_type_check_function (ValaCCodeBaseModule* self, ValaTypeSymbol* type) {
	char* result;
	ValaTypeSymbol* _tmp0_;
	ValaClass* cl;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	cl = _vala_code_node_ref0 ((_tmp0_ = type, VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (cl != NULL) {
		_tmp1_ = vala_class_get_type_check_function (cl) != NULL;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		result = g_strdup (vala_class_get_type_check_function (cl));
		_vala_code_node_unref0 (cl);
		return result;
	} else {
		gboolean _tmp2_;
		gboolean _tmp3_;
		gboolean _tmp4_;
		gboolean _tmp5_;
		_tmp2_ = FALSE;
		_tmp3_ = FALSE;
		_tmp4_ = FALSE;
		_tmp5_ = FALSE;
		if (cl != NULL) {
			_tmp5_ = vala_class_get_is_compact (cl);
		} else {
			_tmp5_ = FALSE;
		}
		if (_tmp5_) {
			_tmp4_ = TRUE;
		} else {
			_tmp4_ = VALA_IS_STRUCT (type);
		}
		if (_tmp4_) {
			_tmp3_ = TRUE;
		} else {
			_tmp3_ = VALA_IS_ENUM (type);
		}
		if (_tmp3_) {
			_tmp2_ = TRUE;
		} else {
			_tmp2_ = VALA_IS_DELEGATE (type);
		}
		if (_tmp2_) {
			result = NULL;
			_vala_code_node_unref0 (cl);
			return result;
		} else {
			result = vala_typesymbol_get_upper_case_cname (type, "IS_");
			_vala_code_node_unref0 (cl);
			return result;
		}
	}
	_vala_code_node_unref0 (cl);
}


static ValaCCodeExpression* vala_ccode_base_module_create_type_check (ValaCCodeBaseModule* self, ValaCCodeNode* ccodenode, ValaDataType* type) {
	ValaCCodeExpression* result;
	ValaDataType* _tmp0_;
	ValaErrorType* et;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (ccodenode != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	et = _vala_code_node_ref0 ((_tmp0_ = type, VALA_IS_ERROR_TYPE (_tmp0_) ? ((ValaErrorType*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (et != NULL) {
		_tmp1_ = vala_error_type_get_error_code (et) != NULL;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		ValaCCodeIdentifier* _tmp2_;
		ValaCCodeFunctionCall* _tmp3_;
		ValaCCodeFunctionCall* matches_call;
		ValaCCodeIdentifier* _tmp5_;
		char* _tmp4_;
		ValaCCodeIdentifier* _tmp7_;
		char* _tmp6_;
		matches_call = (_tmp3_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("g_error_matches"))), _vala_ccode_node_unref0 (_tmp2_), _tmp3_);
		vala_ccode_function_call_add_argument (matches_call, VALA_CCODE_EXPRESSION (ccodenode));
		vala_ccode_function_call_add_argument (matches_call, (ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new (_tmp4_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) vala_error_type_get_error_domain (et), NULL))));
		_vala_ccode_node_unref0 (_tmp5_);
		_g_free0 (_tmp4_);
		vala_ccode_function_call_add_argument (matches_call, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) vala_error_type_get_error_code (et), FALSE))));
		_vala_ccode_node_unref0 (_tmp7_);
		_g_free0 (_tmp6_);
		result = (ValaCCodeExpression*) matches_call;
		_vala_code_node_unref0 (et);
		return result;
	} else {
		gboolean _tmp8_;
		_tmp8_ = FALSE;
		if (et != NULL) {
			_tmp8_ = vala_error_type_get_error_domain (et) != NULL;
		} else {
			_tmp8_ = FALSE;
		}
		if (_tmp8_) {
			ValaCCodeMemberAccess* instance_domain;
			char* _tmp9_;
			ValaCCodeIdentifier* _tmp10_;
			ValaCCodeIdentifier* type_domain;
			instance_domain = vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccodenode), "domain");
			type_domain = (_tmp10_ = vala_ccode_identifier_new (_tmp9_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) vala_error_type_get_error_domain (et), NULL)), _g_free0 (_tmp9_), _tmp10_);
			result = (ValaCCodeExpression*) vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) instance_domain, (ValaCCodeExpression*) type_domain);
			_vala_ccode_node_unref0 (instance_domain);
			_vala_ccode_node_unref0 (type_domain);
			_vala_code_node_unref0 (et);
			return result;
		} else {
			char* type_check_func;
			ValaCCodeIdentifier* _tmp11_;
			ValaCCodeFunctionCall* _tmp12_;
			ValaCCodeFunctionCall* ccheck;
			type_check_func = vala_ccode_base_module_get_type_check_function (self, vala_data_type_get_data_type (type));
			if (type_check_func == NULL) {
				result = (ValaCCodeExpression*) vala_ccode_invalid_expression_new ();
				_g_free0 (type_check_func);
				_vala_code_node_unref0 (et);
				return result;
			}
			ccheck = (_tmp12_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new (type_check_func))), _vala_ccode_node_unref0 (_tmp11_), _tmp12_);
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION (ccodenode));
			result = (ValaCCodeExpression*) ccheck;
			_g_free0 (type_check_func);
			_vala_code_node_unref0 (et);
			return result;
		}
	}
	_vala_code_node_unref0 (et);
}


static void vala_ccode_base_module_real_visit_type_check (ValaCCodeModule* base, ValaTypeCheck* expr) {
	ValaCCodeBaseModule * self;
	ValaCCodeExpression* _tmp0_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (expr != NULL);
	vala_ccode_base_module_generate_type_declaration (self, vala_typecheck_get_type_reference (expr), self->source_declarations);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp0_ = vala_ccode_base_module_create_type_check (self, vala_code_node_get_ccodenode ((ValaCodeNode*) vala_typecheck_get_expression (expr)), vala_typecheck_get_type_reference (expr))));
	_vala_ccode_node_unref0 (_tmp0_);
	if (VALA_IS_CCODE_INVALID_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr))) {
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), "type check expressions not supported for compact classes, structs, and enums");
	}
}


static void vala_ccode_base_module_real_visit_lambda_expression (ValaCCodeModule* base, ValaLambdaExpression* l) {
	ValaCCodeBaseModule * self;
	ValaDelegateType* dt;
	GeeArrayList* old_temp_vars;
	GeeArrayList* old_temp_ref_vars;
	GeeArrayList* _tmp0_;
	GeeArrayList* _tmp1_;
	GeeArrayList* _tmp2_;
	GeeArrayList* _tmp3_;
	ValaCCodeIdentifier* _tmp5_;
	char* _tmp4_;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (l != NULL);
	dt = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_expression_get_target_type ((ValaExpression*) l)));
	vala_method_set_cinstance_parameter_position (vala_lambda_expression_get_method (l), vala_delegate_get_cinstance_parameter_position (vala_delegate_type_get_delegate_symbol (dt)));
	old_temp_vars = _gee_collection_object_ref0 (self->temp_vars);
	old_temp_ref_vars = _gee_collection_object_ref0 (self->temp_ref_vars);
	self->temp_vars = (_tmp0_ = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), _gee_collection_object_unref0 (self->temp_vars), _tmp0_);
	self->temp_ref_vars = (_tmp1_ = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), _gee_collection_object_unref0 (self->temp_ref_vars), _tmp1_);
	vala_code_node_accept_children ((ValaCodeNode*) l, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	self->temp_vars = (_tmp2_ = _gee_collection_object_ref0 (old_temp_vars), _gee_collection_object_unref0 (self->temp_vars), _tmp2_);
	self->temp_ref_vars = (_tmp3_ = _gee_collection_object_ref0 (old_temp_ref_vars), _gee_collection_object_unref0 (self->temp_ref_vars), _tmp3_);
	vala_code_node_set_ccodenode ((ValaCodeNode*) l, (ValaCCodeNode*) (_tmp5_ = vala_ccode_identifier_new (_tmp4_ = vala_method_get_cname (vala_lambda_expression_get_method (l)))));
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	_vala_code_node_unref0 (dt);
	_gee_collection_object_unref0 (old_temp_vars);
	_gee_collection_object_unref0 (old_temp_ref_vars);
}


ValaCCodeExpression* vala_ccode_base_module_convert_from_generic_pointer (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr, ValaDataType* actual_type) {
	ValaCCodeExpression* result;
	ValaCCodeExpression* _result_;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (cexpr != NULL, NULL);
	g_return_val_if_fail (actual_type != NULL, NULL);
	_result_ = _vala_ccode_node_ref0 (cexpr);
	_tmp0_ = FALSE;
	if (vala_ccode_base_module_is_reference_type_argument (self, actual_type)) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = vala_ccode_base_module_is_nullable_value_type_argument (self, actual_type);
	}
	if (_tmp0_) {
		ValaCCodeExpression* _tmp2_;
		char* _tmp1_;
		_result_ = (_tmp2_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (cexpr, _tmp1_ = vala_data_type_get_cname (actual_type)), _vala_ccode_node_unref0 (_result_), _tmp2_);
		_g_free0 (_tmp1_);
	} else {
		if (vala_ccode_base_module_is_signed_integer_type_argument (self, actual_type)) {
			ValaCCodeIdentifier* _tmp3_;
			ValaCCodeFunctionCall* _tmp4_;
			ValaCCodeFunctionCall* cconv;
			ValaCCodeExpression* _tmp5_;
			cconv = (_tmp4_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("GPOINTER_TO_INT"))), _vala_ccode_node_unref0 (_tmp3_), _tmp4_);
			vala_ccode_function_call_add_argument (cconv, cexpr);
			_result_ = (_tmp5_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) cconv), _vala_ccode_node_unref0 (_result_), _tmp5_);
			_vala_ccode_node_unref0 (cconv);
		} else {
			if (vala_ccode_base_module_is_unsigned_integer_type_argument (self, actual_type)) {
				ValaCCodeIdentifier* _tmp6_;
				ValaCCodeFunctionCall* _tmp7_;
				ValaCCodeFunctionCall* cconv;
				ValaCCodeExpression* _tmp8_;
				cconv = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("GPOINTER_TO_UINT"))), _vala_ccode_node_unref0 (_tmp6_), _tmp7_);
				vala_ccode_function_call_add_argument (cconv, cexpr);
				_result_ = (_tmp8_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) cconv), _vala_ccode_node_unref0 (_result_), _tmp8_);
				_vala_ccode_node_unref0 (cconv);
			}
		}
	}
	result = _result_;
	return result;
}


ValaCCodeExpression* vala_ccode_base_module_convert_to_generic_pointer (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr, ValaDataType* actual_type) {
	ValaCCodeExpression* result;
	ValaCCodeExpression* _result_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (cexpr != NULL, NULL);
	g_return_val_if_fail (actual_type != NULL, NULL);
	_result_ = _vala_ccode_node_ref0 (cexpr);
	if (vala_ccode_base_module_is_signed_integer_type_argument (self, actual_type)) {
		ValaCCodeIdentifier* _tmp0_;
		ValaCCodeFunctionCall* _tmp1_;
		ValaCCodeFunctionCall* cconv;
		ValaCCodeExpression* _tmp2_;
		cconv = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("GINT_TO_POINTER"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
		vala_ccode_function_call_add_argument (cconv, cexpr);
		_result_ = (_tmp2_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) cconv), _vala_ccode_node_unref0 (_result_), _tmp2_);
		_vala_ccode_node_unref0 (cconv);
	} else {
		if (vala_ccode_base_module_is_unsigned_integer_type_argument (self, actual_type)) {
			ValaCCodeIdentifier* _tmp3_;
			ValaCCodeFunctionCall* _tmp4_;
			ValaCCodeFunctionCall* cconv;
			ValaCCodeExpression* _tmp5_;
			cconv = (_tmp4_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("GUINT_TO_POINTER"))), _vala_ccode_node_unref0 (_tmp3_), _tmp4_);
			vala_ccode_function_call_add_argument (cconv, cexpr);
			_result_ = (_tmp5_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) cconv), _vala_ccode_node_unref0 (_result_), _tmp5_);
			_vala_ccode_node_unref0 (cconv);
		}
	}
	result = _result_;
	return result;
}


ValaCCodeExpression* vala_ccode_base_module_transform_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr) {
	ValaCCodeExpression* result;
	ValaCCodeExpression* cexpr;
	gboolean _tmp0_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	gboolean _tmp11_;
	gboolean boxing;
	gboolean _tmp12_;
	gboolean _tmp13_;
	gboolean _tmp14_;
	gboolean unboxing;
	gboolean _tmp15_;
	gboolean _tmp16_;
	gboolean _tmp17_;
	gboolean _tmp18_;
	gboolean gvalue_boxing;
	gboolean _tmp20_;
	gboolean _tmp71_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (source_cexpr != NULL, NULL);
	cexpr = _vala_ccode_node_ref0 (source_cexpr);
	if (expression_type == NULL) {
		result = cexpr;
		return result;
	}
	_tmp0_ = FALSE;
	if (vala_data_type_get_value_owned (expression_type)) {
		_tmp0_ = vala_data_type_get_floating_reference (expression_type);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaTypeSymbol* _tmp1_;
		ValaObjectTypeSymbol* cl;
		char* _tmp2_;
		char* sink_func;
		cl = _vala_code_node_ref0 ((_tmp1_ = vala_data_type_get_data_type (expression_type), VALA_IS_OBJECT_TYPE_SYMBOL (_tmp1_) ? ((ValaObjectTypeSymbol*) _tmp1_) : NULL));
		_tmp2_ = NULL;
		if (cl != NULL) {
			char* _tmp3_;
			_tmp2_ = (_tmp3_ = vala_typesymbol_get_ref_sink_function ((ValaTypeSymbol*) cl), _g_free0 (_tmp2_), _tmp3_);
		} else {
			char* _tmp4_;
			_tmp2_ = (_tmp4_ = NULL, _g_free0 (_tmp2_), _tmp4_);
		}
		sink_func = g_strdup (_tmp2_);
		if (sink_func != NULL) {
			ValaCCodeIdentifier* _tmp5_;
			ValaCCodeFunctionCall* _tmp6_;
			ValaCCodeFunctionCall* csink;
			ValaCCodeExpression* _tmp7_;
			csink = (_tmp6_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new (sink_func))), _vala_ccode_node_unref0 (_tmp5_), _tmp6_);
			vala_ccode_function_call_add_argument (csink, cexpr);
			cexpr = (_tmp7_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) csink), _vala_ccode_node_unref0 (cexpr), _tmp7_);
			_vala_ccode_node_unref0 (csink);
		} else {
			char* _tmp8_;
			vala_report_error (NULL, _tmp8_ = g_strdup_printf ("type `%s' does not support floating references", vala_symbol_get_name ((ValaSymbol*) vala_data_type_get_data_type (expression_type))));
			_g_free0 (_tmp8_);
		}
		_vala_code_node_unref0 (cl);
		_g_free0 (_tmp2_);
		_g_free0 (sink_func);
	}
	_tmp9_ = FALSE;
	_tmp10_ = FALSE;
	_tmp11_ = FALSE;
	if (VALA_IS_VALUE_TYPE (expression_type)) {
		_tmp11_ = !vala_data_type_get_nullable (expression_type);
	} else {
		_tmp11_ = FALSE;
	}
	if (_tmp11_) {
		_tmp10_ = VALA_IS_VALUE_TYPE (target_type);
	} else {
		_tmp10_ = FALSE;
	}
	if (_tmp10_) {
		_tmp9_ = vala_data_type_get_nullable (target_type);
	} else {
		_tmp9_ = FALSE;
	}
	boxing = _tmp9_;
	_tmp12_ = FALSE;
	_tmp13_ = FALSE;
	_tmp14_ = FALSE;
	if (VALA_IS_VALUE_TYPE (expression_type)) {
		_tmp14_ = vala_data_type_get_nullable (expression_type);
	} else {
		_tmp14_ = FALSE;
	}
	if (_tmp14_) {
		_tmp13_ = VALA_IS_VALUE_TYPE (target_type);
	} else {
		_tmp13_ = FALSE;
	}
	if (_tmp13_) {
		_tmp12_ = !vala_data_type_get_nullable (target_type);
	} else {
		_tmp12_ = FALSE;
	}
	unboxing = _tmp12_;
	_tmp15_ = FALSE;
	_tmp16_ = FALSE;
	_tmp17_ = FALSE;
	_tmp18_ = FALSE;
	if (vala_code_context_get_profile (self->priv->_context) == VALA_PROFILE_GOBJECT) {
		_tmp18_ = target_type != NULL;
	} else {
		_tmp18_ = FALSE;
	}
	if (_tmp18_) {
		_tmp17_ = vala_data_type_get_data_type (target_type) == VALA_TYPESYMBOL (self->gvalue_type);
	} else {
		_tmp17_ = FALSE;
	}
	if (_tmp17_) {
		_tmp16_ = !VALA_IS_NULL_TYPE (expression_type);
	} else {
		_tmp16_ = FALSE;
	}
	if (_tmp16_) {
		char* _tmp19_;
		_tmp15_ = _vala_strcmp0 (_tmp19_ = vala_data_type_get_type_id (expression_type), "G_TYPE_VALUE") != 0;
		_g_free0 (_tmp19_);
	} else {
		_tmp15_ = FALSE;
	}
	gvalue_boxing = _tmp15_;
	_tmp20_ = FALSE;
	if (vala_data_type_get_value_owned (expression_type)) {
		gboolean _tmp21_;
		gboolean _tmp22_;
		gboolean _tmp23_;
		gboolean _tmp24_;
		_tmp21_ = FALSE;
		_tmp22_ = FALSE;
		_tmp23_ = FALSE;
		_tmp24_ = FALSE;
		if (target_type == NULL) {
			_tmp24_ = TRUE;
		} else {
			_tmp24_ = !vala_data_type_get_value_owned (target_type);
		}
		if (_tmp24_) {
			_tmp23_ = TRUE;
		} else {
			_tmp23_ = boxing;
		}
		if (_tmp23_) {
			_tmp22_ = TRUE;
		} else {
			_tmp22_ = unboxing;
		}
		if (_tmp22_) {
			_tmp21_ = TRUE;
		} else {
			_tmp21_ = gvalue_boxing;
		}
		_tmp20_ = _tmp21_;
	} else {
		_tmp20_ = FALSE;
	}
	if (_tmp20_) {
		ValaDataType* _tmp25_;
		ValaPointerType* pointer_type;
		gboolean _tmp26_;
		pointer_type = _vala_code_node_ref0 ((_tmp25_ = target_type, VALA_IS_POINTER_TYPE (_tmp25_) ? ((ValaPointerType*) _tmp25_) : NULL));
		_tmp26_ = FALSE;
		if (pointer_type != NULL) {
			_tmp26_ = !VALA_IS_VOID_TYPE (vala_pointer_type_get_base_type (pointer_type));
		} else {
			_tmp26_ = FALSE;
		}
		if (_tmp26_) {
		} else {
			if (vala_ccode_base_module_requires_destroy (self, expression_type)) {
				ValaLocalVariable* decl;
				ValaCCodeExpression* _tmp28_;
				ValaCCodeExpression* _tmp27_;
				gboolean _tmp29_;
				decl = vala_ccode_base_module_get_temp_variable (self, expression_type, TRUE, (ValaCodeNode*) expression_type, FALSE);
				gee_list_insert ((GeeList*) self->temp_vars, 0, decl);
				gee_list_insert ((GeeList*) self->temp_ref_vars, 0, decl);
				cexpr = (_tmp28_ = (ValaCCodeExpression*) vala_ccode_assignment_new (_tmp27_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)), cexpr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (cexpr), _tmp28_);
				_vala_ccode_node_unref0 (_tmp27_);
				_tmp29_ = FALSE;
				if (VALA_IS_ARRAY_TYPE (expression_type)) {
					_tmp29_ = expr != NULL;
				} else {
					_tmp29_ = FALSE;
				}
				if (_tmp29_) {
					ValaArrayType* array_type;
					ValaCCodeCommaExpression* ccomma;
					ValaCCodeExpression* _tmp37_;
					ValaCCodeExpression* _tmp38_;
					array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (expression_type));
					ccomma = vala_ccode_comma_expression_new ();
					vala_ccode_comma_expression_append_expression (ccomma, cexpr);
					{
						gint dim;
						dim = 1;
						{
							gboolean _tmp30_;
							_tmp30_ = TRUE;
							while (TRUE) {
								char* _tmp32_;
								ValaDataType* _tmp31_;
								ValaLocalVariable* _tmp33_;
								ValaLocalVariable* len_decl;
								ValaCCodeAssignment* _tmp36_;
								ValaCCodeExpression* _tmp35_;
								ValaCCodeExpression* _tmp34_;
								if (!_tmp30_) {
									dim++;
								}
								_tmp30_ = FALSE;
								if (!(dim <= vala_array_type_get_rank (array_type))) {
									break;
								}
								len_decl = (_tmp33_ = vala_local_variable_new (_tmp31_ = vala_data_type_copy (self->int_type), _tmp32_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) decl), dim), NULL, NULL), _g_free0 (_tmp32_), _vala_code_node_unref0 (_tmp31_), _tmp33_);
								gee_list_insert ((GeeList*) self->temp_vars, 0, len_decl);
								vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_assignment_new (_tmp34_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) len_decl)), _tmp35_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), expr, dim), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
								_vala_ccode_node_unref0 (_tmp36_);
								_vala_ccode_node_unref0 (_tmp35_);
								_vala_ccode_node_unref0 (_tmp34_);
								_vala_code_node_unref0 (len_decl);
							}
						}
					}
					vala_ccode_comma_expression_append_expression (ccomma, _tmp37_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)));
					_vala_ccode_node_unref0 (_tmp37_);
					cexpr = (_tmp38_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (cexpr), _tmp38_);
					_vala_code_node_unref0 (array_type);
					_vala_ccode_node_unref0 (ccomma);
				}
				_vala_code_node_unref0 (decl);
			}
		}
		_vala_code_node_unref0 (pointer_type);
	}
	if (target_type == NULL) {
		result = cexpr;
		return result;
	}
	if (gvalue_boxing) {
		ValaLocalVariable* decl;
		ValaCCodeCommaExpression* ccomma;
		ValaCCodeIdentifier* _tmp45_;
		ValaCCodeFunctionCall* _tmp46_;
		ValaCCodeFunctionCall* ccall;
		ValaCCodeIdentifier* _tmp51_;
		char* _tmp50_;
		ValaCCodeFunctionCall* _tmp53_;
		ValaCCodeIdentifier* _tmp52_;
		ValaCCodeExpression* _tmp57_;
		ValaCCodeExpression* _tmp58_;
		decl = vala_ccode_base_module_get_temp_variable (self, target_type, TRUE, (ValaCodeNode*) target_type, TRUE);
		gee_list_insert ((GeeList*) self->temp_vars, 0, decl);
		ccomma = vala_ccode_comma_expression_new ();
		if (vala_data_type_get_nullable (target_type)) {
			ValaCCodeIdentifier* _tmp39_;
			ValaCCodeFunctionCall* _tmp40_;
			ValaCCodeFunctionCall* newcall;
			ValaCCodeConstant* _tmp41_;
			ValaCCodeConstant* _tmp42_;
			ValaCCodeExpression* _tmp43_;
			ValaCCodeAssignment* _tmp44_;
			ValaCCodeAssignment* newassignment;
			newcall = (_tmp40_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new ("g_new0"))), _vala_ccode_node_unref0 (_tmp39_), _tmp40_);
			vala_ccode_function_call_add_argument (newcall, (ValaCCodeExpression*) (_tmp41_ = vala_ccode_constant_new ("GValue")));
			_vala_ccode_node_unref0 (_tmp41_);
			vala_ccode_function_call_add_argument (newcall, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_constant_new ("1")));
			_vala_ccode_node_unref0 (_tmp42_);
			newassignment = (_tmp44_ = vala_ccode_assignment_new (_tmp43_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)), (ValaCCodeExpression*) newcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (_tmp43_), _tmp44_);
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) newassignment);
			_vala_ccode_node_unref0 (newcall);
			_vala_ccode_node_unref0 (newassignment);
		}
		ccall = (_tmp46_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("g_value_init"))), _vala_ccode_node_unref0 (_tmp45_), _tmp46_);
		if (vala_data_type_get_nullable (target_type)) {
			ValaCCodeExpression* _tmp47_;
			vala_ccode_function_call_add_argument (ccall, _tmp47_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)));
			_vala_ccode_node_unref0 (_tmp47_);
		} else {
			ValaCCodeUnaryExpression* _tmp49_;
			ValaCCodeExpression* _tmp48_;
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp49_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp48_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)))));
			_vala_ccode_node_unref0 (_tmp49_);
			_vala_ccode_node_unref0 (_tmp48_);
		}
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp51_ = vala_ccode_identifier_new (_tmp50_ = vala_data_type_get_type_id (expression_type))));
		_vala_ccode_node_unref0 (_tmp51_);
		_g_free0 (_tmp50_);
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
		ccall = (_tmp53_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp52_ = vala_ccode_module_get_value_setter_function ((ValaCCodeModule*) self, expression_type))), _vala_ccode_node_unref0 (ccall), _tmp53_);
		_vala_ccode_node_unref0 (_tmp52_);
		if (vala_data_type_get_nullable (target_type)) {
			ValaCCodeExpression* _tmp54_;
			vala_ccode_function_call_add_argument (ccall, _tmp54_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)));
			_vala_ccode_node_unref0 (_tmp54_);
		} else {
			ValaCCodeUnaryExpression* _tmp56_;
			ValaCCodeExpression* _tmp55_;
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp56_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp55_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)))));
			_vala_ccode_node_unref0 (_tmp56_);
			_vala_ccode_node_unref0 (_tmp55_);
		}
		vala_ccode_function_call_add_argument (ccall, cexpr);
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
		vala_ccode_comma_expression_append_expression (ccomma, _tmp57_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)));
		_vala_ccode_node_unref0 (_tmp57_);
		cexpr = (_tmp58_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (cexpr), _tmp58_);
		result = cexpr;
		_vala_code_node_unref0 (decl);
		_vala_ccode_node_unref0 (ccomma);
		_vala_ccode_node_unref0 (ccall);
		return result;
	} else {
		if (boxing) {
			ValaCCodeExpression* _tmp59_;
			ValaCCodeUnaryExpression* unary;
			gboolean _tmp60_;
			unary = _vala_ccode_node_ref0 ((_tmp59_ = cexpr, VALA_IS_CCODE_UNARY_EXPRESSION (_tmp59_) ? ((ValaCCodeUnaryExpression*) _tmp59_) : NULL));
			_tmp60_ = FALSE;
			if (unary != NULL) {
				_tmp60_ = vala_ccode_unary_expression_get_operator (unary) == VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION;
			} else {
				_tmp60_ = FALSE;
			}
			if (_tmp60_) {
				ValaCCodeExpression* _tmp61_;
				cexpr = (_tmp61_ = _vala_ccode_node_ref0 (vala_ccode_unary_expression_get_inner (unary)), _vala_ccode_node_unref0 (cexpr), _tmp61_);
			} else {
				gboolean _tmp62_;
				_tmp62_ = FALSE;
				if (VALA_IS_CCODE_IDENTIFIER (cexpr)) {
					_tmp62_ = TRUE;
				} else {
					_tmp62_ = VALA_IS_CCODE_MEMBER_ACCESS (cexpr);
				}
				if (_tmp62_) {
					ValaCCodeExpression* _tmp63_;
					cexpr = (_tmp63_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, cexpr), _vala_ccode_node_unref0 (cexpr), _tmp63_);
				} else {
					ValaLocalVariable* decl;
					ValaCCodeCommaExpression* ccomma;
					ValaCCodeAssignment* _tmp65_;
					ValaCCodeExpression* _tmp64_;
					ValaCCodeUnaryExpression* _tmp67_;
					ValaCCodeExpression* _tmp66_;
					ValaCCodeExpression* _tmp68_;
					decl = vala_ccode_base_module_get_temp_variable (self, expression_type, vala_data_type_get_value_owned (expression_type), (ValaCodeNode*) expression_type, FALSE);
					gee_list_insert ((GeeList*) self->temp_vars, 0, decl);
					ccomma = vala_ccode_comma_expression_new ();
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp65_ = vala_ccode_assignment_new (_tmp64_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)), cexpr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp65_);
					_vala_ccode_node_unref0 (_tmp64_);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp67_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp66_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) decl)))));
					_vala_ccode_node_unref0 (_tmp67_);
					_vala_ccode_node_unref0 (_tmp66_);
					cexpr = (_tmp68_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (cexpr), _tmp68_);
					_vala_code_node_unref0 (decl);
					_vala_ccode_node_unref0 (ccomma);
				}
			}
			_vala_ccode_node_unref0 (unary);
		} else {
			if (unboxing) {
				ValaCCodeExpression* _tmp69_;
				cexpr = (_tmp69_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, cexpr), _vala_ccode_node_unref0 (cexpr), _tmp69_);
			} else {
				ValaCCodeExpression* _tmp70_;
				cexpr = (_tmp70_ = vala_ccode_base_module_get_implicit_cast_expression (self, cexpr, expression_type, target_type, expr), _vala_ccode_node_unref0 (cexpr), _tmp70_);
			}
		}
	}
	_tmp71_ = FALSE;
	if (vala_data_type_get_value_owned (target_type)) {
		gboolean _tmp72_;
		gboolean _tmp73_;
		_tmp72_ = FALSE;
		_tmp73_ = FALSE;
		if (!vala_data_type_get_value_owned (expression_type)) {
			_tmp73_ = TRUE;
		} else {
			_tmp73_ = boxing;
		}
		if (_tmp73_) {
			_tmp72_ = TRUE;
		} else {
			_tmp72_ = unboxing;
		}
		_tmp71_ = _tmp72_;
	} else {
		_tmp71_ = FALSE;
	}
	if (_tmp71_) {
		gboolean _tmp74_;
		_tmp74_ = FALSE;
		if (vala_ccode_base_module_requires_copy (self, target_type)) {
			_tmp74_ = !VALA_IS_NULL_TYPE (expression_type);
		} else {
			_tmp74_ = FALSE;
		}
		if (_tmp74_) {
			ValaCodeNode* node;
			ValaCCodeExpression* _tmp76_;
			node = _vala_code_node_ref0 ((ValaCodeNode*) expr);
			if (node == NULL) {
				ValaCodeNode* _tmp75_;
				node = (_tmp75_ = _vala_code_node_ref0 ((ValaCodeNode*) expression_type), _vala_code_node_unref0 (node), _tmp75_);
			}
			cexpr = (_tmp76_ = vala_ccode_base_module_get_ref_cexpression (self, target_type, cexpr, expr, node), _vala_ccode_node_unref0 (cexpr), _tmp76_);
			_vala_code_node_unref0 (node);
		}
	}
	result = cexpr;
	return result;
}


static ValaCCodeExpression* vala_ccode_base_module_real_get_implicit_cast_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr) {
	ValaCCodeExpression* result;
	ValaCCodeExpression* cexpr;
	gboolean _tmp0_;
	ValaTypeSymbol* _tmp1_;
	ValaClass* cl;
	ValaTypeSymbol* _tmp2_;
	ValaInterface* iface;
	gboolean _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (source_cexpr != NULL, NULL);
	cexpr = _vala_ccode_node_ref0 (source_cexpr);
	_tmp0_ = FALSE;
	if (vala_data_type_get_data_type (expression_type) != NULL) {
		_tmp0_ = vala_data_type_get_data_type (expression_type) == vala_data_type_get_data_type (target_type);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = cexpr;
		return result;
	}
	if (VALA_IS_NULL_TYPE (expression_type)) {
		result = cexpr;
		return result;
	}
	vala_ccode_base_module_generate_type_declaration (self, target_type, self->source_declarations);
	cl = _vala_code_node_ref0 ((_tmp1_ = vala_data_type_get_data_type (target_type), VALA_IS_CLASS (_tmp1_) ? ((ValaClass*) _tmp1_) : NULL));
	iface = _vala_code_node_ref0 ((_tmp2_ = vala_data_type_get_data_type (target_type), VALA_IS_INTERFACE (_tmp2_) ? ((ValaInterface*) _tmp2_) : NULL));
	_tmp3_ = FALSE;
	if (vala_code_context_get_checking (self->priv->_context)) {
		gboolean _tmp4_;
		_tmp4_ = FALSE;
		if (iface != NULL) {
			_tmp4_ = TRUE;
		} else {
			gboolean _tmp5_;
			_tmp5_ = FALSE;
			if (cl != NULL) {
				_tmp5_ = !vala_class_get_is_compact (cl);
			} else {
				_tmp5_ = FALSE;
			}
			_tmp4_ = _tmp5_;
		}
		_tmp3_ = _tmp4_;
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		result = (ValaCCodeExpression*) vala_ccode_base_module_generate_instance_cast (self, cexpr, vala_data_type_get_data_type (target_type));
		_vala_ccode_node_unref0 (cexpr);
		_vala_code_node_unref0 (cl);
		_vala_code_node_unref0 (iface);
		return result;
	} else {
		gboolean _tmp6_;
		_tmp6_ = FALSE;
		if (vala_data_type_get_data_type (target_type) != NULL) {
			char* _tmp8_;
			char* _tmp7_;
			_tmp6_ = _vala_strcmp0 (_tmp7_ = vala_data_type_get_cname (expression_type), _tmp8_ = vala_data_type_get_cname (target_type)) != 0;
			_g_free0 (_tmp8_);
			_g_free0 (_tmp7_);
		} else {
			_tmp6_ = FALSE;
		}
		if (_tmp6_) {
			ValaTypeSymbol* _tmp9_;
			ValaStruct* st;
			gboolean _tmp10_;
			st = _vala_code_node_ref0 ((_tmp9_ = vala_data_type_get_data_type (target_type), VALA_IS_STRUCT (_tmp9_) ? ((ValaStruct*) _tmp9_) : NULL));
			_tmp10_ = FALSE;
			if (vala_typesymbol_is_reference_type (vala_data_type_get_data_type (target_type))) {
				_tmp10_ = TRUE;
			} else {
				gboolean _tmp11_;
				_tmp11_ = FALSE;
				if (st != NULL) {
					_tmp11_ = vala_struct_is_simple_type (st);
				} else {
					_tmp11_ = FALSE;
				}
				_tmp10_ = _tmp11_;
			}
			if (_tmp10_) {
				char* _tmp12_;
				ValaCCodeExpression* _tmp13_;
				result = (_tmp13_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (cexpr, _tmp12_ = vala_data_type_get_cname (target_type)), _g_free0 (_tmp12_), _tmp13_);
				_vala_code_node_unref0 (st);
				_vala_ccode_node_unref0 (cexpr);
				_vala_code_node_unref0 (cl);
				_vala_code_node_unref0 (iface);
				return result;
			} else {
				result = cexpr;
				_vala_code_node_unref0 (st);
				_vala_code_node_unref0 (cl);
				_vala_code_node_unref0 (iface);
				return result;
			}
			_vala_code_node_unref0 (st);
		} else {
			result = cexpr;
			_vala_code_node_unref0 (cl);
			_vala_code_node_unref0 (iface);
			return result;
		}
	}
	_vala_ccode_node_unref0 (cexpr);
	_vala_code_node_unref0 (cl);
	_vala_code_node_unref0 (iface);
}


ValaCCodeExpression* vala_ccode_base_module_get_implicit_cast_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->get_implicit_cast_expression (self, source_cexpr, expression_type, target_type, expr);
}


ValaCCodeFunctionCall* vala_ccode_base_module_get_property_set_call (ValaCCodeBaseModule* self, ValaProperty* prop, ValaMemberAccess* ma, ValaCCodeExpression* cexpr, ValaExpression* rhs) {
	ValaCCodeFunctionCall* result;
	char* set_func;
	ValaProperty* base_property;
	ValaCCodeIdentifier* _tmp23_;
	ValaCCodeFunctionCall* _tmp24_;
	ValaCCodeFunctionCall* ccall;
	ValaDataType* _tmp36_;
	ValaArrayType* array_type;
	gboolean _tmp37_;
	gboolean _tmp38_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (prop != NULL, NULL);
	g_return_val_if_fail (ma != NULL, NULL);
	g_return_val_if_fail (cexpr != NULL, NULL);
	if (VALA_IS_BASE_ACCESS (vala_member_access_get_inner (ma))) {
		if (vala_property_get_base_property (prop) != NULL) {
			ValaClass* base_class;
			ValaCCodeIdentifier* _tmp2_;
			char* _tmp1_;
			char* _tmp0_;
			ValaCCodeFunctionCall* _tmp3_;
			ValaCCodeFunctionCall* vcast;
			ValaCCodeIdentifier* _tmp6_;
			char* _tmp5_;
			char* _tmp4_;
			ValaCCodeMemberAccess* _tmp8_;
			char* _tmp7_;
			ValaCCodeFunctionCall* _tmp9_;
			ValaCCodeFunctionCall* ccall;
			ValaCCodeExpression* _tmp10_;
			base_class = _vala_code_node_ref0 (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_property_get_base_property (prop))));
			vcast = (_tmp3_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new (_tmp1_ = g_strdup_printf ("%s_CLASS", _tmp0_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) base_class, NULL))))), _vala_ccode_node_unref0 (_tmp2_), _g_free0 (_tmp1_), _g_free0 (_tmp0_), _tmp3_);
			vala_ccode_function_call_add_argument (vcast, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new (_tmp5_ = g_strdup_printf ("%s_parent_class", _tmp4_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) vala_ccode_base_module_get_current_class (self), NULL)))));
			_vala_ccode_node_unref0 (_tmp6_);
			_g_free0 (_tmp5_);
			_g_free0 (_tmp4_);
			ccall = (_tmp9_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp8_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) vcast, _tmp7_ = g_strdup_printf ("set_%s", vala_symbol_get_name ((ValaSymbol*) prop))))), _vala_ccode_node_unref0 (_tmp8_), _g_free0 (_tmp7_), _tmp9_);
			vala_ccode_function_call_add_argument (ccall, _tmp10_ = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode (self, (ValaCodeNode*) vala_member_access_get_inner (ma))));
			_vala_ccode_node_unref0 (_tmp10_);
			vala_ccode_function_call_add_argument (ccall, cexpr);
			result = ccall;
			_vala_code_node_unref0 (base_class);
			_vala_ccode_node_unref0 (vcast);
			return result;
		} else {
			if (vala_property_get_base_interface_property (prop) != NULL) {
				ValaInterface* base_iface;
				char* _tmp12_;
				char* _tmp11_;
				char* _tmp13_;
				char* parent_iface_var;
				ValaCCodeMemberAccess* _tmp16_;
				char* _tmp15_;
				ValaCCodeIdentifier* _tmp14_;
				ValaCCodeFunctionCall* _tmp17_;
				ValaCCodeFunctionCall* ccall;
				ValaCCodeExpression* _tmp18_;
				base_iface = _vala_code_node_ref0 (VALA_INTERFACE (vala_symbol_get_parent_symbol ((ValaSymbol*) vala_property_get_base_interface_property (prop))));
				parent_iface_var = (_tmp13_ = g_strdup_printf ("%s_%s_parent_iface", _tmp11_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) vala_ccode_base_module_get_current_class (self), NULL), _tmp12_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) base_iface, NULL)), _g_free0 (_tmp12_), _g_free0 (_tmp11_), _tmp13_);
				ccall = (_tmp17_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new (parent_iface_var)), _tmp15_ = g_strdup_printf ("set_%s", vala_symbol_get_name ((ValaSymbol*) prop))))), _vala_ccode_node_unref0 (_tmp16_), _g_free0 (_tmp15_), _vala_ccode_node_unref0 (_tmp14_), _tmp17_);
				vala_ccode_function_call_add_argument (ccall, _tmp18_ = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode (self, (ValaCodeNode*) vala_member_access_get_inner (ma))));
				_vala_ccode_node_unref0 (_tmp18_);
				vala_ccode_function_call_add_argument (ccall, cexpr);
				result = ccall;
				_vala_code_node_unref0 (base_iface);
				_g_free0 (parent_iface_var);
				return result;
			}
		}
	}
	set_func = g_strdup ("g_object_set");
	base_property = _vala_code_node_ref0 (prop);
	if (!vala_property_get_no_accessor_method (prop)) {
		if (vala_property_get_base_property (prop) != NULL) {
			ValaProperty* _tmp19_;
			base_property = (_tmp19_ = _vala_code_node_ref0 (vala_property_get_base_property (prop)), _vala_code_node_unref0 (base_property), _tmp19_);
		} else {
			if (vala_property_get_base_interface_property (prop) != NULL) {
				ValaProperty* _tmp20_;
				base_property = (_tmp20_ = _vala_code_node_ref0 (vala_property_get_base_interface_property (prop)), _vala_code_node_unref0 (base_property), _tmp20_);
			}
		}
		if (VALA_IS_DYNAMIC_PROPERTY (prop)) {
			char* _tmp21_;
			set_func = (_tmp21_ = vala_ccode_module_get_dynamic_property_setter_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), VALA_DYNAMIC_PROPERTY (prop)), _g_free0 (set_func), _tmp21_);
		} else {
			char* _tmp22_;
			vala_ccode_base_module_generate_property_accessor_declaration (self, vala_property_get_set_accessor (base_property), self->source_declarations);
			set_func = (_tmp22_ = vala_property_accessor_get_cname (vala_property_get_set_accessor (base_property)), _g_free0 (set_func), _tmp22_);
		}
	}
	ccall = (_tmp24_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new (set_func))), _vala_ccode_node_unref0 (_tmp23_), _tmp24_);
	if (vala_property_get_binding (prop) == MEMBER_BINDING_INSTANCE) {
		ValaCCodeExpression* instance;
		instance = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode (self, (ValaCodeNode*) vala_member_access_get_inner (ma)));
		if (VALA_IS_STRUCT (vala_symbol_get_parent_symbol ((ValaSymbol*) prop))) {
			ValaCCodeExpression* _tmp25_;
			ValaCCodeUnaryExpression* unary;
			gboolean _tmp26_;
			unary = _vala_ccode_node_ref0 ((_tmp25_ = instance, VALA_IS_CCODE_UNARY_EXPRESSION (_tmp25_) ? ((ValaCCodeUnaryExpression*) _tmp25_) : NULL));
			_tmp26_ = FALSE;
			if (unary != NULL) {
				_tmp26_ = vala_ccode_unary_expression_get_operator (unary) == VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION;
			} else {
				_tmp26_ = FALSE;
			}
			if (_tmp26_) {
				ValaCCodeExpression* _tmp27_;
				instance = (_tmp27_ = _vala_ccode_node_ref0 (vala_ccode_unary_expression_get_inner (unary)), _vala_ccode_node_unref0 (instance), _tmp27_);
			} else {
				gboolean _tmp28_;
				_tmp28_ = FALSE;
				if (VALA_IS_CCODE_IDENTIFIER (instance)) {
					_tmp28_ = TRUE;
				} else {
					_tmp28_ = VALA_IS_CCODE_MEMBER_ACCESS (instance);
				}
				if (_tmp28_) {
					ValaCCodeExpression* _tmp29_;
					instance = (_tmp29_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, instance), _vala_ccode_node_unref0 (instance), _tmp29_);
				} else {
					ValaCCodeCommaExpression* ccomma;
					ValaLocalVariable* temp_var;
					ValaCCodeAssignment* _tmp31_;
					ValaCCodeExpression* _tmp30_;
					ValaCCodeUnaryExpression* _tmp33_;
					ValaCCodeExpression* _tmp32_;
					ValaCCodeExpression* _tmp34_;
					ccomma = vala_ccode_comma_expression_new ();
					temp_var = vala_ccode_base_module_get_temp_variable (self, vala_expression_get_target_type (vala_member_access_get_inner (ma)), TRUE, NULL, FALSE);
					gee_list_insert ((GeeList*) self->temp_vars, 0, temp_var);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp31_ = vala_ccode_assignment_new (_tmp30_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)), instance, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					_vala_ccode_node_unref0 (_tmp31_);
					_vala_ccode_node_unref0 (_tmp30_);
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp32_ = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_var)))));
					_vala_ccode_node_unref0 (_tmp33_);
					_vala_ccode_node_unref0 (_tmp32_);
					instance = (_tmp34_ = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ccomma), _vala_ccode_node_unref0 (instance), _tmp34_);
					_vala_ccode_node_unref0 (ccomma);
					_vala_code_node_unref0 (temp_var);
				}
			}
			_vala_ccode_node_unref0 (unary);
		}
		vala_ccode_function_call_add_argument (ccall, instance);
		_vala_ccode_node_unref0 (instance);
	}
	if (vala_property_get_no_accessor_method (prop)) {
		ValaCCodeConstant* _tmp35_;
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp35_ = vala_property_get_canonical_cconstant (prop)));
		_vala_ccode_node_unref0 (_tmp35_);
	}
	vala_ccode_function_call_add_argument (ccall, cexpr);
	array_type = _vala_code_node_ref0 ((_tmp36_ = vala_property_get_property_type (prop), VALA_IS_ARRAY_TYPE (_tmp36_) ? ((ValaArrayType*) _tmp36_) : NULL));
	_tmp37_ = FALSE;
	_tmp38_ = FALSE;
	if (array_type != NULL) {
		_tmp38_ = !vala_property_get_no_array_length (prop);
	} else {
		_tmp38_ = FALSE;
	}
	if (_tmp38_) {
		_tmp37_ = rhs != NULL;
	} else {
		_tmp37_ = FALSE;
	}
	if (_tmp37_) {
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp39_;
				_tmp39_ = TRUE;
				while (TRUE) {
					ValaCCodeExpression* _tmp40_;
					if (!_tmp39_) {
						dim++;
					}
					_tmp39_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					vala_ccode_function_call_add_argument (ccall, _tmp40_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), rhs, dim));
					_vala_ccode_node_unref0 (_tmp40_);
				}
			}
		}
	} else {
		gboolean _tmp41_;
		_tmp41_ = FALSE;
		if (VALA_IS_DELEGATE_TYPE (vala_property_get_property_type (prop))) {
			_tmp41_ = rhs != NULL;
		} else {
			_tmp41_ = FALSE;
		}
		if (_tmp41_) {
			ValaDelegateType* delegate_type;
			delegate_type = _vala_code_node_ref0 (VALA_DELEGATE_TYPE (vala_property_get_property_type (prop)));
			if (vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type))) {
				ValaCCodeExpression* _tmp42_;
				vala_ccode_function_call_add_argument (ccall, _tmp42_ = vala_ccode_base_module_get_delegate_target_cexpression (self, rhs));
				_vala_ccode_node_unref0 (_tmp42_);
			}
			_vala_code_node_unref0 (delegate_type);
		}
	}
	if (vala_property_get_no_accessor_method (prop)) {
		ValaCCodeConstant* _tmp43_;
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp43_ = vala_ccode_constant_new ("NULL")));
		_vala_ccode_node_unref0 (_tmp43_);
	}
	result = ccall;
	_g_free0 (set_func);
	_vala_code_node_unref0 (base_property);
	_vala_code_node_unref0 (array_type);
	return result;
}


gboolean vala_ccode_base_module_is_address_of_possible (ValaCCodeBaseModule* self, ValaExpression* e) {
	gboolean result;
	ValaExpression* _tmp0_;
	ValaMemberAccess* ma;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (e != NULL, FALSE);
	ma = _vala_code_node_ref0 ((_tmp0_ = e, VALA_IS_MEMBER_ACCESS (_tmp0_) ? ((ValaMemberAccess*) _tmp0_) : NULL));
	if (ma == NULL) {
		result = FALSE;
		_vala_code_node_unref0 (ma);
		return result;
	}
	if (vala_expression_get_symbol_reference ((ValaExpression*) ma) == NULL) {
		result = FALSE;
		_vala_code_node_unref0 (ma);
		return result;
	}
	if (VALA_IS_FORMAL_PARAMETER (vala_expression_get_symbol_reference ((ValaExpression*) ma))) {
		result = TRUE;
		_vala_code_node_unref0 (ma);
		return result;
	}
	if (VALA_IS_LOCAL_VARIABLE (vala_expression_get_symbol_reference ((ValaExpression*) ma))) {
		result = TRUE;
		_vala_code_node_unref0 (ma);
		return result;
	}
	if (VALA_IS_FIELD (vala_expression_get_symbol_reference ((ValaExpression*) ma))) {
		result = TRUE;
		_vala_code_node_unref0 (ma);
		return result;
	}
	result = FALSE;
	_vala_code_node_unref0 (ma);
	return result;
}


ValaCCodeExpression* vala_ccode_base_module_get_address_of_expression (ValaCCodeBaseModule* self, ValaExpression* e, ValaCCodeExpression* ce) {
	ValaCCodeExpression* result;
	ValaCCodeCommaExpression* ccomma;
	ValaLocalVariable* temp_decl;
	ValaCCodeExpression* ctemp;
	ValaCCodeAssignment* _tmp0_;
	ValaCCodeUnaryExpression* _tmp1_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (e != NULL, NULL);
	g_return_val_if_fail (ce != NULL, NULL);
	if (vala_ccode_base_module_is_address_of_possible (self, e)) {
		result = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ce);
		return result;
	}
	ccomma = vala_ccode_comma_expression_new ();
	temp_decl = vala_ccode_base_module_get_temp_variable (self, vala_expression_get_value_type (e), TRUE, NULL, FALSE);
	ctemp = vala_ccode_base_module_get_variable_cexpression (self, vala_symbol_get_name ((ValaSymbol*) temp_decl));
	gee_collection_add ((GeeCollection*) self->temp_vars, temp_decl);
	vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp0_ = vala_ccode_assignment_new (ctemp, ce, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
	_vala_ccode_node_unref0 (_tmp0_);
	vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp1_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, ctemp)));
	_vala_ccode_node_unref0 (_tmp1_);
	result = (ValaCCodeExpression*) ccomma;
	_vala_code_node_unref0 (temp_decl);
	_vala_ccode_node_unref0 (ctemp);
	return result;
}


gboolean vala_ccode_base_module_add_wrapper (ValaCCodeBaseModule* self, const char* wrapper_name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (wrapper_name != NULL, FALSE);
	result = gee_collection_add ((GeeCollection*) self->wrappers, wrapper_name);
	return result;
}


gboolean vala_ccode_base_module_add_generated_external_symbol (ValaCCodeBaseModule* self, ValaSymbol* external_symbol) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (external_symbol != NULL, FALSE);
	result = gee_collection_add ((GeeCollection*) self->priv->generated_external_symbols, external_symbol);
	return result;
}


ValaDataType* vala_ccode_base_module_get_data_type_for_symbol (ValaTypeSymbol* sym) {
	ValaDataType* result;
	ValaDataType* type;
	g_return_val_if_fail (sym != NULL, NULL);
	type = NULL;
	if (VALA_IS_CLASS (sym)) {
		ValaDataType* _tmp0_;
		type = (_tmp0_ = (ValaDataType*) vala_object_type_new ((ValaObjectTypeSymbol*) VALA_CLASS (sym)), _vala_code_node_unref0 (type), _tmp0_);
	} else {
		if (VALA_IS_INTERFACE (sym)) {
			ValaDataType* _tmp1_;
			type = (_tmp1_ = (ValaDataType*) vala_object_type_new ((ValaObjectTypeSymbol*) VALA_INTERFACE (sym)), _vala_code_node_unref0 (type), _tmp1_);
		} else {
			if (VALA_IS_STRUCT (sym)) {
				ValaStruct* st;
				st = _vala_code_node_ref0 (VALA_STRUCT (sym));
				if (vala_struct_is_boolean_type (st)) {
					ValaDataType* _tmp2_;
					type = (_tmp2_ = (ValaDataType*) vala_boolean_type_new (st), _vala_code_node_unref0 (type), _tmp2_);
				} else {
					if (vala_struct_is_integer_type (st)) {
						ValaDataType* _tmp3_;
						type = (_tmp3_ = (ValaDataType*) vala_integer_type_new (st, NULL, NULL), _vala_code_node_unref0 (type), _tmp3_);
					} else {
						if (vala_struct_is_floating_type (st)) {
							ValaDataType* _tmp4_;
							type = (_tmp4_ = (ValaDataType*) vala_floating_type_new (st), _vala_code_node_unref0 (type), _tmp4_);
						} else {
							ValaDataType* _tmp5_;
							type = (_tmp5_ = (ValaDataType*) vala_struct_value_type_new (st), _vala_code_node_unref0 (type), _tmp5_);
						}
					}
				}
				_vala_code_node_unref0 (st);
			} else {
				if (VALA_IS_ENUM (sym)) {
					ValaDataType* _tmp6_;
					type = (_tmp6_ = (ValaDataType*) vala_enum_value_type_new (VALA_ENUM (sym)), _vala_code_node_unref0 (type), _tmp6_);
				} else {
					if (VALA_IS_ERROR_DOMAIN (sym)) {
						ValaDataType* _tmp7_;
						type = (_tmp7_ = (ValaDataType*) vala_error_type_new (VALA_ERROR_DOMAIN (sym), NULL, NULL), _vala_code_node_unref0 (type), _tmp7_);
					} else {
						if (VALA_IS_ERROR_CODE (sym)) {
							ValaDataType* _tmp8_;
							type = (_tmp8_ = (ValaDataType*) vala_error_type_new (VALA_ERROR_DOMAIN (vala_symbol_get_parent_symbol ((ValaSymbol*) sym)), VALA_ERROR_CODE (sym), NULL), _vala_code_node_unref0 (type), _tmp8_);
						} else {
							char* _tmp10_;
							char* _tmp9_;
							vala_report_error (NULL, _tmp10_ = g_strdup_printf ("internal error: `%s' is not a supported type", _tmp9_ = vala_symbol_get_full_name ((ValaSymbol*) sym)));
							_g_free0 (_tmp10_);
							_g_free0 (_tmp9_);
							result = (ValaDataType*) vala_invalid_type_new ();
							_vala_code_node_unref0 (type);
							return result;
						}
					}
				}
			}
		}
	}
	result = type;
	return result;
}


ValaCCodeExpression* vala_ccode_base_module_default_value_for_type (ValaCCodeBaseModule* self, ValaDataType* type, gboolean initializer_expression) {
	ValaCCodeExpression* result;
	ValaTypeSymbol* _tmp0_;
	ValaStruct* st;
	ValaDataType* _tmp1_;
	ValaArrayType* array_type;
	gboolean _tmp2_;
	gboolean _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	st = _vala_code_node_ref0 ((_tmp0_ = vala_data_type_get_data_type (type), VALA_IS_STRUCT (_tmp0_) ? ((ValaStruct*) _tmp0_) : NULL));
	array_type = _vala_code_node_ref0 ((_tmp1_ = type, VALA_IS_ARRAY_TYPE (_tmp1_) ? ((ValaArrayType*) _tmp1_) : NULL));
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	if (initializer_expression) {
		_tmp3_ = !vala_data_type_get_nullable (type);
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		gboolean _tmp4_;
		gboolean _tmp5_;
		_tmp4_ = FALSE;
		_tmp5_ = FALSE;
		if (st != NULL) {
			_tmp5_ = !vala_struct_is_simple_type (st);
		} else {
			_tmp5_ = FALSE;
		}
		if (_tmp5_) {
			_tmp4_ = TRUE;
		} else {
			gboolean _tmp6_;
			_tmp6_ = FALSE;
			if (array_type != NULL) {
				_tmp6_ = vala_array_type_get_fixed_length (array_type);
			} else {
				_tmp6_ = FALSE;
			}
			_tmp4_ = _tmp6_;
		}
		_tmp2_ = _tmp4_;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		ValaCCodeInitializerList* clist;
		ValaCCodeConstant* _tmp7_;
		clist = vala_ccode_initializer_list_new ();
		vala_ccode_initializer_list_append (clist, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_constant_new ("0")));
		_vala_ccode_node_unref0 (_tmp7_);
		result = (ValaCCodeExpression*) clist;
		_vala_code_node_unref0 (st);
		_vala_code_node_unref0 (array_type);
		return result;
	} else {
		gboolean _tmp8_;
		gboolean _tmp9_;
		gboolean _tmp10_;
		gboolean _tmp11_;
		gboolean _tmp12_;
		_tmp8_ = FALSE;
		_tmp9_ = FALSE;
		_tmp10_ = FALSE;
		_tmp11_ = FALSE;
		_tmp12_ = FALSE;
		if (vala_data_type_get_data_type (type) != NULL) {
			_tmp12_ = vala_typesymbol_is_reference_type (vala_data_type_get_data_type (type));
		} else {
			_tmp12_ = FALSE;
		}
		if (_tmp12_) {
			_tmp11_ = TRUE;
		} else {
			_tmp11_ = vala_data_type_get_nullable (type);
		}
		if (_tmp11_) {
			_tmp10_ = TRUE;
		} else {
			_tmp10_ = VALA_IS_POINTER_TYPE (type);
		}
		if (_tmp10_) {
			_tmp9_ = TRUE;
		} else {
			_tmp9_ = VALA_IS_DELEGATE_TYPE (type);
		}
		if (_tmp9_) {
			_tmp8_ = TRUE;
		} else {
			gboolean _tmp13_;
			_tmp13_ = FALSE;
			if (array_type != NULL) {
				_tmp13_ = !vala_array_type_get_fixed_length (array_type);
			} else {
				_tmp13_ = FALSE;
			}
			_tmp8_ = _tmp13_;
		}
		if (_tmp8_) {
			result = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL");
			_vala_code_node_unref0 (st);
			_vala_code_node_unref0 (array_type);
			return result;
		} else {
			gboolean _tmp14_;
			_tmp14_ = FALSE;
			if (vala_data_type_get_data_type (type) != NULL) {
				char* _tmp15_;
				_tmp14_ = (_tmp15_ = vala_typesymbol_get_default_value (vala_data_type_get_data_type (type))) != NULL;
				_g_free0 (_tmp15_);
			} else {
				_tmp14_ = FALSE;
			}
			if (_tmp14_) {
				char* _tmp16_;
				ValaCCodeExpression* _tmp17_;
				result = (_tmp17_ = (ValaCCodeExpression*) vala_ccode_constant_new (_tmp16_ = vala_typesymbol_get_default_value (vala_data_type_get_data_type (type))), _g_free0 (_tmp16_), _tmp17_);
				_vala_code_node_unref0 (st);
				_vala_code_node_unref0 (array_type);
				return result;
			} else {
				if (vala_data_type_get_type_parameter (type) != NULL) {
					result = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL");
					_vala_code_node_unref0 (st);
					_vala_code_node_unref0 (array_type);
					return result;
				} else {
					if (VALA_IS_ERROR_TYPE (type)) {
						result = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL");
						_vala_code_node_unref0 (st);
						_vala_code_node_unref0 (array_type);
						return result;
					}
				}
			}
		}
	}
	result = NULL;
	_vala_code_node_unref0 (st);
	_vala_code_node_unref0 (array_type);
	return result;
}


static ValaCCodeStatement* vala_ccode_base_module_create_property_type_check_statement (ValaCCodeBaseModule* self, ValaProperty* prop, gboolean check_return_type, ValaTypeSymbol* t, gboolean non_null, const char* var_name) {
	ValaCCodeStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (prop != NULL, NULL);
	g_return_val_if_fail (t != NULL, NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	if (check_return_type) {
		result = vala_ccode_base_module_create_type_check_statement (self, (ValaCodeNode*) prop, vala_property_get_property_type (prop), t, non_null, var_name);
		return result;
	} else {
		ValaVoidType* _tmp0_;
		ValaCCodeStatement* _tmp1_;
		result = (_tmp1_ = vala_ccode_base_module_create_type_check_statement (self, (ValaCodeNode*) prop, (ValaDataType*) (_tmp0_ = vala_void_type_new (NULL)), t, non_null, var_name), _vala_code_node_unref0 (_tmp0_), _tmp1_);
		return result;
	}
}


ValaCCodeStatement* vala_ccode_base_module_create_type_check_statement (ValaCCodeBaseModule* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaTypeSymbol* t, gboolean non_null, const char* var_name) {
	ValaCCodeStatement* result;
	ValaCCodeFunctionCall* ccheck;
	ValaCodeNode* _tmp15_;
	ValaCreationMethod* cm;
	gboolean _tmp16_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (method_node != NULL, NULL);
	g_return_val_if_fail (ret_type != NULL, NULL);
	g_return_val_if_fail (t != NULL, NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	ccheck = vala_ccode_function_call_new (NULL);
	if (!vala_code_context_get_assert (self->priv->_context)) {
		result = NULL;
		_vala_ccode_node_unref0 (ccheck);
		return result;
	} else {
		gboolean _tmp0_;
		_tmp0_ = FALSE;
		if (vala_code_context_get_checking (self->priv->_context)) {
			gboolean _tmp1_;
			gboolean _tmp2_;
			_tmp1_ = FALSE;
			_tmp2_ = FALSE;
			if (VALA_IS_CLASS (t)) {
				_tmp2_ = !vala_class_get_is_compact (VALA_CLASS (t));
			} else {
				_tmp2_ = FALSE;
			}
			if (_tmp2_) {
				_tmp1_ = TRUE;
			} else {
				_tmp1_ = VALA_IS_INTERFACE (t);
			}
			_tmp0_ = _tmp1_;
		} else {
			_tmp0_ = FALSE;
		}
		if (_tmp0_) {
			ValaCCodeIdentifier* _tmp4_;
			char* _tmp3_;
			ValaCCodeFunctionCall* _tmp5_;
			ValaCCodeFunctionCall* ctype_check;
			ValaCCodeIdentifier* _tmp6_;
			ValaCCodeExpression* cexpr;
			ctype_check = (_tmp5_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new (_tmp3_ = vala_ccode_base_module_get_type_check_function (self, t)))), _vala_ccode_node_unref0 (_tmp4_), _g_free0 (_tmp3_), _tmp5_);
			vala_ccode_function_call_add_argument (ctype_check, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new (var_name)));
			_vala_ccode_node_unref0 (_tmp6_);
			cexpr = _vala_ccode_node_ref0 ((ValaCCodeExpression*) ctype_check);
			if (!non_null) {
				ValaCCodeConstant* _tmp8_;
				ValaCCodeIdentifier* _tmp7_;
				ValaCCodeBinaryExpression* _tmp9_;
				ValaCCodeBinaryExpression* cnull;
				ValaCCodeExpression* _tmp10_;
				cnull = (_tmp9_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (var_name)), (ValaCCodeExpression*) (_tmp8_ = vala_ccode_constant_new ("NULL"))), _vala_ccode_node_unref0 (_tmp8_), _vala_ccode_node_unref0 (_tmp7_), _tmp9_);
				cexpr = (_tmp10_ = (ValaCCodeExpression*) vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_OR, (ValaCCodeExpression*) cnull, (ValaCCodeExpression*) ctype_check), _vala_ccode_node_unref0 (cexpr), _tmp10_);
				_vala_ccode_node_unref0 (cnull);
			}
			vala_ccode_function_call_add_argument (ccheck, cexpr);
			_vala_ccode_node_unref0 (ctype_check);
			_vala_ccode_node_unref0 (cexpr);
		} else {
			if (!non_null) {
				result = NULL;
				_vala_ccode_node_unref0 (ccheck);
				return result;
			} else {
				gboolean _tmp11_;
				_tmp11_ = FALSE;
				if (t == VALA_TYPESYMBOL (self->glist_type)) {
					_tmp11_ = TRUE;
				} else {
					_tmp11_ = t == VALA_TYPESYMBOL (self->gslist_type);
				}
				if (_tmp11_) {
					result = NULL;
					_vala_ccode_node_unref0 (ccheck);
					return result;
				} else {
					ValaCCodeConstant* _tmp13_;
					ValaCCodeIdentifier* _tmp12_;
					ValaCCodeBinaryExpression* _tmp14_;
					ValaCCodeBinaryExpression* cnonnull;
					cnonnull = (_tmp14_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new (var_name)), (ValaCCodeExpression*) (_tmp13_ = vala_ccode_constant_new ("NULL"))), _vala_ccode_node_unref0 (_tmp13_), _vala_ccode_node_unref0 (_tmp12_), _tmp14_);
					vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) cnonnull);
					_vala_ccode_node_unref0 (cnonnull);
				}
			}
		}
	}
	cm = _vala_code_node_ref0 ((_tmp15_ = method_node, VALA_IS_CREATION_METHOD (_tmp15_) ? ((ValaCreationMethod*) _tmp15_) : NULL));
	_tmp16_ = FALSE;
	if (cm != NULL) {
		_tmp16_ = VALA_IS_OBJECT_TYPE_SYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) cm));
	} else {
		_tmp16_ = FALSE;
	}
	if (_tmp16_) {
		ValaCCodeIdentifier* _tmp17_;
		ValaCCodeConstant* _tmp18_;
		vala_ccode_function_call_set_call (ccheck, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("g_return_val_if_fail")));
		_vala_ccode_node_unref0 (_tmp17_);
		vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp18_ = vala_ccode_constant_new ("NULL")));
		_vala_ccode_node_unref0 (_tmp18_);
	} else {
		if (VALA_IS_VOID_TYPE (ret_type)) {
			ValaCCodeIdentifier* _tmp19_;
			vala_ccode_function_call_set_call (ccheck, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("g_return_if_fail")));
			_vala_ccode_node_unref0 (_tmp19_);
		} else {
			ValaCCodeIdentifier* _tmp20_;
			ValaCCodeExpression* cdefault;
			vala_ccode_function_call_set_call (ccheck, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("g_return_val_if_fail")));
			_vala_ccode_node_unref0 (_tmp20_);
			cdefault = vala_ccode_base_module_default_value_for_type (self, ret_type, FALSE);
			if (cdefault != NULL) {
				vala_ccode_function_call_add_argument (ccheck, cdefault);
			} else {
				result = NULL;
				_vala_ccode_node_unref0 (cdefault);
				_vala_ccode_node_unref0 (ccheck);
				_vala_code_node_unref0 (cm);
				return result;
			}
			_vala_ccode_node_unref0 (cdefault);
		}
	}
	result = (ValaCCodeStatement*) vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccheck);
	_vala_ccode_node_unref0 (ccheck);
	_vala_code_node_unref0 (cm);
	return result;
}


gint vala_ccode_base_module_get_param_pos (ValaCCodeBaseModule* self, double param_pos, gboolean ellipsis) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	if (!ellipsis) {
		if (param_pos >= 0) {
			result = (gint) (param_pos * 1000);
			return result;
		} else {
			result = (gint) ((100 + param_pos) * 1000);
			return result;
		}
	} else {
		if (param_pos >= 0) {
			result = (gint) ((100 + param_pos) * 1000);
			return result;
		} else {
			result = (gint) ((200 + param_pos) * 1000);
			return result;
		}
	}
}


ValaCCodeNode* vala_ccode_base_module_get_ccodenode (ValaCCodeBaseModule* self, ValaCodeNode* node) {
	ValaCCodeNode* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	if (vala_code_node_get_ccodenode (node) == NULL) {
		vala_code_node_accept (node, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	}
	result = _vala_ccode_node_ref0 (vala_code_node_get_ccodenode (node));
	return result;
}


static void vala_ccode_base_module_real_visit_class (ValaCCodeModule* base, ValaClass* cl) {
	ValaCCodeBaseModule * self;
	self = (ValaCCodeBaseModule*) base;
	g_return_if_fail (cl != NULL);
}


ValaCCodeStatement* vala_ccode_base_module_create_postcondition_statement (ValaCCodeBaseModule* self, ValaExpression* postcondition) {
	ValaCCodeStatement* result;
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* cassert;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (postcondition != NULL, NULL);
	cassert = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("g_warn_if_fail"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (cassert, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) postcondition)));
	result = (ValaCCodeStatement*) vala_ccode_expression_statement_new ((ValaCCodeExpression*) cassert);
	_vala_ccode_node_unref0 (cassert);
	return result;
}


static gboolean vala_ccode_base_module_real_is_gobject_property (ValaCCodeBaseModule* self, ValaProperty* prop) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (prop != NULL, FALSE);
	result = FALSE;
	return result;
}


gboolean vala_ccode_base_module_is_gobject_property (ValaCCodeBaseModule* self, ValaProperty* prop) {
	return VALA_CCODE_BASE_MODULE_GET_CLASS (self)->is_gobject_property (self, prop);
}


ValaDataType* vala_ccode_base_module_get_this_type (ValaCCodeBaseModule* self) {
	ValaDataType* result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = FALSE;
	if (vala_ccode_base_module_get_current_method (self) != NULL) {
		_tmp0_ = vala_method_get_binding (vala_ccode_base_module_get_current_method (self)) == MEMBER_BINDING_INSTANCE;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = _vala_code_node_ref0 (vala_formal_parameter_get_parameter_type (vala_method_get_this_parameter (vala_ccode_base_module_get_current_method (self))));
		return result;
	} else {
		gboolean _tmp1_;
		_tmp1_ = FALSE;
		if (vala_ccode_base_module_get_current_property_accessor (self) != NULL) {
			_tmp1_ = vala_property_get_binding (vala_property_accessor_get_prop (vala_ccode_base_module_get_current_property_accessor (self))) == MEMBER_BINDING_INSTANCE;
		} else {
			_tmp1_ = FALSE;
		}
		if (_tmp1_) {
			result = _vala_code_node_ref0 (vala_formal_parameter_get_parameter_type (vala_property_get_this_parameter (vala_property_accessor_get_prop (vala_ccode_base_module_get_current_property_accessor (self)))));
			return result;
		}
	}
	result = NULL;
	return result;
}


ValaCCodeFunctionCall* vala_ccode_base_module_generate_instance_cast (ValaCCodeBaseModule* self, ValaCCodeExpression* expr, ValaTypeSymbol* type) {
	ValaCCodeFunctionCall* result;
	ValaCCodeIdentifier* _tmp1_;
	char* _tmp0_;
	ValaCCodeFunctionCall* _tmp2_;
	ValaCCodeFunctionCall* _result_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (expr != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	_result_ = (_tmp2_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp1_ = vala_ccode_identifier_new (_tmp0_ = vala_typesymbol_get_upper_case_cname (type, NULL)))), _vala_ccode_node_unref0 (_tmp1_), _g_free0 (_tmp0_), _tmp2_);
	vala_ccode_function_call_add_argument (_result_, expr);
	result = _result_;
	return result;
}


static void vala_ccode_base_module_generate_struct_destroy_function (ValaCCodeBaseModule* self, ValaStruct* st) {
	char* _tmp0_;
	gboolean _tmp1_;
	char* _tmp2_;
	ValaCCodeFunction* _tmp3_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	ValaCCodeBlock* cblock;
	ValaCCodeFunction* _tmp16_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_declaration (self->source_declarations, _tmp0_ = vala_typesymbol_get_destroy_function ((ValaTypeSymbol*) st)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
	function = (_tmp3_ = vala_ccode_function_new (_tmp2_ = vala_typesymbol_get_destroy_function ((ValaTypeSymbol*) st), "void"), _g_free0 (_tmp2_), _tmp3_);
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp6_ = vala_ccode_formal_parameter_new ("self", _tmp5_ = g_strconcat (_tmp4_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp6_);
	_g_free0 (_tmp5_);
	_g_free0 (_tmp4_);
	cblock = vala_ccode_block_new ();
	{
		GeeList* _tmp7_;
		GeeIterator* _tmp8_;
		GeeIterator* _f_it;
		_f_it = (_tmp8_ = gee_iterable_iterator ((GeeIterable*) (_tmp7_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp7_), _tmp8_);
		while (TRUE) {
			ValaField* f;
			if (!gee_iterator_next (_f_it)) {
				break;
			}
			f = (ValaField*) gee_iterator_get (_f_it);
			if (vala_field_get_binding (f) == MEMBER_BINDING_INSTANCE) {
				if (vala_ccode_base_module_requires_destroy (self, vala_field_get_field_type (f))) {
					char* _tmp10_;
					ValaCCodeIdentifier* _tmp9_;
					ValaCCodeMemberAccess* _tmp11_;
					ValaCCodeMemberAccess* lhs;
					ValaMemberAccess* this_access;
					ValaDataType* _tmp12_;
					ValaCCodeIdentifier* _tmp13_;
					ValaMemberAccess* ma;
					ValaCCodeExpressionStatement* _tmp15_;
					ValaCCodeExpression* _tmp14_;
					lhs = (_tmp11_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("self")), _tmp10_ = vala_field_get_cname (f)), _g_free0 (_tmp10_), _vala_ccode_node_unref0 (_tmp9_), _tmp11_);
					this_access = vala_member_access_new_simple ("this", NULL);
					vala_expression_set_value_type ((ValaExpression*) this_access, _tmp12_ = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) f))));
					_vala_code_node_unref0 (_tmp12_);
					vala_code_node_set_ccodenode ((ValaCodeNode*) this_access, (ValaCCodeNode*) (_tmp13_ = vala_ccode_identifier_new ("(*self)")));
					_vala_ccode_node_unref0 (_tmp13_);
					ma = vala_member_access_new ((ValaExpression*) this_access, vala_symbol_get_name ((ValaSymbol*) f), NULL);
					vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) f);
					vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp15_ = vala_ccode_expression_statement_new (_tmp14_ = vala_ccode_base_module_get_unref_expression (self, (ValaCCodeExpression*) lhs, vala_field_get_field_type (f), (ValaExpression*) ma, FALSE))));
					_vala_ccode_node_unref0 (_tmp15_);
					_vala_ccode_node_unref0 (_tmp14_);
					_vala_ccode_node_unref0 (lhs);
					_vala_code_node_unref0 (this_access);
					_vala_code_node_unref0 (ma);
				}
			}
			_vala_code_node_unref0 (f);
		}
		_gee_collection_object_unref0 (_f_it);
	}
	vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp16_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_ccode_function_set_block (function, cblock);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) function);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (cblock);
}


static void vala_ccode_base_module_generate_struct_copy_function (ValaCCodeBaseModule* self, ValaStruct* st) {
	char* _tmp0_;
	gboolean _tmp1_;
	char* _tmp2_;
	ValaCCodeFunction* _tmp3_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp7_;
	char* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	ValaCCodeFormalParameter* _tmp10_;
	char* _tmp9_;
	char* _tmp8_;
	gint old_next_temp_var_id;
	GeeArrayList* old_temp_vars;
	GeeArrayList* old_temp_ref_vars;
	GeeMap* old_variable_name_map;
	GeeArrayList* _tmp11_;
	GeeArrayList* _tmp12_;
	GeeMap* _tmp13_;
	ValaCCodeBlock* cblock;
	ValaCCodeFragment* cfrag;
	GeeArrayList* _tmp47_;
	GeeArrayList* _tmp48_;
	GeeMap* _tmp49_;
	ValaCCodeFunction* _tmp50_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (st != NULL);
	if ((_tmp1_ = vala_ccode_declaration_space_add_declaration (self->source_declarations, _tmp0_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st)), _g_free0 (_tmp0_), _tmp1_)) {
		return;
	}
	function = (_tmp3_ = vala_ccode_function_new (_tmp2_ = vala_typesymbol_get_copy_function ((ValaTypeSymbol*) st), "void"), _g_free0 (_tmp2_), _tmp3_);
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp7_ = vala_ccode_formal_parameter_new ("self", _tmp6_ = g_strconcat (_tmp5_ = g_strconcat ("const ", _tmp4_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), NULL), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp7_);
	_g_free0 (_tmp6_);
	_g_free0 (_tmp5_);
	_g_free0 (_tmp4_);
	vala_ccode_function_add_parameter (function, _tmp10_ = vala_ccode_formal_parameter_new ("dest", _tmp9_ = g_strconcat (_tmp8_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) st, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp10_);
	_g_free0 (_tmp9_);
	_g_free0 (_tmp8_);
	old_next_temp_var_id = self->next_temp_var_id;
	old_temp_vars = _gee_collection_object_ref0 (self->temp_vars);
	old_temp_ref_vars = _gee_collection_object_ref0 (self->temp_ref_vars);
	old_variable_name_map = _gee_collection_object_ref0 (self->variable_name_map);
	self->next_temp_var_id = 0;
	self->temp_vars = (_tmp11_ = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), _gee_collection_object_unref0 (self->temp_vars), _tmp11_);
	self->temp_ref_vars = (_tmp12_ = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), _gee_collection_object_unref0 (self->temp_ref_vars), _tmp12_);
	self->variable_name_map = (_tmp13_ = (GeeMap*) gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal, g_direct_equal), _gee_collection_object_unref0 (self->variable_name_map), _tmp13_);
	cblock = vala_ccode_block_new ();
	cfrag = vala_ccode_fragment_new ();
	vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) cfrag);
	{
		GeeList* _tmp14_;
		GeeIterator* _tmp15_;
		GeeIterator* _f_it;
		_f_it = (_tmp15_ = gee_iterable_iterator ((GeeIterable*) (_tmp14_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp14_), _tmp15_);
		while (TRUE) {
			ValaField* f;
			if (!gee_iterator_next (_f_it)) {
				break;
			}
			f = (ValaField*) gee_iterator_get (_f_it);
			if (vala_field_get_binding (f) == MEMBER_BINDING_INSTANCE) {
				ValaCCodeIdentifier* _tmp16_;
				ValaCCodeExpression* _tmp17_;
				ValaCCodeExpression* copy;
				ValaCCodeIdentifier* _tmp21_;
				ValaCCodeMemberAccess* _tmp22_;
				ValaCCodeMemberAccess* dest;
				ValaDataType* _tmp23_;
				ValaArrayType* array_type;
				gboolean _tmp24_;
				copy = (_tmp17_ = (ValaCCodeExpression*) vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new ("self")), vala_symbol_get_name ((ValaSymbol*) f)), _vala_ccode_node_unref0 (_tmp16_), _tmp17_);
				if (vala_ccode_base_module_requires_copy (self, vala_field_get_field_type (f))) {
					ValaMemberAccess* this_access;
					ValaDataType* _tmp18_;
					ValaCCodeIdentifier* _tmp19_;
					ValaMemberAccess* ma;
					ValaCCodeExpression* _tmp20_;
					this_access = vala_member_access_new_simple ("this", NULL);
					vala_expression_set_value_type ((ValaExpression*) this_access, _tmp18_ = vala_ccode_base_module_get_data_type_for_symbol (VALA_TYPESYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) f))));
					_vala_code_node_unref0 (_tmp18_);
					vala_code_node_set_ccodenode ((ValaCodeNode*) this_access, (ValaCCodeNode*) (_tmp19_ = vala_ccode_identifier_new ("(*self)")));
					_vala_ccode_node_unref0 (_tmp19_);
					ma = vala_member_access_new ((ValaExpression*) this_access, vala_symbol_get_name ((ValaSymbol*) f), NULL);
					vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) f);
					copy = (_tmp20_ = vala_ccode_base_module_get_ref_cexpression (self, vala_field_get_field_type (f), copy, (ValaExpression*) ma, (ValaCodeNode*) f), _vala_ccode_node_unref0 (copy), _tmp20_);
					_vala_code_node_unref0 (this_access);
					_vala_code_node_unref0 (ma);
				}
				dest = (_tmp22_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("dest")), vala_symbol_get_name ((ValaSymbol*) f)), _vala_ccode_node_unref0 (_tmp21_), _tmp22_);
				array_type = _vala_code_node_ref0 ((_tmp23_ = vala_field_get_field_type (f), VALA_IS_ARRAY_TYPE (_tmp23_) ? ((ValaArrayType*) _tmp23_) : NULL));
				_tmp24_ = FALSE;
				if (array_type != NULL) {
					_tmp24_ = vala_array_type_get_fixed_length (array_type);
				} else {
					_tmp24_ = FALSE;
				}
				if (_tmp24_) {
					ValaCCodeIdentifier* _tmp25_;
					ValaCCodeFunctionCall* _tmp26_;
					ValaCCodeFunctionCall* sizeof_call;
					ValaCCodeIdentifier* _tmp28_;
					char* _tmp27_;
					ValaCCodeConstant* _tmp30_;
					char* _tmp29_;
					ValaCCodeBinaryExpression* _tmp31_;
					ValaCCodeBinaryExpression* size;
					ValaCCodeIdentifier* _tmp32_;
					ValaCCodeFunctionCall* _tmp33_;
					ValaCCodeFunctionCall* array_copy_call;
					ValaCCodeExpressionStatement* _tmp34_;
					vala_ccode_declaration_space_add_include (self->source_declarations, "string.h", FALSE);
					sizeof_call = (_tmp26_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp25_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp25_), _tmp26_);
					vala_ccode_function_call_add_argument (sizeof_call, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new (_tmp27_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
					_vala_ccode_node_unref0 (_tmp28_);
					_g_free0 (_tmp27_);
					size = (_tmp31_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_constant_new (_tmp29_ = g_strdup_printf ("%d", vala_array_type_get_length (array_type)))), (ValaCCodeExpression*) sizeof_call), _vala_ccode_node_unref0 (_tmp30_), _g_free0 (_tmp29_), _tmp31_);
					array_copy_call = (_tmp33_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp32_ = vala_ccode_identifier_new ("memcpy"))), _vala_ccode_node_unref0 (_tmp32_), _tmp33_);
					vala_ccode_function_call_add_argument (array_copy_call, (ValaCCodeExpression*) dest);
					vala_ccode_function_call_add_argument (array_copy_call, copy);
					vala_ccode_function_call_add_argument (array_copy_call, (ValaCCodeExpression*) size);
					vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp34_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) array_copy_call)));
					_vala_ccode_node_unref0 (_tmp34_);
					_vala_ccode_node_unref0 (sizeof_call);
					_vala_ccode_node_unref0 (size);
					_vala_ccode_node_unref0 (array_copy_call);
				} else {
					ValaCCodeExpressionStatement* _tmp36_;
					ValaCCodeAssignment* _tmp35_;
					gboolean _tmp37_;
					vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp36_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp35_ = vala_ccode_assignment_new ((ValaCCodeExpression*) dest, copy, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
					_vala_ccode_node_unref0 (_tmp36_);
					_vala_ccode_node_unref0 (_tmp35_);
					_tmp37_ = FALSE;
					if (array_type != NULL) {
						_tmp37_ = !vala_field_get_no_array_length (f);
					} else {
						_tmp37_ = FALSE;
					}
					if (_tmp37_) {
						{
							gint dim;
							dim = 1;
							{
								gboolean _tmp38_;
								_tmp38_ = TRUE;
								while (TRUE) {
									char* _tmp40_;
									ValaCCodeIdentifier* _tmp39_;
									ValaCCodeMemberAccess* _tmp41_;
									ValaCCodeMemberAccess* len_src;
									char* _tmp43_;
									ValaCCodeIdentifier* _tmp42_;
									ValaCCodeMemberAccess* _tmp44_;
									ValaCCodeMemberAccess* len_dest;
									ValaCCodeExpressionStatement* _tmp46_;
									ValaCCodeAssignment* _tmp45_;
									if (!_tmp38_) {
										dim++;
									}
									_tmp38_ = FALSE;
									if (!(dim <= vala_array_type_get_rank (array_type))) {
										break;
									}
									len_src = (_tmp41_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new ("self")), _tmp40_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, vala_symbol_get_name ((ValaSymbol*) f), dim)), _g_free0 (_tmp40_), _vala_ccode_node_unref0 (_tmp39_), _tmp41_);
									len_dest = (_tmp44_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("dest")), _tmp43_ = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, vala_symbol_get_name ((ValaSymbol*) f), dim)), _g_free0 (_tmp43_), _vala_ccode_node_unref0 (_tmp42_), _tmp44_);
									vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) (_tmp46_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp45_ = vala_ccode_assignment_new ((ValaCCodeExpression*) len_dest, (ValaCCodeExpression*) len_src, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
									_vala_ccode_node_unref0 (_tmp46_);
									_vala_ccode_node_unref0 (_tmp45_);
									_vala_ccode_node_unref0 (len_src);
									_vala_ccode_node_unref0 (len_dest);
								}
							}
						}
					}
				}
				_vala_ccode_node_unref0 (copy);
				_vala_ccode_node_unref0 (dest);
				_vala_code_node_unref0 (array_type);
			}
			_vala_code_node_unref0 (f);
		}
		_gee_collection_object_unref0 (_f_it);
	}
	vala_ccode_base_module_append_temp_decl (self, cfrag, (GeeList*) self->temp_vars);
	gee_collection_clear ((GeeCollection*) self->temp_vars);
	self->next_temp_var_id = old_next_temp_var_id;
	self->temp_vars = (_tmp47_ = _gee_collection_object_ref0 (old_temp_vars), _gee_collection_object_unref0 (self->temp_vars), _tmp47_);
	self->temp_ref_vars = (_tmp48_ = _gee_collection_object_ref0 (old_temp_ref_vars), _gee_collection_object_unref0 (self->temp_ref_vars), _tmp48_);
	self->variable_name_map = (_tmp49_ = _gee_collection_object_ref0 (old_variable_name_map), _gee_collection_object_unref0 (self->variable_name_map), _tmp49_);
	vala_ccode_declaration_space_add_type_member_declaration (self->source_declarations, (ValaCCodeNode*) (_tmp50_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp50_);
	vala_ccode_function_set_block (function, cblock);
	vala_ccode_fragment_append (self->source_type_member_definition, (ValaCCodeNode*) function);
	_vala_ccode_node_unref0 (function);
	_gee_collection_object_unref0 (old_temp_vars);
	_gee_collection_object_unref0 (old_temp_ref_vars);
	_gee_collection_object_unref0 (old_variable_name_map);
	_vala_ccode_node_unref0 (cblock);
	_vala_ccode_node_unref0 (cfrag);
}


ValaCodeContext* vala_ccode_base_module_get_context (ValaCCodeBaseModule* self) {
	ValaCodeContext* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_context;
	return result;
}


static gpointer _vala_code_context_ref0 (gpointer self) {
	return self ? vala_code_context_ref (self) : NULL;
}


void vala_ccode_base_module_set_context (ValaCCodeBaseModule* self, ValaCodeContext* value) {
	ValaCodeContext* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_context = (_tmp0_ = _vala_code_context_ref0 (value), _vala_code_context_unref0 (self->priv->_context), _tmp0_);
}


ValaTypeSymbol* vala_ccode_base_module_get_current_type_symbol (ValaCCodeBaseModule* self) {
	ValaTypeSymbol* result;
	ValaSymbol* sym;
	g_return_val_if_fail (self != NULL, NULL);
	sym = _vala_code_node_ref0 (self->current_symbol);
	while (TRUE) {
		ValaSymbol* _tmp0_;
		if (!(sym != NULL)) {
			break;
		}
		if (VALA_IS_TYPESYMBOL (sym)) {
			result = VALA_TYPESYMBOL (sym);
			_vala_code_node_unref0 (sym);
			return result;
		}
		sym = (_tmp0_ = _vala_code_node_ref0 (vala_symbol_get_parent_symbol (sym)), _vala_code_node_unref0 (sym), _tmp0_);
	}
	result = NULL;
	_vala_code_node_unref0 (sym);
	return result;
}


ValaClass* vala_ccode_base_module_get_current_class (ValaCCodeBaseModule* self) {
	ValaClass* result;
	ValaTypeSymbol* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	result = (_tmp0_ = vala_ccode_base_module_get_current_type_symbol (self), VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL);
	return result;
}


ValaMethod* vala_ccode_base_module_get_current_method (ValaCCodeBaseModule* self) {
	ValaMethod* result;
	ValaSymbol* sym;
	ValaSymbol* _tmp1_;
	g_return_val_if_fail (self != NULL, NULL);
	sym = _vala_code_node_ref0 (self->current_symbol);
	while (TRUE) {
		ValaSymbol* _tmp0_;
		if (!VALA_IS_BLOCK (sym)) {
			break;
		}
		sym = (_tmp0_ = _vala_code_node_ref0 (vala_symbol_get_parent_symbol (sym)), _vala_code_node_unref0 (sym), _tmp0_);
	}
	result = (_tmp1_ = sym, VALA_IS_METHOD (_tmp1_) ? ((ValaMethod*) _tmp1_) : NULL);
	_vala_code_node_unref0 (sym);
	return result;
	_vala_code_node_unref0 (sym);
}


ValaPropertyAccessor* vala_ccode_base_module_get_current_property_accessor (ValaCCodeBaseModule* self) {
	ValaPropertyAccessor* result;
	ValaSymbol* sym;
	ValaSymbol* _tmp1_;
	g_return_val_if_fail (self != NULL, NULL);
	sym = _vala_code_node_ref0 (self->current_symbol);
	while (TRUE) {
		ValaSymbol* _tmp0_;
		if (!VALA_IS_BLOCK (sym)) {
			break;
		}
		sym = (_tmp0_ = _vala_code_node_ref0 (vala_symbol_get_parent_symbol (sym)), _vala_code_node_unref0 (sym), _tmp0_);
	}
	result = (_tmp1_ = sym, VALA_IS_PROPERTY_ACCESSOR (_tmp1_) ? ((ValaPropertyAccessor*) _tmp1_) : NULL);
	_vala_code_node_unref0 (sym);
	return result;
	_vala_code_node_unref0 (sym);
}


ValaDataType* vala_ccode_base_module_get_current_return_type (ValaCCodeBaseModule* self) {
	ValaDataType* result;
	ValaMethod* m;
	ValaPropertyAccessor* acc;
	g_return_val_if_fail (self != NULL, NULL);
	m = _vala_code_node_ref0 (vala_ccode_base_module_get_current_method (self));
	if (m != NULL) {
		result = vala_method_get_return_type (m);
		_vala_code_node_unref0 (m);
		return result;
	}
	acc = _vala_code_node_ref0 (vala_ccode_base_module_get_current_property_accessor (self));
	if (acc != NULL) {
		if (vala_property_accessor_get_readable (acc)) {
			result = vala_property_accessor_get_value_type (acc);
			_vala_code_node_unref0 (m);
			_vala_code_node_unref0 (acc);
			return result;
		} else {
			result = self->void_type;
			_vala_code_node_unref0 (m);
			_vala_code_node_unref0 (acc);
			return result;
		}
	}
	result = NULL;
	_vala_code_node_unref0 (m);
	_vala_code_node_unref0 (acc);
	return result;
}


gboolean vala_ccode_base_module_get_in_creation_method (ValaCCodeBaseModule* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = VALA_IS_CREATION_METHOD (vala_ccode_base_module_get_current_method (self));
	return result;
}


static void vala_ccode_base_module_class_init (ValaCCodeBaseModuleClass * klass) {
	vala_ccode_base_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->finalize = vala_ccode_base_module_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeBaseModulePrivate));
	VALA_CCODE_MODULE_CLASS (klass)->emit = vala_ccode_base_module_real_emit;
	VALA_CCODE_MODULE_CLASS (klass)->get_value_setter_function = vala_ccode_base_module_real_get_value_setter_function;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->append_vala_array_free = vala_ccode_base_module_real_append_vala_array_free;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->append_vala_array_move = vala_ccode_base_module_real_append_vala_array_move;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->append_vala_array_length = vala_ccode_base_module_real_append_vala_array_length;
	VALA_CCODE_MODULE_CLASS (klass)->visit_source_file = vala_ccode_base_module_real_visit_source_file;
	VALA_CCODE_MODULE_CLASS (klass)->visit_enum = vala_ccode_base_module_real_visit_enum;
	VALA_CCODE_MODULE_CLASS (klass)->visit_member = vala_ccode_base_module_real_visit_member;
	VALA_CCODE_MODULE_CLASS (klass)->visit_constant = vala_ccode_base_module_real_visit_constant;
	VALA_CCODE_MODULE_CLASS (klass)->visit_field = vala_ccode_base_module_real_visit_field;
	VALA_CCODE_MODULE_CLASS (klass)->visit_formal_parameter = vala_ccode_base_module_real_visit_formal_parameter;
	VALA_CCODE_MODULE_CLASS (klass)->visit_property = vala_ccode_base_module_real_visit_property;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_class_struct_declaration = vala_ccode_base_module_real_generate_class_struct_declaration;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_struct_declaration = vala_ccode_base_module_real_generate_struct_declaration;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_delegate_declaration = vala_ccode_base_module_real_generate_delegate_declaration;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_cparameters = vala_ccode_base_module_real_generate_cparameters;
	VALA_CCODE_MODULE_CLASS (klass)->visit_property_accessor = vala_ccode_base_module_real_visit_property_accessor;
	VALA_CCODE_MODULE_CLASS (klass)->visit_destructor = vala_ccode_base_module_real_visit_destructor;
	VALA_CCODE_MODULE_CLASS (klass)->visit_block = vala_ccode_base_module_real_visit_block;
	VALA_CCODE_MODULE_CLASS (klass)->visit_empty_statement = vala_ccode_base_module_real_visit_empty_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_declaration_statement = vala_ccode_base_module_real_visit_declaration_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_local_variable = vala_ccode_base_module_real_visit_local_variable;
	VALA_CCODE_MODULE_CLASS (klass)->visit_initializer_list = vala_ccode_base_module_real_visit_initializer_list;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_dup_func_expression = vala_ccode_base_module_real_get_dup_func_expression;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->append_struct_array_free = vala_ccode_base_module_real_append_struct_array_free;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_unref_expression = vala_ccode_base_module_real_get_unref_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_end_full_expression = vala_ccode_base_module_real_visit_end_full_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_expression_statement = vala_ccode_base_module_real_visit_expression_statement;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->append_local_free = vala_ccode_base_module_real_append_local_free;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->variable_accessible_in_finally = vala_ccode_base_module_real_variable_accessible_in_finally;
	VALA_CCODE_MODULE_CLASS (klass)->visit_return_statement = vala_ccode_base_module_real_visit_return_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_lock_statement = vala_ccode_base_module_real_visit_lock_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_delete_statement = vala_ccode_base_module_real_visit_delete_statement;
	VALA_CCODE_MODULE_CLASS (klass)->visit_expression = vala_ccode_base_module_real_visit_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_boolean_literal = vala_ccode_base_module_real_visit_boolean_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_character_literal = vala_ccode_base_module_real_visit_character_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_integer_literal = vala_ccode_base_module_real_visit_integer_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_real_literal = vala_ccode_base_module_real_visit_real_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_string_literal = vala_ccode_base_module_real_visit_string_literal;
	VALA_CCODE_MODULE_CLASS (klass)->visit_null_literal = vala_ccode_base_module_real_visit_null_literal;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_delegate_target_cname = vala_ccode_base_module_real_get_delegate_target_cname;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_delegate_target_cexpression = vala_ccode_base_module_real_get_delegate_target_cexpression;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_delegate_target_destroy_notify_cname = vala_ccode_base_module_real_get_delegate_target_destroy_notify_cname;
	VALA_CCODE_MODULE_CLASS (klass)->visit_base_access = vala_ccode_base_module_real_visit_base_access;
	VALA_CCODE_MODULE_CLASS (klass)->visit_postfix_expression = vala_ccode_base_module_real_visit_postfix_expression;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_ref_cexpression = vala_ccode_base_module_real_get_ref_cexpression;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_class_declaration = vala_ccode_base_module_real_generate_class_declaration;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_interface_declaration = vala_ccode_base_module_real_generate_interface_declaration;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_method_declaration = vala_ccode_base_module_real_generate_method_declaration;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->generate_error_domain_declaration = vala_ccode_base_module_real_generate_error_domain_declaration;
	VALA_CCODE_MODULE_CLASS (klass)->visit_object_creation_expression = vala_ccode_base_module_real_visit_object_creation_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_sizeof_expression = vala_ccode_base_module_real_visit_sizeof_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_typeof_expression = vala_ccode_base_module_real_visit_typeof_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_unary_expression = vala_ccode_base_module_real_visit_unary_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_cast_expression = vala_ccode_base_module_real_visit_cast_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_pointer_indirection = vala_ccode_base_module_real_visit_pointer_indirection;
	VALA_CCODE_MODULE_CLASS (klass)->visit_addressof_expression = vala_ccode_base_module_real_visit_addressof_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_reference_transfer_expression = vala_ccode_base_module_real_visit_reference_transfer_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_binary_expression = vala_ccode_base_module_real_visit_binary_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_type_check = vala_ccode_base_module_real_visit_type_check;
	VALA_CCODE_MODULE_CLASS (klass)->visit_lambda_expression = vala_ccode_base_module_real_visit_lambda_expression;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->get_implicit_cast_expression = vala_ccode_base_module_real_get_implicit_cast_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_class = vala_ccode_base_module_real_visit_class;
	VALA_CCODE_BASE_MODULE_CLASS (klass)->is_gobject_property = vala_ccode_base_module_real_is_gobject_property;
}


static void vala_ccode_base_module_instance_init (ValaCCodeBaseModule * self) {
	self->priv = VALA_CCODE_BASE_MODULE_GET_PRIVATE (self);
	self->temp_vars = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
	self->temp_ref_vars = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
	self->next_temp_var_id = 0;
	self->in_constructor = FALSE;
	self->in_static_or_class_context = FALSE;
	self->current_method_inner_error = FALSE;
	self->next_coroutine_state = 1;
	self->priv->next_block_id = 0;
	self->priv->block_map = (GeeMap*) gee_hash_map_new (VALA_TYPE_BLOCK, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, G_TYPE_INT, NULL, NULL, g_direct_hash, g_direct_equal, g_direct_equal);
	self->void_type = (ValaDataType*) vala_void_type_new (NULL);
	self->in_plugin = FALSE;
	self->variable_name_map = (GeeMap*) gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal, g_direct_equal);
}


static void vala_ccode_base_module_finalize (ValaCCodeModule* obj) {
	ValaCCodeBaseModule * self;
	self = VALA_CCODE_BASE_MODULE (obj);
	_vala_code_context_unref0 (self->priv->_context);
	_vala_code_node_unref0 (self->root_symbol);
	_vala_code_node_unref0 (self->current_symbol);
	_vala_code_node_unref0 (self->current_try);
	_vala_ccode_declaration_space_unref0 (self->header_declarations);
	_vala_ccode_declaration_space_unref0 (self->internal_header_declarations);
	_vala_ccode_declaration_space_unref0 (self->source_declarations);
	_vala_ccode_node_unref0 (self->source_signal_marshaller_declaration);
	_vala_ccode_node_unref0 (self->source_type_member_definition);
	_vala_ccode_node_unref0 (self->class_init_fragment);
	_vala_ccode_node_unref0 (self->base_init_fragment);
	_vala_ccode_node_unref0 (self->class_finalize_fragment);
	_vala_ccode_node_unref0 (self->base_finalize_fragment);
	_vala_ccode_node_unref0 (self->instance_init_fragment);
	_vala_ccode_node_unref0 (self->instance_finalize_fragment);
	_vala_ccode_node_unref0 (self->source_signal_marshaller_definition);
	_vala_ccode_node_unref0 (self->module_init_fragment);
	_vala_ccode_node_unref0 (self->param_spec_struct);
	_vala_ccode_node_unref0 (self->closure_struct);
	_vala_ccode_node_unref0 (self->prop_enum);
	_vala_ccode_node_unref0 (self->function);
	_vala_ccode_node_unref0 (self->pre_statement_fragment);
	_gee_collection_object_unref0 (self->temp_vars);
	_gee_collection_object_unref0 (self->temp_ref_vars);
	_gee_collection_object_unref0 (self->user_marshal_set);
	_gee_collection_object_unref0 (self->predefined_marshal_set);
	_gee_collection_object_unref0 (self->priv->reserved_identifiers);
	_gee_collection_object_unref0 (self->priv->block_map);
	_vala_code_node_unref0 (self->void_type);
	_vala_code_node_unref0 (self->bool_type);
	_vala_code_node_unref0 (self->char_type);
	_vala_code_node_unref0 (self->uchar_type);
	_vala_code_node_unref0 (self->unichar_type);
	_vala_code_node_unref0 (self->short_type);
	_vala_code_node_unref0 (self->ushort_type);
	_vala_code_node_unref0 (self->int_type);
	_vala_code_node_unref0 (self->uint_type);
	_vala_code_node_unref0 (self->long_type);
	_vala_code_node_unref0 (self->ulong_type);
	_vala_code_node_unref0 (self->int8_type);
	_vala_code_node_unref0 (self->uint8_type);
	_vala_code_node_unref0 (self->int16_type);
	_vala_code_node_unref0 (self->uint16_type);
	_vala_code_node_unref0 (self->int32_type);
	_vala_code_node_unref0 (self->uint32_type);
	_vala_code_node_unref0 (self->int64_type);
	_vala_code_node_unref0 (self->uint64_type);
	_vala_code_node_unref0 (self->string_type);
	_vala_code_node_unref0 (self->float_type);
	_vala_code_node_unref0 (self->double_type);
	_vala_code_node_unref0 (self->gtype_type);
	_vala_code_node_unref0 (self->gobject_type);
	_vala_code_node_unref0 (self->gerror_type);
	_vala_code_node_unref0 (self->glist_type);
	_vala_code_node_unref0 (self->gslist_type);
	_vala_code_node_unref0 (self->gvaluearray_type);
	_vala_code_node_unref0 (self->gstringbuilder_type);
	_vala_code_node_unref0 (self->garray_type);
	_vala_code_node_unref0 (self->gbytearray_type);
	_vala_code_node_unref0 (self->gptrarray_type);
	_vala_code_node_unref0 (self->gquark_type);
	_vala_code_node_unref0 (self->gvalue_type);
	_vala_code_node_unref0 (self->mutex_type);
	_vala_code_node_unref0 (self->type_module_type);
	_vala_code_node_unref0 (self->dbus_object_type);
	_g_free0 (self->module_init_param_name);
	_gee_collection_object_unref0 (self->wrappers);
	_gee_collection_object_unref0 (self->priv->generated_external_symbols);
	_gee_collection_object_unref0 (self->variable_name_map);
	VALA_CCODE_MODULE_CLASS (vala_ccode_base_module_parent_class)->finalize (obj);
}


GType vala_ccode_base_module_get_type (void) {
	static GType vala_ccode_base_module_type_id = 0;
	if (vala_ccode_base_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeBaseModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_base_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeBaseModule), 0, (GInstanceInitFunc) vala_ccode_base_module_instance_init, NULL };
		vala_ccode_base_module_type_id = g_type_register_static (VALA_TYPE_CCODE_MODULE, "ValaCCodeBaseModule", &g_define_type_info, 0);
	}
	return vala_ccode_base_module_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




