/* valadbusclientmodule.vala
 *
 * Copyright (C) 2007-2009  Jürg Billeter
*  Copyright (C) 2008  Philip Van Hoof
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 * 	Philip Van Hoof <pvanhoof@gnome.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>
#include <float.h>
#include <math.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))

typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;

#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))

typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;

#define VALA_TYPE_CCODE_STRUCT_MODULE (vala_ccode_struct_module_get_type ())
#define VALA_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModule))
#define VALA_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
#define VALA_IS_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_IS_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_CCODE_STRUCT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))

typedef struct _ValaCCodeStructModule ValaCCodeStructModule;
typedef struct _ValaCCodeStructModuleClass ValaCCodeStructModuleClass;
typedef struct _ValaCCodeStructModulePrivate ValaCCodeStructModulePrivate;

#define VALA_TYPE_CCODE_METHOD_MODULE (vala_ccode_method_module_get_type ())
#define VALA_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModule))
#define VALA_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))
#define VALA_IS_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_IS_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_CCODE_METHOD_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))

typedef struct _ValaCCodeMethodModule ValaCCodeMethodModule;
typedef struct _ValaCCodeMethodModuleClass ValaCCodeMethodModuleClass;
typedef struct _ValaCCodeMethodModulePrivate ValaCCodeMethodModulePrivate;

#define VALA_TYPE_CCODE_CONTROL_FLOW_MODULE (vala_ccode_control_flow_module_get_type ())
#define VALA_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModule))
#define VALA_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_CCODE_CONTROL_FLOW_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))

typedef struct _ValaCCodeControlFlowModule ValaCCodeControlFlowModule;
typedef struct _ValaCCodeControlFlowModuleClass ValaCCodeControlFlowModuleClass;
typedef struct _ValaCCodeControlFlowModulePrivate ValaCCodeControlFlowModulePrivate;

#define VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE (vala_ccode_member_access_module_get_type ())
#define VALA_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModule))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))

typedef struct _ValaCCodeMemberAccessModule ValaCCodeMemberAccessModule;
typedef struct _ValaCCodeMemberAccessModuleClass ValaCCodeMemberAccessModuleClass;
typedef struct _ValaCCodeMemberAccessModulePrivate ValaCCodeMemberAccessModulePrivate;

#define VALA_TYPE_CCODE_ASSIGNMENT_MODULE (vala_ccode_assignment_module_get_type ())
#define VALA_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModule))
#define VALA_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_CCODE_ASSIGNMENT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))

typedef struct _ValaCCodeAssignmentModule ValaCCodeAssignmentModule;
typedef struct _ValaCCodeAssignmentModuleClass ValaCCodeAssignmentModuleClass;
typedef struct _ValaCCodeAssignmentModulePrivate ValaCCodeAssignmentModulePrivate;

#define VALA_TYPE_CCODE_METHOD_CALL_MODULE (vala_ccode_method_call_module_get_type ())
#define VALA_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModule))
#define VALA_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))
#define VALA_IS_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_IS_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_CCODE_METHOD_CALL_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))

typedef struct _ValaCCodeMethodCallModule ValaCCodeMethodCallModule;
typedef struct _ValaCCodeMethodCallModuleClass ValaCCodeMethodCallModuleClass;
typedef struct _ValaCCodeMethodCallModulePrivate ValaCCodeMethodCallModulePrivate;

#define VALA_TYPE_CCODE_ARRAY_MODULE (vala_ccode_array_module_get_type ())
#define VALA_CCODE_ARRAY_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_ARRAY_MODULE, ValaCCodeArrayModule))
#define VALA_CCODE_ARRAY_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_ARRAY_MODULE, ValaCCodeArrayModuleClass))
#define VALA_IS_CCODE_ARRAY_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_ARRAY_MODULE))
#define VALA_IS_CCODE_ARRAY_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_ARRAY_MODULE))
#define VALA_CCODE_ARRAY_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_ARRAY_MODULE, ValaCCodeArrayModuleClass))

typedef struct _ValaCCodeArrayModule ValaCCodeArrayModule;
typedef struct _ValaCCodeArrayModuleClass ValaCCodeArrayModuleClass;
typedef struct _ValaCCodeArrayModulePrivate ValaCCodeArrayModulePrivate;

#define VALA_TYPE_CCODE_DELEGATE_MODULE (vala_ccode_delegate_module_get_type ())
#define VALA_CCODE_DELEGATE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DELEGATE_MODULE, ValaCCodeDelegateModule))
#define VALA_CCODE_DELEGATE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DELEGATE_MODULE, ValaCCodeDelegateModuleClass))
#define VALA_IS_CCODE_DELEGATE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DELEGATE_MODULE))
#define VALA_IS_CCODE_DELEGATE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DELEGATE_MODULE))
#define VALA_CCODE_DELEGATE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DELEGATE_MODULE, ValaCCodeDelegateModuleClass))

typedef struct _ValaCCodeDelegateModule ValaCCodeDelegateModule;
typedef struct _ValaCCodeDelegateModuleClass ValaCCodeDelegateModuleClass;
typedef struct _ValaCCodeDelegateModulePrivate ValaCCodeDelegateModulePrivate;

#define VALA_TYPE_GERROR_MODULE (vala_gerror_module_get_type ())
#define VALA_GERROR_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GERROR_MODULE, ValaGErrorModule))
#define VALA_GERROR_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GERROR_MODULE, ValaGErrorModuleClass))
#define VALA_IS_GERROR_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GERROR_MODULE))
#define VALA_IS_GERROR_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GERROR_MODULE))
#define VALA_GERROR_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GERROR_MODULE, ValaGErrorModuleClass))

typedef struct _ValaGErrorModule ValaGErrorModule;
typedef struct _ValaGErrorModuleClass ValaGErrorModuleClass;
typedef struct _ValaGErrorModulePrivate ValaGErrorModulePrivate;

#define VALA_TYPE_GTYPE_MODULE (vala_gtype_module_get_type ())
#define VALA_GTYPE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GTYPE_MODULE, ValaGTypeModule))
#define VALA_GTYPE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GTYPE_MODULE, ValaGTypeModuleClass))
#define VALA_IS_GTYPE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GTYPE_MODULE))
#define VALA_IS_GTYPE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GTYPE_MODULE))
#define VALA_GTYPE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GTYPE_MODULE, ValaGTypeModuleClass))

typedef struct _ValaGTypeModule ValaGTypeModule;
typedef struct _ValaGTypeModuleClass ValaGTypeModuleClass;
typedef struct _ValaGTypeModulePrivate ValaGTypeModulePrivate;

#define VALA_TYPE_TYPEREGISTER_FUNCTION (vala_typeregister_function_get_type ())
#define VALA_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunction))
#define VALA_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
#define VALA_IS_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_IS_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_TYPEREGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))

typedef struct _ValaTypeRegisterFunction ValaTypeRegisterFunction;
typedef struct _ValaTypeRegisterFunctionClass ValaTypeRegisterFunctionClass;

#define VALA_TYPE_GOBJECT_MODULE (vala_gobject_module_get_type ())
#define VALA_GOBJECT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GOBJECT_MODULE, ValaGObjectModule))
#define VALA_GOBJECT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GOBJECT_MODULE, ValaGObjectModuleClass))
#define VALA_IS_GOBJECT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GOBJECT_MODULE))
#define VALA_IS_GOBJECT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GOBJECT_MODULE))
#define VALA_GOBJECT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GOBJECT_MODULE, ValaGObjectModuleClass))

typedef struct _ValaGObjectModule ValaGObjectModule;
typedef struct _ValaGObjectModuleClass ValaGObjectModuleClass;
typedef struct _ValaGObjectModulePrivate ValaGObjectModulePrivate;

#define VALA_TYPE_GSIGNAL_MODULE (vala_gsignal_module_get_type ())
#define VALA_GSIGNAL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GSIGNAL_MODULE, ValaGSignalModule))
#define VALA_GSIGNAL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GSIGNAL_MODULE, ValaGSignalModuleClass))
#define VALA_IS_GSIGNAL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GSIGNAL_MODULE))
#define VALA_IS_GSIGNAL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GSIGNAL_MODULE))
#define VALA_GSIGNAL_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GSIGNAL_MODULE, ValaGSignalModuleClass))

typedef struct _ValaGSignalModule ValaGSignalModule;
typedef struct _ValaGSignalModuleClass ValaGSignalModuleClass;
typedef struct _ValaGSignalModulePrivate ValaGSignalModulePrivate;

#define VALA_TYPE_GASYNC_MODULE (vala_gasync_module_get_type ())
#define VALA_GASYNC_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GASYNC_MODULE, ValaGAsyncModule))
#define VALA_GASYNC_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GASYNC_MODULE, ValaGAsyncModuleClass))
#define VALA_IS_GASYNC_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GASYNC_MODULE))
#define VALA_IS_GASYNC_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GASYNC_MODULE))
#define VALA_GASYNC_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GASYNC_MODULE, ValaGAsyncModuleClass))

typedef struct _ValaGAsyncModule ValaGAsyncModule;
typedef struct _ValaGAsyncModuleClass ValaGAsyncModuleClass;
typedef struct _ValaGAsyncModulePrivate ValaGAsyncModulePrivate;

#define VALA_TYPE_DBUS_MODULE (vala_dbus_module_get_type ())
#define VALA_DBUS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DBUS_MODULE, ValaDBusModule))
#define VALA_DBUS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DBUS_MODULE, ValaDBusModuleClass))
#define VALA_IS_DBUS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DBUS_MODULE))
#define VALA_IS_DBUS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DBUS_MODULE))
#define VALA_DBUS_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DBUS_MODULE, ValaDBusModuleClass))

typedef struct _ValaDBusModule ValaDBusModule;
typedef struct _ValaDBusModuleClass ValaDBusModuleClass;
typedef struct _ValaDBusModulePrivate ValaDBusModulePrivate;

#define VALA_TYPE_DBUS_CLIENT_MODULE (vala_dbus_client_module_get_type ())
#define VALA_DBUS_CLIENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DBUS_CLIENT_MODULE, ValaDBusClientModule))
#define VALA_DBUS_CLIENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DBUS_CLIENT_MODULE, ValaDBusClientModuleClass))
#define VALA_IS_DBUS_CLIENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DBUS_CLIENT_MODULE))
#define VALA_IS_DBUS_CLIENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DBUS_CLIENT_MODULE))
#define VALA_DBUS_CLIENT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DBUS_CLIENT_MODULE, ValaDBusClientModuleClass))

typedef struct _ValaDBusClientModule ValaDBusClientModule;
typedef struct _ValaDBusClientModuleClass ValaDBusClientModuleClass;
typedef struct _ValaDBusClientModulePrivate ValaDBusClientModulePrivate;

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))

#define VALA_TYPE_INTERFACE_REGISTER_FUNCTION (vala_interface_register_function_get_type ())
#define VALA_INTERFACE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunction))
#define VALA_INTERFACE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunctionClass))
#define VALA_IS_INTERFACE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_INTERFACE_REGISTER_FUNCTION))
#define VALA_IS_INTERFACE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_INTERFACE_REGISTER_FUNCTION))
#define VALA_INTERFACE_REGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunctionClass))

typedef struct _ValaInterfaceRegisterFunction ValaInterfaceRegisterFunction;
typedef struct _ValaInterfaceRegisterFunctionClass ValaInterfaceRegisterFunctionClass;

#define VALA_TYPE_DBUS_INTERFACE_REGISTER_FUNCTION (vala_dbus_interface_register_function_get_type ())
#define VALA_DBUS_INTERFACE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DBUS_INTERFACE_REGISTER_FUNCTION, ValaDBusInterfaceRegisterFunction))
#define VALA_DBUS_INTERFACE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DBUS_INTERFACE_REGISTER_FUNCTION, ValaDBusInterfaceRegisterFunctionClass))
#define VALA_IS_DBUS_INTERFACE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DBUS_INTERFACE_REGISTER_FUNCTION))
#define VALA_IS_DBUS_INTERFACE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DBUS_INTERFACE_REGISTER_FUNCTION))
#define VALA_DBUS_INTERFACE_REGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DBUS_INTERFACE_REGISTER_FUNCTION, ValaDBusInterfaceRegisterFunctionClass))

typedef struct _ValaDBusInterfaceRegisterFunction ValaDBusInterfaceRegisterFunction;
typedef struct _ValaDBusInterfaceRegisterFunctionClass ValaDBusInterfaceRegisterFunctionClass;

struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

struct _ValaCCodeBaseModule {
	ValaCCodeModule parent_instance;
	ValaCCodeBaseModulePrivate * priv;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaTryStatement* current_try;
	ValaCCodeDeclarationSpace* header_declarations;
	ValaCCodeDeclarationSpace* internal_header_declarations;
	ValaCCodeDeclarationSpace* source_declarations;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* class_init_fragment;
	ValaCCodeFragment* base_init_fragment;
	ValaCCodeFragment* class_finalize_fragment;
	ValaCCodeFragment* base_finalize_fragment;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_finalize_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeFragment* module_init_fragment;
	ValaCCodeStruct* param_spec_struct;
	ValaCCodeStruct* closure_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeFunction* function;
	ValaCCodeFragment* pre_statement_fragment;
	GeeArrayList* temp_vars;
	GeeArrayList* temp_ref_vars;
	GeeSet* user_marshal_set;
	GeeSet* predefined_marshal_set;
	gint next_temp_var_id;
	gboolean in_constructor;
	gboolean in_static_or_class_context;
	gboolean current_method_inner_error;
	gint next_coroutine_state;
	ValaDataType* void_type;
	ValaDataType* bool_type;
	ValaDataType* char_type;
	ValaDataType* uchar_type;
	ValaDataType* unichar_type;
	ValaDataType* short_type;
	ValaDataType* ushort_type;
	ValaDataType* int_type;
	ValaDataType* uint_type;
	ValaDataType* long_type;
	ValaDataType* ulong_type;
	ValaDataType* int8_type;
	ValaDataType* uint8_type;
	ValaDataType* int16_type;
	ValaDataType* uint16_type;
	ValaDataType* int32_type;
	ValaDataType* uint32_type;
	ValaDataType* int64_type;
	ValaDataType* uint64_type;
	ValaDataType* string_type;
	ValaDataType* float_type;
	ValaDataType* double_type;
	ValaTypeSymbol* gtype_type;
	ValaTypeSymbol* gobject_type;
	ValaErrorType* gerror_type;
	ValaClass* glist_type;
	ValaClass* gslist_type;
	ValaClass* gvaluearray_type;
	ValaTypeSymbol* gstringbuilder_type;
	ValaTypeSymbol* garray_type;
	ValaTypeSymbol* gbytearray_type;
	ValaTypeSymbol* gptrarray_type;
	ValaDataType* gquark_type;
	ValaStruct* gvalue_type;
	ValaStruct* mutex_type;
	ValaTypeSymbol* type_module_type;
	ValaTypeSymbol* dbus_object_type;
	gboolean in_plugin;
	char* module_init_param_name;
	gboolean gvaluecollector_h_needed;
	gboolean requires_array_free;
	gboolean requires_array_move;
	gboolean requires_array_length;
	gboolean requires_strcmp0;
	gboolean dbus_glib_h_needed;
	gboolean dbus_glib_h_needed_in_header;
	GeeSet* wrappers;
	GeeMap* variable_name_map;
};

struct _ValaCCodeBaseModuleClass {
	ValaCCodeModuleClass parent_class;
	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
	char* (*append_struct_array_free) (ValaCCodeBaseModule* self, ValaStruct* st);
	ValaCCodeExpression* (*get_unref_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
	void (*append_local_free) (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
	gboolean (*variable_accessible_in_finally) (ValaCCodeBaseModule* self, ValaLocalVariable* local);
	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_ref_cexpression) (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
	gboolean (*is_gobject_property) (ValaCCodeBaseModule* self, ValaProperty* prop);
};

struct _ValaCCodeStructModule {
	ValaCCodeBaseModule parent_instance;
	ValaCCodeStructModulePrivate * priv;
};

struct _ValaCCodeStructModuleClass {
	ValaCCodeBaseModuleClass parent_class;
};

struct _ValaCCodeMethodModule {
	ValaCCodeStructModule parent_instance;
	ValaCCodeMethodModulePrivate * priv;
};

struct _ValaCCodeMethodModuleClass {
	ValaCCodeStructModuleClass parent_class;
	void (*generate_method_result_declaration) (ValaCCodeMethodModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeFunction* cfunc, GeeMap* cparam_map, GeeMap* carg_map);
	void (*generate_parameter) (ValaCCodeMethodModule* self, ValaFormalParameter* param, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, GeeMap* carg_map);
};

struct _ValaCCodeControlFlowModule {
	ValaCCodeMethodModule parent_instance;
	ValaCCodeControlFlowModulePrivate * priv;
};

struct _ValaCCodeControlFlowModuleClass {
	ValaCCodeMethodModuleClass parent_class;
};

struct _ValaCCodeMemberAccessModule {
	ValaCCodeControlFlowModule parent_instance;
	ValaCCodeMemberAccessModulePrivate * priv;
};

struct _ValaCCodeMemberAccessModuleClass {
	ValaCCodeControlFlowModuleClass parent_class;
};

struct _ValaCCodeAssignmentModule {
	ValaCCodeMemberAccessModule parent_instance;
	ValaCCodeAssignmentModulePrivate * priv;
};

struct _ValaCCodeAssignmentModuleClass {
	ValaCCodeMemberAccessModuleClass parent_class;
};

struct _ValaCCodeMethodCallModule {
	ValaCCodeAssignmentModule parent_instance;
	ValaCCodeMethodCallModulePrivate * priv;
};

struct _ValaCCodeMethodCallModuleClass {
	ValaCCodeAssignmentModuleClass parent_class;
};

struct _ValaCCodeArrayModule {
	ValaCCodeMethodCallModule parent_instance;
	ValaCCodeArrayModulePrivate * priv;
};

struct _ValaCCodeArrayModuleClass {
	ValaCCodeMethodCallModuleClass parent_class;
};

struct _ValaCCodeDelegateModule {
	ValaCCodeArrayModule parent_instance;
	ValaCCodeDelegateModulePrivate * priv;
};

struct _ValaCCodeDelegateModuleClass {
	ValaCCodeArrayModuleClass parent_class;
};

struct _ValaGErrorModule {
	ValaCCodeDelegateModule parent_instance;
	ValaGErrorModulePrivate * priv;
};

struct _ValaGErrorModuleClass {
	ValaCCodeDelegateModuleClass parent_class;
	ValaCCodeStatement* (*return_with_exception) (ValaGErrorModule* self, ValaCCodeExpression* error_expr);
};

struct _ValaGTypeModule {
	ValaGErrorModule parent_instance;
	ValaGTypeModulePrivate * priv;
};

struct _ValaGTypeModuleClass {
	ValaGErrorModuleClass parent_class;
	void (*generate_virtual_method_declaration) (ValaGTypeModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeStruct* type_struct);
	void (*generate_class_init) (ValaGTypeModule* self, ValaClass* cl, ValaCCodeBlock* init_block);
	ValaTypeRegisterFunction* (*create_interface_register_function) (ValaGTypeModule* self, ValaInterface* iface);
};

struct _ValaGObjectModule {
	ValaGTypeModule parent_instance;
	ValaGObjectModulePrivate * priv;
};

struct _ValaGObjectModuleClass {
	ValaGTypeModuleClass parent_class;
};

struct _ValaGSignalModule {
	ValaGObjectModule parent_instance;
	ValaGSignalModulePrivate * priv;
};

struct _ValaGSignalModuleClass {
	ValaGObjectModuleClass parent_class;
	ValaCCodeExpression* (*get_dbus_g_type) (ValaGSignalModule* self, ValaDataType* data_type);
};

struct _ValaGAsyncModule {
	ValaGSignalModule parent_instance;
	ValaGAsyncModulePrivate * priv;
};

struct _ValaGAsyncModuleClass {
	ValaGSignalModuleClass parent_class;
};

struct _ValaDBusModule {
	ValaGAsyncModule parent_instance;
	ValaDBusModulePrivate * priv;
};

struct _ValaDBusModuleClass {
	ValaGAsyncModuleClass parent_class;
};

struct _ValaDBusClientModule {
	ValaDBusModule parent_instance;
	ValaDBusClientModulePrivate * priv;
};

struct _ValaDBusClientModuleClass {
	ValaDBusModuleClass parent_class;
};

struct _ValaDBusClientModulePrivate {
	gint dynamic_property_id;
};


static gpointer vala_dbus_client_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_base_module_get_type (void);
gpointer vala_ccode_declaration_space_ref (gpointer instance);
void vala_ccode_declaration_space_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_declaration_space (const GValue* value);
GType vala_ccode_declaration_space_get_type (void);
GType vala_ccode_struct_module_get_type (void);
GType vala_ccode_method_module_get_type (void);
GType vala_ccode_control_flow_module_get_type (void);
GType vala_ccode_member_access_module_get_type (void);
GType vala_ccode_assignment_module_get_type (void);
GType vala_ccode_method_call_module_get_type (void);
GType vala_ccode_array_module_get_type (void);
GType vala_ccode_delegate_module_get_type (void);
GType vala_gerror_module_get_type (void);
GType vala_gtype_module_get_type (void);
gpointer vala_typeregister_function_ref (gpointer instance);
void vala_typeregister_function_unref (gpointer instance);
GParamSpec* vala_param_spec_typeregister_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_typeregister_function (GValue* value, gpointer v_object);
gpointer vala_value_get_typeregister_function (const GValue* value);
GType vala_typeregister_function_get_type (void);
GType vala_gobject_module_get_type (void);
GType vala_gsignal_module_get_type (void);
GType vala_gasync_module_get_type (void);
GType vala_dbus_module_get_type (void);
GType vala_dbus_client_module_get_type (void);
#define VALA_DBUS_CLIENT_MODULE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_DBUS_CLIENT_MODULE, ValaDBusClientModulePrivate))
enum  {
	VALA_DBUS_CLIENT_MODULE_DUMMY_PROPERTY
};
GType vala_ccode_generator_get_type (void);
ValaDBusModule* vala_dbus_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaDBusModule* vala_dbus_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaDBusClientModule* vala_dbus_client_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaDBusClientModule* vala_dbus_client_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCodeContext* vala_ccode_base_module_get_context (ValaCCodeBaseModule* self);
static char* vala_dbus_client_module_get_dynamic_dbus_name (ValaDBusClientModule* self, const char* vala_name);
static gboolean vala_dbus_client_module_has_dbus_error (ValaDBusClientModule* self, GeeList* error_types);
void vala_ccode_base_module_generate_cparameters (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
static void vala_dbus_client_module_generate_dbus_method_wrapper (ValaDBusClientModule* self, ValaMethod* method, ValaCCodeBlock* block);
void vala_ccode_declaration_space_add_type_member_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
static void vala_dbus_client_module_real_generate_dynamic_method_wrapper (ValaCCodeModule* base, ValaDynamicMethod* method);
gboolean vala_dbus_client_module_dbus_use_ptr_array (ValaDBusClientModule* self, ValaArrayType* array_type);
ValaCCodeExpression* vala_gsignal_module_get_dbus_g_type (ValaGSignalModule* self, ValaDataType* data_type);
gboolean vala_ccode_declaration_space_add_declaration (ValaCCodeDeclarationSpace* self, const char* name);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
char* vala_ccode_module_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim);
void vala_ccode_declaration_space_add_include (ValaCCodeDeclarationSpace* self, const char* filename, gboolean local);
char* vala_dbus_module_get_type_signature (ValaDataType* datatype);
gboolean vala_ccode_base_module_requires_copy (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_get_dup_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
ValaCCodeExpression* vala_ccode_base_module_default_value_for_type (ValaCCodeBaseModule* self, ValaDataType* type, gboolean initializer_expression);
static ValaCCodeExpression* vala_dbus_client_module_real_get_dbus_g_type (ValaGSignalModule* base, ValaDataType* data_type);
char* vala_ccode_module_get_dynamic_property_getter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
static void vala_dbus_client_module_generate_dbus_property_getter_wrapper (ValaDBusClientModule* self, ValaDynamicProperty* node, ValaCCodeBlock* block);
static char* vala_dbus_client_module_real_get_dynamic_property_getter_cname (ValaCCodeModule* base, ValaDynamicProperty* prop);
char* vala_ccode_module_get_dynamic_property_setter_cname (ValaCCodeModule* self, ValaDynamicProperty* node);
static void vala_dbus_client_module_generate_dbus_property_setter_wrapper (ValaDBusClientModule* self, ValaDynamicProperty* node, ValaCCodeBlock* block);
static char* vala_dbus_client_module_real_get_dynamic_property_setter_cname (ValaCCodeModule* base, ValaDynamicProperty* prop);
static void vala_dbus_client_module_create_dbus_property_proxy (ValaDBusClientModule* self, ValaDynamicProperty* node, ValaCCodeBlock* block);
char* vala_ccode_module_get_dynamic_signal_connect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node);
char* vala_ccode_module_get_dynamic_signal_cname (ValaCCodeModule* self, ValaDynamicSignal* node);
static void vala_dbus_client_module_generate_dbus_connect_wrapper (ValaDBusClientModule* self, ValaDynamicSignal* sig, ValaCCodeBlock* block);
static char* vala_dbus_client_module_real_get_dynamic_signal_connect_wrapper_name (ValaCCodeModule* base, ValaDynamicSignal* sig);
char* vala_ccode_module_get_dynamic_signal_disconnect_wrapper_name (ValaCCodeModule* self, ValaDynamicSignal* node);
static void vala_dbus_client_module_generate_dbus_disconnect_wrapper (ValaDBusClientModule* self, ValaDynamicSignal* sig, ValaCCodeBlock* block);
static char* vala_dbus_client_module_real_get_dynamic_signal_disconnect_wrapper_name (ValaCCodeModule* base, ValaDynamicSignal* sig);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
void vala_ccode_module_generate_marshaller (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
char* vala_ccode_module_get_marshaller_function (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
void vala_ccode_module_visit_cast_expression (ValaCCodeModule* self, ValaCastExpression* expr);
static void vala_dbus_client_module_real_visit_cast_expression (ValaCCodeModule* base, ValaCastExpression* expr);
void vala_ccode_module_visit_interface (ValaCCodeModule* self, ValaInterface* iface);
char* vala_dbus_module_get_dbus_name (ValaDBusModule* self, ValaTypeSymbol* symbol);
void vala_ccode_declaration_space_add_type_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
void vala_ccode_declaration_space_add_type_definition (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
static void vala_dbus_client_module_generate_proxy_filter_function (ValaDBusClientModule* self, ValaInterface* iface);
static char* vala_dbus_client_module_generate_dbus_proxy_method (ValaDBusClientModule* self, ValaInterface* iface, ValaMethod* m);
static char* vala_dbus_client_module_generate_async_dbus_proxy_method (ValaDBusClientModule* self, ValaInterface* iface, ValaMethod* m);
static char* vala_dbus_client_module_generate_finish_dbus_proxy_method (ValaDBusClientModule* self, ValaInterface* iface, ValaMethod* m);
static char* vala_dbus_client_module_generate_dbus_proxy_property_get (ValaDBusClientModule* self, ValaInterface* iface, ValaProperty* prop);
static char* vala_dbus_client_module_generate_dbus_proxy_property_set (ValaDBusClientModule* self, ValaInterface* iface, ValaProperty* prop);
static void vala_dbus_client_module_real_visit_interface (ValaCCodeModule* base, ValaInterface* iface);
ValaInterfaceRegisterFunction* vala_interface_register_function_new (ValaInterface* iface, ValaCodeContext* context);
ValaInterfaceRegisterFunction* vala_interface_register_function_construct (GType object_type, ValaInterface* iface, ValaCodeContext* context);
GType vala_interface_register_function_get_type (void);
ValaDBusInterfaceRegisterFunction* vala_dbus_interface_register_function_new (ValaInterface* iface, ValaCodeContext* context);
ValaDBusInterfaceRegisterFunction* vala_dbus_interface_register_function_construct (GType object_type, ValaInterface* iface, ValaCodeContext* context);
GType vala_dbus_interface_register_function_get_type (void);
static ValaTypeRegisterFunction* vala_dbus_client_module_real_create_interface_register_function (ValaGTypeModule* base, ValaInterface* iface);
void vala_ccode_module_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr);
static void vala_dbus_client_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr);
static void vala_dbus_client_module_handle_signals (ValaDBusClientModule* self, ValaInterface* iface, ValaCCodeBlock* block);
ValaCCodeExpression* vala_dbus_module_read_expression (ValaDBusModule* self, ValaCCodeFragment* fragment, ValaDataType* type, ValaCCodeExpression* iter_expr, ValaCCodeExpression* expr);
static char* vala_dbus_client_module_generate_dbus_signal_handler (ValaDBusClientModule* self, ValaSignal* sig, ValaObjectTypeSymbol* sym);
static void vala_dbus_client_module_handle_signal (ValaDBusClientModule* self, const char* dbus_iface_name, const char* dbus_signal_name, const char* handler_name, ValaCCodeBlock* block, ValaCCodeIfStatement** clastif);
void vala_dbus_module_write_expression (ValaDBusModule* self, ValaCCodeFragment* fragment, ValaDataType* type, ValaCCodeExpression* iter_expr, ValaCCodeExpression* expr);
static void vala_dbus_client_module_generate_marshalling (ValaDBusClientModule* self, ValaMethod* m, const char* dbus_iface_name, ValaCCodeFragment* prefragment, ValaCCodeFragment* postfragment);
static void vala_dbus_client_module_generate_client_error_cases (ValaDBusClientModule* self, ValaCCodeBlock* error_block, GeeList* error_types, ValaCCodeExpression* dbus_error_name, ValaCCodeExpression* result_edomain, ValaCCodeExpression* result_ecode);
gint vala_ccode_base_module_get_param_pos (ValaCCodeBaseModule* self, double param_pos, gboolean ellipsis);
static void vala_dbus_client_module_finalize (ValaCCodeModule* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



ValaDBusClientModule* vala_dbus_client_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaDBusClientModule* self;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaDBusClientModule*) vala_dbus_module_construct (object_type, codegen, next);
	return self;
}


ValaDBusClientModule* vala_dbus_client_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_dbus_client_module_construct (VALA_TYPE_DBUS_CLIENT_MODULE, codegen, next);
}


static char* vala_dbus_client_module_get_dynamic_dbus_name (ValaDBusClientModule* self, const char* vala_name) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (vala_name != NULL, NULL);
	if (vala_code_context_get_dbus_transformation (vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self))) {
		result = vala_symbol_lower_case_to_camel_case (vala_name);
		return result;
	} else {
		result = g_strdup (vala_name);
		return result;
	}
}


static gboolean vala_dbus_client_module_has_dbus_error (ValaDBusClientModule* self, GeeList* error_types) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (error_types != NULL, FALSE);
	{
		GeeIterator* _error_type_it;
		_error_type_it = gee_iterable_iterator ((GeeIterable*) error_types);
		while (TRUE) {
			ValaDataType* error_type;
			char* _tmp0_;
			gboolean _tmp1_;
			if (!gee_iterator_next (_error_type_it)) {
				break;
			}
			error_type = (ValaDataType*) gee_iterator_get (_error_type_it);
			if ((_tmp1_ = _vala_strcmp0 (_tmp0_ = vala_symbol_get_full_name ((ValaSymbol*) vala_error_type_get_error_domain (VALA_ERROR_TYPE (error_type))), "DBus.Error") == 0, _g_free0 (_tmp0_), _tmp1_)) {
				result = TRUE;
				_vala_code_node_unref0 (error_type);
				_gee_collection_object_unref0 (_error_type_it);
				return result;
			}
			_vala_code_node_unref0 (error_type);
		}
		_gee_collection_object_unref0 (_error_type_it);
	}
	result = FALSE;
	return result;
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


static void vala_dbus_client_module_real_generate_dynamic_method_wrapper (ValaCCodeModule* base, ValaDynamicMethod* method) {
	ValaDBusClientModule * self;
	ValaDynamicMethod* dynamic_method;
	char* _tmp0_;
	ValaCCodeFunction* _tmp1_;
	ValaCCodeFunction* func;
	GeeHashMap* cparam_map;
	ValaCCodeBlock* block;
	ValaCCodeFunction* _tmp4_;
	self = (ValaDBusClientModule*) base;
	g_return_if_fail (method != NULL);
	dynamic_method = _vala_code_node_ref0 (VALA_DYNAMIC_METHOD (method));
	func = (_tmp1_ = vala_ccode_function_new (_tmp0_ = vala_method_get_cname ((ValaMethod*) method), "void"), _g_free0 (_tmp0_), _tmp1_);
	cparam_map = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_FORMAL_PARAMETER, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal);
	vala_ccode_base_module_generate_cparameters ((ValaCCodeBaseModule*) self, (ValaMethod*) method, ((ValaCCodeBaseModule*) self)->source_declarations, (GeeMap*) cparam_map, func, NULL, NULL, NULL, 3);
	block = vala_ccode_block_new ();
	if (vala_data_type_get_data_type (vala_dynamic_method_get_dynamic_type (dynamic_method)) == ((ValaCCodeBaseModule*) self)->dbus_object_type) {
		vala_dbus_client_module_generate_dbus_method_wrapper (self, (ValaMethod*) method, block);
	} else {
		char* _tmp3_;
		char* _tmp2_;
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) method), _tmp3_ = g_strdup_printf ("dynamic methods are not supported for `%s'", _tmp2_ = vala_code_node_to_string ((ValaCodeNode*) vala_dynamic_method_get_dynamic_type (dynamic_method))));
		_g_free0 (_tmp3_);
		_g_free0 (_tmp2_);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp4_ = vala_ccode_function_copy (func)));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_set_block (func, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) func);
	_vala_code_node_unref0 (dynamic_method);
	_vala_ccode_node_unref0 (func);
	_gee_collection_object_unref0 (cparam_map);
	_vala_ccode_node_unref0 (block);
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


static void vala_dbus_client_module_generate_dbus_method_wrapper (ValaDBusClientModule* self, ValaMethod* method, ValaCCodeBlock* block) {
	ValaDynamicMethod* dynamic_method;
	ValaMethodCall* expr;
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp2_;
	gboolean found_out;
	ValaExpression* callback;
	gint callback_index;
	gint arg_index;
	ValaCCodeConstant* _tmp10_;
	char* _tmp9_;
	char* _tmp8_;
	ValaCCodeIdentifier* _tmp148_;
	ValaCCodeFragment* out_marshalling_fragment;
	g_return_if_fail (self != NULL);
	g_return_if_fail (method != NULL);
	g_return_if_fail (block != NULL);
	dynamic_method = _vala_code_node_ref0 (VALA_DYNAMIC_METHOD (method));
	expr = _vala_code_node_ref0 (vala_dynamic_method_get_invocation (dynamic_method));
	ccall = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_g_proxy_begin_call"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp2_);
	found_out = FALSE;
	callback = NULL;
	callback_index = -1;
	arg_index = 1;
	{
		GeeList* _tmp3_;
		GeeIterator* _tmp4_;
		GeeIterator* _arg_it;
		_arg_it = (_tmp4_ = gee_iterable_iterator ((GeeIterable*) (_tmp3_ = vala_method_call_get_argument_list (expr))), _gee_collection_object_unref0 (_tmp3_), _tmp4_);
		while (TRUE) {
			ValaExpression* arg;
			if (!gee_iterator_next (_arg_it)) {
				break;
			}
			arg = (ValaExpression*) gee_iterator_get (_arg_it);
			if (VALA_IS_METHOD (vala_expression_get_symbol_reference (arg))) {
				ValaExpression* _tmp5_;
				if (callback != NULL) {
					vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), "only one reply callback may be specified in invocation of DBus method");
					vala_code_node_set_error ((ValaCodeNode*) expr, TRUE);
					_vala_code_node_unref0 (arg);
					_gee_collection_object_unref0 (_arg_it);
					_vala_code_node_unref0 (dynamic_method);
					_vala_code_node_unref0 (expr);
					_vala_ccode_node_unref0 (ccall);
					_vala_code_node_unref0 (callback);
					return;
				} else {
					if (found_out) {
						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), "out argument and reply callback conflict in invocation of DBus method");
						vala_code_node_set_error ((ValaCodeNode*) expr, TRUE);
						_vala_code_node_unref0 (arg);
						_gee_collection_object_unref0 (_arg_it);
						_vala_code_node_unref0 (dynamic_method);
						_vala_code_node_unref0 (expr);
						_vala_ccode_node_unref0 (ccall);
						_vala_code_node_unref0 (callback);
						return;
					}
				}
				callback = (_tmp5_ = _vala_code_node_ref0 (arg), _vala_code_node_unref0 (callback), _tmp5_);
				callback_index = arg_index;
			} else {
				gboolean _tmp6_;
				_tmp6_ = FALSE;
				if (VALA_IS_UNARY_EXPRESSION (arg)) {
					_tmp6_ = vala_unary_expression_get_operator (VALA_UNARY_EXPRESSION (arg)) == VALA_UNARY_OPERATOR_OUT;
				} else {
					_tmp6_ = FALSE;
				}
				if (_tmp6_) {
					if (callback != NULL) {
						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), "out argument and reply callback conflict in invocation of DBus method");
						vala_code_node_set_error ((ValaCodeNode*) expr, TRUE);
						_vala_code_node_unref0 (arg);
						_gee_collection_object_unref0 (_arg_it);
						_vala_code_node_unref0 (dynamic_method);
						_vala_code_node_unref0 (expr);
						_vala_ccode_node_unref0 (ccall);
						_vala_code_node_unref0 (callback);
						return;
					}
					found_out = TRUE;
				} else {
					gboolean _tmp7_;
					_tmp7_ = FALSE;
					if (callback != NULL) {
						_tmp7_ = TRUE;
					} else {
						_tmp7_ = found_out;
					}
					if (_tmp7_) {
						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), "in argument must not follow out argument or reply callback in invocation of DBus method");
						vala_code_node_set_error ((ValaCodeNode*) expr, TRUE);
						_vala_code_node_unref0 (arg);
						_gee_collection_object_unref0 (_arg_it);
						_vala_code_node_unref0 (dynamic_method);
						_vala_code_node_unref0 (expr);
						_vala_ccode_node_unref0 (ccall);
						_vala_code_node_unref0 (callback);
						return;
					}
				}
			}
			arg_index++;
			_vala_code_node_unref0 (arg);
		}
		_gee_collection_object_unref0 (_arg_it);
	}
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_constant_new (_tmp9_ = g_strdup_printf ("\"%s\"", _tmp8_ = vala_dbus_client_module_get_dynamic_dbus_name (self, vala_symbol_get_name ((ValaSymbol*) method))))));
	_vala_ccode_node_unref0 (_tmp10_);
	_g_free0 (_tmp9_);
	_g_free0 (_tmp8_);
	if (callback != NULL) {
		ValaMethod* reply_method;
		char* _tmp12_;
		char* _tmp11_;
		ValaCCodeFunction* _tmp13_;
		ValaCCodeFunction* cb_fun;
		ValaCCodeFormalParameter* _tmp14_;
		ValaCCodeFormalParameter* _tmp15_;
		ValaCCodeFormalParameter* _tmp16_;
		ValaCCodeBlock* _tmp17_;
		ValaCCodeDeclaration* cerrdecl;
		ValaCCodeVariableDeclarator* _tmp19_;
		ValaCCodeConstant* _tmp18_;
		ValaCCodeIdentifier* _tmp20_;
		ValaCCodeFunctionCall* _tmp21_;
		ValaCCodeFunctionCall* cend_call;
		ValaCCodeIdentifier* _tmp22_;
		ValaCCodeIdentifier* _tmp23_;
		ValaCCodeUnaryExpression* _tmp25_;
		ValaCCodeIdentifier* _tmp24_;
		ValaCCodeFunctionCall* creply_call;
		GeeList* _tmp27_;
		gint _tmp28_;
		gint param_count;
		gint i;
		ValaCCodeIdentifier* _tmp54_;
		ValaCCodeExpressionStatement* _tmp55_;
		ValaCCodeIdentifier* _tmp56_;
		ValaCCodeExpressionStatement* _tmp57_;
		ValaCCodeIdentifier* _tmp58_;
		ValaCCodeConstant* _tmp60_;
		char* _tmp59_;
		ValaCCodeConstant* _tmp61_;
		reply_method = _vala_code_node_ref0 (VALA_METHOD (vala_expression_get_symbol_reference (callback)));
		cb_fun = (_tmp13_ = vala_ccode_function_new (_tmp12_ = g_strdup_printf ("_%s_cb", _tmp11_ = vala_method_get_cname (reply_method)), "void"), _g_free0 (_tmp12_), _g_free0 (_tmp11_), _tmp13_);
		vala_ccode_function_set_modifiers (cb_fun, VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_function_add_parameter (cb_fun, _tmp14_ = vala_ccode_formal_parameter_new ("proxy", "DBusGProxy*"));
		_vala_ccode_node_unref0 (_tmp14_);
		vala_ccode_function_add_parameter (cb_fun, _tmp15_ = vala_ccode_formal_parameter_new ("call", "DBusGProxyCall*"));
		_vala_ccode_node_unref0 (_tmp15_);
		vala_ccode_function_add_parameter (cb_fun, _tmp16_ = vala_ccode_formal_parameter_new ("user_data", "void*"));
		_vala_ccode_node_unref0 (_tmp16_);
		vala_ccode_function_set_block (cb_fun, _tmp17_ = vala_ccode_block_new ());
		_vala_ccode_node_unref0 (_tmp17_);
		cerrdecl = vala_ccode_declaration_new ("GError*");
		vala_ccode_declaration_add_declarator (cerrdecl, (ValaCCodeDeclarator*) (_tmp19_ = vala_ccode_variable_declarator_new ("error", (ValaCCodeExpression*) (_tmp18_ = vala_ccode_constant_new ("NULL")), NULL)));
		_vala_ccode_node_unref0 (_tmp19_);
		_vala_ccode_node_unref0 (_tmp18_);
		vala_ccode_block_add_statement (vala_ccode_function_get_block (cb_fun), (ValaCCodeNode*) cerrdecl);
		cend_call = (_tmp21_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("dbus_g_proxy_end_call"))), _vala_ccode_node_unref0 (_tmp20_), _tmp21_);
		vala_ccode_function_call_add_argument (cend_call, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("proxy")));
		_vala_ccode_node_unref0 (_tmp22_);
		vala_ccode_function_call_add_argument (cend_call, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new ("call")));
		_vala_ccode_node_unref0 (_tmp23_);
		vala_ccode_function_call_add_argument (cend_call, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("error")))));
		_vala_ccode_node_unref0 (_tmp25_);
		_vala_ccode_node_unref0 (_tmp24_);
		creply_call = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) callback)));
		if (vala_method_get_binding (reply_method) != MEMBER_BINDING_STATIC) {
			ValaCCodeIdentifier* _tmp26_;
			vala_ccode_function_call_add_argument (creply_call, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("user_data")));
			_vala_ccode_node_unref0 (_tmp26_);
		}
		param_count = (_tmp28_ = gee_collection_get_size ((GeeCollection*) (_tmp27_ = vala_method_get_parameters (reply_method))), _gee_collection_object_unref0 (_tmp27_), _tmp28_);
		i = 0;
		{
			GeeList* _tmp29_;
			GeeIterator* _tmp30_;
			GeeIterator* _param_it;
			_param_it = (_tmp30_ = gee_iterable_iterator ((GeeIterable*) (_tmp29_ = vala_method_get_parameters (reply_method))), _gee_collection_object_unref0 (_tmp29_), _tmp30_);
			while (TRUE) {
				ValaFormalParameter* param;
				gboolean _tmp31_;
				if (!gee_iterator_next (_param_it)) {
					break;
				}
				param = (ValaFormalParameter*) gee_iterator_get (_param_it);
				if ((i = i + 1) == param_count) {
					if (!VALA_IS_ERROR_TYPE (vala_formal_parameter_get_parameter_type (param))) {
						vala_report_error (NULL, "DBus reply callbacks must end with GLib.Error argument");
						_vala_code_node_unref0 (param);
						_gee_collection_object_unref0 (_param_it);
						_vala_code_node_unref0 (reply_method);
						_vala_ccode_node_unref0 (cb_fun);
						_vala_ccode_node_unref0 (cerrdecl);
						_vala_ccode_node_unref0 (cend_call);
						_vala_ccode_node_unref0 (creply_call);
						_vala_code_node_unref0 (dynamic_method);
						_vala_code_node_unref0 (expr);
						_vala_ccode_node_unref0 (ccall);
						_vala_code_node_unref0 (callback);
						return;
					}
					_vala_code_node_unref0 (param);
					break;
				}
				_tmp31_ = FALSE;
				if (VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param))) {
					_tmp31_ = vala_data_type_get_data_type (vala_array_type_get_element_type (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)))) != vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->string_type);
				} else {
					_tmp31_ = FALSE;
				}
				if (_tmp31_) {
					ValaArrayType* array_type;
					ValaCCodeDeclaration* _cdecl_;
					ValaCCodeVariableDeclarator* _tmp34_;
					ValaCCodeExpression* _tmp35_;
					ValaCCodeUnaryExpression* _tmp37_;
					ValaCCodeIdentifier* _tmp36_;
					const char* _tmp38_;
					ValaCCodeMemberAccess* _tmp40_;
					ValaCCodeIdentifier* _tmp39_;
					ValaCCodeMemberAccess* _tmp42_;
					ValaCCodeIdentifier* _tmp41_;
					array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)));
					_cdecl_ = NULL;
					if (vala_dbus_client_module_dbus_use_ptr_array (self, array_type)) {
						ValaCCodeDeclaration* _tmp32_;
						_cdecl_ = (_tmp32_ = vala_ccode_declaration_new ("GPtrArray*"), _vala_ccode_node_unref0 (_cdecl_), _tmp32_);
					} else {
						ValaCCodeDeclaration* _tmp33_;
						_cdecl_ = (_tmp33_ = vala_ccode_declaration_new ("GArray*"), _vala_ccode_node_unref0 (_cdecl_), _tmp33_);
					}
					vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp34_ = vala_ccode_variable_declarator_new (vala_symbol_get_name ((ValaSymbol*) param), NULL, NULL)));
					_vala_ccode_node_unref0 (_tmp34_);
					vala_ccode_block_add_statement (vala_ccode_function_get_block (cb_fun), (ValaCCodeNode*) _cdecl_);
					vala_ccode_function_call_add_argument (cend_call, _tmp35_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, (ValaDataType*) array_type));
					_vala_ccode_node_unref0 (_tmp35_);
					vala_ccode_function_call_add_argument (cend_call, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))))));
					_vala_ccode_node_unref0 (_tmp37_);
					_vala_ccode_node_unref0 (_tmp36_);
					_tmp38_ = NULL;
					if (vala_dbus_client_module_dbus_use_ptr_array (self, array_type)) {
						_tmp38_ = "pdata";
					} else {
						_tmp38_ = "data";
					}
					vala_ccode_function_call_add_argument (creply_call, (ValaCCodeExpression*) (_tmp40_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))), _tmp38_)));
					_vala_ccode_node_unref0 (_tmp40_);
					_vala_ccode_node_unref0 (_tmp39_);
					vala_ccode_function_call_add_argument (creply_call, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp41_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))), "len")));
					_vala_ccode_node_unref0 (_tmp42_);
					_vala_ccode_node_unref0 (_tmp41_);
					_vala_code_node_unref0 (array_type);
					_vala_ccode_node_unref0 (_cdecl_);
				} else {
					char* _tmp43_;
					ValaCCodeDeclaration* _tmp44_;
					ValaCCodeDeclaration* _cdecl_;
					ValaCCodeVariableDeclarator* _tmp45_;
					ValaCCodeExpression* _tmp46_;
					ValaCCodeUnaryExpression* _tmp48_;
					ValaCCodeIdentifier* _tmp47_;
					ValaCCodeIdentifier* _tmp49_;
					gboolean _tmp50_;
					_cdecl_ = (_tmp44_ = vala_ccode_declaration_new (_tmp43_ = vala_data_type_get_cname (vala_formal_parameter_get_parameter_type (param))), _g_free0 (_tmp43_), _tmp44_);
					vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp45_ = vala_ccode_variable_declarator_new (vala_symbol_get_name ((ValaSymbol*) param), NULL, NULL)));
					_vala_ccode_node_unref0 (_tmp45_);
					vala_ccode_block_add_statement (vala_ccode_function_get_block (cb_fun), (ValaCCodeNode*) _cdecl_);
					vala_ccode_function_call_add_argument (cend_call, _tmp46_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_formal_parameter_get_parameter_type (param)));
					_vala_ccode_node_unref0 (_tmp46_);
					vala_ccode_function_call_add_argument (cend_call, (ValaCCodeExpression*) (_tmp48_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))))));
					_vala_ccode_node_unref0 (_tmp48_);
					_vala_ccode_node_unref0 (_tmp47_);
					vala_ccode_function_call_add_argument (creply_call, (ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
					_vala_ccode_node_unref0 (_tmp49_);
					_tmp50_ = FALSE;
					if (VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param))) {
						_tmp50_ = vala_data_type_get_data_type (vala_array_type_get_element_type (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)))) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->string_type);
					} else {
						_tmp50_ = FALSE;
					}
					if (_tmp50_) {
						ValaCCodeIdentifier* _tmp51_;
						ValaCCodeFunctionCall* _tmp52_;
						ValaCCodeFunctionCall* cstrvlen;
						ValaCCodeIdentifier* _tmp53_;
						cstrvlen = (_tmp52_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp51_ = vala_ccode_identifier_new ("g_strv_length"))), _vala_ccode_node_unref0 (_tmp51_), _tmp52_);
						vala_ccode_function_call_add_argument (cstrvlen, (ValaCCodeExpression*) (_tmp53_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
						_vala_ccode_node_unref0 (_tmp53_);
						vala_ccode_function_call_add_argument (creply_call, (ValaCCodeExpression*) cstrvlen);
						_vala_ccode_node_unref0 (cstrvlen);
					}
					_vala_ccode_node_unref0 (_cdecl_);
				}
				_vala_code_node_unref0 (param);
			}
			_gee_collection_object_unref0 (_param_it);
		}
		vala_ccode_function_call_add_argument (cend_call, (ValaCCodeExpression*) (_tmp54_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
		_vala_ccode_node_unref0 (_tmp54_);
		vala_ccode_block_add_statement (vala_ccode_function_get_block (cb_fun), (ValaCCodeNode*) (_tmp55_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cend_call)));
		_vala_ccode_node_unref0 (_tmp55_);
		vala_ccode_function_call_add_argument (creply_call, (ValaCCodeExpression*) (_tmp56_ = vala_ccode_identifier_new ("error")));
		_vala_ccode_node_unref0 (_tmp56_);
		vala_ccode_block_add_statement (vala_ccode_function_get_block (cb_fun), (ValaCCodeNode*) (_tmp57_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) creply_call)));
		_vala_ccode_node_unref0 (_tmp57_);
		if (!vala_ccode_declaration_space_add_declaration (((ValaCCodeBaseModule*) self)->source_declarations, vala_ccode_function_get_name (cb_fun))) {
			vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) cb_fun);
		}
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp58_ = vala_ccode_identifier_new (vala_ccode_function_get_name (cb_fun))));
		_vala_ccode_node_unref0 (_tmp58_);
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp60_ = vala_ccode_constant_new (_tmp59_ = g_strdup_printf ("param%d_target", callback_index))));
		_vala_ccode_node_unref0 (_tmp60_);
		_g_free0 (_tmp59_);
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp61_ = vala_ccode_constant_new ("NULL")));
		_vala_ccode_node_unref0 (_tmp61_);
		_vala_code_node_unref0 (reply_method);
		_vala_ccode_node_unref0 (cb_fun);
		_vala_ccode_node_unref0 (cerrdecl);
		_vala_ccode_node_unref0 (cend_call);
		_vala_ccode_node_unref0 (creply_call);
	} else {
		ValaCCodeIdentifier* _tmp62_;
		ValaCCodeIdentifier* _tmp63_;
		vala_ccode_function_call_set_call (ccall, (ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new ("dbus_g_proxy_call")));
		_vala_ccode_node_unref0 (_tmp62_);
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp63_ = vala_ccode_identifier_new ("error")));
		_vala_ccode_node_unref0 (_tmp63_);
	}
	{
		GeeList* _tmp64_;
		GeeIterator* _tmp65_;
		GeeIterator* _param_it;
		_param_it = (_tmp65_ = gee_iterable_iterator ((GeeIterable*) (_tmp64_ = vala_method_get_parameters (method))), _gee_collection_object_unref0 (_tmp64_), _tmp65_);
		while (TRUE) {
			ValaFormalParameter* param;
			gboolean _tmp66_;
			ValaDataType* _tmp67_;
			ValaArrayType* array_type;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			_tmp66_ = FALSE;
			if (VALA_IS_METHOD_TYPE (vala_formal_parameter_get_parameter_type (param))) {
				_tmp66_ = TRUE;
			} else {
				_tmp66_ = VALA_IS_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param));
			}
			if (_tmp66_) {
				_vala_code_node_unref0 (param);
				break;
			}
			if (vala_formal_parameter_get_direction (param) != VALA_PARAMETER_DIRECTION_IN) {
				_vala_code_node_unref0 (param);
				continue;
			}
			array_type = _vala_code_node_ref0 ((_tmp67_ = vala_formal_parameter_get_parameter_type (param), VALA_IS_ARRAY_TYPE (_tmp67_) ? ((ValaArrayType*) _tmp67_) : NULL));
			if (array_type != NULL) {
				if (vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) != vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->string_type)) {
					ValaCCodeExpression* _tmp68_;
					ValaCCodeIdentifier* _tmp69_;
					ValaCCodeFunctionCall* _tmp70_;
					ValaCCodeFunctionCall* sizeof_call;
					ValaCCodeIdentifier* _tmp72_;
					char* _tmp71_;
					ValaCCodeDeclaration* _cdecl_;
					ValaCCodeFunctionCall* array_construct;
					ValaCCodeVariableDeclarator* _tmp84_;
					char* _tmp83_;
					ValaCCodeIdentifier* _tmp111_;
					char* _tmp110_;
					vala_ccode_function_call_add_argument (ccall, _tmp68_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, (ValaDataType*) array_type));
					_vala_ccode_node_unref0 (_tmp68_);
					sizeof_call = (_tmp70_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp69_ = vala_ccode_identifier_new ("sizeof"))), _vala_ccode_node_unref0 (_tmp69_), _tmp70_);
					vala_ccode_function_call_add_argument (sizeof_call, (ValaCCodeExpression*) (_tmp72_ = vala_ccode_identifier_new (_tmp71_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
					_vala_ccode_node_unref0 (_tmp72_);
					_g_free0 (_tmp71_);
					_cdecl_ = NULL;
					array_construct = NULL;
					if (vala_dbus_client_module_dbus_use_ptr_array (self, array_type)) {
						ValaCCodeDeclaration* _tmp73_;
						ValaCCodeFunctionCall* _tmp75_;
						ValaCCodeIdentifier* _tmp74_;
						ValaCCodeIdentifier* _tmp77_;
						char* _tmp76_;
						_cdecl_ = (_tmp73_ = vala_ccode_declaration_new ("GPtrArray*"), _vala_ccode_node_unref0 (_cdecl_), _tmp73_);
						array_construct = (_tmp75_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp74_ = vala_ccode_identifier_new ("g_ptr_array_sized_new"))), _vala_ccode_node_unref0 (array_construct), _tmp75_);
						_vala_ccode_node_unref0 (_tmp74_);
						vala_ccode_function_call_add_argument (array_construct, (ValaCCodeExpression*) (_tmp77_ = vala_ccode_identifier_new (_tmp76_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) param), 1))));
						_vala_ccode_node_unref0 (_tmp77_);
						_g_free0 (_tmp76_);
					} else {
						ValaCCodeDeclaration* _tmp78_;
						ValaCCodeFunctionCall* _tmp80_;
						ValaCCodeIdentifier* _tmp79_;
						ValaCCodeConstant* _tmp81_;
						ValaCCodeConstant* _tmp82_;
						_cdecl_ = (_tmp78_ = vala_ccode_declaration_new ("GArray*"), _vala_ccode_node_unref0 (_cdecl_), _tmp78_);
						array_construct = (_tmp80_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp79_ = vala_ccode_identifier_new ("g_array_new"))), _vala_ccode_node_unref0 (array_construct), _tmp80_);
						_vala_ccode_node_unref0 (_tmp79_);
						vala_ccode_function_call_add_argument (array_construct, (ValaCCodeExpression*) (_tmp81_ = vala_ccode_constant_new ("TRUE")));
						_vala_ccode_node_unref0 (_tmp81_);
						vala_ccode_function_call_add_argument (array_construct, (ValaCCodeExpression*) (_tmp82_ = vala_ccode_constant_new ("TRUE")));
						_vala_ccode_node_unref0 (_tmp82_);
						vala_ccode_function_call_add_argument (array_construct, (ValaCCodeExpression*) sizeof_call);
					}
					vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp84_ = vala_ccode_variable_declarator_new (_tmp83_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)), (ValaCCodeExpression*) array_construct, NULL)));
					_vala_ccode_node_unref0 (_tmp84_);
					_g_free0 (_tmp83_);
					vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
					if (vala_dbus_client_module_dbus_use_ptr_array (self, array_type)) {
						ValaCCodeIdentifier* _tmp85_;
						ValaCCodeFunctionCall* _tmp86_;
						ValaCCodeFunctionCall* memcpy_call;
						ValaCCodeMemberAccess* _tmp89_;
						ValaCCodeIdentifier* _tmp88_;
						char* _tmp87_;
						ValaCCodeIdentifier* _tmp90_;
						ValaCCodeBinaryExpression* _tmp93_;
						ValaCCodeIdentifier* _tmp92_;
						char* _tmp91_;
						ValaCCodeExpressionStatement* _tmp94_;
						ValaCCodeIdentifier* _tmp99_;
						char* _tmp98_;
						ValaCCodeMemberAccess* _tmp97_;
						ValaCCodeIdentifier* _tmp96_;
						char* _tmp95_;
						ValaCCodeAssignment* _tmp100_;
						ValaCCodeAssignment* len_assignment;
						ValaCCodeExpressionStatement* _tmp101_;
						vala_ccode_declaration_space_add_include (((ValaCCodeBaseModule*) self)->source_declarations, "string.h", FALSE);
						memcpy_call = (_tmp86_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp85_ = vala_ccode_identifier_new ("memcpy"))), _vala_ccode_node_unref0 (_tmp85_), _tmp86_);
						vala_ccode_function_call_add_argument (memcpy_call, (ValaCCodeExpression*) (_tmp89_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp88_ = vala_ccode_identifier_new (_tmp87_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))), "pdata")));
						_vala_ccode_node_unref0 (_tmp89_);
						_vala_ccode_node_unref0 (_tmp88_);
						_g_free0 (_tmp87_);
						vala_ccode_function_call_add_argument (memcpy_call, (ValaCCodeExpression*) (_tmp90_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
						_vala_ccode_node_unref0 (_tmp90_);
						vala_ccode_function_call_add_argument (memcpy_call, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, (ValaCCodeExpression*) (_tmp92_ = vala_ccode_identifier_new (_tmp91_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) param), 1))), (ValaCCodeExpression*) sizeof_call)));
						_vala_ccode_node_unref0 (_tmp93_);
						_vala_ccode_node_unref0 (_tmp92_);
						_g_free0 (_tmp91_);
						vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp94_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) memcpy_call)));
						_vala_ccode_node_unref0 (_tmp94_);
						len_assignment = (_tmp100_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp97_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp96_ = vala_ccode_identifier_new (_tmp95_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))), "len")), (ValaCCodeExpression*) (_tmp99_ = vala_ccode_identifier_new (_tmp98_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) param), 1))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (_tmp99_), _g_free0 (_tmp98_), _vala_ccode_node_unref0 (_tmp97_), _vala_ccode_node_unref0 (_tmp96_), _g_free0 (_tmp95_), _tmp100_);
						vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp101_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) len_assignment)));
						_vala_ccode_node_unref0 (_tmp101_);
						_vala_ccode_node_unref0 (memcpy_call);
						_vala_ccode_node_unref0 (len_assignment);
					} else {
						ValaCCodeIdentifier* _tmp102_;
						ValaCCodeFunctionCall* _tmp103_;
						ValaCCodeFunctionCall* cappend_call;
						ValaCCodeIdentifier* _tmp105_;
						char* _tmp104_;
						ValaCCodeIdentifier* _tmp106_;
						ValaCCodeIdentifier* _tmp108_;
						char* _tmp107_;
						ValaCCodeExpressionStatement* _tmp109_;
						cappend_call = (_tmp103_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp102_ = vala_ccode_identifier_new ("g_array_append_vals"))), _vala_ccode_node_unref0 (_tmp102_), _tmp103_);
						vala_ccode_function_call_add_argument (cappend_call, (ValaCCodeExpression*) (_tmp105_ = vala_ccode_identifier_new (_tmp104_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))));
						_vala_ccode_node_unref0 (_tmp105_);
						_g_free0 (_tmp104_);
						vala_ccode_function_call_add_argument (cappend_call, (ValaCCodeExpression*) (_tmp106_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
						_vala_ccode_node_unref0 (_tmp106_);
						vala_ccode_function_call_add_argument (cappend_call, (ValaCCodeExpression*) (_tmp108_ = vala_ccode_identifier_new (_tmp107_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) param), 1))));
						_vala_ccode_node_unref0 (_tmp108_);
						_g_free0 (_tmp107_);
						vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp109_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cappend_call)));
						_vala_ccode_node_unref0 (_tmp109_);
						_vala_ccode_node_unref0 (cappend_call);
					}
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp111_ = vala_ccode_identifier_new (_tmp110_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))));
					_vala_ccode_node_unref0 (_tmp111_);
					_g_free0 (_tmp110_);
					_vala_ccode_node_unref0 (sizeof_call);
					_vala_ccode_node_unref0 (_cdecl_);
					_vala_ccode_node_unref0 (array_construct);
				} else {
					ValaCCodeIdentifier* _tmp112_;
					ValaCCodeIdentifier* _tmp113_;
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp112_ = vala_ccode_identifier_new ("G_TYPE_STRV")));
					_vala_ccode_node_unref0 (_tmp112_);
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp113_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
					_vala_ccode_node_unref0 (_tmp113_);
				}
			} else {
				char* _tmp114_;
				gboolean _tmp115_;
				if ((_tmp115_ = g_str_has_prefix (_tmp114_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param)), "("), _g_free0 (_tmp114_), _tmp115_)) {
					ValaStruct* st;
					ValaCCodeIdentifier* _tmp116_;
					ValaCCodeFunctionCall* _tmp117_;
					ValaCCodeFunctionCall* array_construct;
					ValaCCodeConstant* _tmp118_;
					ValaCCodeDeclaration* _cdecl_;
					ValaCCodeVariableDeclarator* _tmp120_;
					char* _tmp119_;
					ValaCCodeExpression* _tmp143_;
					ValaCCodeIdentifier* _tmp145_;
					char* _tmp144_;
					st = _vala_code_node_ref0 (VALA_STRUCT (vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param))));
					array_construct = (_tmp117_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp116_ = vala_ccode_identifier_new ("g_value_array_new"))), _vala_ccode_node_unref0 (_tmp116_), _tmp117_);
					vala_ccode_function_call_add_argument (array_construct, (ValaCCodeExpression*) (_tmp118_ = vala_ccode_constant_new ("0")));
					_vala_ccode_node_unref0 (_tmp118_);
					_cdecl_ = vala_ccode_declaration_new ("GValueArray*");
					vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp120_ = vala_ccode_variable_declarator_new (_tmp119_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)), (ValaCCodeExpression*) array_construct, NULL)));
					_vala_ccode_node_unref0 (_tmp120_);
					_g_free0 (_tmp119_);
					vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
					{
						GeeList* _tmp121_;
						GeeIterator* _tmp122_;
						GeeIterator* _f_it;
						_f_it = (_tmp122_ = gee_iterable_iterator ((GeeIterable*) (_tmp121_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp121_), _tmp122_);
						while (TRUE) {
							ValaField* f;
							char* val_name;
							ValaCCodeInitializerList* cvalinit;
							ValaCCodeConstant* _tmp123_;
							ValaCCodeDeclaration* cval_decl;
							ValaCCodeVariableDeclarator* _tmp124_;
							ValaCCodeIdentifier* _tmp125_;
							ValaCCodeUnaryExpression* _tmp126_;
							ValaCCodeUnaryExpression* val_ptr;
							ValaCCodeIdentifier* _tmp127_;
							ValaCCodeFunctionCall* _tmp128_;
							ValaCCodeFunctionCall* cinit_call;
							ValaCCodeIdentifier* _tmp130_;
							char* _tmp129_;
							ValaCCodeExpressionStatement* _tmp131_;
							ValaCCodeIdentifier* _tmp133_;
							char* _tmp132_;
							ValaCCodeFunctionCall* _tmp134_;
							ValaCCodeFunctionCall* cset_call;
							ValaCCodeMemberAccess* _tmp136_;
							ValaCCodeIdentifier* _tmp135_;
							ValaCCodeExpressionStatement* _tmp137_;
							ValaCCodeIdentifier* _tmp138_;
							ValaCCodeFunctionCall* _tmp139_;
							ValaCCodeFunctionCall* cappend_call;
							ValaCCodeIdentifier* _tmp141_;
							char* _tmp140_;
							ValaCCodeExpressionStatement* _tmp142_;
							if (!gee_iterator_next (_f_it)) {
								break;
							}
							f = (ValaField*) gee_iterator_get (_f_it);
							if (vala_field_get_binding (f) != MEMBER_BINDING_INSTANCE) {
								_vala_code_node_unref0 (f);
								continue;
							}
							val_name = g_strdup_printf ("val_%s_%s", vala_symbol_get_name ((ValaSymbol*) param), vala_symbol_get_name ((ValaSymbol*) f));
							cvalinit = vala_ccode_initializer_list_new ();
							vala_ccode_initializer_list_append (cvalinit, (ValaCCodeExpression*) (_tmp123_ = vala_ccode_constant_new ("0")));
							_vala_ccode_node_unref0 (_tmp123_);
							cval_decl = vala_ccode_declaration_new ("GValue");
							vala_ccode_declaration_add_declarator (cval_decl, (ValaCCodeDeclarator*) (_tmp124_ = vala_ccode_variable_declarator_new (val_name, (ValaCCodeExpression*) cvalinit, NULL)));
							_vala_ccode_node_unref0 (_tmp124_);
							vala_ccode_block_add_statement (block, (ValaCCodeNode*) cval_decl);
							val_ptr = (_tmp126_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp125_ = vala_ccode_identifier_new (val_name))), _vala_ccode_node_unref0 (_tmp125_), _tmp126_);
							cinit_call = (_tmp128_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp127_ = vala_ccode_identifier_new ("g_value_init"))), _vala_ccode_node_unref0 (_tmp127_), _tmp128_);
							vala_ccode_function_call_add_argument (cinit_call, (ValaCCodeExpression*) val_ptr);
							vala_ccode_function_call_add_argument (cinit_call, (ValaCCodeExpression*) (_tmp130_ = vala_ccode_identifier_new (_tmp129_ = vala_typesymbol_get_type_id (vala_data_type_get_data_type (vala_field_get_field_type (f))))));
							_vala_ccode_node_unref0 (_tmp130_);
							_g_free0 (_tmp129_);
							vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp131_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cinit_call)));
							_vala_ccode_node_unref0 (_tmp131_);
							cset_call = (_tmp134_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp133_ = vala_ccode_identifier_new (_tmp132_ = vala_typesymbol_get_set_value_function (vala_data_type_get_data_type (vala_field_get_field_type (f)))))), _vala_ccode_node_unref0 (_tmp133_), _g_free0 (_tmp132_), _tmp134_);
							vala_ccode_function_call_add_argument (cset_call, (ValaCCodeExpression*) val_ptr);
							vala_ccode_function_call_add_argument (cset_call, (ValaCCodeExpression*) (_tmp136_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp135_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))), vala_symbol_get_name ((ValaSymbol*) f))));
							_vala_ccode_node_unref0 (_tmp136_);
							_vala_ccode_node_unref0 (_tmp135_);
							vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp137_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cset_call)));
							_vala_ccode_node_unref0 (_tmp137_);
							cappend_call = (_tmp139_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp138_ = vala_ccode_identifier_new ("g_value_array_append"))), _vala_ccode_node_unref0 (_tmp138_), _tmp139_);
							vala_ccode_function_call_add_argument (cappend_call, (ValaCCodeExpression*) (_tmp141_ = vala_ccode_identifier_new (_tmp140_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))));
							_vala_ccode_node_unref0 (_tmp141_);
							_g_free0 (_tmp140_);
							vala_ccode_function_call_add_argument (cappend_call, (ValaCCodeExpression*) val_ptr);
							vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp142_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cappend_call)));
							_vala_ccode_node_unref0 (_tmp142_);
							_vala_code_node_unref0 (f);
							_g_free0 (val_name);
							_vala_ccode_node_unref0 (cvalinit);
							_vala_ccode_node_unref0 (cval_decl);
							_vala_ccode_node_unref0 (val_ptr);
							_vala_ccode_node_unref0 (cinit_call);
							_vala_ccode_node_unref0 (cset_call);
							_vala_ccode_node_unref0 (cappend_call);
						}
						_gee_collection_object_unref0 (_f_it);
					}
					vala_ccode_function_call_add_argument (ccall, _tmp143_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_formal_parameter_get_parameter_type (param)));
					_vala_ccode_node_unref0 (_tmp143_);
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp145_ = vala_ccode_identifier_new (_tmp144_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))));
					_vala_ccode_node_unref0 (_tmp145_);
					_g_free0 (_tmp144_);
					_vala_code_node_unref0 (st);
					_vala_ccode_node_unref0 (array_construct);
					_vala_ccode_node_unref0 (_cdecl_);
				} else {
					ValaCCodeExpression* _tmp146_;
					ValaCCodeIdentifier* _tmp147_;
					vala_ccode_function_call_add_argument (ccall, _tmp146_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_formal_parameter_get_parameter_type (param)));
					_vala_ccode_node_unref0 (_tmp146_);
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp147_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
					_vala_ccode_node_unref0 (_tmp147_);
				}
			}
			_vala_code_node_unref0 (param);
			_vala_code_node_unref0 (array_type);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp148_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
	_vala_ccode_node_unref0 (_tmp148_);
	out_marshalling_fragment = vala_ccode_fragment_new ();
	{
		GeeList* _tmp149_;
		GeeIterator* _tmp150_;
		GeeIterator* _param_it;
		_param_it = (_tmp150_ = gee_iterable_iterator ((GeeIterable*) (_tmp149_ = vala_method_get_parameters (method))), _gee_collection_object_unref0 (_tmp149_), _tmp150_);
		while (TRUE) {
			ValaFormalParameter* param;
			char* _tmp151_;
			gboolean _tmp152_;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			if (VALA_IS_METHOD_TYPE (vala_formal_parameter_get_parameter_type (param))) {
				_vala_code_node_unref0 (param);
				break;
			}
			if (vala_formal_parameter_get_direction (param) != VALA_PARAMETER_DIRECTION_OUT) {
				_vala_code_node_unref0 (param);
				continue;
			}
			if ((_tmp152_ = g_str_has_prefix (_tmp151_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param)), "("), _g_free0 (_tmp151_), _tmp152_)) {
				ValaStruct* st;
				ValaCCodeDeclaration* _cdecl_;
				ValaCCodeVariableDeclarator* _tmp154_;
				char* _tmp153_;
				gint i;
				ValaCCodeExpression* _tmp172_;
				ValaCCodeUnaryExpression* _tmp175_;
				ValaCCodeIdentifier* _tmp174_;
				char* _tmp173_;
				st = _vala_code_node_ref0 (VALA_STRUCT (vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param))));
				_cdecl_ = vala_ccode_declaration_new ("GValueArray*");
				vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp154_ = vala_ccode_variable_declarator_new (_tmp153_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)), NULL, NULL)));
				_vala_ccode_node_unref0 (_tmp154_);
				_g_free0 (_tmp153_);
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
				i = 0;
				{
					GeeList* _tmp155_;
					GeeIterator* _tmp156_;
					GeeIterator* _f_it;
					_f_it = (_tmp156_ = gee_iterable_iterator ((GeeIterable*) (_tmp155_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp155_), _tmp156_);
					while (TRUE) {
						ValaField* f;
						ValaCCodeIdentifier* _tmp158_;
						char* _tmp157_;
						ValaCCodeFunctionCall* _tmp159_;
						ValaCCodeFunctionCall* cget_call;
						ValaCCodeUnaryExpression* _tmp166_;
						ValaCCodeElementAccess* _tmp165_;
						ValaCCodeConstant* _tmp164_;
						char* _tmp163_;
						ValaCCodeMemberAccess* _tmp162_;
						ValaCCodeIdentifier* _tmp161_;
						char* _tmp160_;
						ValaCCodeFunctionCall* converted_value;
						ValaCCodeMemberAccess* _tmp169_;
						ValaCCodeIdentifier* _tmp168_;
						ValaCCodeAssignment* _tmp170_;
						ValaCCodeAssignment* assign;
						ValaCCodeExpressionStatement* _tmp171_;
						if (!gee_iterator_next (_f_it)) {
							break;
						}
						f = (ValaField*) gee_iterator_get (_f_it);
						if (vala_field_get_binding (f) != MEMBER_BINDING_INSTANCE) {
							_vala_code_node_unref0 (f);
							continue;
						}
						cget_call = (_tmp159_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp158_ = vala_ccode_identifier_new (_tmp157_ = vala_typesymbol_get_get_value_function (vala_data_type_get_data_type (vala_field_get_field_type (f)))))), _vala_ccode_node_unref0 (_tmp158_), _g_free0 (_tmp157_), _tmp159_);
						vala_ccode_function_call_add_argument (cget_call, (ValaCCodeExpression*) (_tmp166_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp165_ = vala_ccode_element_access_new ((ValaCCodeExpression*) (_tmp162_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp161_ = vala_ccode_identifier_new (_tmp160_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))), "values")), (ValaCCodeExpression*) (_tmp164_ = vala_ccode_constant_new (_tmp163_ = g_strdup_printf ("%i", i))))))));
						_vala_ccode_node_unref0 (_tmp166_);
						_vala_ccode_node_unref0 (_tmp165_);
						_vala_ccode_node_unref0 (_tmp164_);
						_g_free0 (_tmp163_);
						_vala_ccode_node_unref0 (_tmp162_);
						_vala_ccode_node_unref0 (_tmp161_);
						_g_free0 (_tmp160_);
						converted_value = _vala_ccode_node_ref0 (cget_call);
						if (vala_ccode_base_module_requires_copy ((ValaCCodeBaseModule*) self, vala_field_get_field_type (f))) {
							ValaCCodeExpression* dupexpr;
							ValaCCodeFunctionCall* _tmp167_;
							dupexpr = vala_ccode_base_module_get_dup_func_expression ((ValaCCodeBaseModule*) self, vala_field_get_field_type (f), vala_code_node_get_source_reference ((ValaCodeNode*) expr), FALSE);
							converted_value = (_tmp167_ = vala_ccode_function_call_new (dupexpr), _vala_ccode_node_unref0 (converted_value), _tmp167_);
							vala_ccode_function_call_add_argument (converted_value, (ValaCCodeExpression*) cget_call);
							_vala_ccode_node_unref0 (dupexpr);
						}
						assign = (_tmp170_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp169_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp168_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))), vala_symbol_get_name ((ValaSymbol*) f))), (ValaCCodeExpression*) converted_value, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (_tmp169_), _vala_ccode_node_unref0 (_tmp168_), _tmp170_);
						vala_ccode_fragment_append (out_marshalling_fragment, (ValaCCodeNode*) (_tmp171_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) assign)));
						_vala_ccode_node_unref0 (_tmp171_);
						i++;
						_vala_code_node_unref0 (f);
						_vala_ccode_node_unref0 (cget_call);
						_vala_ccode_node_unref0 (converted_value);
						_vala_ccode_node_unref0 (assign);
					}
					_gee_collection_object_unref0 (_f_it);
				}
				vala_ccode_function_call_add_argument (ccall, _tmp172_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_formal_parameter_get_parameter_type (param)));
				_vala_ccode_node_unref0 (_tmp172_);
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp175_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp174_ = vala_ccode_identifier_new (_tmp173_ = g_strdup_printf ("dbus_%s", vala_symbol_get_name ((ValaSymbol*) param)))))));
				_vala_ccode_node_unref0 (_tmp175_);
				_vala_ccode_node_unref0 (_tmp174_);
				_g_free0 (_tmp173_);
				_vala_code_node_unref0 (st);
				_vala_ccode_node_unref0 (_cdecl_);
			} else {
				ValaCCodeExpression* _tmp176_;
				ValaCCodeIdentifier* _tmp177_;
				vala_ccode_function_call_add_argument (ccall, _tmp176_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_formal_parameter_get_parameter_type (param)));
				_vala_ccode_node_unref0 (_tmp176_);
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp177_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
				_vala_ccode_node_unref0 (_tmp177_);
			}
			_vala_code_node_unref0 (param);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (method))) {
		ValaDataType* _tmp178_;
		ValaArrayType* array_type;
		gboolean _tmp179_;
		array_type = _vala_code_node_ref0 ((_tmp178_ = vala_method_get_return_type (method), VALA_IS_ARRAY_TYPE (_tmp178_) ? ((ValaArrayType*) _tmp178_) : NULL));
		_tmp179_ = FALSE;
		if (array_type != NULL) {
			_tmp179_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) != vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->string_type);
		} else {
			_tmp179_ = FALSE;
		}
		if (_tmp179_) {
			ValaCCodeExpression* _tmp180_;
			ValaCCodeDeclaration* _cdecl_;
			ValaCCodeVariableDeclarator* _tmp183_;
			ValaCCodeUnaryExpression* _tmp185_;
			ValaCCodeIdentifier* _tmp184_;
			ValaCCodeIdentifier* _tmp186_;
			ValaCCodeExpressionStatement* _tmp187_;
			ValaCCodeBlock* creturnblock;
			ValaCCodeReturnStatement* _tmp189_;
			ValaCCodeExpression* _tmp188_;
			ValaCCodeUnaryExpression* _tmp191_;
			ValaCCodeIdentifier* _tmp190_;
			ValaCCodeIfStatement* _tmp192_;
			ValaCCodeIfStatement* cerrorif;
			ValaCCodeIdentifier* _tmp193_;
			ValaCCodeMemberAccess* _tmp194_;
			ValaCCodeMemberAccess* garray_length;
			ValaCCodeIdentifier* _tmp195_;
			ValaCCodeUnaryExpression* _tmp196_;
			ValaCCodeUnaryExpression* result_length;
			ValaCCodeAssignment* assign;
			ValaCCodeExpressionStatement* _tmp197_;
			const char* _tmp198_;
			ValaCCodeReturnStatement* _tmp203_;
			ValaCCodeCastExpression* _tmp202_;
			char* _tmp201_;
			ValaCCodeMemberAccess* _tmp200_;
			ValaCCodeIdentifier* _tmp199_;
			vala_ccode_function_call_add_argument (ccall, _tmp180_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, (ValaDataType*) array_type));
			_vala_ccode_node_unref0 (_tmp180_);
			_cdecl_ = NULL;
			if (vala_dbus_client_module_dbus_use_ptr_array (self, array_type)) {
				ValaCCodeDeclaration* _tmp181_;
				_cdecl_ = (_tmp181_ = vala_ccode_declaration_new ("GPtrArray*"), _vala_ccode_node_unref0 (_cdecl_), _tmp181_);
			} else {
				ValaCCodeDeclaration* _tmp182_;
				_cdecl_ = (_tmp182_ = vala_ccode_declaration_new ("GArray*"), _vala_ccode_node_unref0 (_cdecl_), _tmp182_);
			}
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp183_ = vala_ccode_variable_declarator_new ("result", NULL, NULL)));
			_vala_ccode_node_unref0 (_tmp183_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp185_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp184_ = vala_ccode_identifier_new ("result")))));
			_vala_ccode_node_unref0 (_tmp185_);
			_vala_ccode_node_unref0 (_tmp184_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp186_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
			_vala_ccode_node_unref0 (_tmp186_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp187_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
			_vala_ccode_node_unref0 (_tmp187_);
			creturnblock = vala_ccode_block_new ();
			vala_ccode_block_add_statement (creturnblock, (ValaCCodeNode*) (_tmp189_ = vala_ccode_return_statement_new (_tmp188_ = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, vala_method_get_return_type (method), FALSE))));
			_vala_ccode_node_unref0 (_tmp189_);
			_vala_ccode_node_unref0 (_tmp188_);
			cerrorif = (_tmp192_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp191_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp190_ = vala_ccode_identifier_new ("error")))), (ValaCCodeStatement*) creturnblock, NULL), _vala_ccode_node_unref0 (_tmp191_), _vala_ccode_node_unref0 (_tmp190_), _tmp192_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) cerrorif);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) out_marshalling_fragment);
			garray_length = (_tmp194_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp193_ = vala_ccode_identifier_new ("result")), "len"), _vala_ccode_node_unref0 (_tmp193_), _tmp194_);
			result_length = (_tmp196_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp195_ = vala_ccode_identifier_new ("result_length1"))), _vala_ccode_node_unref0 (_tmp195_), _tmp196_);
			assign = vala_ccode_assignment_new ((ValaCCodeExpression*) result_length, (ValaCCodeExpression*) garray_length, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp197_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) assign)));
			_vala_ccode_node_unref0 (_tmp197_);
			_tmp198_ = NULL;
			if (vala_dbus_client_module_dbus_use_ptr_array (self, array_type)) {
				_tmp198_ = "pdata";
			} else {
				_tmp198_ = "data";
			}
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp203_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp202_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp200_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp199_ = vala_ccode_identifier_new ("result")), _tmp198_)), _tmp201_ = vala_data_type_get_cname (vala_method_get_return_type (method)))))));
			_vala_ccode_node_unref0 (_tmp203_);
			_vala_ccode_node_unref0 (_tmp202_);
			_g_free0 (_tmp201_);
			_vala_ccode_node_unref0 (_tmp200_);
			_vala_ccode_node_unref0 (_tmp199_);
			_vala_ccode_node_unref0 (_cdecl_);
			_vala_ccode_node_unref0 (creturnblock);
			_vala_ccode_node_unref0 (cerrorif);
			_vala_ccode_node_unref0 (garray_length);
			_vala_ccode_node_unref0 (result_length);
			_vala_ccode_node_unref0 (assign);
		} else {
			ValaCCodeExpression* _tmp204_;
			char* _tmp205_;
			ValaCCodeDeclaration* _tmp206_;
			ValaCCodeDeclaration* _cdecl_;
			ValaCCodeVariableDeclarator* _tmp207_;
			ValaCCodeUnaryExpression* _tmp209_;
			ValaCCodeIdentifier* _tmp208_;
			ValaCCodeIdentifier* _tmp210_;
			ValaCCodeExpressionStatement* _tmp211_;
			ValaCCodeBlock* creturnblock;
			ValaCCodeReturnStatement* _tmp213_;
			ValaCCodeExpression* _tmp212_;
			ValaCCodeUnaryExpression* _tmp215_;
			ValaCCodeIdentifier* _tmp214_;
			ValaCCodeIfStatement* _tmp216_;
			ValaCCodeIfStatement* cerrorif;
			ValaCCodeReturnStatement* _tmp224_;
			ValaCCodeIdentifier* _tmp223_;
			vala_ccode_function_call_add_argument (ccall, _tmp204_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_method_get_return_type (method)));
			_vala_ccode_node_unref0 (_tmp204_);
			_cdecl_ = (_tmp206_ = vala_ccode_declaration_new (_tmp205_ = vala_data_type_get_cname (vala_method_get_return_type (method))), _g_free0 (_tmp205_), _tmp206_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp207_ = vala_ccode_variable_declarator_new ("result", NULL, NULL)));
			_vala_ccode_node_unref0 (_tmp207_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp209_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp208_ = vala_ccode_identifier_new ("result")))));
			_vala_ccode_node_unref0 (_tmp209_);
			_vala_ccode_node_unref0 (_tmp208_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp210_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
			_vala_ccode_node_unref0 (_tmp210_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp211_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
			_vala_ccode_node_unref0 (_tmp211_);
			creturnblock = vala_ccode_block_new ();
			vala_ccode_block_add_statement (creturnblock, (ValaCCodeNode*) (_tmp213_ = vala_ccode_return_statement_new (_tmp212_ = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, vala_method_get_return_type (method), FALSE))));
			_vala_ccode_node_unref0 (_tmp213_);
			_vala_ccode_node_unref0 (_tmp212_);
			cerrorif = (_tmp216_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp215_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp214_ = vala_ccode_identifier_new ("error")))), (ValaCCodeStatement*) creturnblock, NULL), _vala_ccode_node_unref0 (_tmp215_), _vala_ccode_node_unref0 (_tmp214_), _tmp216_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) cerrorif);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) out_marshalling_fragment);
			if (array_type != NULL) {
				ValaCCodeIdentifier* _tmp217_;
				ValaCCodeFunctionCall* _tmp218_;
				ValaCCodeFunctionCall* cstrvlen;
				ValaCCodeIdentifier* _tmp219_;
				ValaCCodeIdentifier* _tmp220_;
				ValaCCodeUnaryExpression* _tmp221_;
				ValaCCodeUnaryExpression* result_length;
				ValaCCodeAssignment* assign;
				ValaCCodeExpressionStatement* _tmp222_;
				cstrvlen = (_tmp218_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp217_ = vala_ccode_identifier_new ("g_strv_length"))), _vala_ccode_node_unref0 (_tmp217_), _tmp218_);
				vala_ccode_function_call_add_argument (cstrvlen, (ValaCCodeExpression*) (_tmp219_ = vala_ccode_identifier_new ("result")));
				_vala_ccode_node_unref0 (_tmp219_);
				result_length = (_tmp221_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp220_ = vala_ccode_identifier_new ("result_length1"))), _vala_ccode_node_unref0 (_tmp220_), _tmp221_);
				assign = vala_ccode_assignment_new ((ValaCCodeExpression*) result_length, (ValaCCodeExpression*) cstrvlen, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp222_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) assign)));
				_vala_ccode_node_unref0 (_tmp222_);
				_vala_ccode_node_unref0 (cstrvlen);
				_vala_ccode_node_unref0 (result_length);
				_vala_ccode_node_unref0 (assign);
			}
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp224_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp223_ = vala_ccode_identifier_new ("result")))));
			_vala_ccode_node_unref0 (_tmp224_);
			_vala_ccode_node_unref0 (_tmp223_);
			_vala_ccode_node_unref0 (_cdecl_);
			_vala_ccode_node_unref0 (creturnblock);
			_vala_ccode_node_unref0 (cerrorif);
		}
		_vala_code_node_unref0 (array_type);
	} else {
		ValaCCodeIdentifier* _tmp225_;
		ValaCCodeExpressionStatement* _tmp226_;
		ValaCCodeBlock* creturnblock;
		ValaCCodeReturnStatement* _tmp227_;
		ValaCCodeUnaryExpression* _tmp229_;
		ValaCCodeIdentifier* _tmp228_;
		ValaCCodeIfStatement* _tmp230_;
		ValaCCodeIfStatement* cerrorif;
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp225_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
		_vala_ccode_node_unref0 (_tmp225_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp226_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
		_vala_ccode_node_unref0 (_tmp226_);
		creturnblock = vala_ccode_block_new ();
		vala_ccode_block_add_statement (creturnblock, (ValaCCodeNode*) (_tmp227_ = vala_ccode_return_statement_new (NULL)));
		_vala_ccode_node_unref0 (_tmp227_);
		cerrorif = (_tmp230_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp229_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp228_ = vala_ccode_identifier_new ("error")))), (ValaCCodeStatement*) creturnblock, NULL), _vala_ccode_node_unref0 (_tmp229_), _vala_ccode_node_unref0 (_tmp228_), _tmp230_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) cerrorif);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) out_marshalling_fragment);
		_vala_ccode_node_unref0 (creturnblock);
		_vala_ccode_node_unref0 (cerrorif);
	}
	_vala_code_node_unref0 (dynamic_method);
	_vala_code_node_unref0 (expr);
	_vala_ccode_node_unref0 (ccall);
	_vala_code_node_unref0 (callback);
	_vala_ccode_node_unref0 (out_marshalling_fragment);
}


static ValaCCodeExpression* vala_dbus_client_module_real_get_dbus_g_type (ValaGSignalModule* base, ValaDataType* data_type) {
	ValaDBusClientModule * self;
	ValaCCodeExpression* result;
	self = (ValaDBusClientModule*) base;
	g_return_val_if_fail (data_type != NULL, NULL);
	if (VALA_IS_ARRAY_TYPE (data_type)) {
		ValaDataType* _tmp0_;
		ValaArrayType* array_type;
		ValaCCodeIdentifier* _tmp1_;
		ValaCCodeFunctionCall* _tmp2_;
		ValaCCodeFunctionCall* carray_type;
		ValaCCodeExpression* _tmp5_;
		array_type = _vala_code_node_ref0 ((_tmp0_ = data_type, VALA_IS_ARRAY_TYPE (_tmp0_) ? ((ValaArrayType*) _tmp0_) : NULL));
		if (vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->string_type)) {
			result = (ValaCCodeExpression*) vala_ccode_identifier_new ("G_TYPE_STRV");
			_vala_code_node_unref0 (array_type);
			return result;
		}
		carray_type = (_tmp2_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp1_ = vala_ccode_identifier_new ("dbus_g_type_get_collection"))), _vala_ccode_node_unref0 (_tmp1_), _tmp2_);
		if (vala_dbus_client_module_dbus_use_ptr_array (self, array_type)) {
			ValaCCodeConstant* _tmp3_;
			vala_ccode_function_call_add_argument (carray_type, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_constant_new ("\"GPtrArray\"")));
			_vala_ccode_node_unref0 (_tmp3_);
		} else {
			ValaCCodeConstant* _tmp4_;
			vala_ccode_function_call_add_argument (carray_type, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new ("\"GArray\"")));
			_vala_ccode_node_unref0 (_tmp4_);
		}
		vala_ccode_function_call_add_argument (carray_type, _tmp5_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_array_type_get_element_type (array_type)));
		_vala_ccode_node_unref0 (_tmp5_);
		result = (ValaCCodeExpression*) carray_type;
		_vala_code_node_unref0 (array_type);
		return result;
	} else {
		if (VALA_IS_ENUM (vala_data_type_get_data_type (data_type))) {
			ValaEnum* en;
			en = _vala_code_node_ref0 (VALA_ENUM (vala_data_type_get_data_type (data_type)));
			if (vala_enum_get_is_flags (en)) {
				result = (ValaCCodeExpression*) vala_ccode_identifier_new ("G_TYPE_UINT");
				_vala_code_node_unref0 (en);
				return result;
			} else {
				result = (ValaCCodeExpression*) vala_ccode_identifier_new ("G_TYPE_INT");
				_vala_code_node_unref0 (en);
				return result;
			}
			_vala_code_node_unref0 (en);
		} else {
			if (vala_data_type_get_data_type (data_type) == NULL) {
				char* _tmp6_;
				g_critical ("valadbusclientmodule.vala:488: Internal error during DBus type generation with: %s", _tmp6_ = vala_code_node_to_string ((ValaCodeNode*) data_type));
				_g_free0 (_tmp6_);
				result = (ValaCCodeExpression*) vala_ccode_identifier_new ("G_TYPE_NONE");
				return result;
			} else {
				char* _tmp7_;
				gboolean _tmp8_;
				if ((_tmp8_ = _vala_strcmp0 (_tmp7_ = vala_symbol_get_full_name ((ValaSymbol*) vala_data_type_get_data_type (data_type)), "GLib.HashTable") == 0, _g_free0 (_tmp7_), _tmp8_)) {
					ValaCCodeIdentifier* _tmp9_;
					ValaCCodeFunctionCall* _tmp10_;
					ValaCCodeFunctionCall* cmap_type;
					GeeList* type_args;
					ValaCCodeConstant* _tmp11_;
					cmap_type = (_tmp10_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("dbus_g_type_get_map"))), _vala_ccode_node_unref0 (_tmp9_), _tmp10_);
					type_args = vala_data_type_get_type_arguments (data_type);
					vala_ccode_function_call_add_argument (cmap_type, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_constant_new ("\"GHashTable\"")));
					_vala_ccode_node_unref0 (_tmp11_);
					{
						GeeIterator* _type_arg_it;
						_type_arg_it = gee_iterable_iterator ((GeeIterable*) type_args);
						while (TRUE) {
							ValaDataType* type_arg;
							ValaCCodeExpression* _tmp12_;
							if (!gee_iterator_next (_type_arg_it)) {
								break;
							}
							type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
							vala_ccode_function_call_add_argument (cmap_type, _tmp12_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, type_arg));
							_vala_ccode_node_unref0 (_tmp12_);
							_vala_code_node_unref0 (type_arg);
						}
						_gee_collection_object_unref0 (_type_arg_it);
					}
					result = (ValaCCodeExpression*) cmap_type;
					_gee_collection_object_unref0 (type_args);
					return result;
				} else {
					char* _tmp13_;
					gboolean _tmp14_;
					if ((_tmp14_ = g_str_has_prefix (_tmp13_ = vala_dbus_module_get_type_signature (data_type), "("), _g_free0 (_tmp13_), _tmp14_)) {
						ValaStruct* st;
						ValaCCodeIdentifier* _tmp15_;
						ValaCCodeFunctionCall* _tmp16_;
						ValaCCodeFunctionCall* type_call;
						ValaCCodeConstant* _tmp17_;
						ValaCCodeConstant* _tmp21_;
						st = _vala_code_node_ref0 (VALA_STRUCT (vala_data_type_get_data_type (data_type)));
						type_call = (_tmp16_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("dbus_g_type_get_struct"))), _vala_ccode_node_unref0 (_tmp15_), _tmp16_);
						vala_ccode_function_call_add_argument (type_call, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_constant_new ("\"GValueArray\"")));
						_vala_ccode_node_unref0 (_tmp17_);
						{
							GeeList* _tmp18_;
							GeeIterator* _tmp19_;
							GeeIterator* _f_it;
							_f_it = (_tmp19_ = gee_iterable_iterator ((GeeIterable*) (_tmp18_ = vala_struct_get_fields (st))), _gee_collection_object_unref0 (_tmp18_), _tmp19_);
							while (TRUE) {
								ValaField* f;
								ValaCCodeExpression* _tmp20_;
								if (!gee_iterator_next (_f_it)) {
									break;
								}
								f = (ValaField*) gee_iterator_get (_f_it);
								if (vala_field_get_binding (f) != MEMBER_BINDING_INSTANCE) {
									_vala_code_node_unref0 (f);
									continue;
								}
								vala_ccode_function_call_add_argument (type_call, _tmp20_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_field_get_field_type (f)));
								_vala_ccode_node_unref0 (_tmp20_);
								_vala_code_node_unref0 (f);
							}
							_gee_collection_object_unref0 (_f_it);
						}
						vala_ccode_function_call_add_argument (type_call, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_constant_new ("G_TYPE_INVALID")));
						_vala_ccode_node_unref0 (_tmp21_);
						result = (ValaCCodeExpression*) type_call;
						_vala_code_node_unref0 (st);
						return result;
					} else {
						char* _tmp22_;
						ValaCCodeExpression* _tmp23_;
						result = (_tmp23_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp22_ = vala_typesymbol_get_type_id (vala_data_type_get_data_type (data_type))), _g_free0 (_tmp22_), _tmp23_);
						return result;
					}
				}
			}
		}
	}
}


gboolean vala_dbus_client_module_dbus_use_ptr_array (ValaDBusClientModule* self, ValaArrayType* array_type) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (array_type != NULL, FALSE);
	if (vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->string_type)) {
		result = FALSE;
		return result;
	} else {
		gboolean _tmp0_;
		gboolean _tmp1_;
		gboolean _tmp2_;
		gboolean _tmp3_;
		gboolean _tmp4_;
		gboolean _tmp5_;
		gboolean _tmp6_;
		gboolean _tmp7_;
		gboolean _tmp8_;
		gboolean _tmp9_;
		gboolean _tmp10_;
		gboolean _tmp11_;
		gboolean _tmp12_;
		_tmp0_ = FALSE;
		_tmp1_ = FALSE;
		_tmp2_ = FALSE;
		_tmp3_ = FALSE;
		_tmp4_ = FALSE;
		_tmp5_ = FALSE;
		_tmp6_ = FALSE;
		_tmp7_ = FALSE;
		_tmp8_ = FALSE;
		_tmp9_ = FALSE;
		_tmp10_ = FALSE;
		_tmp11_ = FALSE;
		_tmp12_ = FALSE;
		if (vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->bool_type)) {
			_tmp12_ = TRUE;
		} else {
			_tmp12_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->char_type);
		}
		if (_tmp12_) {
			_tmp11_ = TRUE;
		} else {
			_tmp11_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->uchar_type);
		}
		if (_tmp11_) {
			_tmp10_ = TRUE;
		} else {
			_tmp10_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->int_type);
		}
		if (_tmp10_) {
			_tmp9_ = TRUE;
		} else {
			_tmp9_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->uint_type);
		}
		if (_tmp9_) {
			_tmp8_ = TRUE;
		} else {
			_tmp8_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->long_type);
		}
		if (_tmp8_) {
			_tmp7_ = TRUE;
		} else {
			_tmp7_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->ulong_type);
		}
		if (_tmp7_) {
			_tmp6_ = TRUE;
		} else {
			_tmp6_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->int8_type);
		}
		if (_tmp6_) {
			_tmp5_ = TRUE;
		} else {
			_tmp5_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->uint8_type);
		}
		if (_tmp5_) {
			_tmp4_ = TRUE;
		} else {
			_tmp4_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->int32_type);
		}
		if (_tmp4_) {
			_tmp3_ = TRUE;
		} else {
			_tmp3_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->uint32_type);
		}
		if (_tmp3_) {
			_tmp2_ = TRUE;
		} else {
			_tmp2_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->int64_type);
		}
		if (_tmp2_) {
			_tmp1_ = TRUE;
		} else {
			_tmp1_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->uint64_type);
		}
		if (_tmp1_) {
			_tmp0_ = TRUE;
		} else {
			_tmp0_ = vala_data_type_get_data_type (vala_array_type_get_element_type (array_type)) == vala_data_type_get_data_type (((ValaCCodeBaseModule*) self)->double_type);
		}
		if (_tmp0_) {
			result = FALSE;
			return result;
		} else {
			result = TRUE;
			return result;
		}
	}
}


static char* vala_dbus_client_module_real_get_dynamic_property_getter_cname (ValaCCodeModule* base, ValaDynamicProperty* prop) {
	ValaDBusClientModule * self;
	char* result;
	char* getter_cname;
	char* _tmp0_;
	gboolean _tmp1_;
	char* _tmp4_;
	ValaCCodeFunction* _tmp5_;
	ValaCCodeFunction* func;
	ValaCCodeFormalParameter* _tmp7_;
	char* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeFunction* _tmp8_;
	self = (ValaDBusClientModule*) base;
	g_return_val_if_fail (prop != NULL, NULL);
	if (vala_data_type_get_data_type (vala_dynamic_property_get_dynamic_type (prop)) != ((ValaCCodeBaseModule*) self)->dbus_object_type) {
		result = VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->get_dynamic_property_getter_cname ((ValaCCodeModule*) VALA_DBUS_MODULE (self), prop);
		return result;
	}
	getter_cname = g_strdup_printf ("_dynamic_get_%s%d", vala_symbol_get_name ((ValaSymbol*) prop), self->priv->dynamic_property_id++);
	if ((_tmp1_ = (_tmp0_ = vala_dbus_module_get_type_signature (vala_property_get_property_type ((ValaProperty*) prop))) == NULL, _g_free0 (_tmp0_), _tmp1_)) {
		char* _tmp3_;
		char* _tmp2_;
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_property_get_property_type ((ValaProperty*) prop)), _tmp3_ = g_strdup_printf ("D-Bus serialization of type `%s' is not supported", _tmp2_ = vala_code_node_to_string ((ValaCodeNode*) vala_property_get_property_type ((ValaProperty*) prop))));
		_g_free0 (_tmp3_);
		_g_free0 (_tmp2_);
		result = getter_cname;
		return result;
	}
	func = (_tmp5_ = vala_ccode_function_new (getter_cname, _tmp4_ = vala_data_type_get_cname (vala_property_get_property_type ((ValaProperty*) prop))), _g_free0 (_tmp4_), _tmp5_);
	vala_ccode_function_set_modifiers (func, vala_ccode_function_get_modifiers (func) | (VALA_CCODE_MODIFIERS_STATIC | VALA_CCODE_MODIFIERS_INLINE));
	vala_ccode_function_add_parameter (func, _tmp7_ = vala_ccode_formal_parameter_new ("obj", _tmp6_ = vala_data_type_get_cname (vala_dynamic_property_get_dynamic_type (prop))));
	_vala_ccode_node_unref0 (_tmp7_);
	_g_free0 (_tmp6_);
	block = vala_ccode_block_new ();
	vala_dbus_client_module_generate_dbus_property_getter_wrapper (self, prop, block);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp8_ = vala_ccode_function_copy (func)));
	_vala_ccode_node_unref0 (_tmp8_);
	vala_ccode_function_set_block (func, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) func);
	result = getter_cname;
	_vala_ccode_node_unref0 (func);
	_vala_ccode_node_unref0 (block);
	return result;
}


static char* vala_dbus_client_module_real_get_dynamic_property_setter_cname (ValaCCodeModule* base, ValaDynamicProperty* prop) {
	ValaDBusClientModule * self;
	char* result;
	char* setter_cname;
	char* _tmp0_;
	gboolean _tmp1_;
	ValaCCodeFunction* func;
	ValaCCodeFormalParameter* _tmp5_;
	char* _tmp4_;
	ValaCCodeFormalParameter* _tmp7_;
	char* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeFunction* _tmp8_;
	self = (ValaDBusClientModule*) base;
	g_return_val_if_fail (prop != NULL, NULL);
	if (vala_data_type_get_data_type (vala_dynamic_property_get_dynamic_type (prop)) != ((ValaCCodeBaseModule*) self)->dbus_object_type) {
		result = VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->get_dynamic_property_setter_cname ((ValaCCodeModule*) VALA_DBUS_MODULE (self), prop);
		return result;
	}
	setter_cname = g_strdup_printf ("_dynamic_set_%s%d", vala_symbol_get_name ((ValaSymbol*) prop), self->priv->dynamic_property_id++);
	if ((_tmp1_ = (_tmp0_ = vala_dbus_module_get_type_signature (vala_property_get_property_type ((ValaProperty*) prop))) == NULL, _g_free0 (_tmp0_), _tmp1_)) {
		char* _tmp3_;
		char* _tmp2_;
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_property_get_property_type ((ValaProperty*) prop)), _tmp3_ = g_strdup_printf ("D-Bus serialization of type `%s' is not supported", _tmp2_ = vala_code_node_to_string ((ValaCodeNode*) vala_property_get_property_type ((ValaProperty*) prop))));
		_g_free0 (_tmp3_);
		_g_free0 (_tmp2_);
		result = setter_cname;
		return result;
	}
	func = vala_ccode_function_new (setter_cname, "void");
	vala_ccode_function_set_modifiers (func, vala_ccode_function_get_modifiers (func) | (VALA_CCODE_MODIFIERS_STATIC | VALA_CCODE_MODIFIERS_INLINE));
	vala_ccode_function_add_parameter (func, _tmp5_ = vala_ccode_formal_parameter_new ("obj", _tmp4_ = vala_data_type_get_cname (vala_dynamic_property_get_dynamic_type (prop))));
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	vala_ccode_function_add_parameter (func, _tmp7_ = vala_ccode_formal_parameter_new ("value", _tmp6_ = vala_data_type_get_cname (vala_property_get_property_type ((ValaProperty*) prop))));
	_vala_ccode_node_unref0 (_tmp7_);
	_g_free0 (_tmp6_);
	block = vala_ccode_block_new ();
	vala_dbus_client_module_generate_dbus_property_setter_wrapper (self, prop, block);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp8_ = vala_ccode_function_copy (func)));
	_vala_ccode_node_unref0 (_tmp8_);
	vala_ccode_function_set_block (func, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) func);
	result = setter_cname;
	_vala_ccode_node_unref0 (func);
	_vala_ccode_node_unref0 (block);
	return result;
}


static void vala_dbus_client_module_create_dbus_property_proxy (ValaDBusClientModule* self, ValaDynamicProperty* node, ValaCCodeBlock* block) {
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* prop_proxy_call;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeConstant* _tmp3_;
	ValaCCodeConstant* _tmp4_;
	ValaCCodeDeclaration* prop_proxy_decl;
	ValaCCodeVariableDeclarator* _tmp5_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	g_return_if_fail (block != NULL);
	prop_proxy_call = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_g_proxy_new_from_proxy"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (prop_proxy_call, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("obj")));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_function_call_add_argument (prop_proxy_call, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_constant_new ("DBUS_INTERFACE_PROPERTIES")));
	_vala_ccode_node_unref0 (_tmp3_);
	vala_ccode_function_call_add_argument (prop_proxy_call, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp4_);
	prop_proxy_decl = vala_ccode_declaration_new ("DBusGProxy*");
	vala_ccode_declaration_add_declarator (prop_proxy_decl, (ValaCCodeDeclarator*) (_tmp5_ = vala_ccode_variable_declarator_new ("property_proxy", (ValaCCodeExpression*) prop_proxy_call, NULL)));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prop_proxy_decl);
	_vala_ccode_node_unref0 (prop_proxy_call);
	_vala_ccode_node_unref0 (prop_proxy_decl);
}


static void vala_dbus_client_module_generate_dbus_property_getter_wrapper (ValaDBusClientModule* self, ValaDynamicProperty* node, ValaCCodeBlock* block) {
	ValaCCodeInitializerList* cvalinit;
	ValaCCodeConstant* _tmp0_;
	ValaCCodeDeclaration* cval_decl;
	ValaCCodeVariableDeclarator* _tmp1_;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeUnaryExpression* _tmp3_;
	ValaCCodeUnaryExpression* val_ptr;
	char* _tmp4_;
	ValaCCodeDeclaration* _tmp5_;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp6_;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeFunctionCall* _tmp8_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp9_;
	ValaCCodeConstant* _tmp10_;
	ValaCCodeConstant* _tmp11_;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeFunctionCall* _tmp14_;
	ValaCCodeFunctionCall* get_iface;
	ValaCCodeIdentifier* _tmp15_;
	ValaCCodeIdentifier* _tmp16_;
	ValaCCodeConstant* _tmp19_;
	char* _tmp18_;
	char* _tmp17_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeIdentifier* _tmp21_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeExpressionStatement* _tmp23_;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeFunctionCall* _tmp25_;
	ValaCCodeFunctionCall* prop_proxy_unref;
	ValaCCodeIdentifier* _tmp26_;
	ValaCCodeExpressionStatement* _tmp27_;
	ValaCCodeIdentifier* _tmp29_;
	char* _tmp28_;
	ValaCCodeFunctionCall* _tmp30_;
	ValaCCodeFunctionCall* cget_call;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeAssignment* _tmp32_;
	ValaCCodeAssignment* assign;
	ValaCCodeExpressionStatement* _tmp33_;
	ValaCCodeReturnStatement* _tmp35_;
	ValaCCodeIdentifier* _tmp34_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	g_return_if_fail (block != NULL);
	vala_dbus_client_module_create_dbus_property_proxy (self, node, block);
	cvalinit = vala_ccode_initializer_list_new ();
	vala_ccode_initializer_list_append (cvalinit, (ValaCCodeExpression*) (_tmp0_ = vala_ccode_constant_new ("0")));
	_vala_ccode_node_unref0 (_tmp0_);
	cval_decl = vala_ccode_declaration_new ("GValue");
	vala_ccode_declaration_add_declarator (cval_decl, (ValaCCodeDeclarator*) (_tmp1_ = vala_ccode_variable_declarator_new ("gvalue", (ValaCCodeExpression*) cvalinit, NULL)));
	_vala_ccode_node_unref0 (_tmp1_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) cval_decl);
	val_ptr = (_tmp3_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("gvalue"))), _vala_ccode_node_unref0 (_tmp2_), _tmp3_);
	_cdecl_ = (_tmp5_ = vala_ccode_declaration_new (_tmp4_ = vala_data_type_get_cname (vala_property_get_property_type ((ValaProperty*) node))), _g_free0 (_tmp4_), _tmp5_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp6_ = vala_ccode_variable_declarator_new ("result", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp6_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	ccall = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("dbus_g_proxy_call"))), _vala_ccode_node_unref0 (_tmp7_), _tmp8_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("property_proxy")));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_constant_new ("\"Get\"")));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("G_TYPE_STRING")));
	_vala_ccode_node_unref0 (_tmp12_);
	get_iface = (_tmp14_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("dbus_g_proxy_get_interface"))), _vala_ccode_node_unref0 (_tmp13_), _tmp14_);
	vala_ccode_function_call_add_argument (get_iface, (ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("obj")));
	_vala_ccode_node_unref0 (_tmp15_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) get_iface);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new ("G_TYPE_STRING")));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_constant_new (_tmp18_ = g_strdup_printf ("\"%s\"", _tmp17_ = vala_dbus_client_module_get_dynamic_dbus_name (self, vala_symbol_get_name ((ValaSymbol*) node))))));
	_vala_ccode_node_unref0 (_tmp19_);
	_g_free0 (_tmp18_);
	_g_free0 (_tmp17_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
	_vala_ccode_node_unref0 (_tmp20_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("G_TYPE_VALUE")));
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) val_ptr);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp23_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp23_);
	prop_proxy_unref = (_tmp25_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("g_object_unref"))), _vala_ccode_node_unref0 (_tmp24_), _tmp25_);
	vala_ccode_function_call_add_argument (prop_proxy_unref, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("property_proxy")));
	_vala_ccode_node_unref0 (_tmp26_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp27_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) prop_proxy_unref)));
	_vala_ccode_node_unref0 (_tmp27_);
	cget_call = (_tmp30_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new (_tmp28_ = vala_typesymbol_get_get_value_function (vala_data_type_get_data_type (vala_property_get_property_type ((ValaProperty*) node)))))), _vala_ccode_node_unref0 (_tmp29_), _g_free0 (_tmp28_), _tmp30_);
	vala_ccode_function_call_add_argument (cget_call, (ValaCCodeExpression*) val_ptr);
	assign = (_tmp32_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("result")), (ValaCCodeExpression*) cget_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), _vala_ccode_node_unref0 (_tmp31_), _tmp32_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp33_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) assign)));
	_vala_ccode_node_unref0 (_tmp33_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp35_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("result")))));
	_vala_ccode_node_unref0 (_tmp35_);
	_vala_ccode_node_unref0 (_tmp34_);
	_vala_ccode_node_unref0 (cvalinit);
	_vala_ccode_node_unref0 (cval_decl);
	_vala_ccode_node_unref0 (val_ptr);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (get_iface);
	_vala_ccode_node_unref0 (prop_proxy_unref);
	_vala_ccode_node_unref0 (cget_call);
	_vala_ccode_node_unref0 (assign);
}


static void vala_dbus_client_module_generate_dbus_property_setter_wrapper (ValaDBusClientModule* self, ValaDynamicProperty* node, ValaCCodeBlock* block) {
	ValaCCodeInitializerList* cvalinit;
	ValaCCodeConstant* _tmp0_;
	ValaCCodeDeclaration* cval_decl;
	ValaCCodeVariableDeclarator* _tmp1_;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeUnaryExpression* _tmp3_;
	ValaCCodeUnaryExpression* val_ptr;
	ValaCCodeIdentifier* _tmp4_;
	ValaCCodeFunctionCall* _tmp5_;
	ValaCCodeFunctionCall* cinit_call;
	ValaCCodeIdentifier* _tmp7_;
	char* _tmp6_;
	ValaCCodeExpressionStatement* _tmp8_;
	ValaCCodeIdentifier* _tmp10_;
	char* _tmp9_;
	ValaCCodeFunctionCall* _tmp11_;
	ValaCCodeFunctionCall* cset_call;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeExpressionStatement* _tmp13_;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeFunctionCall* _tmp15_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp16_;
	ValaCCodeConstant* _tmp17_;
	ValaCCodeConstant* _tmp18_;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeFunctionCall* _tmp21_;
	ValaCCodeFunctionCall* get_iface;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeIdentifier* _tmp23_;
	ValaCCodeConstant* _tmp26_;
	char* _tmp25_;
	char* _tmp24_;
	ValaCCodeIdentifier* _tmp27_;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeIdentifier* _tmp29_;
	ValaCCodeExpressionStatement* _tmp30_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeFunctionCall* _tmp32_;
	ValaCCodeFunctionCall* prop_proxy_unref;
	ValaCCodeIdentifier* _tmp33_;
	ValaCCodeExpressionStatement* _tmp34_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	g_return_if_fail (block != NULL);
	vala_dbus_client_module_create_dbus_property_proxy (self, node, block);
	cvalinit = vala_ccode_initializer_list_new ();
	vala_ccode_initializer_list_append (cvalinit, (ValaCCodeExpression*) (_tmp0_ = vala_ccode_constant_new ("0")));
	_vala_ccode_node_unref0 (_tmp0_);
	cval_decl = vala_ccode_declaration_new ("GValue");
	vala_ccode_declaration_add_declarator (cval_decl, (ValaCCodeDeclarator*) (_tmp1_ = vala_ccode_variable_declarator_new ("gvalue", (ValaCCodeExpression*) cvalinit, NULL)));
	_vala_ccode_node_unref0 (_tmp1_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) cval_decl);
	val_ptr = (_tmp3_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("gvalue"))), _vala_ccode_node_unref0 (_tmp2_), _tmp3_);
	cinit_call = (_tmp5_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("g_value_init"))), _vala_ccode_node_unref0 (_tmp4_), _tmp5_);
	vala_ccode_function_call_add_argument (cinit_call, (ValaCCodeExpression*) val_ptr);
	vala_ccode_function_call_add_argument (cinit_call, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = vala_typesymbol_get_type_id (vala_data_type_get_data_type (vala_property_get_property_type ((ValaProperty*) node))))));
	_vala_ccode_node_unref0 (_tmp7_);
	_g_free0 (_tmp6_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp8_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cinit_call)));
	_vala_ccode_node_unref0 (_tmp8_);
	cset_call = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new (_tmp9_ = vala_typesymbol_get_set_value_function (vala_data_type_get_data_type (vala_property_get_property_type ((ValaProperty*) node)))))), _vala_ccode_node_unref0 (_tmp10_), _g_free0 (_tmp9_), _tmp11_);
	vala_ccode_function_call_add_argument (cset_call, (ValaCCodeExpression*) val_ptr);
	vala_ccode_function_call_add_argument (cset_call, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("value")));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp13_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cset_call)));
	_vala_ccode_node_unref0 (_tmp13_);
	ccall = (_tmp15_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("dbus_g_proxy_call"))), _vala_ccode_node_unref0 (_tmp14_), _tmp15_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new ("property_proxy")));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_constant_new ("\"Set\"")));
	_vala_ccode_node_unref0 (_tmp17_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp18_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp18_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("G_TYPE_STRING")));
	_vala_ccode_node_unref0 (_tmp19_);
	get_iface = (_tmp21_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("dbus_g_proxy_get_interface"))), _vala_ccode_node_unref0 (_tmp20_), _tmp21_);
	vala_ccode_function_call_add_argument (get_iface, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("obj")));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) get_iface);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new ("G_TYPE_STRING")));
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_constant_new (_tmp25_ = g_strdup_printf ("\"%s\"", _tmp24_ = vala_dbus_client_module_get_dynamic_dbus_name (self, vala_symbol_get_name ((ValaSymbol*) node))))));
	_vala_ccode_node_unref0 (_tmp26_);
	_g_free0 (_tmp25_);
	_g_free0 (_tmp24_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("G_TYPE_VALUE")));
	_vala_ccode_node_unref0 (_tmp27_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) val_ptr);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
	_vala_ccode_node_unref0 (_tmp28_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
	_vala_ccode_node_unref0 (_tmp29_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp30_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp30_);
	prop_proxy_unref = (_tmp32_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("g_object_unref"))), _vala_ccode_node_unref0 (_tmp31_), _tmp32_);
	vala_ccode_function_call_add_argument (prop_proxy_unref, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("property_proxy")));
	_vala_ccode_node_unref0 (_tmp33_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp34_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) prop_proxy_unref)));
	_vala_ccode_node_unref0 (_tmp34_);
	_vala_ccode_node_unref0 (cvalinit);
	_vala_ccode_node_unref0 (cval_decl);
	_vala_ccode_node_unref0 (val_ptr);
	_vala_ccode_node_unref0 (cinit_call);
	_vala_ccode_node_unref0 (cset_call);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (get_iface);
	_vala_ccode_node_unref0 (prop_proxy_unref);
}


static char* vala_dbus_client_module_real_get_dynamic_signal_connect_wrapper_name (ValaCCodeModule* base, ValaDynamicSignal* sig) {
	ValaDBusClientModule * self;
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* connect_wrapper_name;
	ValaCCodeFunction* func;
	ValaCCodeFormalParameter* _tmp2_;
	ValaCCodeFormalParameter* _tmp3_;
	ValaCCodeFormalParameter* _tmp4_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeBlock* block;
	ValaCCodeFunction* _tmp6_;
	self = (ValaDBusClientModule*) base;
	g_return_val_if_fail (sig != NULL, NULL);
	if (vala_data_type_get_data_type (vala_dynamic_signal_get_dynamic_type (sig)) != ((ValaCCodeBaseModule*) self)->dbus_object_type) {
		result = VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->get_dynamic_signal_connect_wrapper_name ((ValaCCodeModule*) VALA_DBUS_MODULE (self), sig);
		return result;
	}
	connect_wrapper_name = (_tmp1_ = g_strdup_printf ("_%sconnect", _tmp0_ = vala_ccode_module_get_dynamic_signal_cname ((ValaCCodeModule*) self, sig)), _g_free0 (_tmp0_), _tmp1_);
	func = vala_ccode_function_new (connect_wrapper_name, "void");
	vala_ccode_function_add_parameter (func, _tmp2_ = vala_ccode_formal_parameter_new ("obj", "gpointer"));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_function_add_parameter (func, _tmp3_ = vala_ccode_formal_parameter_new ("signal_name", "const char *"));
	_vala_ccode_node_unref0 (_tmp3_);
	vala_ccode_function_add_parameter (func, _tmp4_ = vala_ccode_formal_parameter_new ("handler", "GCallback"));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_add_parameter (func, _tmp5_ = vala_ccode_formal_parameter_new ("data", "gpointer"));
	_vala_ccode_node_unref0 (_tmp5_);
	block = vala_ccode_block_new ();
	vala_dbus_client_module_generate_dbus_connect_wrapper (self, sig, block);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp6_ = vala_ccode_function_copy (func)));
	_vala_ccode_node_unref0 (_tmp6_);
	vala_ccode_function_set_block (func, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) func);
	result = connect_wrapper_name;
	_vala_ccode_node_unref0 (func);
	_vala_ccode_node_unref0 (block);
	return result;
}


static char* vala_dbus_client_module_real_get_dynamic_signal_disconnect_wrapper_name (ValaCCodeModule* base, ValaDynamicSignal* sig) {
	ValaDBusClientModule * self;
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* disconnect_wrapper_name;
	ValaCCodeFunction* func;
	ValaCCodeFormalParameter* _tmp2_;
	ValaCCodeFormalParameter* _tmp3_;
	ValaCCodeFormalParameter* _tmp4_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeBlock* block;
	ValaCCodeFunction* _tmp6_;
	self = (ValaDBusClientModule*) base;
	g_return_val_if_fail (sig != NULL, NULL);
	if (vala_data_type_get_data_type (vala_dynamic_signal_get_dynamic_type (sig)) != ((ValaCCodeBaseModule*) self)->dbus_object_type) {
		result = VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->get_dynamic_signal_disconnect_wrapper_name ((ValaCCodeModule*) VALA_DBUS_MODULE (self), sig);
		return result;
	}
	disconnect_wrapper_name = (_tmp1_ = g_strdup_printf ("_%sdisconnect", _tmp0_ = vala_ccode_module_get_dynamic_signal_cname ((ValaCCodeModule*) self, sig)), _g_free0 (_tmp0_), _tmp1_);
	func = vala_ccode_function_new (disconnect_wrapper_name, "void");
	vala_ccode_function_add_parameter (func, _tmp2_ = vala_ccode_formal_parameter_new ("obj", "gpointer"));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_function_add_parameter (func, _tmp3_ = vala_ccode_formal_parameter_new ("signal_name", "const char *"));
	_vala_ccode_node_unref0 (_tmp3_);
	vala_ccode_function_add_parameter (func, _tmp4_ = vala_ccode_formal_parameter_new ("handler", "GCallback"));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_add_parameter (func, _tmp5_ = vala_ccode_formal_parameter_new ("data", "gpointer"));
	_vala_ccode_node_unref0 (_tmp5_);
	block = vala_ccode_block_new ();
	vala_dbus_client_module_generate_dbus_disconnect_wrapper (self, sig, block);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp6_ = vala_ccode_function_copy (func)));
	_vala_ccode_node_unref0 (_tmp6_);
	vala_ccode_function_set_block (func, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) func);
	result = disconnect_wrapper_name;
	_vala_ccode_node_unref0 (func);
	_vala_ccode_node_unref0 (block);
	return result;
}


static void vala_dbus_client_module_generate_dbus_connect_wrapper (ValaDBusClientModule* self, ValaDynamicSignal* sig, ValaCCodeBlock* block) {
	ValaMethod* m;
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* register_call;
	GeeList* _tmp2_;
	ValaCCodeIdentifier* _tmp5_;
	char* _tmp4_;
	GeeList* _tmp3_;
	ValaCCodeIdentifier* _tmp6_;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeFunctionCall* _tmp8_;
	ValaCCodeFunctionCall* add_call;
	ValaCCodeIdentifier* _tmp9_;
	ValaCCodeConstant* _tmp12_;
	char* _tmp11_;
	char* _tmp10_;
	gboolean first;
	ValaCCodeIdentifier* _tmp17_;
	ValaCCodeIdentifier* _tmp18_;
	ValaCCodeExpressionStatement* _tmp19_;
	ValaCCodeExpressionStatement* _tmp20_;
	ValaCCodeIdentifier* _tmp21_;
	ValaCCodeFunctionCall* _tmp22_;
	ValaCCodeFunctionCall* call;
	ValaCCodeIdentifier* _tmp23_;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeIdentifier* _tmp25_;
	ValaCCodeIdentifier* _tmp26_;
	ValaCCodeConstant* _tmp27_;
	ValaCCodeExpressionStatement* _tmp28_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sig != NULL);
	g_return_if_fail (block != NULL);
	m = _vala_code_node_ref0 (VALA_METHOD (vala_expression_get_symbol_reference (vala_dynamic_signal_get_handler (sig))));
	vala_code_node_accept ((ValaCodeNode*) sig, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	register_call = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_g_object_register_marshaller"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_module_generate_marshaller (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp2_ = vala_signal_get_parameters ((ValaSignal*) sig), vala_signal_get_return_type ((ValaSignal*) sig), TRUE);
	_gee_collection_object_unref0 (_tmp2_);
	vala_ccode_function_call_add_argument (register_call, (ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new (_tmp4_ = vala_ccode_module_get_marshaller_function (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp3_ = vala_signal_get_parameters ((ValaSignal*) sig), vala_signal_get_return_type ((ValaSignal*) sig), NULL, TRUE))));
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	_gee_collection_object_unref0 (_tmp3_);
	vala_ccode_function_call_add_argument (register_call, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("G_TYPE_NONE")));
	_vala_ccode_node_unref0 (_tmp6_);
	add_call = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("dbus_g_proxy_add_signal"))), _vala_ccode_node_unref0 (_tmp7_), _tmp8_);
	vala_ccode_function_call_add_argument (add_call, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("obj")));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_function_call_add_argument (add_call, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_constant_new (_tmp11_ = g_strdup_printf ("\"%s\"", _tmp10_ = vala_dbus_client_module_get_dynamic_dbus_name (self, vala_symbol_get_name ((ValaSymbol*) sig))))));
	_vala_ccode_node_unref0 (_tmp12_);
	_g_free0 (_tmp11_);
	_g_free0 (_tmp10_);
	first = TRUE;
	{
		GeeList* _tmp13_;
		GeeIterator* _tmp14_;
		GeeIterator* _param_it;
		_param_it = (_tmp14_ = gee_iterable_iterator ((GeeIterable*) (_tmp13_ = vala_method_get_parameters (m))), _gee_collection_object_unref0 (_tmp13_), _tmp14_);
		while (TRUE) {
			ValaFormalParameter* param;
			ValaCCodeExpression* _tmp15_;
			ValaCCodeExpression* _tmp16_;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			if (first) {
				first = FALSE;
				_vala_code_node_unref0 (param);
				continue;
			}
			vala_ccode_function_call_add_argument (register_call, _tmp15_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_formal_parameter_get_parameter_type (param)));
			_vala_ccode_node_unref0 (_tmp15_);
			vala_ccode_function_call_add_argument (add_call, _tmp16_ = vala_gsignal_module_get_dbus_g_type ((ValaGSignalModule*) self, vala_formal_parameter_get_parameter_type (param)));
			_vala_ccode_node_unref0 (_tmp16_);
			_vala_code_node_unref0 (param);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	vala_ccode_function_call_add_argument (register_call, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
	_vala_ccode_node_unref0 (_tmp17_);
	vala_ccode_function_call_add_argument (add_call, (ValaCCodeExpression*) (_tmp18_ = vala_ccode_identifier_new ("G_TYPE_INVALID")));
	_vala_ccode_node_unref0 (_tmp18_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp19_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) register_call)));
	_vala_ccode_node_unref0 (_tmp19_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp20_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) add_call)));
	_vala_ccode_node_unref0 (_tmp20_);
	call = (_tmp22_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("dbus_g_proxy_connect_signal"))), _vala_ccode_node_unref0 (_tmp21_), _tmp22_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new ("obj")));
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("signal_name")));
	_vala_ccode_node_unref0 (_tmp24_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_identifier_new ("handler")));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("data")));
	_vala_ccode_node_unref0 (_tmp26_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp27_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp27_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp28_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) call)));
	_vala_ccode_node_unref0 (_tmp28_);
	_vala_code_node_unref0 (m);
	_vala_ccode_node_unref0 (register_call);
	_vala_ccode_node_unref0 (add_call);
	_vala_ccode_node_unref0 (call);
}


static void vala_dbus_client_module_generate_dbus_disconnect_wrapper (ValaDBusClientModule* self, ValaDynamicSignal* sig, ValaCCodeBlock* block) {
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* call;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeIdentifier* _tmp3_;
	ValaCCodeIdentifier* _tmp4_;
	ValaCCodeIdentifier* _tmp5_;
	ValaCCodeExpressionStatement* _tmp6_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sig != NULL);
	g_return_if_fail (block != NULL);
	call = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_g_proxy_disconnect_signal"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("obj")));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("signal_name")));
	_vala_ccode_node_unref0 (_tmp3_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("handler")));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_call_add_argument (call, (ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new ("data")));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp6_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) call)));
	_vala_ccode_node_unref0 (_tmp6_);
	_vala_ccode_node_unref0 (call);
}


static void vala_dbus_client_module_real_visit_cast_expression (ValaCCodeModule* base, ValaCastExpression* expr) {
	ValaDBusClientModule * self;
	ValaDataType* _tmp0_;
	ValaObjectType* type;
	ValaExpression* _tmp1_;
	ValaMethodCall* call;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	ValaDataType* _tmp6_;
	ValaMethodType* mtype;
	gboolean _tmp7_;
	GeeList* args;
	ValaExpression* connection;
	ValaExpression* bus_name;
	ValaExpression* object_path;
	ValaCCodeIdentifier* _tmp11_;
	char* _tmp10_;
	char* _tmp9_;
	ValaCCodeFunctionCall* _tmp12_;
	ValaCCodeFunctionCall* ccall;
	self = (ValaDBusClientModule*) base;
	g_return_if_fail (expr != NULL);
	type = _vala_code_node_ref0 ((_tmp0_ = vala_cast_expression_get_type_reference (expr), VALA_IS_OBJECT_TYPE (_tmp0_) ? ((ValaObjectType*) _tmp0_) : NULL));
	call = _vala_code_node_ref0 ((_tmp1_ = vala_cast_expression_get_inner (expr), VALA_IS_METHOD_CALL (_tmp1_) ? ((ValaMethodCall*) _tmp1_) : NULL));
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	if (type == NULL) {
		_tmp4_ = TRUE;
	} else {
		_tmp4_ = !VALA_IS_INTERFACE (vala_object_type_get_type_symbol (type));
	}
	if (_tmp4_) {
		_tmp3_ = TRUE;
	} else {
		ValaAttribute* _tmp5_;
		_tmp3_ = (_tmp5_ = vala_code_node_get_attribute ((ValaCodeNode*) vala_object_type_get_type_symbol (type), "DBus")) == NULL;
		_vala_code_node_unref0 (_tmp5_);
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = call == NULL;
	}
	if (_tmp2_) {
		VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->visit_cast_expression ((ValaCCodeModule*) VALA_DBUS_MODULE (self), expr);
		_vala_code_node_unref0 (type);
		_vala_code_node_unref0 (call);
		return;
	}
	mtype = _vala_code_node_ref0 ((_tmp6_ = vala_expression_get_value_type (vala_method_call_get_call (call)), VALA_IS_METHOD_TYPE (_tmp6_) ? ((ValaMethodType*) _tmp6_) : NULL));
	_tmp7_ = FALSE;
	if (mtype == NULL) {
		_tmp7_ = TRUE;
	} else {
		char* _tmp8_;
		_tmp7_ = _vala_strcmp0 (_tmp8_ = vala_method_get_cname (vala_method_type_get_method_symbol (mtype)), "dbus_g_proxy_new_for_name") != 0;
		_g_free0 (_tmp8_);
	}
	if (_tmp7_) {
		VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->visit_cast_expression ((ValaCCodeModule*) VALA_DBUS_MODULE (self), expr);
		_vala_code_node_unref0 (type);
		_vala_code_node_unref0 (call);
		_vala_code_node_unref0 (mtype);
		return;
	}
	args = vala_method_call_get_argument_list (call);
	connection = _vala_code_node_ref0 (vala_member_access_get_inner (VALA_MEMBER_ACCESS (vala_method_call_get_call (call))));
	bus_name = (ValaExpression*) gee_list_get (args, 0);
	object_path = (ValaExpression*) gee_list_get (args, 1);
	ccall = (_tmp12_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new (_tmp10_ = g_strconcat (_tmp9_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) vala_object_type_get_type_symbol (type)), "dbus_proxy_new", NULL)))), _vala_ccode_node_unref0 (_tmp11_), _g_free0 (_tmp10_), _g_free0 (_tmp9_), _tmp12_);
	vala_code_node_accept ((ValaCodeNode*) connection, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) connection)));
	vala_code_node_accept ((ValaCodeNode*) bus_name, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) bus_name)));
	vala_code_node_accept ((ValaCodeNode*) object_path, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) object_path)));
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall);
	_vala_code_node_unref0 (type);
	_vala_code_node_unref0 (call);
	_vala_code_node_unref0 (mtype);
	_gee_collection_object_unref0 (args);
	_vala_code_node_unref0 (connection);
	_vala_code_node_unref0 (bus_name);
	_vala_code_node_unref0 (object_path);
	_vala_ccode_node_unref0 (ccall);
}


static void vala_dbus_client_module_real_visit_interface (ValaCCodeModule* base, ValaInterface* iface) {
	ValaDBusClientModule * self;
	char* dbus_iface_name;
	char* _tmp0_;
	char* _tmp1_;
	char* cname;
	char* _tmp2_;
	char* _tmp3_;
	char* lower_cname;
	ValaCCodeTypeDefinition* _tmp6_;
	ValaCCodeVariableDeclarator* _tmp5_;
	char* _tmp4_;
	ValaCCodeTypeDefinition* _tmp9_;
	ValaCCodeVariableDeclarator* _tmp8_;
	char* _tmp7_;
	char* _tmp10_;
	ValaCCodeStruct* _tmp11_;
	ValaCCodeStruct* instance_struct;
	ValaCCodeFunction* _tmp13_;
	char* _tmp12_;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeFunctionCall* _tmp15_;
	ValaCCodeFunctionCall* implement;
	ValaCCodeIdentifier* _tmp17_;
	char* _tmp16_;
	ValaCCodeIdentifier* _tmp19_;
	char* _tmp18_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeFunctionCall* _tmp21_;
	ValaCCodeFunctionCall* define_type;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeIdentifier* _tmp23_;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeConstant* _tmp25_;
	ValaCCodeExpressionStatement* _tmp26_;
	char* _tmp29_;
	char* _tmp28_;
	char* _tmp27_;
	ValaCCodeFunction* _tmp30_;
	ValaCCodeFunction* proxy_new;
	ValaCCodeFormalParameter* _tmp31_;
	ValaCCodeFormalParameter* _tmp32_;
	ValaCCodeFormalParameter* _tmp33_;
	ValaCCodeBlock* new_block;
	ValaCCodeIdentifier* _tmp34_;
	ValaCCodeFunctionCall* _tmp35_;
	ValaCCodeFunctionCall* new_call;
	ValaCCodeFunctionCall* _tmp38_;
	ValaCCodeIdentifier* _tmp37_;
	char* _tmp36_;
	ValaCCodeConstant* _tmp39_;
	ValaCCodeIdentifier* _tmp40_;
	ValaCCodeConstant* _tmp41_;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeConstant* _tmp43_;
	ValaCCodeIdentifier* _tmp44_;
	ValaCCodeConstant* _tmp45_;
	ValaCCodeConstant* _tmp47_;
	char* _tmp46_;
	ValaCCodeConstant* _tmp48_;
	char* _tmp50_;
	char* _tmp49_;
	ValaCCodeDeclaration* _tmp51_;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp52_;
	ValaCCodeReturnStatement* _tmp54_;
	ValaCCodeIdentifier* _tmp53_;
	ValaCCodeFunction* _tmp55_;
	char* _tmp56_;
	ValaCCodeFunction* _tmp57_;
	ValaCCodeFunction* proxy_construct;
	ValaCCodeFormalParameter* _tmp58_;
	ValaCCodeFormalParameter* _tmp59_;
	ValaCCodeFormalParameter* _tmp60_;
	ValaCCodeBlock* _tmp61_;
	ValaCCodeIdentifier* _tmp62_;
	ValaCCodeFunctionCall* _tmp63_;
	ValaCCodeFunctionCall* parent_class;
	ValaCCodeIdentifier* _tmp65_;
	char* _tmp64_;
	ValaCCodeMemberAccess* _tmp66_;
	ValaCCodeFunctionCall* _tmp67_;
	ValaCCodeFunctionCall* chainup;
	ValaCCodeIdentifier* _tmp68_;
	ValaCCodeIdentifier* _tmp69_;
	ValaCCodeIdentifier* _tmp70_;
	ValaCCodeDeclaration* _tmp71_;
	ValaCCodeVariableDeclarator* _tmp72_;
	ValaCCodeDeclaration* _tmp73_;
	ValaCCodeVariableDeclarator* _tmp74_;
	ValaCCodeIdentifier* _tmp75_;
	ValaCCodeFunctionCall* _tmp76_;
	ValaCCodeFunctionCall* gconnection;
	ValaCCodeIdentifier* _tmp77_;
	ValaCCodeConstant* _tmp78_;
	ValaCCodeUnaryExpression* _tmp80_;
	ValaCCodeIdentifier* _tmp79_;
	ValaCCodeConstant* _tmp81_;
	ValaCCodeExpressionStatement* _tmp82_;
	ValaCCodeDeclaration* _tmp83_;
	ValaCCodeVariableDeclarator* _tmp84_;
	ValaCCodeIdentifier* _tmp85_;
	ValaCCodeFunctionCall* _tmp86_;
	ValaCCodeFunctionCall* path;
	ValaCCodeIdentifier* _tmp87_;
	ValaCCodeConstant* _tmp88_;
	ValaCCodeUnaryExpression* _tmp90_;
	ValaCCodeIdentifier* _tmp89_;
	ValaCCodeConstant* _tmp91_;
	ValaCCodeExpressionStatement* _tmp92_;
	ValaCCodeIdentifier* _tmp93_;
	ValaCCodeFunctionCall* _tmp94_;
	ValaCCodeFunctionCall* raw_connection;
	ValaCCodeIdentifier* _tmp95_;
	ValaCCodeIdentifier* _tmp96_;
	ValaCCodeFunctionCall* _tmp97_;
	ValaCCodeFunctionCall* filter_call;
	ValaCCodeIdentifier* _tmp99_;
	char* _tmp98_;
	ValaCCodeIdentifier* _tmp100_;
	ValaCCodeConstant* _tmp101_;
	ValaCCodeExpressionStatement* _tmp102_;
	ValaCCodeIdentifier* _tmp103_;
	ValaCCodeFunctionCall* _tmp104_;
	ValaCCodeFunctionCall* filter_printf;
	ValaCCodeConstant* _tmp105_;
	ValaCCodeIdentifier* _tmp106_;
	ValaCCodeDeclaration* _tmp107_;
	ValaCCodeVariableDeclarator* _tmp108_;
	ValaCCodeIdentifier* _tmp109_;
	ValaCCodeFunctionCall* _tmp110_;
	ValaCCodeFunctionCall* match_call;
	ValaCCodeIdentifier* _tmp111_;
	ValaCCodeConstant* _tmp112_;
	ValaCCodeExpressionStatement* _tmp113_;
	ValaCCodeIdentifier* _tmp114_;
	ValaCCodeFunctionCall* _tmp115_;
	ValaCCodeFunctionCall* connection_free;
	ValaCCodeIdentifier* _tmp116_;
	ValaCCodeExpressionStatement* _tmp117_;
	ValaCCodeIdentifier* _tmp118_;
	ValaCCodeFunctionCall* _tmp119_;
	ValaCCodeFunctionCall* path_free;
	ValaCCodeIdentifier* _tmp120_;
	ValaCCodeExpressionStatement* _tmp121_;
	ValaCCodeIdentifier* _tmp122_;
	ValaCCodeFunctionCall* _tmp123_;
	ValaCCodeFunctionCall* filter_free;
	ValaCCodeIdentifier* _tmp124_;
	ValaCCodeExpressionStatement* _tmp125_;
	ValaCCodeReturnStatement* _tmp127_;
	ValaCCodeIdentifier* _tmp126_;
	char* _tmp128_;
	ValaCCodeFunction* _tmp129_;
	ValaCCodeFunction* proxy_dispose;
	ValaCCodeFormalParameter* _tmp130_;
	ValaCCodeBlock* _tmp131_;
	ValaCCodeDeclaration* _tmp132_;
	ValaCCodeVariableDeclarator* _tmp133_;
	ValaCCodeBlock* dispose_return_block;
	ValaCCodeReturnStatement* _tmp134_;
	ValaCCodeIfStatement* _tmp139_;
	ValaCCodeMemberAccess* _tmp138_;
	ValaCCodeCastExpression* _tmp137_;
	char* _tmp136_;
	ValaCCodeIdentifier* _tmp135_;
	ValaCCodeExpressionStatement* _tmp146_;
	ValaCCodeAssignment* _tmp145_;
	ValaCCodeConstant* _tmp144_;
	ValaCCodeMemberAccess* _tmp143_;
	ValaCCodeCastExpression* _tmp142_;
	char* _tmp141_;
	ValaCCodeIdentifier* _tmp140_;
	ValaCCodeFunctionCall* _tmp148_;
	ValaCCodeIdentifier* _tmp147_;
	ValaCCodeIdentifier* _tmp149_;
	ValaCCodeConstant* _tmp150_;
	ValaCCodeUnaryExpression* _tmp152_;
	ValaCCodeIdentifier* _tmp151_;
	ValaCCodeConstant* _tmp153_;
	ValaCCodeExpressionStatement* _tmp154_;
	ValaCCodeFunctionCall* _tmp156_;
	ValaCCodeIdentifier* _tmp155_;
	ValaCCodeIdentifier* _tmp158_;
	char* _tmp157_;
	ValaCCodeIdentifier* _tmp159_;
	ValaCCodeExpressionStatement* _tmp160_;
	ValaCCodeFunctionCall* _tmp162_;
	ValaCCodeIdentifier* _tmp161_;
	ValaCCodeIdentifier* _tmp164_;
	char* _tmp163_;
	ValaCCodeFunctionCall* _tmp166_;
	ValaCCodeMemberAccess* _tmp165_;
	ValaCCodeIdentifier* _tmp167_;
	ValaCCodeExpressionStatement* _tmp168_;
	char* _tmp169_;
	ValaCCodeFunction* _tmp170_;
	ValaCCodeFunction* proxy_class_init;
	ValaCCodeFormalParameter* _tmp172_;
	char* _tmp171_;
	ValaCCodeBlock* _tmp173_;
	ValaCCodeIdentifier* _tmp174_;
	ValaCCodeFunctionCall* _tmp175_;
	ValaCCodeFunctionCall* gobject_class;
	ValaCCodeIdentifier* _tmp176_;
	ValaCCodeExpressionStatement* _tmp181_;
	ValaCCodeAssignment* _tmp180_;
	ValaCCodeIdentifier* _tmp179_;
	char* _tmp178_;
	ValaCCodeMemberAccess* _tmp177_;
	ValaCCodeExpressionStatement* _tmp186_;
	ValaCCodeAssignment* _tmp185_;
	ValaCCodeIdentifier* _tmp184_;
	char* _tmp183_;
	ValaCCodeMemberAccess* _tmp182_;
	ValaCCodeExpressionStatement* _tmp191_;
	ValaCCodeAssignment* _tmp190_;
	ValaCCodeIdentifier* _tmp189_;
	char* _tmp188_;
	ValaCCodeMemberAccess* _tmp187_;
	ValaCCodeExpressionStatement* _tmp196_;
	ValaCCodeAssignment* _tmp195_;
	ValaCCodeIdentifier* _tmp194_;
	char* _tmp193_;
	ValaCCodeMemberAccess* _tmp192_;
	ValaCCodeEnum* _tmp197_;
	ValaCCodeEnumValue* _tmp200_;
	char* _tmp199_;
	char* _tmp198_;
	char* _tmp211_;
	ValaCCodeFunction* _tmp212_;
	ValaCCodeFunction* proxy_instance_init;
	ValaCCodeFormalParameter* _tmp214_;
	char* _tmp213_;
	ValaCCodeBlock* _tmp215_;
	char* _tmp216_;
	ValaCCodeFunction* _tmp217_;
	ValaCCodeFunction* proxy_iface_init;
	ValaCCodeFormalParameter* _tmp220_;
	char* _tmp219_;
	char* _tmp218_;
	ValaCCodeBlock* iface_block;
	ValaCCodeFunction* _tmp256_;
	char* _tmp257_;
	ValaCCodeFunction* _tmp258_;
	ValaCCodeFunction* get_prop;
	ValaCCodeFormalParameter* _tmp259_;
	ValaCCodeFormalParameter* _tmp260_;
	ValaCCodeFormalParameter* _tmp261_;
	ValaCCodeFormalParameter* _tmp262_;
	ValaCCodeFunction* _tmp263_;
	ValaCCodeBlock* _tmp264_;
	char* _tmp265_;
	ValaCCodeFunction* _tmp266_;
	ValaCCodeFunction* set_prop;
	ValaCCodeFormalParameter* _tmp267_;
	ValaCCodeFormalParameter* _tmp268_;
	ValaCCodeFormalParameter* _tmp269_;
	ValaCCodeFormalParameter* _tmp270_;
	ValaCCodeFunction* _tmp271_;
	ValaCCodeBlock* _tmp272_;
	self = (ValaDBusClientModule*) base;
	g_return_if_fail (iface != NULL);
	VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->visit_interface ((ValaCCodeModule*) VALA_DBUS_MODULE (self), iface);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	if (dbus_iface_name == NULL) {
		_g_free0 (dbus_iface_name);
		return;
	}
	cname = (_tmp1_ = g_strconcat (_tmp0_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), "DBusProxy", NULL), _g_free0 (_tmp0_), _tmp1_);
	lower_cname = (_tmp3_ = g_strconcat (_tmp2_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), "dbus_proxy", NULL), _g_free0 (_tmp2_), _tmp3_);
	if (vala_symbol_get_access ((ValaSymbol*) iface) != VALA_SYMBOL_ACCESSIBILITY_PRIVATE) {
		((ValaCCodeBaseModule*) self)->dbus_glib_h_needed_in_header = TRUE;
	} else {
		((ValaCCodeBaseModule*) self)->dbus_glib_h_needed = TRUE;
	}
	vala_ccode_declaration_space_add_type_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp6_ = vala_ccode_type_definition_new (_tmp4_ = g_strdup_printf ("struct _%s", cname), (ValaCCodeDeclarator*) (_tmp5_ = vala_ccode_variable_declarator_new (cname, NULL, NULL)))));
	_vala_ccode_node_unref0 (_tmp6_);
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	vala_ccode_declaration_space_add_type_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp9_ = vala_ccode_type_definition_new ("DBusGProxyClass", (ValaCCodeDeclarator*) (_tmp8_ = vala_ccode_variable_declarator_new (_tmp7_ = g_strconcat (cname, "Class", NULL), NULL, NULL)))));
	_vala_ccode_node_unref0 (_tmp9_);
	_vala_ccode_node_unref0 (_tmp8_);
	_g_free0 (_tmp7_);
	instance_struct = (_tmp11_ = vala_ccode_struct_new (_tmp10_ = g_strdup_printf ("_%s", cname)), _g_free0 (_tmp10_), _tmp11_);
	vala_ccode_struct_add_field (instance_struct, "DBusGProxy", "parent_instance");
	vala_ccode_struct_add_field (instance_struct, "gboolean", "disposed");
	vala_ccode_declaration_space_add_type_definition (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) instance_struct);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp13_ = vala_ccode_function_new (_tmp12_ = g_strconcat (lower_cname, "_get_type", NULL), "GType")));
	_vala_ccode_node_unref0 (_tmp13_);
	_g_free0 (_tmp12_);
	implement = (_tmp15_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("G_IMPLEMENT_INTERFACE"))), _vala_ccode_node_unref0 (_tmp14_), _tmp15_);
	vala_ccode_function_call_add_argument (implement, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new (_tmp16_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) iface, "TYPE_"))));
	_vala_ccode_node_unref0 (_tmp17_);
	_g_free0 (_tmp16_);
	vala_ccode_function_call_add_argument (implement, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new (_tmp18_ = g_strconcat (lower_cname, "_interface_init", NULL))));
	_vala_ccode_node_unref0 (_tmp19_);
	_g_free0 (_tmp18_);
	define_type = (_tmp21_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("G_DEFINE_TYPE_EXTENDED"))), _vala_ccode_node_unref0 (_tmp20_), _tmp21_);
	vala_ccode_function_call_add_argument (define_type, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new (cname)));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_function_call_add_argument (define_type, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new (lower_cname)));
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_function_call_add_argument (define_type, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("DBUS_TYPE_G_PROXY")));
	_vala_ccode_node_unref0 (_tmp24_);
	vala_ccode_function_call_add_argument (define_type, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_constant_new ("0")));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_function_call_add_argument (define_type, (ValaCCodeExpression*) implement);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) (_tmp26_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) define_type)));
	_vala_ccode_node_unref0 (_tmp26_);
	proxy_new = (_tmp30_ = vala_ccode_function_new (_tmp27_ = g_strconcat (lower_cname, "_new", NULL), _tmp29_ = g_strconcat (_tmp28_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), "*", NULL)), _g_free0 (_tmp29_), _g_free0 (_tmp28_), _g_free0 (_tmp27_), _tmp30_);
	vala_ccode_function_add_parameter (proxy_new, _tmp31_ = vala_ccode_formal_parameter_new ("connection", "DBusGConnection*"));
	_vala_ccode_node_unref0 (_tmp31_);
	vala_ccode_function_add_parameter (proxy_new, _tmp32_ = vala_ccode_formal_parameter_new ("name", "const char*"));
	_vala_ccode_node_unref0 (_tmp32_);
	vala_ccode_function_add_parameter (proxy_new, _tmp33_ = vala_ccode_formal_parameter_new ("path", "const char*"));
	_vala_ccode_node_unref0 (_tmp33_);
	new_block = vala_ccode_block_new ();
	new_call = (_tmp35_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("g_object_new"))), _vala_ccode_node_unref0 (_tmp34_), _tmp35_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp38_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp37_ = vala_ccode_identifier_new (_tmp36_ = g_strconcat (lower_cname, "_get_type", NULL))))));
	_vala_ccode_node_unref0 (_tmp38_);
	_vala_ccode_node_unref0 (_tmp37_);
	_g_free0 (_tmp36_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp39_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp39_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp40_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp40_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp41_ = vala_ccode_constant_new ("\"name\"")));
	_vala_ccode_node_unref0 (_tmp41_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("name")));
	_vala_ccode_node_unref0 (_tmp42_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp43_ = vala_ccode_constant_new ("\"path\"")));
	_vala_ccode_node_unref0 (_tmp43_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("path")));
	_vala_ccode_node_unref0 (_tmp44_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp45_ = vala_ccode_constant_new ("\"interface\"")));
	_vala_ccode_node_unref0 (_tmp45_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_constant_new (_tmp46_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp47_);
	_g_free0 (_tmp46_);
	vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp48_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp48_);
	_cdecl_ = (_tmp51_ = vala_ccode_declaration_new (_tmp50_ = g_strconcat (_tmp49_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), "*", NULL)), _g_free0 (_tmp50_), _g_free0 (_tmp49_), _tmp51_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp52_ = vala_ccode_variable_declarator_new ("self", (ValaCCodeExpression*) new_call, NULL)));
	_vala_ccode_node_unref0 (_tmp52_);
	vala_ccode_block_add_statement (new_block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_block_add_statement (new_block, (ValaCCodeNode*) (_tmp54_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp53_ = vala_ccode_identifier_new ("self")))));
	_vala_ccode_node_unref0 (_tmp54_);
	_vala_ccode_node_unref0 (_tmp53_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp55_ = vala_ccode_function_copy (proxy_new)));
	_vala_ccode_node_unref0 (_tmp55_);
	vala_ccode_function_set_block (proxy_new, new_block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) proxy_new);
	proxy_construct = (_tmp57_ = vala_ccode_function_new (_tmp56_ = g_strconcat (lower_cname, "_construct", NULL), "GObject*"), _g_free0 (_tmp56_), _tmp57_);
	vala_ccode_function_add_parameter (proxy_construct, _tmp58_ = vala_ccode_formal_parameter_new ("gtype", "GType"));
	_vala_ccode_node_unref0 (_tmp58_);
	vala_ccode_function_add_parameter (proxy_construct, _tmp59_ = vala_ccode_formal_parameter_new ("n_properties", "guint"));
	_vala_ccode_node_unref0 (_tmp59_);
	vala_ccode_function_add_parameter (proxy_construct, _tmp60_ = vala_ccode_formal_parameter_new ("properties", "GObjectConstructParam*"));
	_vala_ccode_node_unref0 (_tmp60_);
	vala_ccode_function_set_modifiers (proxy_construct, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_set_block (proxy_construct, _tmp61_ = vala_ccode_block_new ());
	_vala_ccode_node_unref0 (_tmp61_);
	parent_class = (_tmp63_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new ("G_OBJECT_CLASS"))), _vala_ccode_node_unref0 (_tmp62_), _tmp63_);
	vala_ccode_function_call_add_argument (parent_class, (ValaCCodeExpression*) (_tmp65_ = vala_ccode_identifier_new (_tmp64_ = g_strconcat (lower_cname, "_parent_class", NULL))));
	_vala_ccode_node_unref0 (_tmp65_);
	_g_free0 (_tmp64_);
	chainup = (_tmp67_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp66_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) parent_class, "constructor"))), _vala_ccode_node_unref0 (_tmp66_), _tmp67_);
	vala_ccode_function_call_add_argument (chainup, (ValaCCodeExpression*) (_tmp68_ = vala_ccode_identifier_new ("gtype")));
	_vala_ccode_node_unref0 (_tmp68_);
	vala_ccode_function_call_add_argument (chainup, (ValaCCodeExpression*) (_tmp69_ = vala_ccode_identifier_new ("n_properties")));
	_vala_ccode_node_unref0 (_tmp69_);
	vala_ccode_function_call_add_argument (chainup, (ValaCCodeExpression*) (_tmp70_ = vala_ccode_identifier_new ("properties")));
	_vala_ccode_node_unref0 (_tmp70_);
	_cdecl_ = (_tmp71_ = vala_ccode_declaration_new ("GObject*"), _vala_ccode_node_unref0 (_cdecl_), _tmp71_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp72_ = vala_ccode_variable_declarator_new ("self", (ValaCCodeExpression*) chainup, NULL)));
	_vala_ccode_node_unref0 (_tmp72_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp73_ = vala_ccode_declaration_new ("DBusGConnection"), _vala_ccode_node_unref0 (_cdecl_), _tmp73_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp74_ = vala_ccode_variable_declarator_new ("*connection", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp74_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) _cdecl_);
	gconnection = (_tmp76_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp75_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (_tmp75_), _tmp76_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp77_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp77_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp78_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp78_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp80_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp79_ = vala_ccode_identifier_new ("connection")))));
	_vala_ccode_node_unref0 (_tmp80_);
	_vala_ccode_node_unref0 (_tmp79_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp81_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp81_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp82_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) gconnection)));
	_vala_ccode_node_unref0 (_tmp82_);
	_cdecl_ = (_tmp83_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp83_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp84_ = vala_ccode_variable_declarator_new ("path", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp84_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) _cdecl_);
	path = (_tmp86_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp85_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (_tmp85_), _tmp86_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp87_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp87_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp88_ = vala_ccode_constant_new ("\"path\"")));
	_vala_ccode_node_unref0 (_tmp88_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp90_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp89_ = vala_ccode_identifier_new ("path")))));
	_vala_ccode_node_unref0 (_tmp90_);
	_vala_ccode_node_unref0 (_tmp89_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp91_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp91_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp92_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) path)));
	_vala_ccode_node_unref0 (_tmp92_);
	raw_connection = (_tmp94_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp93_ = vala_ccode_identifier_new ("dbus_g_connection_get_connection"))), _vala_ccode_node_unref0 (_tmp93_), _tmp94_);
	vala_ccode_function_call_add_argument (raw_connection, (ValaCCodeExpression*) (_tmp95_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp95_);
	filter_call = (_tmp97_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp96_ = vala_ccode_identifier_new ("dbus_connection_add_filter"))), _vala_ccode_node_unref0 (_tmp96_), _tmp97_);
	vala_ccode_function_call_add_argument (filter_call, (ValaCCodeExpression*) raw_connection);
	vala_ccode_function_call_add_argument (filter_call, (ValaCCodeExpression*) (_tmp99_ = vala_ccode_identifier_new (_tmp98_ = g_strconcat (lower_cname, "_filter", NULL))));
	_vala_ccode_node_unref0 (_tmp99_);
	_g_free0 (_tmp98_);
	vala_ccode_function_call_add_argument (filter_call, (ValaCCodeExpression*) (_tmp100_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp100_);
	vala_ccode_function_call_add_argument (filter_call, (ValaCCodeExpression*) (_tmp101_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp101_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp102_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) filter_call)));
	_vala_ccode_node_unref0 (_tmp102_);
	filter_printf = (_tmp104_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp103_ = vala_ccode_identifier_new ("g_strdup_printf"))), _vala_ccode_node_unref0 (_tmp103_), _tmp104_);
	vala_ccode_function_call_add_argument (filter_printf, (ValaCCodeExpression*) (_tmp105_ = vala_ccode_constant_new ("\"type='signal',path='%s'\"")));
	_vala_ccode_node_unref0 (_tmp105_);
	vala_ccode_function_call_add_argument (filter_printf, (ValaCCodeExpression*) (_tmp106_ = vala_ccode_identifier_new ("path")));
	_vala_ccode_node_unref0 (_tmp106_);
	_cdecl_ = (_tmp107_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp107_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp108_ = vala_ccode_variable_declarator_new ("filter", (ValaCCodeExpression*) filter_printf, NULL)));
	_vala_ccode_node_unref0 (_tmp108_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) _cdecl_);
	match_call = (_tmp110_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp109_ = vala_ccode_identifier_new ("dbus_bus_add_match"))), _vala_ccode_node_unref0 (_tmp109_), _tmp110_);
	vala_ccode_function_call_add_argument (match_call, (ValaCCodeExpression*) raw_connection);
	vala_ccode_function_call_add_argument (match_call, (ValaCCodeExpression*) (_tmp111_ = vala_ccode_identifier_new ("filter")));
	_vala_ccode_node_unref0 (_tmp111_);
	vala_ccode_function_call_add_argument (match_call, (ValaCCodeExpression*) (_tmp112_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp112_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp113_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) match_call)));
	_vala_ccode_node_unref0 (_tmp113_);
	connection_free = (_tmp115_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp114_ = vala_ccode_identifier_new ("dbus_g_connection_unref"))), _vala_ccode_node_unref0 (_tmp114_), _tmp115_);
	vala_ccode_function_call_add_argument (connection_free, (ValaCCodeExpression*) (_tmp116_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp116_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp117_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) connection_free)));
	_vala_ccode_node_unref0 (_tmp117_);
	path_free = (_tmp119_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp118_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp118_), _tmp119_);
	vala_ccode_function_call_add_argument (path_free, (ValaCCodeExpression*) (_tmp120_ = vala_ccode_identifier_new ("path")));
	_vala_ccode_node_unref0 (_tmp120_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp121_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) path_free)));
	_vala_ccode_node_unref0 (_tmp121_);
	filter_free = (_tmp123_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp122_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp122_), _tmp123_);
	vala_ccode_function_call_add_argument (filter_free, (ValaCCodeExpression*) (_tmp124_ = vala_ccode_identifier_new ("filter")));
	_vala_ccode_node_unref0 (_tmp124_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp125_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) filter_free)));
	_vala_ccode_node_unref0 (_tmp125_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_construct), (ValaCCodeNode*) (_tmp127_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp126_ = vala_ccode_identifier_new ("self")))));
	_vala_ccode_node_unref0 (_tmp127_);
	_vala_ccode_node_unref0 (_tmp126_);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) proxy_construct);
	vala_dbus_client_module_generate_proxy_filter_function (self, iface);
	proxy_dispose = (_tmp129_ = vala_ccode_function_new (_tmp128_ = g_strconcat (lower_cname, "_dispose", NULL), "void"), _g_free0 (_tmp128_), _tmp129_);
	vala_ccode_function_add_parameter (proxy_dispose, _tmp130_ = vala_ccode_formal_parameter_new ("self", "GObject*"));
	_vala_ccode_node_unref0 (_tmp130_);
	vala_ccode_function_set_modifiers (proxy_dispose, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_set_block (proxy_dispose, _tmp131_ = vala_ccode_block_new ());
	_vala_ccode_node_unref0 (_tmp131_);
	_cdecl_ = (_tmp132_ = vala_ccode_declaration_new ("DBusGConnection"), _vala_ccode_node_unref0 (_cdecl_), _tmp132_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp133_ = vala_ccode_variable_declarator_new ("*connection", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp133_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_dispose), (ValaCCodeNode*) _cdecl_);
	dispose_return_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (dispose_return_block, (ValaCCodeNode*) (_tmp134_ = vala_ccode_return_statement_new (NULL)));
	_vala_ccode_node_unref0 (_tmp134_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_dispose), (ValaCCodeNode*) (_tmp139_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp138_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp137_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp135_ = vala_ccode_identifier_new ("self")), _tmp136_ = g_strconcat (cname, "*", NULL))), "disposed")), (ValaCCodeStatement*) dispose_return_block, NULL)));
	_vala_ccode_node_unref0 (_tmp139_);
	_vala_ccode_node_unref0 (_tmp138_);
	_vala_ccode_node_unref0 (_tmp137_);
	_g_free0 (_tmp136_);
	_vala_ccode_node_unref0 (_tmp135_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_dispose), (ValaCCodeNode*) (_tmp146_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp145_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp143_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp142_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp140_ = vala_ccode_identifier_new ("self")), _tmp141_ = g_strconcat (cname, "*", NULL))), "disposed")), (ValaCCodeExpression*) (_tmp144_ = vala_ccode_constant_new ("TRUE")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp146_);
	_vala_ccode_node_unref0 (_tmp145_);
	_vala_ccode_node_unref0 (_tmp144_);
	_vala_ccode_node_unref0 (_tmp143_);
	_vala_ccode_node_unref0 (_tmp142_);
	_g_free0 (_tmp141_);
	_vala_ccode_node_unref0 (_tmp140_);
	gconnection = (_tmp148_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp147_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (gconnection), _tmp148_);
	_vala_ccode_node_unref0 (_tmp147_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp149_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp149_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp150_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp150_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp152_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp151_ = vala_ccode_identifier_new ("connection")))));
	_vala_ccode_node_unref0 (_tmp152_);
	_vala_ccode_node_unref0 (_tmp151_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp153_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp153_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_dispose), (ValaCCodeNode*) (_tmp154_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) gconnection)));
	_vala_ccode_node_unref0 (_tmp154_);
	filter_call = (_tmp156_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp155_ = vala_ccode_identifier_new ("dbus_connection_remove_filter"))), _vala_ccode_node_unref0 (filter_call), _tmp156_);
	_vala_ccode_node_unref0 (_tmp155_);
	vala_ccode_function_call_add_argument (filter_call, (ValaCCodeExpression*) raw_connection);
	vala_ccode_function_call_add_argument (filter_call, (ValaCCodeExpression*) (_tmp158_ = vala_ccode_identifier_new (_tmp157_ = g_strconcat (lower_cname, "_filter", NULL))));
	_vala_ccode_node_unref0 (_tmp158_);
	_g_free0 (_tmp157_);
	vala_ccode_function_call_add_argument (filter_call, (ValaCCodeExpression*) (_tmp159_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp159_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_dispose), (ValaCCodeNode*) (_tmp160_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) filter_call)));
	_vala_ccode_node_unref0 (_tmp160_);
	parent_class = (_tmp162_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp161_ = vala_ccode_identifier_new ("G_OBJECT_CLASS"))), _vala_ccode_node_unref0 (parent_class), _tmp162_);
	_vala_ccode_node_unref0 (_tmp161_);
	vala_ccode_function_call_add_argument (parent_class, (ValaCCodeExpression*) (_tmp164_ = vala_ccode_identifier_new (_tmp163_ = g_strconcat (lower_cname, "_parent_class", NULL))));
	_vala_ccode_node_unref0 (_tmp164_);
	_g_free0 (_tmp163_);
	chainup = (_tmp166_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp165_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) parent_class, "dispose"))), _vala_ccode_node_unref0 (chainup), _tmp166_);
	_vala_ccode_node_unref0 (_tmp165_);
	vala_ccode_function_call_add_argument (chainup, (ValaCCodeExpression*) (_tmp167_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp167_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_dispose), (ValaCCodeNode*) (_tmp168_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) chainup)));
	_vala_ccode_node_unref0 (_tmp168_);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) proxy_dispose);
	proxy_class_init = (_tmp170_ = vala_ccode_function_new (_tmp169_ = g_strconcat (lower_cname, "_class_init", NULL), "void"), _g_free0 (_tmp169_), _tmp170_);
	vala_ccode_function_add_parameter (proxy_class_init, _tmp172_ = vala_ccode_formal_parameter_new ("klass", _tmp171_ = g_strconcat (cname, "Class*", NULL)));
	_vala_ccode_node_unref0 (_tmp172_);
	_g_free0 (_tmp171_);
	vala_ccode_function_set_modifiers (proxy_class_init, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_set_block (proxy_class_init, _tmp173_ = vala_ccode_block_new ());
	_vala_ccode_node_unref0 (_tmp173_);
	gobject_class = (_tmp175_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp174_ = vala_ccode_identifier_new ("G_OBJECT_CLASS"))), _vala_ccode_node_unref0 (_tmp174_), _tmp175_);
	vala_ccode_function_call_add_argument (gobject_class, (ValaCCodeExpression*) (_tmp176_ = vala_ccode_identifier_new ("klass")));
	_vala_ccode_node_unref0 (_tmp176_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_class_init), (ValaCCodeNode*) (_tmp181_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp180_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp177_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) gobject_class, "constructor")), (ValaCCodeExpression*) (_tmp179_ = vala_ccode_identifier_new (_tmp178_ = g_strconcat (lower_cname, "_construct", NULL))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp181_);
	_vala_ccode_node_unref0 (_tmp180_);
	_vala_ccode_node_unref0 (_tmp179_);
	_g_free0 (_tmp178_);
	_vala_ccode_node_unref0 (_tmp177_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_class_init), (ValaCCodeNode*) (_tmp186_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp185_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp182_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) gobject_class, "dispose")), (ValaCCodeExpression*) (_tmp184_ = vala_ccode_identifier_new (_tmp183_ = g_strconcat (lower_cname, "_dispose", NULL))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp186_);
	_vala_ccode_node_unref0 (_tmp185_);
	_vala_ccode_node_unref0 (_tmp184_);
	_g_free0 (_tmp183_);
	_vala_ccode_node_unref0 (_tmp182_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_class_init), (ValaCCodeNode*) (_tmp191_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp190_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp187_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) gobject_class, "get_property")), (ValaCCodeExpression*) (_tmp189_ = vala_ccode_identifier_new (_tmp188_ = g_strconcat (lower_cname, "_get_property", NULL))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp191_);
	_vala_ccode_node_unref0 (_tmp190_);
	_vala_ccode_node_unref0 (_tmp189_);
	_g_free0 (_tmp188_);
	_vala_ccode_node_unref0 (_tmp187_);
	vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_class_init), (ValaCCodeNode*) (_tmp196_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp195_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp192_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) gobject_class, "set_property")), (ValaCCodeExpression*) (_tmp194_ = vala_ccode_identifier_new (_tmp193_ = g_strconcat (lower_cname, "_set_property", NULL))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp196_);
	_vala_ccode_node_unref0 (_tmp195_);
	_vala_ccode_node_unref0 (_tmp194_);
	_g_free0 (_tmp193_);
	_vala_ccode_node_unref0 (_tmp192_);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) proxy_class_init);
	((ValaCCodeBaseModule*) self)->prop_enum = (_tmp197_ = vala_ccode_enum_new (NULL), _vala_ccode_node_unref0 (((ValaCCodeBaseModule*) self)->prop_enum), _tmp197_);
	vala_ccode_enum_add_value (((ValaCCodeBaseModule*) self)->prop_enum, _tmp200_ = vala_ccode_enum_value_new (_tmp199_ = g_strdup_printf ("%s_DUMMY_PROPERTY", _tmp198_ = g_utf8_strup (lower_cname, -1)), NULL));
	_vala_ccode_node_unref0 (_tmp200_);
	_g_free0 (_tmp199_);
	_g_free0 (_tmp198_);
	{
		GeeList* _tmp201_;
		GeeIterator* _tmp202_;
		GeeIterator* _prop_it;
		_prop_it = (_tmp202_ = gee_iterable_iterator ((GeeIterable*) (_tmp201_ = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) iface))), _gee_collection_object_unref0 (_tmp201_), _tmp202_);
		while (TRUE) {
			ValaProperty* prop;
			ValaCCodeIdentifier* _tmp203_;
			ValaCCodeFunctionCall* _tmp204_;
			ValaCCodeFunctionCall* cinst;
			ValaCCodeConstant* _tmp206_;
			char* _tmp205_;
			ValaCCodeConstant* _tmp207_;
			ValaCCodeExpressionStatement* _tmp208_;
			ValaCCodeEnumValue* _tmp210_;
			char* _tmp209_;
			if (!gee_iterator_next (_prop_it)) {
				break;
			}
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			if (!vala_property_get_is_abstract (prop)) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (VALA_IS_ARRAY_TYPE (vala_property_get_property_type (prop))) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			cinst = (_tmp204_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp203_ = vala_ccode_identifier_new ("g_object_class_override_property"))), _vala_ccode_node_unref0 (_tmp203_), _tmp204_);
			vala_ccode_function_call_add_argument (cinst, (ValaCCodeExpression*) gobject_class);
			vala_ccode_function_call_add_argument (cinst, (ValaCCodeExpression*) (_tmp206_ = vala_ccode_constant_new (_tmp205_ = vala_property_get_upper_case_cname (prop))));
			_vala_ccode_node_unref0 (_tmp206_);
			_g_free0 (_tmp205_);
			vala_ccode_function_call_add_argument (cinst, (ValaCCodeExpression*) (_tmp207_ = vala_property_get_canonical_cconstant (prop)));
			_vala_ccode_node_unref0 (_tmp207_);
			vala_ccode_block_add_statement (vala_ccode_function_get_block (proxy_class_init), (ValaCCodeNode*) (_tmp208_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cinst)));
			_vala_ccode_node_unref0 (_tmp208_);
			vala_ccode_enum_add_value (((ValaCCodeBaseModule*) self)->prop_enum, _tmp210_ = vala_ccode_enum_value_new (_tmp209_ = vala_property_get_upper_case_cname (prop), NULL));
			_vala_ccode_node_unref0 (_tmp210_);
			_g_free0 (_tmp209_);
			_vala_code_node_unref0 (prop);
			_vala_ccode_node_unref0 (cinst);
		}
		_gee_collection_object_unref0 (_prop_it);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) ((ValaCCodeBaseModule*) self)->prop_enum);
	proxy_instance_init = (_tmp212_ = vala_ccode_function_new (_tmp211_ = g_strconcat (lower_cname, "_init", NULL), "void"), _g_free0 (_tmp211_), _tmp212_);
	vala_ccode_function_add_parameter (proxy_instance_init, _tmp214_ = vala_ccode_formal_parameter_new ("self", _tmp213_ = g_strconcat (cname, "*", NULL)));
	_vala_ccode_node_unref0 (_tmp214_);
	_g_free0 (_tmp213_);
	vala_ccode_function_set_modifiers (proxy_instance_init, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_set_block (proxy_instance_init, _tmp215_ = vala_ccode_block_new ());
	_vala_ccode_node_unref0 (_tmp215_);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) proxy_instance_init);
	proxy_iface_init = (_tmp217_ = vala_ccode_function_new (_tmp216_ = g_strconcat (lower_cname, "_interface_init", NULL), "void"), _g_free0 (_tmp216_), _tmp217_);
	vala_ccode_function_add_parameter (proxy_iface_init, _tmp220_ = vala_ccode_formal_parameter_new ("iface", _tmp219_ = g_strconcat (_tmp218_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), "Iface*", NULL)));
	_vala_ccode_node_unref0 (_tmp220_);
	_g_free0 (_tmp219_);
	_g_free0 (_tmp218_);
	iface_block = vala_ccode_block_new ();
	{
		GeeList* _tmp221_;
		GeeIterator* _tmp222_;
		GeeIterator* _m_it;
		_m_it = (_tmp222_ = gee_iterable_iterator ((GeeIterable*) (_tmp221_ = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) iface))), _gee_collection_object_unref0 (_tmp221_), _tmp222_);
		while (TRUE) {
			ValaMethod* m;
			ValaCCodeIdentifier* _tmp223_;
			ValaCCodeMemberAccess* _tmp224_;
			ValaCCodeMemberAccess* vfunc_entry;
			if (!gee_iterator_next (_m_it)) {
				break;
			}
			m = (ValaMethod*) gee_iterator_get (_m_it);
			vfunc_entry = (_tmp224_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp223_ = vala_ccode_identifier_new ("iface")), vala_method_get_vfunc_name (m)), _vala_ccode_node_unref0 (_tmp223_), _tmp224_);
			if (!vala_method_get_coroutine (m)) {
				ValaCCodeExpressionStatement* _tmp228_;
				ValaCCodeAssignment* _tmp227_;
				ValaCCodeIdentifier* _tmp226_;
				char* _tmp225_;
				vala_ccode_block_add_statement (iface_block, (ValaCCodeNode*) (_tmp228_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp227_ = vala_ccode_assignment_new ((ValaCCodeExpression*) vfunc_entry, (ValaCCodeExpression*) (_tmp226_ = vala_ccode_identifier_new (_tmp225_ = vala_dbus_client_module_generate_dbus_proxy_method (self, iface, m))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp228_);
				_vala_ccode_node_unref0 (_tmp227_);
				_vala_ccode_node_unref0 (_tmp226_);
				_g_free0 (_tmp225_);
			} else {
				ValaCCodeExpressionStatement* _tmp232_;
				ValaCCodeAssignment* _tmp231_;
				ValaCCodeIdentifier* _tmp230_;
				char* _tmp229_;
				ValaCCodeMemberAccess* _tmp235_;
				char* _tmp234_;
				ValaCCodeIdentifier* _tmp233_;
				ValaCCodeExpressionStatement* _tmp239_;
				ValaCCodeAssignment* _tmp238_;
				ValaCCodeIdentifier* _tmp237_;
				char* _tmp236_;
				vala_ccode_block_add_statement (iface_block, (ValaCCodeNode*) (_tmp232_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp231_ = vala_ccode_assignment_new ((ValaCCodeExpression*) vfunc_entry, (ValaCCodeExpression*) (_tmp230_ = vala_ccode_identifier_new (_tmp229_ = vala_dbus_client_module_generate_async_dbus_proxy_method (self, iface, m))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp232_);
				_vala_ccode_node_unref0 (_tmp231_);
				_vala_ccode_node_unref0 (_tmp230_);
				_g_free0 (_tmp229_);
				vfunc_entry = (_tmp235_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp233_ = vala_ccode_identifier_new ("iface")), _tmp234_ = vala_method_get_finish_vfunc_name (m)), _vala_ccode_node_unref0 (vfunc_entry), _tmp235_);
				_g_free0 (_tmp234_);
				_vala_ccode_node_unref0 (_tmp233_);
				vala_ccode_block_add_statement (iface_block, (ValaCCodeNode*) (_tmp239_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp238_ = vala_ccode_assignment_new ((ValaCCodeExpression*) vfunc_entry, (ValaCCodeExpression*) (_tmp237_ = vala_ccode_identifier_new (_tmp236_ = vala_dbus_client_module_generate_finish_dbus_proxy_method (self, iface, m))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp239_);
				_vala_ccode_node_unref0 (_tmp238_);
				_vala_ccode_node_unref0 (_tmp237_);
				_g_free0 (_tmp236_);
			}
			_vala_code_node_unref0 (m);
			_vala_ccode_node_unref0 (vfunc_entry);
		}
		_gee_collection_object_unref0 (_m_it);
	}
	{
		GeeList* _tmp240_;
		GeeIterator* _tmp241_;
		GeeIterator* _prop_it;
		_prop_it = (_tmp241_ = gee_iterable_iterator ((GeeIterable*) (_tmp240_ = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) iface))), _gee_collection_object_unref0 (_tmp240_), _tmp241_);
		while (TRUE) {
			ValaProperty* prop;
			if (!gee_iterator_next (_prop_it)) {
				break;
			}
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			if (vala_property_get_get_accessor (prop) != NULL) {
				char* _tmp243_;
				ValaCCodeIdentifier* _tmp242_;
				ValaCCodeMemberAccess* _tmp244_;
				ValaCCodeMemberAccess* vfunc_entry;
				ValaCCodeExpressionStatement* _tmp248_;
				ValaCCodeAssignment* _tmp247_;
				ValaCCodeIdentifier* _tmp246_;
				char* _tmp245_;
				vfunc_entry = (_tmp244_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp242_ = vala_ccode_identifier_new ("iface")), _tmp243_ = g_strconcat ("get_", vala_symbol_get_name ((ValaSymbol*) prop), NULL)), _g_free0 (_tmp243_), _vala_ccode_node_unref0 (_tmp242_), _tmp244_);
				vala_ccode_block_add_statement (iface_block, (ValaCCodeNode*) (_tmp248_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp247_ = vala_ccode_assignment_new ((ValaCCodeExpression*) vfunc_entry, (ValaCCodeExpression*) (_tmp246_ = vala_ccode_identifier_new (_tmp245_ = vala_dbus_client_module_generate_dbus_proxy_property_get (self, iface, prop))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp248_);
				_vala_ccode_node_unref0 (_tmp247_);
				_vala_ccode_node_unref0 (_tmp246_);
				_g_free0 (_tmp245_);
				_vala_ccode_node_unref0 (vfunc_entry);
			}
			if (vala_property_get_set_accessor (prop) != NULL) {
				char* _tmp250_;
				ValaCCodeIdentifier* _tmp249_;
				ValaCCodeMemberAccess* _tmp251_;
				ValaCCodeMemberAccess* vfunc_entry;
				ValaCCodeExpressionStatement* _tmp255_;
				ValaCCodeAssignment* _tmp254_;
				ValaCCodeIdentifier* _tmp253_;
				char* _tmp252_;
				vfunc_entry = (_tmp251_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp249_ = vala_ccode_identifier_new ("iface")), _tmp250_ = g_strconcat ("set_", vala_symbol_get_name ((ValaSymbol*) prop), NULL)), _g_free0 (_tmp250_), _vala_ccode_node_unref0 (_tmp249_), _tmp251_);
				vala_ccode_block_add_statement (iface_block, (ValaCCodeNode*) (_tmp255_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp254_ = vala_ccode_assignment_new ((ValaCCodeExpression*) vfunc_entry, (ValaCCodeExpression*) (_tmp253_ = vala_ccode_identifier_new (_tmp252_ = vala_dbus_client_module_generate_dbus_proxy_property_set (self, iface, prop))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp255_);
				_vala_ccode_node_unref0 (_tmp254_);
				_vala_ccode_node_unref0 (_tmp253_);
				_g_free0 (_tmp252_);
				_vala_ccode_node_unref0 (vfunc_entry);
			}
			_vala_code_node_unref0 (prop);
		}
		_gee_collection_object_unref0 (_prop_it);
	}
	vala_ccode_function_set_modifiers (proxy_iface_init, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp256_ = vala_ccode_function_copy (proxy_iface_init)));
	_vala_ccode_node_unref0 (_tmp256_);
	vala_ccode_function_set_block (proxy_iface_init, iface_block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) proxy_iface_init);
	get_prop = (_tmp258_ = vala_ccode_function_new (_tmp257_ = g_strdup_printf ("%s_get_property", lower_cname), "void"), _g_free0 (_tmp257_), _tmp258_);
	vala_ccode_function_set_modifiers (get_prop, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (get_prop, _tmp259_ = vala_ccode_formal_parameter_new ("object", "GObject *"));
	_vala_ccode_node_unref0 (_tmp259_);
	vala_ccode_function_add_parameter (get_prop, _tmp260_ = vala_ccode_formal_parameter_new ("property_id", "guint"));
	_vala_ccode_node_unref0 (_tmp260_);
	vala_ccode_function_add_parameter (get_prop, _tmp261_ = vala_ccode_formal_parameter_new ("value", "GValue *"));
	_vala_ccode_node_unref0 (_tmp261_);
	vala_ccode_function_add_parameter (get_prop, _tmp262_ = vala_ccode_formal_parameter_new ("pspec", "GParamSpec *"));
	_vala_ccode_node_unref0 (_tmp262_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp263_ = vala_ccode_function_copy (get_prop)));
	_vala_ccode_node_unref0 (_tmp263_);
	vala_ccode_function_set_block (get_prop, _tmp264_ = vala_ccode_block_new ());
	_vala_ccode_node_unref0 (_tmp264_);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) get_prop);
	set_prop = (_tmp266_ = vala_ccode_function_new (_tmp265_ = g_strdup_printf ("%s_set_property", lower_cname), "void"), _g_free0 (_tmp265_), _tmp266_);
	vala_ccode_function_set_modifiers (set_prop, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (set_prop, _tmp267_ = vala_ccode_formal_parameter_new ("object", "GObject *"));
	_vala_ccode_node_unref0 (_tmp267_);
	vala_ccode_function_add_parameter (set_prop, _tmp268_ = vala_ccode_formal_parameter_new ("property_id", "guint"));
	_vala_ccode_node_unref0 (_tmp268_);
	vala_ccode_function_add_parameter (set_prop, _tmp269_ = vala_ccode_formal_parameter_new ("value", "const GValue *"));
	_vala_ccode_node_unref0 (_tmp269_);
	vala_ccode_function_add_parameter (set_prop, _tmp270_ = vala_ccode_formal_parameter_new ("pspec", "GParamSpec *"));
	_vala_ccode_node_unref0 (_tmp270_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp271_ = vala_ccode_function_copy (set_prop)));
	_vala_ccode_node_unref0 (_tmp271_);
	vala_ccode_function_set_block (set_prop, _tmp272_ = vala_ccode_block_new ());
	_vala_ccode_node_unref0 (_tmp272_);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) set_prop);
	_g_free0 (dbus_iface_name);
	_g_free0 (cname);
	_g_free0 (lower_cname);
	_vala_ccode_node_unref0 (instance_struct);
	_vala_ccode_node_unref0 (implement);
	_vala_ccode_node_unref0 (define_type);
	_vala_ccode_node_unref0 (proxy_new);
	_vala_ccode_node_unref0 (new_block);
	_vala_ccode_node_unref0 (new_call);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (proxy_construct);
	_vala_ccode_node_unref0 (parent_class);
	_vala_ccode_node_unref0 (chainup);
	_vala_ccode_node_unref0 (gconnection);
	_vala_ccode_node_unref0 (path);
	_vala_ccode_node_unref0 (raw_connection);
	_vala_ccode_node_unref0 (filter_call);
	_vala_ccode_node_unref0 (filter_printf);
	_vala_ccode_node_unref0 (match_call);
	_vala_ccode_node_unref0 (connection_free);
	_vala_ccode_node_unref0 (path_free);
	_vala_ccode_node_unref0 (filter_free);
	_vala_ccode_node_unref0 (proxy_dispose);
	_vala_ccode_node_unref0 (dispose_return_block);
	_vala_ccode_node_unref0 (proxy_class_init);
	_vala_ccode_node_unref0 (gobject_class);
	_vala_ccode_node_unref0 (proxy_instance_init);
	_vala_ccode_node_unref0 (proxy_iface_init);
	_vala_ccode_node_unref0 (iface_block);
	_vala_ccode_node_unref0 (get_prop);
	_vala_ccode_node_unref0 (set_prop);
}


static ValaTypeRegisterFunction* vala_dbus_client_module_real_create_interface_register_function (ValaGTypeModule* base, ValaInterface* iface) {
	ValaDBusClientModule * self;
	ValaTypeRegisterFunction* result;
	char* dbus_iface_name;
	self = (ValaDBusClientModule*) base;
	g_return_val_if_fail (iface != NULL, NULL);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	if (dbus_iface_name == NULL) {
		result = (ValaTypeRegisterFunction*) vala_interface_register_function_new (iface, vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self));
		_g_free0 (dbus_iface_name);
		return result;
	}
	result = (ValaTypeRegisterFunction*) vala_dbus_interface_register_function_new (iface, vala_ccode_base_module_get_context ((ValaCCodeBaseModule*) self));
	_g_free0 (dbus_iface_name);
	return result;
}


static void vala_dbus_client_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr) {
	ValaDBusClientModule * self;
	ValaDataType* _tmp0_;
	ValaMethodType* mtype;
	gboolean _tmp1_;
	GeeList* args;
	ValaExpression* connection;
	ValaExpression* bus_name;
	ValaExpression* object_path;
	ValaExpression* interface_name;
	ValaExpression* type;
	ValaCCodeIdentifier* _tmp3_;
	ValaCCodeFunctionCall* _tmp4_;
	ValaCCodeFunctionCall* quark_call;
	ValaCCodeConstant* _tmp5_;
	ValaCCodeIdentifier* _tmp6_;
	ValaCCodeFunctionCall* _tmp7_;
	ValaCCodeFunctionCall* qdata_call;
	ValaCCodeCastExpression* _tmp8_;
	ValaCCodeFunctionCall* _tmp9_;
	ValaCCodeFunctionCall* get_type_call;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeFunctionCall* _tmp11_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeConstant* _tmp12_;
	ValaCCodeConstant* _tmp13_;
	ValaCCodeConstant* _tmp14_;
	ValaCCodeConstant* _tmp15_;
	ValaCCodeConstant* _tmp16_;
	self = (ValaDBusClientModule*) base;
	g_return_if_fail (expr != NULL);
	mtype = _vala_code_node_ref0 ((_tmp0_ = vala_expression_get_value_type (vala_method_call_get_call (expr)), VALA_IS_METHOD_TYPE (_tmp0_) ? ((ValaMethodType*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (mtype == NULL) {
		_tmp1_ = TRUE;
	} else {
		char* _tmp2_;
		_tmp1_ = _vala_strcmp0 (_tmp2_ = vala_method_get_cname (vala_method_type_get_method_symbol (mtype)), "dbus_g_proxy_new_from_type") != 0;
		_g_free0 (_tmp2_);
	}
	if (_tmp1_) {
		VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->visit_method_call ((ValaCCodeModule*) VALA_DBUS_MODULE (self), expr);
		_vala_code_node_unref0 (mtype);
		return;
	}
	args = vala_method_call_get_argument_list (expr);
	connection = _vala_code_node_ref0 (vala_member_access_get_inner (VALA_MEMBER_ACCESS (vala_method_call_get_call (expr))));
	bus_name = (ValaExpression*) gee_list_get (args, 0);
	object_path = (ValaExpression*) gee_list_get (args, 1);
	interface_name = (ValaExpression*) gee_list_get (args, 2);
	type = (ValaExpression*) gee_list_get (args, 3);
	quark_call = (_tmp4_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("g_quark_from_string"))), _vala_ccode_node_unref0 (_tmp3_), _tmp4_);
	vala_ccode_function_call_add_argument (quark_call, (ValaCCodeExpression*) (_tmp5_ = vala_ccode_constant_new ("\"ValaDBusInterfaceProxyType\"")));
	_vala_ccode_node_unref0 (_tmp5_);
	qdata_call = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("g_type_get_qdata"))), _vala_ccode_node_unref0 (_tmp6_), _tmp7_);
	vala_code_node_accept ((ValaCodeNode*) type, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (qdata_call, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) type)));
	vala_ccode_function_call_add_argument (qdata_call, (ValaCCodeExpression*) quark_call);
	get_type_call = (_tmp9_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp8_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) qdata_call, "GType (*)(void)"))), _vala_ccode_node_unref0 (_tmp8_), _tmp9_);
	ccall = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("g_object_new"))), _vala_ccode_node_unref0 (_tmp10_), _tmp11_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) get_type_call);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_code_node_accept ((ValaCodeNode*) connection, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) connection)));
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp13_ = vala_ccode_constant_new ("\"name\"")));
	_vala_ccode_node_unref0 (_tmp13_);
	vala_code_node_accept ((ValaCodeNode*) bus_name, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) bus_name)));
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp14_ = vala_ccode_constant_new ("\"path\"")));
	_vala_ccode_node_unref0 (_tmp14_);
	vala_code_node_accept ((ValaCodeNode*) object_path, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) object_path)));
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp15_ = vala_ccode_constant_new ("\"interface\"")));
	_vala_ccode_node_unref0 (_tmp15_);
	vala_code_node_accept ((ValaCodeNode*) interface_name, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) interface_name)));
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall);
	_vala_code_node_unref0 (mtype);
	_gee_collection_object_unref0 (args);
	_vala_code_node_unref0 (connection);
	_vala_code_node_unref0 (bus_name);
	_vala_code_node_unref0 (object_path);
	_vala_code_node_unref0 (interface_name);
	_vala_code_node_unref0 (type);
	_vala_ccode_node_unref0 (quark_call);
	_vala_ccode_node_unref0 (qdata_call);
	_vala_ccode_node_unref0 (get_type_call);
	_vala_ccode_node_unref0 (ccall);
}


static void vala_dbus_client_module_generate_proxy_filter_function (ValaDBusClientModule* self, ValaInterface* iface) {
	char* _tmp0_;
	char* _tmp1_;
	char* lower_cname;
	char* _tmp2_;
	ValaCCodeFunction* _tmp3_;
	ValaCCodeFunction* proxy_filter;
	ValaCCodeFormalParameter* _tmp4_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFormalParameter* _tmp6_;
	ValaCCodeBlock* filter_block;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeFunctionCall* _tmp8_;
	ValaCCodeFunctionCall* path;
	ValaCCodeIdentifier* _tmp9_;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeFunctionCall* _tmp11_;
	ValaCCodeFunctionCall* ccheck;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeBlock* object_filter_block;
	ValaCCodeIfStatement* _tmp13_;
	ValaCCodeReturnStatement* _tmp15_;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeFunction* _tmp16_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (iface != NULL);
	lower_cname = (_tmp1_ = g_strconcat (_tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), "dbus_proxy", NULL), _g_free0 (_tmp0_), _tmp1_);
	proxy_filter = (_tmp3_ = vala_ccode_function_new (_tmp2_ = g_strconcat (lower_cname, "_filter", NULL), "DBusHandlerResult"), _g_free0 (_tmp2_), _tmp3_);
	vala_ccode_function_add_parameter (proxy_filter, _tmp4_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_add_parameter (proxy_filter, _tmp5_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_function_add_parameter (proxy_filter, _tmp6_ = vala_ccode_formal_parameter_new ("user_data", "void*"));
	_vala_ccode_node_unref0 (_tmp6_);
	filter_block = vala_ccode_block_new ();
	path = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("dbus_g_proxy_get_path"))), _vala_ccode_node_unref0 (_tmp7_), _tmp8_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("user_data")));
	_vala_ccode_node_unref0 (_tmp9_);
	ccheck = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("dbus_message_has_path"))), _vala_ccode_node_unref0 (_tmp10_), _tmp11_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) path);
	object_filter_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (filter_block, (ValaCCodeNode*) (_tmp13_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccheck, (ValaCCodeStatement*) object_filter_block, NULL)));
	_vala_ccode_node_unref0 (_tmp13_);
	vala_dbus_client_module_handle_signals (self, iface, object_filter_block);
	vala_ccode_block_add_statement (filter_block, (ValaCCodeNode*) (_tmp15_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp15_);
	_vala_ccode_node_unref0 (_tmp14_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp16_ = vala_ccode_function_copy (proxy_filter)));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_ccode_function_set_block (proxy_filter, filter_block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) proxy_filter);
	_g_free0 (lower_cname);
	_vala_ccode_node_unref0 (proxy_filter);
	_vala_ccode_node_unref0 (filter_block);
	_vala_ccode_node_unref0 (path);
	_vala_ccode_node_unref0 (ccheck);
	_vala_ccode_node_unref0 (object_filter_block);
}


static char* vala_dbus_client_module_generate_dbus_signal_handler (ValaDBusClientModule* self, ValaSignal* sig, ValaObjectTypeSymbol* sym) {
	char* result;
	char* _tmp1_;
	char* _tmp0_;
	char* _tmp2_;
	char* wrapper_name;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	ValaCCodeFormalParameter* _tmp6_;
	ValaCCodeFormalParameter* _tmp7_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeDeclaration* _tmp8_;
	ValaCCodeVariableDeclarator* _tmp9_;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeFunctionCall* _tmp11_;
	ValaCCodeFunctionCall* message_signature;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeFunctionCall* _tmp14_;
	ValaCCodeFunctionCall* signature_check;
	ValaCCodeBlock* signature_error_block;
	ValaCCodeReturnStatement* _tmp15_;
	ValaCCodeIfStatement* _tmp16_;
	ValaCCodeIdentifier* _tmp17_;
	ValaCCodeFunctionCall* _tmp18_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeUnaryExpression* _tmp21_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeExpressionStatement* _tmp22_;
	ValaCCodeIdentifier* _tmp23_;
	ValaCCodeFunctionCall* _tmp24_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp25_;
	ValaCCodeConstant* _tmp26_;
	char* type_signature;
	ValaCCodeConstant* _tmp54_;
	char* _tmp53_;
	ValaCCodeExpressionStatement* _tmp55_;
	ValaCCodeDeclaration* _tmp56_;
	ValaCCodeVariableDeclarator* _tmp57_;
	ValaCCodeFunction* _tmp58_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sig != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	wrapper_name = (_tmp2_ = g_strdup_printf ("_dbus_handle_%s_%s", _tmp0_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) sym, NULL), _tmp1_ = vala_signal_get_cname (sig)), _g_free0 (_tmp1_), _g_free0 (_tmp0_), _tmp2_);
	_cdecl_ = NULL;
	function = vala_ccode_function_new (wrapper_name, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("self", _tmp4_ = g_strconcat (_tmp3_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) sym, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	_g_free0 (_tmp3_);
	vala_ccode_function_add_parameter (function, _tmp6_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp6_);
	vala_ccode_function_add_parameter (function, _tmp7_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp7_);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	_cdecl_ = (_tmp8_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp8_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp9_ = vala_ccode_variable_declarator_new ("iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	message_signature = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("dbus_message_get_signature"))), _vala_ccode_node_unref0 (_tmp10_), _tmp11_);
	vala_ccode_function_call_add_argument (message_signature, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp12_);
	signature_check = (_tmp14_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp13_), _tmp14_);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) message_signature);
	signature_error_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (signature_error_block, (ValaCCodeNode*) (_tmp15_ = vala_ccode_return_statement_new (NULL)));
	_vala_ccode_node_unref0 (_tmp15_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp16_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) signature_check, (ValaCCodeStatement*) signature_error_block, NULL)));
	_vala_ccode_node_unref0 (_tmp16_);
	iter_call = (_tmp18_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (_tmp17_), _tmp18_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp19_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp21_);
	_vala_ccode_node_unref0 (_tmp20_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp22_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp22_);
	ccall = (_tmp24_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new ("g_signal_emit_by_name"))), _vala_ccode_node_unref0 (_tmp23_), _tmp24_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp26_ = vala_signal_get_canonical_cconstant (sig, NULL)));
	_vala_ccode_node_unref0 (_tmp26_);
	type_signature = g_strdup ("");
	{
		GeeList* _tmp27_;
		GeeIterator* _tmp28_;
		GeeIterator* _param_it;
		_param_it = (_tmp28_ = gee_iterable_iterator ((GeeIterable*) (_tmp27_ = vala_signal_get_parameters (sig))), _gee_collection_object_unref0 (_tmp27_), _tmp28_);
		while (TRUE) {
			ValaFormalParameter* param;
			ValaCCodeDeclaration* _tmp30_;
			char* _tmp29_;
			ValaCCodeVariableDeclarator* _tmp32_;
			ValaCCodeExpression* _tmp31_;
			char* _tmp33_;
			gboolean _tmp34_;
			ValaTypeSymbol* _tmp37_;
			ValaStruct* st;
			gboolean _tmp38_;
			char* _tmp48_;
			char* _tmp47_;
			ValaCCodeIdentifier* target;
			ValaCCodeIdentifier* _tmp49_;
			ValaCCodeExpression* _tmp50_;
			ValaCCodeExpression* expr;
			ValaCCodeExpressionStatement* _tmp52_;
			ValaCCodeAssignment* _tmp51_;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			_cdecl_ = (_tmp30_ = vala_ccode_declaration_new (_tmp29_ = vala_data_type_get_cname (vala_formal_parameter_get_parameter_type (param))), _vala_ccode_node_unref0 (_cdecl_), _tmp30_);
			_g_free0 (_tmp29_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp32_ = vala_ccode_variable_declarator_new (vala_symbol_get_name ((ValaSymbol*) param), _tmp31_ = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_parameter_type (param), TRUE), NULL)));
			_vala_ccode_node_unref0 (_tmp32_);
			_vala_ccode_node_unref0 (_tmp31_);
			vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
			if ((_tmp34_ = (_tmp33_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param))) == NULL, _g_free0 (_tmp33_), _tmp34_)) {
				char* _tmp36_;
				char* _tmp35_;
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_formal_parameter_get_parameter_type (param)), _tmp36_ = g_strdup_printf ("D-Bus serialization of type `%s' is not supported", _tmp35_ = vala_code_node_to_string ((ValaCodeNode*) vala_formal_parameter_get_parameter_type (param))));
				_g_free0 (_tmp36_);
				_g_free0 (_tmp35_);
				_vala_code_node_unref0 (param);
				continue;
			}
			st = _vala_code_node_ref0 ((_tmp37_ = vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param)), VALA_IS_STRUCT (_tmp37_) ? ((ValaStruct*) _tmp37_) : NULL));
			_tmp38_ = FALSE;
			if (st != NULL) {
				_tmp38_ = !vala_struct_is_simple_type (st);
			} else {
				_tmp38_ = FALSE;
			}
			if (_tmp38_) {
				ValaCCodeUnaryExpression* _tmp40_;
				ValaCCodeIdentifier* _tmp39_;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp40_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))))));
				_vala_ccode_node_unref0 (_tmp40_);
				_vala_ccode_node_unref0 (_tmp39_);
			} else {
				ValaCCodeIdentifier* _tmp41_;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp41_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
				_vala_ccode_node_unref0 (_tmp41_);
			}
			if (VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param))) {
				ValaArrayType* array_type;
				array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)));
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp42_;
						_tmp42_ = TRUE;
						while (TRUE) {
							char* length_cname;
							ValaCCodeDeclaration* _tmp43_;
							ValaCCodeVariableDeclarator* _tmp45_;
							ValaCCodeConstant* _tmp44_;
							ValaCCodeIdentifier* _tmp46_;
							if (!_tmp42_) {
								dim++;
							}
							_tmp42_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							length_cname = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, vala_symbol_get_name ((ValaSymbol*) param), dim);
							_cdecl_ = (_tmp43_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp43_);
							vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp45_ = vala_ccode_variable_declarator_new (length_cname, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_constant_new ("0")), NULL)));
							_vala_ccode_node_unref0 (_tmp45_);
							_vala_ccode_node_unref0 (_tmp44_);
							vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
							vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp46_ = vala_ccode_identifier_new (length_cname)));
							_vala_ccode_node_unref0 (_tmp46_);
							_g_free0 (length_cname);
						}
					}
				}
				_vala_code_node_unref0 (array_type);
			}
			type_signature = (_tmp48_ = g_strconcat (type_signature, _tmp47_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param)), NULL), _g_free0 (type_signature), _tmp48_);
			_g_free0 (_tmp47_);
			target = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param));
			expr = (_tmp50_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, prefragment, vala_formal_parameter_get_parameter_type (param), (ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp49_), _tmp50_);
			vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp52_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp51_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp52_);
			_vala_ccode_node_unref0 (_tmp51_);
			_vala_code_node_unref0 (param);
			_vala_code_node_unref0 (st);
			_vala_ccode_node_unref0 (target);
			_vala_ccode_node_unref0 (expr);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) (_tmp54_ = vala_ccode_constant_new (_tmp53_ = g_strdup_printf ("\"%s\"", type_signature))));
	_vala_ccode_node_unref0 (_tmp54_);
	_g_free0 (_tmp53_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp55_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp55_);
	_cdecl_ = (_tmp56_ = vala_ccode_declaration_new ("DBusMessage*"), _vala_ccode_node_unref0 (_cdecl_), _tmp56_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp57_ = vala_ccode_variable_declarator_new ("reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp57_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp58_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp58_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = wrapper_name;
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (message_signature);
	_vala_ccode_node_unref0 (signature_check);
	_vala_ccode_node_unref0 (signature_error_block);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (ccall);
	_g_free0 (type_signature);
	return result;
}


static void vala_dbus_client_module_handle_signal (ValaDBusClientModule* self, const char* dbus_iface_name, const char* dbus_signal_name, const char* handler_name, ValaCCodeBlock* block, ValaCCodeIfStatement** clastif) {
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* ccheck;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeConstant* _tmp4_;
	char* _tmp3_;
	ValaCCodeConstant* _tmp6_;
	char* _tmp5_;
	ValaCCodeBlock* callblock;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeFunctionCall* _tmp8_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp9_;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeIdentifier* _tmp11_;
	ValaCCodeExpressionStatement* _tmp12_;
	ValaCCodeIfStatement* cif;
	ValaCCodeIfStatement* _tmp13_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (dbus_iface_name != NULL);
	g_return_if_fail (dbus_signal_name != NULL);
	g_return_if_fail (handler_name != NULL);
	g_return_if_fail (block != NULL);
	g_return_if_fail (clastif != NULL);
	ccheck = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_message_is_signal"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new (_tmp3_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_constant_new (_tmp5_ = g_strdup_printf ("\"%s\"", dbus_signal_name))));
	_vala_ccode_node_unref0 (_tmp6_);
	_g_free0 (_tmp5_);
	callblock = vala_ccode_block_new ();
	ccall = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (handler_name))), _vala_ccode_node_unref0 (_tmp7_), _tmp8_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("user_data")));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_block_add_statement (callblock, (ValaCCodeNode*) (_tmp12_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp12_);
	cif = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccheck, (ValaCCodeStatement*) callblock, NULL);
	if ((*clastif) == NULL) {
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) cif);
	} else {
		vala_ccode_if_statement_set_false_statement ((*clastif), (ValaCCodeStatement*) cif);
	}
	(*clastif) = (_tmp13_ = _vala_ccode_node_ref0 (cif), _vala_ccode_node_unref0 ((*clastif)), _tmp13_);
	_vala_ccode_node_unref0 (ccheck);
	_vala_ccode_node_unref0 (callblock);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (cif);
}


static void vala_dbus_client_module_handle_signals (ValaDBusClientModule* self, ValaInterface* iface, ValaCCodeBlock* block) {
	char* dbus_iface_name;
	ValaCCodeIfStatement* clastif;
	g_return_if_fail (self != NULL);
	g_return_if_fail (iface != NULL);
	g_return_if_fail (block != NULL);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	clastif = NULL;
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _sig_it;
		_sig_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_object_type_symbol_get_signals ((ValaObjectTypeSymbol*) iface))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaSignal* sig;
			char* _tmp3_;
			char* _tmp2_;
			if (!gee_iterator_next (_sig_it)) {
				break;
			}
			sig = (ValaSignal*) gee_iterator_get (_sig_it);
			if (vala_symbol_get_access ((ValaSymbol*) sig) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC) {
				_vala_code_node_unref0 (sig);
				continue;
			}
			vala_dbus_client_module_handle_signal (self, dbus_iface_name, _tmp2_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) sig)), _tmp3_ = vala_dbus_client_module_generate_dbus_signal_handler (self, sig, (ValaObjectTypeSymbol*) iface), block, &clastif);
			_g_free0 (_tmp3_);
			_g_free0 (_tmp2_);
			_vala_code_node_unref0 (sig);
		}
		_gee_collection_object_unref0 (_sig_it);
	}
	_g_free0 (dbus_iface_name);
	_vala_ccode_node_unref0 (clastif);
}


static void vala_dbus_client_module_generate_marshalling (ValaDBusClientModule* self, ValaMethod* m, const char* dbus_iface_name, ValaCCodeFragment* prefragment, ValaCCodeFragment* postfragment) {
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* destination;
	ValaCCodeCastExpression* _tmp3_;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeIdentifier* _tmp4_;
	ValaCCodeFunctionCall* _tmp5_;
	ValaCCodeFunctionCall* path;
	ValaCCodeCastExpression* _tmp7_;
	ValaCCodeIdentifier* _tmp6_;
	ValaCCodeIdentifier* _tmp8_;
	ValaCCodeFunctionCall* _tmp9_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeConstant* _tmp11_;
	char* _tmp10_;
	ValaCCodeConstant* _tmp14_;
	char* _tmp13_;
	char* _tmp12_;
	ValaCCodeExpressionStatement* _tmp17_;
	ValaCCodeAssignment* _tmp16_;
	ValaCCodeIdentifier* _tmp15_;
	ValaCCodeIdentifier* _tmp18_;
	ValaCCodeFunctionCall* _tmp19_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeUnaryExpression* _tmp22_;
	ValaCCodeIdentifier* _tmp21_;
	ValaCCodeExpressionStatement* _tmp23_;
	ValaCCodeFunctionCall* _tmp25_;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeIdentifier* _tmp26_;
	ValaCCodeUnaryExpression* _tmp28_;
	ValaCCodeIdentifier* _tmp27_;
	ValaCCodeExpressionStatement* _tmp29_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (m != NULL);
	g_return_if_fail (dbus_iface_name != NULL);
	g_return_if_fail (prefragment != NULL);
	g_return_if_fail (postfragment != NULL);
	_cdecl_ = NULL;
	destination = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_g_proxy_get_bus_name"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (destination, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("self")), "DBusGProxy*")));
	_vala_ccode_node_unref0 (_tmp3_);
	_vala_ccode_node_unref0 (_tmp2_);
	path = (_tmp5_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("dbus_g_proxy_get_path"))), _vala_ccode_node_unref0 (_tmp4_), _tmp5_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("self")), "DBusGProxy*")));
	_vala_ccode_node_unref0 (_tmp7_);
	_vala_ccode_node_unref0 (_tmp6_);
	msgcall = (_tmp9_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("dbus_message_new_method_call"))), _vala_ccode_node_unref0 (_tmp8_), _tmp9_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) destination);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) path);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_constant_new (_tmp10_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp11_);
	_g_free0 (_tmp10_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp14_ = vala_ccode_constant_new (_tmp13_ = g_strdup_printf ("\"%s\"", _tmp12_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) m))))));
	_vala_ccode_node_unref0 (_tmp14_);
	_g_free0 (_tmp13_);
	_g_free0 (_tmp12_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp17_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("_message")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp17_);
	_vala_ccode_node_unref0 (_tmp16_);
	_vala_ccode_node_unref0 (_tmp15_);
	iter_call = (_tmp19_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (_tmp18_), _tmp19_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp20_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp22_);
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp23_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp23_);
	iter_call = (_tmp25_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (iter_call), _tmp25_);
	_vala_ccode_node_unref0 (_tmp24_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("_reply")));
	_vala_ccode_node_unref0 (_tmp26_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp28_);
	_vala_ccode_node_unref0 (_tmp27_);
	vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp29_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp29_);
	{
		GeeList* _tmp30_;
		GeeIterator* _tmp31_;
		GeeIterator* _param_it;
		_param_it = (_tmp31_ = gee_iterable_iterator ((GeeIterable*) (_tmp30_ = vala_method_get_parameters (m))), _gee_collection_object_unref0 (_tmp30_), _tmp31_);
		while (TRUE) {
			ValaFormalParameter* param;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN) {
				gboolean _tmp32_;
				ValaCCodeExpression* expr;
				ValaCCodeIdentifier* _tmp35_;
				_tmp32_ = FALSE;
				if (vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param)) != NULL) {
					char* _tmp33_;
					_tmp32_ = _vala_strcmp0 (_tmp33_ = vala_symbol_get_full_name ((ValaSymbol*) vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param))), "DBus.BusName") == 0;
					_g_free0 (_tmp33_);
				} else {
					_tmp32_ = FALSE;
				}
				if (_tmp32_) {
					_vala_code_node_unref0 (param);
					continue;
				}
				expr = (ValaCCodeExpression*) vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param));
				if (vala_data_type_is_real_struct_type (vala_formal_parameter_get_parameter_type (param))) {
					ValaCCodeExpression* _tmp34_;
					expr = (_tmp34_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, expr), _vala_ccode_node_unref0 (expr), _tmp34_);
				}
				vala_dbus_module_write_expression ((ValaDBusModule*) self, prefragment, vala_formal_parameter_get_parameter_type (param), (ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new ("_iter")), expr);
				_vala_ccode_node_unref0 (_tmp35_);
				_vala_ccode_node_unref0 (expr);
			} else {
				ValaCCodeDeclaration* _tmp37_;
				char* _tmp36_;
				ValaCCodeVariableDeclarator* _tmp39_;
				char* _tmp38_;
				ValaDataType* _tmp40_;
				ValaArrayType* array_type;
				char* _tmp46_;
				ValaCCodeIdentifier* _tmp47_;
				ValaCCodeIdentifier* target;
				ValaCCodeIdentifier* _tmp48_;
				ValaCCodeExpression* _tmp49_;
				ValaCCodeExpression* expr;
				ValaCCodeExpressionStatement* _tmp51_;
				ValaCCodeAssignment* _tmp50_;
				ValaCCodeExpressionStatement* _tmp55_;
				ValaCCodeAssignment* _tmp54_;
				ValaCCodeUnaryExpression* _tmp53_;
				ValaCCodeIdentifier* _tmp52_;
				_cdecl_ = (_tmp37_ = vala_ccode_declaration_new (_tmp36_ = vala_data_type_get_cname (vala_formal_parameter_get_parameter_type (param))), _vala_ccode_node_unref0 (_cdecl_), _tmp37_);
				_g_free0 (_tmp36_);
				vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp39_ = vala_ccode_variable_declarator_new (_tmp38_ = g_strconcat ("_", vala_symbol_get_name ((ValaSymbol*) param), NULL), NULL, NULL)));
				_vala_ccode_node_unref0 (_tmp39_);
				_g_free0 (_tmp38_);
				vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
				array_type = _vala_code_node_ref0 ((_tmp40_ = vala_formal_parameter_get_parameter_type (param), VALA_IS_ARRAY_TYPE (_tmp40_) ? ((ValaArrayType*) _tmp40_) : NULL));
				if (array_type != NULL) {
					{
						gint dim;
						dim = 1;
						{
							gboolean _tmp41_;
							_tmp41_ = TRUE;
							while (TRUE) {
								ValaCCodeDeclaration* _tmp42_;
								ValaCCodeVariableDeclarator* _tmp45_;
								ValaCCodeConstant* _tmp44_;
								char* _tmp43_;
								if (!_tmp41_) {
									dim++;
								}
								_tmp41_ = FALSE;
								if (!(dim <= vala_array_type_get_rank (array_type))) {
									break;
								}
								_cdecl_ = (_tmp42_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp42_);
								vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp45_ = vala_ccode_variable_declarator_new (_tmp43_ = g_strdup_printf ("_%s_length%d", vala_symbol_get_name ((ValaSymbol*) param), dim), (ValaCCodeExpression*) (_tmp44_ = vala_ccode_constant_new ("0")), NULL)));
								_vala_ccode_node_unref0 (_tmp45_);
								_vala_ccode_node_unref0 (_tmp44_);
								_g_free0 (_tmp43_);
								vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
							}
						}
					}
				}
				target = (_tmp47_ = vala_ccode_identifier_new (_tmp46_ = g_strconcat ("_", vala_symbol_get_name ((ValaSymbol*) param), NULL)), _g_free0 (_tmp46_), _tmp47_);
				expr = (_tmp49_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, postfragment, vala_formal_parameter_get_parameter_type (param), (ValaCCodeExpression*) (_tmp48_ = vala_ccode_identifier_new ("_iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp48_), _tmp49_);
				vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp51_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp50_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp51_);
				_vala_ccode_node_unref0 (_tmp50_);
				vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp55_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp54_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp53_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp52_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))))), (ValaCCodeExpression*) target, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp55_);
				_vala_ccode_node_unref0 (_tmp54_);
				_vala_ccode_node_unref0 (_tmp53_);
				_vala_ccode_node_unref0 (_tmp52_);
				if (array_type != NULL) {
					{
						gint dim;
						dim = 1;
						{
							gboolean _tmp56_;
							_tmp56_ = TRUE;
							while (TRUE) {
								ValaCCodeExpressionStatement* _tmp63_;
								ValaCCodeAssignment* _tmp62_;
								ValaCCodeIdentifier* _tmp61_;
								char* _tmp60_;
								ValaCCodeUnaryExpression* _tmp59_;
								ValaCCodeIdentifier* _tmp58_;
								char* _tmp57_;
								if (!_tmp56_) {
									dim++;
								}
								_tmp56_ = FALSE;
								if (!(dim <= vala_array_type_get_rank (array_type))) {
									break;
								}
								vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp63_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp62_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp59_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp58_ = vala_ccode_identifier_new (_tmp57_ = g_strdup_printf ("%s_length%d", vala_symbol_get_name ((ValaSymbol*) param), dim))))), (ValaCCodeExpression*) (_tmp61_ = vala_ccode_identifier_new (_tmp60_ = g_strdup_printf ("_%s_length%d", vala_symbol_get_name ((ValaSymbol*) param), dim))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
								_vala_ccode_node_unref0 (_tmp63_);
								_vala_ccode_node_unref0 (_tmp62_);
								_vala_ccode_node_unref0 (_tmp61_);
								_g_free0 (_tmp60_);
								_vala_ccode_node_unref0 (_tmp59_);
								_vala_ccode_node_unref0 (_tmp58_);
								_g_free0 (_tmp57_);
							}
						}
					}
				}
				_vala_code_node_unref0 (array_type);
				_vala_ccode_node_unref0 (target);
				_vala_ccode_node_unref0 (expr);
			}
			_vala_code_node_unref0 (param);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
		ValaCCodeDeclaration* _tmp65_;
		char* _tmp64_;
		ValaCCodeVariableDeclarator* _tmp66_;
		ValaDataType* _tmp67_;
		ValaArrayType* array_type;
		ValaCCodeIdentifier* target;
		ValaCCodeIdentifier* _tmp73_;
		ValaCCodeExpression* _tmp74_;
		ValaCCodeExpression* expr;
		ValaCCodeExpressionStatement* _tmp76_;
		ValaCCodeAssignment* _tmp75_;
		_cdecl_ = (_tmp65_ = vala_ccode_declaration_new (_tmp64_ = vala_data_type_get_cname (vala_method_get_return_type (m))), _vala_ccode_node_unref0 (_cdecl_), _tmp65_);
		_g_free0 (_tmp64_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp66_ = vala_ccode_variable_declarator_new ("_result", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp66_);
		vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
		array_type = _vala_code_node_ref0 ((_tmp67_ = vala_method_get_return_type (m), VALA_IS_ARRAY_TYPE (_tmp67_) ? ((ValaArrayType*) _tmp67_) : NULL));
		if (array_type != NULL) {
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp68_;
					_tmp68_ = TRUE;
					while (TRUE) {
						ValaCCodeDeclaration* _tmp69_;
						ValaCCodeVariableDeclarator* _tmp72_;
						ValaCCodeConstant* _tmp71_;
						char* _tmp70_;
						if (!_tmp68_) {
							dim++;
						}
						_tmp68_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						_cdecl_ = (_tmp69_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp69_);
						vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp72_ = vala_ccode_variable_declarator_new (_tmp70_ = g_strdup_printf ("_result_length%d", dim), (ValaCCodeExpression*) (_tmp71_ = vala_ccode_constant_new ("0")), NULL)));
						_vala_ccode_node_unref0 (_tmp72_);
						_vala_ccode_node_unref0 (_tmp71_);
						_g_free0 (_tmp70_);
						vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
					}
				}
			}
		}
		target = vala_ccode_identifier_new ("_result");
		expr = (_tmp74_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, postfragment, vala_method_get_return_type (m), (ValaCCodeExpression*) (_tmp73_ = vala_ccode_identifier_new ("_iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp73_), _tmp74_);
		vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp76_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp75_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp76_);
		_vala_ccode_node_unref0 (_tmp75_);
		if (array_type != NULL) {
			{
				gint dim;
				dim = 1;
				{
					gboolean _tmp77_;
					_tmp77_ = TRUE;
					while (TRUE) {
						ValaCCodeExpressionStatement* _tmp84_;
						ValaCCodeAssignment* _tmp83_;
						ValaCCodeIdentifier* _tmp82_;
						char* _tmp81_;
						ValaCCodeUnaryExpression* _tmp80_;
						ValaCCodeIdentifier* _tmp79_;
						char* _tmp78_;
						if (!_tmp77_) {
							dim++;
						}
						_tmp77_ = FALSE;
						if (!(dim <= vala_array_type_get_rank (array_type))) {
							break;
						}
						vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp84_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp83_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp80_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp79_ = vala_ccode_identifier_new (_tmp78_ = g_strdup_printf ("result_length%d", dim))))), (ValaCCodeExpression*) (_tmp82_ = vala_ccode_identifier_new (_tmp81_ = g_strdup_printf ("_result_length%d", dim))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
						_vala_ccode_node_unref0 (_tmp84_);
						_vala_ccode_node_unref0 (_tmp83_);
						_vala_ccode_node_unref0 (_tmp82_);
						_g_free0 (_tmp81_);
						_vala_ccode_node_unref0 (_tmp80_);
						_vala_ccode_node_unref0 (_tmp79_);
						_g_free0 (_tmp78_);
					}
				}
			}
		}
		_vala_code_node_unref0 (array_type);
		_vala_ccode_node_unref0 (target);
		_vala_ccode_node_unref0 (expr);
	}
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (destination);
	_vala_ccode_node_unref0 (path);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (iter_call);
}


static char* vala_dbus_client_module_generate_dbus_proxy_method (ValaDBusClientModule* self, ValaInterface* iface, ValaMethod* m) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* proxy_name;
	char* dbus_iface_name;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	GeeHashMap* cparam_map;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeFragment* postfragment;
	ValaCCodeBlock* dispose_return_block;
	GeeList* _tmp2_;
	gboolean _tmp3_;
	ValaCCodeIfStatement* _tmp19_;
	ValaCCodeMemberAccess* _tmp18_;
	ValaCCodeCastExpression* _tmp17_;
	char* _tmp16_;
	char* _tmp15_;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeDeclaration* _tmp20_;
	ValaCCodeVariableDeclarator* _tmp21_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeUnaryExpression* _tmp23_;
	ValaCCodeUnaryExpression* dbus_error;
	ValaCCodeDeclaration* _tmp24_;
	ValaCCodeVariableDeclarator* _tmp25_;
	ValaCCodeDeclaration* _tmp26_;
	ValaCCodeVariableDeclarator* _tmp27_;
	ValaCCodeVariableDeclarator* _tmp28_;
	ValaCCodeDeclaration* _tmp29_;
	ValaCCodeVariableDeclarator* _tmp30_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeFunctionCall* _tmp32_;
	ValaCCodeFunctionCall* gconnection;
	ValaCCodeIdentifier* _tmp33_;
	ValaCCodeConstant* _tmp34_;
	ValaCCodeUnaryExpression* _tmp36_;
	ValaCCodeIdentifier* _tmp35_;
	ValaCCodeConstant* _tmp37_;
	ValaCCodeExpressionStatement* _tmp38_;
	ValaCCodeIdentifier* _tmp39_;
	ValaCCodeFunctionCall* _tmp40_;
	ValaCCodeFunctionCall* dbus_error_init;
	ValaCCodeExpressionStatement* _tmp41_;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeFunctionCall* _tmp43_;
	ValaCCodeFunctionCall* connection;
	ValaCCodeIdentifier* _tmp44_;
	ValaCCodeIdentifier* _tmp45_;
	ValaCCodeFunctionCall* _tmp46_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp47_;
	ValaCCodeConstant* _tmp48_;
	ValaCCodeExpressionStatement* _tmp51_;
	ValaCCodeAssignment* _tmp50_;
	ValaCCodeIdentifier* _tmp49_;
	ValaCCodeIdentifier* _tmp52_;
	ValaCCodeFunctionCall* _tmp53_;
	ValaCCodeFunctionCall* conn_unref;
	ValaCCodeIdentifier* _tmp54_;
	ValaCCodeExpressionStatement* _tmp55_;
	ValaCCodeIdentifier* _tmp56_;
	ValaCCodeFunctionCall* _tmp57_;
	ValaCCodeFunctionCall* message_unref;
	ValaCCodeIdentifier* _tmp58_;
	ValaCCodeExpressionStatement* _tmp59_;
	GeeList* error_types;
	ValaCCodeIdentifier* _tmp84_;
	ValaCCodeFunctionCall* _tmp85_;
	ValaCCodeFunctionCall* reply_unref;
	ValaCCodeIdentifier* _tmp86_;
	ValaCCodeExpressionStatement* _tmp87_;
	ValaCCodeFunction* _tmp90_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (iface != NULL, NULL);
	g_return_val_if_fail (m != NULL, NULL);
	proxy_name = (_tmp1_ = g_strdup_printf ("%sdbus_proxy_%s", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), vala_symbol_get_name ((ValaSymbol*) m)), _g_free0 (_tmp0_), _tmp1_);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	_cdecl_ = NULL;
	function = vala_ccode_function_new (proxy_name, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	cparam_map = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_FORMAL_PARAMETER, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal);
	vala_ccode_base_module_generate_cparameters ((ValaCCodeBaseModule*) self, m, ((ValaCCodeBaseModule*) self)->source_declarations, (GeeMap*) cparam_map, function, NULL, NULL, NULL, 3);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	postfragment = vala_ccode_fragment_new ();
	dispose_return_block = vala_ccode_block_new ();
	if ((_tmp3_ = gee_collection_get_size ((GeeCollection*) (_tmp2_ = vala_code_node_get_error_types ((ValaCodeNode*) m))) > 0, _gee_collection_object_unref0 (_tmp2_), _tmp3_)) {
		ValaCCodeIdentifier* _tmp4_;
		ValaCCodeFunctionCall* _tmp5_;
		ValaCCodeFunctionCall* set_error_call;
		ValaCCodeIdentifier* _tmp6_;
		ValaCCodeIdentifier* _tmp7_;
		ValaCCodeIdentifier* _tmp8_;
		ValaCCodeConstant* _tmp9_;
		ValaCCodeExpressionStatement* _tmp10_;
		set_error_call = (_tmp5_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("g_set_error_literal"))), _vala_ccode_node_unref0 (_tmp4_), _tmp5_);
		vala_ccode_function_call_add_argument (set_error_call, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("error")));
		_vala_ccode_node_unref0 (_tmp6_);
		vala_ccode_function_call_add_argument (set_error_call, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("DBUS_GERROR")));
		_vala_ccode_node_unref0 (_tmp7_);
		vala_ccode_function_call_add_argument (set_error_call, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("DBUS_GERROR_DISCONNECTED")));
		_vala_ccode_node_unref0 (_tmp8_);
		vala_ccode_function_call_add_argument (set_error_call, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_constant_new ("\"Connection is closed\"")));
		_vala_ccode_node_unref0 (_tmp9_);
		vala_ccode_block_add_statement (dispose_return_block, (ValaCCodeNode*) (_tmp10_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) set_error_call)));
		_vala_ccode_node_unref0 (_tmp10_);
		_vala_ccode_node_unref0 (set_error_call);
	}
	if (VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
		ValaCCodeReturnStatement* _tmp11_;
		vala_ccode_block_add_statement (dispose_return_block, (ValaCCodeNode*) (_tmp11_ = vala_ccode_return_statement_new (NULL)));
		_vala_ccode_node_unref0 (_tmp11_);
	} else {
		ValaCCodeReturnStatement* _tmp13_;
		ValaCCodeExpression* _tmp12_;
		vala_ccode_block_add_statement (dispose_return_block, (ValaCCodeNode*) (_tmp13_ = vala_ccode_return_statement_new (_tmp12_ = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, vala_method_get_return_type (m), FALSE))));
		_vala_ccode_node_unref0 (_tmp13_);
		_vala_ccode_node_unref0 (_tmp12_);
	}
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp19_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("self")), _tmp16_ = g_strconcat (_tmp15_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), "DBusProxy*", NULL))), "disposed")), (ValaCCodeStatement*) dispose_return_block, NULL)));
	_vala_ccode_node_unref0 (_tmp19_);
	_vala_ccode_node_unref0 (_tmp18_);
	_vala_ccode_node_unref0 (_tmp17_);
	_g_free0 (_tmp16_);
	_g_free0 (_tmp15_);
	_vala_ccode_node_unref0 (_tmp14_);
	_cdecl_ = (_tmp20_ = vala_ccode_declaration_new ("DBusError"), _vala_ccode_node_unref0 (_cdecl_), _tmp20_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp21_ = vala_ccode_variable_declarator_new ("_dbus_error", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	dbus_error = (_tmp23_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("_dbus_error"))), _vala_ccode_node_unref0 (_tmp22_), _tmp23_);
	_cdecl_ = (_tmp24_ = vala_ccode_declaration_new ("DBusGConnection"), _vala_ccode_node_unref0 (_cdecl_), _tmp24_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp25_ = vala_ccode_variable_declarator_new ("*_connection", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp26_ = vala_ccode_declaration_new ("DBusMessage"), _vala_ccode_node_unref0 (_cdecl_), _tmp26_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp27_ = vala_ccode_variable_declarator_new ("*_message", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp27_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp28_ = vala_ccode_variable_declarator_new ("*_reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp28_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp29_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp29_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp30_ = vala_ccode_variable_declarator_new ("_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp30_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	vala_dbus_client_module_generate_marshalling (self, m, dbus_iface_name, prefragment, postfragment);
	gconnection = (_tmp32_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (_tmp31_), _tmp32_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp33_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp34_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp34_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new ("_connection")))));
	_vala_ccode_node_unref0 (_tmp36_);
	_vala_ccode_node_unref0 (_tmp35_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp37_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp38_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) gconnection)));
	_vala_ccode_node_unref0 (_tmp38_);
	dbus_error_init = (_tmp40_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new ("dbus_error_init"))), _vala_ccode_node_unref0 (_tmp39_), _tmp40_);
	vala_ccode_function_call_add_argument (dbus_error_init, (ValaCCodeExpression*) dbus_error);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp41_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) dbus_error_init)));
	_vala_ccode_node_unref0 (_tmp41_);
	connection = (_tmp43_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("dbus_g_connection_get_connection"))), _vala_ccode_node_unref0 (_tmp42_), _tmp43_);
	vala_ccode_function_call_add_argument (connection, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp44_);
	ccall = (_tmp46_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("dbus_connection_send_with_reply_and_block"))), _vala_ccode_node_unref0 (_tmp45_), _tmp46_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) connection);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp47_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp48_ = vala_ccode_constant_new ("-1")));
	_vala_ccode_node_unref0 (_tmp48_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) dbus_error);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp51_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp50_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("_reply")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp51_);
	_vala_ccode_node_unref0 (_tmp50_);
	_vala_ccode_node_unref0 (_tmp49_);
	conn_unref = (_tmp53_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp52_ = vala_ccode_identifier_new ("dbus_g_connection_unref"))), _vala_ccode_node_unref0 (_tmp52_), _tmp53_);
	vala_ccode_function_call_add_argument (conn_unref, (ValaCCodeExpression*) (_tmp54_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp54_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp55_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) conn_unref)));
	_vala_ccode_node_unref0 (_tmp55_);
	message_unref = (_tmp57_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp56_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp56_), _tmp57_);
	vala_ccode_function_call_add_argument (message_unref, (ValaCCodeExpression*) (_tmp58_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp58_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp59_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) message_unref)));
	_vala_ccode_node_unref0 (_tmp59_);
	error_types = vala_code_node_get_error_types ((ValaCodeNode*) m);
	if (!vala_dbus_client_module_has_dbus_error (self, error_types)) {
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) m), "D-Bus methods must throw DBus.Error");
	} else {
		ValaCCodeBlock* error_block;
		ValaCCodeDeclaration* _tmp60_;
		ValaCCodeVariableDeclarator* _tmp61_;
		ValaCCodeDeclaration* _tmp62_;
		ValaCCodeVariableDeclarator* _tmp63_;
		ValaCCodeIdentifier* _tmp67_;
		ValaCCodeIdentifier* _tmp66_;
		ValaCCodeMemberAccess* _tmp65_;
		ValaCCodeIdentifier* _tmp64_;
		ValaCCodeIdentifier* _tmp68_;
		ValaCCodeFunctionCall* _tmp69_;
		ValaCCodeFunctionCall* g_set_error;
		ValaCCodeIdentifier* _tmp70_;
		ValaCCodeIdentifier* _tmp71_;
		ValaCCodeIdentifier* _tmp72_;
		ValaCCodeMemberAccess* _tmp74_;
		ValaCCodeIdentifier* _tmp73_;
		ValaCCodeExpressionStatement* _tmp75_;
		ValaCCodeIdentifier* _tmp76_;
		ValaCCodeFunctionCall* _tmp77_;
		ValaCCodeFunctionCall* dbus_error_free;
		ValaCCodeExpressionStatement* _tmp78_;
		ValaCCodeIdentifier* _tmp81_;
		ValaCCodeFunctionCall* _tmp82_;
		ValaCCodeFunctionCall* dbus_error_is_set;
		ValaCCodeIfStatement* _tmp83_;
		error_block = vala_ccode_block_new ();
		_cdecl_ = (_tmp60_ = vala_ccode_declaration_new ("GQuark"), _vala_ccode_node_unref0 (_cdecl_), _tmp60_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp61_ = vala_ccode_variable_declarator_new ("_edomain", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp61_);
		vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) _cdecl_);
		_cdecl_ = (_tmp62_ = vala_ccode_declaration_new ("gint"), _vala_ccode_node_unref0 (_cdecl_), _tmp62_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp63_ = vala_ccode_variable_declarator_new ("_ecode", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp63_);
		vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) _cdecl_);
		vala_dbus_client_module_generate_client_error_cases (self, error_block, error_types, (ValaCCodeExpression*) (_tmp65_ = vala_ccode_member_access_new ((ValaCCodeExpression*) (_tmp64_ = vala_ccode_identifier_new ("_dbus_error")), "name", FALSE)), (ValaCCodeExpression*) (_tmp66_ = vala_ccode_identifier_new ("_edomain")), (ValaCCodeExpression*) (_tmp67_ = vala_ccode_identifier_new ("_ecode")));
		_vala_ccode_node_unref0 (_tmp67_);
		_vala_ccode_node_unref0 (_tmp66_);
		_vala_ccode_node_unref0 (_tmp65_);
		_vala_ccode_node_unref0 (_tmp64_);
		g_set_error = (_tmp69_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp68_ = vala_ccode_identifier_new ("g_set_error"))), _vala_ccode_node_unref0 (_tmp68_), _tmp69_);
		vala_ccode_function_call_add_argument (g_set_error, (ValaCCodeExpression*) (_tmp70_ = vala_ccode_identifier_new ("error")));
		_vala_ccode_node_unref0 (_tmp70_);
		vala_ccode_function_call_add_argument (g_set_error, (ValaCCodeExpression*) (_tmp71_ = vala_ccode_identifier_new ("_edomain")));
		_vala_ccode_node_unref0 (_tmp71_);
		vala_ccode_function_call_add_argument (g_set_error, (ValaCCodeExpression*) (_tmp72_ = vala_ccode_identifier_new ("_ecode")));
		_vala_ccode_node_unref0 (_tmp72_);
		vala_ccode_function_call_add_argument (g_set_error, (ValaCCodeExpression*) (_tmp74_ = vala_ccode_member_access_new ((ValaCCodeExpression*) (_tmp73_ = vala_ccode_identifier_new ("_dbus_error")), "message", FALSE)));
		_vala_ccode_node_unref0 (_tmp74_);
		_vala_ccode_node_unref0 (_tmp73_);
		vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) (_tmp75_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) g_set_error)));
		_vala_ccode_node_unref0 (_tmp75_);
		dbus_error_free = (_tmp77_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp76_ = vala_ccode_identifier_new ("dbus_error_free"))), _vala_ccode_node_unref0 (_tmp76_), _tmp77_);
		vala_ccode_function_call_add_argument (dbus_error_free, (ValaCCodeExpression*) dbus_error);
		vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) (_tmp78_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) dbus_error_free)));
		_vala_ccode_node_unref0 (_tmp78_);
		if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
			ValaCCodeReturnStatement* _tmp80_;
			ValaCCodeIdentifier* _tmp79_;
			vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) (_tmp80_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp79_ = vala_ccode_identifier_new ("_result")))));
			_vala_ccode_node_unref0 (_tmp80_);
			_vala_ccode_node_unref0 (_tmp79_);
		}
		dbus_error_is_set = (_tmp82_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp81_ = vala_ccode_identifier_new ("dbus_error_is_set"))), _vala_ccode_node_unref0 (_tmp81_), _tmp82_);
		vala_ccode_function_call_add_argument (dbus_error_is_set, (ValaCCodeExpression*) dbus_error);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp83_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) dbus_error_is_set, (ValaCCodeStatement*) error_block, NULL)));
		_vala_ccode_node_unref0 (_tmp83_);
		_vala_ccode_node_unref0 (error_block);
		_vala_ccode_node_unref0 (g_set_error);
		_vala_ccode_node_unref0 (dbus_error_free);
		_vala_ccode_node_unref0 (dbus_error_is_set);
	}
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) postfragment);
	reply_unref = (_tmp85_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp84_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp84_), _tmp85_);
	vala_ccode_function_call_add_argument (reply_unref, (ValaCCodeExpression*) (_tmp86_ = vala_ccode_identifier_new ("_reply")));
	_vala_ccode_node_unref0 (_tmp86_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp87_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) reply_unref)));
	_vala_ccode_node_unref0 (_tmp87_);
	if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
		ValaCCodeReturnStatement* _tmp89_;
		ValaCCodeIdentifier* _tmp88_;
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp89_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp88_ = vala_ccode_identifier_new ("_result")))));
		_vala_ccode_node_unref0 (_tmp89_);
		_vala_ccode_node_unref0 (_tmp88_);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp90_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp90_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = proxy_name;
	_g_free0 (dbus_iface_name);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_gee_collection_object_unref0 (cparam_map);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (postfragment);
	_vala_ccode_node_unref0 (dispose_return_block);
	_vala_ccode_node_unref0 (dbus_error);
	_vala_ccode_node_unref0 (gconnection);
	_vala_ccode_node_unref0 (dbus_error_init);
	_vala_ccode_node_unref0 (connection);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (conn_unref);
	_vala_ccode_node_unref0 (message_unref);
	_gee_collection_object_unref0 (error_types);
	_vala_ccode_node_unref0 (reply_unref);
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static void vala_dbus_client_module_generate_client_error_cases (ValaDBusClientModule* self, ValaCCodeBlock* error_block, GeeList* error_types, ValaCCodeExpression* dbus_error_name, ValaCCodeExpression* result_edomain, ValaCCodeExpression* result_ecode) {
	ValaCCodeStatement* if_else_if;
	ValaCCodeIfStatement* last_statement;
	g_return_if_fail (self != NULL);
	g_return_if_fail (error_block != NULL);
	g_return_if_fail (error_types != NULL);
	g_return_if_fail (dbus_error_name != NULL);
	g_return_if_fail (result_edomain != NULL);
	g_return_if_fail (result_ecode != NULL);
	if_else_if = NULL;
	last_statement = NULL;
	{
		GeeIterator* _error_type_it;
		_error_type_it = gee_iterable_iterator ((GeeIterable*) error_types);
		while (TRUE) {
			ValaDataType* error_type;
			ValaErrorDomain* edomain;
			char* edomain_dbus_name;
			ValaCCodeBlock* true_block;
			char* temp_name;
			ValaCCodeDeclaration* _cdecl_;
			ValaCCodeVariableDeclarator* _tmp0_;
			ValaCCodeExpressionStatement* _tmp4_;
			ValaCCodeAssignment* _tmp3_;
			ValaCCodeIdentifier* _tmp2_;
			char* _tmp1_;
			ValaCCodeExpressionStatement* _tmp10_;
			ValaCCodeAssignment* _tmp9_;
			ValaCCodeBinaryExpression* _tmp8_;
			ValaCCodeConstant* _tmp7_;
			char* _tmp6_;
			ValaCCodeIdentifier* _tmp5_;
			ValaCCodeStatement* inner_if_else_if;
			ValaCCodeIfStatement* inner_last_statement;
			ValaCCodeIdentifier* _tmp29_;
			ValaCCodeFunctionCall* _tmp30_;
			ValaCCodeFunctionCall* string_comparison;
			ValaCCodeConstant* _tmp32_;
			char* _tmp31_;
			ValaCCodeBinaryExpression* _tmp33_;
			ValaCCodeIfStatement* _tmp34_;
			ValaCCodeIfStatement* stmt;
			ValaCCodeIfStatement* _tmp36_;
			if (!gee_iterator_next (_error_type_it)) {
				break;
			}
			error_type = (ValaDataType*) gee_iterator_get (_error_type_it);
			edomain = _vala_code_node_ref0 (vala_error_type_get_error_domain (VALA_ERROR_TYPE (error_type)));
			edomain_dbus_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) edomain);
			if (edomain_dbus_name == NULL) {
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) edomain), "Errordomain must have a DBus.name annotation to be serialized over DBus");
			}
			true_block = vala_ccode_block_new ();
			vala_ccode_block_set_suppress_newline (true_block, TRUE);
			temp_name = g_strdup_printf ("_tmp%d_", ((ValaCCodeBaseModule*) self)->next_temp_var_id++);
			_cdecl_ = vala_ccode_declaration_new ("const char*");
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp0_ = vala_ccode_variable_declarator_new (temp_name, NULL, NULL)));
			_vala_ccode_node_unref0 (_tmp0_);
			vala_ccode_block_add_statement (true_block, (ValaCCodeNode*) _cdecl_);
			vala_ccode_block_add_statement (true_block, (ValaCCodeNode*) (_tmp4_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp3_ = vala_ccode_assignment_new (result_edomain, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new (_tmp1_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) edomain, NULL))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp4_);
			_vala_ccode_node_unref0 (_tmp3_);
			_vala_ccode_node_unref0 (_tmp2_);
			_g_free0 (_tmp1_);
			vala_ccode_block_add_statement (true_block, (ValaCCodeNode*) (_tmp10_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp9_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new (temp_name)), (ValaCCodeExpression*) (_tmp8_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_PLUS, dbus_error_name, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_constant_new (_tmp6_ = g_strdup_printf ("%ld", string_get_length (edomain_dbus_name) + 1))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp10_);
			_vala_ccode_node_unref0 (_tmp9_);
			_vala_ccode_node_unref0 (_tmp8_);
			_vala_ccode_node_unref0 (_tmp7_);
			_g_free0 (_tmp6_);
			_vala_ccode_node_unref0 (_tmp5_);
			inner_if_else_if = NULL;
			inner_last_statement = NULL;
			{
				GeeList* _tmp11_;
				GeeIterator* _tmp12_;
				GeeIterator* _ecode_it;
				_ecode_it = (_tmp12_ = gee_iterable_iterator ((GeeIterable*) (_tmp11_ = vala_error_domain_get_codes (edomain))), _gee_collection_object_unref0 (_tmp11_), _tmp12_);
				while (TRUE) {
					ValaErrorCode* ecode;
					ValaCCodeBlock* inner_true_block;
					ValaCCodeExpressionStatement* _tmp16_;
					ValaCCodeAssignment* _tmp15_;
					ValaCCodeIdentifier* _tmp14_;
					char* _tmp13_;
					char* ecode_dbus_name;
					ValaCCodeIdentifier* _tmp19_;
					ValaCCodeFunctionCall* _tmp20_;
					ValaCCodeFunctionCall* string_comparison;
					ValaCCodeIdentifier* _tmp21_;
					ValaCCodeConstant* _tmp23_;
					char* _tmp22_;
					ValaCCodeBinaryExpression* _tmp25_;
					ValaCCodeConstant* _tmp24_;
					ValaCCodeIfStatement* _tmp26_;
					ValaCCodeIfStatement* stmt;
					ValaCCodeIfStatement* _tmp28_;
					if (!gee_iterator_next (_ecode_it)) {
						break;
					}
					ecode = (ValaErrorCode*) gee_iterator_get (_ecode_it);
					inner_true_block = vala_ccode_block_new ();
					vala_ccode_block_set_suppress_newline (inner_true_block, TRUE);
					vala_ccode_block_add_statement (inner_true_block, (ValaCCodeNode*) (_tmp16_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp15_ = vala_ccode_assignment_new (result_ecode, (ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new (_tmp13_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) ecode, FALSE))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
					_vala_ccode_node_unref0 (_tmp16_);
					_vala_ccode_node_unref0 (_tmp15_);
					_vala_ccode_node_unref0 (_tmp14_);
					_g_free0 (_tmp13_);
					ecode_dbus_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) ecode);
					if (ecode_dbus_name == NULL) {
						char* _tmp18_;
						char* _tmp17_;
						ecode_dbus_name = (_tmp18_ = vala_symbol_lower_case_to_camel_case (_tmp17_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) ecode), -1)), _g_free0 (ecode_dbus_name), _tmp18_);
						_g_free0 (_tmp17_);
					}
					string_comparison = (_tmp20_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp19_), _tmp20_);
					vala_ccode_function_call_add_argument (string_comparison, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new (temp_name)));
					_vala_ccode_node_unref0 (_tmp21_);
					vala_ccode_function_call_add_argument (string_comparison, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_constant_new (_tmp22_ = g_strdup_printf ("\"%s\"", ecode_dbus_name))));
					_vala_ccode_node_unref0 (_tmp23_);
					_g_free0 (_tmp22_);
					stmt = (_tmp26_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp25_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) string_comparison, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_constant_new ("0")))), (ValaCCodeStatement*) inner_true_block, NULL), _vala_ccode_node_unref0 (_tmp25_), _vala_ccode_node_unref0 (_tmp24_), _tmp26_);
					if (inner_last_statement != NULL) {
						vala_ccode_if_statement_set_false_statement (inner_last_statement, (ValaCCodeStatement*) stmt);
					} else {
						ValaCCodeStatement* _tmp27_;
						inner_if_else_if = (_tmp27_ = _vala_ccode_node_ref0 ((ValaCCodeStatement*) stmt), _vala_ccode_node_unref0 (inner_if_else_if), _tmp27_);
					}
					inner_last_statement = (_tmp28_ = _vala_ccode_node_ref0 (stmt), _vala_ccode_node_unref0 (inner_last_statement), _tmp28_);
					_vala_code_node_unref0 (ecode);
					_vala_ccode_node_unref0 (inner_true_block);
					_g_free0 (ecode_dbus_name);
					_vala_ccode_node_unref0 (string_comparison);
					_vala_ccode_node_unref0 (stmt);
				}
				_gee_collection_object_unref0 (_ecode_it);
			}
			vala_ccode_block_add_statement (true_block, (ValaCCodeNode*) inner_if_else_if);
			string_comparison = (_tmp30_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("strstr"))), _vala_ccode_node_unref0 (_tmp29_), _tmp30_);
			vala_ccode_function_call_add_argument (string_comparison, dbus_error_name);
			vala_ccode_function_call_add_argument (string_comparison, (ValaCCodeExpression*) (_tmp32_ = vala_ccode_constant_new (_tmp31_ = g_strdup_printf ("\"%s\"", edomain_dbus_name))));
			_vala_ccode_node_unref0 (_tmp32_);
			_g_free0 (_tmp31_);
			stmt = (_tmp34_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp33_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) string_comparison, dbus_error_name)), (ValaCCodeStatement*) true_block, NULL), _vala_ccode_node_unref0 (_tmp33_), _tmp34_);
			if (last_statement != NULL) {
				vala_ccode_if_statement_set_false_statement (last_statement, (ValaCCodeStatement*) stmt);
			} else {
				ValaCCodeStatement* _tmp35_;
				if_else_if = (_tmp35_ = _vala_ccode_node_ref0 ((ValaCCodeStatement*) stmt), _vala_ccode_node_unref0 (if_else_if), _tmp35_);
			}
			last_statement = (_tmp36_ = _vala_ccode_node_ref0 (stmt), _vala_ccode_node_unref0 (last_statement), _tmp36_);
			_vala_code_node_unref0 (error_type);
			_vala_code_node_unref0 (edomain);
			_g_free0 (edomain_dbus_name);
			_vala_ccode_node_unref0 (true_block);
			_g_free0 (temp_name);
			_vala_ccode_node_unref0 (_cdecl_);
			_vala_ccode_node_unref0 (inner_if_else_if);
			_vala_ccode_node_unref0 (inner_last_statement);
			_vala_ccode_node_unref0 (string_comparison);
			_vala_ccode_node_unref0 (stmt);
		}
		_gee_collection_object_unref0 (_error_type_it);
	}
	vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) if_else_if);
	_vala_ccode_node_unref0 (if_else_if);
	_vala_ccode_node_unref0 (last_statement);
}


static char* vala_dbus_client_module_generate_async_dbus_proxy_method (ValaDBusClientModule* self, ValaInterface* iface, ValaMethod* m) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* proxy_name;
	char* dbus_iface_name;
	ValaCCodeDeclaration* _cdecl_;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp4_;
	char* dataname;
	char* _tmp5_;
	ValaCCodeStruct* _tmp6_;
	ValaCCodeStruct* datastruct;
	ValaCCodeTypeDefinition* _tmp9_;
	ValaCCodeVariableDeclarator* _tmp8_;
	char* _tmp7_;
	ValaCCodeFunction* function;
	GeeHashMap* cparam_map;
	ValaCCodeFormalParameter* _tmp10_;
	ValaCCodeFormalParameter* _tmp11_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeFragment* postfragment;
	ValaCCodeDeclaration* _tmp12_;
	ValaCCodeVariableDeclarator* _tmp13_;
	ValaCCodeDeclaration* _tmp14_;
	ValaCCodeVariableDeclarator* _tmp15_;
	ValaCCodeDeclaration* _tmp16_;
	ValaCCodeVariableDeclarator* _tmp17_;
	ValaCCodeDeclaration* _tmp18_;
	ValaCCodeVariableDeclarator* _tmp19_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeFunctionCall* _tmp21_;
	ValaCCodeFunctionCall* gconnection;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeConstant* _tmp23_;
	ValaCCodeUnaryExpression* _tmp25_;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeConstant* _tmp26_;
	ValaCCodeExpressionStatement* _tmp27_;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeFunctionCall* _tmp29_;
	ValaCCodeFunctionCall* connection;
	ValaCCodeIdentifier* _tmp30_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeFunctionCall* _tmp32_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp33_;
	ValaCCodeUnaryExpression* _tmp35_;
	ValaCCodeIdentifier* _tmp34_;
	ValaCCodeConstant* _tmp36_;
	ValaCCodeExpressionStatement* _tmp37_;
	ValaCCodeIdentifier* _tmp38_;
	ValaCCodeFunctionCall* _tmp39_;
	ValaCCodeFunctionCall* conn_unref;
	ValaCCodeIdentifier* _tmp40_;
	ValaCCodeExpressionStatement* _tmp41_;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeFunctionCall* _tmp43_;
	ValaCCodeFunctionCall* message_unref;
	ValaCCodeIdentifier* _tmp44_;
	ValaCCodeExpressionStatement* _tmp45_;
	ValaCCodeIdentifier* _tmp46_;
	ValaCCodeFunctionCall* _tmp47_;
	ValaCCodeFunctionCall* dataalloc;
	ValaCCodeIdentifier* _tmp48_;
	char* _tmp49_;
	ValaCCodeDeclaration* _tmp50_;
	ValaCCodeDeclaration* datadecl;
	ValaCCodeVariableDeclarator* _tmp51_;
	ValaCCodeExpressionStatement* _tmp54_;
	ValaCCodeAssignment* _tmp53_;
	ValaCCodeIdentifier* _tmp52_;
	ValaCCodeExpressionStatement* _tmp59_;
	ValaCCodeAssignment* _tmp58_;
	ValaCCodeIdentifier* _tmp57_;
	ValaCCodeMemberAccess* _tmp56_;
	ValaCCodeIdentifier* _tmp55_;
	ValaCCodeExpressionStatement* _tmp64_;
	ValaCCodeAssignment* _tmp63_;
	ValaCCodeIdentifier* _tmp62_;
	ValaCCodeMemberAccess* _tmp61_;
	ValaCCodeIdentifier* _tmp60_;
	ValaCCodeExpressionStatement* _tmp69_;
	ValaCCodeAssignment* _tmp68_;
	ValaCCodeIdentifier* _tmp67_;
	ValaCCodeMemberAccess* _tmp66_;
	ValaCCodeIdentifier* _tmp65_;
	ValaCCodeIdentifier* _tmp70_;
	ValaCCodeFunctionCall* _tmp71_;
	ValaCCodeFunctionCall* pending;
	ValaCCodeIdentifier* _tmp72_;
	ValaCCodeIdentifier* _tmp75_;
	char* _tmp74_;
	char* _tmp73_;
	ValaCCodeIdentifier* _tmp76_;
	ValaCCodeConstant* _tmp77_;
	ValaCCodeExpressionStatement* _tmp78_;
	ValaCCodeFunction* _tmp79_;
	ValaCCodeFunction* _tmp82_;
	char* _tmp81_;
	char* _tmp80_;
	ValaCCodeFormalParameter* _tmp83_;
	ValaCCodeFormalParameter* _tmp84_;
	ValaCCodeBlock* _tmp85_;
	ValaCCodeDeclaration* _tmp87_;
	char* _tmp86_;
	ValaCCodeVariableDeclarator* _tmp88_;
	ValaCCodeExpressionStatement* _tmp92_;
	ValaCCodeAssignment* _tmp91_;
	ValaCCodeIdentifier* _tmp90_;
	ValaCCodeIdentifier* _tmp89_;
	ValaCCodeIdentifier* _tmp93_;
	ValaCCodeFunctionCall* _tmp94_;
	ValaCCodeFunctionCall* object_creation;
	ValaCCodeConstant* _tmp95_;
	ValaCCodeConstant* _tmp96_;
	ValaCCodeConstant* _tmp97_;
	ValaCCodeIdentifier* _tmp98_;
	ValaCCodeFunctionCall* _tmp99_;
	ValaCCodeFunctionCall* async_result_creation;
	ValaCCodeMemberAccess* _tmp101_;
	ValaCCodeIdentifier* _tmp100_;
	ValaCCodeMemberAccess* _tmp103_;
	ValaCCodeIdentifier* _tmp102_;
	ValaCCodeIdentifier* _tmp104_;
	ValaCCodeIdentifier* _tmp105_;
	ValaCCodeFunctionCall* _tmp106_;
	ValaCCodeFunctionCall* completecall;
	ValaCCodeExpressionStatement* _tmp107_;
	ValaCCodeFunction* _tmp108_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (iface != NULL, NULL);
	g_return_val_if_fail (m != NULL, NULL);
	proxy_name = (_tmp1_ = g_strdup_printf ("%sdbus_proxy_%s_async", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), vala_symbol_get_name ((ValaSymbol*) m)), _g_free0 (_tmp0_), _tmp1_);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	_cdecl_ = NULL;
	dataname = (_tmp4_ = g_strdup_printf ("%sDBusProxy%sData", _tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), _tmp3_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) m))), _g_free0 (_tmp3_), _g_free0 (_tmp2_), _tmp4_);
	datastruct = (_tmp6_ = vala_ccode_struct_new (_tmp5_ = g_strconcat ("_", dataname, NULL)), _g_free0 (_tmp5_), _tmp6_);
	vala_ccode_struct_add_field (datastruct, "GAsyncReadyCallback", "_callback_");
	vala_ccode_struct_add_field (datastruct, "gpointer", "_user_data_");
	vala_ccode_struct_add_field (datastruct, "DBusPendingCall*", "pending");
	vala_ccode_declaration_space_add_type_definition (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) datastruct);
	vala_ccode_declaration_space_add_type_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp9_ = vala_ccode_type_definition_new (_tmp7_ = g_strconcat ("struct _", dataname, NULL), (ValaCCodeDeclarator*) (_tmp8_ = vala_ccode_variable_declarator_new (dataname, NULL, NULL)))));
	_vala_ccode_node_unref0 (_tmp9_);
	_vala_ccode_node_unref0 (_tmp8_);
	_g_free0 (_tmp7_);
	function = vala_ccode_function_new (proxy_name, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	cparam_map = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_FORMAL_PARAMETER, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal);
	gee_map_set ((GeeMap*) cparam_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), FALSE)), _tmp10_ = vala_ccode_formal_parameter_new ("_callback_", "GAsyncReadyCallback"));
	_vala_ccode_node_unref0 (_tmp10_);
	gee_map_set ((GeeMap*) cparam_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, -0.9, FALSE)), _tmp11_ = vala_ccode_formal_parameter_new ("_user_data_", "gpointer"));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_base_module_generate_cparameters ((ValaCCodeBaseModule*) self, m, ((ValaCCodeBaseModule*) self)->source_declarations, (GeeMap*) cparam_map, function, NULL, NULL, NULL, 1);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	postfragment = vala_ccode_fragment_new ();
	_cdecl_ = (_tmp12_ = vala_ccode_declaration_new ("DBusGConnection"), _vala_ccode_node_unref0 (_cdecl_), _tmp12_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp13_ = vala_ccode_variable_declarator_new ("*_connection", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp13_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp14_ = vala_ccode_declaration_new ("DBusMessage"), _vala_ccode_node_unref0 (_cdecl_), _tmp14_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp15_ = vala_ccode_variable_declarator_new ("*_message", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp15_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp16_ = vala_ccode_declaration_new ("DBusPendingCall"), _vala_ccode_node_unref0 (_cdecl_), _tmp16_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp17_ = vala_ccode_variable_declarator_new ("*_pending", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp17_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp18_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp18_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp19_ = vala_ccode_variable_declarator_new ("_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp19_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	vala_dbus_client_module_generate_marshalling (self, m, dbus_iface_name, prefragment, postfragment);
	gconnection = (_tmp21_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (_tmp20_), _tmp21_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("_connection")))));
	_vala_ccode_node_unref0 (_tmp25_);
	_vala_ccode_node_unref0 (_tmp24_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp26_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp27_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) gconnection)));
	_vala_ccode_node_unref0 (_tmp27_);
	connection = (_tmp29_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("dbus_g_connection_get_connection"))), _vala_ccode_node_unref0 (_tmp28_), _tmp29_);
	vala_ccode_function_call_add_argument (connection, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp30_);
	ccall = (_tmp32_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("dbus_connection_send_with_reply"))), _vala_ccode_node_unref0 (_tmp31_), _tmp32_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) connection);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp33_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp35_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("_pending")))));
	_vala_ccode_node_unref0 (_tmp35_);
	_vala_ccode_node_unref0 (_tmp34_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_constant_new ("-1")));
	_vala_ccode_node_unref0 (_tmp36_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp37_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp37_);
	conn_unref = (_tmp39_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp38_ = vala_ccode_identifier_new ("dbus_g_connection_unref"))), _vala_ccode_node_unref0 (_tmp38_), _tmp39_);
	vala_ccode_function_call_add_argument (conn_unref, (ValaCCodeExpression*) (_tmp40_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp40_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp41_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) conn_unref)));
	_vala_ccode_node_unref0 (_tmp41_);
	message_unref = (_tmp43_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp42_), _tmp43_);
	vala_ccode_function_call_add_argument (message_unref, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp44_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp45_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) message_unref)));
	_vala_ccode_node_unref0 (_tmp45_);
	dataalloc = (_tmp47_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp46_ = vala_ccode_identifier_new ("g_slice_new0"))), _vala_ccode_node_unref0 (_tmp46_), _tmp47_);
	vala_ccode_function_call_add_argument (dataalloc, (ValaCCodeExpression*) (_tmp48_ = vala_ccode_identifier_new (dataname)));
	_vala_ccode_node_unref0 (_tmp48_);
	datadecl = (_tmp50_ = vala_ccode_declaration_new (_tmp49_ = g_strconcat (dataname, "*", NULL)), _g_free0 (_tmp49_), _tmp50_);
	vala_ccode_declaration_add_declarator (datadecl, (ValaCCodeDeclarator*) (_tmp51_ = vala_ccode_variable_declarator_new ("_data_", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp51_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) datadecl);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp54_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp53_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp52_ = vala_ccode_identifier_new ("_data_")), (ValaCCodeExpression*) dataalloc, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp54_);
	_vala_ccode_node_unref0 (_tmp53_);
	_vala_ccode_node_unref0 (_tmp52_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp59_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp58_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp56_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp55_ = vala_ccode_identifier_new ("_data_")), "_callback_")), (ValaCCodeExpression*) (_tmp57_ = vala_ccode_identifier_new ("_callback_")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp59_);
	_vala_ccode_node_unref0 (_tmp58_);
	_vala_ccode_node_unref0 (_tmp57_);
	_vala_ccode_node_unref0 (_tmp56_);
	_vala_ccode_node_unref0 (_tmp55_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp64_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp63_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp61_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new ("_data_")), "_user_data_")), (ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new ("_user_data_")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp64_);
	_vala_ccode_node_unref0 (_tmp63_);
	_vala_ccode_node_unref0 (_tmp62_);
	_vala_ccode_node_unref0 (_tmp61_);
	_vala_ccode_node_unref0 (_tmp60_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp69_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp68_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp66_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp65_ = vala_ccode_identifier_new ("_data_")), "pending")), (ValaCCodeExpression*) (_tmp67_ = vala_ccode_identifier_new ("_pending")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp69_);
	_vala_ccode_node_unref0 (_tmp68_);
	_vala_ccode_node_unref0 (_tmp67_);
	_vala_ccode_node_unref0 (_tmp66_);
	_vala_ccode_node_unref0 (_tmp65_);
	pending = (_tmp71_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp70_ = vala_ccode_identifier_new ("dbus_pending_call_set_notify"))), _vala_ccode_node_unref0 (_tmp70_), _tmp71_);
	vala_ccode_function_call_add_argument (pending, (ValaCCodeExpression*) (_tmp72_ = vala_ccode_identifier_new ("_pending")));
	_vala_ccode_node_unref0 (_tmp72_);
	vala_ccode_function_call_add_argument (pending, (ValaCCodeExpression*) (_tmp75_ = vala_ccode_identifier_new (_tmp74_ = g_strdup_printf ("%sdbus_proxy_%s_ready", _tmp73_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), vala_symbol_get_name ((ValaSymbol*) m)))));
	_vala_ccode_node_unref0 (_tmp75_);
	_g_free0 (_tmp74_);
	_g_free0 (_tmp73_);
	vala_ccode_function_call_add_argument (pending, (ValaCCodeExpression*) (_tmp76_ = vala_ccode_identifier_new ("_data_")));
	_vala_ccode_node_unref0 (_tmp76_);
	vala_ccode_function_call_add_argument (pending, (ValaCCodeExpression*) (_tmp77_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp77_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp78_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) pending)));
	_vala_ccode_node_unref0 (_tmp78_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp79_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp79_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	function = (_tmp82_ = vala_ccode_function_new (_tmp81_ = g_strdup_printf ("%sdbus_proxy_%s_ready", _tmp80_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), vala_symbol_get_name ((ValaSymbol*) m)), "void"), _vala_ccode_node_unref0 (function), _tmp82_);
	_g_free0 (_tmp81_);
	_g_free0 (_tmp80_);
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp83_ = vala_ccode_formal_parameter_new ("pending", "DBusPendingCall*"));
	_vala_ccode_node_unref0 (_tmp83_);
	vala_ccode_function_add_parameter (function, _tmp84_ = vala_ccode_formal_parameter_new ("user_data", "void*"));
	_vala_ccode_node_unref0 (_tmp84_);
	block = (_tmp85_ = vala_ccode_block_new (), _vala_ccode_node_unref0 (block), _tmp85_);
	datadecl = (_tmp87_ = vala_ccode_declaration_new (_tmp86_ = g_strconcat (dataname, "*", NULL)), _vala_ccode_node_unref0 (datadecl), _tmp87_);
	_g_free0 (_tmp86_);
	vala_ccode_declaration_add_declarator (datadecl, (ValaCCodeDeclarator*) (_tmp88_ = vala_ccode_variable_declarator_new ("_data_", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp88_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) datadecl);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp92_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp91_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp89_ = vala_ccode_identifier_new ("_data_")), (ValaCCodeExpression*) (_tmp90_ = vala_ccode_identifier_new ("user_data")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp92_);
	_vala_ccode_node_unref0 (_tmp91_);
	_vala_ccode_node_unref0 (_tmp90_);
	_vala_ccode_node_unref0 (_tmp89_);
	object_creation = (_tmp94_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp93_ = vala_ccode_identifier_new ("g_object_newv"))), _vala_ccode_node_unref0 (_tmp93_), _tmp94_);
	vala_ccode_function_call_add_argument (object_creation, (ValaCCodeExpression*) (_tmp95_ = vala_ccode_constant_new ("G_TYPE_OBJECT")));
	_vala_ccode_node_unref0 (_tmp95_);
	vala_ccode_function_call_add_argument (object_creation, (ValaCCodeExpression*) (_tmp96_ = vala_ccode_constant_new ("0")));
	_vala_ccode_node_unref0 (_tmp96_);
	vala_ccode_function_call_add_argument (object_creation, (ValaCCodeExpression*) (_tmp97_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp97_);
	async_result_creation = (_tmp99_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp98_ = vala_ccode_identifier_new ("g_simple_async_result_new"))), _vala_ccode_node_unref0 (_tmp98_), _tmp99_);
	vala_ccode_function_call_add_argument (async_result_creation, (ValaCCodeExpression*) object_creation);
	vala_ccode_function_call_add_argument (async_result_creation, (ValaCCodeExpression*) (_tmp101_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp100_ = vala_ccode_identifier_new ("_data_")), "_callback_")));
	_vala_ccode_node_unref0 (_tmp101_);
	_vala_ccode_node_unref0 (_tmp100_);
	vala_ccode_function_call_add_argument (async_result_creation, (ValaCCodeExpression*) (_tmp103_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp102_ = vala_ccode_identifier_new ("_data_")), "_user_data_")));
	_vala_ccode_node_unref0 (_tmp103_);
	_vala_ccode_node_unref0 (_tmp102_);
	vala_ccode_function_call_add_argument (async_result_creation, (ValaCCodeExpression*) (_tmp104_ = vala_ccode_identifier_new ("_data_")));
	_vala_ccode_node_unref0 (_tmp104_);
	completecall = (_tmp106_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp105_ = vala_ccode_identifier_new ("g_simple_async_result_complete"))), _vala_ccode_node_unref0 (_tmp105_), _tmp106_);
	vala_ccode_function_call_add_argument (completecall, (ValaCCodeExpression*) async_result_creation);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp107_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) completecall)));
	_vala_ccode_node_unref0 (_tmp107_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp108_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp108_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = proxy_name;
	_g_free0 (dbus_iface_name);
	_vala_ccode_node_unref0 (_cdecl_);
	_g_free0 (dataname);
	_vala_ccode_node_unref0 (datastruct);
	_vala_ccode_node_unref0 (function);
	_gee_collection_object_unref0 (cparam_map);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (postfragment);
	_vala_ccode_node_unref0 (gconnection);
	_vala_ccode_node_unref0 (connection);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (conn_unref);
	_vala_ccode_node_unref0 (message_unref);
	_vala_ccode_node_unref0 (dataalloc);
	_vala_ccode_node_unref0 (datadecl);
	_vala_ccode_node_unref0 (pending);
	_vala_ccode_node_unref0 (object_creation);
	_vala_ccode_node_unref0 (async_result_creation);
	_vala_ccode_node_unref0 (completecall);
	return result;
}


static char* vala_dbus_client_module_generate_finish_dbus_proxy_method (ValaDBusClientModule* self, ValaInterface* iface, ValaMethod* m) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* proxy_name;
	char* dbus_iface_name;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	GeeHashMap* cparam_map;
	ValaCCodeFormalParameter* _tmp2_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeFragment* postfragment;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp5_;
	char* dataname;
	ValaCCodeDeclaration* _tmp7_;
	char* _tmp6_;
	ValaCCodeVariableDeclarator* _tmp8_;
	ValaCCodeDeclaration* _tmp9_;
	ValaCCodeVariableDeclarator* _tmp10_;
	ValaCCodeDeclaration* _tmp11_;
	ValaCCodeVariableDeclarator* _tmp12_;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeFunctionCall* _tmp14_;
	ValaCCodeFunctionCall* get_source_tag;
	ValaCCodeCastExpression* _tmp16_;
	ValaCCodeIdentifier* _tmp15_;
	ValaCCodeExpressionStatement* _tmp19_;
	ValaCCodeAssignment* _tmp18_;
	ValaCCodeIdentifier* _tmp17_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeFunctionCall* _tmp21_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeMemberAccess* _tmp23_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeExpressionStatement* _tmp26_;
	ValaCCodeAssignment* _tmp25_;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeIdentifier* _tmp27_;
	ValaCCodeFunctionCall* _tmp28_;
	ValaCCodeFunctionCall* reply_unref;
	ValaCCodeIdentifier* _tmp29_;
	ValaCCodeExpressionStatement* _tmp30_;
	ValaCCodeFunction* _tmp33_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (iface != NULL, NULL);
	g_return_val_if_fail (m != NULL, NULL);
	proxy_name = (_tmp1_ = g_strdup_printf ("%sdbus_proxy_%s_finish", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), vala_symbol_get_name ((ValaSymbol*) m)), _g_free0 (_tmp0_), _tmp1_);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	_cdecl_ = NULL;
	function = vala_ccode_function_new (proxy_name, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	cparam_map = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_FORMAL_PARAMETER, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal);
	gee_map_set ((GeeMap*) cparam_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, 0.1, FALSE)), _tmp2_ = vala_ccode_formal_parameter_new ("_res_", "GAsyncResult*"));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_base_module_generate_cparameters ((ValaCCodeBaseModule*) self, m, ((ValaCCodeBaseModule*) self)->source_declarations, (GeeMap*) cparam_map, function, NULL, NULL, NULL, 2);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	postfragment = vala_ccode_fragment_new ();
	dataname = (_tmp5_ = g_strdup_printf ("%sDBusProxy%sData", _tmp3_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), _tmp4_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) m))), _g_free0 (_tmp4_), _g_free0 (_tmp3_), _tmp5_);
	_cdecl_ = (_tmp7_ = vala_ccode_declaration_new (_tmp6_ = g_strconcat (dataname, "*", NULL)), _vala_ccode_node_unref0 (_cdecl_), _tmp7_);
	_g_free0 (_tmp6_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp8_ = vala_ccode_variable_declarator_new ("_data_", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp8_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp9_ = vala_ccode_declaration_new ("DBusMessage"), _vala_ccode_node_unref0 (_cdecl_), _tmp9_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp10_ = vala_ccode_variable_declarator_new ("*_reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp11_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp11_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp12_ = vala_ccode_variable_declarator_new ("_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	get_source_tag = (_tmp14_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("g_simple_async_result_get_source_tag"))), _vala_ccode_node_unref0 (_tmp13_), _tmp14_);
	vala_ccode_function_call_add_argument (get_source_tag, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("_res_")), "GSimpleAsyncResult *")));
	_vala_ccode_node_unref0 (_tmp16_);
	_vala_ccode_node_unref0 (_tmp15_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp19_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp18_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("_data_")), (ValaCCodeExpression*) get_source_tag, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp19_);
	_vala_ccode_node_unref0 (_tmp18_);
	_vala_ccode_node_unref0 (_tmp17_);
	ccall = (_tmp21_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("dbus_pending_call_steal_reply"))), _vala_ccode_node_unref0 (_tmp20_), _tmp21_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("_data_")), "pending")));
	_vala_ccode_node_unref0 (_tmp23_);
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp26_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp25_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("_reply")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp26_);
	_vala_ccode_node_unref0 (_tmp25_);
	_vala_ccode_node_unref0 (_tmp24_);
	vala_dbus_client_module_generate_marshalling (self, m, dbus_iface_name, prefragment, postfragment);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) postfragment);
	reply_unref = (_tmp28_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp27_), _tmp28_);
	vala_ccode_function_call_add_argument (reply_unref, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("_reply")));
	_vala_ccode_node_unref0 (_tmp29_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp30_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) reply_unref)));
	_vala_ccode_node_unref0 (_tmp30_);
	if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
		ValaCCodeReturnStatement* _tmp32_;
		ValaCCodeIdentifier* _tmp31_;
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp32_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("_result")))));
		_vala_ccode_node_unref0 (_tmp32_);
		_vala_ccode_node_unref0 (_tmp31_);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp33_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp33_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = proxy_name;
	_g_free0 (dbus_iface_name);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_gee_collection_object_unref0 (cparam_map);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (postfragment);
	_g_free0 (dataname);
	_vala_ccode_node_unref0 (get_source_tag);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (reply_unref);
	return result;
}


static char* vala_dbus_client_module_generate_dbus_proxy_property_get (ValaDBusClientModule* self, ValaInterface* iface, ValaProperty* prop) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* proxy_name;
	char* dbus_iface_name;
	ValaDataType* owned_type;
	gboolean _tmp2_;
	ValaDataType* _tmp3_;
	ValaArrayType* array_type;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	char* _tmp10_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeFragment* postfragment;
	ValaCCodeBlock* dispose_return_block;
	ValaCCodeReturnStatement* _tmp12_;
	ValaCCodeExpression* _tmp11_;
	ValaCCodeIfStatement* _tmp18_;
	ValaCCodeMemberAccess* _tmp17_;
	ValaCCodeCastExpression* _tmp16_;
	char* _tmp15_;
	char* _tmp14_;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeDeclaration* _tmp19_;
	ValaCCodeVariableDeclarator* _tmp20_;
	ValaCCodeDeclaration* _tmp21_;
	ValaCCodeVariableDeclarator* _tmp22_;
	ValaCCodeVariableDeclarator* _tmp23_;
	ValaCCodeDeclaration* _tmp24_;
	ValaCCodeVariableDeclarator* _tmp25_;
	ValaCCodeVariableDeclarator* _tmp26_;
	ValaCCodeIdentifier* _tmp27_;
	ValaCCodeFunctionCall* _tmp28_;
	ValaCCodeFunctionCall* destination;
	ValaCCodeCastExpression* _tmp30_;
	ValaCCodeIdentifier* _tmp29_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeFunctionCall* _tmp32_;
	ValaCCodeFunctionCall* path;
	ValaCCodeCastExpression* _tmp34_;
	ValaCCodeIdentifier* _tmp33_;
	ValaCCodeIdentifier* _tmp35_;
	ValaCCodeFunctionCall* _tmp36_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeConstant* _tmp37_;
	ValaCCodeConstant* _tmp38_;
	ValaCCodeExpressionStatement* _tmp41_;
	ValaCCodeAssignment* _tmp40_;
	ValaCCodeIdentifier* _tmp39_;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeFunctionCall* _tmp43_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp44_;
	ValaCCodeUnaryExpression* _tmp46_;
	ValaCCodeIdentifier* _tmp45_;
	ValaCCodeExpressionStatement* _tmp47_;
	ValaCCodeFunctionCall* _tmp49_;
	ValaCCodeIdentifier* _tmp48_;
	ValaCCodeIdentifier* _tmp50_;
	ValaCCodeUnaryExpression* _tmp52_;
	ValaCCodeIdentifier* _tmp51_;
	ValaCCodeExpressionStatement* _tmp53_;
	ValaCCodeConstant* _tmp56_;
	char* _tmp55_;
	ValaCCodeIdentifier* _tmp54_;
	ValaCCodeConstant* _tmp60_;
	char* _tmp59_;
	char* _tmp58_;
	ValaCCodeIdentifier* _tmp57_;
	ValaCCodeDeclaration* _tmp62_;
	char* _tmp61_;
	ValaCCodeVariableDeclarator* _tmp63_;
	ValaCCodeFunctionCall* _tmp70_;
	ValaCCodeIdentifier* _tmp69_;
	ValaCCodeUnaryExpression* _tmp72_;
	ValaCCodeIdentifier* _tmp71_;
	ValaCCodeUnaryExpression* _tmp74_;
	ValaCCodeIdentifier* _tmp73_;
	ValaCCodeExpressionStatement* _tmp75_;
	ValaCCodeIdentifier* target;
	ValaCCodeIdentifier* _tmp76_;
	ValaCCodeExpression* _tmp77_;
	ValaCCodeExpression* expr;
	ValaCCodeExpressionStatement* _tmp79_;
	ValaCCodeAssignment* _tmp78_;
	ValaCCodeIdentifier* _tmp88_;
	ValaCCodeFunctionCall* _tmp89_;
	ValaCCodeFunctionCall* gconnection;
	ValaCCodeIdentifier* _tmp90_;
	ValaCCodeConstant* _tmp91_;
	ValaCCodeUnaryExpression* _tmp93_;
	ValaCCodeIdentifier* _tmp92_;
	ValaCCodeConstant* _tmp94_;
	ValaCCodeExpressionStatement* _tmp95_;
	ValaCCodeIdentifier* _tmp96_;
	ValaCCodeFunctionCall* _tmp97_;
	ValaCCodeFunctionCall* connection;
	ValaCCodeIdentifier* _tmp98_;
	ValaCCodeIdentifier* _tmp99_;
	ValaCCodeFunctionCall* _tmp100_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp101_;
	ValaCCodeConstant* _tmp102_;
	ValaCCodeConstant* _tmp103_;
	ValaCCodeExpressionStatement* _tmp106_;
	ValaCCodeAssignment* _tmp105_;
	ValaCCodeIdentifier* _tmp104_;
	ValaCCodeIdentifier* _tmp107_;
	ValaCCodeFunctionCall* _tmp108_;
	ValaCCodeFunctionCall* conn_unref;
	ValaCCodeIdentifier* _tmp109_;
	ValaCCodeExpressionStatement* _tmp110_;
	ValaCCodeIdentifier* _tmp111_;
	ValaCCodeFunctionCall* _tmp112_;
	ValaCCodeFunctionCall* message_unref;
	ValaCCodeIdentifier* _tmp113_;
	ValaCCodeExpressionStatement* _tmp114_;
	ValaCCodeIdentifier* _tmp115_;
	ValaCCodeFunctionCall* _tmp116_;
	ValaCCodeFunctionCall* reply_unref;
	ValaCCodeIdentifier* _tmp117_;
	ValaCCodeExpressionStatement* _tmp118_;
	ValaCCodeReturnStatement* _tmp120_;
	ValaCCodeIdentifier* _tmp119_;
	ValaCCodeFunction* _tmp121_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (iface != NULL, NULL);
	g_return_val_if_fail (prop != NULL, NULL);
	proxy_name = (_tmp1_ = g_strdup_printf ("%sdbus_proxy_get_%s", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), vala_symbol_get_name ((ValaSymbol*) prop)), _g_free0 (_tmp0_), _tmp1_);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	owned_type = vala_data_type_copy (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)));
	vala_data_type_set_value_owned (owned_type, TRUE);
	_tmp2_ = FALSE;
	if (vala_data_type_is_disposable (owned_type)) {
		_tmp2_ = !vala_data_type_get_value_owned (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)));
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop))), "Properties used in D-Bus clients require owned get accessor");
	}
	array_type = _vala_code_node_ref0 ((_tmp3_ = vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)), VALA_IS_ARRAY_TYPE (_tmp3_) ? ((ValaArrayType*) _tmp3_) : NULL));
	_cdecl_ = NULL;
	function = vala_ccode_function_new (proxy_name, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp6_ = vala_ccode_formal_parameter_new ("self", _tmp5_ = g_strdup_printf ("%s*", _tmp4_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE))));
	_vala_ccode_node_unref0 (_tmp6_);
	_g_free0 (_tmp5_);
	_g_free0 (_tmp4_);
	if (array_type != NULL) {
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp7_;
				_tmp7_ = TRUE;
				while (TRUE) {
					ValaCCodeFormalParameter* _tmp9_;
					char* _tmp8_;
					if (!_tmp7_) {
						dim++;
					}
					_tmp7_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					vala_ccode_function_add_parameter (function, _tmp9_ = vala_ccode_formal_parameter_new (_tmp8_ = g_strdup_printf ("result_length%d", dim), "int*"));
					_vala_ccode_node_unref0 (_tmp9_);
					_g_free0 (_tmp8_);
				}
			}
		}
	}
	vala_ccode_function_set_return_type (function, _tmp10_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop))));
	_g_free0 (_tmp10_);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	postfragment = vala_ccode_fragment_new ();
	dispose_return_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (dispose_return_block, (ValaCCodeNode*) (_tmp12_ = vala_ccode_return_statement_new (_tmp11_ = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, vala_property_get_property_type (prop), FALSE))));
	_vala_ccode_node_unref0 (_tmp12_);
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp18_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("self")), _tmp15_ = g_strconcat (_tmp14_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), "DBusProxy*", NULL))), "disposed")), (ValaCCodeStatement*) dispose_return_block, NULL)));
	_vala_ccode_node_unref0 (_tmp18_);
	_vala_ccode_node_unref0 (_tmp17_);
	_vala_ccode_node_unref0 (_tmp16_);
	_g_free0 (_tmp15_);
	_g_free0 (_tmp14_);
	_vala_ccode_node_unref0 (_tmp13_);
	_cdecl_ = (_tmp19_ = vala_ccode_declaration_new ("DBusGConnection"), _vala_ccode_node_unref0 (_cdecl_), _tmp19_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp20_ = vala_ccode_variable_declarator_new ("*_connection", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp20_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp21_ = vala_ccode_declaration_new ("DBusMessage"), _vala_ccode_node_unref0 (_cdecl_), _tmp21_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp22_ = vala_ccode_variable_declarator_new ("*_message", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp23_ = vala_ccode_variable_declarator_new ("*_reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp24_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp24_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp25_ = vala_ccode_variable_declarator_new ("_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp26_ = vala_ccode_variable_declarator_new ("_subiter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp26_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	destination = (_tmp28_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("dbus_g_proxy_get_bus_name"))), _vala_ccode_node_unref0 (_tmp27_), _tmp28_);
	vala_ccode_function_call_add_argument (destination, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("self")), "DBusGProxy*")));
	_vala_ccode_node_unref0 (_tmp30_);
	_vala_ccode_node_unref0 (_tmp29_);
	path = (_tmp32_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("dbus_g_proxy_get_path"))), _vala_ccode_node_unref0 (_tmp31_), _tmp32_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp34_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("self")), "DBusGProxy*")));
	_vala_ccode_node_unref0 (_tmp34_);
	_vala_ccode_node_unref0 (_tmp33_);
	msgcall = (_tmp36_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new ("dbus_message_new_method_call"))), _vala_ccode_node_unref0 (_tmp35_), _tmp36_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) destination);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) path);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_constant_new ("\"org.freedesktop.DBus.Properties\"")));
	_vala_ccode_node_unref0 (_tmp37_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp38_ = vala_ccode_constant_new ("\"Get\"")));
	_vala_ccode_node_unref0 (_tmp38_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp41_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp40_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_identifier_new ("_message")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp41_);
	_vala_ccode_node_unref0 (_tmp40_);
	_vala_ccode_node_unref0 (_tmp39_);
	iter_call = (_tmp43_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (_tmp42_), _tmp43_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp44_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp46_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp46_);
	_vala_ccode_node_unref0 (_tmp45_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp47_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp47_);
	iter_call = (_tmp49_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp48_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (iter_call), _tmp49_);
	_vala_ccode_node_unref0 (_tmp48_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp50_ = vala_ccode_identifier_new ("_reply")));
	_vala_ccode_node_unref0 (_tmp50_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp52_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp51_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp52_);
	_vala_ccode_node_unref0 (_tmp51_);
	vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp53_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp53_);
	vala_dbus_module_write_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp54_ = vala_ccode_identifier_new ("_iter")), (ValaCCodeExpression*) (_tmp56_ = vala_ccode_constant_new (_tmp55_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp56_);
	_g_free0 (_tmp55_);
	_vala_ccode_node_unref0 (_tmp54_);
	vala_dbus_module_write_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp57_ = vala_ccode_identifier_new ("_iter")), (ValaCCodeExpression*) (_tmp60_ = vala_ccode_constant_new (_tmp59_ = g_strdup_printf ("\"%s\"", _tmp58_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) prop))))));
	_vala_ccode_node_unref0 (_tmp60_);
	_g_free0 (_tmp59_);
	_g_free0 (_tmp58_);
	_vala_ccode_node_unref0 (_tmp57_);
	_cdecl_ = (_tmp62_ = vala_ccode_declaration_new (_tmp61_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)))), _vala_ccode_node_unref0 (_cdecl_), _tmp62_);
	_g_free0 (_tmp61_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp63_ = vala_ccode_variable_declarator_new ("_result", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp63_);
	vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
	if (array_type != NULL) {
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp64_;
				_tmp64_ = TRUE;
				while (TRUE) {
					ValaCCodeDeclaration* _tmp65_;
					ValaCCodeVariableDeclarator* _tmp68_;
					ValaCCodeConstant* _tmp67_;
					char* _tmp66_;
					if (!_tmp64_) {
						dim++;
					}
					_tmp64_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					_cdecl_ = (_tmp65_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp65_);
					vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp68_ = vala_ccode_variable_declarator_new (_tmp66_ = g_strdup_printf ("_result_length%d", dim), (ValaCCodeExpression*) (_tmp67_ = vala_ccode_constant_new ("0")), NULL)));
					_vala_ccode_node_unref0 (_tmp68_);
					_vala_ccode_node_unref0 (_tmp67_);
					_g_free0 (_tmp66_);
					vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
				}
			}
		}
	}
	iter_call = (_tmp70_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp69_ = vala_ccode_identifier_new ("dbus_message_iter_recurse"))), _vala_ccode_node_unref0 (iter_call), _tmp70_);
	_vala_ccode_node_unref0 (_tmp69_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp72_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp71_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp72_);
	_vala_ccode_node_unref0 (_tmp71_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp74_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp73_ = vala_ccode_identifier_new ("_subiter")))));
	_vala_ccode_node_unref0 (_tmp74_);
	_vala_ccode_node_unref0 (_tmp73_);
	vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp75_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp75_);
	target = vala_ccode_identifier_new ("_result");
	expr = (_tmp77_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, postfragment, vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)), (ValaCCodeExpression*) (_tmp76_ = vala_ccode_identifier_new ("_subiter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp76_), _tmp77_);
	vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp79_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp78_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp79_);
	_vala_ccode_node_unref0 (_tmp78_);
	if (array_type != NULL) {
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp80_;
				_tmp80_ = TRUE;
				while (TRUE) {
					ValaCCodeExpressionStatement* _tmp87_;
					ValaCCodeAssignment* _tmp86_;
					ValaCCodeIdentifier* _tmp85_;
					char* _tmp84_;
					ValaCCodeUnaryExpression* _tmp83_;
					ValaCCodeIdentifier* _tmp82_;
					char* _tmp81_;
					if (!_tmp80_) {
						dim++;
					}
					_tmp80_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp87_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp86_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp83_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, (ValaCCodeExpression*) (_tmp82_ = vala_ccode_identifier_new (_tmp81_ = g_strdup_printf ("result_length%d", dim))))), (ValaCCodeExpression*) (_tmp85_ = vala_ccode_identifier_new (_tmp84_ = g_strdup_printf ("_result_length%d", dim))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
					_vala_ccode_node_unref0 (_tmp87_);
					_vala_ccode_node_unref0 (_tmp86_);
					_vala_ccode_node_unref0 (_tmp85_);
					_g_free0 (_tmp84_);
					_vala_ccode_node_unref0 (_tmp83_);
					_vala_ccode_node_unref0 (_tmp82_);
					_g_free0 (_tmp81_);
				}
			}
		}
	}
	gconnection = (_tmp89_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp88_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (_tmp88_), _tmp89_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp90_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp90_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp91_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp91_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp92_ = vala_ccode_identifier_new ("_connection")))));
	_vala_ccode_node_unref0 (_tmp93_);
	_vala_ccode_node_unref0 (_tmp92_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp94_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp94_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp95_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) gconnection)));
	_vala_ccode_node_unref0 (_tmp95_);
	connection = (_tmp97_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp96_ = vala_ccode_identifier_new ("dbus_g_connection_get_connection"))), _vala_ccode_node_unref0 (_tmp96_), _tmp97_);
	vala_ccode_function_call_add_argument (connection, (ValaCCodeExpression*) (_tmp98_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp98_);
	ccall = (_tmp100_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp99_ = vala_ccode_identifier_new ("dbus_connection_send_with_reply_and_block"))), _vala_ccode_node_unref0 (_tmp99_), _tmp100_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) connection);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp101_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp101_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp102_ = vala_ccode_constant_new ("-1")));
	_vala_ccode_node_unref0 (_tmp102_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp103_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp103_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp106_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp105_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp104_ = vala_ccode_identifier_new ("_reply")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp106_);
	_vala_ccode_node_unref0 (_tmp105_);
	_vala_ccode_node_unref0 (_tmp104_);
	conn_unref = (_tmp108_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp107_ = vala_ccode_identifier_new ("dbus_g_connection_unref"))), _vala_ccode_node_unref0 (_tmp107_), _tmp108_);
	vala_ccode_function_call_add_argument (conn_unref, (ValaCCodeExpression*) (_tmp109_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp109_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp110_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) conn_unref)));
	_vala_ccode_node_unref0 (_tmp110_);
	message_unref = (_tmp112_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp111_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp111_), _tmp112_);
	vala_ccode_function_call_add_argument (message_unref, (ValaCCodeExpression*) (_tmp113_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp113_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp114_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) message_unref)));
	_vala_ccode_node_unref0 (_tmp114_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) postfragment);
	reply_unref = (_tmp116_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp115_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp115_), _tmp116_);
	vala_ccode_function_call_add_argument (reply_unref, (ValaCCodeExpression*) (_tmp117_ = vala_ccode_identifier_new ("_reply")));
	_vala_ccode_node_unref0 (_tmp117_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp118_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) reply_unref)));
	_vala_ccode_node_unref0 (_tmp118_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp120_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp119_ = vala_ccode_identifier_new ("_result")))));
	_vala_ccode_node_unref0 (_tmp120_);
	_vala_ccode_node_unref0 (_tmp119_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp121_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp121_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = proxy_name;
	_g_free0 (dbus_iface_name);
	_vala_code_node_unref0 (owned_type);
	_vala_code_node_unref0 (array_type);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (postfragment);
	_vala_ccode_node_unref0 (dispose_return_block);
	_vala_ccode_node_unref0 (destination);
	_vala_ccode_node_unref0 (path);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (target);
	_vala_ccode_node_unref0 (expr);
	_vala_ccode_node_unref0 (gconnection);
	_vala_ccode_node_unref0 (connection);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (conn_unref);
	_vala_ccode_node_unref0 (message_unref);
	_vala_ccode_node_unref0 (reply_unref);
	return result;
}


static char* vala_dbus_client_module_generate_dbus_proxy_property_set (ValaDBusClientModule* self, ValaInterface* iface, ValaProperty* prop) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* proxy_name;
	char* dbus_iface_name;
	ValaDataType* _tmp2_;
	ValaArrayType* array_type;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	ValaCCodeFormalParameter* _tmp7_;
	char* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeFragment* postfragment;
	ValaCCodeBlock* dispose_return_block;
	ValaCCodeReturnStatement* _tmp11_;
	ValaCCodeIfStatement* _tmp17_;
	ValaCCodeMemberAccess* _tmp16_;
	ValaCCodeCastExpression* _tmp15_;
	char* _tmp14_;
	char* _tmp13_;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeDeclaration* _tmp18_;
	ValaCCodeVariableDeclarator* _tmp19_;
	ValaCCodeDeclaration* _tmp20_;
	ValaCCodeVariableDeclarator* _tmp21_;
	ValaCCodeVariableDeclarator* _tmp22_;
	ValaCCodeDeclaration* _tmp23_;
	ValaCCodeVariableDeclarator* _tmp24_;
	ValaCCodeVariableDeclarator* _tmp25_;
	ValaCCodeIdentifier* _tmp26_;
	ValaCCodeFunctionCall* _tmp27_;
	ValaCCodeFunctionCall* destination;
	ValaCCodeCastExpression* _tmp29_;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeIdentifier* _tmp30_;
	ValaCCodeFunctionCall* _tmp31_;
	ValaCCodeFunctionCall* path;
	ValaCCodeCastExpression* _tmp33_;
	ValaCCodeIdentifier* _tmp32_;
	ValaCCodeIdentifier* _tmp34_;
	ValaCCodeFunctionCall* _tmp35_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeConstant* _tmp36_;
	ValaCCodeConstant* _tmp37_;
	ValaCCodeExpressionStatement* _tmp40_;
	ValaCCodeAssignment* _tmp39_;
	ValaCCodeIdentifier* _tmp38_;
	ValaCCodeIdentifier* _tmp41_;
	ValaCCodeFunctionCall* _tmp42_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp43_;
	ValaCCodeUnaryExpression* _tmp45_;
	ValaCCodeIdentifier* _tmp44_;
	ValaCCodeExpressionStatement* _tmp46_;
	ValaCCodeFunctionCall* _tmp48_;
	ValaCCodeIdentifier* _tmp47_;
	ValaCCodeIdentifier* _tmp49_;
	ValaCCodeUnaryExpression* _tmp51_;
	ValaCCodeIdentifier* _tmp50_;
	ValaCCodeExpressionStatement* _tmp52_;
	ValaCCodeConstant* _tmp55_;
	char* _tmp54_;
	ValaCCodeIdentifier* _tmp53_;
	ValaCCodeConstant* _tmp59_;
	char* _tmp58_;
	char* _tmp57_;
	ValaCCodeIdentifier* _tmp56_;
	ValaCCodeFunctionCall* _tmp61_;
	ValaCCodeIdentifier* _tmp60_;
	ValaCCodeUnaryExpression* _tmp63_;
	ValaCCodeIdentifier* _tmp62_;
	ValaCCodeIdentifier* _tmp64_;
	ValaCCodeConstant* _tmp67_;
	char* _tmp66_;
	char* _tmp65_;
	ValaCCodeUnaryExpression* _tmp69_;
	ValaCCodeIdentifier* _tmp68_;
	ValaCCodeExpressionStatement* _tmp70_;
	ValaCCodeIdentifier* _tmp72_;
	ValaCCodeIdentifier* _tmp71_;
	ValaCCodeFunctionCall* _tmp74_;
	ValaCCodeIdentifier* _tmp73_;
	ValaCCodeUnaryExpression* _tmp76_;
	ValaCCodeIdentifier* _tmp75_;
	ValaCCodeUnaryExpression* _tmp78_;
	ValaCCodeIdentifier* _tmp77_;
	ValaCCodeExpressionStatement* _tmp79_;
	ValaCCodeIdentifier* _tmp80_;
	ValaCCodeFunctionCall* _tmp81_;
	ValaCCodeFunctionCall* gconnection;
	ValaCCodeIdentifier* _tmp82_;
	ValaCCodeConstant* _tmp83_;
	ValaCCodeUnaryExpression* _tmp85_;
	ValaCCodeIdentifier* _tmp84_;
	ValaCCodeConstant* _tmp86_;
	ValaCCodeExpressionStatement* _tmp87_;
	ValaCCodeIdentifier* _tmp88_;
	ValaCCodeFunctionCall* _tmp89_;
	ValaCCodeFunctionCall* connection;
	ValaCCodeIdentifier* _tmp90_;
	ValaCCodeIdentifier* _tmp91_;
	ValaCCodeFunctionCall* _tmp92_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp93_;
	ValaCCodeConstant* _tmp94_;
	ValaCCodeConstant* _tmp95_;
	ValaCCodeExpressionStatement* _tmp98_;
	ValaCCodeAssignment* _tmp97_;
	ValaCCodeIdentifier* _tmp96_;
	ValaCCodeIdentifier* _tmp99_;
	ValaCCodeFunctionCall* _tmp100_;
	ValaCCodeFunctionCall* conn_unref;
	ValaCCodeIdentifier* _tmp101_;
	ValaCCodeExpressionStatement* _tmp102_;
	ValaCCodeIdentifier* _tmp103_;
	ValaCCodeFunctionCall* _tmp104_;
	ValaCCodeFunctionCall* message_unref;
	ValaCCodeIdentifier* _tmp105_;
	ValaCCodeExpressionStatement* _tmp106_;
	ValaCCodeIdentifier* _tmp107_;
	ValaCCodeFunctionCall* _tmp108_;
	ValaCCodeFunctionCall* reply_unref;
	ValaCCodeIdentifier* _tmp109_;
	ValaCCodeExpressionStatement* _tmp110_;
	ValaCCodeFunction* _tmp111_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (iface != NULL, NULL);
	g_return_val_if_fail (prop != NULL, NULL);
	proxy_name = (_tmp1_ = g_strdup_printf ("%sdbus_proxy_set_%s", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) iface), vala_symbol_get_name ((ValaSymbol*) prop)), _g_free0 (_tmp0_), _tmp1_);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) iface);
	array_type = _vala_code_node_ref0 ((_tmp2_ = vala_property_accessor_get_value_type (vala_property_get_set_accessor (prop)), VALA_IS_ARRAY_TYPE (_tmp2_) ? ((ValaArrayType*) _tmp2_) : NULL));
	_cdecl_ = NULL;
	function = vala_ccode_function_new (proxy_name, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("self", _tmp4_ = g_strdup_printf ("%s*", _tmp3_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE))));
	_vala_ccode_node_unref0 (_tmp5_);
	_g_free0 (_tmp4_);
	_g_free0 (_tmp3_);
	vala_ccode_function_add_parameter (function, _tmp7_ = vala_ccode_formal_parameter_new ("value", _tmp6_ = vala_data_type_get_cname (vala_property_accessor_get_value_type (vala_property_get_set_accessor (prop)))));
	_vala_ccode_node_unref0 (_tmp7_);
	_g_free0 (_tmp6_);
	if (array_type != NULL) {
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp8_;
				_tmp8_ = TRUE;
				while (TRUE) {
					ValaCCodeFormalParameter* _tmp10_;
					char* _tmp9_;
					if (!_tmp8_) {
						dim++;
					}
					_tmp8_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					vala_ccode_function_add_parameter (function, _tmp10_ = vala_ccode_formal_parameter_new (_tmp9_ = g_strdup_printf ("value_length%d", dim), "int"));
					_vala_ccode_node_unref0 (_tmp10_);
					_g_free0 (_tmp9_);
				}
			}
		}
	}
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	postfragment = vala_ccode_fragment_new ();
	dispose_return_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (dispose_return_block, (ValaCCodeNode*) (_tmp11_ = vala_ccode_return_statement_new (NULL)));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp17_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp15_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("self")), _tmp14_ = g_strconcat (_tmp13_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) iface, FALSE), "DBusProxy*", NULL))), "disposed")), (ValaCCodeStatement*) dispose_return_block, NULL)));
	_vala_ccode_node_unref0 (_tmp17_);
	_vala_ccode_node_unref0 (_tmp16_);
	_vala_ccode_node_unref0 (_tmp15_);
	_g_free0 (_tmp14_);
	_g_free0 (_tmp13_);
	_vala_ccode_node_unref0 (_tmp12_);
	_cdecl_ = (_tmp18_ = vala_ccode_declaration_new ("DBusGConnection"), _vala_ccode_node_unref0 (_cdecl_), _tmp18_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp19_ = vala_ccode_variable_declarator_new ("*_connection", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp19_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp20_ = vala_ccode_declaration_new ("DBusMessage"), _vala_ccode_node_unref0 (_cdecl_), _tmp20_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp21_ = vala_ccode_variable_declarator_new ("*_message", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp22_ = vala_ccode_variable_declarator_new ("*_reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp23_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp23_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp24_ = vala_ccode_variable_declarator_new ("_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp24_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp25_ = vala_ccode_variable_declarator_new ("_subiter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	destination = (_tmp27_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("dbus_g_proxy_get_bus_name"))), _vala_ccode_node_unref0 (_tmp26_), _tmp27_);
	vala_ccode_function_call_add_argument (destination, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("self")), "DBusGProxy*")));
	_vala_ccode_node_unref0 (_tmp29_);
	_vala_ccode_node_unref0 (_tmp28_);
	path = (_tmp31_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("dbus_g_proxy_get_path"))), _vala_ccode_node_unref0 (_tmp30_), _tmp31_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp32_ = vala_ccode_identifier_new ("self")), "DBusGProxy*")));
	_vala_ccode_node_unref0 (_tmp33_);
	_vala_ccode_node_unref0 (_tmp32_);
	msgcall = (_tmp35_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("dbus_message_new_method_call"))), _vala_ccode_node_unref0 (_tmp34_), _tmp35_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) destination);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) path);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_constant_new ("\"org.freedesktop.DBus.Properties\"")));
	_vala_ccode_node_unref0 (_tmp36_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_constant_new ("\"Set\"")));
	_vala_ccode_node_unref0 (_tmp37_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp40_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp39_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp38_ = vala_ccode_identifier_new ("_message")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp40_);
	_vala_ccode_node_unref0 (_tmp39_);
	_vala_ccode_node_unref0 (_tmp38_);
	iter_call = (_tmp42_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp41_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (_tmp41_), _tmp42_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp43_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp43_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp45_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp45_);
	_vala_ccode_node_unref0 (_tmp44_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp46_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp46_);
	iter_call = (_tmp48_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp47_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (iter_call), _tmp48_);
	_vala_ccode_node_unref0 (_tmp47_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("_reply")));
	_vala_ccode_node_unref0 (_tmp49_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp51_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp50_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp51_);
	_vala_ccode_node_unref0 (_tmp50_);
	vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp52_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp52_);
	vala_dbus_module_write_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp53_ = vala_ccode_identifier_new ("_iter")), (ValaCCodeExpression*) (_tmp55_ = vala_ccode_constant_new (_tmp54_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp55_);
	_g_free0 (_tmp54_);
	_vala_ccode_node_unref0 (_tmp53_);
	vala_dbus_module_write_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp56_ = vala_ccode_identifier_new ("_iter")), (ValaCCodeExpression*) (_tmp59_ = vala_ccode_constant_new (_tmp58_ = g_strdup_printf ("\"%s\"", _tmp57_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) prop))))));
	_vala_ccode_node_unref0 (_tmp59_);
	_g_free0 (_tmp58_);
	_g_free0 (_tmp57_);
	_vala_ccode_node_unref0 (_tmp56_);
	iter_call = (_tmp61_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new ("dbus_message_iter_open_container"))), _vala_ccode_node_unref0 (iter_call), _tmp61_);
	_vala_ccode_node_unref0 (_tmp60_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp63_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp63_);
	_vala_ccode_node_unref0 (_tmp62_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp64_ = vala_ccode_identifier_new ("DBUS_TYPE_VARIANT")));
	_vala_ccode_node_unref0 (_tmp64_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp67_ = vala_ccode_constant_new (_tmp66_ = g_strdup_printf ("\"%s\"", _tmp65_ = vala_data_type_get_type_signature (vala_property_get_property_type (prop))))));
	_vala_ccode_node_unref0 (_tmp67_);
	_g_free0 (_tmp66_);
	_g_free0 (_tmp65_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp69_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp68_ = vala_ccode_identifier_new ("_subiter")))));
	_vala_ccode_node_unref0 (_tmp69_);
	_vala_ccode_node_unref0 (_tmp68_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp70_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp70_);
	vala_dbus_module_write_expression ((ValaDBusModule*) self, prefragment, vala_property_accessor_get_value_type (vala_property_get_set_accessor (prop)), (ValaCCodeExpression*) (_tmp71_ = vala_ccode_identifier_new ("_subiter")), (ValaCCodeExpression*) (_tmp72_ = vala_ccode_identifier_new ("value")));
	_vala_ccode_node_unref0 (_tmp72_);
	_vala_ccode_node_unref0 (_tmp71_);
	iter_call = (_tmp74_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp73_ = vala_ccode_identifier_new ("dbus_message_iter_close_container"))), _vala_ccode_node_unref0 (iter_call), _tmp74_);
	_vala_ccode_node_unref0 (_tmp73_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp76_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp75_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp76_);
	_vala_ccode_node_unref0 (_tmp75_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp78_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp77_ = vala_ccode_identifier_new ("_subiter")))));
	_vala_ccode_node_unref0 (_tmp78_);
	_vala_ccode_node_unref0 (_tmp77_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp79_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp79_);
	gconnection = (_tmp81_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp80_ = vala_ccode_identifier_new ("g_object_get"))), _vala_ccode_node_unref0 (_tmp80_), _tmp81_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp82_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp82_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp83_ = vala_ccode_constant_new ("\"connection\"")));
	_vala_ccode_node_unref0 (_tmp83_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp85_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp84_ = vala_ccode_identifier_new ("_connection")))));
	_vala_ccode_node_unref0 (_tmp85_);
	_vala_ccode_node_unref0 (_tmp84_);
	vala_ccode_function_call_add_argument (gconnection, (ValaCCodeExpression*) (_tmp86_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp86_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp87_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) gconnection)));
	_vala_ccode_node_unref0 (_tmp87_);
	connection = (_tmp89_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp88_ = vala_ccode_identifier_new ("dbus_g_connection_get_connection"))), _vala_ccode_node_unref0 (_tmp88_), _tmp89_);
	vala_ccode_function_call_add_argument (connection, (ValaCCodeExpression*) (_tmp90_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp90_);
	ccall = (_tmp92_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp91_ = vala_ccode_identifier_new ("dbus_connection_send_with_reply_and_block"))), _vala_ccode_node_unref0 (_tmp91_), _tmp92_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) connection);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp93_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp94_ = vala_ccode_constant_new ("-1")));
	_vala_ccode_node_unref0 (_tmp94_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp95_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp95_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp98_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp97_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp96_ = vala_ccode_identifier_new ("_reply")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp98_);
	_vala_ccode_node_unref0 (_tmp97_);
	_vala_ccode_node_unref0 (_tmp96_);
	conn_unref = (_tmp100_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp99_ = vala_ccode_identifier_new ("dbus_g_connection_unref"))), _vala_ccode_node_unref0 (_tmp99_), _tmp100_);
	vala_ccode_function_call_add_argument (conn_unref, (ValaCCodeExpression*) (_tmp101_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp101_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp102_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) conn_unref)));
	_vala_ccode_node_unref0 (_tmp102_);
	message_unref = (_tmp104_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp103_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp103_), _tmp104_);
	vala_ccode_function_call_add_argument (message_unref, (ValaCCodeExpression*) (_tmp105_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp105_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp106_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) message_unref)));
	_vala_ccode_node_unref0 (_tmp106_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) postfragment);
	reply_unref = (_tmp108_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp107_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp107_), _tmp108_);
	vala_ccode_function_call_add_argument (reply_unref, (ValaCCodeExpression*) (_tmp109_ = vala_ccode_identifier_new ("_reply")));
	_vala_ccode_node_unref0 (_tmp109_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp110_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) reply_unref)));
	_vala_ccode_node_unref0 (_tmp110_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp111_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp111_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = proxy_name;
	_g_free0 (dbus_iface_name);
	_vala_code_node_unref0 (array_type);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (postfragment);
	_vala_ccode_node_unref0 (dispose_return_block);
	_vala_ccode_node_unref0 (destination);
	_vala_ccode_node_unref0 (path);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (gconnection);
	_vala_ccode_node_unref0 (connection);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (conn_unref);
	_vala_ccode_node_unref0 (message_unref);
	_vala_ccode_node_unref0 (reply_unref);
	return result;
}


static void vala_dbus_client_module_class_init (ValaDBusClientModuleClass * klass) {
	vala_dbus_client_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->finalize = vala_dbus_client_module_finalize;
	g_type_class_add_private (klass, sizeof (ValaDBusClientModulePrivate));
	VALA_CCODE_MODULE_CLASS (klass)->generate_dynamic_method_wrapper = vala_dbus_client_module_real_generate_dynamic_method_wrapper;
	VALA_GSIGNAL_MODULE_CLASS (klass)->get_dbus_g_type = vala_dbus_client_module_real_get_dbus_g_type;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_property_getter_cname = vala_dbus_client_module_real_get_dynamic_property_getter_cname;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_property_setter_cname = vala_dbus_client_module_real_get_dynamic_property_setter_cname;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_signal_connect_wrapper_name = vala_dbus_client_module_real_get_dynamic_signal_connect_wrapper_name;
	VALA_CCODE_MODULE_CLASS (klass)->get_dynamic_signal_disconnect_wrapper_name = vala_dbus_client_module_real_get_dynamic_signal_disconnect_wrapper_name;
	VALA_CCODE_MODULE_CLASS (klass)->visit_cast_expression = vala_dbus_client_module_real_visit_cast_expression;
	VALA_CCODE_MODULE_CLASS (klass)->visit_interface = vala_dbus_client_module_real_visit_interface;
	VALA_GTYPE_MODULE_CLASS (klass)->create_interface_register_function = vala_dbus_client_module_real_create_interface_register_function;
	VALA_CCODE_MODULE_CLASS (klass)->visit_method_call = vala_dbus_client_module_real_visit_method_call;
}


static void vala_dbus_client_module_instance_init (ValaDBusClientModule * self) {
	self->priv = VALA_DBUS_CLIENT_MODULE_GET_PRIVATE (self);
}


static void vala_dbus_client_module_finalize (ValaCCodeModule* obj) {
	ValaDBusClientModule * self;
	self = VALA_DBUS_CLIENT_MODULE (obj);
	VALA_CCODE_MODULE_CLASS (vala_dbus_client_module_parent_class)->finalize (obj);
}


GType vala_dbus_client_module_get_type (void) {
	static GType vala_dbus_client_module_type_id = 0;
	if (vala_dbus_client_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaDBusClientModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_dbus_client_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaDBusClientModule), 0, (GInstanceInitFunc) vala_dbus_client_module_instance_init, NULL };
		vala_dbus_client_module_type_id = g_type_register_static (VALA_TYPE_DBUS_MODULE, "ValaDBusClientModule", &g_define_type_info, 0);
	}
	return vala_dbus_client_module_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




