/* valadbusservermodule.vala
 *
 * Copyright (C) 2007-2009  Jürg Billeter
*  Copyright (C) 2008  Philip Van Hoof
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 * 	Philip Van Hoof <pvanhoof@gnome.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))

typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;

#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))

typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;

#define VALA_TYPE_CCODE_STRUCT_MODULE (vala_ccode_struct_module_get_type ())
#define VALA_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModule))
#define VALA_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
#define VALA_IS_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_IS_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_CCODE_STRUCT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))

typedef struct _ValaCCodeStructModule ValaCCodeStructModule;
typedef struct _ValaCCodeStructModuleClass ValaCCodeStructModuleClass;
typedef struct _ValaCCodeStructModulePrivate ValaCCodeStructModulePrivate;

#define VALA_TYPE_CCODE_METHOD_MODULE (vala_ccode_method_module_get_type ())
#define VALA_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModule))
#define VALA_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))
#define VALA_IS_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_IS_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_CCODE_METHOD_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))

typedef struct _ValaCCodeMethodModule ValaCCodeMethodModule;
typedef struct _ValaCCodeMethodModuleClass ValaCCodeMethodModuleClass;
typedef struct _ValaCCodeMethodModulePrivate ValaCCodeMethodModulePrivate;

#define VALA_TYPE_CCODE_CONTROL_FLOW_MODULE (vala_ccode_control_flow_module_get_type ())
#define VALA_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModule))
#define VALA_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_CCODE_CONTROL_FLOW_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))

typedef struct _ValaCCodeControlFlowModule ValaCCodeControlFlowModule;
typedef struct _ValaCCodeControlFlowModuleClass ValaCCodeControlFlowModuleClass;
typedef struct _ValaCCodeControlFlowModulePrivate ValaCCodeControlFlowModulePrivate;

#define VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE (vala_ccode_member_access_module_get_type ())
#define VALA_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModule))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))

typedef struct _ValaCCodeMemberAccessModule ValaCCodeMemberAccessModule;
typedef struct _ValaCCodeMemberAccessModuleClass ValaCCodeMemberAccessModuleClass;
typedef struct _ValaCCodeMemberAccessModulePrivate ValaCCodeMemberAccessModulePrivate;

#define VALA_TYPE_CCODE_ASSIGNMENT_MODULE (vala_ccode_assignment_module_get_type ())
#define VALA_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModule))
#define VALA_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_CCODE_ASSIGNMENT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))

typedef struct _ValaCCodeAssignmentModule ValaCCodeAssignmentModule;
typedef struct _ValaCCodeAssignmentModuleClass ValaCCodeAssignmentModuleClass;
typedef struct _ValaCCodeAssignmentModulePrivate ValaCCodeAssignmentModulePrivate;

#define VALA_TYPE_CCODE_METHOD_CALL_MODULE (vala_ccode_method_call_module_get_type ())
#define VALA_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModule))
#define VALA_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))
#define VALA_IS_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_IS_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_CCODE_METHOD_CALL_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))

typedef struct _ValaCCodeMethodCallModule ValaCCodeMethodCallModule;
typedef struct _ValaCCodeMethodCallModuleClass ValaCCodeMethodCallModuleClass;
typedef struct _ValaCCodeMethodCallModulePrivate ValaCCodeMethodCallModulePrivate;

#define VALA_TYPE_CCODE_ARRAY_MODULE (vala_ccode_array_module_get_type ())
#define VALA_CCODE_ARRAY_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_ARRAY_MODULE, ValaCCodeArrayModule))
#define VALA_CCODE_ARRAY_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_ARRAY_MODULE, ValaCCodeArrayModuleClass))
#define VALA_IS_CCODE_ARRAY_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_ARRAY_MODULE))
#define VALA_IS_CCODE_ARRAY_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_ARRAY_MODULE))
#define VALA_CCODE_ARRAY_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_ARRAY_MODULE, ValaCCodeArrayModuleClass))

typedef struct _ValaCCodeArrayModule ValaCCodeArrayModule;
typedef struct _ValaCCodeArrayModuleClass ValaCCodeArrayModuleClass;
typedef struct _ValaCCodeArrayModulePrivate ValaCCodeArrayModulePrivate;

#define VALA_TYPE_CCODE_DELEGATE_MODULE (vala_ccode_delegate_module_get_type ())
#define VALA_CCODE_DELEGATE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DELEGATE_MODULE, ValaCCodeDelegateModule))
#define VALA_CCODE_DELEGATE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DELEGATE_MODULE, ValaCCodeDelegateModuleClass))
#define VALA_IS_CCODE_DELEGATE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DELEGATE_MODULE))
#define VALA_IS_CCODE_DELEGATE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DELEGATE_MODULE))
#define VALA_CCODE_DELEGATE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DELEGATE_MODULE, ValaCCodeDelegateModuleClass))

typedef struct _ValaCCodeDelegateModule ValaCCodeDelegateModule;
typedef struct _ValaCCodeDelegateModuleClass ValaCCodeDelegateModuleClass;
typedef struct _ValaCCodeDelegateModulePrivate ValaCCodeDelegateModulePrivate;

#define VALA_TYPE_GERROR_MODULE (vala_gerror_module_get_type ())
#define VALA_GERROR_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GERROR_MODULE, ValaGErrorModule))
#define VALA_GERROR_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GERROR_MODULE, ValaGErrorModuleClass))
#define VALA_IS_GERROR_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GERROR_MODULE))
#define VALA_IS_GERROR_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GERROR_MODULE))
#define VALA_GERROR_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GERROR_MODULE, ValaGErrorModuleClass))

typedef struct _ValaGErrorModule ValaGErrorModule;
typedef struct _ValaGErrorModuleClass ValaGErrorModuleClass;
typedef struct _ValaGErrorModulePrivate ValaGErrorModulePrivate;

#define VALA_TYPE_GTYPE_MODULE (vala_gtype_module_get_type ())
#define VALA_GTYPE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GTYPE_MODULE, ValaGTypeModule))
#define VALA_GTYPE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GTYPE_MODULE, ValaGTypeModuleClass))
#define VALA_IS_GTYPE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GTYPE_MODULE))
#define VALA_IS_GTYPE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GTYPE_MODULE))
#define VALA_GTYPE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GTYPE_MODULE, ValaGTypeModuleClass))

typedef struct _ValaGTypeModule ValaGTypeModule;
typedef struct _ValaGTypeModuleClass ValaGTypeModuleClass;
typedef struct _ValaGTypeModulePrivate ValaGTypeModulePrivate;

#define VALA_TYPE_TYPEREGISTER_FUNCTION (vala_typeregister_function_get_type ())
#define VALA_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunction))
#define VALA_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
#define VALA_IS_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_IS_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_TYPEREGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))

typedef struct _ValaTypeRegisterFunction ValaTypeRegisterFunction;
typedef struct _ValaTypeRegisterFunctionClass ValaTypeRegisterFunctionClass;

#define VALA_TYPE_GOBJECT_MODULE (vala_gobject_module_get_type ())
#define VALA_GOBJECT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GOBJECT_MODULE, ValaGObjectModule))
#define VALA_GOBJECT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GOBJECT_MODULE, ValaGObjectModuleClass))
#define VALA_IS_GOBJECT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GOBJECT_MODULE))
#define VALA_IS_GOBJECT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GOBJECT_MODULE))
#define VALA_GOBJECT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GOBJECT_MODULE, ValaGObjectModuleClass))

typedef struct _ValaGObjectModule ValaGObjectModule;
typedef struct _ValaGObjectModuleClass ValaGObjectModuleClass;
typedef struct _ValaGObjectModulePrivate ValaGObjectModulePrivate;

#define VALA_TYPE_GSIGNAL_MODULE (vala_gsignal_module_get_type ())
#define VALA_GSIGNAL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GSIGNAL_MODULE, ValaGSignalModule))
#define VALA_GSIGNAL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GSIGNAL_MODULE, ValaGSignalModuleClass))
#define VALA_IS_GSIGNAL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GSIGNAL_MODULE))
#define VALA_IS_GSIGNAL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GSIGNAL_MODULE))
#define VALA_GSIGNAL_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GSIGNAL_MODULE, ValaGSignalModuleClass))

typedef struct _ValaGSignalModule ValaGSignalModule;
typedef struct _ValaGSignalModuleClass ValaGSignalModuleClass;
typedef struct _ValaGSignalModulePrivate ValaGSignalModulePrivate;

#define VALA_TYPE_GASYNC_MODULE (vala_gasync_module_get_type ())
#define VALA_GASYNC_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GASYNC_MODULE, ValaGAsyncModule))
#define VALA_GASYNC_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GASYNC_MODULE, ValaGAsyncModuleClass))
#define VALA_IS_GASYNC_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GASYNC_MODULE))
#define VALA_IS_GASYNC_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GASYNC_MODULE))
#define VALA_GASYNC_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GASYNC_MODULE, ValaGAsyncModuleClass))

typedef struct _ValaGAsyncModule ValaGAsyncModule;
typedef struct _ValaGAsyncModuleClass ValaGAsyncModuleClass;
typedef struct _ValaGAsyncModulePrivate ValaGAsyncModulePrivate;

#define VALA_TYPE_DBUS_MODULE (vala_dbus_module_get_type ())
#define VALA_DBUS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DBUS_MODULE, ValaDBusModule))
#define VALA_DBUS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DBUS_MODULE, ValaDBusModuleClass))
#define VALA_IS_DBUS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DBUS_MODULE))
#define VALA_IS_DBUS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DBUS_MODULE))
#define VALA_DBUS_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DBUS_MODULE, ValaDBusModuleClass))

typedef struct _ValaDBusModule ValaDBusModule;
typedef struct _ValaDBusModuleClass ValaDBusModuleClass;
typedef struct _ValaDBusModulePrivate ValaDBusModulePrivate;

#define VALA_TYPE_DBUS_CLIENT_MODULE (vala_dbus_client_module_get_type ())
#define VALA_DBUS_CLIENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DBUS_CLIENT_MODULE, ValaDBusClientModule))
#define VALA_DBUS_CLIENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DBUS_CLIENT_MODULE, ValaDBusClientModuleClass))
#define VALA_IS_DBUS_CLIENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DBUS_CLIENT_MODULE))
#define VALA_IS_DBUS_CLIENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DBUS_CLIENT_MODULE))
#define VALA_DBUS_CLIENT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DBUS_CLIENT_MODULE, ValaDBusClientModuleClass))

typedef struct _ValaDBusClientModule ValaDBusClientModule;
typedef struct _ValaDBusClientModuleClass ValaDBusClientModuleClass;
typedef struct _ValaDBusClientModulePrivate ValaDBusClientModulePrivate;

#define VALA_TYPE_DBUS_SERVER_MODULE (vala_dbus_server_module_get_type ())
#define VALA_DBUS_SERVER_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DBUS_SERVER_MODULE, ValaDBusServerModule))
#define VALA_DBUS_SERVER_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DBUS_SERVER_MODULE, ValaDBusServerModuleClass))
#define VALA_IS_DBUS_SERVER_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DBUS_SERVER_MODULE))
#define VALA_IS_DBUS_SERVER_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DBUS_SERVER_MODULE))
#define VALA_DBUS_SERVER_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DBUS_SERVER_MODULE, ValaDBusServerModuleClass))

typedef struct _ValaDBusServerModule ValaDBusServerModule;
typedef struct _ValaDBusServerModuleClass ValaDBusServerModuleClass;
typedef struct _ValaDBusServerModulePrivate ValaDBusServerModulePrivate;

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))

struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

struct _ValaCCodeBaseModule {
	ValaCCodeModule parent_instance;
	ValaCCodeBaseModulePrivate * priv;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaTryStatement* current_try;
	ValaCCodeDeclarationSpace* header_declarations;
	ValaCCodeDeclarationSpace* internal_header_declarations;
	ValaCCodeDeclarationSpace* source_declarations;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* class_init_fragment;
	ValaCCodeFragment* base_init_fragment;
	ValaCCodeFragment* class_finalize_fragment;
	ValaCCodeFragment* base_finalize_fragment;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_finalize_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeFragment* module_init_fragment;
	ValaCCodeStruct* param_spec_struct;
	ValaCCodeStruct* closure_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeFunction* function;
	ValaCCodeFragment* pre_statement_fragment;
	GeeArrayList* temp_vars;
	GeeArrayList* temp_ref_vars;
	GeeSet* user_marshal_set;
	GeeSet* predefined_marshal_set;
	gint next_temp_var_id;
	gboolean in_constructor;
	gboolean in_static_or_class_context;
	gboolean current_method_inner_error;
	gint next_coroutine_state;
	ValaDataType* void_type;
	ValaDataType* bool_type;
	ValaDataType* char_type;
	ValaDataType* uchar_type;
	ValaDataType* unichar_type;
	ValaDataType* short_type;
	ValaDataType* ushort_type;
	ValaDataType* int_type;
	ValaDataType* uint_type;
	ValaDataType* long_type;
	ValaDataType* ulong_type;
	ValaDataType* int8_type;
	ValaDataType* uint8_type;
	ValaDataType* int16_type;
	ValaDataType* uint16_type;
	ValaDataType* int32_type;
	ValaDataType* uint32_type;
	ValaDataType* int64_type;
	ValaDataType* uint64_type;
	ValaDataType* string_type;
	ValaDataType* float_type;
	ValaDataType* double_type;
	ValaTypeSymbol* gtype_type;
	ValaTypeSymbol* gobject_type;
	ValaErrorType* gerror_type;
	ValaClass* glist_type;
	ValaClass* gslist_type;
	ValaClass* gvaluearray_type;
	ValaTypeSymbol* gstringbuilder_type;
	ValaTypeSymbol* garray_type;
	ValaTypeSymbol* gbytearray_type;
	ValaTypeSymbol* gptrarray_type;
	ValaDataType* gquark_type;
	ValaStruct* gvalue_type;
	ValaStruct* mutex_type;
	ValaTypeSymbol* type_module_type;
	ValaTypeSymbol* dbus_object_type;
	gboolean in_plugin;
	char* module_init_param_name;
	gboolean gvaluecollector_h_needed;
	gboolean requires_array_free;
	gboolean requires_array_move;
	gboolean requires_array_length;
	gboolean requires_strcmp0;
	gboolean dbus_glib_h_needed;
	gboolean dbus_glib_h_needed_in_header;
	GeeSet* wrappers;
	GeeMap* variable_name_map;
};

struct _ValaCCodeBaseModuleClass {
	ValaCCodeModuleClass parent_class;
	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference, gboolean is_chainup);
	char* (*append_struct_array_free) (ValaCCodeBaseModule* self, ValaStruct* st);
	ValaCCodeExpression* (*get_unref_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
	void (*append_local_free) (ValaCCodeBaseModule* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
	gboolean (*variable_accessible_in_finally) (ValaCCodeBaseModule* self, ValaLocalVariable* local);
	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_ref_cexpression) (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
	gboolean (*is_gobject_property) (ValaCCodeBaseModule* self, ValaProperty* prop);
};

struct _ValaCCodeStructModule {
	ValaCCodeBaseModule parent_instance;
	ValaCCodeStructModulePrivate * priv;
};

struct _ValaCCodeStructModuleClass {
	ValaCCodeBaseModuleClass parent_class;
};

struct _ValaCCodeMethodModule {
	ValaCCodeStructModule parent_instance;
	ValaCCodeMethodModulePrivate * priv;
};

struct _ValaCCodeMethodModuleClass {
	ValaCCodeStructModuleClass parent_class;
	void (*generate_method_result_declaration) (ValaCCodeMethodModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeFunction* cfunc, GeeMap* cparam_map, GeeMap* carg_map);
	void (*generate_parameter) (ValaCCodeMethodModule* self, ValaFormalParameter* param, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, GeeMap* carg_map);
};

struct _ValaCCodeControlFlowModule {
	ValaCCodeMethodModule parent_instance;
	ValaCCodeControlFlowModulePrivate * priv;
};

struct _ValaCCodeControlFlowModuleClass {
	ValaCCodeMethodModuleClass parent_class;
};

struct _ValaCCodeMemberAccessModule {
	ValaCCodeControlFlowModule parent_instance;
	ValaCCodeMemberAccessModulePrivate * priv;
};

struct _ValaCCodeMemberAccessModuleClass {
	ValaCCodeControlFlowModuleClass parent_class;
};

struct _ValaCCodeAssignmentModule {
	ValaCCodeMemberAccessModule parent_instance;
	ValaCCodeAssignmentModulePrivate * priv;
};

struct _ValaCCodeAssignmentModuleClass {
	ValaCCodeMemberAccessModuleClass parent_class;
};

struct _ValaCCodeMethodCallModule {
	ValaCCodeAssignmentModule parent_instance;
	ValaCCodeMethodCallModulePrivate * priv;
};

struct _ValaCCodeMethodCallModuleClass {
	ValaCCodeAssignmentModuleClass parent_class;
};

struct _ValaCCodeArrayModule {
	ValaCCodeMethodCallModule parent_instance;
	ValaCCodeArrayModulePrivate * priv;
};

struct _ValaCCodeArrayModuleClass {
	ValaCCodeMethodCallModuleClass parent_class;
};

struct _ValaCCodeDelegateModule {
	ValaCCodeArrayModule parent_instance;
	ValaCCodeDelegateModulePrivate * priv;
};

struct _ValaCCodeDelegateModuleClass {
	ValaCCodeArrayModuleClass parent_class;
};

struct _ValaGErrorModule {
	ValaCCodeDelegateModule parent_instance;
	ValaGErrorModulePrivate * priv;
};

struct _ValaGErrorModuleClass {
	ValaCCodeDelegateModuleClass parent_class;
	ValaCCodeStatement* (*return_with_exception) (ValaGErrorModule* self, ValaCCodeExpression* error_expr);
};

struct _ValaGTypeModule {
	ValaGErrorModule parent_instance;
	ValaGTypeModulePrivate * priv;
};

struct _ValaGTypeModuleClass {
	ValaGErrorModuleClass parent_class;
	void (*generate_virtual_method_declaration) (ValaGTypeModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeStruct* type_struct);
	void (*generate_class_init) (ValaGTypeModule* self, ValaClass* cl, ValaCCodeBlock* init_block);
	ValaTypeRegisterFunction* (*create_interface_register_function) (ValaGTypeModule* self, ValaInterface* iface);
};

struct _ValaGObjectModule {
	ValaGTypeModule parent_instance;
	ValaGObjectModulePrivate * priv;
};

struct _ValaGObjectModuleClass {
	ValaGTypeModuleClass parent_class;
};

struct _ValaGSignalModule {
	ValaGObjectModule parent_instance;
	ValaGSignalModulePrivate * priv;
};

struct _ValaGSignalModuleClass {
	ValaGObjectModuleClass parent_class;
	ValaCCodeExpression* (*get_dbus_g_type) (ValaGSignalModule* self, ValaDataType* data_type);
};

struct _ValaGAsyncModule {
	ValaGSignalModule parent_instance;
	ValaGAsyncModulePrivate * priv;
};

struct _ValaGAsyncModuleClass {
	ValaGSignalModuleClass parent_class;
};

struct _ValaDBusModule {
	ValaGAsyncModule parent_instance;
	ValaDBusModulePrivate * priv;
};

struct _ValaDBusModuleClass {
	ValaGAsyncModuleClass parent_class;
};

struct _ValaDBusClientModule {
	ValaDBusModule parent_instance;
	ValaDBusClientModulePrivate * priv;
};

struct _ValaDBusClientModuleClass {
	ValaDBusModuleClass parent_class;
};

struct _ValaDBusServerModule {
	ValaDBusClientModule parent_instance;
	ValaDBusServerModulePrivate * priv;
};

struct _ValaDBusServerModuleClass {
	ValaDBusClientModuleClass parent_class;
};


static gpointer vala_dbus_server_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_base_module_get_type (void);
gpointer vala_ccode_declaration_space_ref (gpointer instance);
void vala_ccode_declaration_space_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_declaration_space (const GValue* value);
GType vala_ccode_declaration_space_get_type (void);
GType vala_ccode_struct_module_get_type (void);
GType vala_ccode_method_module_get_type (void);
GType vala_ccode_control_flow_module_get_type (void);
GType vala_ccode_member_access_module_get_type (void);
GType vala_ccode_assignment_module_get_type (void);
GType vala_ccode_method_call_module_get_type (void);
GType vala_ccode_array_module_get_type (void);
GType vala_ccode_delegate_module_get_type (void);
GType vala_gerror_module_get_type (void);
GType vala_gtype_module_get_type (void);
gpointer vala_typeregister_function_ref (gpointer instance);
void vala_typeregister_function_unref (gpointer instance);
GParamSpec* vala_param_spec_typeregister_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_typeregister_function (GValue* value, gpointer v_object);
gpointer vala_value_get_typeregister_function (const GValue* value);
GType vala_typeregister_function_get_type (void);
GType vala_gobject_module_get_type (void);
GType vala_gsignal_module_get_type (void);
GType vala_gasync_module_get_type (void);
GType vala_dbus_module_get_type (void);
GType vala_dbus_client_module_get_type (void);
GType vala_dbus_server_module_get_type (void);
enum  {
	VALA_DBUS_SERVER_MODULE_DUMMY_PROPERTY
};
GType vala_ccode_generator_get_type (void);
ValaDBusClientModule* vala_dbus_client_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaDBusClientModule* vala_dbus_client_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaDBusServerModule* vala_dbus_server_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaDBusServerModule* vala_dbus_server_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
static gboolean vala_dbus_server_module_is_dbus_visible (ValaDBusServerModule* self, ValaCodeNode* node);
static char* vala_dbus_server_module_dbus_result_name (ValaDBusServerModule* self, ValaMethod* m);
static void vala_dbus_server_module_send_reply (ValaDBusServerModule* self, ValaCCodeBlock* block);
static void vala_dbus_server_module_handle_reply (ValaDBusServerModule* self, ValaCCodeBlock* block);
void vala_ccode_declaration_space_add_include (ValaCCodeDeclarationSpace* self, const char* filename, gboolean local);
ValaCCodeExpression* vala_ccode_base_module_default_value_for_type (ValaCCodeBaseModule* self, ValaDataType* type, gboolean initializer_expression);
char* vala_dbus_module_get_type_signature (ValaDataType* datatype);
char* vala_ccode_module_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim);
ValaCCodeExpression* vala_dbus_module_read_expression (ValaDBusModule* self, ValaCCodeFragment* fragment, ValaDataType* type, ValaCCodeExpression* iter_expr, ValaCCodeExpression* expr);
void vala_dbus_module_write_expression (ValaDBusModule* self, ValaCCodeFragment* fragment, ValaDataType* type, ValaCCodeExpression* iter_expr, ValaCCodeExpression* expr);
gboolean vala_ccode_base_module_requires_destroy (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr, gboolean is_macro_definition);
static void vala_dbus_server_module_generate_server_error_cases (ValaDBusServerModule* self, ValaCCodeBlock* error_block, GeeList* error_types, ValaCCodeExpression* _error_, ValaCCodeExpression* message, ValaCCodeExpression* reply);
void vala_ccode_declaration_space_add_type_member_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
static char* vala_dbus_server_module_generate_dbus_wrapper (ValaDBusServerModule* self, ValaMethod* m, ValaObjectTypeSymbol* sym);
char* vala_dbus_module_get_dbus_name (ValaDBusModule* self, ValaTypeSymbol* symbol);
void vala_ccode_method_module_generate_parameter (ValaCCodeMethodModule* self, ValaFormalParameter* param, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, GeeMap* carg_map);
ValaCCodeNode* vala_ccode_base_module_get_ccodenode (ValaCCodeBaseModule* self, ValaCodeNode* node);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
static char* vala_dbus_server_module_generate_dbus_signal_wrapper (ValaDBusServerModule* self, ValaSignal* sig, ValaObjectTypeSymbol* sym, const char* dbus_iface_name);
static ValaCCodeExpression* vala_dbus_server_module_get_path_vtable (ValaDBusServerModule* self, ValaObjectType* object_type);
static void vala_dbus_server_module_handle_signals (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, ValaCCodeBlock* block);
static gboolean vala_dbus_server_module_type_implements_dbus_interface (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym);
static void vala_dbus_server_module_generate_register_function (ValaDBusServerModule* self, ValaObjectType* object_type);
static void vala_dbus_server_module_generate_unregister_function (ValaDBusServerModule* self, ValaObjectType* object_type);
static void vala_dbus_server_module_handle_method (ValaDBusServerModule* self, const char* dbus_iface_name, const char* dbus_method_name, const char* handler_name, ValaCCodeBlock* block, ValaCCodeIfStatement** clastif);
static void vala_dbus_server_module_handle_methods (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name, ValaCCodeBlock* block, ValaCCodeIfStatement** clastif);
static char* vala_dbus_server_module_generate_dbus_property_get_wrapper (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name);
static char* vala_dbus_server_module_generate_dbus_property_get_all_wrapper (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name);
static char* vala_dbus_server_module_generate_dbus_property_set_wrapper (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name);
static char* vala_dbus_server_module_get_dbus_type_introspection (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym);
static char* vala_dbus_server_module_generate_dbus_introspect (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym);
static void vala_dbus_server_module_generate_message_function (ValaDBusServerModule* self, ValaObjectType* object_type);
void vala_ccode_declaration_space_add_constant_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node);
static ValaCCodeExpression* vala_dbus_server_module_get_vtable (ValaDBusServerModule* self, ValaObjectType* object_type);
void vala_ccode_module_visit_method_call (ValaCCodeModule* self, ValaMethodCall* expr);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
static void vala_dbus_server_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr);
static ValaCCodeFragment* vala_dbus_server_module_real_register_dbus_info (ValaCCodeModule* base, ValaObjectTypeSymbol* sym);
static int _vala_strcmp0 (const char * str1, const char * str2);



ValaDBusServerModule* vala_dbus_server_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaDBusServerModule* self;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaDBusServerModule*) vala_dbus_client_module_construct (object_type, codegen, next);
	return self;
}


ValaDBusServerModule* vala_dbus_server_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_dbus_server_module_construct (VALA_TYPE_DBUS_SERVER_MODULE, codegen, next);
}


static gboolean vala_dbus_server_module_is_dbus_visible (ValaDBusServerModule* self, ValaCodeNode* node) {
	gboolean result;
	ValaAttribute* dbus_attribute;
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (node != NULL, FALSE);
	dbus_attribute = vala_code_node_get_attribute (node, "DBus");
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	if (dbus_attribute != NULL) {
		_tmp1_ = vala_attribute_has_argument (dbus_attribute, "visible");
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = !vala_attribute_get_bool (dbus_attribute, "visible");
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = FALSE;
		_vala_code_node_unref0 (dbus_attribute);
		return result;
	}
	result = TRUE;
	_vala_code_node_unref0 (dbus_attribute);
	return result;
}


static char* vala_dbus_server_module_dbus_result_name (ValaDBusServerModule* self, ValaMethod* m) {
	char* result;
	ValaAttribute* dbus_attribute;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (m != NULL, NULL);
	dbus_attribute = vala_code_node_get_attribute ((ValaCodeNode*) m, "DBus");
	_tmp0_ = FALSE;
	if (dbus_attribute != NULL) {
		_tmp0_ = vala_attribute_has_argument (dbus_attribute, "result");
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		char* result_name;
		gboolean _tmp1_;
		result_name = vala_attribute_get_string (dbus_attribute, "result");
		_tmp1_ = FALSE;
		if (result_name != NULL) {
			_tmp1_ = _vala_strcmp0 (result_name, "") != 0;
		} else {
			_tmp1_ = FALSE;
		}
		if (_tmp1_) {
			result = result_name;
			_vala_code_node_unref0 (dbus_attribute);
			return result;
		}
		_g_free0 (result_name);
	}
	result = g_strdup ("result");
	_vala_code_node_unref0 (dbus_attribute);
	return result;
}


static void vala_dbus_server_module_send_reply (ValaDBusServerModule* self, ValaCCodeBlock* block) {
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeIdentifier* _tmp3_;
	ValaCCodeConstant* _tmp4_;
	ValaCCodeExpressionStatement* _tmp5_;
	ValaCCodeFunctionCall* _tmp7_;
	ValaCCodeIdentifier* _tmp6_;
	ValaCCodeIdentifier* _tmp8_;
	ValaCCodeExpressionStatement* _tmp9_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	ccall = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_connection_send"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("reply")));
	_vala_ccode_node_unref0 (_tmp3_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp5_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp5_);
	ccall = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (ccall), _tmp7_);
	_vala_ccode_node_unref0 (_tmp6_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_identifier_new ("reply")));
	_vala_ccode_node_unref0 (_tmp8_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp9_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp9_);
	_vala_ccode_node_unref0 (ccall);
}


static void vala_dbus_server_module_handle_reply (ValaDBusServerModule* self, ValaCCodeBlock* block) {
	ValaCCodeBlock* handled;
	ValaCCodeReturnStatement* _tmp1_;
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeBlock* not_handled;
	ValaCCodeReturnStatement* _tmp3_;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeIfStatement* _tmp5_;
	ValaCCodeIdentifier* _tmp4_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	handled = vala_ccode_block_new ();
	vala_dbus_server_module_send_reply (self, handled);
	vala_ccode_block_add_statement (handled, (ValaCCodeNode*) (_tmp1_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp1_);
	_vala_ccode_node_unref0 (_tmp0_);
	not_handled = vala_ccode_block_new ();
	vala_ccode_block_add_statement (not_handled, (ValaCCodeNode*) (_tmp3_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp3_);
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp5_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("reply")), (ValaCCodeStatement*) handled, (ValaCCodeStatement*) not_handled)));
	_vala_ccode_node_unref0 (_tmp5_);
	_vala_ccode_node_unref0 (_tmp4_);
	_vala_ccode_node_unref0 (handled);
	_vala_ccode_node_unref0 (not_handled);
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


static char* vala_dbus_server_module_generate_dbus_wrapper (ValaDBusServerModule* self, ValaMethod* m, ValaObjectTypeSymbol* sym) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* wrapper_name;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFormalParameter* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeFunction* ready_function;
	ValaCCodeBlock* ready_block;
	ValaCCodeFragment* in_prefragment;
	ValaCCodeFragment* in_postfragment;
	ValaCCodeFragment* out_prefragment;
	ValaCCodeFragment* out_postfragment;
	ValaCCodeDeclaration* _tmp19_;
	ValaCCodeVariableDeclarator* _tmp20_;
	ValaCCodeDeclaration* _tmp23_;
	ValaCCodeVariableDeclarator* _tmp25_;
	ValaCCodeConstant* _tmp24_;
	ValaCCodeIdentifier* _tmp26_;
	ValaCCodeFunctionCall* _tmp27_;
	ValaCCodeFunctionCall* message_signature;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeIdentifier* _tmp29_;
	ValaCCodeFunctionCall* _tmp30_;
	ValaCCodeFunctionCall* signature_check;
	ValaCCodeBlock* signature_error_block;
	ValaCCodeReturnStatement* _tmp32_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeIfStatement* _tmp33_;
	ValaCCodeIdentifier* _tmp34_;
	ValaCCodeFunctionCall* _tmp35_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp36_;
	ValaCCodeUnaryExpression* _tmp38_;
	ValaCCodeIdentifier* _tmp37_;
	ValaCCodeExpressionStatement* _tmp39_;
	ValaCCodeDeclaration* _tmp40_;
	ValaCCodeVariableDeclarator* _tmp41_;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeFunctionCall* _tmp43_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeIdentifier* _tmp44_;
	ValaCCodeExpressionStatement* _tmp47_;
	ValaCCodeAssignment* _tmp46_;
	ValaCCodeIdentifier* _tmp45_;
	ValaCCodeFunctionCall* _tmp49_;
	ValaCCodeIdentifier* _tmp48_;
	ValaCCodeIdentifier* _tmp50_;
	ValaCCodeUnaryExpression* _tmp52_;
	ValaCCodeIdentifier* _tmp51_;
	ValaCCodeExpressionStatement* _tmp53_;
	ValaCCodeIdentifier* _tmp55_;
	char* _tmp54_;
	ValaCCodeFunctionCall* _tmp56_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeFunctionCall* finish_ccall;
	ValaCCodeIdentifier* _tmp62_;
	char* type_signature;
	ValaCCodeConstant* _tmp108_;
	char* _tmp107_;
	GeeList* _tmp174_;
	gboolean _tmp175_;
	ValaCCodeFunction* _tmp205_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (m != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	wrapper_name = (_tmp1_ = g_strdup_printf ("_dbus_%s", _tmp0_ = vala_method_get_cname (m)), _g_free0 (_tmp0_), _tmp1_);
	_cdecl_ = NULL;
	function = vala_ccode_function_new (wrapper_name, "DBusHandlerResult");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp4_ = vala_ccode_formal_parameter_new ("self", _tmp3_ = g_strconcat (_tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) sym, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp2_);
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_function_add_parameter (function, _tmp6_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp6_);
	block = vala_ccode_block_new ();
	ready_function = NULL;
	ready_block = NULL;
	if (vala_method_get_coroutine (m)) {
		ValaCCodeFunction* _tmp8_;
		char* _tmp7_;
		ValaCCodeFormalParameter* _tmp9_;
		ValaCCodeFormalParameter* _tmp10_;
		ValaCCodeFormalParameter* _tmp11_;
		ValaCCodeBlock* _tmp12_;
		ValaCCodeDeclaration* _tmp13_;
		ValaCCodeVariableDeclarator* _tmp15_;
		ValaCCodeIdentifier* _tmp14_;
		ValaCCodeDeclaration* _tmp16_;
		ValaCCodeVariableDeclarator* _tmp18_;
		ValaCCodeIdentifier* _tmp17_;
		vala_ccode_declaration_space_add_include (((ValaCCodeBaseModule*) self)->source_declarations, "gio/gio.h", FALSE);
		ready_function = (_tmp8_ = vala_ccode_function_new (_tmp7_ = g_strconcat (wrapper_name, "_ready", NULL), "void"), _vala_ccode_node_unref0 (ready_function), _tmp8_);
		_g_free0 (_tmp7_);
		vala_ccode_function_set_modifiers (ready_function, VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_function_add_parameter (ready_function, _tmp9_ = vala_ccode_formal_parameter_new ("source_object", "GObject *"));
		_vala_ccode_node_unref0 (_tmp9_);
		vala_ccode_function_add_parameter (ready_function, _tmp10_ = vala_ccode_formal_parameter_new ("_res_", "GAsyncResult *"));
		_vala_ccode_node_unref0 (_tmp10_);
		vala_ccode_function_add_parameter (ready_function, _tmp11_ = vala_ccode_formal_parameter_new ("_user_data_", "gpointer *"));
		_vala_ccode_node_unref0 (_tmp11_);
		ready_block = (_tmp12_ = vala_ccode_block_new (), _vala_ccode_node_unref0 (ready_block), _tmp12_);
		_cdecl_ = (_tmp13_ = vala_ccode_declaration_new ("DBusConnection *"), _vala_ccode_node_unref0 (_cdecl_), _tmp13_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp15_ = vala_ccode_variable_declarator_new ("connection", (ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("_user_data_[0]")), NULL)));
		_vala_ccode_node_unref0 (_tmp15_);
		_vala_ccode_node_unref0 (_tmp14_);
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) _cdecl_);
		_cdecl_ = (_tmp16_ = vala_ccode_declaration_new ("DBusMessage *"), _vala_ccode_node_unref0 (_cdecl_), _tmp16_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp18_ = vala_ccode_variable_declarator_new ("message", (ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("_user_data_[1]")), NULL)));
		_vala_ccode_node_unref0 (_tmp18_);
		_vala_ccode_node_unref0 (_tmp17_);
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) _cdecl_);
	}
	in_prefragment = vala_ccode_fragment_new ();
	in_postfragment = vala_ccode_fragment_new ();
	out_prefragment = _vala_ccode_node_ref0 (in_prefragment);
	out_postfragment = _vala_ccode_node_ref0 (in_postfragment);
	_cdecl_ = (_tmp19_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp19_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp20_ = vala_ccode_variable_declarator_new ("iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp20_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	if (vala_method_get_coroutine (m)) {
		ValaCCodeFragment* _tmp21_;
		ValaCCodeFragment* _tmp22_;
		out_prefragment = (_tmp21_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (out_prefragment), _tmp21_);
		out_postfragment = (_tmp22_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (out_postfragment), _tmp22_);
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) _cdecl_);
	}
	_cdecl_ = (_tmp23_ = vala_ccode_declaration_new ("GError*"), _vala_ccode_node_unref0 (_cdecl_), _tmp23_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp25_ = vala_ccode_variable_declarator_new ("error", (ValaCCodeExpression*) (_tmp24_ = vala_ccode_constant_new ("NULL")), NULL)));
	_vala_ccode_node_unref0 (_tmp25_);
	_vala_ccode_node_unref0 (_tmp24_);
	if (vala_method_get_coroutine (m)) {
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) _cdecl_);
	} else {
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	}
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) in_prefragment);
	if (vala_method_get_coroutine (m)) {
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) out_prefragment);
	}
	message_signature = (_tmp27_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("dbus_message_get_signature"))), _vala_ccode_node_unref0 (_tmp26_), _tmp27_);
	vala_ccode_function_call_add_argument (message_signature, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp28_);
	signature_check = (_tmp30_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp29_), _tmp30_);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) message_signature);
	signature_error_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (signature_error_block, (ValaCCodeNode*) (_tmp32_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp32_);
	_vala_ccode_node_unref0 (_tmp31_);
	vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) (_tmp33_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) signature_check, (ValaCCodeStatement*) signature_error_block, NULL)));
	_vala_ccode_node_unref0 (_tmp33_);
	iter_call = (_tmp35_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (_tmp34_), _tmp35_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp36_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp38_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp38_);
	_vala_ccode_node_unref0 (_tmp37_);
	vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) (_tmp39_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp39_);
	_cdecl_ = (_tmp40_ = vala_ccode_declaration_new ("DBusMessage*"), _vala_ccode_node_unref0 (_cdecl_), _tmp40_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp41_ = vala_ccode_variable_declarator_new ("reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp41_);
	vala_ccode_fragment_append (out_postfragment, (ValaCCodeNode*) _cdecl_);
	msgcall = (_tmp43_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("dbus_message_new_method_return"))), _vala_ccode_node_unref0 (_tmp42_), _tmp43_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp44_);
	vala_ccode_fragment_append (out_postfragment, (ValaCCodeNode*) (_tmp47_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp46_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp47_);
	_vala_ccode_node_unref0 (_tmp46_);
	_vala_ccode_node_unref0 (_tmp45_);
	iter_call = (_tmp49_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp48_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (iter_call), _tmp49_);
	_vala_ccode_node_unref0 (_tmp48_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp50_ = vala_ccode_identifier_new ("reply")));
	_vala_ccode_node_unref0 (_tmp50_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp52_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp51_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp52_);
	_vala_ccode_node_unref0 (_tmp51_);
	vala_ccode_fragment_append (out_postfragment, (ValaCCodeNode*) (_tmp53_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp53_);
	ccall = (_tmp56_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp55_ = vala_ccode_identifier_new (_tmp54_ = vala_method_get_cname (m)))), _vala_ccode_node_unref0 (_tmp55_), _g_free0 (_tmp54_), _tmp56_);
	finish_ccall = NULL;
	if (vala_method_get_coroutine (m)) {
		ValaCCodeFunctionCall* _tmp59_;
		ValaCCodeIdentifier* _tmp58_;
		char* _tmp57_;
		ValaCCodeIdentifier* _tmp60_;
		ValaCCodeIdentifier* _tmp61_;
		finish_ccall = (_tmp59_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp58_ = vala_ccode_identifier_new (_tmp57_ = vala_method_get_finish_cname (m)))), _vala_ccode_node_unref0 (finish_ccall), _tmp59_);
		_vala_ccode_node_unref0 (_tmp58_);
		_g_free0 (_tmp57_);
		vala_ccode_function_call_add_argument (finish_ccall, (ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new ("source_object")));
		_vala_ccode_node_unref0 (_tmp60_);
		vala_ccode_function_call_add_argument (finish_ccall, (ValaCCodeExpression*) (_tmp61_ = vala_ccode_identifier_new ("_res_")));
		_vala_ccode_node_unref0 (_tmp61_);
	}
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp62_ = vala_ccode_identifier_new ("self")));
	_vala_ccode_node_unref0 (_tmp62_);
	type_signature = g_strdup ("");
	{
		GeeList* _tmp63_;
		GeeIterator* _tmp64_;
		GeeIterator* _param_it;
		_param_it = (_tmp64_ = gee_iterable_iterator ((GeeIterable*) (_tmp63_ = vala_method_get_parameters (m))), _gee_collection_object_unref0 (_tmp63_), _tmp64_);
		while (TRUE) {
			ValaFormalParameter* param;
			ValaDataType* owned_type;
			ValaCCodeDeclaration* _tmp66_;
			char* _tmp65_;
			ValaCCodeVariableDeclarator* _tmp68_;
			ValaCCodeExpression* _tmp67_;
			gboolean _tmp69_;
			gboolean _tmp70_;
			gboolean _tmp71_;
			char* _tmp76_;
			gboolean _tmp77_;
			gboolean _tmp80_;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			owned_type = vala_data_type_copy (vala_formal_parameter_get_parameter_type (param));
			vala_data_type_set_value_owned (owned_type, TRUE);
			_cdecl_ = (_tmp66_ = vala_ccode_declaration_new (_tmp65_ = vala_data_type_get_cname (owned_type)), _vala_ccode_node_unref0 (_cdecl_), _tmp66_);
			_g_free0 (_tmp65_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp68_ = vala_ccode_variable_declarator_new (vala_symbol_get_name ((ValaSymbol*) param), _tmp67_ = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_parameter_type (param), TRUE), NULL)));
			_vala_ccode_node_unref0 (_tmp68_);
			_vala_ccode_node_unref0 (_tmp67_);
			if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN) {
				vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) _cdecl_);
			} else {
				vala_ccode_fragment_append (out_prefragment, (ValaCCodeNode*) _cdecl_);
			}
			_tmp69_ = FALSE;
			_tmp70_ = FALSE;
			_tmp71_ = FALSE;
			if (_vala_strcmp0 (type_signature, "") == 0) {
				_tmp71_ = vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN;
			} else {
				_tmp71_ = FALSE;
			}
			if (_tmp71_) {
				_tmp70_ = vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param)) != NULL;
			} else {
				_tmp70_ = FALSE;
			}
			if (_tmp70_) {
				char* _tmp72_;
				_tmp69_ = _vala_strcmp0 (_tmp72_ = vala_symbol_get_full_name ((ValaSymbol*) vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param))), "DBus.BusName") == 0;
				_g_free0 (_tmp72_);
			} else {
				_tmp69_ = FALSE;
			}
			if (_tmp69_) {
				ValaCCodeIdentifier* _tmp73_;
				ValaCCodeFunctionCall* _tmp74_;
				ValaCCodeFunctionCall* get_sender;
				ValaCCodeIdentifier* _tmp75_;
				get_sender = (_tmp74_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp73_ = vala_ccode_identifier_new ("dbus_message_get_sender"))), _vala_ccode_node_unref0 (_tmp73_), _tmp74_);
				vala_ccode_function_call_add_argument (get_sender, (ValaCCodeExpression*) (_tmp75_ = vala_ccode_identifier_new ("message")));
				_vala_ccode_node_unref0 (_tmp75_);
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) get_sender);
				_vala_ccode_node_unref0 (get_sender);
				_vala_code_node_unref0 (param);
				_vala_code_node_unref0 (owned_type);
				continue;
			}
			if ((_tmp77_ = (_tmp76_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param))) == NULL, _g_free0 (_tmp76_), _tmp77_)) {
				char* _tmp79_;
				char* _tmp78_;
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_formal_parameter_get_parameter_type (param)), _tmp79_ = g_strdup_printf ("D-Bus serialization of type `%s' is not supported", _tmp78_ = vala_code_node_to_string ((ValaCodeNode*) vala_formal_parameter_get_parameter_type (param))));
				_g_free0 (_tmp79_);
				_g_free0 (_tmp78_);
				_vala_code_node_unref0 (param);
				_vala_code_node_unref0 (owned_type);
				continue;
			}
			_tmp80_ = FALSE;
			if (!vala_method_get_coroutine (m)) {
				_tmp80_ = TRUE;
			} else {
				_tmp80_ = vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN;
			}
			if (_tmp80_) {
				ValaTypeSymbol* _tmp81_;
				ValaStruct* st;
				gboolean _tmp82_;
				st = _vala_code_node_ref0 ((_tmp81_ = vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param)), VALA_IS_STRUCT (_tmp81_) ? ((ValaStruct*) _tmp81_) : NULL));
				_tmp82_ = FALSE;
				if (vala_formal_parameter_get_direction (param) != VALA_PARAMETER_DIRECTION_IN) {
					_tmp82_ = TRUE;
				} else {
					gboolean _tmp83_;
					_tmp83_ = FALSE;
					if (st != NULL) {
						_tmp83_ = !vala_struct_is_simple_type (st);
					} else {
						_tmp83_ = FALSE;
					}
					_tmp82_ = _tmp83_;
				}
				if (_tmp82_) {
					ValaCCodeUnaryExpression* _tmp85_;
					ValaCCodeIdentifier* _tmp84_;
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp85_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp84_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))))));
					_vala_ccode_node_unref0 (_tmp85_);
					_vala_ccode_node_unref0 (_tmp84_);
				} else {
					ValaCCodeIdentifier* _tmp86_;
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp86_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
					_vala_ccode_node_unref0 (_tmp86_);
				}
				_vala_code_node_unref0 (st);
			} else {
				ValaCCodeUnaryExpression* _tmp88_;
				ValaCCodeIdentifier* _tmp87_;
				vala_ccode_function_call_add_argument (finish_ccall, (ValaCCodeExpression*) (_tmp88_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp87_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))))));
				_vala_ccode_node_unref0 (_tmp88_);
				_vala_ccode_node_unref0 (_tmp87_);
			}
			if (VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param))) {
				ValaArrayType* array_type;
				array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)));
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp89_;
						_tmp89_ = TRUE;
						while (TRUE) {
							char* length_cname;
							ValaCCodeDeclaration* _tmp90_;
							ValaCCodeVariableDeclarator* _tmp92_;
							ValaCCodeConstant* _tmp91_;
							if (!_tmp89_) {
								dim++;
							}
							_tmp89_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							length_cname = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, vala_symbol_get_name ((ValaSymbol*) param), dim);
							_cdecl_ = (_tmp90_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp90_);
							vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp92_ = vala_ccode_variable_declarator_new (length_cname, (ValaCCodeExpression*) (_tmp91_ = vala_ccode_constant_new ("0")), NULL)));
							_vala_ccode_node_unref0 (_tmp92_);
							_vala_ccode_node_unref0 (_tmp91_);
							if (vala_formal_parameter_get_direction (param) != VALA_PARAMETER_DIRECTION_IN) {
								ValaCCodeUnaryExpression* _tmp94_;
								ValaCCodeIdentifier* _tmp93_;
								vala_ccode_fragment_append (out_prefragment, (ValaCCodeNode*) _cdecl_);
								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp94_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_identifier_new (length_cname)))));
								_vala_ccode_node_unref0 (_tmp94_);
								_vala_ccode_node_unref0 (_tmp93_);
							} else {
								ValaCCodeIdentifier* _tmp95_;
								vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) _cdecl_);
								vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp95_ = vala_ccode_identifier_new (length_cname)));
								_vala_ccode_node_unref0 (_tmp95_);
							}
							_g_free0 (length_cname);
						}
					}
				}
				_vala_code_node_unref0 (array_type);
			}
			if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN) {
				char* _tmp97_;
				char* _tmp96_;
				ValaCCodeIdentifier* target;
				ValaCCodeIdentifier* _tmp98_;
				ValaCCodeExpression* _tmp99_;
				ValaCCodeExpression* expr;
				ValaCCodeExpressionStatement* _tmp101_;
				ValaCCodeAssignment* _tmp100_;
				type_signature = (_tmp97_ = g_strconcat (type_signature, _tmp96_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param)), NULL), _g_free0 (type_signature), _tmp97_);
				_g_free0 (_tmp96_);
				target = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param));
				expr = (_tmp99_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, in_prefragment, vala_formal_parameter_get_parameter_type (param), (ValaCCodeExpression*) (_tmp98_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp98_), _tmp99_);
				vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) (_tmp101_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp100_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
				_vala_ccode_node_unref0 (_tmp101_);
				_vala_ccode_node_unref0 (_tmp100_);
				_vala_ccode_node_unref0 (target);
				_vala_ccode_node_unref0 (expr);
			} else {
				ValaCCodeIdentifier* _tmp103_;
				ValaCCodeIdentifier* _tmp102_;
				vala_dbus_module_write_expression ((ValaDBusModule*) self, out_postfragment, vala_formal_parameter_get_parameter_type (param), (ValaCCodeExpression*) (_tmp102_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) (_tmp103_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))));
				_vala_ccode_node_unref0 (_tmp103_);
				_vala_ccode_node_unref0 (_tmp102_);
			}
			if (vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, owned_type)) {
				ValaLocalVariable* local;
				ValaMemberAccess* ma;
				ValaCCodeExpression* _tmp105_;
				ValaCCodeIdentifier* _tmp104_;
				ValaCCodeExpressionStatement* _tmp106_;
				ValaCCodeExpressionStatement* stmt;
				local = vala_local_variable_new (owned_type, vala_symbol_get_name ((ValaSymbol*) param), NULL, NULL);
				ma = vala_member_access_new_simple (vala_symbol_get_name ((ValaSymbol*) param), NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
				stmt = (_tmp106_ = vala_ccode_expression_statement_new (_tmp105_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, (ValaCCodeExpression*) (_tmp104_ = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param))), owned_type, (ValaExpression*) ma, FALSE)), _vala_ccode_node_unref0 (_tmp105_), _vala_ccode_node_unref0 (_tmp104_), _tmp106_);
				if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN) {
					vala_ccode_fragment_append (in_postfragment, (ValaCCodeNode*) stmt);
				} else {
					vala_ccode_fragment_append (out_postfragment, (ValaCCodeNode*) stmt);
				}
				_vala_code_node_unref0 (local);
				_vala_code_node_unref0 (ma);
				_vala_ccode_node_unref0 (stmt);
			}
			_vala_code_node_unref0 (param);
			_vala_code_node_unref0 (owned_type);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) (_tmp108_ = vala_ccode_constant_new (_tmp107_ = g_strdup_printf ("\"%s\"", type_signature))));
	_vala_ccode_node_unref0 (_tmp108_);
	_g_free0 (_tmp107_);
	if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
		char* _tmp109_;
		gboolean _tmp110_;
		if ((_tmp110_ = (_tmp109_ = vala_dbus_module_get_type_signature (vala_method_get_return_type (m))) == NULL, _g_free0 (_tmp109_), _tmp110_)) {
			char* _tmp112_;
			char* _tmp111_;
			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) vala_method_get_return_type (m)), _tmp112_ = g_strdup_printf ("D-Bus serialization of type `%s' is not supported", _tmp111_ = vala_code_node_to_string ((ValaCodeNode*) vala_method_get_return_type (m))));
			_g_free0 (_tmp112_);
			_g_free0 (_tmp111_);
		} else {
			if (vala_data_type_is_real_struct_type (vala_method_get_return_type (m))) {
				ValaCCodeDeclaration* _tmp114_;
				char* _tmp113_;
				ValaCCodeVariableDeclarator* _tmp116_;
				ValaCCodeExpression* _tmp115_;
				ValaCCodeIdentifier* _tmp122_;
				ValaCCodeIdentifier* _tmp121_;
				ValaCCodeExpressionStatement* _tmp126_;
				_cdecl_ = (_tmp114_ = vala_ccode_declaration_new (_tmp113_ = vala_data_type_get_cname (vala_method_get_return_type (m))), _vala_ccode_node_unref0 (_cdecl_), _tmp114_);
				_g_free0 (_tmp113_);
				vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp116_ = vala_ccode_variable_declarator_new ("result", _tmp115_ = vala_ccode_base_module_default_value_for_type ((ValaCCodeBaseModule*) self, vala_method_get_return_type (m), TRUE), NULL)));
				_vala_ccode_node_unref0 (_tmp116_);
				_vala_ccode_node_unref0 (_tmp115_);
				vala_ccode_fragment_append (out_prefragment, (ValaCCodeNode*) _cdecl_);
				if (!vala_method_get_coroutine (m)) {
					ValaCCodeUnaryExpression* _tmp118_;
					ValaCCodeIdentifier* _tmp117_;
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp118_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp117_ = vala_ccode_identifier_new ("result")))));
					_vala_ccode_node_unref0 (_tmp118_);
					_vala_ccode_node_unref0 (_tmp117_);
				} else {
					ValaCCodeUnaryExpression* _tmp120_;
					ValaCCodeIdentifier* _tmp119_;
					vala_ccode_function_call_add_argument (finish_ccall, (ValaCCodeExpression*) (_tmp120_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp119_ = vala_ccode_identifier_new ("result")))));
					_vala_ccode_node_unref0 (_tmp120_);
					_vala_ccode_node_unref0 (_tmp119_);
				}
				vala_dbus_module_write_expression ((ValaDBusModule*) self, out_postfragment, vala_method_get_return_type (m), (ValaCCodeExpression*) (_tmp121_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) (_tmp122_ = vala_ccode_identifier_new ("result")));
				_vala_ccode_node_unref0 (_tmp122_);
				_vala_ccode_node_unref0 (_tmp121_);
				if (vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, vala_method_get_return_type (m))) {
					ValaLocalVariable* local;
					ValaMemberAccess* ma;
					ValaCCodeExpressionStatement* _tmp125_;
					ValaCCodeExpression* _tmp124_;
					ValaCCodeIdentifier* _tmp123_;
					local = vala_local_variable_new (vala_method_get_return_type (m), " result", NULL, NULL);
					ma = vala_member_access_new_simple ("result", NULL);
					vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
					vala_ccode_fragment_append (out_postfragment, (ValaCCodeNode*) (_tmp125_ = vala_ccode_expression_statement_new (_tmp124_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, (ValaCCodeExpression*) (_tmp123_ = vala_ccode_identifier_new ("result")), vala_method_get_return_type (m), (ValaExpression*) ma, FALSE))));
					_vala_ccode_node_unref0 (_tmp125_);
					_vala_ccode_node_unref0 (_tmp124_);
					_vala_ccode_node_unref0 (_tmp123_);
					_vala_code_node_unref0 (local);
					_vala_code_node_unref0 (ma);
				}
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp126_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
				_vala_ccode_node_unref0 (_tmp126_);
				if (vala_method_get_coroutine (m)) {
					ValaCCodeExpressionStatement* _tmp127_;
					vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) (_tmp127_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) finish_ccall)));
					_vala_ccode_node_unref0 (_tmp127_);
				}
			} else {
				ValaCCodeDeclaration* _tmp129_;
				char* _tmp128_;
				ValaCCodeVariableDeclarator* _tmp130_;
				ValaCCodeIdentifier* _tmp147_;
				ValaCCodeIdentifier* _tmp146_;
				_cdecl_ = (_tmp129_ = vala_ccode_declaration_new (_tmp128_ = vala_data_type_get_cname (vala_method_get_return_type (m))), _vala_ccode_node_unref0 (_cdecl_), _tmp129_);
				_g_free0 (_tmp128_);
				vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp130_ = vala_ccode_variable_declarator_new ("result", NULL, NULL)));
				_vala_ccode_node_unref0 (_tmp130_);
				vala_ccode_fragment_append (out_prefragment, (ValaCCodeNode*) _cdecl_);
				if (!vala_method_get_coroutine (m)) {
					ValaCCodeExpressionStatement* _tmp133_;
					ValaCCodeAssignment* _tmp132_;
					ValaCCodeIdentifier* _tmp131_;
					vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp133_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp132_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp131_ = vala_ccode_identifier_new ("result")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
					_vala_ccode_node_unref0 (_tmp133_);
					_vala_ccode_node_unref0 (_tmp132_);
					_vala_ccode_node_unref0 (_tmp131_);
				} else {
					ValaCCodeExpressionStatement* _tmp134_;
					ValaCCodeExpressionStatement* _tmp137_;
					ValaCCodeAssignment* _tmp136_;
					ValaCCodeIdentifier* _tmp135_;
					vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp134_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
					_vala_ccode_node_unref0 (_tmp134_);
					vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) (_tmp137_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp136_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp135_ = vala_ccode_identifier_new ("result")), (ValaCCodeExpression*) finish_ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
					_vala_ccode_node_unref0 (_tmp137_);
					_vala_ccode_node_unref0 (_tmp136_);
					_vala_ccode_node_unref0 (_tmp135_);
				}
				if (VALA_IS_ARRAY_TYPE (vala_method_get_return_type (m))) {
					ValaArrayType* array_type;
					array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_method_get_return_type (m)));
					{
						gint dim;
						dim = 1;
						{
							gboolean _tmp138_;
							_tmp138_ = TRUE;
							while (TRUE) {
								char* length_cname;
								ValaCCodeDeclaration* _tmp139_;
								ValaCCodeVariableDeclarator* _tmp141_;
								ValaCCodeConstant* _tmp140_;
								if (!_tmp138_) {
									dim++;
								}
								_tmp138_ = FALSE;
								if (!(dim <= vala_array_type_get_rank (array_type))) {
									break;
								}
								length_cname = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, "result", dim);
								_cdecl_ = (_tmp139_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp139_);
								vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp141_ = vala_ccode_variable_declarator_new (length_cname, (ValaCCodeExpression*) (_tmp140_ = vala_ccode_constant_new ("0")), NULL)));
								_vala_ccode_node_unref0 (_tmp141_);
								_vala_ccode_node_unref0 (_tmp140_);
								vala_ccode_fragment_append (out_prefragment, (ValaCCodeNode*) _cdecl_);
								if (!vala_method_get_coroutine (m)) {
									ValaCCodeUnaryExpression* _tmp143_;
									ValaCCodeIdentifier* _tmp142_;
									vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp143_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp142_ = vala_ccode_identifier_new (length_cname)))));
									_vala_ccode_node_unref0 (_tmp143_);
									_vala_ccode_node_unref0 (_tmp142_);
								} else {
									ValaCCodeUnaryExpression* _tmp145_;
									ValaCCodeIdentifier* _tmp144_;
									vala_ccode_function_call_add_argument (finish_ccall, (ValaCCodeExpression*) (_tmp145_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp144_ = vala_ccode_identifier_new (length_cname)))));
									_vala_ccode_node_unref0 (_tmp145_);
									_vala_ccode_node_unref0 (_tmp144_);
								}
								_g_free0 (length_cname);
							}
						}
					}
					_vala_code_node_unref0 (array_type);
				}
				vala_dbus_module_write_expression ((ValaDBusModule*) self, out_postfragment, vala_method_get_return_type (m), (ValaCCodeExpression*) (_tmp146_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) (_tmp147_ = vala_ccode_identifier_new ("result")));
				_vala_ccode_node_unref0 (_tmp147_);
				_vala_ccode_node_unref0 (_tmp146_);
				if (vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, vala_method_get_return_type (m))) {
					ValaLocalVariable* local;
					ValaMemberAccess* ma;
					ValaCCodeExpressionStatement* _tmp150_;
					ValaCCodeExpression* _tmp149_;
					ValaCCodeIdentifier* _tmp148_;
					local = vala_local_variable_new (vala_method_get_return_type (m), " result", NULL, NULL);
					ma = vala_member_access_new_simple ("result", NULL);
					vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
					vala_ccode_fragment_append (out_postfragment, (ValaCCodeNode*) (_tmp150_ = vala_ccode_expression_statement_new (_tmp149_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, (ValaCCodeExpression*) (_tmp148_ = vala_ccode_identifier_new ("result")), vala_method_get_return_type (m), (ValaExpression*) ma, FALSE))));
					_vala_ccode_node_unref0 (_tmp150_);
					_vala_ccode_node_unref0 (_tmp149_);
					_vala_ccode_node_unref0 (_tmp148_);
					_vala_code_node_unref0 (local);
					_vala_code_node_unref0 (ma);
				}
			}
		}
	} else {
		ValaCCodeExpressionStatement* _tmp151_;
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp151_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
		_vala_ccode_node_unref0 (_tmp151_);
		if (vala_method_get_coroutine (m)) {
			ValaCCodeExpressionStatement* _tmp152_;
			vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) (_tmp152_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) finish_ccall)));
			_vala_ccode_node_unref0 (_tmp152_);
		}
	}
	if (vala_method_get_coroutine (m)) {
		ValaCCodeIdentifier* _tmp154_;
		char* _tmp153_;
		ValaCCodeIdentifier* _tmp155_;
		ValaCCodeFunctionCall* _tmp156_;
		ValaCCodeFunctionCall* new_call;
		ValaCCodeIdentifier* _tmp157_;
		ValaCCodeConstant* _tmp158_;
		ValaCCodeDeclaration* _tmp159_;
		ValaCCodeVariableDeclarator* _tmp160_;
		ValaCCodeIdentifier* _tmp161_;
		ValaCCodeFunctionCall* _tmp162_;
		ValaCCodeFunctionCall* ref_call;
		ValaCCodeIdentifier* _tmp163_;
		ValaCCodeExpressionStatement* _tmp166_;
		ValaCCodeAssignment* _tmp165_;
		ValaCCodeIdentifier* _tmp164_;
		ValaCCodeFunctionCall* _tmp168_;
		ValaCCodeIdentifier* _tmp167_;
		ValaCCodeIdentifier* _tmp169_;
		ValaCCodeExpressionStatement* _tmp172_;
		ValaCCodeAssignment* _tmp171_;
		ValaCCodeIdentifier* _tmp170_;
		ValaCCodeIdentifier* _tmp173_;
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp154_ = vala_ccode_identifier_new (_tmp153_ = g_strconcat (wrapper_name, "_ready", NULL))));
		_vala_ccode_node_unref0 (_tmp154_);
		_g_free0 (_tmp153_);
		new_call = (_tmp156_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp155_ = vala_ccode_identifier_new ("g_new0"))), _vala_ccode_node_unref0 (_tmp155_), _tmp156_);
		vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp157_ = vala_ccode_identifier_new ("gpointer")));
		_vala_ccode_node_unref0 (_tmp157_);
		vala_ccode_function_call_add_argument (new_call, (ValaCCodeExpression*) (_tmp158_ = vala_ccode_constant_new ("2")));
		_vala_ccode_node_unref0 (_tmp158_);
		_cdecl_ = (_tmp159_ = vala_ccode_declaration_new ("gpointer *"), _vala_ccode_node_unref0 (_cdecl_), _tmp159_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp160_ = vala_ccode_variable_declarator_new ("_user_data_", (ValaCCodeExpression*) new_call, NULL)));
		_vala_ccode_node_unref0 (_tmp160_);
		vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) _cdecl_);
		ref_call = (_tmp162_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp161_ = vala_ccode_identifier_new ("dbus_connection_ref"))), _vala_ccode_node_unref0 (_tmp161_), _tmp162_);
		vala_ccode_function_call_add_argument (ref_call, (ValaCCodeExpression*) (_tmp163_ = vala_ccode_identifier_new ("connection")));
		_vala_ccode_node_unref0 (_tmp163_);
		vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) (_tmp166_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp165_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp164_ = vala_ccode_identifier_new ("_user_data_[0]")), (ValaCCodeExpression*) ref_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp166_);
		_vala_ccode_node_unref0 (_tmp165_);
		_vala_ccode_node_unref0 (_tmp164_);
		ref_call = (_tmp168_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp167_ = vala_ccode_identifier_new ("dbus_message_ref"))), _vala_ccode_node_unref0 (ref_call), _tmp168_);
		_vala_ccode_node_unref0 (_tmp167_);
		vala_ccode_function_call_add_argument (ref_call, (ValaCCodeExpression*) (_tmp169_ = vala_ccode_identifier_new ("message")));
		_vala_ccode_node_unref0 (_tmp169_);
		vala_ccode_fragment_append (in_prefragment, (ValaCCodeNode*) (_tmp172_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp171_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp170_ = vala_ccode_identifier_new ("_user_data_[1]")), (ValaCCodeExpression*) ref_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp172_);
		_vala_ccode_node_unref0 (_tmp171_);
		_vala_ccode_node_unref0 (_tmp170_);
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp173_ = vala_ccode_identifier_new ("_user_data_")));
		_vala_ccode_node_unref0 (_tmp173_);
		_vala_ccode_node_unref0 (new_call);
		_vala_ccode_node_unref0 (ref_call);
	}
	if ((_tmp175_ = gee_collection_get_size ((GeeCollection*) (_tmp174_ = vala_code_node_get_error_types ((ValaCodeNode*) m))) > 0, _gee_collection_object_unref0 (_tmp174_), _tmp175_)) {
		ValaCCodeBlock* error_block;
		ValaCCodeIdentifier* _tmp183_;
		ValaCCodeIdentifier* _tmp182_;
		ValaCCodeIdentifier* _tmp181_;
		GeeList* _tmp180_;
		if (vala_method_get_coroutine (m)) {
			ValaCCodeUnaryExpression* _tmp177_;
			ValaCCodeIdentifier* _tmp176_;
			vala_ccode_function_call_add_argument (finish_ccall, (ValaCCodeExpression*) (_tmp177_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp176_ = vala_ccode_identifier_new ("error")))));
			_vala_ccode_node_unref0 (_tmp177_);
			_vala_ccode_node_unref0 (_tmp176_);
		} else {
			ValaCCodeUnaryExpression* _tmp179_;
			ValaCCodeIdentifier* _tmp178_;
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp179_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp178_ = vala_ccode_identifier_new ("error")))));
			_vala_ccode_node_unref0 (_tmp179_);
			_vala_ccode_node_unref0 (_tmp178_);
		}
		error_block = vala_ccode_block_new ();
		vala_dbus_server_module_generate_server_error_cases (self, error_block, _tmp180_ = vala_code_node_get_error_types ((ValaCodeNode*) m), (ValaCCodeExpression*) (_tmp181_ = vala_ccode_identifier_new ("error")), (ValaCCodeExpression*) (_tmp182_ = vala_ccode_identifier_new ("message")), (ValaCCodeExpression*) (_tmp183_ = vala_ccode_identifier_new ("reply")));
		_vala_ccode_node_unref0 (_tmp183_);
		_vala_ccode_node_unref0 (_tmp182_);
		_vala_ccode_node_unref0 (_tmp181_);
		_gee_collection_object_unref0 (_tmp180_);
		vala_dbus_server_module_send_reply (self, error_block);
		if (vala_method_get_coroutine (m)) {
			ValaCCodeReturnStatement* _tmp184_;
			ValaCCodeIfStatement* _tmp186_;
			ValaCCodeIdentifier* _tmp185_;
			vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) (_tmp184_ = vala_ccode_return_statement_new (NULL)));
			_vala_ccode_node_unref0 (_tmp184_);
			vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) (_tmp186_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp185_ = vala_ccode_identifier_new ("error")), (ValaCCodeStatement*) error_block, NULL)));
			_vala_ccode_node_unref0 (_tmp186_);
			_vala_ccode_node_unref0 (_tmp185_);
		} else {
			ValaCCodeReturnStatement* _tmp188_;
			ValaCCodeIdentifier* _tmp187_;
			ValaCCodeIfStatement* _tmp190_;
			ValaCCodeIdentifier* _tmp189_;
			vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) (_tmp188_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp187_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_HANDLED")))));
			_vala_ccode_node_unref0 (_tmp188_);
			_vala_ccode_node_unref0 (_tmp187_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp190_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp189_ = vala_ccode_identifier_new ("error")), (ValaCCodeStatement*) error_block, NULL)));
			_vala_ccode_node_unref0 (_tmp190_);
			_vala_ccode_node_unref0 (_tmp189_);
		}
		_vala_ccode_node_unref0 (error_block);
	}
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) in_postfragment);
	if (!vala_method_get_coroutine (m)) {
		vala_dbus_server_module_handle_reply (self, block);
	} else {
		ValaCCodeReturnStatement* _tmp192_;
		ValaCCodeIdentifier* _tmp191_;
		ValaCCodeIdentifier* _tmp193_;
		ValaCCodeFunctionCall* _tmp194_;
		ValaCCodeFunctionCall* unref_call;
		ValaCCodeIdentifier* _tmp195_;
		ValaCCodeExpressionStatement* _tmp196_;
		ValaCCodeFunctionCall* _tmp198_;
		ValaCCodeIdentifier* _tmp197_;
		ValaCCodeIdentifier* _tmp199_;
		ValaCCodeExpressionStatement* _tmp200_;
		ValaCCodeIdentifier* _tmp201_;
		ValaCCodeFunctionCall* _tmp202_;
		ValaCCodeFunctionCall* free_call;
		ValaCCodeIdentifier* _tmp203_;
		ValaCCodeExpressionStatement* _tmp204_;
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp192_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp191_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_HANDLED")))));
		_vala_ccode_node_unref0 (_tmp192_);
		_vala_ccode_node_unref0 (_tmp191_);
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) out_postfragment);
		vala_dbus_server_module_send_reply (self, ready_block);
		unref_call = (_tmp194_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp193_ = vala_ccode_identifier_new ("dbus_connection_unref"))), _vala_ccode_node_unref0 (_tmp193_), _tmp194_);
		vala_ccode_function_call_add_argument (unref_call, (ValaCCodeExpression*) (_tmp195_ = vala_ccode_identifier_new ("connection")));
		_vala_ccode_node_unref0 (_tmp195_);
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) (_tmp196_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) unref_call)));
		_vala_ccode_node_unref0 (_tmp196_);
		unref_call = (_tmp198_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp197_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (unref_call), _tmp198_);
		_vala_ccode_node_unref0 (_tmp197_);
		vala_ccode_function_call_add_argument (unref_call, (ValaCCodeExpression*) (_tmp199_ = vala_ccode_identifier_new ("message")));
		_vala_ccode_node_unref0 (_tmp199_);
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) (_tmp200_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) unref_call)));
		_vala_ccode_node_unref0 (_tmp200_);
		free_call = (_tmp202_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp201_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp201_), _tmp202_);
		vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp203_ = vala_ccode_identifier_new ("_user_data_")));
		_vala_ccode_node_unref0 (_tmp203_);
		vala_ccode_block_add_statement (ready_block, (ValaCCodeNode*) (_tmp204_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
		_vala_ccode_node_unref0 (_tmp204_);
		_vala_ccode_node_unref0 (unref_call);
		_vala_ccode_node_unref0 (free_call);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp205_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp205_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	if (vala_method_get_coroutine (m)) {
		ValaCCodeFunction* _tmp206_;
		vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp206_ = vala_ccode_function_copy (ready_function)));
		_vala_ccode_node_unref0 (_tmp206_);
		vala_ccode_function_set_block (ready_function, ready_block);
		vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) ready_function);
	}
	result = wrapper_name;
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (ready_function);
	_vala_ccode_node_unref0 (ready_block);
	_vala_ccode_node_unref0 (in_prefragment);
	_vala_ccode_node_unref0 (in_postfragment);
	_vala_ccode_node_unref0 (out_prefragment);
	_vala_ccode_node_unref0 (out_postfragment);
	_vala_ccode_node_unref0 (message_signature);
	_vala_ccode_node_unref0 (signature_check);
	_vala_ccode_node_unref0 (signature_error_block);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (finish_ccall);
	_g_free0 (type_signature);
	return result;
}


static void vala_dbus_server_module_generate_server_error_cases (ValaDBusServerModule* self, ValaCCodeBlock* error_block, GeeList* error_types, ValaCCodeExpression* _error_, ValaCCodeExpression* message, ValaCCodeExpression* reply) {
	ValaCCodeStatement* if_else_if;
	ValaCCodeIfStatement* last_statement;
	g_return_if_fail (self != NULL);
	g_return_if_fail (error_block != NULL);
	g_return_if_fail (error_types != NULL);
	g_return_if_fail (_error_ != NULL);
	g_return_if_fail (message != NULL);
	g_return_if_fail (reply != NULL);
	if_else_if = NULL;
	last_statement = NULL;
	{
		GeeIterator* _error_type_it;
		_error_type_it = gee_iterable_iterator ((GeeIterable*) error_types);
		while (TRUE) {
			ValaDataType* error_type;
			ValaErrorDomain* edomain;
			char* edomain_dbus_name;
			ValaCCodeBlock* true_block;
			ValaCCodeMemberAccess* _tmp0_;
			ValaCCodeSwitchStatement* _tmp1_;
			ValaCCodeSwitchStatement* cswitch;
			ValaCCodeIdentifier* _tmp19_;
			char* _tmp18_;
			ValaCCodeMemberAccess* _tmp17_;
			ValaCCodeBinaryExpression* _tmp20_;
			ValaCCodeBinaryExpression* equal_test;
			ValaCCodeIfStatement* stmt;
			ValaCCodeIfStatement* _tmp22_;
			if (!gee_iterator_next (_error_type_it)) {
				break;
			}
			error_type = (ValaDataType*) gee_iterator_get (_error_type_it);
			edomain = _vala_code_node_ref0 (vala_error_type_get_error_domain (VALA_ERROR_TYPE (error_type)));
			edomain_dbus_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) edomain);
			if (edomain_dbus_name == NULL) {
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) edomain), "Errordomain must have a DBus.name annotation to be serialized over DBus");
			}
			true_block = vala_ccode_block_new ();
			vala_ccode_block_set_suppress_newline (true_block, TRUE);
			cswitch = (_tmp1_ = vala_ccode_switch_statement_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_member_access_new_pointer (_error_, "code"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
			{
				GeeList* _tmp2_;
				GeeIterator* _tmp3_;
				GeeIterator* _ecode_it;
				_ecode_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_error_domain_get_codes (edomain))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
				while (TRUE) {
					ValaErrorCode* ecode;
					ValaCCodeCaseStatement* _tmp6_;
					ValaCCodeIdentifier* _tmp5_;
					char* _tmp4_;
					char* ecode_dbus_name;
					char* dbus_name;
					ValaCCodeIdentifier* _tmp9_;
					ValaCCodeFunctionCall* _tmp10_;
					ValaCCodeFunctionCall* msgcall;
					ValaCCodeConstant* _tmp12_;
					char* _tmp11_;
					ValaCCodeMemberAccess* _tmp13_;
					ValaCCodeExpressionStatement* _tmp15_;
					ValaCCodeAssignment* _tmp14_;
					ValaCCodeBreakStatement* _tmp16_;
					if (!gee_iterator_next (_ecode_it)) {
						break;
					}
					ecode = (ValaErrorCode*) gee_iterator_get (_ecode_it);
					vala_ccode_block_add_statement ((ValaCCodeBlock*) cswitch, (ValaCCodeNode*) (_tmp6_ = vala_ccode_case_statement_new ((ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new (_tmp4_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) ecode, FALSE))))));
					_vala_ccode_node_unref0 (_tmp6_);
					_vala_ccode_node_unref0 (_tmp5_);
					_g_free0 (_tmp4_);
					ecode_dbus_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) ecode);
					if (ecode_dbus_name == NULL) {
						char* _tmp8_;
						char* _tmp7_;
						ecode_dbus_name = (_tmp8_ = vala_symbol_lower_case_to_camel_case (_tmp7_ = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) ecode), -1)), _g_free0 (ecode_dbus_name), _tmp8_);
						_g_free0 (_tmp7_);
					}
					dbus_name = g_strdup_printf ("%s.%s", edomain_dbus_name, ecode_dbus_name);
					msgcall = (_tmp10_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("dbus_message_new_error"))), _vala_ccode_node_unref0 (_tmp9_), _tmp10_);
					vala_ccode_function_call_add_argument (msgcall, message);
					vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_constant_new (_tmp11_ = g_strdup_printf ("\"%s\"", dbus_name))));
					_vala_ccode_node_unref0 (_tmp12_);
					_g_free0 (_tmp11_);
					vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp13_ = vala_ccode_member_access_new_pointer (_error_, "message")));
					_vala_ccode_node_unref0 (_tmp13_);
					vala_ccode_block_add_statement ((ValaCCodeBlock*) cswitch, (ValaCCodeNode*) (_tmp15_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_assignment_new (reply, (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
					_vala_ccode_node_unref0 (_tmp15_);
					_vala_ccode_node_unref0 (_tmp14_);
					vala_ccode_block_add_statement ((ValaCCodeBlock*) cswitch, (ValaCCodeNode*) (_tmp16_ = vala_ccode_break_statement_new ()));
					_vala_ccode_node_unref0 (_tmp16_);
					_vala_code_node_unref0 (ecode);
					_g_free0 (ecode_dbus_name);
					_g_free0 (dbus_name);
					_vala_ccode_node_unref0 (msgcall);
				}
				_gee_collection_object_unref0 (_ecode_it);
			}
			vala_ccode_block_add_statement (true_block, (ValaCCodeNode*) cswitch);
			equal_test = (_tmp20_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_member_access_new_pointer (_error_, "domain")), (ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new (_tmp18_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) edomain, NULL)))), _vala_ccode_node_unref0 (_tmp19_), _g_free0 (_tmp18_), _vala_ccode_node_unref0 (_tmp17_), _tmp20_);
			stmt = vala_ccode_if_statement_new ((ValaCCodeExpression*) equal_test, (ValaCCodeStatement*) true_block, NULL);
			if (last_statement != NULL) {
				vala_ccode_if_statement_set_false_statement (last_statement, (ValaCCodeStatement*) stmt);
			} else {
				ValaCCodeStatement* _tmp21_;
				if_else_if = (_tmp21_ = _vala_ccode_node_ref0 ((ValaCCodeStatement*) stmt), _vala_ccode_node_unref0 (if_else_if), _tmp21_);
			}
			last_statement = (_tmp22_ = _vala_ccode_node_ref0 (stmt), _vala_ccode_node_unref0 (last_statement), _tmp22_);
			_vala_code_node_unref0 (error_type);
			_vala_code_node_unref0 (edomain);
			_g_free0 (edomain_dbus_name);
			_vala_ccode_node_unref0 (true_block);
			_vala_ccode_node_unref0 (cswitch);
			_vala_ccode_node_unref0 (equal_test);
			_vala_ccode_node_unref0 (stmt);
		}
		_gee_collection_object_unref0 (_error_type_it);
	}
	vala_ccode_block_add_statement (error_block, (ValaCCodeNode*) if_else_if);
	_vala_ccode_node_unref0 (if_else_if);
	_vala_ccode_node_unref0 (last_statement);
}


static char* vala_dbus_server_module_generate_dbus_signal_wrapper (ValaDBusServerModule* self, ValaSignal* sig, ValaObjectTypeSymbol* sym, const char* dbus_iface_name) {
	char* result;
	char* _tmp1_;
	char* _tmp0_;
	char* _tmp2_;
	char* wrapper_name;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp3_;
	ValaCCodeFormalParameter* _tmp11_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeFunctionCall* _tmp13_;
	ValaCCodeFunctionCall* path;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeConstant* _tmp15_;
	ValaCCodeDeclaration* _tmp16_;
	ValaCCodeVariableDeclarator* _tmp17_;
	ValaCCodeDeclaration* _tmp18_;
	ValaCCodeVariableDeclarator* _tmp19_;
	ValaCCodeDeclaration* _tmp20_;
	ValaCCodeVariableDeclarator* _tmp21_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeFunctionCall* _tmp23_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeConstant* _tmp26_;
	char* _tmp25_;
	ValaCCodeConstant* _tmp29_;
	char* _tmp28_;
	char* _tmp27_;
	ValaCCodeExpressionStatement* _tmp32_;
	ValaCCodeAssignment* _tmp31_;
	ValaCCodeIdentifier* _tmp30_;
	ValaCCodeIdentifier* _tmp33_;
	ValaCCodeFunctionCall* _tmp34_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp35_;
	ValaCCodeUnaryExpression* _tmp37_;
	ValaCCodeIdentifier* _tmp36_;
	ValaCCodeExpressionStatement* _tmp38_;
	ValaCCodeIdentifier* _tmp43_;
	ValaCCodeFunctionCall* _tmp44_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp45_;
	ValaCCodeIdentifier* _tmp46_;
	ValaCCodeConstant* _tmp47_;
	ValaCCodeExpressionStatement* _tmp48_;
	ValaCCodeIdentifier* _tmp49_;
	ValaCCodeFunctionCall* _tmp50_;
	ValaCCodeFunctionCall* message_unref;
	ValaCCodeIdentifier* _tmp51_;
	ValaCCodeExpressionStatement* _tmp52_;
	ValaCCodeFunction* _tmp53_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sig != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	g_return_val_if_fail (dbus_iface_name != NULL, NULL);
	wrapper_name = (_tmp2_ = g_strdup_printf ("_dbus_%s_%s", _tmp0_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) sym, NULL), _tmp1_ = vala_signal_get_cname (sig)), _g_free0 (_tmp1_), _g_free0 (_tmp0_), _tmp2_);
	_cdecl_ = NULL;
	function = vala_ccode_function_new (wrapper_name, "void");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp3_ = vala_ccode_formal_parameter_new ("_sender", "GObject*"));
	_vala_ccode_node_unref0 (_tmp3_);
	{
		GeeList* _tmp4_;
		GeeIterator* _tmp5_;
		GeeIterator* _param_it;
		_param_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vala_signal_get_parameters (sig))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
		while (TRUE) {
			ValaFormalParameter* param;
			GeeHashMap* _tmp6_;
			ValaCCodeFormalParameter* _tmp7_;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			vala_ccode_method_module_generate_parameter ((ValaCCodeMethodModule*) self, param, ((ValaCCodeBaseModule*) self)->source_declarations, (GeeMap*) (_tmp6_ = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_FORMAL_PARAMETER, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal)), NULL);
			_gee_collection_object_unref0 (_tmp6_);
			vala_ccode_function_add_parameter (function, _tmp7_ = VALA_CCODE_FORMAL_PARAMETER (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) param)));
			_vala_ccode_node_unref0 (_tmp7_);
			if (VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param))) {
				ValaArrayType* array_type;
				array_type = _vala_code_node_ref0 (VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)));
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp8_;
						_tmp8_ = TRUE;
						while (TRUE) {
							ValaCCodeFormalParameter* _tmp10_;
							char* _tmp9_;
							if (!_tmp8_) {
								dim++;
							}
							_tmp8_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							vala_ccode_function_add_parameter (function, _tmp10_ = vala_ccode_formal_parameter_new (_tmp9_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_symbol_get_name ((ValaSymbol*) param), dim), "int"));
							_vala_ccode_node_unref0 (_tmp10_);
							_g_free0 (_tmp9_);
						}
					}
				}
				_vala_code_node_unref0 (array_type);
			}
			_vala_code_node_unref0 (param);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	vala_ccode_function_add_parameter (function, _tmp11_ = vala_ccode_formal_parameter_new ("_connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp11_);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	path = (_tmp13_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("g_object_get_data"))), _vala_ccode_node_unref0 (_tmp12_), _tmp13_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("_sender")));
	_vala_ccode_node_unref0 (_tmp14_);
	vala_ccode_function_call_add_argument (path, (ValaCCodeExpression*) (_tmp15_ = vala_ccode_constant_new ("\"dbus_object_path\"")));
	_vala_ccode_node_unref0 (_tmp15_);
	_cdecl_ = (_tmp16_ = vala_ccode_declaration_new ("const char *"), _vala_ccode_node_unref0 (_cdecl_), _tmp16_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp17_ = vala_ccode_variable_declarator_new ("_path", (ValaCCodeExpression*) path, NULL)));
	_vala_ccode_node_unref0 (_tmp17_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp18_ = vala_ccode_declaration_new ("DBusMessage"), _vala_ccode_node_unref0 (_cdecl_), _tmp18_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp19_ = vala_ccode_variable_declarator_new ("*_message", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp19_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp20_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp20_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp21_ = vala_ccode_variable_declarator_new ("_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	msgcall = (_tmp23_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("dbus_message_new_signal"))), _vala_ccode_node_unref0 (_tmp22_), _tmp23_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("_path")));
	_vala_ccode_node_unref0 (_tmp24_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_constant_new (_tmp25_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp26_);
	_g_free0 (_tmp25_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_constant_new (_tmp28_ = g_strdup_printf ("\"%s\"", _tmp27_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) sig))))));
	_vala_ccode_node_unref0 (_tmp29_);
	_g_free0 (_tmp28_);
	_g_free0 (_tmp27_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp32_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("_message")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp32_);
	_vala_ccode_node_unref0 (_tmp31_);
	_vala_ccode_node_unref0 (_tmp30_);
	iter_call = (_tmp34_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (_tmp33_), _tmp34_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp35_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_identifier_new ("_iter")))));
	_vala_ccode_node_unref0 (_tmp37_);
	_vala_ccode_node_unref0 (_tmp36_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp38_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp38_);
	{
		GeeList* _tmp39_;
		GeeIterator* _tmp40_;
		GeeIterator* _param_it;
		_param_it = (_tmp40_ = gee_iterable_iterator ((GeeIterable*) (_tmp39_ = vala_signal_get_parameters (sig))), _gee_collection_object_unref0 (_tmp39_), _tmp40_);
		while (TRUE) {
			ValaFormalParameter* param;
			ValaCCodeExpression* expr;
			ValaCCodeIdentifier* _tmp42_;
			if (!gee_iterator_next (_param_it)) {
				break;
			}
			param = (ValaFormalParameter*) gee_iterator_get (_param_it);
			expr = (ValaCCodeExpression*) vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) param));
			if (vala_data_type_is_real_struct_type (vala_formal_parameter_get_parameter_type (param))) {
				ValaCCodeExpression* _tmp41_;
				expr = (_tmp41_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION, expr), _vala_ccode_node_unref0 (expr), _tmp41_);
			}
			vala_dbus_module_write_expression ((ValaDBusModule*) self, prefragment, vala_formal_parameter_get_parameter_type (param), (ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("_iter")), expr);
			_vala_ccode_node_unref0 (_tmp42_);
			_vala_code_node_unref0 (param);
			_vala_ccode_node_unref0 (expr);
		}
		_gee_collection_object_unref0 (_param_it);
	}
	ccall = (_tmp44_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp43_ = vala_ccode_identifier_new ("dbus_connection_send"))), _vala_ccode_node_unref0 (_tmp43_), _tmp44_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("_connection")));
	_vala_ccode_node_unref0 (_tmp45_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp46_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp46_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_constant_new ("NULL")));
	_vala_ccode_node_unref0 (_tmp47_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp48_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
	_vala_ccode_node_unref0 (_tmp48_);
	message_unref = (_tmp50_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp49_), _tmp50_);
	vala_ccode_function_call_add_argument (message_unref, (ValaCCodeExpression*) (_tmp51_ = vala_ccode_identifier_new ("_message")));
	_vala_ccode_node_unref0 (_tmp51_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp52_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) message_unref)));
	_vala_ccode_node_unref0 (_tmp52_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp53_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp53_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = wrapper_name;
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (path);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (message_unref);
	return result;
}


static void vala_dbus_server_module_generate_register_function (ValaDBusServerModule* self, ValaObjectType* object_type) {
	ValaObjectTypeSymbol* sym;
	char* _tmp1_;
	char* _tmp0_;
	ValaCCodeFunction* _tmp2_;
	ValaCCodeFunction* cfunc;
	ValaCCodeFormalParameter* _tmp3_;
	ValaCCodeFormalParameter* _tmp4_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFunction* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeFunctionCall* _tmp8_;
	ValaCCodeFunctionCall* get_path;
	ValaCCodeIdentifier* _tmp9_;
	ValaCCodeConstant* _tmp10_;
	ValaCCodeUnaryExpression* register_check;
	ValaCCodeBlock* register_block;
	ValaCCodeIdentifier* _tmp11_;
	ValaCCodeFunctionCall* _tmp12_;
	ValaCCodeFunctionCall* path_dup;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeFunctionCall* _tmp15_;
	ValaCCodeFunctionCall* set_path;
	ValaCCodeIdentifier* _tmp16_;
	ValaCCodeConstant* _tmp17_;
	ValaCCodeExpressionStatement* _tmp18_;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeFunctionCall* _tmp20_;
	ValaCCodeFunctionCall* cregister;
	ValaCCodeIdentifier* _tmp21_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeUnaryExpression* _tmp24_;
	ValaCCodeExpression* _tmp23_;
	ValaCCodeIdentifier* _tmp25_;
	ValaCCodeExpressionStatement* _tmp26_;
	ValaCCodeIdentifier* _tmp27_;
	ValaCCodeFunctionCall* _tmp28_;
	ValaCCodeFunctionCall* weak_unregister;
	ValaCCodeIdentifier* _tmp29_;
	ValaCCodeIdentifier* _tmp30_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeExpressionStatement* _tmp32_;
	ValaCCodeIfStatement* _tmp33_;
	ValaObjectTypeSymbol* _tmp34_;
	ValaClass* cl;
	g_return_if_fail (self != NULL);
	g_return_if_fail (object_type != NULL);
	sym = _vala_code_node_ref0 (vala_object_type_get_type_symbol (object_type));
	cfunc = (_tmp2_ = vala_ccode_function_new (_tmp1_ = g_strconcat (_tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), "dbus_register_object", NULL), "void"), _g_free0 (_tmp1_), _g_free0 (_tmp0_), _tmp2_);
	vala_ccode_function_add_parameter (cfunc, _tmp3_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp3_);
	vala_ccode_function_add_parameter (cfunc, _tmp4_ = vala_ccode_formal_parameter_new ("path", "const char*"));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_add_parameter (cfunc, _tmp5_ = vala_ccode_formal_parameter_new ("object", "void*"));
	_vala_ccode_node_unref0 (_tmp5_);
	if (!vala_symbol_is_private_symbol ((ValaSymbol*) sym)) {
		((ValaCCodeBaseModule*) self)->dbus_glib_h_needed_in_header = TRUE;
	} else {
		((ValaCCodeBaseModule*) self)->dbus_glib_h_needed = TRUE;
		vala_ccode_function_set_modifiers (cfunc, vala_ccode_function_get_modifiers (cfunc) | VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp6_ = vala_ccode_function_copy (cfunc)));
	_vala_ccode_node_unref0 (_tmp6_);
	block = vala_ccode_block_new ();
	vala_ccode_function_set_block (cfunc, block);
	get_path = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new ("g_object_get_data"))), _vala_ccode_node_unref0 (_tmp7_), _tmp8_);
	vala_ccode_function_call_add_argument (get_path, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("object")));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_function_call_add_argument (get_path, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_constant_new ("\"dbus_object_path\"")));
	_vala_ccode_node_unref0 (_tmp10_);
	register_check = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_LOGICAL_NEGATION, (ValaCCodeExpression*) get_path);
	register_block = vala_ccode_block_new ();
	path_dup = (_tmp12_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new ("g_strdup"))), _vala_ccode_node_unref0 (_tmp11_), _tmp12_);
	vala_ccode_function_call_add_argument (path_dup, (ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("path")));
	_vala_ccode_node_unref0 (_tmp13_);
	set_path = (_tmp15_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("g_object_set_data"))), _vala_ccode_node_unref0 (_tmp14_), _tmp15_);
	vala_ccode_function_call_add_argument (set_path, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new ("object")));
	_vala_ccode_node_unref0 (_tmp16_);
	vala_ccode_function_call_add_argument (set_path, (ValaCCodeExpression*) (_tmp17_ = vala_ccode_constant_new ("\"dbus_object_path\"")));
	_vala_ccode_node_unref0 (_tmp17_);
	vala_ccode_function_call_add_argument (set_path, (ValaCCodeExpression*) path_dup);
	vala_ccode_block_add_statement (register_block, (ValaCCodeNode*) (_tmp18_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) set_path)));
	_vala_ccode_node_unref0 (_tmp18_);
	cregister = (_tmp20_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("dbus_connection_register_object_path"))), _vala_ccode_node_unref0 (_tmp19_), _tmp20_);
	vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp21_);
	vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("path")));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp23_ = vala_dbus_server_module_get_path_vtable (self, object_type))));
	_vala_ccode_node_unref0 (_tmp24_);
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_identifier_new ("object")));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_block_add_statement (register_block, (ValaCCodeNode*) (_tmp26_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) cregister)));
	_vala_ccode_node_unref0 (_tmp26_);
	weak_unregister = (_tmp28_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp27_ = vala_ccode_identifier_new ("g_object_weak_ref"))), _vala_ccode_node_unref0 (_tmp27_), _tmp28_);
	vala_ccode_function_call_add_argument (weak_unregister, (ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("object")));
	_vala_ccode_node_unref0 (_tmp29_);
	vala_ccode_function_call_add_argument (weak_unregister, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("_vala_dbus_unregister_object")));
	_vala_ccode_node_unref0 (_tmp30_);
	vala_ccode_function_call_add_argument (weak_unregister, (ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp31_);
	vala_ccode_block_add_statement (register_block, (ValaCCodeNode*) (_tmp32_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) weak_unregister)));
	_vala_ccode_node_unref0 (_tmp32_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp33_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) register_check, (ValaCCodeStatement*) register_block, NULL)));
	_vala_ccode_node_unref0 (_tmp33_);
	vala_dbus_server_module_handle_signals (self, vala_object_type_get_type_symbol (object_type), block);
	cl = _vala_code_node_ref0 ((_tmp34_ = sym, VALA_IS_CLASS (_tmp34_) ? ((ValaClass*) _tmp34_) : NULL));
	if (cl != NULL) {
		{
			GeeList* _tmp35_;
			GeeIterator* _tmp36_;
			GeeIterator* _base_type_it;
			_base_type_it = (_tmp36_ = gee_iterable_iterator ((GeeIterable*) (_tmp35_ = vala_class_get_base_types (cl))), _gee_collection_object_unref0 (_tmp35_), _tmp36_);
			while (TRUE) {
				ValaDataType* base_type;
				ValaDataType* _tmp37_;
				ValaObjectType* base_obj_type;
				if (!gee_iterator_next (_base_type_it)) {
					break;
				}
				base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
				base_obj_type = _vala_code_node_ref0 ((_tmp37_ = base_type, VALA_IS_OBJECT_TYPE (_tmp37_) ? ((ValaObjectType*) _tmp37_) : NULL));
				if (vala_dbus_server_module_type_implements_dbus_interface (self, vala_object_type_get_type_symbol (base_obj_type))) {
					ValaCCodeIdentifier* _tmp40_;
					char* _tmp39_;
					char* _tmp38_;
					ValaCCodeFunctionCall* _tmp41_;
					ValaCCodeFunctionCall* base_register;
					ValaCCodeIdentifier* _tmp42_;
					ValaCCodeIdentifier* _tmp43_;
					ValaCCodeIdentifier* _tmp44_;
					ValaCCodeExpressionStatement* _tmp45_;
					base_register = (_tmp41_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp40_ = vala_ccode_identifier_new (_tmp39_ = g_strconcat (_tmp38_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) vala_object_type_get_type_symbol (base_obj_type)), "dbus_register_object", NULL)))), _vala_ccode_node_unref0 (_tmp40_), _g_free0 (_tmp39_), _g_free0 (_tmp38_), _tmp41_);
					vala_ccode_function_call_add_argument (base_register, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("connection")));
					_vala_ccode_node_unref0 (_tmp42_);
					vala_ccode_function_call_add_argument (base_register, (ValaCCodeExpression*) (_tmp43_ = vala_ccode_identifier_new ("path")));
					_vala_ccode_node_unref0 (_tmp43_);
					vala_ccode_function_call_add_argument (base_register, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_identifier_new ("object")));
					_vala_ccode_node_unref0 (_tmp44_);
					vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp45_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) base_register)));
					_vala_ccode_node_unref0 (_tmp45_);
					_vala_ccode_node_unref0 (base_register);
				}
				_vala_code_node_unref0 (base_type);
				_vala_code_node_unref0 (base_obj_type);
			}
			_gee_collection_object_unref0 (_base_type_it);
		}
	}
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) cfunc);
	_vala_code_node_unref0 (sym);
	_vala_ccode_node_unref0 (cfunc);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (get_path);
	_vala_ccode_node_unref0 (register_check);
	_vala_ccode_node_unref0 (register_block);
	_vala_ccode_node_unref0 (path_dup);
	_vala_ccode_node_unref0 (set_path);
	_vala_ccode_node_unref0 (cregister);
	_vala_ccode_node_unref0 (weak_unregister);
	_vala_code_node_unref0 (cl);
}


static void vala_dbus_server_module_generate_unregister_function (ValaDBusServerModule* self, ValaObjectType* object_type) {
	ValaObjectTypeSymbol* sym;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	ValaCCodeFunction* _tmp3_;
	ValaCCodeFunction* cfunc;
	ValaCCodeFormalParameter* _tmp4_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFunction* _tmp6_;
	ValaCCodeBlock* block;
	g_return_if_fail (self != NULL);
	g_return_if_fail (object_type != NULL);
	sym = _vala_code_node_ref0 (vala_object_type_get_type_symbol (object_type));
	cfunc = (_tmp3_ = vala_ccode_function_new (_tmp2_ = g_strconcat (_tmp1_ = g_strconcat ("_", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), NULL), "dbus_unregister", NULL), "void"), _g_free0 (_tmp2_), _g_free0 (_tmp1_), _g_free0 (_tmp0_), _tmp3_);
	vala_ccode_function_add_parameter (cfunc, _tmp4_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_add_parameter (cfunc, _tmp5_ = vala_ccode_formal_parameter_new ("_user_data_", "void*"));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp6_ = vala_ccode_function_copy (cfunc)));
	_vala_ccode_node_unref0 (_tmp6_);
	block = vala_ccode_block_new ();
	vala_ccode_function_set_block (cfunc, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) cfunc);
	_vala_code_node_unref0 (sym);
	_vala_ccode_node_unref0 (cfunc);
	_vala_ccode_node_unref0 (block);
}


static void vala_dbus_server_module_handle_method (ValaDBusServerModule* self, const char* dbus_iface_name, const char* dbus_method_name, const char* handler_name, ValaCCodeBlock* block, ValaCCodeIfStatement** clastif) {
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* ccheck;
	ValaCCodeIdentifier* _tmp2_;
	ValaCCodeConstant* _tmp4_;
	char* _tmp3_;
	ValaCCodeConstant* _tmp6_;
	char* _tmp5_;
	ValaCCodeBlock* callblock;
	ValaCCodeIdentifier* _tmp7_;
	ValaCCodeFunctionCall* _tmp8_;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeIdentifier* _tmp9_;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeIdentifier* _tmp11_;
	ValaCCodeExpressionStatement* _tmp14_;
	ValaCCodeAssignment* _tmp13_;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeIfStatement* cif;
	ValaCCodeIfStatement* _tmp15_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (dbus_iface_name != NULL);
	g_return_if_fail (dbus_method_name != NULL);
	g_return_if_fail (handler_name != NULL);
	g_return_if_fail (block != NULL);
	g_return_if_fail (clastif != NULL);
	ccheck = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("dbus_message_is_method_call"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp2_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_constant_new (_tmp3_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	vala_ccode_function_call_add_argument (ccheck, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_constant_new (_tmp5_ = g_strdup_printf ("\"%s\"", dbus_method_name))));
	_vala_ccode_node_unref0 (_tmp6_);
	_g_free0 (_tmp5_);
	callblock = vala_ccode_block_new ();
	ccall = (_tmp8_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (handler_name))), _vala_ccode_node_unref0 (_tmp7_), _tmp8_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("object")));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_block_add_statement (callblock, (ValaCCodeNode*) (_tmp14_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("result")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp14_);
	_vala_ccode_node_unref0 (_tmp13_);
	_vala_ccode_node_unref0 (_tmp12_);
	cif = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccheck, (ValaCCodeStatement*) callblock, NULL);
	if ((*clastif) == NULL) {
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) cif);
	} else {
		vala_ccode_if_statement_set_false_statement ((*clastif), (ValaCCodeStatement*) cif);
	}
	(*clastif) = (_tmp15_ = _vala_ccode_node_ref0 (cif), _vala_ccode_node_unref0 ((*clastif)), _tmp15_);
	_vala_ccode_node_unref0 (ccheck);
	_vala_ccode_node_unref0 (callblock);
	_vala_ccode_node_unref0 (ccall);
	_vala_ccode_node_unref0 (cif);
}


static void vala_dbus_server_module_handle_methods (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name, ValaCCodeBlock* block, ValaCCodeIfStatement** clastif) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sym != NULL);
	g_return_if_fail (dbus_iface_name != NULL);
	g_return_if_fail (block != NULL);
	g_return_if_fail (clastif != NULL);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _m_it;
		_m_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_object_type_symbol_get_methods (sym))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaMethod* m;
			gboolean _tmp2_;
			gboolean _tmp3_;
			gboolean _tmp4_;
			char* _tmp6_;
			char* _tmp5_;
			if (!gee_iterator_next (_m_it)) {
				break;
			}
			m = (ValaMethod*) gee_iterator_get (_m_it);
			_tmp2_ = FALSE;
			_tmp3_ = FALSE;
			_tmp4_ = FALSE;
			if (VALA_IS_CREATION_METHOD (m)) {
				_tmp4_ = TRUE;
			} else {
				_tmp4_ = vala_method_get_binding (m) != MEMBER_BINDING_INSTANCE;
			}
			if (_tmp4_) {
				_tmp3_ = TRUE;
			} else {
				_tmp3_ = vala_method_get_overrides (m);
			}
			if (_tmp3_) {
				_tmp2_ = TRUE;
			} else {
				_tmp2_ = vala_symbol_get_access ((ValaSymbol*) m) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			}
			if (_tmp2_) {
				_vala_code_node_unref0 (m);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) m)) {
				_vala_code_node_unref0 (m);
				continue;
			}
			vala_dbus_server_module_handle_method (self, dbus_iface_name, _tmp5_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) m)), _tmp6_ = vala_dbus_server_module_generate_dbus_wrapper (self, m, sym), block, &(*clastif));
			_g_free0 (_tmp6_);
			_g_free0 (_tmp5_);
			_vala_code_node_unref0 (m);
		}
		_gee_collection_object_unref0 (_m_it);
	}
}


static char* vala_dbus_server_module_generate_dbus_property_get_wrapper (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* wrapper_name;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFormalParameter* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeDeclaration* _tmp7_;
	ValaCCodeVariableDeclarator* _tmp8_;
	ValaCCodeDeclaration* _tmp9_;
	ValaCCodeVariableDeclarator* _tmp10_;
	ValaCCodeVariableDeclarator* _tmp11_;
	ValaCCodeVariableDeclarator* _tmp12_;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeFunctionCall* _tmp14_;
	ValaCCodeFunctionCall* message_signature;
	ValaCCodeIdentifier* _tmp15_;
	ValaCCodeIdentifier* _tmp16_;
	ValaCCodeFunctionCall* _tmp17_;
	ValaCCodeFunctionCall* signature_check;
	ValaCCodeConstant* _tmp18_;
	ValaCCodeBlock* signature_error_block;
	ValaCCodeReturnStatement* _tmp20_;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeIfStatement* _tmp21_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeFunctionCall* _tmp23_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp24_;
	ValaCCodeUnaryExpression* _tmp26_;
	ValaCCodeIdentifier* _tmp25_;
	ValaCCodeExpressionStatement* _tmp27_;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeFunctionCall* _tmp29_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeIdentifier* _tmp30_;
	ValaCCodeExpressionStatement* _tmp33_;
	ValaCCodeAssignment* _tmp32_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeFunctionCall* _tmp35_;
	ValaCCodeIdentifier* _tmp34_;
	ValaCCodeIdentifier* _tmp36_;
	ValaCCodeUnaryExpression* _tmp38_;
	ValaCCodeIdentifier* _tmp37_;
	ValaCCodeExpressionStatement* _tmp39_;
	ValaCCodeDeclaration* _tmp40_;
	ValaCCodeVariableDeclarator* _tmp41_;
	ValaCCodeIdentifier* target;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeExpression* _tmp43_;
	ValaCCodeExpression* expr;
	ValaCCodeExpressionStatement* _tmp45_;
	ValaCCodeAssignment* _tmp44_;
	ValaCCodeDeclaration* _tmp46_;
	ValaCCodeVariableDeclarator* _tmp47_;
	ValaCCodeIdentifier* _tmp48_;
	ValaCCodeExpression* _tmp50_;
	ValaCCodeIdentifier* _tmp49_;
	ValaCCodeExpressionStatement* _tmp52_;
	ValaCCodeAssignment* _tmp51_;
	ValaCCodeIfStatement* clastif;
	ValaCCodeIdentifier* _tmp112_;
	ValaCCodeFunctionCall* _tmp113_;
	ValaCCodeFunctionCall* free_call;
	ValaCCodeIdentifier* _tmp114_;
	ValaCCodeExpressionStatement* _tmp115_;
	ValaCCodeFunctionCall* _tmp117_;
	ValaCCodeIdentifier* _tmp116_;
	ValaCCodeIdentifier* _tmp118_;
	ValaCCodeExpressionStatement* _tmp119_;
	ValaCCodeFunction* _tmp131_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	g_return_val_if_fail (dbus_iface_name != NULL, NULL);
	wrapper_name = (_tmp1_ = g_strdup_printf ("_dbus_%s_property_get", _tmp0_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) sym, NULL)), _g_free0 (_tmp0_), _tmp1_);
	_cdecl_ = NULL;
	function = vala_ccode_function_new (wrapper_name, "DBusHandlerResult");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp4_ = vala_ccode_formal_parameter_new ("self", _tmp3_ = g_strconcat (_tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) sym, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp2_);
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_function_add_parameter (function, _tmp6_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp6_);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	_cdecl_ = (_tmp7_ = vala_ccode_declaration_new ("DBusMessage*"), _vala_ccode_node_unref0 (_cdecl_), _tmp7_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp8_ = vala_ccode_variable_declarator_new ("reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp8_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp9_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp9_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp10_ = vala_ccode_variable_declarator_new ("iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp11_ = vala_ccode_variable_declarator_new ("reply_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp11_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp12_ = vala_ccode_variable_declarator_new ("subiter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	message_signature = (_tmp14_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("dbus_message_get_signature"))), _vala_ccode_node_unref0 (_tmp13_), _tmp14_);
	vala_ccode_function_call_add_argument (message_signature, (ValaCCodeExpression*) (_tmp15_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp15_);
	signature_check = (_tmp17_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp16_), _tmp17_);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) message_signature);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) (_tmp18_ = vala_ccode_constant_new ("\"ss\"")));
	_vala_ccode_node_unref0 (_tmp18_);
	signature_error_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (signature_error_block, (ValaCCodeNode*) (_tmp20_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp20_);
	_vala_ccode_node_unref0 (_tmp19_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp21_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) signature_check, (ValaCCodeStatement*) signature_error_block, NULL)));
	_vala_ccode_node_unref0 (_tmp21_);
	iter_call = (_tmp23_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (_tmp22_), _tmp23_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp24_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp26_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp25_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp26_);
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp27_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp27_);
	msgcall = (_tmp29_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("dbus_message_new_method_return"))), _vala_ccode_node_unref0 (_tmp28_), _tmp29_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp30_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp33_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp32_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp33_);
	_vala_ccode_node_unref0 (_tmp32_);
	_vala_ccode_node_unref0 (_tmp31_);
	iter_call = (_tmp35_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (iter_call), _tmp35_);
	_vala_ccode_node_unref0 (_tmp34_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_identifier_new ("reply")));
	_vala_ccode_node_unref0 (_tmp36_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp38_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp37_ = vala_ccode_identifier_new ("reply_iter")))));
	_vala_ccode_node_unref0 (_tmp38_);
	_vala_ccode_node_unref0 (_tmp37_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp39_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp39_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	_cdecl_ = (_tmp40_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp40_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp41_ = vala_ccode_variable_declarator_new ("interface_name", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp41_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
	target = vala_ccode_identifier_new ("interface_name");
	expr = (_tmp43_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp42_), _tmp43_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp45_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp44_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp45_);
	_vala_ccode_node_unref0 (_tmp44_);
	_cdecl_ = (_tmp46_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp46_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp47_ = vala_ccode_variable_declarator_new ("property_name", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp47_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
	target = (_tmp48_ = vala_ccode_identifier_new ("property_name"), _vala_ccode_node_unref0 (target), _tmp48_);
	expr = (_tmp50_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (expr), _tmp50_);
	_vala_ccode_node_unref0 (_tmp49_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp52_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp51_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp52_);
	_vala_ccode_node_unref0 (_tmp51_);
	clastif = NULL;
	{
		GeeList* _tmp53_;
		GeeIterator* _tmp54_;
		GeeIterator* _prop_it;
		_prop_it = (_tmp54_ = gee_iterable_iterator ((GeeIterable*) (_tmp53_ = vala_object_type_symbol_get_properties (sym))), _gee_collection_object_unref0 (_tmp53_), _tmp54_);
		while (TRUE) {
			ValaProperty* prop;
			gboolean _tmp55_;
			gboolean _tmp56_;
			ValaCCodeBlock* prop_block;
			ValaCCodeFragment* postfragment;
			ValaCCodeIdentifier* _tmp57_;
			ValaCCodeFunctionCall* _tmp58_;
			ValaCCodeFunctionCall* ccmp;
			ValaCCodeIdentifier* _tmp59_;
			ValaCCodeConstant* _tmp61_;
			char* _tmp60_;
			ValaCCodeConstant* _tmp62_;
			ValaCCodeBinaryExpression* _tmp63_;
			ValaCCodeBinaryExpression* ccheck1;
			ValaCCodeFunctionCall* _tmp65_;
			ValaCCodeIdentifier* _tmp64_;
			ValaCCodeIdentifier* _tmp66_;
			ValaCCodeConstant* _tmp69_;
			char* _tmp68_;
			char* _tmp67_;
			ValaCCodeConstant* _tmp70_;
			ValaCCodeBinaryExpression* _tmp71_;
			ValaCCodeBinaryExpression* ccheck2;
			ValaCCodeBinaryExpression* ccheck;
			ValaCCodeFunctionCall* _tmp73_;
			ValaCCodeIdentifier* _tmp72_;
			ValaCCodeUnaryExpression* _tmp75_;
			ValaCCodeIdentifier* _tmp74_;
			ValaCCodeIdentifier* _tmp76_;
			ValaCCodeConstant* _tmp79_;
			char* _tmp78_;
			char* _tmp77_;
			ValaCCodeUnaryExpression* _tmp81_;
			ValaCCodeIdentifier* _tmp80_;
			ValaCCodeExpressionStatement* _tmp82_;
			ValaCCodeIdentifier* _tmp84_;
			char* _tmp83_;
			ValaCCodeFunctionCall* _tmp85_;
			ValaCCodeFunctionCall* ccall;
			ValaCCodeIdentifier* _tmp86_;
			ValaCCodeDeclaration* _tmp88_;
			char* _tmp87_;
			ValaCCodeVariableDeclarator* _tmp89_;
			ValaCCodeExpressionStatement* _tmp92_;
			ValaCCodeAssignment* _tmp91_;
			ValaCCodeIdentifier* _tmp90_;
			ValaDataType* _tmp93_;
			ValaArrayType* array_type;
			ValaCCodeIdentifier* _tmp100_;
			ValaCCodeIdentifier* _tmp99_;
			ValaCCodeFunctionCall* _tmp105_;
			ValaCCodeIdentifier* _tmp104_;
			ValaCCodeUnaryExpression* _tmp107_;
			ValaCCodeIdentifier* _tmp106_;
			ValaCCodeUnaryExpression* _tmp109_;
			ValaCCodeIdentifier* _tmp108_;
			ValaCCodeExpressionStatement* _tmp110_;
			ValaCCodeIfStatement* cif;
			ValaCCodeIfStatement* _tmp111_;
			if (!gee_iterator_next (_prop_it)) {
				break;
			}
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			_tmp55_ = FALSE;
			_tmp56_ = FALSE;
			if (vala_property_get_binding (prop) != MEMBER_BINDING_INSTANCE) {
				_tmp56_ = TRUE;
			} else {
				_tmp56_ = vala_property_get_overrides (prop);
			}
			if (_tmp56_) {
				_tmp55_ = TRUE;
			} else {
				_tmp55_ = vala_symbol_get_access ((ValaSymbol*) prop) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			}
			if (_tmp55_) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) prop)) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (vala_property_get_get_accessor (prop) == NULL) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			prop_block = vala_ccode_block_new ();
			postfragment = vala_ccode_fragment_new ();
			vala_ccode_block_add_statement (prop_block, (ValaCCodeNode*) postfragment);
			ccmp = (_tmp58_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp57_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp57_), _tmp58_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp59_ = vala_ccode_identifier_new ("interface_name")));
			_vala_ccode_node_unref0 (_tmp59_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp61_ = vala_ccode_constant_new (_tmp60_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
			_vala_ccode_node_unref0 (_tmp61_);
			_g_free0 (_tmp60_);
			ccheck1 = (_tmp63_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) ccmp, (ValaCCodeExpression*) (_tmp62_ = vala_ccode_constant_new ("0"))), _vala_ccode_node_unref0 (_tmp62_), _tmp63_);
			ccmp = (_tmp65_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp64_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (ccmp), _tmp65_);
			_vala_ccode_node_unref0 (_tmp64_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp66_ = vala_ccode_identifier_new ("property_name")));
			_vala_ccode_node_unref0 (_tmp66_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp69_ = vala_ccode_constant_new (_tmp68_ = g_strdup_printf ("\"%s\"", _tmp67_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) prop))))));
			_vala_ccode_node_unref0 (_tmp69_);
			_g_free0 (_tmp68_);
			_g_free0 (_tmp67_);
			ccheck2 = (_tmp71_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) ccmp, (ValaCCodeExpression*) (_tmp70_ = vala_ccode_constant_new ("0"))), _vala_ccode_node_unref0 (_tmp70_), _tmp71_);
			ccheck = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_AND, (ValaCCodeExpression*) ccheck1, (ValaCCodeExpression*) ccheck2);
			iter_call = (_tmp73_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp72_ = vala_ccode_identifier_new ("dbus_message_iter_open_container"))), _vala_ccode_node_unref0 (iter_call), _tmp73_);
			_vala_ccode_node_unref0 (_tmp72_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp75_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp74_ = vala_ccode_identifier_new ("reply_iter")))));
			_vala_ccode_node_unref0 (_tmp75_);
			_vala_ccode_node_unref0 (_tmp74_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp76_ = vala_ccode_identifier_new ("DBUS_TYPE_VARIANT")));
			_vala_ccode_node_unref0 (_tmp76_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp79_ = vala_ccode_constant_new (_tmp78_ = g_strdup_printf ("\"%s\"", _tmp77_ = vala_dbus_module_get_type_signature (vala_property_get_property_type (prop))))));
			_vala_ccode_node_unref0 (_tmp79_);
			_g_free0 (_tmp78_);
			_g_free0 (_tmp77_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp81_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp80_ = vala_ccode_identifier_new ("subiter")))));
			_vala_ccode_node_unref0 (_tmp81_);
			_vala_ccode_node_unref0 (_tmp80_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp82_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
			_vala_ccode_node_unref0 (_tmp82_);
			ccall = (_tmp85_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp84_ = vala_ccode_identifier_new (_tmp83_ = vala_property_accessor_get_cname (vala_property_get_get_accessor (prop))))), _vala_ccode_node_unref0 (_tmp84_), _g_free0 (_tmp83_), _tmp85_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp86_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp86_);
			_cdecl_ = (_tmp88_ = vala_ccode_declaration_new (_tmp87_ = vala_data_type_get_cname (vala_property_get_property_type (prop))), _vala_ccode_node_unref0 (_cdecl_), _tmp88_);
			_g_free0 (_tmp87_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp89_ = vala_ccode_variable_declarator_new ("result", NULL, NULL)));
			_vala_ccode_node_unref0 (_tmp89_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp92_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp91_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp90_ = vala_ccode_identifier_new ("result")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp92_);
			_vala_ccode_node_unref0 (_tmp91_);
			_vala_ccode_node_unref0 (_tmp90_);
			array_type = _vala_code_node_ref0 ((_tmp93_ = vala_property_get_property_type (prop), VALA_IS_ARRAY_TYPE (_tmp93_) ? ((ValaArrayType*) _tmp93_) : NULL));
			if (array_type != NULL) {
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp94_;
						_tmp94_ = TRUE;
						while (TRUE) {
							char* length_cname;
							ValaCCodeDeclaration* _tmp95_;
							ValaCCodeVariableDeclarator* _tmp96_;
							ValaCCodeUnaryExpression* _tmp98_;
							ValaCCodeIdentifier* _tmp97_;
							if (!_tmp94_) {
								dim++;
							}
							_tmp94_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							length_cname = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, "result", dim);
							_cdecl_ = (_tmp95_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp95_);
							vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp96_ = vala_ccode_variable_declarator_new (length_cname, NULL, NULL)));
							_vala_ccode_node_unref0 (_tmp96_);
							vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
							vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp98_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp97_ = vala_ccode_identifier_new (length_cname)))));
							_vala_ccode_node_unref0 (_tmp98_);
							_vala_ccode_node_unref0 (_tmp97_);
							_g_free0 (length_cname);
						}
					}
				}
			}
			vala_dbus_module_write_expression ((ValaDBusModule*) self, postfragment, vala_property_get_property_type (prop), (ValaCCodeExpression*) (_tmp99_ = vala_ccode_identifier_new ("subiter")), (ValaCCodeExpression*) (_tmp100_ = vala_ccode_identifier_new ("result")));
			_vala_ccode_node_unref0 (_tmp100_);
			_vala_ccode_node_unref0 (_tmp99_);
			if (vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)))) {
				ValaLocalVariable* local;
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp103_;
				ValaCCodeExpression* _tmp102_;
				ValaCCodeIdentifier* _tmp101_;
				local = vala_local_variable_new (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)), " result", NULL, NULL);
				ma = vala_member_access_new_simple ("result", NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
				vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp103_ = vala_ccode_expression_statement_new (_tmp102_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, (ValaCCodeExpression*) (_tmp101_ = vala_ccode_identifier_new ("result")), vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)), (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp103_);
				_vala_ccode_node_unref0 (_tmp102_);
				_vala_ccode_node_unref0 (_tmp101_);
				_vala_code_node_unref0 (local);
				_vala_code_node_unref0 (ma);
			}
			iter_call = (_tmp105_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp104_ = vala_ccode_identifier_new ("dbus_message_iter_close_container"))), _vala_ccode_node_unref0 (iter_call), _tmp105_);
			_vala_ccode_node_unref0 (_tmp104_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp107_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp106_ = vala_ccode_identifier_new ("reply_iter")))));
			_vala_ccode_node_unref0 (_tmp107_);
			_vala_ccode_node_unref0 (_tmp106_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp109_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp108_ = vala_ccode_identifier_new ("subiter")))));
			_vala_ccode_node_unref0 (_tmp109_);
			_vala_ccode_node_unref0 (_tmp108_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp110_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
			_vala_ccode_node_unref0 (_tmp110_);
			cif = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccheck, (ValaCCodeStatement*) prop_block, NULL);
			if (clastif == NULL) {
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) cif);
			} else {
				vala_ccode_if_statement_set_false_statement (clastif, (ValaCCodeStatement*) cif);
			}
			clastif = (_tmp111_ = _vala_ccode_node_ref0 (cif), _vala_ccode_node_unref0 (clastif), _tmp111_);
			_vala_code_node_unref0 (prop);
			_vala_ccode_node_unref0 (prop_block);
			_vala_ccode_node_unref0 (postfragment);
			_vala_ccode_node_unref0 (ccmp);
			_vala_ccode_node_unref0 (ccheck1);
			_vala_ccode_node_unref0 (ccheck2);
			_vala_ccode_node_unref0 (ccheck);
			_vala_ccode_node_unref0 (ccall);
			_vala_code_node_unref0 (array_type);
			_vala_ccode_node_unref0 (cif);
		}
		_gee_collection_object_unref0 (_prop_it);
	}
	free_call = (_tmp113_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp112_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp112_), _tmp113_);
	vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp114_ = vala_ccode_identifier_new ("interface_name")));
	_vala_ccode_node_unref0 (_tmp114_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp115_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
	_vala_ccode_node_unref0 (_tmp115_);
	free_call = (_tmp117_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp116_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (free_call), _tmp117_);
	_vala_ccode_node_unref0 (_tmp116_);
	vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp118_ = vala_ccode_identifier_new ("property_name")));
	_vala_ccode_node_unref0 (_tmp118_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp119_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
	_vala_ccode_node_unref0 (_tmp119_);
	if (clastif == NULL) {
		ValaCCodeBlock* _tmp120_;
		ValaCCodeReturnStatement* _tmp122_;
		ValaCCodeConstant* _tmp121_;
		block = (_tmp120_ = vala_ccode_block_new (), _vala_ccode_node_unref0 (block), _tmp120_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp122_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp121_ = vala_ccode_constant_new ("NULL")))));
		_vala_ccode_node_unref0 (_tmp122_);
		_vala_ccode_node_unref0 (_tmp121_);
	} else {
		ValaCCodeBlock* else_block;
		ValaCCodeIdentifier* _tmp123_;
		ValaCCodeFunctionCall* _tmp124_;
		ValaCCodeFunctionCall* unref_call;
		ValaCCodeIdentifier* _tmp125_;
		ValaCCodeExpressionStatement* _tmp126_;
		ValaCCodeExpressionStatement* _tmp130_;
		ValaCCodeAssignment* _tmp129_;
		ValaCCodeConstant* _tmp128_;
		ValaCCodeIdentifier* _tmp127_;
		else_block = vala_ccode_block_new ();
		unref_call = (_tmp124_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp123_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp123_), _tmp124_);
		vala_ccode_function_call_add_argument (unref_call, (ValaCCodeExpression*) (_tmp125_ = vala_ccode_identifier_new ("reply")));
		_vala_ccode_node_unref0 (_tmp125_);
		vala_ccode_block_add_statement (else_block, (ValaCCodeNode*) (_tmp126_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) unref_call)));
		_vala_ccode_node_unref0 (_tmp126_);
		vala_ccode_block_add_statement (else_block, (ValaCCodeNode*) (_tmp130_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp129_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp127_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) (_tmp128_ = vala_ccode_constant_new ("NULL")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp130_);
		_vala_ccode_node_unref0 (_tmp129_);
		_vala_ccode_node_unref0 (_tmp128_);
		_vala_ccode_node_unref0 (_tmp127_);
		vala_ccode_if_statement_set_false_statement (clastif, (ValaCCodeStatement*) else_block);
		vala_dbus_server_module_handle_reply (self, block);
		_vala_ccode_node_unref0 (else_block);
		_vala_ccode_node_unref0 (unref_call);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp131_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp131_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = wrapper_name;
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (message_signature);
	_vala_ccode_node_unref0 (signature_check);
	_vala_ccode_node_unref0 (signature_error_block);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (target);
	_vala_ccode_node_unref0 (expr);
	_vala_ccode_node_unref0 (clastif);
	_vala_ccode_node_unref0 (free_call);
	return result;
}


static char* vala_dbus_server_module_generate_dbus_property_get_all_wrapper (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* wrapper_name;
	gboolean has_readable_properties;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp8_;
	char* _tmp7_;
	char* _tmp6_;
	ValaCCodeFormalParameter* _tmp9_;
	ValaCCodeFormalParameter* _tmp10_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeDeclaration* _tmp11_;
	ValaCCodeVariableDeclarator* _tmp12_;
	ValaCCodeDeclaration* _tmp13_;
	ValaCCodeVariableDeclarator* _tmp14_;
	ValaCCodeVariableDeclarator* _tmp15_;
	ValaCCodeVariableDeclarator* _tmp16_;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeFunctionCall* _tmp20_;
	ValaCCodeFunctionCall* message_signature;
	ValaCCodeIdentifier* _tmp21_;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeFunctionCall* _tmp23_;
	ValaCCodeFunctionCall* signature_check;
	ValaCCodeConstant* _tmp24_;
	ValaCCodeBlock* signature_error_block;
	ValaCCodeReturnStatement* _tmp26_;
	ValaCCodeIdentifier* _tmp25_;
	ValaCCodeIfStatement* _tmp27_;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeFunctionCall* _tmp29_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp30_;
	ValaCCodeUnaryExpression* _tmp32_;
	ValaCCodeIdentifier* _tmp31_;
	ValaCCodeExpressionStatement* _tmp33_;
	ValaCCodeIdentifier* _tmp34_;
	ValaCCodeFunctionCall* _tmp35_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeIdentifier* _tmp36_;
	ValaCCodeExpressionStatement* _tmp39_;
	ValaCCodeAssignment* _tmp38_;
	ValaCCodeIdentifier* _tmp37_;
	ValaCCodeFunctionCall* _tmp41_;
	ValaCCodeIdentifier* _tmp40_;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeUnaryExpression* _tmp44_;
	ValaCCodeIdentifier* _tmp43_;
	ValaCCodeExpressionStatement* _tmp45_;
	ValaCCodeDeclaration* _tmp46_;
	ValaCCodeVariableDeclarator* _tmp47_;
	ValaCCodeIdentifier* target;
	ValaCCodeIdentifier* _tmp48_;
	ValaCCodeExpression* _tmp49_;
	ValaCCodeExpression* expr;
	ValaCCodeExpressionStatement* _tmp51_;
	ValaCCodeAssignment* _tmp50_;
	ValaCCodeBlock* prop_block;
	ValaCCodeFunctionCall* _tmp55_;
	ValaCCodeIdentifier* _tmp54_;
	ValaCCodeUnaryExpression* _tmp57_;
	ValaCCodeIdentifier* _tmp56_;
	ValaCCodeIdentifier* _tmp58_;
	ValaCCodeConstant* _tmp59_;
	ValaCCodeUnaryExpression* _tmp61_;
	ValaCCodeIdentifier* _tmp60_;
	ValaCCodeExpressionStatement* _tmp62_;
	ValaCCodeFunctionCall* _tmp137_;
	ValaCCodeIdentifier* _tmp136_;
	ValaCCodeUnaryExpression* _tmp139_;
	ValaCCodeIdentifier* _tmp138_;
	ValaCCodeUnaryExpression* _tmp141_;
	ValaCCodeIdentifier* _tmp140_;
	ValaCCodeExpressionStatement* _tmp142_;
	ValaCCodeIdentifier* _tmp143_;
	ValaCCodeFunctionCall* _tmp144_;
	ValaCCodeFunctionCall* ccmp;
	ValaCCodeIdentifier* _tmp145_;
	ValaCCodeConstant* _tmp147_;
	char* _tmp146_;
	ValaCCodeConstant* _tmp148_;
	ValaCCodeBinaryExpression* _tmp149_;
	ValaCCodeBinaryExpression* ccheck;
	ValaCCodeBlock* else_block;
	ValaCCodeIdentifier* _tmp150_;
	ValaCCodeFunctionCall* _tmp151_;
	ValaCCodeFunctionCall* unref_call;
	ValaCCodeIdentifier* _tmp152_;
	ValaCCodeExpressionStatement* _tmp153_;
	ValaCCodeExpressionStatement* _tmp157_;
	ValaCCodeAssignment* _tmp156_;
	ValaCCodeConstant* _tmp155_;
	ValaCCodeIdentifier* _tmp154_;
	ValaCCodeIfStatement* _tmp158_;
	ValaCCodeIdentifier* _tmp159_;
	ValaCCodeFunctionCall* _tmp160_;
	ValaCCodeFunctionCall* free_call;
	ValaCCodeIdentifier* _tmp161_;
	ValaCCodeExpressionStatement* _tmp162_;
	ValaCCodeFunction* _tmp163_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	g_return_val_if_fail (dbus_iface_name != NULL, NULL);
	wrapper_name = (_tmp1_ = g_strdup_printf ("_dbus_%s_property_get_all", _tmp0_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) sym, NULL)), _g_free0 (_tmp0_), _tmp1_);
	has_readable_properties = FALSE;
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _prop_it;
		_prop_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_object_type_symbol_get_properties (sym))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			ValaProperty* prop;
			gboolean _tmp4_;
			gboolean _tmp5_;
			if (!gee_iterator_next (_prop_it)) {
				break;
			}
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			_tmp4_ = FALSE;
			_tmp5_ = FALSE;
			if (vala_property_get_binding (prop) != MEMBER_BINDING_INSTANCE) {
				_tmp5_ = TRUE;
			} else {
				_tmp5_ = vala_property_get_overrides (prop);
			}
			if (_tmp5_) {
				_tmp4_ = TRUE;
			} else {
				_tmp4_ = vala_symbol_get_access ((ValaSymbol*) prop) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			}
			if (_tmp4_) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) prop)) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (vala_property_get_get_accessor (prop) != NULL) {
				has_readable_properties = TRUE;
			}
			_vala_code_node_unref0 (prop);
		}
		_gee_collection_object_unref0 (_prop_it);
	}
	_cdecl_ = NULL;
	function = vala_ccode_function_new (wrapper_name, "DBusHandlerResult");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp8_ = vala_ccode_formal_parameter_new ("self", _tmp7_ = g_strconcat (_tmp6_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) sym, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp8_);
	_g_free0 (_tmp7_);
	_g_free0 (_tmp6_);
	vala_ccode_function_add_parameter (function, _tmp9_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_function_add_parameter (function, _tmp10_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp10_);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	_cdecl_ = (_tmp11_ = vala_ccode_declaration_new ("DBusMessage*"), _vala_ccode_node_unref0 (_cdecl_), _tmp11_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp12_ = vala_ccode_variable_declarator_new ("reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp13_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp13_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp14_ = vala_ccode_variable_declarator_new ("iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp14_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp15_ = vala_ccode_variable_declarator_new ("reply_iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp15_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp16_ = vala_ccode_variable_declarator_new ("subiter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp16_);
	if (has_readable_properties) {
		ValaCCodeVariableDeclarator* _tmp17_;
		ValaCCodeVariableDeclarator* _tmp18_;
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp17_ = vala_ccode_variable_declarator_new ("entry_iter", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp17_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp18_ = vala_ccode_variable_declarator_new ("value_iter", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp18_);
	}
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	message_signature = (_tmp20_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("dbus_message_get_signature"))), _vala_ccode_node_unref0 (_tmp19_), _tmp20_);
	vala_ccode_function_call_add_argument (message_signature, (ValaCCodeExpression*) (_tmp21_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp21_);
	signature_check = (_tmp23_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp22_), _tmp23_);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) message_signature);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_constant_new ("\"s\"")));
	_vala_ccode_node_unref0 (_tmp24_);
	signature_error_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (signature_error_block, (ValaCCodeNode*) (_tmp26_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp25_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp26_);
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp27_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) signature_check, (ValaCCodeStatement*) signature_error_block, NULL)));
	_vala_ccode_node_unref0 (_tmp27_);
	iter_call = (_tmp29_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (_tmp28_), _tmp29_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp30_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp32_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp31_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp32_);
	_vala_ccode_node_unref0 (_tmp31_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp33_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp33_);
	msgcall = (_tmp35_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("dbus_message_new_method_return"))), _vala_ccode_node_unref0 (_tmp34_), _tmp35_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp36_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp36_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp39_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp38_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp37_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp39_);
	_vala_ccode_node_unref0 (_tmp38_);
	_vala_ccode_node_unref0 (_tmp37_);
	iter_call = (_tmp41_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp40_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (iter_call), _tmp41_);
	_vala_ccode_node_unref0 (_tmp40_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("reply")));
	_vala_ccode_node_unref0 (_tmp42_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp44_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp43_ = vala_ccode_identifier_new ("reply_iter")))));
	_vala_ccode_node_unref0 (_tmp44_);
	_vala_ccode_node_unref0 (_tmp43_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp45_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp45_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	_cdecl_ = (_tmp46_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp46_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp47_ = vala_ccode_variable_declarator_new ("interface_name", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp47_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
	target = vala_ccode_identifier_new ("interface_name");
	expr = (_tmp49_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp48_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp48_), _tmp49_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp51_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp50_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp51_);
	_vala_ccode_node_unref0 (_tmp50_);
	if (has_readable_properties) {
		ValaCCodeDeclaration* _tmp52_;
		ValaCCodeVariableDeclarator* _tmp53_;
		_cdecl_ = (_tmp52_ = vala_ccode_declaration_new ("const char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp52_);
		vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp53_ = vala_ccode_variable_declarator_new ("property_name", NULL, NULL)));
		_vala_ccode_node_unref0 (_tmp53_);
		vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
	}
	prop_block = vala_ccode_block_new ();
	iter_call = (_tmp55_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp54_ = vala_ccode_identifier_new ("dbus_message_iter_open_container"))), _vala_ccode_node_unref0 (iter_call), _tmp55_);
	_vala_ccode_node_unref0 (_tmp54_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp57_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp56_ = vala_ccode_identifier_new ("reply_iter")))));
	_vala_ccode_node_unref0 (_tmp57_);
	_vala_ccode_node_unref0 (_tmp56_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp58_ = vala_ccode_identifier_new ("DBUS_TYPE_ARRAY")));
	_vala_ccode_node_unref0 (_tmp58_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp59_ = vala_ccode_constant_new ("\"{sv}\"")));
	_vala_ccode_node_unref0 (_tmp59_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp61_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new ("subiter")))));
	_vala_ccode_node_unref0 (_tmp61_);
	_vala_ccode_node_unref0 (_tmp60_);
	vala_ccode_block_add_statement (prop_block, (ValaCCodeNode*) (_tmp62_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp62_);
	{
		GeeList* _tmp63_;
		GeeIterator* _tmp64_;
		GeeIterator* _prop_it;
		_prop_it = (_tmp64_ = gee_iterable_iterator ((GeeIterable*) (_tmp63_ = vala_object_type_symbol_get_properties (sym))), _gee_collection_object_unref0 (_tmp63_), _tmp64_);
		while (TRUE) {
			ValaProperty* prop;
			gboolean _tmp65_;
			gboolean _tmp66_;
			ValaCCodeBlock* inner_block;
			ValaCCodeFragment* postfragment;
			ValaCCodeFunctionCall* _tmp68_;
			ValaCCodeIdentifier* _tmp67_;
			ValaCCodeUnaryExpression* _tmp70_;
			ValaCCodeIdentifier* _tmp69_;
			ValaCCodeIdentifier* _tmp71_;
			ValaCCodeConstant* _tmp72_;
			ValaCCodeUnaryExpression* _tmp74_;
			ValaCCodeIdentifier* _tmp73_;
			ValaCCodeExpressionStatement* _tmp75_;
			ValaCCodeExpressionStatement* _tmp81_;
			ValaCCodeAssignment* _tmp80_;
			ValaCCodeConstant* _tmp79_;
			char* _tmp78_;
			char* _tmp77_;
			ValaCCodeIdentifier* _tmp76_;
			ValaCCodeFunctionCall* _tmp83_;
			ValaCCodeIdentifier* _tmp82_;
			ValaCCodeUnaryExpression* _tmp85_;
			ValaCCodeIdentifier* _tmp84_;
			ValaCCodeIdentifier* _tmp86_;
			ValaCCodeUnaryExpression* _tmp88_;
			ValaCCodeIdentifier* _tmp87_;
			ValaCCodeExpressionStatement* _tmp89_;
			ValaCCodeFunctionCall* _tmp91_;
			ValaCCodeIdentifier* _tmp90_;
			ValaCCodeUnaryExpression* _tmp93_;
			ValaCCodeIdentifier* _tmp92_;
			ValaCCodeIdentifier* _tmp94_;
			ValaCCodeConstant* _tmp97_;
			char* _tmp96_;
			char* _tmp95_;
			ValaCCodeUnaryExpression* _tmp99_;
			ValaCCodeIdentifier* _tmp98_;
			ValaCCodeExpressionStatement* _tmp100_;
			ValaCCodeIdentifier* _tmp102_;
			char* _tmp101_;
			ValaCCodeFunctionCall* _tmp103_;
			ValaCCodeFunctionCall* ccall;
			ValaCCodeIdentifier* _tmp104_;
			ValaCCodeDeclaration* _tmp106_;
			char* _tmp105_;
			ValaCCodeVariableDeclarator* _tmp107_;
			ValaCCodeExpressionStatement* _tmp110_;
			ValaCCodeAssignment* _tmp109_;
			ValaCCodeIdentifier* _tmp108_;
			ValaDataType* _tmp111_;
			ValaArrayType* array_type;
			ValaCCodeIdentifier* _tmp118_;
			ValaCCodeIdentifier* _tmp117_;
			ValaCCodeFunctionCall* _tmp123_;
			ValaCCodeIdentifier* _tmp122_;
			ValaCCodeUnaryExpression* _tmp125_;
			ValaCCodeIdentifier* _tmp124_;
			ValaCCodeUnaryExpression* _tmp127_;
			ValaCCodeIdentifier* _tmp126_;
			ValaCCodeExpressionStatement* _tmp128_;
			ValaCCodeFunctionCall* _tmp130_;
			ValaCCodeIdentifier* _tmp129_;
			ValaCCodeUnaryExpression* _tmp132_;
			ValaCCodeIdentifier* _tmp131_;
			ValaCCodeUnaryExpression* _tmp134_;
			ValaCCodeIdentifier* _tmp133_;
			ValaCCodeExpressionStatement* _tmp135_;
			if (!gee_iterator_next (_prop_it)) {
				break;
			}
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			_tmp65_ = FALSE;
			_tmp66_ = FALSE;
			if (vala_property_get_binding (prop) != MEMBER_BINDING_INSTANCE) {
				_tmp66_ = TRUE;
			} else {
				_tmp66_ = vala_property_get_overrides (prop);
			}
			if (_tmp66_) {
				_tmp65_ = TRUE;
			} else {
				_tmp65_ = vala_symbol_get_access ((ValaSymbol*) prop) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			}
			if (_tmp65_) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) prop)) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (vala_property_get_get_accessor (prop) == NULL) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			inner_block = vala_ccode_block_new ();
			vala_ccode_block_add_statement (prop_block, (ValaCCodeNode*) inner_block);
			postfragment = vala_ccode_fragment_new ();
			vala_ccode_block_add_statement (inner_block, (ValaCCodeNode*) postfragment);
			iter_call = (_tmp68_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp67_ = vala_ccode_identifier_new ("dbus_message_iter_open_container"))), _vala_ccode_node_unref0 (iter_call), _tmp68_);
			_vala_ccode_node_unref0 (_tmp67_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp70_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp69_ = vala_ccode_identifier_new ("subiter")))));
			_vala_ccode_node_unref0 (_tmp70_);
			_vala_ccode_node_unref0 (_tmp69_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp71_ = vala_ccode_identifier_new ("DBUS_TYPE_DICT_ENTRY")));
			_vala_ccode_node_unref0 (_tmp71_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp72_ = vala_ccode_constant_new ("NULL")));
			_vala_ccode_node_unref0 (_tmp72_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp74_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp73_ = vala_ccode_identifier_new ("entry_iter")))));
			_vala_ccode_node_unref0 (_tmp74_);
			_vala_ccode_node_unref0 (_tmp73_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp75_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
			_vala_ccode_node_unref0 (_tmp75_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp81_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp80_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp76_ = vala_ccode_identifier_new ("property_name")), (ValaCCodeExpression*) (_tmp79_ = vala_ccode_constant_new (_tmp78_ = g_strdup_printf ("\"%s\"", _tmp77_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) prop))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp81_);
			_vala_ccode_node_unref0 (_tmp80_);
			_vala_ccode_node_unref0 (_tmp79_);
			_g_free0 (_tmp78_);
			_g_free0 (_tmp77_);
			_vala_ccode_node_unref0 (_tmp76_);
			iter_call = (_tmp83_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp82_ = vala_ccode_identifier_new ("dbus_message_iter_append_basic"))), _vala_ccode_node_unref0 (iter_call), _tmp83_);
			_vala_ccode_node_unref0 (_tmp82_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp85_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp84_ = vala_ccode_identifier_new ("entry_iter")))));
			_vala_ccode_node_unref0 (_tmp85_);
			_vala_ccode_node_unref0 (_tmp84_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp86_ = vala_ccode_identifier_new ("DBUS_TYPE_STRING")));
			_vala_ccode_node_unref0 (_tmp86_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp88_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp87_ = vala_ccode_identifier_new ("property_name")))));
			_vala_ccode_node_unref0 (_tmp88_);
			_vala_ccode_node_unref0 (_tmp87_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp89_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
			_vala_ccode_node_unref0 (_tmp89_);
			iter_call = (_tmp91_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp90_ = vala_ccode_identifier_new ("dbus_message_iter_open_container"))), _vala_ccode_node_unref0 (iter_call), _tmp91_);
			_vala_ccode_node_unref0 (_tmp90_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp92_ = vala_ccode_identifier_new ("entry_iter")))));
			_vala_ccode_node_unref0 (_tmp93_);
			_vala_ccode_node_unref0 (_tmp92_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp94_ = vala_ccode_identifier_new ("DBUS_TYPE_VARIANT")));
			_vala_ccode_node_unref0 (_tmp94_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp97_ = vala_ccode_constant_new (_tmp96_ = g_strdup_printf ("\"%s\"", _tmp95_ = vala_dbus_module_get_type_signature (vala_property_get_property_type (prop))))));
			_vala_ccode_node_unref0 (_tmp97_);
			_g_free0 (_tmp96_);
			_g_free0 (_tmp95_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp99_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp98_ = vala_ccode_identifier_new ("value_iter")))));
			_vala_ccode_node_unref0 (_tmp99_);
			_vala_ccode_node_unref0 (_tmp98_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp100_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
			_vala_ccode_node_unref0 (_tmp100_);
			ccall = (_tmp103_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp102_ = vala_ccode_identifier_new (_tmp101_ = vala_property_accessor_get_cname (vala_property_get_get_accessor (prop))))), _vala_ccode_node_unref0 (_tmp102_), _g_free0 (_tmp101_), _tmp103_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp104_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp104_);
			_cdecl_ = (_tmp106_ = vala_ccode_declaration_new (_tmp105_ = vala_data_type_get_cname (vala_property_get_property_type (prop))), _vala_ccode_node_unref0 (_cdecl_), _tmp106_);
			_g_free0 (_tmp105_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp107_ = vala_ccode_variable_declarator_new ("result", NULL, NULL)));
			_vala_ccode_node_unref0 (_tmp107_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp110_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp109_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp108_ = vala_ccode_identifier_new ("result")), (ValaCCodeExpression*) ccall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp110_);
			_vala_ccode_node_unref0 (_tmp109_);
			_vala_ccode_node_unref0 (_tmp108_);
			array_type = _vala_code_node_ref0 ((_tmp111_ = vala_property_get_property_type (prop), VALA_IS_ARRAY_TYPE (_tmp111_) ? ((ValaArrayType*) _tmp111_) : NULL));
			if (array_type != NULL) {
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp112_;
						_tmp112_ = TRUE;
						while (TRUE) {
							char* length_cname;
							ValaCCodeDeclaration* _tmp113_;
							ValaCCodeVariableDeclarator* _tmp114_;
							ValaCCodeUnaryExpression* _tmp116_;
							ValaCCodeIdentifier* _tmp115_;
							if (!_tmp112_) {
								dim++;
							}
							_tmp112_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							length_cname = vala_ccode_module_get_array_length_cname ((ValaCCodeModule*) self, "result", dim);
							_cdecl_ = (_tmp113_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp113_);
							vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp114_ = vala_ccode_variable_declarator_new (length_cname, NULL, NULL)));
							_vala_ccode_node_unref0 (_tmp114_);
							vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) _cdecl_);
							vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp116_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp115_ = vala_ccode_identifier_new (length_cname)))));
							_vala_ccode_node_unref0 (_tmp116_);
							_vala_ccode_node_unref0 (_tmp115_);
							_g_free0 (length_cname);
						}
					}
				}
			}
			vala_dbus_module_write_expression ((ValaDBusModule*) self, postfragment, vala_property_get_property_type (prop), (ValaCCodeExpression*) (_tmp117_ = vala_ccode_identifier_new ("value_iter")), (ValaCCodeExpression*) (_tmp118_ = vala_ccode_identifier_new ("result")));
			_vala_ccode_node_unref0 (_tmp118_);
			_vala_ccode_node_unref0 (_tmp117_);
			if (vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)))) {
				ValaLocalVariable* local;
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp121_;
				ValaCCodeExpression* _tmp120_;
				ValaCCodeIdentifier* _tmp119_;
				local = vala_local_variable_new (vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)), " result", NULL, NULL);
				ma = vala_member_access_new_simple ("result", NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
				vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp121_ = vala_ccode_expression_statement_new (_tmp120_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, (ValaCCodeExpression*) (_tmp119_ = vala_ccode_identifier_new ("result")), vala_property_accessor_get_value_type (vala_property_get_get_accessor (prop)), (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp121_);
				_vala_ccode_node_unref0 (_tmp120_);
				_vala_ccode_node_unref0 (_tmp119_);
				_vala_code_node_unref0 (local);
				_vala_code_node_unref0 (ma);
			}
			iter_call = (_tmp123_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp122_ = vala_ccode_identifier_new ("dbus_message_iter_close_container"))), _vala_ccode_node_unref0 (iter_call), _tmp123_);
			_vala_ccode_node_unref0 (_tmp122_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp125_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp124_ = vala_ccode_identifier_new ("entry_iter")))));
			_vala_ccode_node_unref0 (_tmp125_);
			_vala_ccode_node_unref0 (_tmp124_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp127_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp126_ = vala_ccode_identifier_new ("value_iter")))));
			_vala_ccode_node_unref0 (_tmp127_);
			_vala_ccode_node_unref0 (_tmp126_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp128_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
			_vala_ccode_node_unref0 (_tmp128_);
			iter_call = (_tmp130_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp129_ = vala_ccode_identifier_new ("dbus_message_iter_close_container"))), _vala_ccode_node_unref0 (iter_call), _tmp130_);
			_vala_ccode_node_unref0 (_tmp129_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp132_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp131_ = vala_ccode_identifier_new ("subiter")))));
			_vala_ccode_node_unref0 (_tmp132_);
			_vala_ccode_node_unref0 (_tmp131_);
			vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp134_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp133_ = vala_ccode_identifier_new ("entry_iter")))));
			_vala_ccode_node_unref0 (_tmp134_);
			_vala_ccode_node_unref0 (_tmp133_);
			vala_ccode_fragment_append (postfragment, (ValaCCodeNode*) (_tmp135_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
			_vala_ccode_node_unref0 (_tmp135_);
			_vala_code_node_unref0 (prop);
			_vala_ccode_node_unref0 (inner_block);
			_vala_ccode_node_unref0 (postfragment);
			_vala_ccode_node_unref0 (ccall);
			_vala_code_node_unref0 (array_type);
		}
		_gee_collection_object_unref0 (_prop_it);
	}
	iter_call = (_tmp137_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp136_ = vala_ccode_identifier_new ("dbus_message_iter_close_container"))), _vala_ccode_node_unref0 (iter_call), _tmp137_);
	_vala_ccode_node_unref0 (_tmp136_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp139_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp138_ = vala_ccode_identifier_new ("reply_iter")))));
	_vala_ccode_node_unref0 (_tmp139_);
	_vala_ccode_node_unref0 (_tmp138_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp141_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp140_ = vala_ccode_identifier_new ("subiter")))));
	_vala_ccode_node_unref0 (_tmp141_);
	_vala_ccode_node_unref0 (_tmp140_);
	vala_ccode_block_add_statement (prop_block, (ValaCCodeNode*) (_tmp142_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp142_);
	ccmp = (_tmp144_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp143_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp143_), _tmp144_);
	vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp145_ = vala_ccode_identifier_new ("interface_name")));
	_vala_ccode_node_unref0 (_tmp145_);
	vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp147_ = vala_ccode_constant_new (_tmp146_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
	_vala_ccode_node_unref0 (_tmp147_);
	_g_free0 (_tmp146_);
	ccheck = (_tmp149_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) ccmp, (ValaCCodeExpression*) (_tmp148_ = vala_ccode_constant_new ("0"))), _vala_ccode_node_unref0 (_tmp148_), _tmp149_);
	else_block = vala_ccode_block_new ();
	unref_call = (_tmp151_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp150_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp150_), _tmp151_);
	vala_ccode_function_call_add_argument (unref_call, (ValaCCodeExpression*) (_tmp152_ = vala_ccode_identifier_new ("reply")));
	_vala_ccode_node_unref0 (_tmp152_);
	vala_ccode_block_add_statement (else_block, (ValaCCodeNode*) (_tmp153_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) unref_call)));
	_vala_ccode_node_unref0 (_tmp153_);
	vala_ccode_block_add_statement (else_block, (ValaCCodeNode*) (_tmp157_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp156_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp154_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) (_tmp155_ = vala_ccode_constant_new ("NULL")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp157_);
	_vala_ccode_node_unref0 (_tmp156_);
	_vala_ccode_node_unref0 (_tmp155_);
	_vala_ccode_node_unref0 (_tmp154_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp158_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccheck, (ValaCCodeStatement*) prop_block, (ValaCCodeStatement*) else_block)));
	_vala_ccode_node_unref0 (_tmp158_);
	free_call = (_tmp160_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp159_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp159_), _tmp160_);
	vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp161_ = vala_ccode_identifier_new ("interface_name")));
	_vala_ccode_node_unref0 (_tmp161_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp162_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
	_vala_ccode_node_unref0 (_tmp162_);
	vala_dbus_server_module_handle_reply (self, block);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp163_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp163_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = wrapper_name;
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (message_signature);
	_vala_ccode_node_unref0 (signature_check);
	_vala_ccode_node_unref0 (signature_error_block);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (target);
	_vala_ccode_node_unref0 (expr);
	_vala_ccode_node_unref0 (prop_block);
	_vala_ccode_node_unref0 (ccmp);
	_vala_ccode_node_unref0 (ccheck);
	_vala_ccode_node_unref0 (else_block);
	_vala_ccode_node_unref0 (unref_call);
	_vala_ccode_node_unref0 (free_call);
	return result;
}


static char* vala_dbus_server_module_generate_dbus_property_set_wrapper (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, const char* dbus_iface_name) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* wrapper_name;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFormalParameter* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeFragment* prefragment;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp7_;
	ValaCCodeDeclaration* _tmp8_;
	ValaCCodeVariableDeclarator* _tmp9_;
	ValaCCodeVariableDeclarator* _tmp10_;
	ValaCCodeIdentifier* _tmp11_;
	ValaCCodeFunctionCall* _tmp12_;
	ValaCCodeFunctionCall* message_signature;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeIdentifier* _tmp14_;
	ValaCCodeFunctionCall* _tmp15_;
	ValaCCodeFunctionCall* signature_check;
	ValaCCodeConstant* _tmp16_;
	ValaCCodeBlock* signature_error_block;
	ValaCCodeReturnStatement* _tmp18_;
	ValaCCodeIdentifier* _tmp17_;
	ValaCCodeIfStatement* _tmp19_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeFunctionCall* _tmp21_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp22_;
	ValaCCodeUnaryExpression* _tmp24_;
	ValaCCodeIdentifier* _tmp23_;
	ValaCCodeExpressionStatement* _tmp25_;
	ValaCCodeIdentifier* _tmp26_;
	ValaCCodeFunctionCall* _tmp27_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeExpressionStatement* _tmp31_;
	ValaCCodeAssignment* _tmp30_;
	ValaCCodeIdentifier* _tmp29_;
	ValaCCodeDeclaration* _tmp32_;
	ValaCCodeVariableDeclarator* _tmp33_;
	ValaCCodeIdentifier* target;
	ValaCCodeIdentifier* _tmp34_;
	ValaCCodeExpression* _tmp35_;
	ValaCCodeExpression* expr;
	ValaCCodeExpressionStatement* _tmp37_;
	ValaCCodeAssignment* _tmp36_;
	ValaCCodeDeclaration* _tmp38_;
	ValaCCodeVariableDeclarator* _tmp39_;
	ValaCCodeIdentifier* _tmp40_;
	ValaCCodeExpression* _tmp42_;
	ValaCCodeIdentifier* _tmp41_;
	ValaCCodeExpressionStatement* _tmp44_;
	ValaCCodeAssignment* _tmp43_;
	ValaCCodeFunctionCall* _tmp46_;
	ValaCCodeIdentifier* _tmp45_;
	ValaCCodeUnaryExpression* _tmp48_;
	ValaCCodeIdentifier* _tmp47_;
	ValaCCodeUnaryExpression* _tmp50_;
	ValaCCodeIdentifier* _tmp49_;
	ValaCCodeExpressionStatement* _tmp51_;
	ValaCCodeIfStatement* clastif;
	ValaCCodeIdentifier* _tmp97_;
	ValaCCodeFunctionCall* _tmp98_;
	ValaCCodeFunctionCall* free_call;
	ValaCCodeIdentifier* _tmp99_;
	ValaCCodeExpressionStatement* _tmp100_;
	ValaCCodeFunctionCall* _tmp102_;
	ValaCCodeIdentifier* _tmp101_;
	ValaCCodeIdentifier* _tmp103_;
	ValaCCodeExpressionStatement* _tmp104_;
	ValaCCodeFunction* _tmp116_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	g_return_val_if_fail (dbus_iface_name != NULL, NULL);
	wrapper_name = (_tmp1_ = g_strdup_printf ("_dbus_%s_property_set", _tmp0_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) sym, NULL)), _g_free0 (_tmp0_), _tmp1_);
	function = vala_ccode_function_new (wrapper_name, "DBusHandlerResult");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp4_ = vala_ccode_formal_parameter_new ("self", _tmp3_ = g_strconcat (_tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) sym, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp2_);
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_function_add_parameter (function, _tmp6_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp6_);
	block = vala_ccode_block_new ();
	prefragment = vala_ccode_fragment_new ();
	_cdecl_ = vala_ccode_declaration_new ("DBusMessage*");
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp7_ = vala_ccode_variable_declarator_new ("reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp7_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp8_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp8_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp9_ = vala_ccode_variable_declarator_new ("iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp10_ = vala_ccode_variable_declarator_new ("subiter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp10_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	message_signature = (_tmp12_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp11_ = vala_ccode_identifier_new ("dbus_message_get_signature"))), _vala_ccode_node_unref0 (_tmp11_), _tmp12_);
	vala_ccode_function_call_add_argument (message_signature, (ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp13_);
	signature_check = (_tmp15_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp14_), _tmp15_);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) message_signature);
	vala_ccode_function_call_add_argument (signature_check, (ValaCCodeExpression*) (_tmp16_ = vala_ccode_constant_new ("\"ssv\"")));
	_vala_ccode_node_unref0 (_tmp16_);
	signature_error_block = vala_ccode_block_new ();
	vala_ccode_block_add_statement (signature_error_block, (ValaCCodeNode*) (_tmp18_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp18_);
	_vala_ccode_node_unref0 (_tmp17_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp19_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) signature_check, (ValaCCodeStatement*) signature_error_block, NULL)));
	_vala_ccode_node_unref0 (_tmp19_);
	iter_call = (_tmp21_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("dbus_message_iter_init"))), _vala_ccode_node_unref0 (_tmp20_), _tmp21_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp22_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp22_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp24_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp23_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp24_);
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp25_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp25_);
	msgcall = (_tmp27_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp26_ = vala_ccode_identifier_new ("dbus_message_new_method_return"))), _vala_ccode_node_unref0 (_tmp26_), _tmp27_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp28_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp31_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp29_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp31_);
	_vala_ccode_node_unref0 (_tmp30_);
	_vala_ccode_node_unref0 (_tmp29_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) prefragment);
	_cdecl_ = (_tmp32_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp32_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp33_ = vala_ccode_variable_declarator_new ("interface_name", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp33_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
	target = vala_ccode_identifier_new ("interface_name");
	expr = (_tmp35_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (_tmp34_), _tmp35_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp37_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp36_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp37_);
	_vala_ccode_node_unref0 (_tmp36_);
	_cdecl_ = (_tmp38_ = vala_ccode_declaration_new ("char*"), _vala_ccode_node_unref0 (_cdecl_), _tmp38_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp39_ = vala_ccode_variable_declarator_new ("property_name", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp39_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
	target = (_tmp40_ = vala_ccode_identifier_new ("property_name"), _vala_ccode_node_unref0 (target), _tmp40_);
	expr = (_tmp42_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, prefragment, ((ValaCCodeBaseModule*) self)->string_type, (ValaCCodeExpression*) (_tmp41_ = vala_ccode_identifier_new ("iter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (expr), _tmp42_);
	_vala_ccode_node_unref0 (_tmp41_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp44_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp43_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp44_);
	_vala_ccode_node_unref0 (_tmp43_);
	iter_call = (_tmp46_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp45_ = vala_ccode_identifier_new ("dbus_message_iter_recurse"))), _vala_ccode_node_unref0 (iter_call), _tmp46_);
	_vala_ccode_node_unref0 (_tmp45_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp48_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp47_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp48_);
	_vala_ccode_node_unref0 (_tmp47_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp50_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("subiter")))));
	_vala_ccode_node_unref0 (_tmp50_);
	_vala_ccode_node_unref0 (_tmp49_);
	vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp51_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp51_);
	clastif = NULL;
	{
		GeeList* _tmp52_;
		GeeIterator* _tmp53_;
		GeeIterator* _prop_it;
		_prop_it = (_tmp53_ = gee_iterable_iterator ((GeeIterable*) (_tmp52_ = vala_object_type_symbol_get_properties (sym))), _gee_collection_object_unref0 (_tmp52_), _tmp53_);
		while (TRUE) {
			ValaProperty* prop;
			gboolean _tmp54_;
			gboolean _tmp55_;
			ValaCCodeBlock* prop_block;
			ValaCCodeFragment* _tmp56_;
			ValaCCodeIdentifier* _tmp57_;
			ValaCCodeFunctionCall* _tmp58_;
			ValaCCodeFunctionCall* ccmp;
			ValaCCodeIdentifier* _tmp59_;
			ValaCCodeConstant* _tmp61_;
			char* _tmp60_;
			ValaCCodeConstant* _tmp62_;
			ValaCCodeBinaryExpression* _tmp63_;
			ValaCCodeBinaryExpression* ccheck1;
			ValaCCodeFunctionCall* _tmp65_;
			ValaCCodeIdentifier* _tmp64_;
			ValaCCodeIdentifier* _tmp66_;
			ValaCCodeConstant* _tmp69_;
			char* _tmp68_;
			char* _tmp67_;
			ValaCCodeConstant* _tmp70_;
			ValaCCodeBinaryExpression* _tmp71_;
			ValaCCodeBinaryExpression* ccheck2;
			ValaCCodeBinaryExpression* ccheck;
			ValaDataType* owned_type;
			ValaCCodeDeclaration* _tmp73_;
			char* _tmp72_;
			ValaCCodeVariableDeclarator* _tmp74_;
			ValaCCodeIdentifier* _tmp75_;
			ValaCCodeExpression* _tmp77_;
			ValaCCodeIdentifier* _tmp76_;
			ValaCCodeExpressionStatement* _tmp79_;
			ValaCCodeAssignment* _tmp78_;
			ValaCCodeIdentifier* _tmp81_;
			char* _tmp80_;
			ValaCCodeFunctionCall* _tmp82_;
			ValaCCodeFunctionCall* ccall;
			ValaCCodeIdentifier* _tmp83_;
			ValaCCodeIdentifier* _tmp84_;
			ValaDataType* _tmp85_;
			ValaArrayType* array_type;
			ValaCCodeExpressionStatement* _tmp92_;
			ValaCCodeIfStatement* cif;
			ValaCCodeIfStatement* _tmp96_;
			if (!gee_iterator_next (_prop_it)) {
				break;
			}
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			_tmp54_ = FALSE;
			_tmp55_ = FALSE;
			if (vala_property_get_binding (prop) != MEMBER_BINDING_INSTANCE) {
				_tmp55_ = TRUE;
			} else {
				_tmp55_ = vala_property_get_overrides (prop);
			}
			if (_tmp55_) {
				_tmp54_ = TRUE;
			} else {
				_tmp54_ = vala_symbol_get_access ((ValaSymbol*) prop) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			}
			if (_tmp54_) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) prop)) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (vala_property_get_set_accessor (prop) == NULL) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			prop_block = vala_ccode_block_new ();
			prefragment = (_tmp56_ = vala_ccode_fragment_new (), _vala_ccode_node_unref0 (prefragment), _tmp56_);
			vala_ccode_block_add_statement (prop_block, (ValaCCodeNode*) prefragment);
			ccmp = (_tmp58_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp57_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (_tmp57_), _tmp58_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp59_ = vala_ccode_identifier_new ("interface_name")));
			_vala_ccode_node_unref0 (_tmp59_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp61_ = vala_ccode_constant_new (_tmp60_ = g_strdup_printf ("\"%s\"", dbus_iface_name))));
			_vala_ccode_node_unref0 (_tmp61_);
			_g_free0 (_tmp60_);
			ccheck1 = (_tmp63_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) ccmp, (ValaCCodeExpression*) (_tmp62_ = vala_ccode_constant_new ("0"))), _vala_ccode_node_unref0 (_tmp62_), _tmp63_);
			ccmp = (_tmp65_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp64_ = vala_ccode_identifier_new ("strcmp"))), _vala_ccode_node_unref0 (ccmp), _tmp65_);
			_vala_ccode_node_unref0 (_tmp64_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp66_ = vala_ccode_identifier_new ("property_name")));
			_vala_ccode_node_unref0 (_tmp66_);
			vala_ccode_function_call_add_argument (ccmp, (ValaCCodeExpression*) (_tmp69_ = vala_ccode_constant_new (_tmp68_ = g_strdup_printf ("\"%s\"", _tmp67_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) prop))))));
			_vala_ccode_node_unref0 (_tmp69_);
			_g_free0 (_tmp68_);
			_g_free0 (_tmp67_);
			ccheck2 = (_tmp71_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) ccmp, (ValaCCodeExpression*) (_tmp70_ = vala_ccode_constant_new ("0"))), _vala_ccode_node_unref0 (_tmp70_), _tmp71_);
			ccheck = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_AND, (ValaCCodeExpression*) ccheck1, (ValaCCodeExpression*) ccheck2);
			owned_type = vala_data_type_copy (vala_property_get_property_type (prop));
			vala_data_type_set_value_owned (owned_type, TRUE);
			_cdecl_ = (_tmp73_ = vala_ccode_declaration_new (_tmp72_ = vala_data_type_get_cname (owned_type)), _vala_ccode_node_unref0 (_cdecl_), _tmp73_);
			_g_free0 (_tmp72_);
			vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp74_ = vala_ccode_variable_declarator_new ("value", NULL, NULL)));
			_vala_ccode_node_unref0 (_tmp74_);
			vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
			target = (_tmp75_ = vala_ccode_identifier_new ("value"), _vala_ccode_node_unref0 (target), _tmp75_);
			expr = (_tmp77_ = vala_dbus_module_read_expression ((ValaDBusModule*) self, prefragment, vala_property_get_property_type (prop), (ValaCCodeExpression*) (_tmp76_ = vala_ccode_identifier_new ("subiter")), (ValaCCodeExpression*) target), _vala_ccode_node_unref0 (expr), _tmp77_);
			_vala_ccode_node_unref0 (_tmp76_);
			vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) (_tmp79_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp78_ = vala_ccode_assignment_new ((ValaCCodeExpression*) target, expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			_vala_ccode_node_unref0 (_tmp79_);
			_vala_ccode_node_unref0 (_tmp78_);
			ccall = (_tmp82_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp81_ = vala_ccode_identifier_new (_tmp80_ = vala_property_accessor_get_cname (vala_property_get_set_accessor (prop))))), _vala_ccode_node_unref0 (_tmp81_), _g_free0 (_tmp80_), _tmp82_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp83_ = vala_ccode_identifier_new ("self")));
			_vala_ccode_node_unref0 (_tmp83_);
			vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp84_ = vala_ccode_identifier_new ("value")));
			_vala_ccode_node_unref0 (_tmp84_);
			array_type = _vala_code_node_ref0 ((_tmp85_ = vala_property_get_property_type (prop), VALA_IS_ARRAY_TYPE (_tmp85_) ? ((ValaArrayType*) _tmp85_) : NULL));
			if (array_type != NULL) {
				{
					gint dim;
					dim = 1;
					{
						gboolean _tmp86_;
						_tmp86_ = TRUE;
						while (TRUE) {
							ValaCCodeDeclaration* _tmp87_;
							ValaCCodeVariableDeclarator* _tmp89_;
							char* _tmp88_;
							ValaCCodeIdentifier* _tmp91_;
							char* _tmp90_;
							if (!_tmp86_) {
								dim++;
							}
							_tmp86_ = FALSE;
							if (!(dim <= vala_array_type_get_rank (array_type))) {
								break;
							}
							_cdecl_ = (_tmp87_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp87_);
							vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp89_ = vala_ccode_variable_declarator_new (_tmp88_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), "value", dim), NULL, NULL)));
							_vala_ccode_node_unref0 (_tmp89_);
							_g_free0 (_tmp88_);
							vala_ccode_fragment_append (prefragment, (ValaCCodeNode*) _cdecl_);
							vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp91_ = vala_ccode_identifier_new (_tmp90_ = vala_ccode_module_get_array_length_cname (vala_ccode_module_get_head ((ValaCCodeModule*) self), "value", dim))));
							_vala_ccode_node_unref0 (_tmp91_);
							_g_free0 (_tmp90_);
						}
					}
				}
			}
			vala_ccode_block_add_statement (prop_block, (ValaCCodeNode*) (_tmp92_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) ccall)));
			_vala_ccode_node_unref0 (_tmp92_);
			if (vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, owned_type)) {
				ValaLocalVariable* local;
				ValaMemberAccess* ma;
				ValaCCodeExpressionStatement* _tmp95_;
				ValaCCodeExpression* _tmp94_;
				ValaCCodeIdentifier* _tmp93_;
				local = vala_local_variable_new (owned_type, "value", NULL, NULL);
				ma = vala_member_access_new_simple ("value", NULL);
				vala_expression_set_symbol_reference ((ValaExpression*) ma, (ValaSymbol*) local);
				vala_ccode_block_add_statement (prop_block, (ValaCCodeNode*) (_tmp95_ = vala_ccode_expression_statement_new (_tmp94_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_identifier_new ("value")), owned_type, (ValaExpression*) ma, FALSE))));
				_vala_ccode_node_unref0 (_tmp95_);
				_vala_ccode_node_unref0 (_tmp94_);
				_vala_ccode_node_unref0 (_tmp93_);
				_vala_code_node_unref0 (local);
				_vala_code_node_unref0 (ma);
			}
			cif = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccheck, (ValaCCodeStatement*) prop_block, NULL);
			if (clastif == NULL) {
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) cif);
			} else {
				vala_ccode_if_statement_set_false_statement (clastif, (ValaCCodeStatement*) cif);
			}
			clastif = (_tmp96_ = _vala_ccode_node_ref0 (cif), _vala_ccode_node_unref0 (clastif), _tmp96_);
			_vala_code_node_unref0 (prop);
			_vala_ccode_node_unref0 (prop_block);
			_vala_ccode_node_unref0 (ccmp);
			_vala_ccode_node_unref0 (ccheck1);
			_vala_ccode_node_unref0 (ccheck2);
			_vala_ccode_node_unref0 (ccheck);
			_vala_code_node_unref0 (owned_type);
			_vala_ccode_node_unref0 (ccall);
			_vala_code_node_unref0 (array_type);
			_vala_ccode_node_unref0 (cif);
		}
		_gee_collection_object_unref0 (_prop_it);
	}
	free_call = (_tmp98_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp97_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (_tmp97_), _tmp98_);
	vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp99_ = vala_ccode_identifier_new ("interface_name")));
	_vala_ccode_node_unref0 (_tmp99_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp100_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
	_vala_ccode_node_unref0 (_tmp100_);
	free_call = (_tmp102_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp101_ = vala_ccode_identifier_new ("g_free"))), _vala_ccode_node_unref0 (free_call), _tmp102_);
	_vala_ccode_node_unref0 (_tmp101_);
	vala_ccode_function_call_add_argument (free_call, (ValaCCodeExpression*) (_tmp103_ = vala_ccode_identifier_new ("property_name")));
	_vala_ccode_node_unref0 (_tmp103_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp104_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) free_call)));
	_vala_ccode_node_unref0 (_tmp104_);
	if (clastif == NULL) {
		ValaCCodeBlock* _tmp105_;
		ValaCCodeReturnStatement* _tmp107_;
		ValaCCodeConstant* _tmp106_;
		block = (_tmp105_ = vala_ccode_block_new (), _vala_ccode_node_unref0 (block), _tmp105_);
		vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp107_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp106_ = vala_ccode_constant_new ("NULL")))));
		_vala_ccode_node_unref0 (_tmp107_);
		_vala_ccode_node_unref0 (_tmp106_);
	} else {
		ValaCCodeBlock* else_block;
		ValaCCodeIdentifier* _tmp108_;
		ValaCCodeFunctionCall* _tmp109_;
		ValaCCodeFunctionCall* unref_call;
		ValaCCodeIdentifier* _tmp110_;
		ValaCCodeExpressionStatement* _tmp111_;
		ValaCCodeExpressionStatement* _tmp115_;
		ValaCCodeAssignment* _tmp114_;
		ValaCCodeIdentifier* _tmp113_;
		ValaCCodeIdentifier* _tmp112_;
		else_block = vala_ccode_block_new ();
		unref_call = (_tmp109_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp108_ = vala_ccode_identifier_new ("dbus_message_unref"))), _vala_ccode_node_unref0 (_tmp108_), _tmp109_);
		vala_ccode_function_call_add_argument (unref_call, (ValaCCodeExpression*) (_tmp110_ = vala_ccode_identifier_new ("reply")));
		_vala_ccode_node_unref0 (_tmp110_);
		vala_ccode_block_add_statement (else_block, (ValaCCodeNode*) (_tmp111_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) unref_call)));
		_vala_ccode_node_unref0 (_tmp111_);
		vala_ccode_block_add_statement (else_block, (ValaCCodeNode*) (_tmp115_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp114_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp112_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) (_tmp113_ = vala_ccode_identifier_new ("NULL")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
		_vala_ccode_node_unref0 (_tmp115_);
		_vala_ccode_node_unref0 (_tmp114_);
		_vala_ccode_node_unref0 (_tmp113_);
		_vala_ccode_node_unref0 (_tmp112_);
		vala_ccode_if_statement_set_false_statement (clastif, (ValaCCodeStatement*) else_block);
		vala_dbus_server_module_handle_reply (self, block);
		_vala_ccode_node_unref0 (else_block);
		_vala_ccode_node_unref0 (unref_call);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp116_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp116_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = wrapper_name;
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (prefragment);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (message_signature);
	_vala_ccode_node_unref0 (signature_check);
	_vala_ccode_node_unref0 (signature_error_block);
	_vala_ccode_node_unref0 (iter_call);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (target);
	_vala_ccode_node_unref0 (expr);
	_vala_ccode_node_unref0 (clastif);
	_vala_ccode_node_unref0 (free_call);
	return result;
}


static char* vala_dbus_server_module_get_dbus_type_introspection (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym) {
	char* result;
	char* _result_;
	ValaObjectTypeSymbol* _tmp0_;
	ValaClass* cl;
	char* dbus_iface_name;
	char* _tmp7_;
	char* _tmp6_;
	char* _tmp50_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	_result_ = g_strdup ("");
	cl = _vala_code_node_ref0 ((_tmp0_ = sym, VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
	if (cl != NULL) {
		{
			GeeList* _tmp1_;
			GeeIterator* _tmp2_;
			GeeIterator* _base_type_it;
			_base_type_it = (_tmp2_ = gee_iterable_iterator ((GeeIterable*) (_tmp1_ = vala_class_get_base_types (cl))), _gee_collection_object_unref0 (_tmp1_), _tmp2_);
			while (TRUE) {
				ValaDataType* base_type;
				ValaDataType* _tmp3_;
				ValaObjectType* base_obj_type;
				char* _tmp5_;
				char* _tmp4_;
				if (!gee_iterator_next (_base_type_it)) {
					break;
				}
				base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
				base_obj_type = _vala_code_node_ref0 ((_tmp3_ = base_type, VALA_IS_OBJECT_TYPE (_tmp3_) ? ((ValaObjectType*) _tmp3_) : NULL));
				_result_ = (_tmp5_ = g_strconcat (_result_, _tmp4_ = vala_dbus_server_module_get_dbus_type_introspection (self, vala_object_type_get_type_symbol (base_obj_type)), NULL), _g_free0 (_result_), _tmp5_);
				_g_free0 (_tmp4_);
				_vala_code_node_unref0 (base_type);
				_vala_code_node_unref0 (base_obj_type);
			}
			_gee_collection_object_unref0 (_base_type_it);
		}
	}
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) sym);
	if (dbus_iface_name == NULL) {
		result = _result_;
		_vala_code_node_unref0 (cl);
		_g_free0 (dbus_iface_name);
		return result;
	}
	_result_ = (_tmp7_ = g_strconcat (_result_, _tmp6_ = g_strdup_printf ("<interface name=\"%s\">\n", dbus_iface_name), NULL), _g_free0 (_result_), _tmp7_);
	_g_free0 (_tmp6_);
	{
		GeeList* _tmp8_;
		GeeIterator* _tmp9_;
		GeeIterator* _m_it;
		_m_it = (_tmp9_ = gee_iterable_iterator ((GeeIterable*) (_tmp8_ = vala_object_type_symbol_get_methods (sym))), _gee_collection_object_unref0 (_tmp8_), _tmp9_);
		while (TRUE) {
			ValaMethod* m;
			gboolean _tmp10_;
			gboolean _tmp11_;
			gboolean _tmp12_;
			char* _tmp15_;
			char* _tmp14_;
			char* _tmp13_;
			char* _tmp28_;
			if (!gee_iterator_next (_m_it)) {
				break;
			}
			m = (ValaMethod*) gee_iterator_get (_m_it);
			_tmp10_ = FALSE;
			_tmp11_ = FALSE;
			_tmp12_ = FALSE;
			if (VALA_IS_CREATION_METHOD (m)) {
				_tmp12_ = TRUE;
			} else {
				_tmp12_ = vala_method_get_binding (m) != MEMBER_BINDING_INSTANCE;
			}
			if (_tmp12_) {
				_tmp11_ = TRUE;
			} else {
				_tmp11_ = vala_method_get_overrides (m);
			}
			if (_tmp11_) {
				_tmp10_ = TRUE;
			} else {
				_tmp10_ = vala_symbol_get_access ((ValaSymbol*) m) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			}
			if (_tmp10_) {
				_vala_code_node_unref0 (m);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) m)) {
				_vala_code_node_unref0 (m);
				continue;
			}
			_result_ = (_tmp15_ = g_strconcat (_result_, _tmp14_ = g_strdup_printf ("  <method name=\"%s\">\n", _tmp13_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) m))), NULL), _g_free0 (_result_), _tmp15_);
			_g_free0 (_tmp14_);
			_g_free0 (_tmp13_);
			{
				GeeList* _tmp16_;
				GeeIterator* _tmp17_;
				GeeIterator* _param_it;
				_param_it = (_tmp17_ = gee_iterable_iterator ((GeeIterable*) (_tmp16_ = vala_method_get_parameters (m))), _gee_collection_object_unref0 (_tmp16_), _tmp17_);
				while (TRUE) {
					ValaFormalParameter* param;
					gboolean _tmp18_;
					const char* _tmp20_;
					char* direction;
					char* _tmp23_;
					char* _tmp22_;
					char* _tmp21_;
					if (!gee_iterator_next (_param_it)) {
						break;
					}
					param = (ValaFormalParameter*) gee_iterator_get (_param_it);
					_tmp18_ = FALSE;
					if (vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param)) != NULL) {
						char* _tmp19_;
						_tmp18_ = _vala_strcmp0 (_tmp19_ = vala_symbol_get_full_name ((ValaSymbol*) vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (param))), "DBus.BusName") == 0;
						_g_free0 (_tmp19_);
					} else {
						_tmp18_ = FALSE;
					}
					if (_tmp18_) {
						_vala_code_node_unref0 (param);
						continue;
					}
					_tmp20_ = NULL;
					if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_IN) {
						_tmp20_ = "in";
					} else {
						_tmp20_ = "out";
					}
					direction = g_strdup (_tmp20_);
					_result_ = (_tmp23_ = g_strconcat (_result_, _tmp22_ = g_strdup_printf ("    <arg name=\"%s\" type=\"%s\" direction=\"%s\"/>\n", vala_symbol_get_name ((ValaSymbol*) param), _tmp21_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param)), direction), NULL), _g_free0 (_result_), _tmp23_);
					_g_free0 (_tmp22_);
					_g_free0 (_tmp21_);
					_vala_code_node_unref0 (param);
					_g_free0 (direction);
				}
				_gee_collection_object_unref0 (_param_it);
			}
			if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
				char* _tmp27_;
				char* _tmp26_;
				char* _tmp25_;
				char* _tmp24_;
				_result_ = (_tmp27_ = g_strconcat (_result_, _tmp26_ = g_strdup_printf ("    <arg name=\"%s\" type=\"%s\" direction=\"out\"/>\n", _tmp24_ = vala_dbus_server_module_dbus_result_name (self, m), _tmp25_ = vala_dbus_module_get_type_signature (vala_method_get_return_type (m))), NULL), _g_free0 (_result_), _tmp27_);
				_g_free0 (_tmp26_);
				_g_free0 (_tmp25_);
				_g_free0 (_tmp24_);
			}
			_result_ = (_tmp28_ = g_strconcat (_result_, "  </method>\n", NULL), _g_free0 (_result_), _tmp28_);
			_vala_code_node_unref0 (m);
		}
		_gee_collection_object_unref0 (_m_it);
	}
	{
		GeeList* _tmp29_;
		GeeIterator* _tmp30_;
		GeeIterator* _prop_it;
		_prop_it = (_tmp30_ = gee_iterable_iterator ((GeeIterable*) (_tmp29_ = vala_object_type_symbol_get_properties (sym))), _gee_collection_object_unref0 (_tmp29_), _tmp30_);
		while (TRUE) {
			ValaProperty* prop;
			gboolean _tmp31_;
			gboolean _tmp32_;
			const char* _tmp33_;
			const char* _tmp34_;
			char* access;
			char* _tmp38_;
			char* _tmp37_;
			char* _tmp36_;
			char* _tmp35_;
			if (!gee_iterator_next (_prop_it)) {
				break;
			}
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			_tmp31_ = FALSE;
			_tmp32_ = FALSE;
			if (vala_property_get_binding (prop) != MEMBER_BINDING_INSTANCE) {
				_tmp32_ = TRUE;
			} else {
				_tmp32_ = vala_property_get_overrides (prop);
			}
			if (_tmp32_) {
				_tmp31_ = TRUE;
			} else {
				_tmp31_ = vala_symbol_get_access ((ValaSymbol*) prop) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			}
			if (_tmp31_) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) prop)) {
				_vala_code_node_unref0 (prop);
				continue;
			}
			_tmp33_ = NULL;
			if (vala_property_get_get_accessor (prop) != NULL) {
				_tmp33_ = "read";
			} else {
				_tmp33_ = "";
			}
			_tmp34_ = NULL;
			if (vala_property_get_set_accessor (prop) != NULL) {
				_tmp34_ = "write";
			} else {
				_tmp34_ = "";
			}
			access = g_strconcat (_tmp33_, _tmp34_, NULL);
			_result_ = (_tmp38_ = g_strconcat (_result_, _tmp37_ = g_strdup_printf ("  <property name=\"%s\" type=\"%s\" access=\"%s\"/>\n", _tmp35_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) prop)), _tmp36_ = vala_dbus_module_get_type_signature (vala_property_get_property_type (prop)), access), NULL), _g_free0 (_result_), _tmp38_);
			_g_free0 (_tmp37_);
			_g_free0 (_tmp36_);
			_g_free0 (_tmp35_);
			_vala_code_node_unref0 (prop);
			_g_free0 (access);
		}
		_gee_collection_object_unref0 (_prop_it);
	}
	{
		GeeList* _tmp39_;
		GeeIterator* _tmp40_;
		GeeIterator* _sig_it;
		_sig_it = (_tmp40_ = gee_iterable_iterator ((GeeIterable*) (_tmp39_ = vala_object_type_symbol_get_signals (sym))), _gee_collection_object_unref0 (_tmp39_), _tmp40_);
		while (TRUE) {
			ValaSignal* sig;
			char* _tmp43_;
			char* _tmp42_;
			char* _tmp41_;
			char* _tmp49_;
			if (!gee_iterator_next (_sig_it)) {
				break;
			}
			sig = (ValaSignal*) gee_iterator_get (_sig_it);
			if (vala_symbol_get_access ((ValaSymbol*) sig) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC) {
				_vala_code_node_unref0 (sig);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) sig)) {
				_vala_code_node_unref0 (sig);
				continue;
			}
			_result_ = (_tmp43_ = g_strconcat (_result_, _tmp42_ = g_strdup_printf ("  <signal name=\"%s\">\n", _tmp41_ = vala_symbol_lower_case_to_camel_case (vala_symbol_get_name ((ValaSymbol*) sig))), NULL), _g_free0 (_result_), _tmp43_);
			_g_free0 (_tmp42_);
			_g_free0 (_tmp41_);
			{
				GeeList* _tmp44_;
				GeeIterator* _tmp45_;
				GeeIterator* _param_it;
				_param_it = (_tmp45_ = gee_iterable_iterator ((GeeIterable*) (_tmp44_ = vala_signal_get_parameters (sig))), _gee_collection_object_unref0 (_tmp44_), _tmp45_);
				while (TRUE) {
					ValaFormalParameter* param;
					char* _tmp48_;
					char* _tmp47_;
					char* _tmp46_;
					if (!gee_iterator_next (_param_it)) {
						break;
					}
					param = (ValaFormalParameter*) gee_iterator_get (_param_it);
					_result_ = (_tmp48_ = g_strconcat (_result_, _tmp47_ = g_strdup_printf ("    <arg name=\"%s\" type=\"%s\"/>\n", vala_symbol_get_name ((ValaSymbol*) param), _tmp46_ = vala_dbus_module_get_type_signature (vala_formal_parameter_get_parameter_type (param))), NULL), _g_free0 (_result_), _tmp48_);
					_g_free0 (_tmp47_);
					_g_free0 (_tmp46_);
					_vala_code_node_unref0 (param);
				}
				_gee_collection_object_unref0 (_param_it);
			}
			_result_ = (_tmp49_ = g_strconcat (_result_, "  </signal>\n", NULL), _g_free0 (_result_), _tmp49_);
			_vala_code_node_unref0 (sig);
		}
		_gee_collection_object_unref0 (_sig_it);
	}
	_result_ = (_tmp50_ = g_strconcat (_result_, "</interface>\n", NULL), _g_free0 (_result_), _tmp50_);
	result = _result_;
	_vala_code_node_unref0 (cl);
	_g_free0 (dbus_iface_name);
	return result;
}


static char* vala_dbus_server_module_generate_dbus_introspect (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym) {
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* wrapper_name;
	ValaCCodeFunction* function;
	ValaCCodeFormalParameter* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFormalParameter* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp7_;
	ValaCCodeDeclaration* _tmp8_;
	ValaCCodeVariableDeclarator* _tmp9_;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeFunctionCall* _tmp11_;
	ValaCCodeFunctionCall* msgcall;
	ValaCCodeIdentifier* _tmp12_;
	ValaCCodeExpressionStatement* _tmp15_;
	ValaCCodeAssignment* _tmp14_;
	ValaCCodeIdentifier* _tmp13_;
	ValaCCodeIdentifier* _tmp16_;
	ValaCCodeFunctionCall* _tmp17_;
	ValaCCodeFunctionCall* iter_call;
	ValaCCodeIdentifier* _tmp18_;
	ValaCCodeUnaryExpression* _tmp20_;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeExpressionStatement* _tmp21_;
	ValaCCodeDeclaration* _tmp22_;
	ValaCCodeVariableDeclarator* _tmp23_;
	ValaCCodeDeclaration* _tmp24_;
	ValaCCodeVariableDeclarator* _tmp25_;
	ValaCCodeDeclaration* _tmp26_;
	ValaCCodeVariableDeclarator* _tmp27_;
	char* xml_data;
	ValaCCodeIdentifier* _tmp28_;
	ValaCCodeFunctionCall* _tmp29_;
	ValaCCodeFunctionCall* str_call;
	ValaCCodeConstant* _tmp32_;
	char* _tmp31_;
	char* _tmp30_;
	ValaCCodeExpressionStatement* _tmp35_;
	ValaCCodeAssignment* _tmp34_;
	ValaCCodeIdentifier* _tmp33_;
	char* _tmp36_;
	char* _tmp37_;
	char* _tmp39_;
	char* _tmp38_;
	ValaCCodeFunctionCall* _tmp41_;
	ValaCCodeIdentifier* _tmp40_;
	ValaCCodeIdentifier* _tmp42_;
	ValaCCodeConstant* _tmp45_;
	char* _tmp44_;
	char* _tmp43_;
	ValaCCodeExpressionStatement* _tmp46_;
	ValaCCodeIdentifier* _tmp47_;
	ValaCCodeFunctionCall* _tmp48_;
	ValaCCodeFunctionCall* get_path;
	ValaCCodeCastExpression* _tmp50_;
	ValaCCodeIdentifier* _tmp49_;
	ValaCCodeConstant* _tmp51_;
	ValaCCodeIdentifier* _tmp52_;
	ValaCCodeFunctionCall* _tmp53_;
	ValaCCodeFunctionCall* list_call;
	ValaCCodeIdentifier* _tmp54_;
	ValaCCodeUnaryExpression* _tmp56_;
	ValaCCodeIdentifier* _tmp55_;
	ValaCCodeExpressionStatement* _tmp57_;
	ValaCCodeBlock* child_block;
	ValaCCodeFunctionCall* _tmp59_;
	ValaCCodeIdentifier* _tmp58_;
	ValaCCodeIdentifier* _tmp60_;
	ValaCCodeConstant* _tmp63_;
	char* _tmp62_;
	char* _tmp61_;
	ValaCCodeElementAccess* _tmp66_;
	ValaCCodeIdentifier* _tmp65_;
	ValaCCodeIdentifier* _tmp64_;
	ValaCCodeExpressionStatement* _tmp67_;
	ValaCCodeElementAccess* _tmp70_;
	ValaCCodeIdentifier* _tmp69_;
	ValaCCodeIdentifier* _tmp68_;
	ValaCCodeForStatement* _tmp71_;
	ValaCCodeForStatement* cfor;
	ValaCCodeAssignment* _tmp74_;
	ValaCCodeConstant* _tmp73_;
	ValaCCodeIdentifier* _tmp72_;
	ValaCCodeUnaryExpression* _tmp76_;
	ValaCCodeIdentifier* _tmp75_;
	ValaCCodeIdentifier* _tmp77_;
	ValaCCodeFunctionCall* _tmp78_;
	ValaCCodeFunctionCall* list_free_call;
	ValaCCodeIdentifier* _tmp79_;
	ValaCCodeExpressionStatement* _tmp80_;
	char* _tmp81_;
	ValaCCodeFunctionCall* _tmp83_;
	ValaCCodeIdentifier* _tmp82_;
	ValaCCodeIdentifier* _tmp84_;
	ValaCCodeConstant* _tmp87_;
	char* _tmp86_;
	char* _tmp85_;
	ValaCCodeExpressionStatement* _tmp88_;
	ValaCCodeFunctionCall* _tmp90_;
	ValaCCodeIdentifier* _tmp89_;
	ValaCCodeUnaryExpression* _tmp92_;
	ValaCCodeIdentifier* _tmp91_;
	ValaCCodeIdentifier* _tmp93_;
	ValaCCodeUnaryExpression* _tmp96_;
	ValaCCodeMemberAccess* _tmp95_;
	ValaCCodeIdentifier* _tmp94_;
	ValaCCodeExpressionStatement* _tmp97_;
	ValaCCodeFunctionCall* _tmp99_;
	ValaCCodeIdentifier* _tmp98_;
	ValaCCodeIdentifier* _tmp100_;
	ValaCCodeConstant* _tmp101_;
	ValaCCodeExpressionStatement* _tmp102_;
	ValaCCodeFunction* _tmp103_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sym != NULL, NULL);
	wrapper_name = (_tmp1_ = g_strdup_printf ("_dbus_%s_introspect", _tmp0_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) sym, NULL)), _g_free0 (_tmp0_), _tmp1_);
	function = vala_ccode_function_new (wrapper_name, "DBusHandlerResult");
	vala_ccode_function_set_modifiers (function, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_function_add_parameter (function, _tmp4_ = vala_ccode_formal_parameter_new ("self", _tmp3_ = g_strconcat (_tmp2_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) sym, FALSE), "*", NULL)));
	_vala_ccode_node_unref0 (_tmp4_);
	_g_free0 (_tmp3_);
	_g_free0 (_tmp2_);
	vala_ccode_function_add_parameter (function, _tmp5_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp5_);
	vala_ccode_function_add_parameter (function, _tmp6_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp6_);
	block = vala_ccode_block_new ();
	_cdecl_ = vala_ccode_declaration_new ("DBusMessage*");
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp7_ = vala_ccode_variable_declarator_new ("reply", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp7_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp8_ = vala_ccode_declaration_new ("DBusMessageIter"), _vala_ccode_node_unref0 (_cdecl_), _tmp8_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp9_ = vala_ccode_variable_declarator_new ("iter", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp9_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	msgcall = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("dbus_message_new_method_return"))), _vala_ccode_node_unref0 (_tmp10_), _tmp11_);
	vala_ccode_function_call_add_argument (msgcall, (ValaCCodeExpression*) (_tmp12_ = vala_ccode_identifier_new ("message")));
	_vala_ccode_node_unref0 (_tmp12_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp15_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp14_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp13_ = vala_ccode_identifier_new ("reply")), (ValaCCodeExpression*) msgcall, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp15_);
	_vala_ccode_node_unref0 (_tmp14_);
	_vala_ccode_node_unref0 (_tmp13_);
	iter_call = (_tmp17_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new ("dbus_message_iter_init_append"))), _vala_ccode_node_unref0 (_tmp16_), _tmp17_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp18_ = vala_ccode_identifier_new ("reply")));
	_vala_ccode_node_unref0 (_tmp18_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp20_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp20_);
	_vala_ccode_node_unref0 (_tmp19_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp21_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp21_);
	_cdecl_ = (_tmp22_ = vala_ccode_declaration_new ("GString*"), _vala_ccode_node_unref0 (_cdecl_), _tmp22_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp23_ = vala_ccode_variable_declarator_new ("xml_data", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp23_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp24_ = vala_ccode_declaration_new ("char**"), _vala_ccode_node_unref0 (_cdecl_), _tmp24_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp25_ = vala_ccode_variable_declarator_new ("children", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp25_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	_cdecl_ = (_tmp26_ = vala_ccode_declaration_new ("int"), _vala_ccode_node_unref0 (_cdecl_), _tmp26_);
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp27_ = vala_ccode_variable_declarator_new ("i", NULL, NULL)));
	_vala_ccode_node_unref0 (_tmp27_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	xml_data = g_strdup ("<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\" \"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n");
	str_call = (_tmp29_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp28_ = vala_ccode_identifier_new ("g_string_new"))), _vala_ccode_node_unref0 (_tmp28_), _tmp29_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp32_ = vala_ccode_constant_new (_tmp31_ = g_strdup_printf ("\"%s\"", _tmp30_ = g_strescape (xml_data, "")))));
	_vala_ccode_node_unref0 (_tmp32_);
	_g_free0 (_tmp31_);
	_g_free0 (_tmp30_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp35_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp34_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("xml_data")), (ValaCCodeExpression*) str_call, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
	_vala_ccode_node_unref0 (_tmp35_);
	_vala_ccode_node_unref0 (_tmp34_);
	_vala_ccode_node_unref0 (_tmp33_);
	xml_data = (_tmp36_ = g_strdup ("<node>\n"), _g_free0 (xml_data), _tmp36_);
	xml_data = (_tmp37_ = g_strconcat (xml_data, "<interface name=\"org.freedesktop.DBus.Introspectable\">\n  <method name=\"Introspect\">\n    <arg name=\"data\" direction=\"out\" type=\"s\"/>\n  </method>\n</interface>\n<interface name=\"org.freedesktop.DBus.Properties\">\n  <method name=\"Get\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"out\" type=\"v\"/>\n  </method>\n  <method name=\"Set\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"in\" type=\"v\"/>\n  </method>\n  <method name=\"GetAll\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"props\" direction=\"out\" type=\"a{sv}\"/>\n  </method>\n</interface>\n", NULL), _g_free0 (xml_data), _tmp37_);
	xml_data = (_tmp39_ = g_strconcat (xml_data, _tmp38_ = vala_dbus_server_module_get_dbus_type_introspection (self, sym), NULL), _g_free0 (xml_data), _tmp39_);
	_g_free0 (_tmp38_);
	str_call = (_tmp41_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp40_ = vala_ccode_identifier_new ("g_string_append"))), _vala_ccode_node_unref0 (str_call), _tmp41_);
	_vala_ccode_node_unref0 (_tmp40_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp42_ = vala_ccode_identifier_new ("xml_data")));
	_vala_ccode_node_unref0 (_tmp42_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp45_ = vala_ccode_constant_new (_tmp44_ = g_strdup_printf ("\"%s\"", _tmp43_ = g_strescape (xml_data, "")))));
	_vala_ccode_node_unref0 (_tmp45_);
	_g_free0 (_tmp44_);
	_g_free0 (_tmp43_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp46_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) str_call)));
	_vala_ccode_node_unref0 (_tmp46_);
	get_path = (_tmp48_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp47_ = vala_ccode_identifier_new ("g_object_get_data"))), _vala_ccode_node_unref0 (_tmp47_), _tmp48_);
	vala_ccode_function_call_add_argument (get_path, (ValaCCodeExpression*) (_tmp50_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new ("self")), "GObject *")));
	_vala_ccode_node_unref0 (_tmp50_);
	_vala_ccode_node_unref0 (_tmp49_);
	vala_ccode_function_call_add_argument (get_path, (ValaCCodeExpression*) (_tmp51_ = vala_ccode_constant_new ("\"dbus_object_path\"")));
	_vala_ccode_node_unref0 (_tmp51_);
	list_call = (_tmp53_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp52_ = vala_ccode_identifier_new ("dbus_connection_list_registered"))), _vala_ccode_node_unref0 (_tmp52_), _tmp53_);
	vala_ccode_function_call_add_argument (list_call, (ValaCCodeExpression*) (_tmp54_ = vala_ccode_identifier_new ("connection")));
	_vala_ccode_node_unref0 (_tmp54_);
	vala_ccode_function_call_add_argument (list_call, (ValaCCodeExpression*) get_path);
	vala_ccode_function_call_add_argument (list_call, (ValaCCodeExpression*) (_tmp56_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp55_ = vala_ccode_identifier_new ("children")))));
	_vala_ccode_node_unref0 (_tmp56_);
	_vala_ccode_node_unref0 (_tmp55_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp57_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) list_call)));
	_vala_ccode_node_unref0 (_tmp57_);
	child_block = vala_ccode_block_new ();
	str_call = (_tmp59_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp58_ = vala_ccode_identifier_new ("g_string_append_printf"))), _vala_ccode_node_unref0 (str_call), _tmp59_);
	_vala_ccode_node_unref0 (_tmp58_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new ("xml_data")));
	_vala_ccode_node_unref0 (_tmp60_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp63_ = vala_ccode_constant_new (_tmp62_ = g_strdup_printf ("\"%s\"", _tmp61_ = g_strescape ("<node name=\"%s\"/>\n", "")))));
	_vala_ccode_node_unref0 (_tmp63_);
	_g_free0 (_tmp62_);
	_g_free0 (_tmp61_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp66_ = vala_ccode_element_access_new ((ValaCCodeExpression*) (_tmp64_ = vala_ccode_identifier_new ("children")), (ValaCCodeExpression*) (_tmp65_ = vala_ccode_identifier_new ("i")))));
	_vala_ccode_node_unref0 (_tmp66_);
	_vala_ccode_node_unref0 (_tmp65_);
	_vala_ccode_node_unref0 (_tmp64_);
	vala_ccode_block_add_statement (child_block, (ValaCCodeNode*) (_tmp67_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) str_call)));
	_vala_ccode_node_unref0 (_tmp67_);
	cfor = (_tmp71_ = vala_ccode_for_statement_new ((ValaCCodeExpression*) (_tmp70_ = vala_ccode_element_access_new ((ValaCCodeExpression*) (_tmp68_ = vala_ccode_identifier_new ("children")), (ValaCCodeExpression*) (_tmp69_ = vala_ccode_identifier_new ("i")))), (ValaCCodeStatement*) child_block), _vala_ccode_node_unref0 (_tmp70_), _vala_ccode_node_unref0 (_tmp69_), _vala_ccode_node_unref0 (_tmp68_), _tmp71_);
	vala_ccode_for_statement_add_initializer (cfor, (ValaCCodeExpression*) (_tmp74_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp72_ = vala_ccode_identifier_new ("i")), (ValaCCodeExpression*) (_tmp73_ = vala_ccode_constant_new ("0")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
	_vala_ccode_node_unref0 (_tmp74_);
	_vala_ccode_node_unref0 (_tmp73_);
	_vala_ccode_node_unref0 (_tmp72_);
	vala_ccode_for_statement_add_iterator (cfor, (ValaCCodeExpression*) (_tmp76_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT, (ValaCCodeExpression*) (_tmp75_ = vala_ccode_identifier_new ("i")))));
	_vala_ccode_node_unref0 (_tmp76_);
	_vala_ccode_node_unref0 (_tmp75_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) cfor);
	list_free_call = (_tmp78_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp77_ = vala_ccode_identifier_new ("dbus_free_string_array"))), _vala_ccode_node_unref0 (_tmp77_), _tmp78_);
	vala_ccode_function_call_add_argument (list_free_call, (ValaCCodeExpression*) (_tmp79_ = vala_ccode_identifier_new ("children")));
	_vala_ccode_node_unref0 (_tmp79_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp80_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) list_free_call)));
	_vala_ccode_node_unref0 (_tmp80_);
	xml_data = (_tmp81_ = g_strdup ("</node>\n"), _g_free0 (xml_data), _tmp81_);
	str_call = (_tmp83_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp82_ = vala_ccode_identifier_new ("g_string_append"))), _vala_ccode_node_unref0 (str_call), _tmp83_);
	_vala_ccode_node_unref0 (_tmp82_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp84_ = vala_ccode_identifier_new ("xml_data")));
	_vala_ccode_node_unref0 (_tmp84_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp87_ = vala_ccode_constant_new (_tmp86_ = g_strdup_printf ("\"%s\"", _tmp85_ = g_strescape (xml_data, "")))));
	_vala_ccode_node_unref0 (_tmp87_);
	_g_free0 (_tmp86_);
	_g_free0 (_tmp85_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp88_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) str_call)));
	_vala_ccode_node_unref0 (_tmp88_);
	iter_call = (_tmp90_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp89_ = vala_ccode_identifier_new ("dbus_message_iter_append_basic"))), _vala_ccode_node_unref0 (iter_call), _tmp90_);
	_vala_ccode_node_unref0 (_tmp89_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp92_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp91_ = vala_ccode_identifier_new ("iter")))));
	_vala_ccode_node_unref0 (_tmp92_);
	_vala_ccode_node_unref0 (_tmp91_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp93_ = vala_ccode_identifier_new ("DBUS_TYPE_STRING")));
	_vala_ccode_node_unref0 (_tmp93_);
	vala_ccode_function_call_add_argument (iter_call, (ValaCCodeExpression*) (_tmp96_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, (ValaCCodeExpression*) (_tmp95_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp94_ = vala_ccode_identifier_new ("xml_data")), "str")))));
	_vala_ccode_node_unref0 (_tmp96_);
	_vala_ccode_node_unref0 (_tmp95_);
	_vala_ccode_node_unref0 (_tmp94_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp97_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) iter_call)));
	_vala_ccode_node_unref0 (_tmp97_);
	str_call = (_tmp99_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp98_ = vala_ccode_identifier_new ("g_string_free"))), _vala_ccode_node_unref0 (str_call), _tmp99_);
	_vala_ccode_node_unref0 (_tmp98_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp100_ = vala_ccode_identifier_new ("xml_data")));
	_vala_ccode_node_unref0 (_tmp100_);
	vala_ccode_function_call_add_argument (str_call, (ValaCCodeExpression*) (_tmp101_ = vala_ccode_constant_new ("TRUE")));
	_vala_ccode_node_unref0 (_tmp101_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp102_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) str_call)));
	_vala_ccode_node_unref0 (_tmp102_);
	vala_dbus_server_module_handle_reply (self, block);
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp103_ = vala_ccode_function_copy (function)));
	_vala_ccode_node_unref0 (_tmp103_);
	vala_ccode_function_set_block (function, block);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) function);
	result = wrapper_name;
	_vala_ccode_node_unref0 (function);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (msgcall);
	_vala_ccode_node_unref0 (iter_call);
	_g_free0 (xml_data);
	_vala_ccode_node_unref0 (str_call);
	_vala_ccode_node_unref0 (get_path);
	_vala_ccode_node_unref0 (list_call);
	_vala_ccode_node_unref0 (child_block);
	_vala_ccode_node_unref0 (cfor);
	_vala_ccode_node_unref0 (list_free_call);
	return result;
}


static void vala_dbus_server_module_handle_signals (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym, ValaCCodeBlock* block) {
	char* dbus_iface_name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sym != NULL);
	g_return_if_fail (block != NULL);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) sym);
	if (dbus_iface_name == NULL) {
		_g_free0 (dbus_iface_name);
		return;
	}
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _sig_it;
		_sig_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_object_type_symbol_get_signals (sym))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaSignal* sig;
			ValaCCodeIdentifier* _tmp2_;
			ValaCCodeFunctionCall* _tmp3_;
			ValaCCodeFunctionCall* connect;
			ValaCCodeIdentifier* _tmp4_;
			ValaCCodeConstant* _tmp5_;
			ValaCCodeCastExpression* _tmp8_;
			ValaCCodeIdentifier* _tmp7_;
			char* _tmp6_;
			ValaCCodeIdentifier* _tmp9_;
			ValaCCodeExpressionStatement* _tmp10_;
			if (!gee_iterator_next (_sig_it)) {
				break;
			}
			sig = (ValaSignal*) gee_iterator_get (_sig_it);
			if (vala_symbol_get_access ((ValaSymbol*) sig) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC) {
				_vala_code_node_unref0 (sig);
				continue;
			}
			if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) sig)) {
				_vala_code_node_unref0 (sig);
				continue;
			}
			connect = (_tmp3_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new ("g_signal_connect"))), _vala_ccode_node_unref0 (_tmp2_), _tmp3_);
			vala_ccode_function_call_add_argument (connect, (ValaCCodeExpression*) (_tmp4_ = vala_ccode_identifier_new ("object")));
			_vala_ccode_node_unref0 (_tmp4_);
			vala_ccode_function_call_add_argument (connect, (ValaCCodeExpression*) (_tmp5_ = vala_signal_get_canonical_cconstant (sig, NULL)));
			_vala_ccode_node_unref0 (_tmp5_);
			vala_ccode_function_call_add_argument (connect, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = vala_dbus_server_module_generate_dbus_signal_wrapper (self, sig, sym, dbus_iface_name))), "GCallback")));
			_vala_ccode_node_unref0 (_tmp8_);
			_vala_ccode_node_unref0 (_tmp7_);
			_g_free0 (_tmp6_);
			vala_ccode_function_call_add_argument (connect, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new ("connection")));
			_vala_ccode_node_unref0 (_tmp9_);
			vala_ccode_block_add_statement (block, (ValaCCodeNode*) (_tmp10_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) connect)));
			_vala_ccode_node_unref0 (_tmp10_);
			_vala_code_node_unref0 (sig);
			_vala_ccode_node_unref0 (connect);
		}
		_gee_collection_object_unref0 (_sig_it);
	}
	_g_free0 (dbus_iface_name);
}


static void vala_dbus_server_module_generate_message_function (ValaDBusServerModule* self, ValaObjectType* object_type) {
	ValaObjectTypeSymbol* sym;
	char* _tmp1_;
	char* _tmp0_;
	ValaCCodeFunction* _tmp2_;
	ValaCCodeFunction* cfunc;
	ValaCCodeFormalParameter* _tmp3_;
	ValaCCodeFormalParameter* _tmp4_;
	ValaCCodeFormalParameter* _tmp5_;
	ValaCCodeFunction* _tmp6_;
	ValaCCodeBlock* block;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp8_;
	ValaCCodeConstant* _tmp7_;
	ValaCCodeIfStatement* clastif;
	char* _tmp9_;
	char* dbus_iface_name;
	ValaCCodeBlock* resultblock;
	ValaCCodeReturnStatement* _tmp18_;
	ValaCCodeIdentifier* _tmp17_;
	ValaCCodeBinaryExpression* _tmp21_;
	ValaCCodeIdentifier* _tmp20_;
	ValaCCodeIdentifier* _tmp19_;
	ValaCCodeIfStatement* _tmp22_;
	ValaCCodeIfStatement* cif;
	ValaCCodeIfStatement* _tmp23_;
	ValaObjectTypeSymbol* _tmp24_;
	ValaClass* cl;
	ValaCCodeBlock* retblock;
	ValaCCodeReturnStatement* _tmp42_;
	ValaCCodeIdentifier* _tmp41_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (object_type != NULL);
	sym = _vala_code_node_ref0 (vala_object_type_get_type_symbol (object_type));
	((ValaCCodeBaseModule*) self)->dbus_glib_h_needed = TRUE;
	cfunc = (_tmp2_ = vala_ccode_function_new (_tmp1_ = g_strconcat (_tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), "dbus_message", NULL), "DBusHandlerResult"), _g_free0 (_tmp1_), _g_free0 (_tmp0_), _tmp2_);
	vala_ccode_function_add_parameter (cfunc, _tmp3_ = vala_ccode_formal_parameter_new ("connection", "DBusConnection*"));
	_vala_ccode_node_unref0 (_tmp3_);
	vala_ccode_function_add_parameter (cfunc, _tmp4_ = vala_ccode_formal_parameter_new ("message", "DBusMessage*"));
	_vala_ccode_node_unref0 (_tmp4_);
	vala_ccode_function_add_parameter (cfunc, _tmp5_ = vala_ccode_formal_parameter_new ("object", "void*"));
	_vala_ccode_node_unref0 (_tmp5_);
	if (vala_symbol_is_private_symbol ((ValaSymbol*) sym)) {
		vala_ccode_function_set_modifiers (cfunc, vala_ccode_function_get_modifiers (cfunc) | VALA_CCODE_MODIFIERS_STATIC);
	}
	vala_ccode_declaration_space_add_type_member_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) (_tmp6_ = vala_ccode_function_copy (cfunc)));
	_vala_ccode_node_unref0 (_tmp6_);
	block = vala_ccode_block_new ();
	vala_ccode_function_set_block (cfunc, block);
	_cdecl_ = vala_ccode_declaration_new ("DBusHandlerResult");
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp8_ = vala_ccode_variable_declarator_new ("result", (ValaCCodeExpression*) (_tmp7_ = vala_ccode_constant_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")), NULL)));
	_vala_ccode_node_unref0 (_tmp8_);
	_vala_ccode_node_unref0 (_tmp7_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) _cdecl_);
	clastif = NULL;
	vala_dbus_server_module_handle_method (self, "org.freedesktop.DBus.Introspectable", "Introspect", _tmp9_ = vala_dbus_server_module_generate_dbus_introspect (self, sym), block, &clastif);
	_g_free0 (_tmp9_);
	dbus_iface_name = vala_dbus_module_get_dbus_name ((ValaDBusModule*) self, (ValaTypeSymbol*) sym);
	if (dbus_iface_name != NULL) {
		gboolean need_property_get;
		gboolean need_property_set;
		char* _tmp16_;
		need_property_get = FALSE;
		need_property_set = FALSE;
		{
			GeeList* _tmp10_;
			GeeIterator* _tmp11_;
			GeeIterator* _prop_it;
			_prop_it = (_tmp11_ = gee_iterable_iterator ((GeeIterable*) (_tmp10_ = vala_object_type_symbol_get_properties (sym))), _gee_collection_object_unref0 (_tmp10_), _tmp11_);
			while (TRUE) {
				ValaProperty* prop;
				gboolean _tmp12_;
				gboolean _tmp13_;
				if (!gee_iterator_next (_prop_it)) {
					break;
				}
				prop = (ValaProperty*) gee_iterator_get (_prop_it);
				_tmp12_ = FALSE;
				_tmp13_ = FALSE;
				if (vala_property_get_binding (prop) != MEMBER_BINDING_INSTANCE) {
					_tmp13_ = TRUE;
				} else {
					_tmp13_ = vala_property_get_overrides (prop);
				}
				if (_tmp13_) {
					_tmp12_ = TRUE;
				} else {
					_tmp12_ = vala_symbol_get_access ((ValaSymbol*) prop) != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
				}
				if (_tmp12_) {
					_vala_code_node_unref0 (prop);
					continue;
				}
				if (!vala_dbus_server_module_is_dbus_visible (self, (ValaCodeNode*) prop)) {
					_vala_code_node_unref0 (prop);
					continue;
				}
				if (vala_property_get_get_accessor (prop) != NULL) {
					need_property_get = TRUE;
				}
				if (vala_property_get_set_accessor (prop) != NULL) {
					need_property_set = TRUE;
				}
				_vala_code_node_unref0 (prop);
			}
			_gee_collection_object_unref0 (_prop_it);
		}
		if (need_property_get) {
			char* _tmp14_;
			vala_dbus_server_module_handle_method (self, "org.freedesktop.DBus.Properties", "Get", _tmp14_ = vala_dbus_server_module_generate_dbus_property_get_wrapper (self, sym, dbus_iface_name), block, &clastif);
			_g_free0 (_tmp14_);
		}
		if (need_property_set) {
			char* _tmp15_;
			vala_dbus_server_module_handle_method (self, "org.freedesktop.DBus.Properties", "Set", _tmp15_ = vala_dbus_server_module_generate_dbus_property_set_wrapper (self, sym, dbus_iface_name), block, &clastif);
			_g_free0 (_tmp15_);
		}
		vala_dbus_server_module_handle_method (self, "org.freedesktop.DBus.Properties", "GetAll", _tmp16_ = vala_dbus_server_module_generate_dbus_property_get_all_wrapper (self, sym, dbus_iface_name), block, &clastif);
		_g_free0 (_tmp16_);
		vala_dbus_server_module_handle_methods (self, sym, dbus_iface_name, block, &clastif);
	}
	resultblock = vala_ccode_block_new ();
	vala_ccode_block_add_statement (resultblock, (ValaCCodeNode*) (_tmp18_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp17_ = vala_ccode_identifier_new ("result")))));
	_vala_ccode_node_unref0 (_tmp18_);
	_vala_ccode_node_unref0 (_tmp17_);
	cif = (_tmp22_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) (_tmp21_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) (_tmp19_ = vala_ccode_identifier_new ("result")), (ValaCCodeExpression*) (_tmp20_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_HANDLED")))), (ValaCCodeStatement*) resultblock, NULL), _vala_ccode_node_unref0 (_tmp21_), _vala_ccode_node_unref0 (_tmp20_), _vala_ccode_node_unref0 (_tmp19_), _tmp22_);
	vala_ccode_block_add_statement (block, (ValaCCodeNode*) cif);
	clastif = (_tmp23_ = _vala_ccode_node_ref0 (cif), _vala_ccode_node_unref0 (clastif), _tmp23_);
	cl = _vala_code_node_ref0 ((_tmp24_ = sym, VALA_IS_CLASS (_tmp24_) ? ((ValaClass*) _tmp24_) : NULL));
	if (cl != NULL) {
		{
			GeeList* _tmp25_;
			GeeIterator* _tmp26_;
			GeeIterator* _base_type_it;
			_base_type_it = (_tmp26_ = gee_iterable_iterator ((GeeIterable*) (_tmp25_ = vala_class_get_base_types (cl))), _gee_collection_object_unref0 (_tmp25_), _tmp26_);
			while (TRUE) {
				ValaDataType* base_type;
				ValaDataType* _tmp27_;
				ValaObjectType* base_obj_type;
				if (!gee_iterator_next (_base_type_it)) {
					break;
				}
				base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
				base_obj_type = _vala_code_node_ref0 ((_tmp27_ = base_type, VALA_IS_OBJECT_TYPE (_tmp27_) ? ((ValaObjectType*) _tmp27_) : NULL));
				if (vala_dbus_server_module_type_implements_dbus_interface (self, vala_object_type_get_type_symbol (base_obj_type))) {
					ValaCCodeIdentifier* _tmp30_;
					char* _tmp29_;
					char* _tmp28_;
					ValaCCodeFunctionCall* _tmp31_;
					ValaCCodeFunctionCall* base_call;
					ValaCCodeIdentifier* _tmp32_;
					ValaCCodeIdentifier* _tmp33_;
					ValaCCodeIdentifier* _tmp34_;
					ValaCCodeIdentifier* _tmp35_;
					ValaCCodeBinaryExpression* _tmp36_;
					ValaCCodeBinaryExpression* ccheck;
					ValaCCodeBlock* base_block;
					ValaCCodeReturnStatement* _tmp38_;
					ValaCCodeIdentifier* _tmp37_;
					ValaCCodeIfStatement* _tmp39_;
					ValaCCodeIfStatement* _tmp40_;
					base_call = (_tmp31_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp30_ = vala_ccode_identifier_new (_tmp29_ = g_strconcat (_tmp28_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) vala_object_type_get_type_symbol (base_obj_type)), "dbus_message", NULL)))), _vala_ccode_node_unref0 (_tmp30_), _g_free0 (_tmp29_), _g_free0 (_tmp28_), _tmp31_);
					vala_ccode_function_call_add_argument (base_call, (ValaCCodeExpression*) (_tmp32_ = vala_ccode_identifier_new ("connection")));
					_vala_ccode_node_unref0 (_tmp32_);
					vala_ccode_function_call_add_argument (base_call, (ValaCCodeExpression*) (_tmp33_ = vala_ccode_identifier_new ("message")));
					_vala_ccode_node_unref0 (_tmp33_);
					vala_ccode_function_call_add_argument (base_call, (ValaCCodeExpression*) (_tmp34_ = vala_ccode_identifier_new ("object")));
					_vala_ccode_node_unref0 (_tmp34_);
					ccheck = (_tmp36_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, (ValaCCodeExpression*) base_call, (ValaCCodeExpression*) (_tmp35_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_HANDLED"))), _vala_ccode_node_unref0 (_tmp35_), _tmp36_);
					base_block = vala_ccode_block_new ();
					vala_ccode_block_add_statement (base_block, (ValaCCodeNode*) (_tmp38_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp37_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_HANDLED")))));
					_vala_ccode_node_unref0 (_tmp38_);
					_vala_ccode_node_unref0 (_tmp37_);
					cif = (_tmp39_ = vala_ccode_if_statement_new ((ValaCCodeExpression*) ccheck, (ValaCCodeStatement*) base_block, NULL), _vala_ccode_node_unref0 (cif), _tmp39_);
					vala_ccode_if_statement_set_false_statement (clastif, (ValaCCodeStatement*) cif);
					clastif = (_tmp40_ = _vala_ccode_node_ref0 (cif), _vala_ccode_node_unref0 (clastif), _tmp40_);
					_vala_ccode_node_unref0 (base_call);
					_vala_ccode_node_unref0 (ccheck);
					_vala_ccode_node_unref0 (base_block);
				}
				_vala_code_node_unref0 (base_type);
				_vala_code_node_unref0 (base_obj_type);
			}
			_gee_collection_object_unref0 (_base_type_it);
		}
	}
	retblock = vala_ccode_block_new ();
	vala_ccode_block_add_statement (retblock, (ValaCCodeNode*) (_tmp42_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp41_ = vala_ccode_identifier_new ("DBUS_HANDLER_RESULT_NOT_YET_HANDLED")))));
	_vala_ccode_node_unref0 (_tmp42_);
	_vala_ccode_node_unref0 (_tmp41_);
	vala_ccode_if_statement_set_false_statement (clastif, (ValaCCodeStatement*) retblock);
	vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->source_type_member_definition, (ValaCCodeNode*) cfunc);
	_vala_code_node_unref0 (sym);
	_vala_ccode_node_unref0 (cfunc);
	_vala_ccode_node_unref0 (block);
	_vala_ccode_node_unref0 (_cdecl_);
	_vala_ccode_node_unref0 (clastif);
	_g_free0 (dbus_iface_name);
	_vala_ccode_node_unref0 (resultblock);
	_vala_ccode_node_unref0 (cif);
	_vala_code_node_unref0 (cl);
	_vala_ccode_node_unref0 (retblock);
}


static ValaCCodeExpression* vala_dbus_server_module_get_vtable (ValaDBusServerModule* self, ValaObjectType* object_type) {
	ValaCCodeExpression* result;
	ValaObjectTypeSymbol* sym;
	ValaCCodeInitializerList* vtable;
	ValaCCodeIdentifier* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp9_;
	char* _tmp8_;
	char* _tmp7_;
	ValaCCodeExpression* _tmp10_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (object_type != NULL, NULL);
	sym = _vala_code_node_ref0 (vala_object_type_get_type_symbol (object_type));
	vtable = vala_ccode_initializer_list_new ();
	vala_ccode_initializer_list_append (vtable, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_identifier_new (_tmp1_ = g_strconcat (_tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), "dbus_register_object", NULL))));
	_vala_ccode_node_unref0 (_tmp2_);
	_g_free0 (_tmp1_);
	_g_free0 (_tmp0_);
	vala_dbus_server_module_generate_register_function (self, object_type);
	_cdecl_ = vala_ccode_declaration_new ("const _DBusObjectVTable");
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp6_ = vala_ccode_variable_declarator_new (_tmp5_ = g_strconcat (_tmp4_ = g_strconcat ("_", _tmp3_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), NULL), "dbus_vtable", NULL), (ValaCCodeExpression*) vtable, NULL)));
	_vala_ccode_node_unref0 (_tmp6_);
	_g_free0 (_tmp5_);
	_g_free0 (_tmp4_);
	_g_free0 (_tmp3_);
	vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_declaration_space_add_constant_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) _cdecl_);
	result = (_tmp10_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp9_ = g_strconcat (_tmp8_ = g_strconcat ("_", _tmp7_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), NULL), "dbus_vtable", NULL)), _g_free0 (_tmp9_), _g_free0 (_tmp8_), _g_free0 (_tmp7_), _tmp10_);
	_vala_code_node_unref0 (sym);
	_vala_ccode_node_unref0 (vtable);
	_vala_ccode_node_unref0 (_cdecl_);
	return result;
	_vala_code_node_unref0 (sym);
	_vala_ccode_node_unref0 (vtable);
	_vala_ccode_node_unref0 (_cdecl_);
}


static ValaCCodeExpression* vala_dbus_server_module_get_path_vtable (ValaDBusServerModule* self, ValaObjectType* object_type) {
	ValaCCodeExpression* result;
	ValaObjectTypeSymbol* sym;
	ValaCCodeInitializerList* vtable;
	ValaCCodeIdentifier* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	ValaCCodeIdentifier* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	ValaCCodeDeclaration* _cdecl_;
	ValaCCodeVariableDeclarator* _tmp10_;
	char* _tmp9_;
	char* _tmp8_;
	char* _tmp7_;
	char* _tmp13_;
	char* _tmp12_;
	char* _tmp11_;
	ValaCCodeExpression* _tmp14_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (object_type != NULL, NULL);
	sym = _vala_code_node_ref0 (vala_object_type_get_type_symbol (object_type));
	vtable = vala_ccode_initializer_list_new ();
	vala_ccode_initializer_list_append (vtable, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new (_tmp2_ = g_strconcat (_tmp1_ = g_strconcat ("_", _tmp0_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), NULL), "dbus_unregister", NULL))));
	_vala_ccode_node_unref0 (_tmp3_);
	_g_free0 (_tmp2_);
	_g_free0 (_tmp1_);
	_g_free0 (_tmp0_);
	vala_ccode_initializer_list_append (vtable, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new (_tmp5_ = g_strconcat (_tmp4_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), "dbus_message", NULL))));
	_vala_ccode_node_unref0 (_tmp6_);
	_g_free0 (_tmp5_);
	_g_free0 (_tmp4_);
	vala_dbus_server_module_generate_unregister_function (self, object_type);
	vala_dbus_server_module_generate_message_function (self, object_type);
	_cdecl_ = vala_ccode_declaration_new ("const DBusObjectPathVTable");
	vala_ccode_declaration_add_declarator (_cdecl_, (ValaCCodeDeclarator*) (_tmp10_ = vala_ccode_variable_declarator_new (_tmp9_ = g_strconcat (_tmp8_ = g_strconcat ("_", _tmp7_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), NULL), "dbus_path_vtable", NULL), (ValaCCodeExpression*) vtable, NULL)));
	_vala_ccode_node_unref0 (_tmp10_);
	_g_free0 (_tmp9_);
	_g_free0 (_tmp8_);
	_g_free0 (_tmp7_);
	vala_ccode_declaration_set_modifiers (_cdecl_, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_declaration_space_add_constant_declaration (((ValaCCodeBaseModule*) self)->source_declarations, (ValaCCodeNode*) _cdecl_);
	result = (_tmp14_ = (ValaCCodeExpression*) vala_ccode_identifier_new (_tmp13_ = g_strconcat (_tmp12_ = g_strconcat ("_", _tmp11_ = vala_symbol_get_lower_case_cprefix ((ValaSymbol*) sym), NULL), "dbus_path_vtable", NULL)), _g_free0 (_tmp13_), _g_free0 (_tmp12_), _g_free0 (_tmp11_), _tmp14_);
	_vala_code_node_unref0 (sym);
	_vala_ccode_node_unref0 (vtable);
	_vala_ccode_node_unref0 (_cdecl_);
	return result;
	_vala_code_node_unref0 (sym);
	_vala_ccode_node_unref0 (vtable);
	_vala_ccode_node_unref0 (_cdecl_);
}


static void vala_dbus_server_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr) {
	ValaDBusServerModule * self;
	ValaDataType* _tmp0_;
	ValaMethodType* mtype;
	gboolean _tmp1_;
	ValaMemberAccess* ma;
	ValaCCodeIdentifier* _tmp6_;
	ValaCCodeFunctionCall* _tmp7_;
	ValaCCodeFunctionCall* raw_conn;
	GeeList* _tmp8_;
	GeeIterator* _tmp9_;
	GeeIterator* args_it;
	ValaExpression* path_arg;
	ValaExpression* obj_arg;
	ValaCCodeIdentifier* _tmp10_;
	ValaCCodeFunctionCall* _tmp11_;
	ValaCCodeFunctionCall* cregister;
	char* _tmp12_;
	gboolean _tmp13_;
	self = (ValaDBusServerModule*) base;
	g_return_if_fail (expr != NULL);
	mtype = _vala_code_node_ref0 ((_tmp0_ = vala_expression_get_value_type (vala_method_call_get_call (expr)), VALA_IS_METHOD_TYPE (_tmp0_) ? ((ValaMethodType*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (mtype == NULL) {
		_tmp1_ = TRUE;
	} else {
		gboolean _tmp2_;
		char* _tmp3_;
		gboolean _tmp4_;
		_tmp2_ = FALSE;
		if ((_tmp4_ = _vala_strcmp0 (_tmp3_ = vala_method_get_cname (vala_method_type_get_method_symbol (mtype)), "dbus_connection_register_g_object") != 0, _g_free0 (_tmp3_), _tmp4_)) {
			char* _tmp5_;
			_tmp2_ = _vala_strcmp0 (_tmp5_ = vala_method_get_cname (vala_method_type_get_method_symbol (mtype)), "dbus_g_connection_register_g_object") != 0;
			_g_free0 (_tmp5_);
		} else {
			_tmp2_ = FALSE;
		}
		_tmp1_ = _tmp2_;
	}
	if (_tmp1_) {
		VALA_CCODE_MODULE_CLASS (vala_dbus_server_module_parent_class)->visit_method_call ((ValaCCodeModule*) VALA_DBUS_CLIENT_MODULE (self), expr);
		_vala_code_node_unref0 (mtype);
		return;
	}
	((ValaCCodeBaseModule*) self)->dbus_glib_h_needed = TRUE;
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	ma = _vala_code_node_ref0 (VALA_MEMBER_ACCESS (vala_method_call_get_call (expr)));
	raw_conn = (_tmp7_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new ("dbus_g_connection_get_connection"))), _vala_ccode_node_unref0 (_tmp6_), _tmp7_);
	vala_ccode_function_call_add_argument (raw_conn, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma))));
	args_it = (_tmp9_ = gee_iterable_iterator ((GeeIterable*) (_tmp8_ = vala_method_call_get_argument_list (expr))), _gee_collection_object_unref0 (_tmp8_), _tmp9_);
	gee_iterator_next (args_it);
	path_arg = (ValaExpression*) gee_iterator_get (args_it);
	gee_iterator_next (args_it);
	obj_arg = (ValaExpression*) gee_iterator_get (args_it);
	cregister = (_tmp11_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp10_ = vala_ccode_identifier_new ("_vala_dbus_register_object"))), _vala_ccode_node_unref0 (_tmp10_), _tmp11_);
	if ((_tmp13_ = _vala_strcmp0 (_tmp12_ = vala_method_get_cname (vala_method_type_get_method_symbol (mtype)), "dbus_g_connection_register_g_object") == 0, _g_free0 (_tmp12_), _tmp13_)) {
		vala_ccode_function_call_add_argument (cregister, (ValaCCodeExpression*) raw_conn);
	} else {
		vala_ccode_function_call_add_argument (cregister, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma))));
	}
	vala_ccode_function_call_add_argument (cregister, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) path_arg)));
	vala_ccode_function_call_add_argument (cregister, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) obj_arg)));
	vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) cregister);
	_vala_code_node_unref0 (mtype);
	_vala_code_node_unref0 (ma);
	_vala_ccode_node_unref0 (raw_conn);
	_gee_collection_object_unref0 (args_it);
	_vala_code_node_unref0 (path_arg);
	_vala_code_node_unref0 (obj_arg);
	_vala_ccode_node_unref0 (cregister);
}


static gboolean vala_dbus_server_module_type_implements_dbus_interface (ValaDBusServerModule* self, ValaObjectTypeSymbol* sym) {
	gboolean result;
	ValaAttribute* dbus;
	ValaObjectTypeSymbol* _tmp0_;
	ValaClass* cl;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sym != NULL, FALSE);
	dbus = vala_code_node_get_attribute ((ValaCodeNode*) sym, "DBus");
	if (dbus != NULL) {
		result = TRUE;
		_vala_code_node_unref0 (dbus);
		return result;
	}
	cl = _vala_code_node_ref0 ((_tmp0_ = sym, VALA_IS_CLASS (_tmp0_) ? ((ValaClass*) _tmp0_) : NULL));
	if (cl != NULL) {
		{
			GeeList* _tmp1_;
			GeeIterator* _tmp2_;
			GeeIterator* _base_type_it;
			_base_type_it = (_tmp2_ = gee_iterable_iterator ((GeeIterable*) (_tmp1_ = vala_class_get_base_types (cl))), _gee_collection_object_unref0 (_tmp1_), _tmp2_);
			while (TRUE) {
				ValaDataType* base_type;
				ValaDataType* _tmp3_;
				ValaObjectType* base_obj_type;
				if (!gee_iterator_next (_base_type_it)) {
					break;
				}
				base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
				base_obj_type = _vala_code_node_ref0 ((_tmp3_ = base_type, VALA_IS_OBJECT_TYPE (_tmp3_) ? ((ValaObjectType*) _tmp3_) : NULL));
				if (vala_dbus_server_module_type_implements_dbus_interface (self, vala_object_type_get_type_symbol (base_obj_type))) {
					result = TRUE;
					_vala_code_node_unref0 (base_type);
					_vala_code_node_unref0 (base_obj_type);
					_gee_collection_object_unref0 (_base_type_it);
					_vala_code_node_unref0 (dbus);
					_vala_code_node_unref0 (cl);
					return result;
				}
				_vala_code_node_unref0 (base_type);
				_vala_code_node_unref0 (base_obj_type);
			}
			_gee_collection_object_unref0 (_base_type_it);
		}
	}
	result = FALSE;
	_vala_code_node_unref0 (dbus);
	_vala_code_node_unref0 (cl);
	return result;
}


static ValaCCodeFragment* vala_dbus_server_module_real_register_dbus_info (ValaCCodeModule* base, ValaObjectTypeSymbol* sym) {
	ValaDBusServerModule * self;
	ValaCCodeFragment* result;
	ValaCCodeFragment* fragment;
	ValaCCodeIdentifier* _tmp0_;
	ValaCCodeFunctionCall* _tmp1_;
	ValaCCodeFunctionCall* quark;
	ValaCCodeConstant* _tmp2_;
	ValaCCodeIdentifier* _tmp3_;
	ValaCCodeFunctionCall* _tmp4_;
	ValaCCodeFunctionCall* set_qdata;
	ValaCCodeIdentifier* _tmp6_;
	char* _tmp5_;
	ValaCCodeCastExpression* _tmp10_;
	ValaCCodeUnaryExpression* _tmp9_;
	ValaCCodeExpression* _tmp8_;
	ValaObjectType* _tmp7_;
	ValaCCodeExpressionStatement* _tmp11_;
	self = (ValaDBusServerModule*) base;
	g_return_val_if_fail (sym != NULL, NULL);
	fragment = vala_ccode_fragment_new ();
	if (!vala_dbus_server_module_type_implements_dbus_interface (self, sym)) {
		result = fragment;
		return result;
	}
	quark = (_tmp1_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp0_ = vala_ccode_identifier_new ("g_quark_from_static_string"))), _vala_ccode_node_unref0 (_tmp0_), _tmp1_);
	vala_ccode_function_call_add_argument (quark, (ValaCCodeExpression*) (_tmp2_ = vala_ccode_constant_new ("\"DBusObjectVTable\"")));
	_vala_ccode_node_unref0 (_tmp2_);
	set_qdata = (_tmp4_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("g_type_set_qdata"))), _vala_ccode_node_unref0 (_tmp3_), _tmp4_);
	vala_ccode_function_call_add_argument (set_qdata, (ValaCCodeExpression*) (_tmp6_ = vala_ccode_identifier_new (_tmp5_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) sym, "TYPE_"))));
	_vala_ccode_node_unref0 (_tmp6_);
	_g_free0 (_tmp5_);
	vala_ccode_function_call_add_argument (set_qdata, (ValaCCodeExpression*) quark);
	vala_ccode_function_call_add_argument (set_qdata, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) (_tmp9_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp8_ = vala_dbus_server_module_get_vtable (self, _tmp7_ = vala_object_type_new (sym)))), "void*")));
	_vala_ccode_node_unref0 (_tmp10_);
	_vala_ccode_node_unref0 (_tmp9_);
	_vala_ccode_node_unref0 (_tmp8_);
	_vala_code_node_unref0 (_tmp7_);
	vala_ccode_fragment_append (fragment, (ValaCCodeNode*) (_tmp11_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) set_qdata)));
	_vala_ccode_node_unref0 (_tmp11_);
	result = fragment;
	_vala_ccode_node_unref0 (quark);
	_vala_ccode_node_unref0 (set_qdata);
	return result;
}


static void vala_dbus_server_module_class_init (ValaDBusServerModuleClass * klass) {
	vala_dbus_server_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->visit_method_call = vala_dbus_server_module_real_visit_method_call;
	VALA_CCODE_MODULE_CLASS (klass)->register_dbus_info = vala_dbus_server_module_real_register_dbus_info;
}


static void vala_dbus_server_module_instance_init (ValaDBusServerModule * self) {
}


GType vala_dbus_server_module_get_type (void) {
	static GType vala_dbus_server_module_type_id = 0;
	if (vala_dbus_server_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaDBusServerModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_dbus_server_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaDBusServerModule), 0, (GInstanceInitFunc) vala_dbus_server_module_instance_init, NULL };
		vala_dbus_server_module_type_id = g_type_register_static (VALA_TYPE_DBUS_CLIENT_MODULE, "ValaDBusServerModule", &g_define_type_info, 0);
	}
	return vala_dbus_server_module_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




