/* valaflowanalyzer.vala
 *
 * Copyright (C) 2008-2009  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_CODE_VISITOR (vala_code_visitor_get_type ())
#define VALA_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitor))
#define VALA_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))
#define VALA_IS_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_VISITOR))
#define VALA_IS_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_VISITOR))
#define VALA_CODE_VISITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))

typedef struct _ValaCodeVisitor ValaCodeVisitor;
typedef struct _ValaCodeVisitorClass ValaCodeVisitorClass;
typedef struct _ValaCodeVisitorPrivate ValaCodeVisitorPrivate;

#define VALA_TYPE_SOURCE_FILE (vala_source_file_get_type ())
#define VALA_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFile))
#define VALA_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
#define VALA_IS_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_FILE))
#define VALA_IS_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_FILE))
#define VALA_SOURCE_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))

typedef struct _ValaSourceFile ValaSourceFile;
typedef struct _ValaSourceFileClass ValaSourceFileClass;

#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))

typedef struct _ValaCodeNode ValaCodeNode;
typedef struct _ValaCodeNodeClass ValaCodeNodeClass;

#define VALA_TYPE_SYMBOL (vala_symbol_get_type ())
#define VALA_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SYMBOL, ValaSymbol))
#define VALA_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SYMBOL, ValaSymbolClass))
#define VALA_IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SYMBOL))
#define VALA_IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SYMBOL))
#define VALA_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SYMBOL, ValaSymbolClass))

typedef struct _ValaSymbol ValaSymbol;
typedef struct _ValaSymbolClass ValaSymbolClass;

#define VALA_TYPE_NAMESPACE (vala_namespace_get_type ())
#define VALA_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_NAMESPACE, ValaNamespace))
#define VALA_NAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_NAMESPACE, ValaNamespaceClass))
#define VALA_IS_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_NAMESPACE))
#define VALA_IS_NAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_NAMESPACE))
#define VALA_NAMESPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_NAMESPACE, ValaNamespaceClass))

typedef struct _ValaNamespace ValaNamespace;
typedef struct _ValaNamespaceClass ValaNamespaceClass;

#define VALA_TYPE_TYPESYMBOL (vala_typesymbol_get_type ())
#define VALA_TYPESYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPESYMBOL, ValaTypeSymbol))
#define VALA_TYPESYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPESYMBOL, ValaTypeSymbolClass))
#define VALA_IS_TYPESYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPESYMBOL))
#define VALA_IS_TYPESYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPESYMBOL))
#define VALA_TYPESYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPESYMBOL, ValaTypeSymbolClass))

typedef struct _ValaTypeSymbol ValaTypeSymbol;
typedef struct _ValaTypeSymbolClass ValaTypeSymbolClass;

#define VALA_TYPE_OBJECT_TYPE_SYMBOL (vala_object_type_symbol_get_type ())
#define VALA_OBJECT_TYPE_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_OBJECT_TYPE_SYMBOL, ValaObjectTypeSymbol))
#define VALA_OBJECT_TYPE_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_OBJECT_TYPE_SYMBOL, ValaObjectTypeSymbolClass))
#define VALA_IS_OBJECT_TYPE_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_OBJECT_TYPE_SYMBOL))
#define VALA_IS_OBJECT_TYPE_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_OBJECT_TYPE_SYMBOL))
#define VALA_OBJECT_TYPE_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_OBJECT_TYPE_SYMBOL, ValaObjectTypeSymbolClass))

typedef struct _ValaObjectTypeSymbol ValaObjectTypeSymbol;
typedef struct _ValaObjectTypeSymbolClass ValaObjectTypeSymbolClass;

#define VALA_TYPE_CLASS (vala_class_get_type ())
#define VALA_CLASS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CLASS, ValaClass))
#define VALA_CLASS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CLASS, ValaClassClass))
#define VALA_IS_CLASS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CLASS))
#define VALA_IS_CLASS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CLASS))
#define VALA_CLASS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CLASS, ValaClassClass))

typedef struct _ValaClass ValaClass;
typedef struct _ValaClassClass ValaClassClass;

#define VALA_TYPE_STRUCT (vala_struct_get_type ())
#define VALA_STRUCT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_STRUCT, ValaStruct))
#define VALA_STRUCT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_STRUCT, ValaStructClass))
#define VALA_IS_STRUCT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_STRUCT))
#define VALA_IS_STRUCT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_STRUCT))
#define VALA_STRUCT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_STRUCT, ValaStructClass))

typedef struct _ValaStruct ValaStruct;
typedef struct _ValaStructClass ValaStructClass;

#define VALA_TYPE_INTERFACE (vala_interface_get_type ())
#define VALA_INTERFACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_INTERFACE, ValaInterface))
#define VALA_INTERFACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_INTERFACE, ValaInterfaceClass))
#define VALA_IS_INTERFACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_INTERFACE))
#define VALA_IS_INTERFACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_INTERFACE))
#define VALA_INTERFACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_INTERFACE, ValaInterfaceClass))

typedef struct _ValaInterface ValaInterface;
typedef struct _ValaInterfaceClass ValaInterfaceClass;

#define VALA_TYPE_ENUM (vala_enum_get_type ())
#define VALA_ENUM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ENUM, ValaEnum))
#define VALA_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ENUM, ValaEnumClass))
#define VALA_IS_ENUM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ENUM))
#define VALA_IS_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ENUM))
#define VALA_ENUM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ENUM, ValaEnumClass))

typedef struct _ValaEnum ValaEnum;
typedef struct _ValaEnumClass ValaEnumClass;

#define VALA_TYPE_ENUM_VALUE (vala_enum_value_get_type ())
#define VALA_ENUM_VALUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ENUM_VALUE, ValaEnumValue))
#define VALA_ENUM_VALUE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ENUM_VALUE, ValaEnumValueClass))
#define VALA_IS_ENUM_VALUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ENUM_VALUE))
#define VALA_IS_ENUM_VALUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ENUM_VALUE))
#define VALA_ENUM_VALUE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ENUM_VALUE, ValaEnumValueClass))

typedef struct _ValaEnumValue ValaEnumValue;
typedef struct _ValaEnumValueClass ValaEnumValueClass;

#define VALA_TYPE_ERROR_DOMAIN (vala_error_domain_get_type ())
#define VALA_ERROR_DOMAIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ERROR_DOMAIN, ValaErrorDomain))
#define VALA_ERROR_DOMAIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ERROR_DOMAIN, ValaErrorDomainClass))
#define VALA_IS_ERROR_DOMAIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ERROR_DOMAIN))
#define VALA_IS_ERROR_DOMAIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ERROR_DOMAIN))
#define VALA_ERROR_DOMAIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ERROR_DOMAIN, ValaErrorDomainClass))

typedef struct _ValaErrorDomain ValaErrorDomain;
typedef struct _ValaErrorDomainClass ValaErrorDomainClass;

#define VALA_TYPE_ERROR_CODE (vala_error_code_get_type ())
#define VALA_ERROR_CODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ERROR_CODE, ValaErrorCode))
#define VALA_ERROR_CODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ERROR_CODE, ValaErrorCodeClass))
#define VALA_IS_ERROR_CODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ERROR_CODE))
#define VALA_IS_ERROR_CODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ERROR_CODE))
#define VALA_ERROR_CODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ERROR_CODE, ValaErrorCodeClass))

typedef struct _ValaErrorCode ValaErrorCode;
typedef struct _ValaErrorCodeClass ValaErrorCodeClass;

#define VALA_TYPE_DELEGATE (vala_delegate_get_type ())
#define VALA_DELEGATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DELEGATE, ValaDelegate))
#define VALA_DELEGATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DELEGATE, ValaDelegateClass))
#define VALA_IS_DELEGATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DELEGATE))
#define VALA_IS_DELEGATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DELEGATE))
#define VALA_DELEGATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DELEGATE, ValaDelegateClass))

typedef struct _ValaDelegate ValaDelegate;
typedef struct _ValaDelegateClass ValaDelegateClass;

#define VALA_TYPE_MEMBER (vala_member_get_type ())
#define VALA_MEMBER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_MEMBER, ValaMember))
#define VALA_MEMBER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_MEMBER, ValaMemberClass))
#define VALA_IS_MEMBER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_MEMBER))
#define VALA_IS_MEMBER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_MEMBER))
#define VALA_MEMBER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_MEMBER, ValaMemberClass))

typedef struct _ValaMember ValaMember;
typedef struct _ValaMemberClass ValaMemberClass;

#define VALA_TYPE_CONSTANT (vala_constant_get_type ())
#define VALA_CONSTANT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CONSTANT, ValaConstant))
#define VALA_CONSTANT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CONSTANT, ValaConstantClass))
#define VALA_IS_CONSTANT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CONSTANT))
#define VALA_IS_CONSTANT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CONSTANT))
#define VALA_CONSTANT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CONSTANT, ValaConstantClass))

typedef struct _ValaConstant ValaConstant;
typedef struct _ValaConstantClass ValaConstantClass;

#define VALA_TYPE_FIELD (vala_field_get_type ())
#define VALA_FIELD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_FIELD, ValaField))
#define VALA_FIELD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_FIELD, ValaFieldClass))
#define VALA_IS_FIELD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_FIELD))
#define VALA_IS_FIELD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_FIELD))
#define VALA_FIELD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_FIELD, ValaFieldClass))

typedef struct _ValaField ValaField;
typedef struct _ValaFieldClass ValaFieldClass;

#define VALA_TYPE_METHOD (vala_method_get_type ())
#define VALA_METHOD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_METHOD, ValaMethod))
#define VALA_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_METHOD, ValaMethodClass))
#define VALA_IS_METHOD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_METHOD))
#define VALA_IS_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_METHOD))
#define VALA_METHOD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_METHOD, ValaMethodClass))

typedef struct _ValaMethod ValaMethod;
typedef struct _ValaMethodClass ValaMethodClass;

#define VALA_TYPE_CREATION_METHOD (vala_creation_method_get_type ())
#define VALA_CREATION_METHOD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CREATION_METHOD, ValaCreationMethod))
#define VALA_CREATION_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CREATION_METHOD, ValaCreationMethodClass))
#define VALA_IS_CREATION_METHOD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CREATION_METHOD))
#define VALA_IS_CREATION_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CREATION_METHOD))
#define VALA_CREATION_METHOD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CREATION_METHOD, ValaCreationMethodClass))

typedef struct _ValaCreationMethod ValaCreationMethod;
typedef struct _ValaCreationMethodClass ValaCreationMethodClass;

#define VALA_TYPE_FORMAL_PARAMETER (vala_formal_parameter_get_type ())
#define VALA_FORMAL_PARAMETER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_FORMAL_PARAMETER, ValaFormalParameter))
#define VALA_FORMAL_PARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_FORMAL_PARAMETER, ValaFormalParameterClass))
#define VALA_IS_FORMAL_PARAMETER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_FORMAL_PARAMETER))
#define VALA_IS_FORMAL_PARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_FORMAL_PARAMETER))
#define VALA_FORMAL_PARAMETER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_FORMAL_PARAMETER, ValaFormalParameterClass))

typedef struct _ValaFormalParameter ValaFormalParameter;
typedef struct _ValaFormalParameterClass ValaFormalParameterClass;

#define VALA_TYPE_PROPERTY (vala_property_get_type ())
#define VALA_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PROPERTY, ValaProperty))
#define VALA_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PROPERTY, ValaPropertyClass))
#define VALA_IS_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PROPERTY))
#define VALA_IS_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PROPERTY))
#define VALA_PROPERTY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PROPERTY, ValaPropertyClass))

typedef struct _ValaProperty ValaProperty;
typedef struct _ValaPropertyClass ValaPropertyClass;

#define VALA_TYPE_PROPERTY_ACCESSOR (vala_property_accessor_get_type ())
#define VALA_PROPERTY_ACCESSOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PROPERTY_ACCESSOR, ValaPropertyAccessor))
#define VALA_PROPERTY_ACCESSOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PROPERTY_ACCESSOR, ValaPropertyAccessorClass))
#define VALA_IS_PROPERTY_ACCESSOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PROPERTY_ACCESSOR))
#define VALA_IS_PROPERTY_ACCESSOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PROPERTY_ACCESSOR))
#define VALA_PROPERTY_ACCESSOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PROPERTY_ACCESSOR, ValaPropertyAccessorClass))

typedef struct _ValaPropertyAccessor ValaPropertyAccessor;
typedef struct _ValaPropertyAccessorClass ValaPropertyAccessorClass;

#define VALA_TYPE_SIGNAL (vala_signal_get_type ())
#define VALA_SIGNAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SIGNAL, ValaSignal))
#define VALA_SIGNAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SIGNAL, ValaSignalClass))
#define VALA_IS_SIGNAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SIGNAL))
#define VALA_IS_SIGNAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SIGNAL))
#define VALA_SIGNAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SIGNAL, ValaSignalClass))

typedef struct _ValaSignal ValaSignal;
typedef struct _ValaSignalClass ValaSignalClass;

#define VALA_TYPE_CONSTRUCTOR (vala_constructor_get_type ())
#define VALA_CONSTRUCTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CONSTRUCTOR, ValaConstructor))
#define VALA_CONSTRUCTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CONSTRUCTOR, ValaConstructorClass))
#define VALA_IS_CONSTRUCTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CONSTRUCTOR))
#define VALA_IS_CONSTRUCTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CONSTRUCTOR))
#define VALA_CONSTRUCTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CONSTRUCTOR, ValaConstructorClass))

typedef struct _ValaConstructor ValaConstructor;
typedef struct _ValaConstructorClass ValaConstructorClass;

#define VALA_TYPE_DESTRUCTOR (vala_destructor_get_type ())
#define VALA_DESTRUCTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DESTRUCTOR, ValaDestructor))
#define VALA_DESTRUCTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DESTRUCTOR, ValaDestructorClass))
#define VALA_IS_DESTRUCTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DESTRUCTOR))
#define VALA_IS_DESTRUCTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DESTRUCTOR))
#define VALA_DESTRUCTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DESTRUCTOR, ValaDestructorClass))

typedef struct _ValaDestructor ValaDestructor;
typedef struct _ValaDestructorClass ValaDestructorClass;

#define VALA_TYPE_TYPEPARAMETER (vala_typeparameter_get_type ())
#define VALA_TYPEPARAMETER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEPARAMETER, ValaTypeParameter))
#define VALA_TYPEPARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEPARAMETER, ValaTypeParameterClass))
#define VALA_IS_TYPEPARAMETER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEPARAMETER))
#define VALA_IS_TYPEPARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEPARAMETER))
#define VALA_TYPEPARAMETER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEPARAMETER, ValaTypeParameterClass))

typedef struct _ValaTypeParameter ValaTypeParameter;
typedef struct _ValaTypeParameterClass ValaTypeParameterClass;

#define VALA_TYPE_USING_DIRECTIVE (vala_using_directive_get_type ())
#define VALA_USING_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_USING_DIRECTIVE, ValaUsingDirective))
#define VALA_USING_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_USING_DIRECTIVE, ValaUsingDirectiveClass))
#define VALA_IS_USING_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_USING_DIRECTIVE))
#define VALA_IS_USING_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_USING_DIRECTIVE))
#define VALA_USING_DIRECTIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_USING_DIRECTIVE, ValaUsingDirectiveClass))

typedef struct _ValaUsingDirective ValaUsingDirective;
typedef struct _ValaUsingDirectiveClass ValaUsingDirectiveClass;

#define VALA_TYPE_DATA_TYPE (vala_data_type_get_type ())
#define VALA_DATA_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DATA_TYPE, ValaDataType))
#define VALA_DATA_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DATA_TYPE, ValaDataTypeClass))
#define VALA_IS_DATA_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DATA_TYPE))
#define VALA_IS_DATA_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DATA_TYPE))
#define VALA_DATA_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DATA_TYPE, ValaDataTypeClass))

typedef struct _ValaDataType ValaDataType;
typedef struct _ValaDataTypeClass ValaDataTypeClass;

#define VALA_TYPE_BLOCK (vala_block_get_type ())
#define VALA_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BLOCK, ValaBlock))
#define VALA_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BLOCK, ValaBlockClass))
#define VALA_IS_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BLOCK))
#define VALA_IS_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BLOCK))
#define VALA_BLOCK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BLOCK, ValaBlockClass))

typedef struct _ValaBlock ValaBlock;
typedef struct _ValaBlockClass ValaBlockClass;

#define VALA_TYPE_EMPTY_STATEMENT (vala_empty_statement_get_type ())
#define VALA_EMPTY_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_EMPTY_STATEMENT, ValaEmptyStatement))
#define VALA_EMPTY_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_EMPTY_STATEMENT, ValaEmptyStatementClass))
#define VALA_IS_EMPTY_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_EMPTY_STATEMENT))
#define VALA_IS_EMPTY_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_EMPTY_STATEMENT))
#define VALA_EMPTY_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_EMPTY_STATEMENT, ValaEmptyStatementClass))

typedef struct _ValaEmptyStatement ValaEmptyStatement;
typedef struct _ValaEmptyStatementClass ValaEmptyStatementClass;

#define VALA_TYPE_DECLARATION_STATEMENT (vala_declaration_statement_get_type ())
#define VALA_DECLARATION_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DECLARATION_STATEMENT, ValaDeclarationStatement))
#define VALA_DECLARATION_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DECLARATION_STATEMENT, ValaDeclarationStatementClass))
#define VALA_IS_DECLARATION_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DECLARATION_STATEMENT))
#define VALA_IS_DECLARATION_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DECLARATION_STATEMENT))
#define VALA_DECLARATION_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DECLARATION_STATEMENT, ValaDeclarationStatementClass))

typedef struct _ValaDeclarationStatement ValaDeclarationStatement;
typedef struct _ValaDeclarationStatementClass ValaDeclarationStatementClass;

#define VALA_TYPE_LOCAL_VARIABLE (vala_local_variable_get_type ())
#define VALA_LOCAL_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariable))
#define VALA_LOCAL_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariableClass))
#define VALA_IS_LOCAL_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_LOCAL_VARIABLE))
#define VALA_IS_LOCAL_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_LOCAL_VARIABLE))
#define VALA_LOCAL_VARIABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariableClass))

typedef struct _ValaLocalVariable ValaLocalVariable;
typedef struct _ValaLocalVariableClass ValaLocalVariableClass;

#define VALA_TYPE_EXPRESSION (vala_expression_get_type ())
#define VALA_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_EXPRESSION, ValaExpression))
#define VALA_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_EXPRESSION, ValaExpressionClass))
#define VALA_IS_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_EXPRESSION))
#define VALA_IS_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_EXPRESSION))
#define VALA_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_EXPRESSION, ValaExpressionClass))

typedef struct _ValaExpression ValaExpression;
typedef struct _ValaExpressionClass ValaExpressionClass;

#define VALA_TYPE_INITIALIZER_LIST (vala_initializer_list_get_type ())
#define VALA_INITIALIZER_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_INITIALIZER_LIST, ValaInitializerList))
#define VALA_INITIALIZER_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_INITIALIZER_LIST, ValaInitializerListClass))
#define VALA_IS_INITIALIZER_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_INITIALIZER_LIST))
#define VALA_IS_INITIALIZER_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_INITIALIZER_LIST))
#define VALA_INITIALIZER_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_INITIALIZER_LIST, ValaInitializerListClass))

typedef struct _ValaInitializerList ValaInitializerList;
typedef struct _ValaInitializerListClass ValaInitializerListClass;

#define VALA_TYPE_EXPRESSION_STATEMENT (vala_expression_statement_get_type ())
#define VALA_EXPRESSION_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_EXPRESSION_STATEMENT, ValaExpressionStatement))
#define VALA_EXPRESSION_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_EXPRESSION_STATEMENT, ValaExpressionStatementClass))
#define VALA_IS_EXPRESSION_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_EXPRESSION_STATEMENT))
#define VALA_IS_EXPRESSION_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_EXPRESSION_STATEMENT))
#define VALA_EXPRESSION_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_EXPRESSION_STATEMENT, ValaExpressionStatementClass))

typedef struct _ValaExpressionStatement ValaExpressionStatement;
typedef struct _ValaExpressionStatementClass ValaExpressionStatementClass;

#define VALA_TYPE_IF_STATEMENT (vala_if_statement_get_type ())
#define VALA_IF_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_IF_STATEMENT, ValaIfStatement))
#define VALA_IF_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_IF_STATEMENT, ValaIfStatementClass))
#define VALA_IS_IF_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_IF_STATEMENT))
#define VALA_IS_IF_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_IF_STATEMENT))
#define VALA_IF_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_IF_STATEMENT, ValaIfStatementClass))

typedef struct _ValaIfStatement ValaIfStatement;
typedef struct _ValaIfStatementClass ValaIfStatementClass;

#define VALA_TYPE_SWITCH_STATEMENT (vala_switch_statement_get_type ())
#define VALA_SWITCH_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SWITCH_STATEMENT, ValaSwitchStatement))
#define VALA_SWITCH_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SWITCH_STATEMENT, ValaSwitchStatementClass))
#define VALA_IS_SWITCH_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SWITCH_STATEMENT))
#define VALA_IS_SWITCH_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SWITCH_STATEMENT))
#define VALA_SWITCH_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SWITCH_STATEMENT, ValaSwitchStatementClass))

typedef struct _ValaSwitchStatement ValaSwitchStatement;
typedef struct _ValaSwitchStatementClass ValaSwitchStatementClass;

#define VALA_TYPE_SWITCH_SECTION (vala_switch_section_get_type ())
#define VALA_SWITCH_SECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SWITCH_SECTION, ValaSwitchSection))
#define VALA_SWITCH_SECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SWITCH_SECTION, ValaSwitchSectionClass))
#define VALA_IS_SWITCH_SECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SWITCH_SECTION))
#define VALA_IS_SWITCH_SECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SWITCH_SECTION))
#define VALA_SWITCH_SECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SWITCH_SECTION, ValaSwitchSectionClass))

typedef struct _ValaSwitchSection ValaSwitchSection;
typedef struct _ValaSwitchSectionClass ValaSwitchSectionClass;

#define VALA_TYPE_SWITCH_LABEL (vala_switch_label_get_type ())
#define VALA_SWITCH_LABEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SWITCH_LABEL, ValaSwitchLabel))
#define VALA_SWITCH_LABEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SWITCH_LABEL, ValaSwitchLabelClass))
#define VALA_IS_SWITCH_LABEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SWITCH_LABEL))
#define VALA_IS_SWITCH_LABEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SWITCH_LABEL))
#define VALA_SWITCH_LABEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SWITCH_LABEL, ValaSwitchLabelClass))

typedef struct _ValaSwitchLabel ValaSwitchLabel;
typedef struct _ValaSwitchLabelClass ValaSwitchLabelClass;

#define VALA_TYPE_LOOP (vala_loop_get_type ())
#define VALA_LOOP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_LOOP, ValaLoop))
#define VALA_LOOP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_LOOP, ValaLoopClass))
#define VALA_IS_LOOP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_LOOP))
#define VALA_IS_LOOP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_LOOP))
#define VALA_LOOP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_LOOP, ValaLoopClass))

typedef struct _ValaLoop ValaLoop;
typedef struct _ValaLoopClass ValaLoopClass;

#define VALA_TYPE_WHILE_STATEMENT (vala_while_statement_get_type ())
#define VALA_WHILE_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_WHILE_STATEMENT, ValaWhileStatement))
#define VALA_WHILE_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_WHILE_STATEMENT, ValaWhileStatementClass))
#define VALA_IS_WHILE_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_WHILE_STATEMENT))
#define VALA_IS_WHILE_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_WHILE_STATEMENT))
#define VALA_WHILE_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_WHILE_STATEMENT, ValaWhileStatementClass))

typedef struct _ValaWhileStatement ValaWhileStatement;
typedef struct _ValaWhileStatementClass ValaWhileStatementClass;

#define VALA_TYPE_DO_STATEMENT (vala_do_statement_get_type ())
#define VALA_DO_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DO_STATEMENT, ValaDoStatement))
#define VALA_DO_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DO_STATEMENT, ValaDoStatementClass))
#define VALA_IS_DO_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DO_STATEMENT))
#define VALA_IS_DO_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DO_STATEMENT))
#define VALA_DO_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DO_STATEMENT, ValaDoStatementClass))

typedef struct _ValaDoStatement ValaDoStatement;
typedef struct _ValaDoStatementClass ValaDoStatementClass;

#define VALA_TYPE_FOR_STATEMENT (vala_for_statement_get_type ())
#define VALA_FOR_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_FOR_STATEMENT, ValaForStatement))
#define VALA_FOR_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_FOR_STATEMENT, ValaForStatementClass))
#define VALA_IS_FOR_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_FOR_STATEMENT))
#define VALA_IS_FOR_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_FOR_STATEMENT))
#define VALA_FOR_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_FOR_STATEMENT, ValaForStatementClass))

typedef struct _ValaForStatement ValaForStatement;
typedef struct _ValaForStatementClass ValaForStatementClass;

#define VALA_TYPE_FOREACH_STATEMENT (vala_foreach_statement_get_type ())
#define VALA_FOREACH_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_FOREACH_STATEMENT, ValaForeachStatement))
#define VALA_FOREACH_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_FOREACH_STATEMENT, ValaForeachStatementClass))
#define VALA_IS_FOREACH_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_FOREACH_STATEMENT))
#define VALA_IS_FOREACH_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_FOREACH_STATEMENT))
#define VALA_FOREACH_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_FOREACH_STATEMENT, ValaForeachStatementClass))

typedef struct _ValaForeachStatement ValaForeachStatement;
typedef struct _ValaForeachStatementClass ValaForeachStatementClass;

#define VALA_TYPE_BREAK_STATEMENT (vala_break_statement_get_type ())
#define VALA_BREAK_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BREAK_STATEMENT, ValaBreakStatement))
#define VALA_BREAK_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BREAK_STATEMENT, ValaBreakStatementClass))
#define VALA_IS_BREAK_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BREAK_STATEMENT))
#define VALA_IS_BREAK_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BREAK_STATEMENT))
#define VALA_BREAK_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BREAK_STATEMENT, ValaBreakStatementClass))

typedef struct _ValaBreakStatement ValaBreakStatement;
typedef struct _ValaBreakStatementClass ValaBreakStatementClass;

#define VALA_TYPE_CONTINUE_STATEMENT (vala_continue_statement_get_type ())
#define VALA_CONTINUE_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CONTINUE_STATEMENT, ValaContinueStatement))
#define VALA_CONTINUE_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CONTINUE_STATEMENT, ValaContinueStatementClass))
#define VALA_IS_CONTINUE_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CONTINUE_STATEMENT))
#define VALA_IS_CONTINUE_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CONTINUE_STATEMENT))
#define VALA_CONTINUE_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CONTINUE_STATEMENT, ValaContinueStatementClass))

typedef struct _ValaContinueStatement ValaContinueStatement;
typedef struct _ValaContinueStatementClass ValaContinueStatementClass;

#define VALA_TYPE_RETURN_STATEMENT (vala_return_statement_get_type ())
#define VALA_RETURN_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_RETURN_STATEMENT, ValaReturnStatement))
#define VALA_RETURN_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_RETURN_STATEMENT, ValaReturnStatementClass))
#define VALA_IS_RETURN_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_RETURN_STATEMENT))
#define VALA_IS_RETURN_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_RETURN_STATEMENT))
#define VALA_RETURN_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_RETURN_STATEMENT, ValaReturnStatementClass))

typedef struct _ValaReturnStatement ValaReturnStatement;
typedef struct _ValaReturnStatementClass ValaReturnStatementClass;

#define VALA_TYPE_YIELD_STATEMENT (vala_yield_statement_get_type ())
#define VALA_YIELD_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_YIELD_STATEMENT, ValaYieldStatement))
#define VALA_YIELD_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_YIELD_STATEMENT, ValaYieldStatementClass))
#define VALA_IS_YIELD_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_YIELD_STATEMENT))
#define VALA_IS_YIELD_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_YIELD_STATEMENT))
#define VALA_YIELD_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_YIELD_STATEMENT, ValaYieldStatementClass))

typedef struct _ValaYieldStatement ValaYieldStatement;
typedef struct _ValaYieldStatementClass ValaYieldStatementClass;

#define VALA_TYPE_THROW_STATEMENT (vala_throw_statement_get_type ())
#define VALA_THROW_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_THROW_STATEMENT, ValaThrowStatement))
#define VALA_THROW_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_THROW_STATEMENT, ValaThrowStatementClass))
#define VALA_IS_THROW_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_THROW_STATEMENT))
#define VALA_IS_THROW_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_THROW_STATEMENT))
#define VALA_THROW_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_THROW_STATEMENT, ValaThrowStatementClass))

typedef struct _ValaThrowStatement ValaThrowStatement;
typedef struct _ValaThrowStatementClass ValaThrowStatementClass;

#define VALA_TYPE_TRY_STATEMENT (vala_try_statement_get_type ())
#define VALA_TRY_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TRY_STATEMENT, ValaTryStatement))
#define VALA_TRY_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TRY_STATEMENT, ValaTryStatementClass))
#define VALA_IS_TRY_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TRY_STATEMENT))
#define VALA_IS_TRY_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TRY_STATEMENT))
#define VALA_TRY_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TRY_STATEMENT, ValaTryStatementClass))

typedef struct _ValaTryStatement ValaTryStatement;
typedef struct _ValaTryStatementClass ValaTryStatementClass;

#define VALA_TYPE_CATCH_CLAUSE (vala_catch_clause_get_type ())
#define VALA_CATCH_CLAUSE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CATCH_CLAUSE, ValaCatchClause))
#define VALA_CATCH_CLAUSE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CATCH_CLAUSE, ValaCatchClauseClass))
#define VALA_IS_CATCH_CLAUSE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CATCH_CLAUSE))
#define VALA_IS_CATCH_CLAUSE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CATCH_CLAUSE))
#define VALA_CATCH_CLAUSE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CATCH_CLAUSE, ValaCatchClauseClass))

typedef struct _ValaCatchClause ValaCatchClause;
typedef struct _ValaCatchClauseClass ValaCatchClauseClass;

#define VALA_TYPE_LOCK_STATEMENT (vala_lock_statement_get_type ())
#define VALA_LOCK_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_LOCK_STATEMENT, ValaLockStatement))
#define VALA_LOCK_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_LOCK_STATEMENT, ValaLockStatementClass))
#define VALA_IS_LOCK_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_LOCK_STATEMENT))
#define VALA_IS_LOCK_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_LOCK_STATEMENT))
#define VALA_LOCK_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_LOCK_STATEMENT, ValaLockStatementClass))

typedef struct _ValaLockStatement ValaLockStatement;
typedef struct _ValaLockStatementClass ValaLockStatementClass;

#define VALA_TYPE_DELETE_STATEMENT (vala_delete_statement_get_type ())
#define VALA_DELETE_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DELETE_STATEMENT, ValaDeleteStatement))
#define VALA_DELETE_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DELETE_STATEMENT, ValaDeleteStatementClass))
#define VALA_IS_DELETE_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DELETE_STATEMENT))
#define VALA_IS_DELETE_STATEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DELETE_STATEMENT))
#define VALA_DELETE_STATEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DELETE_STATEMENT, ValaDeleteStatementClass))

typedef struct _ValaDeleteStatement ValaDeleteStatement;
typedef struct _ValaDeleteStatementClass ValaDeleteStatementClass;

#define VALA_TYPE_ARRAY_CREATION_EXPRESSION (vala_array_creation_expression_get_type ())
#define VALA_ARRAY_CREATION_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ARRAY_CREATION_EXPRESSION, ValaArrayCreationExpression))
#define VALA_ARRAY_CREATION_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ARRAY_CREATION_EXPRESSION, ValaArrayCreationExpressionClass))
#define VALA_IS_ARRAY_CREATION_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ARRAY_CREATION_EXPRESSION))
#define VALA_IS_ARRAY_CREATION_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ARRAY_CREATION_EXPRESSION))
#define VALA_ARRAY_CREATION_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ARRAY_CREATION_EXPRESSION, ValaArrayCreationExpressionClass))

typedef struct _ValaArrayCreationExpression ValaArrayCreationExpression;
typedef struct _ValaArrayCreationExpressionClass ValaArrayCreationExpressionClass;

#define VALA_TYPE_LITERAL (vala_literal_get_type ())
#define VALA_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_LITERAL, ValaLiteral))
#define VALA_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_LITERAL, ValaLiteralClass))
#define VALA_IS_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_LITERAL))
#define VALA_IS_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_LITERAL))
#define VALA_LITERAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_LITERAL, ValaLiteralClass))

typedef struct _ValaLiteral ValaLiteral;
typedef struct _ValaLiteralClass ValaLiteralClass;

#define VALA_TYPE_BOOLEAN_LITERAL (vala_boolean_literal_get_type ())
#define VALA_BOOLEAN_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BOOLEAN_LITERAL, ValaBooleanLiteral))
#define VALA_BOOLEAN_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BOOLEAN_LITERAL, ValaBooleanLiteralClass))
#define VALA_IS_BOOLEAN_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BOOLEAN_LITERAL))
#define VALA_IS_BOOLEAN_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BOOLEAN_LITERAL))
#define VALA_BOOLEAN_LITERAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BOOLEAN_LITERAL, ValaBooleanLiteralClass))

typedef struct _ValaBooleanLiteral ValaBooleanLiteral;
typedef struct _ValaBooleanLiteralClass ValaBooleanLiteralClass;

#define VALA_TYPE_CHARACTER_LITERAL (vala_character_literal_get_type ())
#define VALA_CHARACTER_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CHARACTER_LITERAL, ValaCharacterLiteral))
#define VALA_CHARACTER_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CHARACTER_LITERAL, ValaCharacterLiteralClass))
#define VALA_IS_CHARACTER_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CHARACTER_LITERAL))
#define VALA_IS_CHARACTER_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CHARACTER_LITERAL))
#define VALA_CHARACTER_LITERAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CHARACTER_LITERAL, ValaCharacterLiteralClass))

typedef struct _ValaCharacterLiteral ValaCharacterLiteral;
typedef struct _ValaCharacterLiteralClass ValaCharacterLiteralClass;

#define VALA_TYPE_INTEGER_LITERAL (vala_integer_literal_get_type ())
#define VALA_INTEGER_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_INTEGER_LITERAL, ValaIntegerLiteral))
#define VALA_INTEGER_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_INTEGER_LITERAL, ValaIntegerLiteralClass))
#define VALA_IS_INTEGER_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_INTEGER_LITERAL))
#define VALA_IS_INTEGER_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_INTEGER_LITERAL))
#define VALA_INTEGER_LITERAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_INTEGER_LITERAL, ValaIntegerLiteralClass))

typedef struct _ValaIntegerLiteral ValaIntegerLiteral;
typedef struct _ValaIntegerLiteralClass ValaIntegerLiteralClass;

#define VALA_TYPE_REAL_LITERAL (vala_real_literal_get_type ())
#define VALA_REAL_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_REAL_LITERAL, ValaRealLiteral))
#define VALA_REAL_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_REAL_LITERAL, ValaRealLiteralClass))
#define VALA_IS_REAL_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_REAL_LITERAL))
#define VALA_IS_REAL_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_REAL_LITERAL))
#define VALA_REAL_LITERAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_REAL_LITERAL, ValaRealLiteralClass))

typedef struct _ValaRealLiteral ValaRealLiteral;
typedef struct _ValaRealLiteralClass ValaRealLiteralClass;

#define VALA_TYPE_STRING_LITERAL (vala_string_literal_get_type ())
#define VALA_STRING_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_STRING_LITERAL, ValaStringLiteral))
#define VALA_STRING_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_STRING_LITERAL, ValaStringLiteralClass))
#define VALA_IS_STRING_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_STRING_LITERAL))
#define VALA_IS_STRING_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_STRING_LITERAL))
#define VALA_STRING_LITERAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_STRING_LITERAL, ValaStringLiteralClass))

typedef struct _ValaStringLiteral ValaStringLiteral;
typedef struct _ValaStringLiteralClass ValaStringLiteralClass;

#define VALA_TYPE_NULL_LITERAL (vala_null_literal_get_type ())
#define VALA_NULL_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_NULL_LITERAL, ValaNullLiteral))
#define VALA_NULL_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_NULL_LITERAL, ValaNullLiteralClass))
#define VALA_IS_NULL_LITERAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_NULL_LITERAL))
#define VALA_IS_NULL_LITERAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_NULL_LITERAL))
#define VALA_NULL_LITERAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_NULL_LITERAL, ValaNullLiteralClass))

typedef struct _ValaNullLiteral ValaNullLiteral;
typedef struct _ValaNullLiteralClass ValaNullLiteralClass;

#define VALA_TYPE_MEMBER_ACCESS (vala_member_access_get_type ())
#define VALA_MEMBER_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_MEMBER_ACCESS, ValaMemberAccess))
#define VALA_MEMBER_ACCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_MEMBER_ACCESS, ValaMemberAccessClass))
#define VALA_IS_MEMBER_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_MEMBER_ACCESS))
#define VALA_IS_MEMBER_ACCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_MEMBER_ACCESS))
#define VALA_MEMBER_ACCESS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_MEMBER_ACCESS, ValaMemberAccessClass))

typedef struct _ValaMemberAccess ValaMemberAccess;
typedef struct _ValaMemberAccessClass ValaMemberAccessClass;

#define VALA_TYPE_METHOD_CALL (vala_method_call_get_type ())
#define VALA_METHOD_CALL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_METHOD_CALL, ValaMethodCall))
#define VALA_METHOD_CALL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_METHOD_CALL, ValaMethodCallClass))
#define VALA_IS_METHOD_CALL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_METHOD_CALL))
#define VALA_IS_METHOD_CALL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_METHOD_CALL))
#define VALA_METHOD_CALL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_METHOD_CALL, ValaMethodCallClass))

typedef struct _ValaMethodCall ValaMethodCall;
typedef struct _ValaMethodCallClass ValaMethodCallClass;

#define VALA_TYPE_ELEMENT_ACCESS (vala_element_access_get_type ())
#define VALA_ELEMENT_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ELEMENT_ACCESS, ValaElementAccess))
#define VALA_ELEMENT_ACCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ELEMENT_ACCESS, ValaElementAccessClass))
#define VALA_IS_ELEMENT_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ELEMENT_ACCESS))
#define VALA_IS_ELEMENT_ACCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ELEMENT_ACCESS))
#define VALA_ELEMENT_ACCESS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ELEMENT_ACCESS, ValaElementAccessClass))

typedef struct _ValaElementAccess ValaElementAccess;
typedef struct _ValaElementAccessClass ValaElementAccessClass;

#define VALA_TYPE_BASE_ACCESS (vala_base_access_get_type ())
#define VALA_BASE_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BASE_ACCESS, ValaBaseAccess))
#define VALA_BASE_ACCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BASE_ACCESS, ValaBaseAccessClass))
#define VALA_IS_BASE_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BASE_ACCESS))
#define VALA_IS_BASE_ACCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BASE_ACCESS))
#define VALA_BASE_ACCESS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BASE_ACCESS, ValaBaseAccessClass))

typedef struct _ValaBaseAccess ValaBaseAccess;
typedef struct _ValaBaseAccessClass ValaBaseAccessClass;

#define VALA_TYPE_POSTFIX_EXPRESSION (vala_postfix_expression_get_type ())
#define VALA_POSTFIX_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_POSTFIX_EXPRESSION, ValaPostfixExpression))
#define VALA_POSTFIX_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_POSTFIX_EXPRESSION, ValaPostfixExpressionClass))
#define VALA_IS_POSTFIX_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_POSTFIX_EXPRESSION))
#define VALA_IS_POSTFIX_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_POSTFIX_EXPRESSION))
#define VALA_POSTFIX_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_POSTFIX_EXPRESSION, ValaPostfixExpressionClass))

typedef struct _ValaPostfixExpression ValaPostfixExpression;
typedef struct _ValaPostfixExpressionClass ValaPostfixExpressionClass;

#define VALA_TYPE_OBJECT_CREATION_EXPRESSION (vala_object_creation_expression_get_type ())
#define VALA_OBJECT_CREATION_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_OBJECT_CREATION_EXPRESSION, ValaObjectCreationExpression))
#define VALA_OBJECT_CREATION_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_OBJECT_CREATION_EXPRESSION, ValaObjectCreationExpressionClass))
#define VALA_IS_OBJECT_CREATION_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_OBJECT_CREATION_EXPRESSION))
#define VALA_IS_OBJECT_CREATION_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_OBJECT_CREATION_EXPRESSION))
#define VALA_OBJECT_CREATION_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_OBJECT_CREATION_EXPRESSION, ValaObjectCreationExpressionClass))

typedef struct _ValaObjectCreationExpression ValaObjectCreationExpression;
typedef struct _ValaObjectCreationExpressionClass ValaObjectCreationExpressionClass;

#define VALA_TYPE_SIZEOF_EXPRESSION (vala_sizeof_expression_get_type ())
#define VALA_SIZEOF_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SIZEOF_EXPRESSION, ValaSizeofExpression))
#define VALA_SIZEOF_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SIZEOF_EXPRESSION, ValaSizeofExpressionClass))
#define VALA_IS_SIZEOF_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SIZEOF_EXPRESSION))
#define VALA_IS_SIZEOF_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SIZEOF_EXPRESSION))
#define VALA_SIZEOF_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SIZEOF_EXPRESSION, ValaSizeofExpressionClass))

typedef struct _ValaSizeofExpression ValaSizeofExpression;
typedef struct _ValaSizeofExpressionClass ValaSizeofExpressionClass;

#define VALA_TYPE_TYPEOF_EXPRESSION (vala_typeof_expression_get_type ())
#define VALA_TYPEOF_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEOF_EXPRESSION, ValaTypeofExpression))
#define VALA_TYPEOF_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEOF_EXPRESSION, ValaTypeofExpressionClass))
#define VALA_IS_TYPEOF_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEOF_EXPRESSION))
#define VALA_IS_TYPEOF_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEOF_EXPRESSION))
#define VALA_TYPEOF_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEOF_EXPRESSION, ValaTypeofExpressionClass))

typedef struct _ValaTypeofExpression ValaTypeofExpression;
typedef struct _ValaTypeofExpressionClass ValaTypeofExpressionClass;

#define VALA_TYPE_UNARY_EXPRESSION (vala_unary_expression_get_type ())
#define VALA_UNARY_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_UNARY_EXPRESSION, ValaUnaryExpression))
#define VALA_UNARY_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_UNARY_EXPRESSION, ValaUnaryExpressionClass))
#define VALA_IS_UNARY_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_UNARY_EXPRESSION))
#define VALA_IS_UNARY_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_UNARY_EXPRESSION))
#define VALA_UNARY_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_UNARY_EXPRESSION, ValaUnaryExpressionClass))

typedef struct _ValaUnaryExpression ValaUnaryExpression;
typedef struct _ValaUnaryExpressionClass ValaUnaryExpressionClass;

#define VALA_TYPE_CAST_EXPRESSION (vala_cast_expression_get_type ())
#define VALA_CAST_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CAST_EXPRESSION, ValaCastExpression))
#define VALA_CAST_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CAST_EXPRESSION, ValaCastExpressionClass))
#define VALA_IS_CAST_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CAST_EXPRESSION))
#define VALA_IS_CAST_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CAST_EXPRESSION))
#define VALA_CAST_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CAST_EXPRESSION, ValaCastExpressionClass))

typedef struct _ValaCastExpression ValaCastExpression;
typedef struct _ValaCastExpressionClass ValaCastExpressionClass;

#define VALA_TYPE_POINTER_INDIRECTION (vala_pointer_indirection_get_type ())
#define VALA_POINTER_INDIRECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_POINTER_INDIRECTION, ValaPointerIndirection))
#define VALA_POINTER_INDIRECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_POINTER_INDIRECTION, ValaPointerIndirectionClass))
#define VALA_IS_POINTER_INDIRECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_POINTER_INDIRECTION))
#define VALA_IS_POINTER_INDIRECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_POINTER_INDIRECTION))
#define VALA_POINTER_INDIRECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_POINTER_INDIRECTION, ValaPointerIndirectionClass))

typedef struct _ValaPointerIndirection ValaPointerIndirection;
typedef struct _ValaPointerIndirectionClass ValaPointerIndirectionClass;

#define VALA_TYPE_ADDRESSOF_EXPRESSION (vala_addressof_expression_get_type ())
#define VALA_ADDRESSOF_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ADDRESSOF_EXPRESSION, ValaAddressofExpression))
#define VALA_ADDRESSOF_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ADDRESSOF_EXPRESSION, ValaAddressofExpressionClass))
#define VALA_IS_ADDRESSOF_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ADDRESSOF_EXPRESSION))
#define VALA_IS_ADDRESSOF_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ADDRESSOF_EXPRESSION))
#define VALA_ADDRESSOF_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ADDRESSOF_EXPRESSION, ValaAddressofExpressionClass))

typedef struct _ValaAddressofExpression ValaAddressofExpression;
typedef struct _ValaAddressofExpressionClass ValaAddressofExpressionClass;

#define VALA_TYPE_REFERENCE_TRANSFER_EXPRESSION (vala_reference_transfer_expression_get_type ())
#define VALA_REFERENCE_TRANSFER_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_REFERENCE_TRANSFER_EXPRESSION, ValaReferenceTransferExpression))
#define VALA_REFERENCE_TRANSFER_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_REFERENCE_TRANSFER_EXPRESSION, ValaReferenceTransferExpressionClass))
#define VALA_IS_REFERENCE_TRANSFER_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_REFERENCE_TRANSFER_EXPRESSION))
#define VALA_IS_REFERENCE_TRANSFER_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_REFERENCE_TRANSFER_EXPRESSION))
#define VALA_REFERENCE_TRANSFER_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_REFERENCE_TRANSFER_EXPRESSION, ValaReferenceTransferExpressionClass))

typedef struct _ValaReferenceTransferExpression ValaReferenceTransferExpression;
typedef struct _ValaReferenceTransferExpressionClass ValaReferenceTransferExpressionClass;

#define VALA_TYPE_BINARY_EXPRESSION (vala_binary_expression_get_type ())
#define VALA_BINARY_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BINARY_EXPRESSION, ValaBinaryExpression))
#define VALA_BINARY_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BINARY_EXPRESSION, ValaBinaryExpressionClass))
#define VALA_IS_BINARY_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BINARY_EXPRESSION))
#define VALA_IS_BINARY_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BINARY_EXPRESSION))
#define VALA_BINARY_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BINARY_EXPRESSION, ValaBinaryExpressionClass))

typedef struct _ValaBinaryExpression ValaBinaryExpression;
typedef struct _ValaBinaryExpressionClass ValaBinaryExpressionClass;

#define VALA_TYPE_TYPECHECK (vala_typecheck_get_type ())
#define VALA_TYPECHECK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPECHECK, ValaTypeCheck))
#define VALA_TYPECHECK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPECHECK, ValaTypeCheckClass))
#define VALA_IS_TYPECHECK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPECHECK))
#define VALA_IS_TYPECHECK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPECHECK))
#define VALA_TYPECHECK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPECHECK, ValaTypeCheckClass))

typedef struct _ValaTypeCheck ValaTypeCheck;
typedef struct _ValaTypeCheckClass ValaTypeCheckClass;

#define VALA_TYPE_CONDITIONAL_EXPRESSION (vala_conditional_expression_get_type ())
#define VALA_CONDITIONAL_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CONDITIONAL_EXPRESSION, ValaConditionalExpression))
#define VALA_CONDITIONAL_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CONDITIONAL_EXPRESSION, ValaConditionalExpressionClass))
#define VALA_IS_CONDITIONAL_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CONDITIONAL_EXPRESSION))
#define VALA_IS_CONDITIONAL_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CONDITIONAL_EXPRESSION))
#define VALA_CONDITIONAL_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CONDITIONAL_EXPRESSION, ValaConditionalExpressionClass))

typedef struct _ValaConditionalExpression ValaConditionalExpression;
typedef struct _ValaConditionalExpressionClass ValaConditionalExpressionClass;

#define VALA_TYPE_LAMBDA_EXPRESSION (vala_lambda_expression_get_type ())
#define VALA_LAMBDA_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_LAMBDA_EXPRESSION, ValaLambdaExpression))
#define VALA_LAMBDA_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_LAMBDA_EXPRESSION, ValaLambdaExpressionClass))
#define VALA_IS_LAMBDA_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_LAMBDA_EXPRESSION))
#define VALA_IS_LAMBDA_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_LAMBDA_EXPRESSION))
#define VALA_LAMBDA_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_LAMBDA_EXPRESSION, ValaLambdaExpressionClass))

typedef struct _ValaLambdaExpression ValaLambdaExpression;
typedef struct _ValaLambdaExpressionClass ValaLambdaExpressionClass;

#define VALA_TYPE_ASSIGNMENT (vala_assignment_get_type ())
#define VALA_ASSIGNMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ASSIGNMENT, ValaAssignment))
#define VALA_ASSIGNMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ASSIGNMENT, ValaAssignmentClass))
#define VALA_IS_ASSIGNMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ASSIGNMENT))
#define VALA_IS_ASSIGNMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ASSIGNMENT))
#define VALA_ASSIGNMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ASSIGNMENT, ValaAssignmentClass))

typedef struct _ValaAssignment ValaAssignment;
typedef struct _ValaAssignmentClass ValaAssignmentClass;

#define VALA_TYPE_FLOW_ANALYZER (vala_flow_analyzer_get_type ())
#define VALA_FLOW_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_FLOW_ANALYZER, ValaFlowAnalyzer))
#define VALA_FLOW_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_FLOW_ANALYZER, ValaFlowAnalyzerClass))
#define VALA_IS_FLOW_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_FLOW_ANALYZER))
#define VALA_IS_FLOW_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_FLOW_ANALYZER))
#define VALA_FLOW_ANALYZER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_FLOW_ANALYZER, ValaFlowAnalyzerClass))

typedef struct _ValaFlowAnalyzer ValaFlowAnalyzer;
typedef struct _ValaFlowAnalyzerClass ValaFlowAnalyzerClass;
typedef struct _ValaFlowAnalyzerPrivate ValaFlowAnalyzerPrivate;

#define VALA_TYPE_CODE_CONTEXT (vala_code_context_get_type ())
#define VALA_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContext))
#define VALA_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))
#define VALA_IS_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_CONTEXT))
#define VALA_IS_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_CONTEXT))
#define VALA_CODE_CONTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))

typedef struct _ValaCodeContext ValaCodeContext;
typedef struct _ValaCodeContextClass ValaCodeContextClass;

#define VALA_TYPE_BASIC_BLOCK (vala_basic_block_get_type ())
#define VALA_BASIC_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BASIC_BLOCK, ValaBasicBlock))
#define VALA_BASIC_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockClass))
#define VALA_IS_BASIC_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BASIC_BLOCK))
#define VALA_IS_BASIC_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BASIC_BLOCK))
#define VALA_BASIC_BLOCK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockClass))

typedef struct _ValaBasicBlock ValaBasicBlock;
typedef struct _ValaBasicBlockClass ValaBasicBlockClass;

#define VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET (vala_flow_analyzer_jump_target_get_type ())
#define VALA_FLOW_ANALYZER_JUMP_TARGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, ValaFlowAnalyzerJumpTarget))
#define VALA_FLOW_ANALYZER_JUMP_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, ValaFlowAnalyzerJumpTargetClass))
#define VALA_FLOW_ANALYZER_IS_JUMP_TARGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET))
#define VALA_FLOW_ANALYZER_IS_JUMP_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET))
#define VALA_FLOW_ANALYZER_JUMP_TARGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, ValaFlowAnalyzerJumpTargetClass))

typedef struct _ValaFlowAnalyzerJumpTarget ValaFlowAnalyzerJumpTarget;
typedef struct _ValaFlowAnalyzerJumpTargetClass ValaFlowAnalyzerJumpTargetClass;

#define VALA_TYPE_PHI_FUNCTION (vala_phi_function_get_type ())
#define VALA_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunction))
#define VALA_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))
#define VALA_IS_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PHI_FUNCTION))
#define VALA_IS_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PHI_FUNCTION))
#define VALA_PHI_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))

typedef struct _ValaPhiFunction ValaPhiFunction;
typedef struct _ValaPhiFunctionClass ValaPhiFunctionClass;
#define _vala_code_context_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_context_unref (var), NULL)))
#define _vala_basic_block_unref0(var) ((var == NULL) ? NULL : (var = (vala_basic_block_unref (var), NULL)))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _vala_source_file_unref0(var) ((var == NULL) ? NULL : (var = (vala_source_file_unref (var), NULL)))

#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))

typedef struct _ValaSourceReference ValaSourceReference;
typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_flow_analyzer_jump_target_unref0(var) ((var == NULL) ? NULL : (var = (vala_flow_analyzer_jump_target_unref (var), NULL)))

#define VALA_TYPE_VOID_TYPE (vala_void_type_get_type ())
#define VALA_VOID_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_VOID_TYPE, ValaVoidType))
#define VALA_VOID_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_VOID_TYPE, ValaVoidTypeClass))
#define VALA_IS_VOID_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_VOID_TYPE))
#define VALA_IS_VOID_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_VOID_TYPE))
#define VALA_VOID_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_VOID_TYPE, ValaVoidTypeClass))

typedef struct _ValaVoidType ValaVoidType;
typedef struct _ValaVoidTypeClass ValaVoidTypeClass;
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _vala_phi_function_unref0(var) ((var == NULL) ? NULL : (var = (vala_phi_function_unref (var), NULL)))

#define VALA_TYPE_ATTRIBUTE (vala_attribute_get_type ())
#define VALA_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ATTRIBUTE, ValaAttribute))
#define VALA_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ATTRIBUTE, ValaAttributeClass))
#define VALA_IS_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ATTRIBUTE))
#define VALA_IS_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ATTRIBUTE))
#define VALA_ATTRIBUTE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ATTRIBUTE, ValaAttributeClass))

typedef struct _ValaAttribute ValaAttribute;
typedef struct _ValaAttributeClass ValaAttributeClass;

#define VALA_TYPE_STATEMENT (vala_statement_get_type ())
#define VALA_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_STATEMENT, ValaStatement))
#define VALA_IS_STATEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_STATEMENT))
#define VALA_STATEMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALA_TYPE_STATEMENT, ValaStatementIface))

typedef struct _ValaStatement ValaStatement;
typedef struct _ValaStatementIface ValaStatementIface;

#define VALA_TYPE_REFERENCE_TYPE (vala_reference_type_get_type ())
#define VALA_REFERENCE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_REFERENCE_TYPE, ValaReferenceType))
#define VALA_REFERENCE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_REFERENCE_TYPE, ValaReferenceTypeClass))
#define VALA_IS_REFERENCE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_REFERENCE_TYPE))
#define VALA_IS_REFERENCE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_REFERENCE_TYPE))
#define VALA_REFERENCE_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_REFERENCE_TYPE, ValaReferenceTypeClass))

typedef struct _ValaReferenceType ValaReferenceType;
typedef struct _ValaReferenceTypeClass ValaReferenceTypeClass;

#define VALA_TYPE_ERROR_TYPE (vala_error_type_get_type ())
#define VALA_ERROR_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ERROR_TYPE, ValaErrorType))
#define VALA_ERROR_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ERROR_TYPE, ValaErrorTypeClass))
#define VALA_IS_ERROR_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ERROR_TYPE))
#define VALA_IS_ERROR_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ERROR_TYPE))
#define VALA_ERROR_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ERROR_TYPE, ValaErrorTypeClass))

typedef struct _ValaErrorType ValaErrorType;
typedef struct _ValaErrorTypeClass ValaErrorTypeClass;
typedef struct _ValaFlowAnalyzerJumpTargetPrivate ValaFlowAnalyzerJumpTargetPrivate;
typedef struct _ValaFlowAnalyzerParamSpecJumpTarget ValaFlowAnalyzerParamSpecJumpTarget;

struct _ValaCodeVisitor {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCodeVisitorPrivate * priv;
};

struct _ValaCodeVisitorClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCodeVisitor *self);
	void (*visit_source_file) (ValaCodeVisitor* self, ValaSourceFile* source_file);
	void (*visit_namespace) (ValaCodeVisitor* self, ValaNamespace* ns);
	void (*visit_class) (ValaCodeVisitor* self, ValaClass* cl);
	void (*visit_struct) (ValaCodeVisitor* self, ValaStruct* st);
	void (*visit_interface) (ValaCodeVisitor* self, ValaInterface* iface);
	void (*visit_enum) (ValaCodeVisitor* self, ValaEnum* en);
	void (*visit_enum_value) (ValaCodeVisitor* self, ValaEnumValue* ev);
	void (*visit_error_domain) (ValaCodeVisitor* self, ValaErrorDomain* edomain);
	void (*visit_error_code) (ValaCodeVisitor* self, ValaErrorCode* ecode);
	void (*visit_delegate) (ValaCodeVisitor* self, ValaDelegate* d);
	void (*visit_member) (ValaCodeVisitor* self, ValaMember* m);
	void (*visit_constant) (ValaCodeVisitor* self, ValaConstant* c);
	void (*visit_field) (ValaCodeVisitor* self, ValaField* f);
	void (*visit_method) (ValaCodeVisitor* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCodeVisitor* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCodeVisitor* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCodeVisitor* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCodeVisitor* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCodeVisitor* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCodeVisitor* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCodeVisitor* self, ValaDestructor* d);
	void (*visit_type_parameter) (ValaCodeVisitor* self, ValaTypeParameter* p);
	void (*visit_using_directive) (ValaCodeVisitor* self, ValaUsingDirective* ns);
	void (*visit_data_type) (ValaCodeVisitor* self, ValaDataType* type);
	void (*visit_block) (ValaCodeVisitor* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCodeVisitor* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCodeVisitor* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCodeVisitor* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCodeVisitor* self, ValaInitializerList* list);
	void (*visit_expression_statement) (ValaCodeVisitor* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCodeVisitor* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCodeVisitor* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCodeVisitor* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCodeVisitor* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCodeVisitor* self, ValaLoop* stmt);
	void (*visit_while_statement) (ValaCodeVisitor* self, ValaWhileStatement* stmt);
	void (*visit_do_statement) (ValaCodeVisitor* self, ValaDoStatement* stmt);
	void (*visit_for_statement) (ValaCodeVisitor* self, ValaForStatement* stmt);
	void (*visit_foreach_statement) (ValaCodeVisitor* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCodeVisitor* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCodeVisitor* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCodeVisitor* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCodeVisitor* self, ValaYieldStatement* y);
	void (*visit_throw_statement) (ValaCodeVisitor* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCodeVisitor* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCodeVisitor* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCodeVisitor* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCodeVisitor* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCodeVisitor* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCodeVisitor* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCodeVisitor* self, ValaBooleanLiteral* lit);
	void (*visit_character_literal) (ValaCodeVisitor* self, ValaCharacterLiteral* lit);
	void (*visit_integer_literal) (ValaCodeVisitor* self, ValaIntegerLiteral* lit);
	void (*visit_real_literal) (ValaCodeVisitor* self, ValaRealLiteral* lit);
	void (*visit_string_literal) (ValaCodeVisitor* self, ValaStringLiteral* lit);
	void (*visit_null_literal) (ValaCodeVisitor* self, ValaNullLiteral* lit);
	void (*visit_member_access) (ValaCodeVisitor* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCodeVisitor* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCodeVisitor* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCodeVisitor* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCodeVisitor* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCodeVisitor* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCodeVisitor* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCodeVisitor* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCodeVisitor* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCodeVisitor* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCodeVisitor* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCodeVisitor* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCodeVisitor* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCodeVisitor* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCodeVisitor* self, ValaTypeCheck* expr);
	void (*visit_conditional_expression) (ValaCodeVisitor* self, ValaConditionalExpression* expr);
	void (*visit_lambda_expression) (ValaCodeVisitor* self, ValaLambdaExpression* expr);
	void (*visit_assignment) (ValaCodeVisitor* self, ValaAssignment* a);
	void (*visit_end_full_expression) (ValaCodeVisitor* self, ValaExpression* expr);
};

struct _ValaFlowAnalyzer {
	ValaCodeVisitor parent_instance;
	ValaFlowAnalyzerPrivate * priv;
};

struct _ValaFlowAnalyzerClass {
	ValaCodeVisitorClass parent_class;
};

struct _ValaFlowAnalyzerPrivate {
	ValaCodeContext* context;
	ValaBasicBlock* current_block;
	gboolean unreachable_reported;
	GeeList* jump_stack;
	GeeMap* var_map;
	GeeSet* used_vars;
	GeeMap* phi_functions;
};

struct _ValaStatementIface {
	GTypeInterface parent_iface;
};

struct _ValaFlowAnalyzerJumpTarget {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaFlowAnalyzerJumpTargetPrivate * priv;
};

struct _ValaFlowAnalyzerJumpTargetClass {
	GTypeClass parent_class;
	void (*finalize) (ValaFlowAnalyzerJumpTarget *self);
};

struct _ValaFlowAnalyzerJumpTargetPrivate {
	gboolean _is_break_target;
	gboolean _is_continue_target;
	gboolean _is_return_target;
	gboolean _is_error_target;
	ValaErrorDomain* _error_domain;
	ValaErrorCode* _error_code;
	gboolean _is_finally_clause;
	ValaBasicBlock* _basic_block;
	ValaBasicBlock* _last_block;
	ValaCatchClause* _catch_clause;
};

struct _ValaFlowAnalyzerParamSpecJumpTarget {
	GParamSpec parent_instance;
};


static gpointer vala_flow_analyzer_jump_target_parent_class = NULL;
static gpointer vala_flow_analyzer_parent_class = NULL;

gpointer vala_code_visitor_ref (gpointer instance);
void vala_code_visitor_unref (gpointer instance);
GParamSpec* vala_param_spec_code_visitor (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_code_visitor (GValue* value, gpointer v_object);
gpointer vala_value_get_code_visitor (const GValue* value);
GType vala_code_visitor_get_type (void);
gpointer vala_source_file_ref (gpointer instance);
void vala_source_file_unref (gpointer instance);
GParamSpec* vala_param_spec_source_file (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_source_file (GValue* value, gpointer v_object);
gpointer vala_value_get_source_file (const GValue* value);
GType vala_source_file_get_type (void);
gpointer vala_code_node_ref (gpointer instance);
void vala_code_node_unref (gpointer instance);
GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_code_node (GValue* value, gpointer v_object);
gpointer vala_value_get_code_node (const GValue* value);
GType vala_code_node_get_type (void);
GType vala_symbol_get_type (void);
GType vala_namespace_get_type (void);
GType vala_typesymbol_get_type (void);
GType vala_object_type_symbol_get_type (void);
GType vala_class_get_type (void);
GType vala_struct_get_type (void);
GType vala_interface_get_type (void);
GType vala_enum_get_type (void);
GType vala_enum_value_get_type (void);
GType vala_error_domain_get_type (void);
GType vala_error_code_get_type (void);
GType vala_delegate_get_type (void);
GType vala_member_get_type (void);
GType vala_constant_get_type (void);
GType vala_field_get_type (void);
GType vala_method_get_type (void);
GType vala_creation_method_get_type (void);
GType vala_formal_parameter_get_type (void);
GType vala_property_get_type (void);
GType vala_property_accessor_get_type (void);
GType vala_signal_get_type (void);
GType vala_constructor_get_type (void);
GType vala_destructor_get_type (void);
GType vala_typeparameter_get_type (void);
GType vala_using_directive_get_type (void);
GType vala_data_type_get_type (void);
GType vala_block_get_type (void);
GType vala_empty_statement_get_type (void);
GType vala_declaration_statement_get_type (void);
GType vala_local_variable_get_type (void);
GType vala_expression_get_type (void);
GType vala_initializer_list_get_type (void);
GType vala_expression_statement_get_type (void);
GType vala_if_statement_get_type (void);
GType vala_switch_statement_get_type (void);
GType vala_switch_section_get_type (void);
GType vala_switch_label_get_type (void);
GType vala_loop_get_type (void);
GType vala_while_statement_get_type (void);
GType vala_do_statement_get_type (void);
GType vala_for_statement_get_type (void);
GType vala_foreach_statement_get_type (void);
GType vala_break_statement_get_type (void);
GType vala_continue_statement_get_type (void);
GType vala_return_statement_get_type (void);
GType vala_yield_statement_get_type (void);
GType vala_throw_statement_get_type (void);
GType vala_try_statement_get_type (void);
GType vala_catch_clause_get_type (void);
GType vala_lock_statement_get_type (void);
GType vala_delete_statement_get_type (void);
GType vala_array_creation_expression_get_type (void);
GType vala_literal_get_type (void);
GType vala_boolean_literal_get_type (void);
GType vala_character_literal_get_type (void);
GType vala_integer_literal_get_type (void);
GType vala_real_literal_get_type (void);
GType vala_string_literal_get_type (void);
GType vala_null_literal_get_type (void);
GType vala_member_access_get_type (void);
GType vala_method_call_get_type (void);
GType vala_element_access_get_type (void);
GType vala_base_access_get_type (void);
GType vala_postfix_expression_get_type (void);
GType vala_object_creation_expression_get_type (void);
GType vala_sizeof_expression_get_type (void);
GType vala_typeof_expression_get_type (void);
GType vala_unary_expression_get_type (void);
GType vala_cast_expression_get_type (void);
GType vala_pointer_indirection_get_type (void);
GType vala_addressof_expression_get_type (void);
GType vala_reference_transfer_expression_get_type (void);
GType vala_binary_expression_get_type (void);
GType vala_typecheck_get_type (void);
GType vala_conditional_expression_get_type (void);
GType vala_lambda_expression_get_type (void);
GType vala_assignment_get_type (void);
GType vala_flow_analyzer_get_type (void);
gpointer vala_code_context_ref (gpointer instance);
void vala_code_context_unref (gpointer instance);
GParamSpec* vala_param_spec_code_context (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_code_context (GValue* value, gpointer v_object);
gpointer vala_value_get_code_context (const GValue* value);
GType vala_code_context_get_type (void);
gpointer vala_basic_block_ref (gpointer instance);
void vala_basic_block_unref (gpointer instance);
GParamSpec* vala_param_spec_basic_block (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_basic_block (GValue* value, gpointer v_object);
gpointer vala_value_get_basic_block (const GValue* value);
GType vala_basic_block_get_type (void);
static gpointer vala_flow_analyzer_jump_target_ref (gpointer instance);
static void vala_flow_analyzer_jump_target_unref (gpointer instance);
static GParamSpec* vala_flow_analyzer_param_spec_jump_target (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
static void vala_flow_analyzer_value_set_jump_target (GValue* value, gpointer v_object);
static gpointer vala_flow_analyzer_value_get_jump_target (const GValue* value);
static GType vala_flow_analyzer_jump_target_get_type (void);
gpointer vala_phi_function_ref (gpointer instance);
void vala_phi_function_unref (gpointer instance);
GParamSpec* vala_param_spec_phi_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_phi_function (GValue* value, gpointer v_object);
gpointer vala_value_get_phi_function (const GValue* value);
GType vala_phi_function_get_type (void);
#define VALA_FLOW_ANALYZER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_FLOW_ANALYZER, ValaFlowAnalyzerPrivate))
enum  {
	VALA_FLOW_ANALYZER_DUMMY_PROPERTY
};
ValaCodeVisitor* vala_code_visitor_construct (GType object_type);
ValaFlowAnalyzer* vala_flow_analyzer_new (void);
ValaFlowAnalyzer* vala_flow_analyzer_construct (GType object_type);
GeeList* vala_code_context_get_source_files (ValaCodeContext* self);
gboolean vala_source_file_get_external_package (ValaSourceFile* self);
void vala_source_file_accept (ValaSourceFile* self, ValaCodeVisitor* visitor);
void vala_flow_analyzer_analyze (ValaFlowAnalyzer* self, ValaCodeContext* context);
void vala_source_file_accept_children (ValaSourceFile* self, ValaCodeVisitor* visitor);
static void vala_flow_analyzer_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
void vala_code_node_accept_children (ValaCodeNode* self, ValaCodeVisitor* visitor);
static void vala_flow_analyzer_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_flow_analyzer_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_flow_analyzer_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_flow_analyzer_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_flow_analyzer_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed);
gboolean vala_symbol_is_internal_symbol (ValaSymbol* self);
gboolean vala_symbol_get_used (ValaSymbol* self);
gpointer vala_source_reference_ref (gpointer instance);
void vala_source_reference_unref (gpointer instance);
GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_source_reference (GValue* value, gpointer v_object);
gpointer vala_value_get_source_reference (const GValue* value);
GType vala_source_reference_get_type (void);
void vala_report_warning (ValaSourceReference* source, const char* message);
ValaSourceReference* vala_code_node_get_source_reference (ValaCodeNode* self);
char* vala_symbol_get_full_name (ValaSymbol* self);
static void vala_flow_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f);
gboolean vala_method_get_entry_point (ValaMethod* self);
gboolean vala_method_get_overrides (ValaMethod* self);
ValaMethod* vala_method_get_base_interface_method (ValaMethod* self);
ValaBlock* vala_method_get_body (ValaMethod* self);
ValaBasicBlock* vala_basic_block_new_entry (void);
ValaBasicBlock* vala_basic_block_construct_entry (GType object_type);
void vala_method_set_entry_block (ValaMethod* self, ValaBasicBlock* value);
ValaBasicBlock* vala_basic_block_new_exit (void);
ValaBasicBlock* vala_basic_block_construct_exit (GType object_type);
void vala_method_set_exit_block (ValaMethod* self, ValaBasicBlock* value);
ValaBasicBlock* vala_basic_block_new (void);
ValaBasicBlock* vala_basic_block_construct (GType object_type);
ValaBasicBlock* vala_method_get_entry_block (ValaMethod* self);
void vala_basic_block_connect (ValaBasicBlock* self, ValaBasicBlock* target);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_return_target (ValaBasicBlock* basic_block);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_return_target (GType object_type, ValaBasicBlock* basic_block);
ValaBasicBlock* vala_method_get_exit_block (ValaMethod* self);
ValaDataType* vala_method_get_return_type (ValaMethod* self);
GType vala_void_type_get_type (void);
void vala_report_error (ValaSourceReference* source, const char* message);
void vala_code_node_set_error (ValaCodeNode* self, gboolean value);
static void vala_flow_analyzer_build_dominator_tree (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block);
static void vala_flow_analyzer_build_dominator_frontier (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block);
static void vala_flow_analyzer_insert_phi_functions (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block);
static void vala_flow_analyzer_check_variables (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block);
static void vala_flow_analyzer_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_flow_analyzer_depth_first_traverse (ValaFlowAnalyzer* self, ValaBasicBlock* current, GeeList* list);
static GeeList* vala_flow_analyzer_get_depth_first_list (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block);
GeeList* vala_basic_block_get_successors (ValaBasicBlock* self);
GeeList* vala_basic_block_get_predecessors (ValaBasicBlock* self);
void vala_basic_block_add_child (ValaBasicBlock* self, ValaBasicBlock* block);
ValaBasicBlock* vala_basic_block_get_parent (ValaBasicBlock* self);
void vala_basic_block_add_dominator_frontier (ValaBasicBlock* self, ValaBasicBlock* block);
GeeList* vala_basic_block_get_children (ValaBasicBlock* self);
GeeSet* vala_basic_block_get_dominator_frontier (ValaBasicBlock* self);
GeeList* vala_basic_block_get_nodes (ValaBasicBlock* self);
void vala_code_node_get_defined_variables (ValaCodeNode* self, GeeCollection* collection);
static GeeMap* vala_flow_analyzer_get_assignment_map (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block);
void vala_basic_block_add_phi_function (ValaBasicBlock* self, ValaPhiFunction* phi);
ValaPhiFunction* vala_phi_function_new (ValaLocalVariable* variable, gint num_of_ops);
ValaPhiFunction* vala_phi_function_construct (GType object_type, ValaLocalVariable* variable, gint num_of_ops);
static void vala_flow_analyzer_check_block_variables (ValaFlowAnalyzer* self, ValaBasicBlock* block);
GeeList* vala_phi_function_get_operands (ValaPhiFunction* self);
const char* vala_symbol_get_name (ValaSymbol* self);
void vala_code_node_set_source_reference (ValaCodeNode* self, ValaSourceReference* value);
GeeSet* vala_basic_block_get_phi_functions (ValaBasicBlock* self);
static ValaLocalVariable* vala_flow_analyzer_process_assignment (ValaFlowAnalyzer* self, GeeMap* var_map, ValaLocalVariable* var_symbol);
ValaLocalVariable* vala_phi_function_get_original_variable (ValaPhiFunction* self);
void vala_code_node_get_used_variables (ValaCodeNode* self, GeeCollection* collection);
ValaDataType* vala_local_variable_get_variable_type (ValaLocalVariable* self);
ValaLocalVariable* vala_local_variable_new (ValaDataType* variable_type, const char* name, ValaExpression* initializer, ValaSourceReference* source_reference);
ValaLocalVariable* vala_local_variable_construct (GType object_type, ValaDataType* variable_type, const char* name, ValaExpression* initializer, ValaSourceReference* source_reference);
void vala_code_visitor_visit_method (ValaCodeVisitor* self, ValaMethod* m);
static void vala_flow_analyzer_real_visit_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
static void vala_flow_analyzer_real_visit_property (ValaCodeVisitor* base, ValaProperty* prop);
ValaBlock* vala_property_accessor_get_body (ValaPropertyAccessor* self);
void vala_property_accessor_set_entry_block (ValaPropertyAccessor* self, ValaBasicBlock* value);
void vala_property_accessor_set_exit_block (ValaPropertyAccessor* self, ValaBasicBlock* value);
ValaBasicBlock* vala_property_accessor_get_entry_block (ValaPropertyAccessor* self);
ValaBasicBlock* vala_property_accessor_get_exit_block (ValaPropertyAccessor* self);
gboolean vala_property_accessor_get_readable (ValaPropertyAccessor* self);
static void vala_flow_analyzer_real_visit_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_flow_analyzer_real_visit_block (ValaCodeVisitor* base, ValaBlock* b);
static gboolean vala_flow_analyzer_unreachable (ValaFlowAnalyzer* self, ValaCodeNode* node);
ValaSymbol* vala_declaration_statement_get_declaration (ValaDeclarationStatement* self);
void vala_basic_block_add_node (ValaBasicBlock* self, ValaCodeNode* node);
ValaExpression* vala_local_variable_get_initializer (ValaLocalVariable* self);
static void vala_flow_analyzer_handle_errors (ValaFlowAnalyzer* self, ValaCodeNode* node, gboolean always_fail);
static void vala_flow_analyzer_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt);
ValaExpression* vala_expression_statement_get_expression (ValaExpressionStatement* self);
ValaExpression* vala_method_call_get_call (ValaMethodCall* self);
ValaSymbol* vala_expression_get_symbol_reference (ValaExpression* self);
GType vala_attribute_get_type (void);
ValaAttribute* vala_code_node_get_attribute (ValaCodeNode* self, const char* name);
static void vala_flow_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
gboolean vala_boolean_literal_get_value (ValaBooleanLiteral* self);
static gboolean vala_flow_analyzer_always_true (ValaFlowAnalyzer* self, ValaExpression* condition);
static gboolean vala_flow_analyzer_always_false (ValaFlowAnalyzer* self, ValaExpression* condition);
ValaExpression* vala_if_statement_get_condition (ValaIfStatement* self);
ValaBlock* vala_if_statement_get_true_statement (ValaIfStatement* self);
void vala_code_node_accept (ValaCodeNode* self, ValaCodeVisitor* visitor);
ValaBlock* vala_if_statement_get_false_statement (ValaIfStatement* self);
static void vala_flow_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_break_target (ValaBasicBlock* basic_block);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_break_target (GType object_type, ValaBasicBlock* basic_block);
ValaExpression* vala_switch_statement_get_expression (ValaSwitchStatement* self);
GeeList* vala_switch_statement_get_sections (ValaSwitchStatement* self);
GType vala_statement_get_type (void);
GeeList* vala_block_get_statements (ValaBlock* self);
gboolean vala_switch_section_has_default_label (ValaSwitchSection* self);
static void vala_flow_analyzer_real_visit_switch_statement (ValaCodeVisitor* base, ValaSwitchStatement* stmt);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_continue_target (ValaBasicBlock* basic_block);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_continue_target (GType object_type, ValaBasicBlock* basic_block);
ValaBlock* vala_loop_get_body (ValaLoop* self);
static void vala_flow_analyzer_real_visit_loop (ValaCodeVisitor* base, ValaLoop* stmt);
ValaExpression* vala_foreach_statement_get_collection (ValaForeachStatement* self);
ValaBlock* vala_foreach_statement_get_body (ValaForeachStatement* self);
static void vala_flow_analyzer_real_visit_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static gboolean vala_flow_analyzer_jump_target_get_is_break_target (ValaFlowAnalyzerJumpTarget* self);
static ValaBasicBlock* vala_flow_analyzer_jump_target_get_basic_block (ValaFlowAnalyzerJumpTarget* self);
static gboolean vala_flow_analyzer_jump_target_get_is_finally_clause (ValaFlowAnalyzerJumpTarget* self);
static ValaBasicBlock* vala_flow_analyzer_jump_target_get_last_block (ValaFlowAnalyzerJumpTarget* self);
static void vala_flow_analyzer_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt);
static gboolean vala_flow_analyzer_jump_target_get_is_continue_target (ValaFlowAnalyzerJumpTarget* self);
static void vala_flow_analyzer_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt);
ValaExpression* vala_return_statement_get_return_expression (ValaReturnStatement* self);
static gboolean vala_flow_analyzer_jump_target_get_is_return_target (ValaFlowAnalyzerJumpTarget* self);
static void vala_flow_analyzer_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
gboolean vala_code_node_get_tree_can_fail (ValaCodeNode* self);
GeeList* vala_code_node_get_error_types (ValaCodeNode* self);
GType vala_reference_type_get_type (void);
GType vala_error_type_get_type (void);
static gboolean vala_flow_analyzer_jump_target_get_is_error_target (ValaFlowAnalyzerJumpTarget* self);
static ValaErrorDomain* vala_flow_analyzer_jump_target_get_error_domain (ValaFlowAnalyzerJumpTarget* self);
ValaErrorDomain* vala_error_type_get_error_domain (ValaErrorType* self);
static ValaErrorCode* vala_flow_analyzer_jump_target_get_error_code (ValaFlowAnalyzerJumpTarget* self);
ValaErrorCode* vala_error_type_get_error_code (ValaErrorType* self);
static void vala_flow_analyzer_real_visit_yield_statement (ValaCodeVisitor* base, ValaYieldStatement* stmt);
static void vala_flow_analyzer_real_visit_throw_statement (ValaCodeVisitor* base, ValaThrowStatement* stmt);
ValaBlock* vala_try_statement_get_finally_body (ValaTryStatement* self);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_any_target (ValaBasicBlock* basic_block);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_any_target (GType object_type, ValaBasicBlock* basic_block);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_finally_clause (ValaBasicBlock* basic_block, ValaBasicBlock* last_block);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_finally_clause (GType object_type, ValaBasicBlock* basic_block, ValaBasicBlock* last_block);
GeeList* vala_try_statement_get_catch_clauses (ValaTryStatement* self);
ValaDataType* vala_catch_clause_get_error_type (ValaCatchClause* self);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_error_target (ValaBasicBlock* basic_block, ValaCatchClause* catch_clause, ValaErrorDomain* error_domain, ValaErrorCode* error_code);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_error_target (GType object_type, ValaBasicBlock* basic_block, ValaCatchClause* catch_clause, ValaErrorDomain* error_domain, ValaErrorCode* error_code);
ValaTypeSymbol* vala_data_type_get_data_type (ValaDataType* self);
ValaBlock* vala_try_statement_get_body (ValaTryStatement* self);
static ValaCatchClause* vala_flow_analyzer_jump_target_get_catch_clause (ValaFlowAnalyzerJumpTarget* self);
ValaBlock* vala_catch_clause_get_body (ValaCatchClause* self);
static void vala_flow_analyzer_real_visit_try_statement (ValaCodeVisitor* base, ValaTryStatement* stmt);
ValaBlock* vala_lock_statement_get_body (ValaLockStatement* self);
static void vala_flow_analyzer_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt);
#define VALA_FLOW_ANALYZER_JUMP_TARGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, ValaFlowAnalyzerJumpTargetPrivate))
enum  {
	VALA_FLOW_ANALYZER_JUMP_TARGET_DUMMY_PROPERTY
};
static void vala_flow_analyzer_jump_target_set_basic_block (ValaFlowAnalyzerJumpTarget* self, ValaBasicBlock* value);
static void vala_flow_analyzer_jump_target_set_is_break_target (ValaFlowAnalyzerJumpTarget* self, gboolean value);
static void vala_flow_analyzer_jump_target_set_is_continue_target (ValaFlowAnalyzerJumpTarget* self, gboolean value);
static void vala_flow_analyzer_jump_target_set_is_return_target (ValaFlowAnalyzerJumpTarget* self, gboolean value);
static void vala_flow_analyzer_jump_target_set_catch_clause (ValaFlowAnalyzerJumpTarget* self, ValaCatchClause* value);
static void vala_flow_analyzer_jump_target_set_error_domain (ValaFlowAnalyzerJumpTarget* self, ValaErrorDomain* value);
static void vala_flow_analyzer_jump_target_set_error_code (ValaFlowAnalyzerJumpTarget* self, ValaErrorCode* value);
static void vala_flow_analyzer_jump_target_set_is_error_target (ValaFlowAnalyzerJumpTarget* self, gboolean value);
static void vala_flow_analyzer_jump_target_set_last_block (ValaFlowAnalyzerJumpTarget* self, ValaBasicBlock* value);
static void vala_flow_analyzer_jump_target_set_is_finally_clause (ValaFlowAnalyzerJumpTarget* self, gboolean value);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new (void);
static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct (GType object_type);
static void vala_flow_analyzer_jump_target_finalize (ValaFlowAnalyzerJumpTarget* obj);
static void vala_flow_analyzer_finalize (ValaCodeVisitor* obj);



ValaFlowAnalyzer* vala_flow_analyzer_construct (GType object_type) {
	ValaFlowAnalyzer* self;
	self = (ValaFlowAnalyzer*) vala_code_visitor_construct (object_type);
	return self;
}


ValaFlowAnalyzer* vala_flow_analyzer_new (void) {
	return vala_flow_analyzer_construct (VALA_TYPE_FLOW_ANALYZER);
}


static gpointer _vala_code_context_ref0 (gpointer self) {
	return self ? vala_code_context_ref (self) : NULL;
}


void vala_flow_analyzer_analyze (ValaFlowAnalyzer* self, ValaCodeContext* context) {
	ValaCodeContext* _tmp0_;
	GeeList* source_files;
	g_return_if_fail (self != NULL);
	g_return_if_fail (context != NULL);
	self->priv->context = (_tmp0_ = _vala_code_context_ref0 (context), _vala_code_context_unref0 (self->priv->context), _tmp0_);
	source_files = vala_code_context_get_source_files (context);
	{
		GeeIterator* _file_it;
		_file_it = gee_iterable_iterator ((GeeIterable*) source_files);
		while (TRUE) {
			ValaSourceFile* file;
			if (!gee_iterator_next (_file_it)) {
				break;
			}
			file = (ValaSourceFile*) gee_iterator_get (_file_it);
			if (!vala_source_file_get_external_package (file)) {
				vala_source_file_accept (file, (ValaCodeVisitor*) self);
			}
			_vala_source_file_unref0 (file);
		}
		_gee_collection_object_unref0 (_file_it);
	}
	_gee_collection_object_unref0 (source_files);
}


static void vala_flow_analyzer_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (source_file != NULL);
	vala_source_file_accept_children (source_file, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (cl != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (st != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (iface != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) iface, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (en != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (ed != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) ed, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f) {
	ValaFlowAnalyzer * self;
	gboolean _tmp0_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (f != NULL);
	_tmp0_ = FALSE;
	if (vala_symbol_is_internal_symbol ((ValaSymbol*) f)) {
		_tmp0_ = !vala_symbol_get_used ((ValaSymbol*) f);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		char* _tmp2_;
		char* _tmp1_;
		vala_report_warning (vala_code_node_get_source_reference ((ValaCodeNode*) f), _tmp2_ = g_strdup_printf ("field `%s' never used", _tmp1_ = vala_symbol_get_full_name ((ValaSymbol*) f)));
		_g_free0 (_tmp2_);
		_g_free0 (_tmp1_);
	}
}


static void vala_flow_analyzer_real_visit_method (ValaCodeVisitor* base, ValaMethod* m) {
	ValaFlowAnalyzer * self;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	ValaBasicBlock* _tmp8_;
	ValaBasicBlock* _tmp9_;
	ValaBasicBlock* _tmp10_;
	ValaFlowAnalyzerJumpTarget* _tmp11_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (m != NULL);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	if (vala_symbol_is_internal_symbol ((ValaSymbol*) m)) {
		_tmp4_ = !vala_symbol_get_used ((ValaSymbol*) m);
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		_tmp3_ = !vala_method_get_entry_point (m);
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		_tmp2_ = !vala_method_get_overrides (m);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		gboolean _tmp5_;
		_tmp5_ = FALSE;
		if (vala_method_get_base_interface_method (m) == NULL) {
			_tmp5_ = TRUE;
		} else {
			_tmp5_ = vala_method_get_base_interface_method (m) == m;
		}
		_tmp1_ = _tmp5_;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = !VALA_IS_CREATION_METHOD (m);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		char* _tmp7_;
		char* _tmp6_;
		vala_report_warning (vala_code_node_get_source_reference ((ValaCodeNode*) m), _tmp7_ = g_strdup_printf ("method `%s' never used", _tmp6_ = vala_symbol_get_full_name ((ValaSymbol*) m)));
		_g_free0 (_tmp7_);
		_g_free0 (_tmp6_);
	}
	if (vala_method_get_body (m) == NULL) {
		return;
	}
	vala_method_set_entry_block (m, _tmp8_ = vala_basic_block_new_entry ());
	_vala_basic_block_unref0 (_tmp8_);
	vala_method_set_exit_block (m, _tmp9_ = vala_basic_block_new_exit ());
	_vala_basic_block_unref0 (_tmp9_);
	self->priv->current_block = (_tmp10_ = vala_basic_block_new (), _vala_basic_block_unref0 (self->priv->current_block), _tmp10_);
	vala_basic_block_connect (vala_method_get_entry_block (m), self->priv->current_block);
	gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp11_ = vala_flow_analyzer_jump_target_new_return_target (vala_method_get_exit_block (m)));
	_vala_flow_analyzer_jump_target_unref0 (_tmp11_);
	vala_code_node_accept_children ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	if (self->priv->current_block != NULL) {
		if (!VALA_IS_VOID_TYPE (vala_method_get_return_type (m))) {
			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) m), "missing return statement at end of method body");
			vala_code_node_set_error ((ValaCodeNode*) m, TRUE);
		}
		vala_basic_block_connect (self->priv->current_block, vala_method_get_exit_block (m));
	}
	vala_flow_analyzer_build_dominator_tree (self, vala_method_get_entry_block (m));
	vala_flow_analyzer_build_dominator_frontier (self, vala_method_get_entry_block (m));
	vala_flow_analyzer_insert_phi_functions (self, vala_method_get_entry_block (m));
	vala_flow_analyzer_check_variables (self, vala_method_get_entry_block (m));
}


static GeeList* vala_flow_analyzer_get_depth_first_list (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block) {
	GeeList* result;
	GeeArrayList* list;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (entry_block != NULL, NULL);
	list = gee_array_list_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_equal);
	vala_flow_analyzer_depth_first_traverse (self, entry_block, (GeeList*) list);
	result = (GeeList*) list;
	return result;
}


static gpointer _vala_basic_block_ref0 (gpointer self) {
	return self ? vala_basic_block_ref (self) : NULL;
}


static void vala_flow_analyzer_depth_first_traverse (ValaFlowAnalyzer* self, ValaBasicBlock* current, GeeList* list) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (current != NULL);
	g_return_if_fail (list != NULL);
	if (gee_collection_contains ((GeeCollection*) list, current)) {
		return;
	}
	gee_collection_add ((GeeCollection*) list, current);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _succ_it;
		_succ_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_basic_block_get_successors (current))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaBasicBlock* succ;
			if (!gee_iterator_next (_succ_it)) {
				break;
			}
			succ = _vala_basic_block_ref0 ((ValaBasicBlock*) gee_iterator_get (_succ_it));
			vala_flow_analyzer_depth_first_traverse (self, succ, list);
			_vala_basic_block_unref0 (succ);
		}
		_gee_collection_object_unref0 (_succ_it);
	}
}


static void vala_flow_analyzer_build_dominator_tree (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block) {
	GeeHashMap* dom;
	GeeList* block_list;
	GeeHashSet* entry_dom_set;
	gboolean changes;
	g_return_if_fail (self != NULL);
	g_return_if_fail (entry_block != NULL);
	dom = gee_hash_map_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, GEE_TYPE_SET, (GBoxedCopyFunc) gee_collection_object_ref, gee_collection_object_unref, g_direct_hash, g_direct_equal, g_direct_equal);
	block_list = vala_flow_analyzer_get_depth_first_list (self, entry_block);
	{
		GeeIterator* _block_it;
		_block_it = gee_iterable_iterator ((GeeIterable*) block_list);
		while (TRUE) {
			ValaBasicBlock* block;
			GeeHashSet* block_set;
			if (!gee_iterator_next (_block_it)) {
				break;
			}
			block = (ValaBasicBlock*) gee_iterator_get (_block_it);
			block_set = gee_hash_set_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_hash, g_direct_equal);
			{
				GeeIterator* _b_it;
				_b_it = gee_iterable_iterator ((GeeIterable*) block_list);
				while (TRUE) {
					ValaBasicBlock* b;
					if (!gee_iterator_next (_b_it)) {
						break;
					}
					b = (ValaBasicBlock*) gee_iterator_get (_b_it);
					gee_collection_add ((GeeCollection*) block_set, b);
					_vala_basic_block_unref0 (b);
				}
				_gee_collection_object_unref0 (_b_it);
			}
			gee_map_set ((GeeMap*) dom, block, (GeeSet*) block_set);
			_vala_basic_block_unref0 (block);
			_gee_collection_object_unref0 (block_set);
		}
		_gee_collection_object_unref0 (_block_it);
	}
	entry_dom_set = gee_hash_set_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_hash, g_direct_equal);
	gee_collection_add ((GeeCollection*) entry_dom_set, entry_block);
	gee_map_set ((GeeMap*) dom, entry_block, (GeeSet*) entry_dom_set);
	changes = TRUE;
	while (TRUE) {
		if (!changes) {
			break;
		}
		changes = FALSE;
		{
			GeeIterator* _block_it;
			_block_it = gee_iterable_iterator ((GeeIterable*) block_list);
			while (TRUE) {
				ValaBasicBlock* block;
				GeeHashSet* dom_set;
				gboolean first;
				GeeSet* _tmp2_;
				gboolean _tmp3_;
				if (!gee_iterator_next (_block_it)) {
					break;
				}
				block = (ValaBasicBlock*) gee_iterator_get (_block_it);
				dom_set = gee_hash_set_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_hash, g_direct_equal);
				first = TRUE;
				{
					GeeList* _tmp0_;
					GeeIterator* _tmp1_;
					GeeIterator* _pred_it;
					_pred_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_basic_block_get_predecessors (block))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
					while (TRUE) {
						ValaBasicBlock* pred;
						GeeSet* pred_dom_set;
						if (!gee_iterator_next (_pred_it)) {
							break;
						}
						pred = _vala_basic_block_ref0 ((ValaBasicBlock*) gee_iterator_get (_pred_it));
						pred_dom_set = (GeeSet*) gee_map_get ((GeeMap*) dom, pred);
						if (first) {
							{
								GeeIterator* _dom_block_it;
								_dom_block_it = gee_iterable_iterator ((GeeIterable*) pred_dom_set);
								while (TRUE) {
									ValaBasicBlock* dom_block;
									if (!gee_iterator_next (_dom_block_it)) {
										break;
									}
									dom_block = (ValaBasicBlock*) gee_iterator_get (_dom_block_it);
									gee_collection_add ((GeeCollection*) dom_set, dom_block);
									_vala_basic_block_unref0 (dom_block);
								}
								_gee_collection_object_unref0 (_dom_block_it);
							}
							first = FALSE;
						} else {
							GeeArrayList* remove_queue;
							remove_queue = gee_array_list_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_equal);
							{
								GeeIterator* _dom_block_it;
								_dom_block_it = gee_iterable_iterator ((GeeIterable*) dom_set);
								while (TRUE) {
									ValaBasicBlock* dom_block;
									if (!gee_iterator_next (_dom_block_it)) {
										break;
									}
									dom_block = (ValaBasicBlock*) gee_iterator_get (_dom_block_it);
									if (!gee_collection_contains ((GeeCollection*) pred_dom_set, dom_block)) {
										gee_collection_add ((GeeCollection*) remove_queue, dom_block);
									}
									_vala_basic_block_unref0 (dom_block);
								}
								_gee_collection_object_unref0 (_dom_block_it);
							}
							{
								GeeIterator* _dom_block_it;
								_dom_block_it = gee_iterable_iterator ((GeeIterable*) remove_queue);
								while (TRUE) {
									ValaBasicBlock* dom_block;
									if (!gee_iterator_next (_dom_block_it)) {
										break;
									}
									dom_block = (ValaBasicBlock*) gee_iterator_get (_dom_block_it);
									gee_collection_remove ((GeeCollection*) dom_set, dom_block);
									_vala_basic_block_unref0 (dom_block);
								}
								_gee_collection_object_unref0 (_dom_block_it);
							}
							_gee_collection_object_unref0 (remove_queue);
						}
						_vala_basic_block_unref0 (pred);
						_gee_collection_object_unref0 (pred_dom_set);
					}
					_gee_collection_object_unref0 (_pred_it);
				}
				gee_collection_add ((GeeCollection*) dom_set, block);
				if ((_tmp3_ = gee_collection_get_size ((GeeCollection*) (_tmp2_ = (GeeSet*) gee_map_get ((GeeMap*) dom, block))) != gee_collection_get_size ((GeeCollection*) dom_set), _gee_collection_object_unref0 (_tmp2_), _tmp3_)) {
					changes = TRUE;
				} else {
					{
						GeeSet* _tmp4_;
						GeeIterator* _tmp5_;
						GeeIterator* _dom_block_it;
						_dom_block_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = (GeeSet*) gee_map_get ((GeeMap*) dom, block))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
						while (TRUE) {
							ValaBasicBlock* dom_block;
							if (!gee_iterator_next (_dom_block_it)) {
								break;
							}
							dom_block = (ValaBasicBlock*) gee_iterator_get (_dom_block_it);
							if (!gee_collection_contains ((GeeCollection*) dom_set, dom_block)) {
								changes = TRUE;
							}
							_vala_basic_block_unref0 (dom_block);
						}
						_gee_collection_object_unref0 (_dom_block_it);
					}
				}
				gee_map_set ((GeeMap*) dom, block, (GeeSet*) dom_set);
				_vala_basic_block_unref0 (block);
				_gee_collection_object_unref0 (dom_set);
			}
			_gee_collection_object_unref0 (_block_it);
		}
	}
	{
		GeeIterator* _block_it;
		_block_it = gee_iterable_iterator ((GeeIterable*) block_list);
		while (TRUE) {
			ValaBasicBlock* block;
			ValaBasicBlock* immediate_dominator;
			if (!gee_iterator_next (_block_it)) {
				break;
			}
			block = (ValaBasicBlock*) gee_iterator_get (_block_it);
			if (block == entry_block) {
				_vala_basic_block_unref0 (block);
				continue;
			}
			immediate_dominator = NULL;
			{
				GeeSet* _tmp6_;
				GeeIterator* _tmp7_;
				GeeIterator* _dominator_it;
				_dominator_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = (GeeSet*) gee_map_get ((GeeMap*) dom, block))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
				while (TRUE) {
					ValaBasicBlock* dominator;
					if (!gee_iterator_next (_dominator_it)) {
						break;
					}
					dominator = (ValaBasicBlock*) gee_iterator_get (_dominator_it);
					if (dominator == block) {
						_vala_basic_block_unref0 (dominator);
						continue;
					}
					if (immediate_dominator == NULL) {
						ValaBasicBlock* _tmp8_;
						immediate_dominator = (_tmp8_ = _vala_basic_block_ref0 (dominator), _vala_basic_block_unref0 (immediate_dominator), _tmp8_);
					} else {
						GeeSet* _tmp9_;
						gboolean _tmp10_;
						if ((_tmp10_ = gee_collection_contains ((GeeCollection*) (_tmp9_ = (GeeSet*) gee_map_get ((GeeMap*) dom, dominator)), immediate_dominator), _gee_collection_object_unref0 (_tmp9_), _tmp10_)) {
							ValaBasicBlock* _tmp11_;
							immediate_dominator = (_tmp11_ = _vala_basic_block_ref0 (dominator), _vala_basic_block_unref0 (immediate_dominator), _tmp11_);
						}
					}
					_vala_basic_block_unref0 (dominator);
				}
				_gee_collection_object_unref0 (_dominator_it);
			}
			vala_basic_block_add_child (immediate_dominator, block);
			_vala_basic_block_unref0 (block);
			_vala_basic_block_unref0 (immediate_dominator);
		}
		_gee_collection_object_unref0 (_block_it);
	}
	_gee_collection_object_unref0 (dom);
	_gee_collection_object_unref0 (block_list);
	_gee_collection_object_unref0 (entry_dom_set);
}


static void vala_flow_analyzer_build_dominator_frontier (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block) {
	GeeList* block_list;
	g_return_if_fail (self != NULL);
	g_return_if_fail (entry_block != NULL);
	block_list = vala_flow_analyzer_get_depth_first_list (self, entry_block);
	{
		gint i;
		i = gee_collection_get_size ((GeeCollection*) block_list) - 1;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				ValaBasicBlock* block;
				if (!_tmp0_) {
					i--;
				}
				_tmp0_ = FALSE;
				if (!(i >= 0)) {
					break;
				}
				block = (ValaBasicBlock*) gee_list_get (block_list, i);
				{
					GeeList* _tmp1_;
					GeeIterator* _tmp2_;
					GeeIterator* _succ_it;
					_succ_it = (_tmp2_ = gee_iterable_iterator ((GeeIterable*) (_tmp1_ = vala_basic_block_get_successors (block))), _gee_collection_object_unref0 (_tmp1_), _tmp2_);
					while (TRUE) {
						ValaBasicBlock* succ;
						if (!gee_iterator_next (_succ_it)) {
							break;
						}
						succ = _vala_basic_block_ref0 ((ValaBasicBlock*) gee_iterator_get (_succ_it));
						if (vala_basic_block_get_parent (succ) != block) {
							vala_basic_block_add_dominator_frontier (block, succ);
						}
						_vala_basic_block_unref0 (succ);
					}
					_gee_collection_object_unref0 (_succ_it);
				}
				{
					GeeList* _tmp3_;
					GeeIterator* _tmp4_;
					GeeIterator* _child_it;
					_child_it = (_tmp4_ = gee_iterable_iterator ((GeeIterable*) (_tmp3_ = vala_basic_block_get_children (block))), _gee_collection_object_unref0 (_tmp3_), _tmp4_);
					while (TRUE) {
						ValaBasicBlock* child;
						if (!gee_iterator_next (_child_it)) {
							break;
						}
						child = (ValaBasicBlock*) gee_iterator_get (_child_it);
						{
							GeeSet* _tmp5_;
							GeeIterator* _tmp6_;
							GeeIterator* _child_frontier_it;
							_child_frontier_it = (_tmp6_ = gee_iterable_iterator ((GeeIterable*) (_tmp5_ = vala_basic_block_get_dominator_frontier (child))), _gee_collection_object_unref0 (_tmp5_), _tmp6_);
							while (TRUE) {
								ValaBasicBlock* child_frontier;
								if (!gee_iterator_next (_child_frontier_it)) {
									break;
								}
								child_frontier = (ValaBasicBlock*) gee_iterator_get (_child_frontier_it);
								if (vala_basic_block_get_parent (child_frontier) != block) {
									vala_basic_block_add_dominator_frontier (block, child_frontier);
								}
								_vala_basic_block_unref0 (child_frontier);
							}
							_gee_collection_object_unref0 (_child_frontier_it);
						}
						_vala_basic_block_unref0 (child);
					}
					_gee_collection_object_unref0 (_child_it);
				}
				_vala_basic_block_unref0 (block);
			}
		}
	}
	_gee_collection_object_unref0 (block_list);
}


static GeeMap* vala_flow_analyzer_get_assignment_map (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block) {
	GeeMap* result;
	GeeHashMap* map;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (entry_block != NULL, NULL);
	map = gee_hash_map_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, GEE_TYPE_SET, (GBoxedCopyFunc) gee_collection_object_ref, gee_collection_object_unref, g_direct_hash, g_direct_equal, g_direct_equal);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _block_it;
		_block_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_flow_analyzer_get_depth_first_list (self, entry_block))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaBasicBlock* block;
			GeeArrayList* defined_variables;
			if (!gee_iterator_next (_block_it)) {
				break;
			}
			block = (ValaBasicBlock*) gee_iterator_get (_block_it);
			defined_variables = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
			{
				GeeList* _tmp2_;
				GeeIterator* _tmp3_;
				GeeIterator* _node_it;
				_node_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_basic_block_get_nodes (block))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
				while (TRUE) {
					ValaCodeNode* node;
					if (!gee_iterator_next (_node_it)) {
						break;
					}
					node = (ValaCodeNode*) gee_iterator_get (_node_it);
					vala_code_node_get_defined_variables (node, (GeeCollection*) defined_variables);
					_vala_code_node_unref0 (node);
				}
				_gee_collection_object_unref0 (_node_it);
			}
			{
				GeeIterator* _local_it;
				_local_it = gee_iterable_iterator ((GeeIterable*) defined_variables);
				while (TRUE) {
					ValaLocalVariable* local;
					GeeSet* block_set;
					if (!gee_iterator_next (_local_it)) {
						break;
					}
					local = (ValaLocalVariable*) gee_iterator_get (_local_it);
					block_set = (GeeSet*) gee_map_get ((GeeMap*) map, local);
					if (block_set == NULL) {
						GeeSet* _tmp4_;
						block_set = (_tmp4_ = (GeeSet*) gee_hash_set_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_hash, g_direct_equal), _gee_collection_object_unref0 (block_set), _tmp4_);
						gee_map_set ((GeeMap*) map, local, block_set);
					}
					gee_collection_add ((GeeCollection*) block_set, block);
					_vala_code_node_unref0 (local);
					_gee_collection_object_unref0 (block_set);
				}
				_gee_collection_object_unref0 (_local_it);
			}
			_vala_basic_block_unref0 (block);
			_gee_collection_object_unref0 (defined_variables);
		}
		_gee_collection_object_unref0 (_block_it);
	}
	result = (GeeMap*) map;
	return result;
}


static void vala_flow_analyzer_insert_phi_functions (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block) {
	GeeMap* assign;
	gint counter;
	GeeArrayList* work_list;
	GeeHashMap* added;
	GeeHashMap* phi;
	g_return_if_fail (self != NULL);
	g_return_if_fail (entry_block != NULL);
	assign = vala_flow_analyzer_get_assignment_map (self, entry_block);
	counter = 0;
	work_list = gee_array_list_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_equal);
	added = gee_hash_map_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, G_TYPE_INT, NULL, NULL, g_direct_hash, g_direct_equal, g_direct_equal);
	phi = gee_hash_map_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, G_TYPE_INT, NULL, NULL, g_direct_hash, g_direct_equal, g_direct_equal);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _block_it;
		_block_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_flow_analyzer_get_depth_first_list (self, entry_block))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaBasicBlock* block;
			if (!gee_iterator_next (_block_it)) {
				break;
			}
			block = (ValaBasicBlock*) gee_iterator_get (_block_it);
			gee_map_set ((GeeMap*) added, block, GINT_TO_POINTER (0));
			gee_map_set ((GeeMap*) phi, block, GINT_TO_POINTER (0));
			_vala_basic_block_unref0 (block);
		}
		_gee_collection_object_unref0 (_block_it);
	}
	{
		GeeSet* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _local_it;
		_local_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = gee_map_get_keys (assign))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			ValaLocalVariable* local;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			counter++;
			{
				GeeSet* _tmp4_;
				GeeIterator* _tmp5_;
				GeeIterator* _block_it;
				_block_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = (GeeSet*) gee_map_get (assign, local))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
				while (TRUE) {
					ValaBasicBlock* block;
					if (!gee_iterator_next (_block_it)) {
						break;
					}
					block = (ValaBasicBlock*) gee_iterator_get (_block_it);
					gee_collection_add ((GeeCollection*) work_list, block);
					gee_map_set ((GeeMap*) added, block, GINT_TO_POINTER (counter));
					_vala_basic_block_unref0 (block);
				}
				_gee_collection_object_unref0 (_block_it);
			}
			while (TRUE) {
				ValaBasicBlock* block;
				if (!(gee_collection_get_size ((GeeCollection*) work_list) > 0)) {
					break;
				}
				block = (ValaBasicBlock*) gee_list_get ((GeeList*) work_list, 0);
				gee_list_remove_at ((GeeList*) work_list, 0);
				{
					GeeSet* _tmp6_;
					GeeIterator* _tmp7_;
					GeeIterator* _frontier_it;
					_frontier_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = vala_basic_block_get_dominator_frontier (block))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
					while (TRUE) {
						ValaBasicBlock* frontier;
						gint blockPhi;
						if (!gee_iterator_next (_frontier_it)) {
							break;
						}
						frontier = (ValaBasicBlock*) gee_iterator_get (_frontier_it);
						blockPhi = GPOINTER_TO_INT (gee_map_get ((GeeMap*) phi, frontier));
						if (blockPhi < counter) {
							ValaPhiFunction* _tmp9_;
							GeeList* _tmp8_;
							gint block_added;
							vala_basic_block_add_phi_function (frontier, _tmp9_ = vala_phi_function_new (local, gee_collection_get_size ((GeeCollection*) (_tmp8_ = vala_basic_block_get_predecessors (frontier)))));
							_vala_phi_function_unref0 (_tmp9_);
							_gee_collection_object_unref0 (_tmp8_);
							gee_map_set ((GeeMap*) phi, frontier, GINT_TO_POINTER (counter));
							block_added = GPOINTER_TO_INT (gee_map_get ((GeeMap*) added, frontier));
							if (block_added < counter) {
								gee_map_set ((GeeMap*) added, frontier, GINT_TO_POINTER (counter));
								gee_collection_add ((GeeCollection*) work_list, frontier);
							}
						}
						_vala_basic_block_unref0 (frontier);
					}
					_gee_collection_object_unref0 (_frontier_it);
				}
				_vala_basic_block_unref0 (block);
			}
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	_gee_collection_object_unref0 (assign);
	_gee_collection_object_unref0 (work_list);
	_gee_collection_object_unref0 (added);
	_gee_collection_object_unref0 (phi);
}


static void vala_flow_analyzer_check_variables (ValaFlowAnalyzer* self, ValaBasicBlock* entry_block) {
	GeeMap* _tmp0_;
	GeeSet* _tmp1_;
	GeeMap* _tmp2_;
	GeeArrayList* used_vars_queue;
	g_return_if_fail (self != NULL);
	g_return_if_fail (entry_block != NULL);
	self->priv->var_map = (_tmp0_ = (GeeMap*) gee_hash_map_new (VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, GEE_TYPE_LIST, (GBoxedCopyFunc) gee_collection_object_ref, gee_collection_object_unref, g_direct_hash, g_direct_equal, g_direct_equal), _gee_collection_object_unref0 (self->priv->var_map), _tmp0_);
	self->priv->used_vars = (_tmp1_ = (GeeSet*) gee_hash_set_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_hash, g_direct_equal), _gee_collection_object_unref0 (self->priv->used_vars), _tmp1_);
	self->priv->phi_functions = (_tmp2_ = (GeeMap*) gee_hash_map_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, VALA_TYPE_PHI_FUNCTION, (GBoxedCopyFunc) vala_phi_function_ref, vala_phi_function_unref, g_direct_hash, g_direct_equal, g_direct_equal), _gee_collection_object_unref0 (self->priv->phi_functions), _tmp2_);
	vala_flow_analyzer_check_block_variables (self, entry_block);
	used_vars_queue = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
	{
		GeeIterator* _local_it;
		_local_it = gee_iterable_iterator ((GeeIterable*) self->priv->used_vars);
		while (TRUE) {
			ValaLocalVariable* local;
			if (!gee_iterator_next (_local_it)) {
				break;
			}
			local = (ValaLocalVariable*) gee_iterator_get (_local_it);
			gee_collection_add ((GeeCollection*) used_vars_queue, local);
			_vala_code_node_unref0 (local);
		}
		_gee_collection_object_unref0 (_local_it);
	}
	while (TRUE) {
		ValaLocalVariable* used_var;
		ValaPhiFunction* phi;
		if (!(gee_collection_get_size ((GeeCollection*) used_vars_queue) > 0)) {
			break;
		}
		used_var = (ValaLocalVariable*) gee_list_get ((GeeList*) used_vars_queue, 0);
		gee_list_remove_at ((GeeList*) used_vars_queue, 0);
		phi = (ValaPhiFunction*) gee_map_get (self->priv->phi_functions, used_var);
		if (phi != NULL) {
			{
				GeeIterator* _local_it;
				_local_it = gee_iterable_iterator ((GeeIterable*) vala_phi_function_get_operands (phi));
				while (TRUE) {
					ValaLocalVariable* local;
					if (!gee_iterator_next (_local_it)) {
						break;
					}
					local = (ValaLocalVariable*) gee_iterator_get (_local_it);
					if (local == NULL) {
						char* _tmp3_;
						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) used_var), _tmp3_ = g_strdup_printf ("use of possibly unassigned local variable `%s'", vala_symbol_get_name ((ValaSymbol*) used_var)));
						_g_free0 (_tmp3_);
						_vala_code_node_unref0 (local);
						continue;
					}
					if (!gee_collection_contains ((GeeCollection*) self->priv->used_vars, local)) {
						vala_code_node_set_source_reference ((ValaCodeNode*) local, vala_code_node_get_source_reference ((ValaCodeNode*) used_var));
						gee_collection_add ((GeeCollection*) self->priv->used_vars, local);
						gee_collection_add ((GeeCollection*) used_vars_queue, local);
					}
					_vala_code_node_unref0 (local);
				}
				_gee_collection_object_unref0 (_local_it);
			}
		}
		_vala_code_node_unref0 (used_var);
		_vala_phi_function_unref0 (phi);
	}
	_gee_collection_object_unref0 (used_vars_queue);
}


static void vala_flow_analyzer_check_block_variables (ValaFlowAnalyzer* self, ValaBasicBlock* block) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	{
		GeeSet* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _phi_it;
		_phi_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_basic_block_get_phi_functions (block))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaPhiFunction* phi;
			ValaLocalVariable* versioned_var;
			if (!gee_iterator_next (_phi_it)) {
				break;
			}
			phi = (ValaPhiFunction*) gee_iterator_get (_phi_it);
			versioned_var = vala_flow_analyzer_process_assignment (self, self->priv->var_map, vala_phi_function_get_original_variable (phi));
			gee_map_set (self->priv->phi_functions, versioned_var, phi);
			_vala_phi_function_unref0 (phi);
			_vala_code_node_unref0 (versioned_var);
		}
		_gee_collection_object_unref0 (_phi_it);
	}
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _node_it;
		_node_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_basic_block_get_nodes (block))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			ValaCodeNode* node;
			GeeArrayList* used_variables;
			GeeArrayList* defined_variables;
			if (!gee_iterator_next (_node_it)) {
				break;
			}
			node = (ValaCodeNode*) gee_iterator_get (_node_it);
			used_variables = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
			vala_code_node_get_used_variables (node, (GeeCollection*) used_variables);
			{
				GeeIterator* _var_symbol_it;
				_var_symbol_it = gee_iterable_iterator ((GeeIterable*) used_variables);
				while (TRUE) {
					ValaLocalVariable* var_symbol;
					GeeList* variable_stack;
					gboolean _tmp4_;
					ValaLocalVariable* versioned_local;
					if (!gee_iterator_next (_var_symbol_it)) {
						break;
					}
					var_symbol = (ValaLocalVariable*) gee_iterator_get (_var_symbol_it);
					variable_stack = (GeeList*) gee_map_get (self->priv->var_map, (ValaSymbol*) var_symbol);
					_tmp4_ = FALSE;
					if (variable_stack == NULL) {
						_tmp4_ = TRUE;
					} else {
						_tmp4_ = gee_collection_get_size ((GeeCollection*) variable_stack) == 0;
					}
					if (_tmp4_) {
						char* _tmp5_;
						vala_report_error (vala_code_node_get_source_reference (node), _tmp5_ = g_strdup_printf ("use of possibly unassigned local variable `%s'", vala_symbol_get_name ((ValaSymbol*) var_symbol)));
						_g_free0 (_tmp5_);
						_vala_code_node_unref0 (var_symbol);
						_gee_collection_object_unref0 (variable_stack);
						continue;
					}
					versioned_local = (ValaLocalVariable*) gee_list_get (variable_stack, gee_collection_get_size ((GeeCollection*) variable_stack) - 1);
					if (!gee_collection_contains ((GeeCollection*) self->priv->used_vars, versioned_local)) {
						vala_code_node_set_source_reference ((ValaCodeNode*) versioned_local, vala_code_node_get_source_reference (node));
					}
					gee_collection_add ((GeeCollection*) self->priv->used_vars, versioned_local);
					_vala_code_node_unref0 (var_symbol);
					_gee_collection_object_unref0 (variable_stack);
					_vala_code_node_unref0 (versioned_local);
				}
				_gee_collection_object_unref0 (_var_symbol_it);
			}
			defined_variables = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
			vala_code_node_get_defined_variables (node, (GeeCollection*) defined_variables);
			{
				GeeIterator* _local_it;
				_local_it = gee_iterable_iterator ((GeeIterable*) defined_variables);
				while (TRUE) {
					ValaLocalVariable* local;
					ValaLocalVariable* _tmp6_;
					if (!gee_iterator_next (_local_it)) {
						break;
					}
					local = (ValaLocalVariable*) gee_iterator_get (_local_it);
					_tmp6_ = vala_flow_analyzer_process_assignment (self, self->priv->var_map, local);
					_vala_code_node_unref0 (_tmp6_);
					_vala_code_node_unref0 (local);
				}
				_gee_collection_object_unref0 (_local_it);
			}
			_vala_code_node_unref0 (node);
			_gee_collection_object_unref0 (used_variables);
			_gee_collection_object_unref0 (defined_variables);
		}
		_gee_collection_object_unref0 (_node_it);
	}
	{
		GeeList* _tmp7_;
		GeeIterator* _tmp8_;
		GeeIterator* _succ_it;
		_succ_it = (_tmp8_ = gee_iterable_iterator ((GeeIterable*) (_tmp7_ = vala_basic_block_get_successors (block))), _gee_collection_object_unref0 (_tmp7_), _tmp8_);
		while (TRUE) {
			ValaBasicBlock* succ;
			gint j;
			if (!gee_iterator_next (_succ_it)) {
				break;
			}
			succ = _vala_basic_block_ref0 ((ValaBasicBlock*) gee_iterator_get (_succ_it));
			j = 0;
			{
				GeeList* _tmp9_;
				GeeIterator* _tmp10_;
				GeeIterator* _pred_it;
				_pred_it = (_tmp10_ = gee_iterable_iterator ((GeeIterable*) (_tmp9_ = vala_basic_block_get_predecessors (succ))), _gee_collection_object_unref0 (_tmp9_), _tmp10_);
				while (TRUE) {
					ValaBasicBlock* pred;
					if (!gee_iterator_next (_pred_it)) {
						break;
					}
					pred = _vala_basic_block_ref0 ((ValaBasicBlock*) gee_iterator_get (_pred_it));
					if (pred == block) {
						_vala_basic_block_unref0 (pred);
						break;
					}
					j++;
					_vala_basic_block_unref0 (pred);
				}
				_gee_collection_object_unref0 (_pred_it);
			}
			{
				GeeSet* _tmp11_;
				GeeIterator* _tmp12_;
				GeeIterator* _phi_it;
				_phi_it = (_tmp12_ = gee_iterable_iterator ((GeeIterable*) (_tmp11_ = vala_basic_block_get_phi_functions (succ))), _gee_collection_object_unref0 (_tmp11_), _tmp12_);
				while (TRUE) {
					ValaPhiFunction* phi;
					GeeList* variable_stack;
					gboolean _tmp13_;
					if (!gee_iterator_next (_phi_it)) {
						break;
					}
					phi = (ValaPhiFunction*) gee_iterator_get (_phi_it);
					variable_stack = (GeeList*) gee_map_get (self->priv->var_map, (ValaSymbol*) vala_phi_function_get_original_variable (phi));
					_tmp13_ = FALSE;
					if (variable_stack != NULL) {
						_tmp13_ = gee_collection_get_size ((GeeCollection*) variable_stack) > 0;
					} else {
						_tmp13_ = FALSE;
					}
					if (_tmp13_) {
						ValaLocalVariable* _tmp14_;
						gee_list_set (vala_phi_function_get_operands (phi), j, _tmp14_ = (ValaLocalVariable*) gee_list_get (variable_stack, gee_collection_get_size ((GeeCollection*) variable_stack) - 1));
						_vala_code_node_unref0 (_tmp14_);
					}
					_vala_phi_function_unref0 (phi);
					_gee_collection_object_unref0 (variable_stack);
				}
				_gee_collection_object_unref0 (_phi_it);
			}
			_vala_basic_block_unref0 (succ);
		}
		_gee_collection_object_unref0 (_succ_it);
	}
	{
		GeeList* _tmp15_;
		GeeIterator* _tmp16_;
		GeeIterator* _child_it;
		_child_it = (_tmp16_ = gee_iterable_iterator ((GeeIterable*) (_tmp15_ = vala_basic_block_get_children (block))), _gee_collection_object_unref0 (_tmp15_), _tmp16_);
		while (TRUE) {
			ValaBasicBlock* child;
			if (!gee_iterator_next (_child_it)) {
				break;
			}
			child = (ValaBasicBlock*) gee_iterator_get (_child_it);
			vala_flow_analyzer_check_block_variables (self, child);
			_vala_basic_block_unref0 (child);
		}
		_gee_collection_object_unref0 (_child_it);
	}
	{
		GeeSet* _tmp17_;
		GeeIterator* _tmp18_;
		GeeIterator* _phi_it;
		_phi_it = (_tmp18_ = gee_iterable_iterator ((GeeIterable*) (_tmp17_ = vala_basic_block_get_phi_functions (block))), _gee_collection_object_unref0 (_tmp17_), _tmp18_);
		while (TRUE) {
			ValaPhiFunction* phi;
			GeeList* variable_stack;
			if (!gee_iterator_next (_phi_it)) {
				break;
			}
			phi = (ValaPhiFunction*) gee_iterator_get (_phi_it);
			variable_stack = (GeeList*) gee_map_get (self->priv->var_map, (ValaSymbol*) vala_phi_function_get_original_variable (phi));
			gee_list_remove_at (variable_stack, gee_collection_get_size ((GeeCollection*) variable_stack) - 1);
			_vala_phi_function_unref0 (phi);
			_gee_collection_object_unref0 (variable_stack);
		}
		_gee_collection_object_unref0 (_phi_it);
	}
	{
		GeeList* _tmp19_;
		GeeIterator* _tmp20_;
		GeeIterator* _node_it;
		_node_it = (_tmp20_ = gee_iterable_iterator ((GeeIterable*) (_tmp19_ = vala_basic_block_get_nodes (block))), _gee_collection_object_unref0 (_tmp19_), _tmp20_);
		while (TRUE) {
			ValaCodeNode* node;
			GeeArrayList* defined_variables;
			if (!gee_iterator_next (_node_it)) {
				break;
			}
			node = (ValaCodeNode*) gee_iterator_get (_node_it);
			defined_variables = gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
			vala_code_node_get_defined_variables (node, (GeeCollection*) defined_variables);
			{
				GeeIterator* _local_it;
				_local_it = gee_iterable_iterator ((GeeIterable*) defined_variables);
				while (TRUE) {
					ValaLocalVariable* local;
					GeeList* variable_stack;
					if (!gee_iterator_next (_local_it)) {
						break;
					}
					local = (ValaLocalVariable*) gee_iterator_get (_local_it);
					variable_stack = (GeeList*) gee_map_get (self->priv->var_map, (ValaSymbol*) local);
					gee_list_remove_at (variable_stack, gee_collection_get_size ((GeeCollection*) variable_stack) - 1);
					_vala_code_node_unref0 (local);
					_gee_collection_object_unref0 (variable_stack);
				}
				_gee_collection_object_unref0 (_local_it);
			}
			_vala_code_node_unref0 (node);
			_gee_collection_object_unref0 (defined_variables);
		}
		_gee_collection_object_unref0 (_node_it);
	}
}


static ValaLocalVariable* vala_flow_analyzer_process_assignment (ValaFlowAnalyzer* self, GeeMap* var_map, ValaLocalVariable* var_symbol) {
	ValaLocalVariable* result;
	GeeList* variable_stack;
	ValaLocalVariable* versioned_var;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (var_map != NULL, NULL);
	g_return_val_if_fail (var_symbol != NULL, NULL);
	variable_stack = (GeeList*) gee_map_get (var_map, (ValaSymbol*) var_symbol);
	if (variable_stack == NULL) {
		GeeList* _tmp0_;
		variable_stack = (_tmp0_ = (GeeList*) gee_array_list_new (VALA_TYPE_LOCAL_VARIABLE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), _gee_collection_object_unref0 (variable_stack), _tmp0_);
		gee_map_set (var_map, (ValaSymbol*) var_symbol, variable_stack);
	}
	versioned_var = vala_local_variable_new (vala_local_variable_get_variable_type (var_symbol), vala_symbol_get_name ((ValaSymbol*) var_symbol), NULL, vala_code_node_get_source_reference ((ValaCodeNode*) var_symbol));
	gee_collection_add ((GeeCollection*) variable_stack, versioned_var);
	result = versioned_var;
	_gee_collection_object_unref0 (variable_stack);
	return result;
}


static void vala_flow_analyzer_real_visit_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (m != NULL);
	vala_code_visitor_visit_method ((ValaCodeVisitor*) self, (ValaMethod*) m);
}


static void vala_flow_analyzer_real_visit_property (ValaCodeVisitor* base, ValaProperty* prop) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (prop != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) prop, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc) {
	ValaFlowAnalyzer * self;
	ValaBasicBlock* _tmp0_;
	ValaBasicBlock* _tmp1_;
	ValaBasicBlock* _tmp2_;
	ValaFlowAnalyzerJumpTarget* _tmp3_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (acc != NULL);
	if (vala_property_accessor_get_body (acc) == NULL) {
		return;
	}
	vala_property_accessor_set_entry_block (acc, _tmp0_ = vala_basic_block_new_entry ());
	_vala_basic_block_unref0 (_tmp0_);
	vala_property_accessor_set_exit_block (acc, _tmp1_ = vala_basic_block_new_exit ());
	_vala_basic_block_unref0 (_tmp1_);
	self->priv->current_block = (_tmp2_ = vala_basic_block_new (), _vala_basic_block_unref0 (self->priv->current_block), _tmp2_);
	vala_basic_block_connect (vala_property_accessor_get_entry_block (acc), self->priv->current_block);
	gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp3_ = vala_flow_analyzer_jump_target_new_return_target (vala_property_accessor_get_exit_block (acc)));
	_vala_flow_analyzer_jump_target_unref0 (_tmp3_);
	vala_code_node_accept_children ((ValaCodeNode*) acc, (ValaCodeVisitor*) self);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	if (self->priv->current_block != NULL) {
		if (vala_property_accessor_get_readable (acc)) {
			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) acc), "missing return statement at end of property getter body");
			vala_code_node_set_error ((ValaCodeNode*) acc, TRUE);
		}
		vala_basic_block_connect (self->priv->current_block, vala_property_accessor_get_exit_block (acc));
	}
	vala_flow_analyzer_build_dominator_tree (self, vala_property_accessor_get_entry_block (acc));
	vala_flow_analyzer_build_dominator_frontier (self, vala_property_accessor_get_entry_block (acc));
	vala_flow_analyzer_insert_phi_functions (self, vala_property_accessor_get_entry_block (acc));
	vala_flow_analyzer_check_variables (self, vala_property_accessor_get_entry_block (acc));
}


static void vala_flow_analyzer_real_visit_block (ValaCodeVisitor* base, ValaBlock* b) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (b != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) b, (ValaCodeVisitor*) self);
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


static void vala_flow_analyzer_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt) {
	ValaFlowAnalyzer * self;
	ValaSymbol* _tmp1_;
	ValaLocalVariable* local;
	gboolean _tmp2_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	if (!vala_symbol_get_used (vala_declaration_statement_get_declaration (stmt))) {
		char* _tmp0_;
		vala_report_warning (vala_code_node_get_source_reference ((ValaCodeNode*) vala_declaration_statement_get_declaration (stmt)), _tmp0_ = g_strdup_printf ("local variable `%s' declared but never used", vala_symbol_get_name (vala_declaration_statement_get_declaration (stmt))));
		_g_free0 (_tmp0_);
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) stmt);
	local = _vala_code_node_ref0 ((_tmp1_ = vala_declaration_statement_get_declaration (stmt), VALA_IS_LOCAL_VARIABLE (_tmp1_) ? ((ValaLocalVariable*) _tmp1_) : NULL));
	_tmp2_ = FALSE;
	if (local != NULL) {
		_tmp2_ = vala_local_variable_get_initializer (local) != NULL;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		vala_flow_analyzer_handle_errors (self, (ValaCodeNode*) vala_local_variable_get_initializer (local), FALSE);
	}
	_vala_code_node_unref0 (local);
}


static void vala_flow_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) stmt);
	vala_flow_analyzer_handle_errors (self, (ValaCodeNode*) stmt, FALSE);
	if (VALA_IS_METHOD_CALL (vala_expression_statement_get_expression (stmt))) {
		ValaMethodCall* expr;
		ValaExpression* _tmp0_;
		ValaMemberAccess* ma;
		gboolean _tmp1_;
		gboolean _tmp2_;
		expr = _vala_code_node_ref0 (VALA_METHOD_CALL (vala_expression_statement_get_expression (stmt)));
		ma = _vala_code_node_ref0 ((_tmp0_ = vala_method_call_get_call (expr), VALA_IS_MEMBER_ACCESS (_tmp0_) ? ((ValaMemberAccess*) _tmp0_) : NULL));
		_tmp1_ = FALSE;
		_tmp2_ = FALSE;
		if (ma != NULL) {
			_tmp2_ = vala_expression_get_symbol_reference ((ValaExpression*) ma) != NULL;
		} else {
			_tmp2_ = FALSE;
		}
		if (_tmp2_) {
			ValaAttribute* _tmp3_;
			_tmp1_ = (_tmp3_ = vala_code_node_get_attribute ((ValaCodeNode*) vala_expression_get_symbol_reference ((ValaExpression*) ma), "NoReturn")) != NULL;
			_vala_code_node_unref0 (_tmp3_);
		} else {
			_tmp1_ = FALSE;
		}
		if (_tmp1_) {
			ValaBasicBlock* _tmp4_;
			self->priv->current_block = (_tmp4_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp4_);
			self->priv->unreachable_reported = FALSE;
			_vala_code_node_unref0 (expr);
			_vala_code_node_unref0 (ma);
			return;
		}
		_vala_code_node_unref0 (expr);
		_vala_code_node_unref0 (ma);
	}
}


static gboolean vala_flow_analyzer_always_true (ValaFlowAnalyzer* self, ValaExpression* condition) {
	gboolean result;
	ValaExpression* _tmp0_;
	ValaBooleanLiteral* literal;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (condition != NULL, FALSE);
	literal = _vala_code_node_ref0 ((_tmp0_ = condition, VALA_IS_BOOLEAN_LITERAL (_tmp0_) ? ((ValaBooleanLiteral*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (literal != NULL) {
		_tmp1_ = vala_boolean_literal_get_value (literal);
	} else {
		_tmp1_ = FALSE;
	}
	result = _tmp1_;
	_vala_code_node_unref0 (literal);
	return result;
}


static gboolean vala_flow_analyzer_always_false (ValaFlowAnalyzer* self, ValaExpression* condition) {
	gboolean result;
	ValaExpression* _tmp0_;
	ValaBooleanLiteral* literal;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (condition != NULL, FALSE);
	literal = _vala_code_node_ref0 ((_tmp0_ = condition, VALA_IS_BOOLEAN_LITERAL (_tmp0_) ? ((ValaBooleanLiteral*) _tmp0_) : NULL));
	_tmp1_ = FALSE;
	if (literal != NULL) {
		_tmp1_ = !vala_boolean_literal_get_value (literal);
	} else {
		_tmp1_ = FALSE;
	}
	result = _tmp1_;
	_vala_code_node_unref0 (literal);
	return result;
}


static void vala_flow_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt) {
	ValaFlowAnalyzer * self;
	ValaBasicBlock* last_block;
	ValaBasicBlock* last_true_block;
	ValaBasicBlock* last_false_block;
	gboolean _tmp4_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) vala_if_statement_get_condition (stmt));
	vala_flow_analyzer_handle_errors (self, (ValaCodeNode*) vala_if_statement_get_condition (stmt), FALSE);
	last_block = _vala_basic_block_ref0 (self->priv->current_block);
	if (vala_flow_analyzer_always_false (self, vala_if_statement_get_condition (stmt))) {
		ValaBasicBlock* _tmp0_;
		self->priv->current_block = (_tmp0_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp0_);
		self->priv->unreachable_reported = FALSE;
	} else {
		ValaBasicBlock* _tmp1_;
		self->priv->current_block = (_tmp1_ = vala_basic_block_new (), _vala_basic_block_unref0 (self->priv->current_block), _tmp1_);
		vala_basic_block_connect (last_block, self->priv->current_block);
	}
	vala_code_node_accept ((ValaCodeNode*) vala_if_statement_get_true_statement (stmt), (ValaCodeVisitor*) self);
	last_true_block = _vala_basic_block_ref0 (self->priv->current_block);
	if (vala_flow_analyzer_always_true (self, vala_if_statement_get_condition (stmt))) {
		ValaBasicBlock* _tmp2_;
		self->priv->current_block = (_tmp2_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp2_);
		self->priv->unreachable_reported = FALSE;
	} else {
		ValaBasicBlock* _tmp3_;
		self->priv->current_block = (_tmp3_ = vala_basic_block_new (), _vala_basic_block_unref0 (self->priv->current_block), _tmp3_);
		vala_basic_block_connect (last_block, self->priv->current_block);
	}
	if (vala_if_statement_get_false_statement (stmt) != NULL) {
		vala_code_node_accept ((ValaCodeNode*) vala_if_statement_get_false_statement (stmt), (ValaCodeVisitor*) self);
	}
	last_false_block = _vala_basic_block_ref0 (self->priv->current_block);
	_tmp4_ = FALSE;
	if (last_true_block != NULL) {
		_tmp4_ = TRUE;
	} else {
		_tmp4_ = last_false_block != NULL;
	}
	if (_tmp4_) {
		ValaBasicBlock* _tmp5_;
		self->priv->current_block = (_tmp5_ = vala_basic_block_new (), _vala_basic_block_unref0 (self->priv->current_block), _tmp5_);
		if (last_true_block != NULL) {
			vala_basic_block_connect (last_true_block, self->priv->current_block);
		}
		if (last_false_block != NULL) {
			vala_basic_block_connect (last_false_block, self->priv->current_block);
		}
	}
	_vala_basic_block_unref0 (last_block);
	_vala_basic_block_unref0 (last_true_block);
	_vala_basic_block_unref0 (last_false_block);
}


static void vala_flow_analyzer_real_visit_switch_statement (ValaCodeVisitor* base, ValaSwitchStatement* stmt) {
	ValaFlowAnalyzer * self;
	ValaBasicBlock* after_switch_block;
	ValaFlowAnalyzerJumpTarget* _tmp0_;
	ValaBasicBlock* condition_block;
	gboolean has_default_label;
	GeeList* _tmp6_;
	gboolean _tmp7_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	after_switch_block = vala_basic_block_new ();
	gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp0_ = vala_flow_analyzer_jump_target_new_break_target (after_switch_block));
	_vala_flow_analyzer_jump_target_unref0 (_tmp0_);
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) vala_switch_statement_get_expression (stmt));
	condition_block = _vala_basic_block_ref0 (self->priv->current_block);
	vala_flow_analyzer_handle_errors (self, (ValaCodeNode*) vala_switch_statement_get_expression (stmt), FALSE);
	has_default_label = FALSE;
	{
		GeeList* _tmp1_;
		GeeIterator* _tmp2_;
		GeeIterator* _section_it;
		_section_it = (_tmp2_ = gee_iterable_iterator ((GeeIterable*) (_tmp1_ = vala_switch_statement_get_sections (stmt))), _gee_collection_object_unref0 (_tmp1_), _tmp2_);
		while (TRUE) {
			ValaSwitchSection* section;
			ValaBasicBlock* _tmp3_;
			if (!gee_iterator_next (_section_it)) {
				break;
			}
			section = (ValaSwitchSection*) gee_iterator_get (_section_it);
			self->priv->current_block = (_tmp3_ = vala_basic_block_new (), _vala_basic_block_unref0 (self->priv->current_block), _tmp3_);
			vala_basic_block_connect (condition_block, self->priv->current_block);
			{
				GeeList* _tmp4_;
				GeeIterator* _tmp5_;
				GeeIterator* _section_stmt_it;
				_section_stmt_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vala_block_get_statements ((ValaBlock*) section))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
				while (TRUE) {
					ValaStatement* section_stmt;
					if (!gee_iterator_next (_section_stmt_it)) {
						break;
					}
					section_stmt = (ValaStatement*) gee_iterator_get (_section_stmt_it);
					vala_code_node_accept ((ValaCodeNode*) section_stmt, (ValaCodeVisitor*) self);
					_vala_code_node_unref0 (section_stmt);
				}
				_gee_collection_object_unref0 (_section_stmt_it);
			}
			if (vala_switch_section_has_default_label (section)) {
				has_default_label = TRUE;
			}
			if (self->priv->current_block != NULL) {
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) section), "missing break statement at end of switch section");
				vala_code_node_set_error ((ValaCodeNode*) section, TRUE);
				vala_basic_block_connect (self->priv->current_block, after_switch_block);
			}
			_vala_code_node_unref0 (section);
		}
		_gee_collection_object_unref0 (_section_it);
	}
	if (!has_default_label) {
		vala_basic_block_connect (condition_block, after_switch_block);
	}
	if ((_tmp7_ = gee_collection_get_size ((GeeCollection*) (_tmp6_ = vala_basic_block_get_predecessors (after_switch_block))) > 0, _gee_collection_object_unref0 (_tmp6_), _tmp7_)) {
		ValaBasicBlock* _tmp8_;
		self->priv->current_block = (_tmp8_ = _vala_basic_block_ref0 (after_switch_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp8_);
	} else {
		ValaBasicBlock* _tmp9_;
		self->priv->current_block = (_tmp9_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp9_);
		self->priv->unreachable_reported = FALSE;
	}
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	_vala_basic_block_unref0 (after_switch_block);
	_vala_basic_block_unref0 (condition_block);
}


static void vala_flow_analyzer_real_visit_loop (ValaCodeVisitor* base, ValaLoop* stmt) {
	ValaFlowAnalyzer * self;
	ValaBasicBlock* loop_block;
	ValaFlowAnalyzerJumpTarget* _tmp0_;
	ValaBasicBlock* after_loop_block;
	ValaFlowAnalyzerJumpTarget* _tmp1_;
	ValaBasicBlock* last_block;
	ValaBasicBlock* _tmp2_;
	GeeList* _tmp3_;
	gboolean _tmp4_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	loop_block = vala_basic_block_new ();
	gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp0_ = vala_flow_analyzer_jump_target_new_continue_target (loop_block));
	_vala_flow_analyzer_jump_target_unref0 (_tmp0_);
	after_loop_block = vala_basic_block_new ();
	gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp1_ = vala_flow_analyzer_jump_target_new_break_target (after_loop_block));
	_vala_flow_analyzer_jump_target_unref0 (_tmp1_);
	last_block = _vala_basic_block_ref0 (self->priv->current_block);
	vala_basic_block_connect (last_block, loop_block);
	self->priv->current_block = (_tmp2_ = _vala_basic_block_ref0 (loop_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp2_);
	vala_code_node_accept ((ValaCodeNode*) vala_loop_get_body (stmt), (ValaCodeVisitor*) self);
	if (self->priv->current_block != NULL) {
		vala_basic_block_connect (self->priv->current_block, loop_block);
	}
	if ((_tmp4_ = gee_collection_get_size ((GeeCollection*) (_tmp3_ = vala_basic_block_get_predecessors (after_loop_block))) == 0, _gee_collection_object_unref0 (_tmp3_), _tmp4_)) {
		ValaBasicBlock* _tmp5_;
		self->priv->current_block = (_tmp5_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp5_);
		self->priv->unreachable_reported = FALSE;
	} else {
		ValaBasicBlock* _tmp6_;
		self->priv->current_block = (_tmp6_ = _vala_basic_block_ref0 (after_loop_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp6_);
	}
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	_vala_basic_block_unref0 (loop_block);
	_vala_basic_block_unref0 (after_loop_block);
	_vala_basic_block_unref0 (last_block);
}


static void vala_flow_analyzer_real_visit_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt) {
	ValaFlowAnalyzer * self;
	ValaBasicBlock* loop_block;
	ValaFlowAnalyzerJumpTarget* _tmp0_;
	ValaBasicBlock* after_loop_block;
	ValaFlowAnalyzerJumpTarget* _tmp1_;
	ValaBasicBlock* last_block;
	ValaBasicBlock* _tmp2_;
	ValaBasicBlock* _tmp3_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) vala_foreach_statement_get_collection (stmt));
	vala_flow_analyzer_handle_errors (self, (ValaCodeNode*) vala_foreach_statement_get_collection (stmt), FALSE);
	loop_block = vala_basic_block_new ();
	gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp0_ = vala_flow_analyzer_jump_target_new_continue_target (loop_block));
	_vala_flow_analyzer_jump_target_unref0 (_tmp0_);
	after_loop_block = vala_basic_block_new ();
	gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp1_ = vala_flow_analyzer_jump_target_new_break_target (after_loop_block));
	_vala_flow_analyzer_jump_target_unref0 (_tmp1_);
	last_block = _vala_basic_block_ref0 (self->priv->current_block);
	vala_basic_block_connect (last_block, loop_block);
	self->priv->current_block = (_tmp2_ = _vala_basic_block_ref0 (loop_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp2_);
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) stmt);
	vala_code_node_accept ((ValaCodeNode*) vala_foreach_statement_get_body (stmt), (ValaCodeVisitor*) self);
	if (self->priv->current_block != NULL) {
		vala_basic_block_connect (self->priv->current_block, loop_block);
	}
	vala_basic_block_connect (last_block, after_loop_block);
	if (self->priv->current_block != NULL) {
		vala_basic_block_connect (self->priv->current_block, after_loop_block);
	}
	self->priv->current_block = (_tmp3_ = _vala_basic_block_ref0 (after_loop_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp3_);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	_vala_basic_block_unref0 (loop_block);
	_vala_basic_block_unref0 (after_loop_block);
	_vala_basic_block_unref0 (last_block);
}


static void vala_flow_analyzer_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) stmt);
	{
		gint i;
		i = gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				ValaFlowAnalyzerJumpTarget* jump_target;
				if (!_tmp0_) {
					i--;
				}
				_tmp0_ = FALSE;
				if (!(i >= 0)) {
					break;
				}
				jump_target = (ValaFlowAnalyzerJumpTarget*) gee_list_get (self->priv->jump_stack, i);
				if (vala_flow_analyzer_jump_target_get_is_break_target (jump_target)) {
					ValaBasicBlock* _tmp1_;
					vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
					self->priv->current_block = (_tmp1_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp1_);
					self->priv->unreachable_reported = FALSE;
					_vala_flow_analyzer_jump_target_unref0 (jump_target);
					return;
				} else {
					if (vala_flow_analyzer_jump_target_get_is_finally_clause (jump_target)) {
						ValaBasicBlock* _tmp2_;
						vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
						self->priv->current_block = (_tmp2_ = _vala_basic_block_ref0 (vala_flow_analyzer_jump_target_get_last_block (jump_target)), _vala_basic_block_unref0 (self->priv->current_block), _tmp2_);
					}
				}
				_vala_flow_analyzer_jump_target_unref0 (jump_target);
			}
		}
	}
	vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) stmt), "no enclosing loop or switch statement found");
	vala_code_node_set_error ((ValaCodeNode*) stmt, TRUE);
}


static void vala_flow_analyzer_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) stmt);
	{
		gint i;
		i = gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				ValaFlowAnalyzerJumpTarget* jump_target;
				if (!_tmp0_) {
					i--;
				}
				_tmp0_ = FALSE;
				if (!(i >= 0)) {
					break;
				}
				jump_target = (ValaFlowAnalyzerJumpTarget*) gee_list_get (self->priv->jump_stack, i);
				if (vala_flow_analyzer_jump_target_get_is_continue_target (jump_target)) {
					ValaBasicBlock* _tmp1_;
					vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
					self->priv->current_block = (_tmp1_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp1_);
					self->priv->unreachable_reported = FALSE;
					_vala_flow_analyzer_jump_target_unref0 (jump_target);
					return;
				} else {
					if (vala_flow_analyzer_jump_target_get_is_finally_clause (jump_target)) {
						ValaBasicBlock* _tmp2_;
						vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
						self->priv->current_block = (_tmp2_ = _vala_basic_block_ref0 (vala_flow_analyzer_jump_target_get_last_block (jump_target)), _vala_basic_block_unref0 (self->priv->current_block), _tmp2_);
					}
				}
				_vala_flow_analyzer_jump_target_unref0 (jump_target);
			}
		}
	}
	vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) stmt), "no enclosing loop found");
	vala_code_node_set_error ((ValaCodeNode*) stmt, TRUE);
}


static void vala_flow_analyzer_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) stmt);
	if (vala_return_statement_get_return_expression (stmt) != NULL) {
		vala_flow_analyzer_handle_errors (self, (ValaCodeNode*) vala_return_statement_get_return_expression (stmt), FALSE);
	}
	{
		gint i;
		i = gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				ValaFlowAnalyzerJumpTarget* jump_target;
				if (!_tmp0_) {
					i--;
				}
				_tmp0_ = FALSE;
				if (!(i >= 0)) {
					break;
				}
				jump_target = (ValaFlowAnalyzerJumpTarget*) gee_list_get (self->priv->jump_stack, i);
				if (vala_flow_analyzer_jump_target_get_is_return_target (jump_target)) {
					ValaBasicBlock* _tmp1_;
					vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
					self->priv->current_block = (_tmp1_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp1_);
					self->priv->unreachable_reported = FALSE;
					_vala_flow_analyzer_jump_target_unref0 (jump_target);
					return;
				} else {
					if (vala_flow_analyzer_jump_target_get_is_finally_clause (jump_target)) {
						ValaBasicBlock* _tmp2_;
						vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
						self->priv->current_block = (_tmp2_ = _vala_basic_block_ref0 (vala_flow_analyzer_jump_target_get_last_block (jump_target)), _vala_basic_block_unref0 (self->priv->current_block), _tmp2_);
					}
				}
				_vala_flow_analyzer_jump_target_unref0 (jump_target);
			}
		}
	}
	vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) stmt), "no enclosing loop found");
	vala_code_node_set_error ((ValaCodeNode*) stmt, TRUE);
}


static void vala_flow_analyzer_handle_errors (ValaFlowAnalyzer* self, ValaCodeNode* node, gboolean always_fail) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	if (vala_code_node_get_tree_can_fail (node)) {
		ValaBasicBlock* last_block;
		last_block = _vala_basic_block_ref0 (self->priv->current_block);
		{
			GeeList* _tmp0_;
			GeeIterator* _tmp1_;
			GeeIterator* _error_data_type_it;
			_error_data_type_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_code_node_get_error_types (node))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
			while (TRUE) {
				ValaDataType* error_data_type;
				ValaDataType* _tmp2_;
				ValaErrorType* error_type;
				ValaBasicBlock* _tmp3_;
				if (!gee_iterator_next (_error_data_type_it)) {
					break;
				}
				error_data_type = (ValaDataType*) gee_iterator_get (_error_data_type_it);
				error_type = _vala_code_node_ref0 ((_tmp2_ = error_data_type, VALA_IS_ERROR_TYPE (_tmp2_) ? ((ValaErrorType*) _tmp2_) : NULL));
				self->priv->current_block = (_tmp3_ = _vala_basic_block_ref0 (last_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp3_);
				self->priv->unreachable_reported = TRUE;
				{
					gint i;
					i = gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1;
					{
						gboolean _tmp4_;
						_tmp4_ = TRUE;
						while (TRUE) {
							ValaFlowAnalyzerJumpTarget* jump_target;
							if (!_tmp4_) {
								i--;
							}
							_tmp4_ = FALSE;
							if (!(i >= 0)) {
								break;
							}
							jump_target = (ValaFlowAnalyzerJumpTarget*) gee_list_get (self->priv->jump_stack, i);
							if (vala_flow_analyzer_jump_target_get_is_return_target (jump_target)) {
								ValaBasicBlock* _tmp5_;
								vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
								self->priv->current_block = (_tmp5_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp5_);
								self->priv->unreachable_reported = FALSE;
								_vala_flow_analyzer_jump_target_unref0 (jump_target);
								break;
							} else {
								if (vala_flow_analyzer_jump_target_get_is_error_target (jump_target)) {
									gboolean _tmp6_;
									_tmp6_ = FALSE;
									if (vala_flow_analyzer_jump_target_get_error_domain (jump_target) == NULL) {
										_tmp6_ = TRUE;
									} else {
										gboolean _tmp7_;
										_tmp7_ = FALSE;
										if (vala_flow_analyzer_jump_target_get_error_domain (jump_target) == vala_error_type_get_error_domain (error_type)) {
											gboolean _tmp8_;
											_tmp8_ = FALSE;
											if (vala_flow_analyzer_jump_target_get_error_code (jump_target) == NULL) {
												_tmp8_ = TRUE;
											} else {
												_tmp8_ = vala_flow_analyzer_jump_target_get_error_code (jump_target) == vala_error_type_get_error_code (error_type);
											}
											_tmp7_ = _tmp8_;
										} else {
											_tmp7_ = FALSE;
										}
										_tmp6_ = _tmp7_;
									}
									if (_tmp6_) {
										ValaBasicBlock* _tmp9_;
										vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
										self->priv->current_block = (_tmp9_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp9_);
										self->priv->unreachable_reported = FALSE;
										_vala_flow_analyzer_jump_target_unref0 (jump_target);
										break;
									}
								} else {
									if (vala_flow_analyzer_jump_target_get_is_finally_clause (jump_target)) {
										ValaBasicBlock* _tmp10_;
										vala_basic_block_connect (self->priv->current_block, vala_flow_analyzer_jump_target_get_basic_block (jump_target));
										self->priv->current_block = (_tmp10_ = _vala_basic_block_ref0 (vala_flow_analyzer_jump_target_get_last_block (jump_target)), _vala_basic_block_unref0 (self->priv->current_block), _tmp10_);
									}
								}
							}
							_vala_flow_analyzer_jump_target_unref0 (jump_target);
						}
					}
				}
				_vala_code_node_unref0 (error_data_type);
				_vala_code_node_unref0 (error_type);
			}
			_gee_collection_object_unref0 (_error_data_type_it);
		}
		if (!always_fail) {
			ValaBasicBlock* _tmp11_;
			self->priv->current_block = (_tmp11_ = vala_basic_block_new (), _vala_basic_block_unref0 (self->priv->current_block), _tmp11_);
			vala_basic_block_connect (last_block, self->priv->current_block);
		}
		_vala_basic_block_unref0 (last_block);
	}
}


static void vala_flow_analyzer_real_visit_yield_statement (ValaCodeVisitor* base, ValaYieldStatement* stmt) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_code_node_accept_children ((ValaCodeNode*) stmt, (ValaCodeVisitor*) self);
}


static void vala_flow_analyzer_real_visit_throw_statement (ValaCodeVisitor* base, ValaThrowStatement* stmt) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) stmt);
	vala_flow_analyzer_handle_errors (self, (ValaCodeNode*) stmt, TRUE);
}


static void vala_flow_analyzer_real_visit_try_statement (ValaCodeVisitor* base, ValaTryStatement* stmt) {
	ValaFlowAnalyzer * self;
	ValaBasicBlock* before_try_block;
	ValaBasicBlock* after_try_block;
	ValaBasicBlock* finally_block;
	gint finally_jump_stack_size;
	GeeList* catch_clauses;
	ValaBasicBlock* _tmp13_;
	GeeList* catch_stack;
	GeeList* _tmp21_;
	gboolean _tmp22_;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	before_try_block = _vala_basic_block_ref0 (self->priv->current_block);
	after_try_block = vala_basic_block_new ();
	finally_block = NULL;
	if (vala_try_statement_get_finally_body (stmt) != NULL) {
		ValaBasicBlock* _tmp0_;
		ValaBasicBlock* _tmp1_;
		ValaBasicBlock* invalid_block;
		ValaFlowAnalyzerJumpTarget* _tmp2_;
		GeeList* _tmp3_;
		gboolean _tmp4_;
		ValaFlowAnalyzerJumpTarget* _tmp5_;
		finally_block = (_tmp0_ = vala_basic_block_new (), _vala_basic_block_unref0 (finally_block), _tmp0_);
		self->priv->current_block = (_tmp1_ = _vala_basic_block_ref0 (finally_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp1_);
		invalid_block = vala_basic_block_new ();
		gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp2_ = vala_flow_analyzer_jump_target_new_any_target (invalid_block));
		_vala_flow_analyzer_jump_target_unref0 (_tmp2_);
		vala_code_node_accept ((ValaCodeNode*) vala_try_statement_get_finally_body (stmt), (ValaCodeVisitor*) self);
		if ((_tmp4_ = gee_collection_get_size ((GeeCollection*) (_tmp3_ = vala_basic_block_get_predecessors (invalid_block))) > 0, _gee_collection_object_unref0 (_tmp3_), _tmp4_)) {
			vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) stmt), "jump out of finally block not permitted");
			vala_code_node_set_error ((ValaCodeNode*) stmt, TRUE);
			_vala_basic_block_unref0 (invalid_block);
			_vala_basic_block_unref0 (before_try_block);
			_vala_basic_block_unref0 (after_try_block);
			_vala_basic_block_unref0 (finally_block);
			return;
		}
		gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
		gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp5_ = vala_flow_analyzer_jump_target_new_finally_clause (finally_block, self->priv->current_block));
		_vala_flow_analyzer_jump_target_unref0 (_tmp5_);
		_vala_basic_block_unref0 (invalid_block);
	}
	finally_jump_stack_size = gee_collection_get_size ((GeeCollection*) self->priv->jump_stack);
	catch_clauses = vala_try_statement_get_catch_clauses (stmt);
	{
		gint i;
		i = gee_collection_get_size ((GeeCollection*) catch_clauses) - 1;
		{
			gboolean _tmp6_;
			_tmp6_ = TRUE;
			while (TRUE) {
				ValaCatchClause* catch_clause;
				if (!_tmp6_) {
					i--;
				}
				_tmp6_ = FALSE;
				if (!(i >= 0)) {
					break;
				}
				catch_clause = (ValaCatchClause*) gee_list_get (catch_clauses, i);
				if (vala_catch_clause_get_error_type (catch_clause) != NULL) {
					ValaDataType* _tmp7_;
					ValaErrorType* error_type;
					ValaFlowAnalyzerJumpTarget* _tmp10_;
					ValaBasicBlock* _tmp8_;
					ValaTypeSymbol* _tmp9_;
					error_type = _vala_code_node_ref0 ((_tmp7_ = vala_catch_clause_get_error_type (catch_clause), VALA_IS_ERROR_TYPE (_tmp7_) ? ((ValaErrorType*) _tmp7_) : NULL));
					gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp10_ = vala_flow_analyzer_jump_target_new_error_target (_tmp8_ = vala_basic_block_new (), catch_clause, (_tmp9_ = vala_data_type_get_data_type (vala_catch_clause_get_error_type (catch_clause)), VALA_IS_ERROR_DOMAIN (_tmp9_) ? ((ValaErrorDomain*) _tmp9_) : NULL), vala_error_type_get_error_code (error_type)));
					_vala_flow_analyzer_jump_target_unref0 (_tmp10_);
					_vala_basic_block_unref0 (_tmp8_);
					_vala_code_node_unref0 (error_type);
				} else {
					ValaFlowAnalyzerJumpTarget* _tmp12_;
					ValaBasicBlock* _tmp11_;
					gee_collection_add ((GeeCollection*) self->priv->jump_stack, _tmp12_ = vala_flow_analyzer_jump_target_new_error_target (_tmp11_ = vala_basic_block_new (), catch_clause, NULL, NULL));
					_vala_flow_analyzer_jump_target_unref0 (_tmp12_);
					_vala_basic_block_unref0 (_tmp11_);
				}
				_vala_code_node_unref0 (catch_clause);
			}
		}
	}
	self->priv->current_block = (_tmp13_ = _vala_basic_block_ref0 (before_try_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp13_);
	vala_code_node_accept ((ValaCodeNode*) vala_try_statement_get_body (stmt), (ValaCodeVisitor*) self);
	if (self->priv->current_block != NULL) {
		if (finally_block != NULL) {
			ValaBasicBlock* _tmp14_;
			vala_basic_block_connect (self->priv->current_block, finally_block);
			self->priv->current_block = (_tmp14_ = _vala_basic_block_ref0 (finally_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp14_);
		}
		vala_basic_block_connect (self->priv->current_block, after_try_block);
	}
	catch_stack = (GeeList*) gee_array_list_new (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, (GBoxedCopyFunc) vala_flow_analyzer_jump_target_ref, vala_flow_analyzer_jump_target_unref, g_direct_equal);
	{
		gint i;
		i = gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1;
		{
			gboolean _tmp15_;
			_tmp15_ = TRUE;
			while (TRUE) {
				ValaFlowAnalyzerJumpTarget* jump_target;
				if (!_tmp15_) {
					i--;
				}
				_tmp15_ = FALSE;
				if (!(i >= finally_jump_stack_size)) {
					break;
				}
				jump_target = (ValaFlowAnalyzerJumpTarget*) gee_list_get (self->priv->jump_stack, i);
				gee_collection_add ((GeeCollection*) catch_stack, jump_target);
				gee_list_remove_at (self->priv->jump_stack, i);
				_vala_flow_analyzer_jump_target_unref0 (jump_target);
			}
		}
	}
	{
		GeeIterator* _jump_target_it;
		_jump_target_it = gee_iterable_iterator ((GeeIterable*) catch_stack);
		while (TRUE) {
			ValaFlowAnalyzerJumpTarget* jump_target;
			GeeList* _tmp17_;
			gboolean _tmp18_;
			if (!gee_iterator_next (_jump_target_it)) {
				break;
			}
			jump_target = (ValaFlowAnalyzerJumpTarget*) gee_iterator_get (_jump_target_it);
			{
				GeeIterator* _prev_target_it;
				_prev_target_it = gee_iterable_iterator ((GeeIterable*) catch_stack);
				while (TRUE) {
					ValaFlowAnalyzerJumpTarget* prev_target;
					gboolean _tmp16_;
					if (!gee_iterator_next (_prev_target_it)) {
						break;
					}
					prev_target = (ValaFlowAnalyzerJumpTarget*) gee_iterator_get (_prev_target_it);
					if (prev_target == jump_target) {
						_vala_flow_analyzer_jump_target_unref0 (prev_target);
						break;
					}
					_tmp16_ = FALSE;
					if (vala_flow_analyzer_jump_target_get_error_domain (prev_target) == vala_flow_analyzer_jump_target_get_error_domain (jump_target)) {
						_tmp16_ = vala_flow_analyzer_jump_target_get_error_code (prev_target) == vala_flow_analyzer_jump_target_get_error_code (jump_target);
					} else {
						_tmp16_ = FALSE;
					}
					if (_tmp16_) {
						vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) stmt), "double catch clause of same error detected");
						vala_code_node_set_error ((ValaCodeNode*) stmt, TRUE);
						_vala_flow_analyzer_jump_target_unref0 (prev_target);
						_gee_collection_object_unref0 (_prev_target_it);
						_vala_flow_analyzer_jump_target_unref0 (jump_target);
						_gee_collection_object_unref0 (_jump_target_it);
						_vala_basic_block_unref0 (before_try_block);
						_vala_basic_block_unref0 (after_try_block);
						_vala_basic_block_unref0 (finally_block);
						_gee_collection_object_unref0 (catch_clauses);
						_gee_collection_object_unref0 (catch_stack);
						return;
					}
					_vala_flow_analyzer_jump_target_unref0 (prev_target);
				}
				_gee_collection_object_unref0 (_prev_target_it);
			}
			if ((_tmp18_ = gee_collection_get_size ((GeeCollection*) (_tmp17_ = vala_basic_block_get_predecessors (vala_flow_analyzer_jump_target_get_basic_block (jump_target)))) == 0, _gee_collection_object_unref0 (_tmp17_), _tmp18_)) {
				vala_report_warning (vala_code_node_get_source_reference ((ValaCodeNode*) vala_flow_analyzer_jump_target_get_catch_clause (jump_target)), "unreachable catch clause detected");
			} else {
				ValaBasicBlock* _tmp19_;
				self->priv->current_block = (_tmp19_ = _vala_basic_block_ref0 (vala_flow_analyzer_jump_target_get_basic_block (jump_target)), _vala_basic_block_unref0 (self->priv->current_block), _tmp19_);
				vala_basic_block_add_node (self->priv->current_block, (ValaCodeNode*) vala_flow_analyzer_jump_target_get_catch_clause (jump_target));
				vala_code_node_accept ((ValaCodeNode*) vala_catch_clause_get_body (vala_flow_analyzer_jump_target_get_catch_clause (jump_target)), (ValaCodeVisitor*) self);
				if (self->priv->current_block != NULL) {
					if (finally_block != NULL) {
						ValaBasicBlock* _tmp20_;
						vala_basic_block_connect (self->priv->current_block, finally_block);
						self->priv->current_block = (_tmp20_ = _vala_basic_block_ref0 (finally_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp20_);
					}
					vala_basic_block_connect (self->priv->current_block, after_try_block);
				}
			}
			_vala_flow_analyzer_jump_target_unref0 (jump_target);
		}
		_gee_collection_object_unref0 (_jump_target_it);
	}
	if (finally_block != NULL) {
		gee_list_remove_at (self->priv->jump_stack, gee_collection_get_size ((GeeCollection*) self->priv->jump_stack) - 1);
	}
	if ((_tmp22_ = gee_collection_get_size ((GeeCollection*) (_tmp21_ = vala_basic_block_get_predecessors (after_try_block))) > 0, _gee_collection_object_unref0 (_tmp21_), _tmp22_)) {
		ValaBasicBlock* _tmp23_;
		self->priv->current_block = (_tmp23_ = _vala_basic_block_ref0 (after_try_block), _vala_basic_block_unref0 (self->priv->current_block), _tmp23_);
	} else {
		ValaBasicBlock* _tmp24_;
		self->priv->current_block = (_tmp24_ = NULL, _vala_basic_block_unref0 (self->priv->current_block), _tmp24_);
		self->priv->unreachable_reported = FALSE;
	}
	_vala_basic_block_unref0 (before_try_block);
	_vala_basic_block_unref0 (after_try_block);
	_vala_basic_block_unref0 (finally_block);
	_gee_collection_object_unref0 (catch_clauses);
	_gee_collection_object_unref0 (catch_stack);
}


static void vala_flow_analyzer_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt) {
	ValaFlowAnalyzer * self;
	self = (ValaFlowAnalyzer*) base;
	g_return_if_fail (stmt != NULL);
	if (vala_flow_analyzer_unreachable (self, (ValaCodeNode*) stmt)) {
		return;
	}
	vala_code_node_accept ((ValaCodeNode*) vala_lock_statement_get_body (stmt), (ValaCodeVisitor*) self);
}


static gboolean vala_flow_analyzer_unreachable (ValaFlowAnalyzer* self, ValaCodeNode* node) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (node != NULL, FALSE);
	if (self->priv->current_block == NULL) {
		if (!self->priv->unreachable_reported) {
			vala_report_warning (vala_code_node_get_source_reference (node), "unreachable code detected");
			self->priv->unreachable_reported = TRUE;
		}
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_break_target (GType object_type, ValaBasicBlock* basic_block) {
	ValaFlowAnalyzerJumpTarget* self;
	g_return_val_if_fail (basic_block != NULL, NULL);
	self = (ValaFlowAnalyzerJumpTarget*) g_type_create_instance (object_type);
	vala_flow_analyzer_jump_target_set_basic_block (self, basic_block);
	vala_flow_analyzer_jump_target_set_is_break_target (self, TRUE);
	return self;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_break_target (ValaBasicBlock* basic_block) {
	return vala_flow_analyzer_jump_target_construct_break_target (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, basic_block);
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_continue_target (GType object_type, ValaBasicBlock* basic_block) {
	ValaFlowAnalyzerJumpTarget* self;
	g_return_val_if_fail (basic_block != NULL, NULL);
	self = (ValaFlowAnalyzerJumpTarget*) g_type_create_instance (object_type);
	vala_flow_analyzer_jump_target_set_basic_block (self, basic_block);
	vala_flow_analyzer_jump_target_set_is_continue_target (self, TRUE);
	return self;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_continue_target (ValaBasicBlock* basic_block) {
	return vala_flow_analyzer_jump_target_construct_continue_target (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, basic_block);
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_return_target (GType object_type, ValaBasicBlock* basic_block) {
	ValaFlowAnalyzerJumpTarget* self;
	g_return_val_if_fail (basic_block != NULL, NULL);
	self = (ValaFlowAnalyzerJumpTarget*) g_type_create_instance (object_type);
	vala_flow_analyzer_jump_target_set_basic_block (self, basic_block);
	vala_flow_analyzer_jump_target_set_is_return_target (self, TRUE);
	return self;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_return_target (ValaBasicBlock* basic_block) {
	return vala_flow_analyzer_jump_target_construct_return_target (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, basic_block);
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_error_target (GType object_type, ValaBasicBlock* basic_block, ValaCatchClause* catch_clause, ValaErrorDomain* error_domain, ValaErrorCode* error_code) {
	ValaFlowAnalyzerJumpTarget* self;
	g_return_val_if_fail (basic_block != NULL, NULL);
	g_return_val_if_fail (catch_clause != NULL, NULL);
	self = (ValaFlowAnalyzerJumpTarget*) g_type_create_instance (object_type);
	vala_flow_analyzer_jump_target_set_basic_block (self, basic_block);
	vala_flow_analyzer_jump_target_set_catch_clause (self, catch_clause);
	vala_flow_analyzer_jump_target_set_error_domain (self, error_domain);
	vala_flow_analyzer_jump_target_set_error_code (self, error_code);
	vala_flow_analyzer_jump_target_set_is_error_target (self, TRUE);
	return self;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_error_target (ValaBasicBlock* basic_block, ValaCatchClause* catch_clause, ValaErrorDomain* error_domain, ValaErrorCode* error_code) {
	return vala_flow_analyzer_jump_target_construct_error_target (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, basic_block, catch_clause, error_domain, error_code);
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_any_target (GType object_type, ValaBasicBlock* basic_block) {
	ValaFlowAnalyzerJumpTarget* self;
	g_return_val_if_fail (basic_block != NULL, NULL);
	self = (ValaFlowAnalyzerJumpTarget*) g_type_create_instance (object_type);
	vala_flow_analyzer_jump_target_set_basic_block (self, basic_block);
	vala_flow_analyzer_jump_target_set_is_break_target (self, TRUE);
	vala_flow_analyzer_jump_target_set_is_continue_target (self, TRUE);
	vala_flow_analyzer_jump_target_set_is_return_target (self, TRUE);
	vala_flow_analyzer_jump_target_set_is_error_target (self, TRUE);
	return self;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_any_target (ValaBasicBlock* basic_block) {
	return vala_flow_analyzer_jump_target_construct_any_target (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, basic_block);
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct_finally_clause (GType object_type, ValaBasicBlock* basic_block, ValaBasicBlock* last_block) {
	ValaFlowAnalyzerJumpTarget* self;
	g_return_val_if_fail (basic_block != NULL, NULL);
	g_return_val_if_fail (last_block != NULL, NULL);
	self = (ValaFlowAnalyzerJumpTarget*) g_type_create_instance (object_type);
	vala_flow_analyzer_jump_target_set_basic_block (self, basic_block);
	vala_flow_analyzer_jump_target_set_last_block (self, last_block);
	vala_flow_analyzer_jump_target_set_is_finally_clause (self, TRUE);
	return self;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new_finally_clause (ValaBasicBlock* basic_block, ValaBasicBlock* last_block) {
	return vala_flow_analyzer_jump_target_construct_finally_clause (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, basic_block, last_block);
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_construct (GType object_type) {
	ValaFlowAnalyzerJumpTarget* self;
	self = (ValaFlowAnalyzerJumpTarget*) g_type_create_instance (object_type);
	return self;
}


static ValaFlowAnalyzerJumpTarget* vala_flow_analyzer_jump_target_new (void) {
	return vala_flow_analyzer_jump_target_construct (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET);
}


static gboolean vala_flow_analyzer_jump_target_get_is_break_target (ValaFlowAnalyzerJumpTarget* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_break_target;
	return result;
}


static void vala_flow_analyzer_jump_target_set_is_break_target (ValaFlowAnalyzerJumpTarget* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_is_break_target = value;
}


static gboolean vala_flow_analyzer_jump_target_get_is_continue_target (ValaFlowAnalyzerJumpTarget* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_continue_target;
	return result;
}


static void vala_flow_analyzer_jump_target_set_is_continue_target (ValaFlowAnalyzerJumpTarget* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_is_continue_target = value;
}


static gboolean vala_flow_analyzer_jump_target_get_is_return_target (ValaFlowAnalyzerJumpTarget* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_return_target;
	return result;
}


static void vala_flow_analyzer_jump_target_set_is_return_target (ValaFlowAnalyzerJumpTarget* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_is_return_target = value;
}


static gboolean vala_flow_analyzer_jump_target_get_is_error_target (ValaFlowAnalyzerJumpTarget* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_error_target;
	return result;
}


static void vala_flow_analyzer_jump_target_set_is_error_target (ValaFlowAnalyzerJumpTarget* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_is_error_target = value;
}


static ValaErrorDomain* vala_flow_analyzer_jump_target_get_error_domain (ValaFlowAnalyzerJumpTarget* self) {
	ValaErrorDomain* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_error_domain;
	return result;
}


static void vala_flow_analyzer_jump_target_set_error_domain (ValaFlowAnalyzerJumpTarget* self, ValaErrorDomain* value) {
	ValaErrorDomain* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_error_domain = (_tmp0_ = _vala_code_node_ref0 (value), _vala_code_node_unref0 (self->priv->_error_domain), _tmp0_);
}


static ValaErrorCode* vala_flow_analyzer_jump_target_get_error_code (ValaFlowAnalyzerJumpTarget* self) {
	ValaErrorCode* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_error_code;
	return result;
}


static void vala_flow_analyzer_jump_target_set_error_code (ValaFlowAnalyzerJumpTarget* self, ValaErrorCode* value) {
	ValaErrorCode* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_error_code = (_tmp0_ = _vala_code_node_ref0 (value), _vala_code_node_unref0 (self->priv->_error_code), _tmp0_);
}


static gboolean vala_flow_analyzer_jump_target_get_is_finally_clause (ValaFlowAnalyzerJumpTarget* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_finally_clause;
	return result;
}


static void vala_flow_analyzer_jump_target_set_is_finally_clause (ValaFlowAnalyzerJumpTarget* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_is_finally_clause = value;
}


static ValaBasicBlock* vala_flow_analyzer_jump_target_get_basic_block (ValaFlowAnalyzerJumpTarget* self) {
	ValaBasicBlock* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_basic_block;
	return result;
}


static void vala_flow_analyzer_jump_target_set_basic_block (ValaFlowAnalyzerJumpTarget* self, ValaBasicBlock* value) {
	ValaBasicBlock* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_basic_block = (_tmp0_ = _vala_basic_block_ref0 (value), _vala_basic_block_unref0 (self->priv->_basic_block), _tmp0_);
}


static ValaBasicBlock* vala_flow_analyzer_jump_target_get_last_block (ValaFlowAnalyzerJumpTarget* self) {
	ValaBasicBlock* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_last_block;
	return result;
}


static void vala_flow_analyzer_jump_target_set_last_block (ValaFlowAnalyzerJumpTarget* self, ValaBasicBlock* value) {
	ValaBasicBlock* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_last_block = (_tmp0_ = _vala_basic_block_ref0 (value), _vala_basic_block_unref0 (self->priv->_last_block), _tmp0_);
}


static ValaCatchClause* vala_flow_analyzer_jump_target_get_catch_clause (ValaFlowAnalyzerJumpTarget* self) {
	ValaCatchClause* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_catch_clause;
	return result;
}


static void vala_flow_analyzer_jump_target_set_catch_clause (ValaFlowAnalyzerJumpTarget* self, ValaCatchClause* value) {
	ValaCatchClause* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_catch_clause = (_tmp0_ = _vala_code_node_ref0 (value), _vala_code_node_unref0 (self->priv->_catch_clause), _tmp0_);
}


static void vala_flow_analyzer_value_jump_target_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_flow_analyzer_value_jump_target_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_flow_analyzer_jump_target_unref (value->data[0].v_pointer);
	}
}


static void vala_flow_analyzer_value_jump_target_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_flow_analyzer_jump_target_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_flow_analyzer_value_jump_target_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_flow_analyzer_value_jump_target_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaFlowAnalyzerJumpTarget* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_flow_analyzer_jump_target_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_flow_analyzer_value_jump_target_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaFlowAnalyzerJumpTarget** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_flow_analyzer_jump_target_ref (value->data[0].v_pointer);
	}
	return NULL;
}


static GParamSpec* vala_flow_analyzer_param_spec_jump_target (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaFlowAnalyzerParamSpecJumpTarget* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


static gpointer vala_flow_analyzer_value_get_jump_target (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET), NULL);
	return value->data[0].v_pointer;
}


static void vala_flow_analyzer_value_set_jump_target (GValue* value, gpointer v_object) {
	ValaFlowAnalyzerJumpTarget* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_flow_analyzer_jump_target_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_flow_analyzer_jump_target_unref (old);
	}
}


static void vala_flow_analyzer_jump_target_class_init (ValaFlowAnalyzerJumpTargetClass * klass) {
	vala_flow_analyzer_jump_target_parent_class = g_type_class_peek_parent (klass);
	VALA_FLOW_ANALYZER_JUMP_TARGET_CLASS (klass)->finalize = vala_flow_analyzer_jump_target_finalize;
	g_type_class_add_private (klass, sizeof (ValaFlowAnalyzerJumpTargetPrivate));
}


static void vala_flow_analyzer_jump_target_instance_init (ValaFlowAnalyzerJumpTarget * self) {
	self->priv = VALA_FLOW_ANALYZER_JUMP_TARGET_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_flow_analyzer_jump_target_finalize (ValaFlowAnalyzerJumpTarget* obj) {
	ValaFlowAnalyzerJumpTarget * self;
	self = VALA_FLOW_ANALYZER_JUMP_TARGET (obj);
	_vala_code_node_unref0 (self->priv->_error_domain);
	_vala_code_node_unref0 (self->priv->_error_code);
	_vala_basic_block_unref0 (self->priv->_basic_block);
	_vala_basic_block_unref0 (self->priv->_last_block);
	_vala_code_node_unref0 (self->priv->_catch_clause);
}


static GType vala_flow_analyzer_jump_target_get_type (void) {
	static GType vala_flow_analyzer_jump_target_type_id = 0;
	if (vala_flow_analyzer_jump_target_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_flow_analyzer_value_jump_target_init, vala_flow_analyzer_value_jump_target_free_value, vala_flow_analyzer_value_jump_target_copy_value, vala_flow_analyzer_value_jump_target_peek_pointer, "p", vala_flow_analyzer_value_jump_target_collect_value, "p", vala_flow_analyzer_value_jump_target_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaFlowAnalyzerJumpTargetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_flow_analyzer_jump_target_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaFlowAnalyzerJumpTarget), 0, (GInstanceInitFunc) vala_flow_analyzer_jump_target_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_flow_analyzer_jump_target_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaFlowAnalyzerJumpTarget", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vala_flow_analyzer_jump_target_type_id;
}


static gpointer vala_flow_analyzer_jump_target_ref (gpointer instance) {
	ValaFlowAnalyzerJumpTarget* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


static void vala_flow_analyzer_jump_target_unref (gpointer instance) {
	ValaFlowAnalyzerJumpTarget* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_FLOW_ANALYZER_JUMP_TARGET_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void vala_flow_analyzer_class_init (ValaFlowAnalyzerClass * klass) {
	vala_flow_analyzer_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_VISITOR_CLASS (klass)->finalize = vala_flow_analyzer_finalize;
	g_type_class_add_private (klass, sizeof (ValaFlowAnalyzerPrivate));
	VALA_CODE_VISITOR_CLASS (klass)->visit_source_file = vala_flow_analyzer_real_visit_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = vala_flow_analyzer_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_struct = vala_flow_analyzer_real_visit_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_interface = vala_flow_analyzer_real_visit_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum = vala_flow_analyzer_real_visit_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_error_domain = vala_flow_analyzer_real_visit_error_domain;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_flow_analyzer_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_method = vala_flow_analyzer_real_visit_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_creation_method = vala_flow_analyzer_real_visit_creation_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_property = vala_flow_analyzer_real_visit_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_property_accessor = vala_flow_analyzer_real_visit_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_block = vala_flow_analyzer_real_visit_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_declaration_statement = vala_flow_analyzer_real_visit_declaration_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_flow_analyzer_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_flow_analyzer_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_switch_statement = vala_flow_analyzer_real_visit_switch_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_loop = vala_flow_analyzer_real_visit_loop;
	VALA_CODE_VISITOR_CLASS (klass)->visit_foreach_statement = vala_flow_analyzer_real_visit_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_break_statement = vala_flow_analyzer_real_visit_break_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_continue_statement = vala_flow_analyzer_real_visit_continue_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_return_statement = vala_flow_analyzer_real_visit_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_yield_statement = vala_flow_analyzer_real_visit_yield_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_throw_statement = vala_flow_analyzer_real_visit_throw_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_try_statement = vala_flow_analyzer_real_visit_try_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_lock_statement = vala_flow_analyzer_real_visit_lock_statement;
}


static void vala_flow_analyzer_instance_init (ValaFlowAnalyzer * self) {
	self->priv = VALA_FLOW_ANALYZER_GET_PRIVATE (self);
	self->priv->jump_stack = (GeeList*) gee_array_list_new (VALA_FLOW_ANALYZER_TYPE_JUMP_TARGET, (GBoxedCopyFunc) vala_flow_analyzer_jump_target_ref, vala_flow_analyzer_jump_target_unref, g_direct_equal);
}


static void vala_flow_analyzer_finalize (ValaCodeVisitor* obj) {
	ValaFlowAnalyzer * self;
	self = VALA_FLOW_ANALYZER (obj);
	_vala_code_context_unref0 (self->priv->context);
	_vala_basic_block_unref0 (self->priv->current_block);
	_gee_collection_object_unref0 (self->priv->jump_stack);
	_gee_collection_object_unref0 (self->priv->var_map);
	_gee_collection_object_unref0 (self->priv->used_vars);
	_gee_collection_object_unref0 (self->priv->phi_functions);
	VALA_CODE_VISITOR_CLASS (vala_flow_analyzer_parent_class)->finalize (obj);
}


GType vala_flow_analyzer_get_type (void) {
	static GType vala_flow_analyzer_type_id = 0;
	if (vala_flow_analyzer_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaFlowAnalyzerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_flow_analyzer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaFlowAnalyzer), 0, (GInstanceInitFunc) vala_flow_analyzer_instance_init, NULL };
		vala_flow_analyzer_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaFlowAnalyzer", &g_define_type_info, 0);
	}
	return vala_flow_analyzer_type_id;
}




